<?PHP

/*
  ***************************************************
  ***************************************************
  CUSTOM KORE 2012 MODULE - NOVA EDITION

  THIS MODULE IS CALLED 'KORE' TO DISTINGUISH IT
  FROM THE OLDER GENERATION 'CORE' MODULES.

  THIS MODULE CONSISTS OF A COLLECTION OF CUSTOM
  DEFINED CONSTANTS AND FUNCTIONS.  THE MAIN
  EMPHASIS IS ON SCIENCE AND MATHEMATICS
  PROGRAMMING.

  IT IS THE NEXT GENERATION CORE LOGIC MODULE AROUND
  WHICH TO CONSTRUCT THE NeoProgrammics WEB SITE.

  ---------------------------------
  PROGRAMMER : Jay Tanner
  LANGUAGE   : PHP 5.2.5
  CONCEIVED  : 2008 OCT 16 at 22:00
  UPDATED    : 2012 AUG 01

  ***************************************************
  ***************************************************
*/



// ----------------------------
// Define light speed constant.

   define ("c", "299792458"); // Units: m/s

// ---------------------------------------------------
// Define the mathematical constant pi to 64 decimals.

   define ("pi", "3.141592653589793238462643383279502884197169399375105820974944592307816");

// ------------------------------------------------------------
// Define the constant, e, to 64 decimals.  this is the base of
// natural logarithms.  It is the value of e to the 1st power.

   define ("e", "2.718281828459045235360287471352662497757247093699959574966967627724077");

// -----------------------------------------
// Define factors to convert between degrees
// and radians to up to 64 decimals.

// Multiply degrees by this factor to obtain radians.
   define ("RAD_PER_DEG", "0.01745329251994329576923690768488612713442871888541725456097191440171009");

// Multiply radians by this factor to obtain degrees.
   define ("DEG_PER_RAD", "57.2957795130823208767981548141051703324054724665643215491602438612028509");


/*

 -----------------------------------------------
 The metric length units symbols recognized are:

 ------   ----
 SYMBOL   UNIT

   A      Angstroms
   nm     Nanometers
   u|um   Micrometers (or Microns)
   mm     Millimeters
   cm     Centimeters
   in     Inches
   ft     Feet
   yd     Yards
   m      Meters
   km     Kilometers
   mi     Miles (Statute)
   nmi    Nautical miles
   AU     Astronomical Units
   LY     Light Years (Based on Julian year)
   pc     Parsecs (Based on Julian year)


*/


// ------------
// YEAR LENGTHS

// Define length of Julian year.

   define ("S_PER_JYR", "31557600"); // Units: seconds

// Define length of Gregorian year.

   define ("S_PER_GYR", "31556952"); // Units: seconds



// The following metric length interconversion factor constants are
// based on the following standard values.  Currently, 15 units are
// recognized.


// -----------------------------------------------------------------
// Define some EXACT standard metric length/distance interconversion
// factors in terms of metres per unit (M_PER_UNIT).  This is the
// factor to multiply the units by to compute the equivalent metres.

   define ("M_PER_A"   , "0.0000000001");  // m per angstrom
   define ("M_PER_NM"  , "0.000000001");   // m per nanometer
   define ("M_PER_UM"  , "0.000001");      // m per micrometer (micron)
   define ("M_PER_MM"  , "0.001");         // m per millimeter
   define ("M_PER_CM"  , "0.01");          // m per centimeter
   define ("M_PER_IN"  , "0.0254");        // m per inch
   define ("M_PER_FT"  , "0.3048");        // m per foot
   define ("M_PER_YD"  , "0.9144");        // m per yard
   define ("M_PER_M"   , "1");             // m per meter
   define ("M_PER_KM"  , "1000");          // m per kilometer
   define ("M_PER_MI"  , "1609.344");      // m per mile (statute)
   define ("M_PER_NMI" , "1852");          // m per nautical mile
   define ("KM_PER_NMI", "1.852");         // km per nautical mile
   define ("KM_PER_MI" , "1.609344");      // Kilometres per Mile (statute)



// ------------------------
// Define Astronomical Unit

   define ("M_PER_AU" , "149597870691.0"); // NASA/JPL-DE405
   define ("KM_PER_AU", "149597870.691");  // NASA/JPL-DE405
   define ("KM_PER_IAU", "149597871.0");   // IAU value




// -----------------------------------------------------
// Derived factors based on the above adopted standards.
// The AU used is the IAU value and the year value used
// is the standard Julian year.
//
// ********


   define ("M_PER_LY"  , "9460730472580800.0");
   define ("M_PER_PC"  , "30856775876793.1146632722718115");
   define ("AU_PER_LY" , "63241.0769574441336802");
   define ("AU_PER_PC" , "206264.8062470963551565");
   define ("LY_PER_PC" , "3.2615637837080957");



// ------------------------------------------
// Define some EXACT metric weight standards.

   define ("KG_PER_OZ" , "0.028349523125");
   define ("KG_PER_LB" , "0.45959237");
   define ("G_PER_OZ"  , "28.349523125");



// ------------------------------------------------
// Define standard and universal gravity constants.

   define ("g", "9.80665");  // m per s
   define ("G", "0.000000000066742");  // Universal G (SI units)



// --------------------------
// Define Planck constant (h).

   define ("h", "6.62606896E-34");  // J/s (watts)



// -----------------------------------------
// Define Stefan-Boltzmann constant (sigma).
//
// radiated energy = sigma * K**4 watts per sq. metre
// Where K = temperature in kelvins

   define ("sigma", "0.000000056704");  // Watts per sq. metre













/*
   ************************************************************************
   This function takes a string argument and strips any
   redundant white spaces.  Any words separated by more
   than one space will be adjusted to have only a single
   space between them instead, or what can be called
   normalized word spacing. A normalized text string
   is easier to parse when all elements are uniformly
   spaced.
*/

   function Strip_Whitespace($ArgStr)
{
   return preg_replace("/\s+/", " ", trim($ArgStr));
}

// ************************************************************************


/*
   ************************************************************************
   This function generates a positive, 16-digit (RandomDecimal)
   value built around the PHP MT_Rand() function and using a
   construction method of random digit alternation.

   The construction method of random digit alternation means that
   as the random 16 digits are collected, they are added to the
   existing digits string at either the left end or right end of
   the string, according to a random 50/50 process, until the
   random 16-digit string is complete.

   Where the random decimal has the range:

   0.0000000000000000 <= (RandomDecimal) <= 0.9999999999999999
*/


   function Rand_Dec ($NumDecimalsArg=16)
{
// -------------------------------------
// Construct a random decimal value of a
// specified number of digits, one digit
// at a time using a random 50/50 digit
// alternation method.

   $n = $NumDecimalsArg;  if ($n <= 0) {$n=16;}

   $RandomDecimal = "";

   for ($i=0;   $i < $n;   $i++)
       {
        if (MT_Rand(0, 100000000) % 2 == 0)

           {$RandomDecimal .= (MT_Rand(1, 10)-1);}
        else
           {$RandomDecimal  = (MT_Rand(1, 10)-1) . $RandomDecimal;}
       }

// ---------------------------------------
// Done.  Return the random decimal value.

   return "0.$RandomDecimal";
} // End of  Rand_Dec()

// ************************************************************************


/*
   ************************************************************************
   This function returns a random integer string of a
   specified numbr of digits.

   NOTE: The 1st (leftmost) digit may possibly be a zero.
*/


   function Rand_Int ($NumDigitsArg=1)
{

// -------------------------------------
// Construct a random integer value of a
// specified number of digits, one digit
// at a time using a random 50/50 digit
// alternation method.

   $n = $NumDigitsArg;

   $RandomInteger = "";

   for ($i=0;   $i < $n;   $i++)
       {
        if (MT_Rand(0, 100000000) % 2 == 0)

           {$RandomInteger .= (MT_Rand(1, 10)-1);}
        else
           {$RandomInteger  = (MT_Rand(1, 10)-1) . $RandomInteger;}
       }

// -----
// Done.

   return "$RandomInteger";
} // End of  Rand_Int()



// ------------------------------------------
// Replacement for above Rand_Int() function.
//
// This function generates any number of random
// digits in any given base from 2 to 36.


   function Random_Digits ($NumDigits, $base=10)
{

// ------------------------------------------
// Define available digits according to base.

   $AvailDigits = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $base);

// ----------------------------------------
// Define accumulated digits output string.

   $RandDigits = "";

// -------------------------------------------
// Construct an n-digit random sequence in the
// given base, one digit at a time using a
// random 50/50 digit alternation method.
// There must be at least one digit.

   for ($i=0;   $i < $NumDigits;   $i++)
  {
   if (MT_Rand() % 2 == 0)

      {$RandDigits .= substr($AvailDigits, MT_Rand(0, $base-1), 1);}
   else
      {$RandDigits  = substr($AvailDigits, MT_Rand(0, $base-1), 1) . $RandDigits;}
  }

   return "$RandomDigits";

} // End of  Random_Digits()

// ************************************************************************

















/*
   ************************************************************************
   This function flips a mathematical coin and returns
   a random 0 or 1 result, corresponding to the two
   opposite sides of the coin.

   The odds should mimic the ordinary 50/50 coin flip
   probabilities.

   ----------
   DEPENDENCY

   This function calls:

   Rand_Dec()

*/


   function Coin_Flip()
{
// Return a random zero or one value.  These
// bivariate values may represent any two
// opposite states such as heads/tails,
// true/false, etc..

   return (mt_rand() % 2 == 0)? 0:1;
}

// ************************************************************************









/*

   ***********************************************
   This function is meant for generally formatting
   numerical values as strings for output.  This
   is to simplify neatly formatting the printed
   results of numerical computations such as
   astronomical ephemeris tables.

   ---------
   ARGUMENTS

   $PosSignFlagArg    = Positive sign flag for ($xArg).
                        Only if this flag is TRUE, is a positive (+)
                        sign is attached to any positive value.
                        This flag has no effect on negative values.

   $xArg              = The raw numerical X-argument value to be formatted.

   $LPadZeroDigitsArg = Padding zeros to apply to left of integer part.

   $NumDecimalsArg    = Number of rounding decimals.

   ------
   ERRORS

   This function does NOT check for errors.

*/


   function Format_Num ($PosSignFlagArg, $xArg, $LPadZeroDigitsArg, $NumDecimalsArg)
{

// ---------------------------
// Read input argument values.
   $x = trim($xArg);
   $z = $LPadZeroDigitsArg;
   $d = $NumDecimalsArg;

// ------------------------
// Remember numerical sign.
   $sign = ($x < 0)? "-" : "+";

// ------------------------------------
// Work internally with absolute value.
   if (SubStr($x, 0,1) == "-")
      {
       $x = SubStr($x, 1, StrLen($x));
      }

// ----------------------------------
// Attach positive sign if indicated.
   if ($PosSignFlagArg === FALSE && $sign == "+")  {$sign = "";}

// ---------------------------------
   $x = sPrintF("%10.$d" ."f", $x);
   $i = StrPos($x, ".");
   $IntPart = trim(SubStr($x, 0, $i));

// -----------------------------
// Handle case of integer value.
   if ($d == 0)
      {
       $FracPart = "";
       $IntPart = sPrintf("%d", $x);
      }

   else

//     ---------------------------------
//     Handle case of non-integer value.
      {
       $FracPart = "." . SubStr($x, $i+1, StrLen($x));
      }


   while (StrLen($IntPart) < $z) {$IntPart = "0$IntPart";}

// -----
// Done.
   return "$sign$IntPart$FracPart";

} // End of  Format_Num()















/*

   ************************************************************************
   This function validates an encoded date
   string according to the calendar mode.

   If date is valid, then TRUE is returned,
   otherwise FALSE is returned.


   -----------
   DEPENDENCY:

   This function calls the functions

   YYYYMMDD_To_JD12()
   and
   JD_To_YYYYMMDD()


*/


   function Is_Valid_Date ($yyyymmddArg, $CalModeArg=1)

{

// ---------------------
// Read input arguments.

   $ymmdd   = trim($yyyymmddArg)+0;
   $CalMode = trim($CalModeArg)+0;

// ----------------------
// Compute JD12 for date.

   $jd12 = YYYYMMDD_To_JD12($ymmdd, $CalMode);

// ----------------------------------------------------------------
// Reverse the process and compute a check date value ($ymmddCheck)
// from the previously computed ($jd12) value.

   $ymmddCheck = JD_To_YYYYMMDD($jd12, $CalMode);

// ------------------------------------------------------------------
// If the check date value perfectly matches the input date argument,
// then the date is OK (TRUE), else the date is invalid (FALSE).

   if ($ymmddCheck == $ymmdd) {return TRUE;} else {return FALSE;}

}

// ************************************************************************













/*

  ************************************************************************

  This function computes the Julian Day (JD) number
  for any given date on the Julian or Gregorian
  calendar at 12:00:00 TDB on the date.  The JD
  value returned (JD12) will always be an integer
  value not less than zero.

  The JD12 value corresponds to the mathematical centre
  of the date and represents the unique serial number
  of the corresponding calendar date counted from the
  mathematical beginning of the  calendar system, where
  the 1st date JD12 = 0.


  FOR THE JULIAN CALENDAR (All dates up to AD 1582 OCT 04 THU)
  Mathematical origin date (JD12=0) -47130101 = BC 4713 JAN 01 MON

  FOR THE GREGORIAN CALENDAR (For all dates since AD 1582 OCT 15 FRI)
  Mathematical origin date (JD12=0) -47141124 = BC 4714 NOV 24 MON


  ---------
  ARGUMENTS

  $ymmddd = Year, month and day arguments as encoded integer
            Min Greg Date: -47141124 for 4714 BC NOV 24 MON
            Min Jul  Date: -47130101 for 4713 BC JAN 01 MON

  $CalModeArg = Calendar mode: 0=Julian  or  1=Gregorian



  ------
  ERRORS

  This function does NOT check for invalid arguments.



*/



   function YYYYMMDD_To_JD12 ($yyyymmddArg, $CalModeArg=1)

{

   $ymmdd = $yyyymmddArg;

   if ($ymmdd < 0) {$sign = -1;} else {$sign = 1;}

   $CalMode = $CalModeArg;

// -------------------------------------------------
// Extract date elements from integer date argument.

   $w = floor($sign*$ymmdd / 10000);
   $m = floor(($sign*$ymmdd - 10000*$w) / 100);
   $d = ($sign*$ymmdd - 10000*$w - 100*$m);
   $y =  $sign * $w;

// ---------------------------------
// Adjust for BC year, if indicated.

   if ($y < 0)  {$a = $y+1;} else {$a = $y;}

// ----------------------------------------------------
// Make special mathematical adjustments, if necessary.
// Jan and Feb are equated to 13th and 14th months of
// the previous year.

   $b = floor((14 - $m) / 12);
   $c = $a - $b;
   $e = floor(30.6001*($m + 12*$b + 1)) + $d;
   $f = $CalMode * floor($c/100);
   $g = floor(365.25*($c + 4716));
   $h = $CalMode*(floor($f/4) - $f + 2) - 1524;

// ----------------------------------------------
// Compute JD value for 12:00 of date.  This will
// always be an integer value not less than zero
// when within the valid span.

   $JD12 = $e + $g + $h;

// -----
// Done.

   return $JD12;


} // End of  YYYYMMDD_To_JD12()

// ************************************************************************





// REPLACEMENT FOR YYYYMMDD_To_JD12()

   function Date_To_JD12 ($yyyymmddArg, $CalModeArg=1)

{

   $ymmdd = $yyyymmddArg;

   if ($ymmdd < 0) {$sign = -1;} else {$sign = 1;}

   $CalMode = $CalModeArg;

// -------------------------------------------------
// Extract date elements from integer date argument.

   $w = floor($sign*$ymmdd / 10000);
   $m = floor(($sign*$ymmdd - 10000*$w) / 100);
   $d = ($sign*$ymmdd - 10000*$w - 100*$m);
   $y =  $sign * $w;

// ---------------------------------
// Adjust for BC year, if indicated.

   if ($y < 0)  {$a = $y+1;} else {$a = $y;}

// ----------------------------------------------------
// Make special mathematical adjustments, if necessary.
// Jan and Feb are equated to 13th and 14th months of
// the previous year.

   $b = floor((14 - $m) / 12);
   $c = $a - $b;
   $e = floor(30.6001*($m + 12*$b + 1)) + $d;
   $f = $CalMode * floor($c/100);
   $g = floor(365.25*($c + 4716));
   $h = $CalMode*(floor($f/4) - $f + 2) - 1524;

// ----------------------------------------------
// Compute JD value for 12:00 of date.  This will
// always be an integer value not less than zero
// when within the valid span.

   $JD12 = $e + $g + $h;

// -----
// Done.

   return $JD12;


} // End of  Date_To_JD12()

// ************************************************************************







// General JD number for any given date and time of day


   function YYYYMMDD_To_JD ($yyyymmddArg, $CalModeArg, $hhArg,$mmArg,$ssArg)

{

   $ymmdd = trim($yyyymmddArg);

   $hh = trim($hhArg);
   $mm = trim($mmArg);
   $ss = trim($ssArg);

   if ($ymmdd < 0) {$sign = -1;} else {$sign = 1;}

   $CalMode = $CalModeArg;

// -------------------------------------------------
// Extract date elements from integer date argument.

   $w = floor($sign*$ymmdd / 10000);
   $m = floor(($sign*$ymmdd - 10000*$w) / 100);
   $d = ($sign*$ymmdd - 10000*$w - 100*$m);
   $y =  $sign * $w;

// ---------------------------------
// Adjust for BC year, if indicated.

   if ($y < 0)  {$a = $y+1;} else {$a = $y;}

// ----------------------------------------------------
// Make special mathematical adjustments, if necessary.
// Jan and Feb are equated to 13th and 14th months of
// the previous year.

   $b = floor((14-$m) / 12);
   $c = $a - $b;
   $e = $m + 12*$b + 1;
   $f = $CalMode * floor($c/100);
   $g = $c + 4716;

// ----------------------------------------------
// Compute JD value for 12:00 of date.  This will
// always be an integer value not less than zero
// over the valid span.

   $JD12 = floor(365.25*$g) + floor(30.6001*$e) - 1524 + $d
         + $CalMode*(floor($f/4) - $f + 2);

   $JD00 = $JD12 - 0.5;

   $TimeOfDayFrac = HMS_To_DayFrac (1, $hh, $mm, $ss);

   $JD = bcAdd($JD00, $TimeOfDayFrac, 12);

// -----
// Done.

// Trim off any redundant zeros and/or decimal point.
   $JD = RTrim(RTrim($JD, "0"), ".");

   return $JD;


} // End of  YYYYMMDD_To_JD()

// ************************************************************************


















/*

   ************************************************************************

   This function returns the encoded calendar date (ymmdd)
   corresponding to a general JD number and calendar mode.

   It is the inverse of the JD function in that it returns
   the calendar date on which the given general JD number
   occurs.

   The general JD number argument can be a non-integer
   decimal value.


   ---------
   ARGUMENTS

   $JDArg = General JD number value argument, NOT restricted
            to integer values.

   $CalModeArg = Calendar mode: (0=Julian  or  1=Gregorian)


   ------
   ERRORS

   This function does NOT check for invalid arguments.

*/



   function JD_To_YYYYMMDD ($JDArg, $CalModeArg=1)

  {

// ----------------------------------------------------
// Read calendar mode argument and force to upper case.

   $CalMode = trim($CalModeArg)+0;

// ---------------------------------------------------
// If not Julian mode, then default to Gregorian mode.

   if ($CalMode != 0) {$CalMode = 1;}

// --------------------------------
// Compute JD value for 12h on date
// from the given JD input argument.

   $JD12 = floor($JDArg + 0.5);

// --------------------------
// Account for calendar mode.

   $e = $CalMode * floor(($JD12 - 1867216.25) / 36524.25);
   $f = $JD12 + $CalMode * ($e - floor($e/4) + 1);

// -------------------------
// Auxiliary work variables.

   $g = $f + 1524;
   $h = floor(($g - 122.1) / 365.25);
   $i = floor(365.25 * $h);
   $j = floor(($g - $i) / 30.6001);

// -----------------------------------
// Compute the calendar date elements.

   $d = $g - $i - floor(30.6001 * $j);
   $m = $j - 12*floor($j/14) - 1;
   $y = $h - 4716 + floor((14 - $m) / 12);

   if ($y <= 0) {$k = $y - 1;} else {$k = $y;}

// -------------------------------
// Remember numerical sign of (y).

   if ($k < 0) {$sign = -1;} else {$sign = 1;}

// ------------------------------------------------------
// Construct encoded numerical data vector containing the
// calendar date elements in a single  integer value.

   $ymmdd = $sign * (10000*abs($k) + 100*$m + $d);

   return $ymmdd;

} // End of  JD_To_YYYYMMDD()







// REPLACEMENT FOR JD_To_YYYYMMDD()

   function JD_To_Date ($JDArg, $CalModeArg=1)

  {

// ----------------------------------------------------
// Read calendar mode argument and force to upper case.

   $CalMode = trim($CalModeArg)+0;

// ---------------------------------------------------
// If not Julian mode, then default to Gregorian mode.

   if ($CalMode != 0) {$CalMode = 1;}

// --------------------------------
// Compute JD value for 12h on date
// from the given JD input argument.

   $JD12 = floor($JDArg + 0.5);

// --------------------------
// Account for calendar mode.

   $e = $CalMode * floor(($JD12 - 1867216.25) / 36524.25);
   $f = $JD12 + $CalMode * ($e - floor($e/4) + 1);

// -------------------------
// Auxiliary work variables.

   $g = $f + 1524;
   $h = floor(($g - 122.1) / 365.25);
   $i = floor(365.25 * $h);
   $j = floor(($g - $i) / 30.6001);

// -----------------------------------
// Compute the calendar date elements.

   $d = $g - $i - floor(30.6001 * $j);
   $m = $j - 12*floor($j/14) - 1;
   $y = $h - 4716 + floor((14 - $m) / 12);

   if ($y <= 0) {$k = $y - 1;} else {$k = $y;}

// -------------------------------
// Remember numerical sign of (y).

   if ($k < 0) {$sign = -1;} else {$sign = 1;}

// ------------------------------------------------------
// Construct encoded numerical data vector containing the
// calendar date elements in a single  integer value.

   $ymmdd = $sign * (10000*abs($k) + 100*$m + $d);

   return $ymmdd;

} // End of  JD_To_Date()
// ************************************************************************









/*

   ************************************************************************
   This function returns the day of the week
   name string ('Sunday' to 'Saturday') for
   any given calendar date and mode.

   ----------
   DEPENDENCY

   This function calls the function

   YYYYMMDD_To_JD12()


   ------
   ERRORS

   This function does NOT check for invalid arguments.

*/


   function YYYYMMDD_To_DoW ($yyyymmddArg, $CalModeArg=1)

{

// --------------------------
// Define weekday names list.

   $weekdays = "Sunday   Monday   Tuesday  WednesdayThursday Friday   Saturday";

// ------------------
// Compte JD12 value.

   $jd12 = YYYYMMDD_To_JD12($yyyymmddArg, $CalModeArg);

// -----------------------------------------------------
// Get day of week name substring from ($weekdays) list.

   $DoW = trim(substr($weekdays, 9*(($jd12 + 1) % 7), 9));

// -----
// Done.

   return $DoW;

}

// ************************************************************************


















// ************************************************************************

   function HMS_To_DayFrac ($SignNumArg, $hhArg, $mmArg, $ssArg)

{

// -----------------------------------------------
// Read numerical sign argument.
// Where:  -1=Neg  or  1=Pos

   $sign = ($SignNumArg < 0)? '-' : '+';


// ----------------------------
// Read time element arguments.

   $hh = trim($hhArg);
   $mm = trim($mmArg);
   $ss = trim($ssArg);


// ------------------------------------------
// Compute day fraction corresponding to time

   $w1 = bcMul("3600", $hh, 24);
   $w2 = bcMul("60",   $mm, 24);

   $w3 = bcAdd($w1, $w2, 24);
   $w4 = bcAdd($w3, $ss, 24);

   $w5 = bcDiv($w4, "86400", 24);

   $w5 = bcAdd($w5, "0.00000000000000005", 16);

   $DayFrac = RTrim(RTrim($w5, "0"), ".");

   return "$sign$DayFrac";

} // End of  HMS_To_DayFrac ()

// ************************************************************************

















// ************************************************************************

// This function returns the number of days for any given
// year, month and calendar mode.

   function Days_In_Month ($yArg, $mNumArg,  $CalModeArg=1)

{

// Read input arguments.
   $CalMode = $CalModeArg;
   $y       = $yArg;
   $m       = floatval(LTrim($mNumArg, "0"));

// Get number of days in month (m).
   $days = substr("312831303130313130313031", 2*($m-1), 2);

// PERFORM LEAP YEAR TEST ACCORDING TO CALENDAR MODE

// Adjust BC/AD calendar year value, if necessary.
// The value w = Mathematical year value.
   $w = ($y > 0)? $y : $y+1;

// Perform standard Julian leap year test.
   if ($w % 4 == 0) {$LeapFlag=1;} else {$LeapFlag=0;}

// If Gregorian mode, then check for special
// case of century year.  Otherwise, skip
// this test.
   if ($w % 100 == 0 && $CalMode == 1)
      {
       if ($w % 400 == 0)  {$LeapFlag=1;} else {$LeapFlag=0;}
      }

// Account for February, if necessary.
   if ($m == 2) {$days += $LeapFlag;}

// Done.
   return $days;

} // End of  Days_In_Month()










// ************************************************************************





// This function checks if a given year is a leap year.
// It returns TRUE if a leap year or FALSE otherwise.
//
// NOTE:
// Convention used is: Negative year = BC year

function is_leap_year ($yArg, $CalModeArg)

{

   $CalMode = $CalModeArg;
         $y = $yArg;

// Adjust BC/AD year value, if necessary.
// w = Mathematical year value.
   $w = ($y > 0)? $y : $y + 1;

// Perform standard Julian leap year test.
   if ($w % 4 == 0) {$LeapYearFlag = TRUE;} else {$LeapYearFlag = FALSE;}

// Finished if Julian calendar mode.
// Otherwise, drop through for Gregorian.
   if ($CalMode == 0) {return $LeapYearFlag;}

// Check for Gregorian century year.
   if ($w % 100 == 0)
      {
       if   ($w % 400 == 0)  {$LeapYearFlag = TRUE;}
       else {$LeapYearFlag = FALSE;}
      }

// Done.
   return $LeapYearFlag;

} // End of  is_leap_year()






// This function checks if a given year is a leap year.
// It returns 1 if a leap year or 0 otherwise.
//
// NOTE:
// Convention used is: Negative year = BC year

function is_leap ($yArg, $CalModeArg)

{

   $CalMode = $CalModeArg;
         $y = $yArg;

// Adjust BC/AD year value, if necessary.
// w = Mathematical year value.
   $w = ($y > 0)? $y : $y + 1;

// Perform standard Julian leap year test.
   if ($w % 4 == 0) {$LeapYearFlag = 1;} else {$LeapYearFlag = 0;}

// Finished if Julian calendar mode.
// Otherwise, drop through for Gregorian.
   if ($CalMode == 0) {return $LeapYearFlag;}

// Check for Gregorian century year.
   if ($w % 100 == 0)
      {
       if   ($w % 400 == 0)  {$LeapYearFlag = 1;}
       else {$LeapYearFlag = 0;}
      }

// Done.
   return $LeapYearFlag;

} // End of  is_leap()











/*

************************************************************************
This function converts decimal days into a
time data vector in the general format:

"wk days hours minutes seconds"

-------
EXAMPLE

input  = "16071.1748842592592593"

output = "2295wk 6d 04h 11m 50s"

Resolution is to nearest second.

*/


   function Days_To_WkDHMS ($DaysArg)
{

// ------------------------------------
// Get working value of numerical input
// string, assumed to be in decimal days.
   $Days = trim($DaysArg);

// ----------------------------
// Equate an empty input string
// to the digit zero.
   if ($Days == "") {$Days = 0;}

// ------------------------
// Remember numerical sign.
   $sign = ($Days < 0) ? -1 : +1;

// --------------------------------------------------
// Convert working value ($DecDays) into its absolute
// value at 24-decimals of precision.
   $Days = bcMul($sign, $Days, 24);

// ---------------------------------------------------
// Compute weeks, residual days and fraction of a day.
   $wk = bcDiv($Days, "7");
   $rd = bcSub($Days, bcMul($wk, "7"), 24);
    $d = bcAdd($rd, "0");
   $fd = bcSub($rd, $d, 16);

// -----------------------------------
// Compute HMS  time string equivalent
// to any residual day fraction.
   $hms = Days_To_HMS ($fd, 0);

// ------------------------------------
// Construct output data vector string,
// recalling original numerical sign.
   $q = "$wk" . "wk  $d" . "d  $hms";
   $q = ($sign < 0) ? "-" . $q : $q;

// ---------
// Finished.
   return $q;

} // End of Days_To_WkDHMS()

// ************************************************************************





/*

   ************************************************************************
   Return hours, minutes and seconds corresponding to a given
   decimal days value.  The argument can be either negative
   or positive.

   The absolute value of the returned result can be > 24 hours.

   The returned seconds value may optionally be rounded off to
   up to 3 decimals.  If no decimals are specified, then the
   default zero value will be used.

*/

   function Days_To_HMS ($DaysArg, $NumDecimalsArg=0)

{

// ---------------------------
// Read input argument values.
   $days = trim($DaysArg);
   $d = floor(abs((trim($NumDecimalsArg))));

// --------------------------------------
// If days argument is empty, then set to
// default zero value.
   if ($days == "") {$days = 0;}

// ---------------------------------------
// If decimals argument is empty, then set
// to default zero value.
   if ($d == "") {$d = 0;}

// ---------------------
// If decimals > 3, then
// set to default 3 limit.
   if ($d > 3) {$d = 3;}

// ------------------------
// Remember numerical sign.
   if ($days < 0) {$sign = "-";} else {$sign = "";}

// -------------------------------------------------
// Work with absolute days value.  Any negative sign
// will be reattached to the returned result
   $days = abs($days);

// --------------------
// Compute hours value.
   $w   = $days * 24;
   $hh  = floor($w);

// ----------------------
// Compute minutes value.
   $w   = 60*($w - $hh);
   $mm  = floor($w);

// ----------------------
// Compute seconds value.
   $ss  = 60*($w - $mm);

// ------------------------------------
// Round seconds value to given number
// of decimals.
   $ss = sprintf ("%01.$d" . "f", $ss);
   if ($ss < 10) {$ss = "0$ss";}

// --------------------------------------
// Account for that annoying 60s problem.
   if ($ss == 60) {$mm += 1;  $ss = 0;}
   if ($mm == 60) {$hh += 1;  $mm = 0;}

// ------------------------------------
// Format the time elements for output.
   if ($hh < 10) {$hh = "0$hh";}
   if ($mm < 10) {$mm = "0$mm";}

   $ss = sprintf ("%01.$d" . "f", $ss);

   if ($ss < 10) {$ss = "0$ss";}

   $hh .= "h";  $mm .= "m";  $ss .= "s";

// ------------------------
// Done.
   return "$sign$hh $mm $ss";

} // End of  Days_To_HMS()

// ************************************************************************






/*

************************************************************************
Return full month name string (January to December)
corresponding to a given month argument.

The month argument may be a number from 1 to 12
or a 3-letter abbreviation of the month name.

Month names/abbreviations are NOT case sensitive.

If the argument is already an abbreviation from Jan to Dec,
then it is returned as is.  If it is a number, then the
corresponding month abbreviation from Jan to Dec is returned.

------
ERRORS

Boolean FALSE is returned if the month argument
is invalid.

*/

   function Month_Str ($MonthArg)
{

// ------------------------------------------
// Define 3-letter month abbreviations table.
   $months = "JanFebMarAprMayJunJulAugSepOctNovDec";

// ---------------------------------------------
// Get up to the first 3 characters of the month
// argument and force any text to upper case.
   $m = StrToUpper(substr($MonthArg, 0,3));

// -------------------------------------------------
// If month is already in the correct numeric range,
// then return its 3-letter abbreviation.
   if ($m >= 1 && $m < 13)
      {return SubStr($months, 3*($m-1), 3);}

// OTHERWISE, MONTH MUST BE A NON-NUMERIC STRING

// ----------------------------------------------
// Error if month abbreviation is < 3 characters.
   if (StrLen($m) < 3) {return FALSE;}

// --------------------------------------
// Get 3-letter month abbreviation index.
   $i = StrPos(StrToUpper($months), $m);

// ------------------------------------
// Error if invalid month abbreviation.
   if ($i === FALSE) {return FALSE;}

// Get month number for abbreviation.
//   $m = 1 + ($i / 3);

// --------------------------
// Replace valid month number
// with the corresponding valid
// month abbreviation: Jan to Dec
   $m = SubStr($months, $i, 3);

// -----
// Done.
   return $m;

} // End of  Month_Str()

// ************************************************************************













   function Month_String ($MonthNumArg)
{

// -------------------------
// Define month names table.
   $months = "January  Febuary  March    April    May      June     July     August   SeptemberOctober  November December ";

// -------------------------
// Get month numberargument.
   $m = trim($MonthNumArg);

// -------------------------------------------------
// If month is already in the correct numeric range,
// then return the month name string.
   if ($m >= 1 && $m < 13)

      {return trim(SubStr($months, 9*($m-1), 9));}
   else
      {return "";}


} // End of  Month_String()














/*

************************************************************************
Return month number (1 to 12) corresponding to
a given month argument.

The month argument may be a number from 1 to 12
or a 3-letter abbreviation of the month name.

Month abbreviations are NOT case sensitive.

If the argument is already a number from 1 to 12,
then it is returned as is.  If it is a 3-letter
abbreviation, then the corresponding month number
from 1 to 12 is returned.

------
ERRORS

Boolean FALSE is returned if the month argument
is invalid.

*/

   function Month_Num ($MonthArg)
{

// ---------------------------------------------
// Get up to the first 3 characters of the month
// argument and force any text to upper case.
   $m = StrToUpper(substr($MonthArg, 0,3));

// -------------------------------------------------
// If month is already in the correct numeric range,
// then return its floored (integer) value.
   if ($m >= 1 && $m < 13) {return floor($m);}

// ----------------------------------------------
// Error if month abbreviation is < 3 characters.
   if (StrLen($m) < 3) {return FALSE;}

// --------------------------------------
// Get 3-letter month abbreviation index.
   $i = StrPos("JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC", $m);

// ------------------------------------
// Error if invalid month abbreviation.
   if ($i === FALSE) {return FALSE;}

// --------------------------------
// Replace valid month abbreviation
// with the corresponding valid month
// number from 1 to 12.
   {$m = 1 + ($i / 3);}

// -----
// Done.
   return $m;

} // End of  Month_Num()


// ************************************************************************














/*

   ************************************************************************
   Compute the mean  obliquity of the ecliptic in degrees for the
   current JD number argument at any instant.

   This ecliptic obliquity function is based on a formula derived
   by J. Laskar, Astronomy and Astrophysics, 1968, Vol. 157, p68.

   The estimated accuracy of this formula is 0.01 arc second between
   the years 1000 AD and 3000 AD and a few arc seconds after 10,000
   years.  It is only valid in the range 10,000 years either way
   of J2000.0

   Over the long term, it is more accurate than the formula
   adopted by the International Astronomical Union (IAU)
   for general ephemeris computations.

   ---------
   ARGUMENTS

   $JDArg = General JD number for date and time.


   ------
   ERRORS

   An error results and FALSE is returned if the
   JD argument is not a valid numeric string.

*/


   function Epsilon_Mean ($JDArg)

{


// ------------------------------------
// ERROR if JD argument is not numeric.

   if (!is_numeric($JDArg)) {return FALSE;}

// ----------------------------------------------
//  Compute the time in Julian centuries reckoned
//  from J2000.0 corresponding to the JD argument
//  and divide by 100.

   $t = ($JDArg - 2451545.0) / 36525.0 / 100.0;

// -----------------------------
// Compute mean obliquity of the
// ecliptic in arc seconds.

   $p = $t*$t;

   $EpsMeanSS = 84381.448 - 4680.93*$t;
   $EpsMeanSS -=    1.55*$p;  $p *= $t;
   $EpsMeanSS += 1999.25*$p;  $p *= $t;
   $EpsMeanSS -=   51.38*$p;  $p *= $t;
   $EpsMeanSS -=  249.67*$p;  $p *= $t;
   $EpsMeanSS -=   39.05*$p;  $p *= $t;
   $EpsMeanSS +=    7.12*$p;  $p *= $t;
   $EpsMeanSS +=   27.87*$p;  $p *= $t;
   $EpsMeanSS +=    5.79*$p;  $p *= $t;
   $EpsMeanSS +=    2.45*$p;

// ---------------------------------
// Compute mean obliquity in degrees
// from the arc seconds value.

   $EpsMeanDeg = $EpsMeanSS / 3600.0;

// -----
// Done.

   return $EpsMeanDeg;

} // End of  Epsilon_Mean()

// ************************************************************************








/*

   *************************************************
   Compute the nutation in Epsilon, Delta Epsilon or
   nutation in obliquity of the ecliptic, in degrees.

   This is the correction to apply to the MEAN value
   of the obliquity to obtain the TRUE obliquity.

   The general precision level is  0.001 arc sec.

   This function is based on the 1980 IAU theory.

   $t  = Julian time factor as measured from J2000.0
   $t2 = $t to the power of 2
   $t3 = $t to the power of 3

   ==============================================
   Nutation correction elements (1980 IAU Theory)

   $w  = Nutational series accumulator
   $w1 = Mean elongation of the moon from the sun
   $w2 = Mean anomaly of the sun
   $w3 = Mean anomaly of the moon
   $w4 = Argument of latitude of the moon
   $w5 = Longitude of lunar ascending node

*/

   Function Delta_Epsilon ($JDArg)
{
   $T1 = ($JDArg - 2451545) / 36525;
   $T2 = $T1*$T1;
   $T3 = $T2*$T1;

// -----------------------------------------
// Compute mean lunar elongation in radians.

   $w1 = 297.85036 + 445267.11148*$T1 - 0.0019142*$T2 + ($T3 / 189474);
   $w1 = Deg2Rad($w1);

// --------------------------------------
// Compute mean solar anomaly in radians.

   $w2 = 357.52772 + 35999.05034*$T1 - 0.0001603*$T2 - ($T3 / 300000);
   $w2 = Deg2Rad($w2);

// --------------------------------------
// Compute mean lunar anomaly in radians.

   $w3 = 134.96298 + 477198.867398*$T1 + 0.0086972*$T2 + ($T3 / 56250);
   $w3 = Deg2Rad($w3);

// ----------------------------------------------
// Compute lunar argument of latitude in radians.

   $w4 = 93.27191 + 483202.017538*$T1 - 0.0036825*$T2 + ($T3 / 327270);
   $w4 = Deg2Rad($w4);

// -----------------------------------------------------
// Compute longitude of lunar ascending node in radians.

   $w5 = 125.04452 - 1934.136261*$T1 + 0.0020708*$T2 + ($T3 / 450000);
   $w5 = Deg2Rad($w5);

// ---------------------------------------------------
// Compute the nutation in obliquity (ArcSec * 10000).

   $w  = cos($w5)*(92025 + 8.9*$T1);
   $w += cos(2*($w4 - $w1 + $w5))*(5736 - 3.1*$T1);
   $w += cos(2*($w4 + $w5))*(977 - 0.5*$T1);
   $w += cos(2*$w5)*(0.5*$T1 - 895);
   $w += cos($w2)*(54 - 0.1*$T1);
   $w += cos($w2 + 2*($w4 - $w1 + $w5))*(224 - 0.6*$T1);
   $w += cos($w3 + 2*($w4 + $w5))*(129 - 0.1*$T1);
   $w += cos(2*($w4 - $w1 + $w5) - $w2)*(0.3*$T1 - 95);
   $w += 200*cos(2*$w4 + $w5);
   $w -=  70*cos(2*($w4 - $w1) + $w5);
   $w -=  53*cos(2*($w4 + $w5) - $w3);
   $w -=  33*cos($w3 + $w5);
   $w +=  26*cos(2*($w1 + $w4 + $w5) - $w3);
   $w +=  32*cos($w5 - $w3);
   $w +=  27*cos($w3 + 2*$w4 + $w5);
   $w -=  24*cos(2*($w4 - $w3) + $w5);
   $w +=  16*cos(2*($w1 + $w4 + $w5));
   $w +=  13*cos(2*($w3 + $w4 + $w5));
   $w -=  12*cos($w3 + 2*($w4 - $w1 + $w5));
   $w -=  10*cos(2*$w4 + $w5 - $w3);
   $w -=   8*cos(2*$w1 - $w3 + $w5);
   $w +=   7*cos(2*($w2 - $w1 + $w4 + $w5));
   $w -=   7*cos($w3);
   $w +=   9*cos($w2 + $w5);
   $w +=   7*cos($w3 + $w5 - 2*$w1);
   $w +=   6*cos($w5 - $w2);
   $w +=   5*cos(2*($w1 + $w4) - $w3 + $w5);
   $w +=   3*cos($w3 + 2*($w4 + $w1 + $w5));
   $w -=   3*cos($w2 + 2*($w4 + $w5));
   $w +=   3*cos(2*($w4 + $w5) - $w2);
   $w +=   3*cos(2*($w1 + $w4) + $w5);
   $w -=   3*cos(2*($w3 + $w4 + $w5 - $w1));
   $w -=   3*cos($w3 + 2*($w4 - $w1) + $w5);
   $w +=   3*cos(2*($w1 - $w3) + $w5);
   $w +=   3*cos(2*$w1 + $w5);
   $w +=   3*cos(2*($w4 - $w1) + $w5 - $w2);
   $w +=   3*cos($w5 - 2*$w1);
   $w +=   3*cos(2*($w3 + $w4) + $w5);

// -----------------------------------
// Done.  Return nutation in obliquity
// expressed in degrees.

   return $w / 36000000;

} // End of  Delta_Epsilon()








/*

   Compute nutation in ecliptical longitude in decimal
   degrees.  The precision is to 0.001 arc second.

   This function is based on the 1980 IAU theory.

   $t  = Julian centuries since J2000.0
   $t2 = $t to the power of 2
   $t3 = $t to the power of 3

   Nutation correction elements (1980 IAU Theory)
   $w  = Nutational series accumulator
   $w1 = Mean elongation of the moon from the sun
   $w2 = Mean anomaly of the sun
   $w3 = Mean anomaly of the moon
   $w4 = Argument of lunar latitude
   $w5 = Longitude of lunar ascending node

   ERRORS:
   An error results if the JD argument is not numeric.
   On error, FALSE is returned.

*/


   function Delta_Psi ($JDArg)
{

// -----------------------
// Read JD argument value.
// Error if argument is not numeric.

   if (!Is_Numeric(trim($JDArg))) {return FALSE;}

// -------------------------------------------------------
// Compute powers of time in Julian centuries from J2000.0

   $T1 = ($JDArg - 2451545) / 36525;
   $T2 = $T1*$T1;
   $T3 = $T2*$T1;

// -----------------------------------------
// Compute mean lunar elongation in radians.

   $w1 = 297.85036 + 445267.11148*$T1 - 0.0019142*$T2 + ($T3 / 189474);
   $w1 = Deg2Rad($w1);

// --------------------------------------
// Compute mean solar anomaly in radians.

   $w2 = 357.52772 + 35999.05034*$T1 - 0.0001603*$T2 - ($T3 / 300000);
   $w2 = Deg2Rad($w2);

// --------------------------------------
// Compute mean lunar anomaly in radians.

   $w3 = 134.96298 + 477198.867398*$T1 + 0.0086972*$T2 + ($T3 / 56250);
   $w3 = Deg2Rad($w3);

// ----------------------------------------------
// Compute lunar argument of latitude in radians.

   $w4 = 93.27191 + 483202.017538*$T1 - 0.0036825*$T2 + ($T3 / 327270);
   $w4 = Deg2Rad($w4);

// ----------------------------------------
// Compute lunar ascending node in radians.

   $w5 = 125.04452 - 1934.136261*$T1 + 0.0020708*$T2 + ($T3 / 450000);
   $w5 = Deg2Rad($w5);

// ---------------------------------------------
// Compute the nutation in ecliptical longitude.

   $w  = sin($w5)*(-174.2*$T1 - 171996);
   $w += sin(2*($w4 + $w5 - $w1))*(-1.6*$T1 - 13187);
   $w += sin(2*($w4 + $w5))*(-2274 - 0.2*$T1);
   $w += sin(2 * $w5)*(0.2*$T1 + 2062);
   $w += sin($w2)*(1426 - 3.4*$T1);
   $w += sin($w3)*(0.1*$T1 + 712);
   $w += sin(2*($w4 + $w5 - $w1) + $w2)*(1.2*$T1 - 517);
   $w += sin(2 * $w4 + $w5)*(-0.4*$T1 - 386);
   $w += sin(2*($w4 + $w5 - $w1) - $w2)*(217 - 0.5*$T1);
   $w += sin(2*($w4 - $w1) + $w5)*(129 + 0.1*$T1);
   $w += sin($w3 + $w5)*(0.1*$T1 + 63);
   $w += sin($w5 - $w3)*(-0.1*$T1 - 58);
   $w += sin(2*$w2)*(17 - 0.1*$T1);
   $w += sin(2*($w2 + $w4 + $w5 - $w1))*(0.1*$T1 - 16);
   $w -= 301*sin(2*($w4 + $w5) + $w3);
   $w -= 158*sin($w3 - 2*$w1);
   $w += 123*sin(2*($w4 + $w5) - $w3);
   $w +=  63*sin(2*$w1);
   $w -=  59*sin(2*($w1 + $w4 + $w5) - $w3);
   $w -=  51*sin(2 * $w4 + $w3 + $w5);
   $w +=  48*sin(2*($w3 - $w1));
   $w +=  46*sin(2*($w4 - $w3) + $w5);
   $w -=  38*sin(2*($w1 + $w4 + $w5));
   $w -=  31*sin(2*($w3 + $w4 + $w5));
   $w +=  29*sin(2*$w3);
   $w +=  29*sin(2*($w4 + $w5 - $w1) + $w3);
   $w +=  26*sin(2*$w4);
   $w -=  22*sin(2*($w4 - $w1));
   $w +=  21*sin(2*$w4 + $w5 - $w3);
   $w +=  16*sin(2*$w1 - $w3 + $w5);
   $w -=  15*sin($w2 + $w5);
   $w -=  13*sin($w3 + $w5 - 2*$w1);
   $w -=  12*sin($w5 - $w2);
   $w +=  11*sin(2*($w3 - $w4));
   $w -=  10*sin(2*($w4 + $w1) + $w5 - $w3);
   $w -=   8*sin(2*($w4 + $w1 + $w5) + $w3);
   $w +=   7*sin(2*($w4 + $w5) + $w2);
   $w -=   7*sin($w3 - 2*$w1 + $w2);
   $w -=   7*sin(2*($w4 + $w5) - $w2);
   $w -=   7*sin(2*$w1 + 2*$w4 + $w5);
   $w +=   6*sin(2*$w1 + $w3);
   $w +=   6*sin(2*($w3 + $w4 + $w5 - $w1));
   $w +=   6*sin(2*($w4 - $w1) + $w3 + $w5);
   $w -=   6*sin(2*($w1 - $w3) + $w5);
   $w -=   6*sin(2*$w1 + $w5);
   $w +=   5*sin($w3 - $w2);
   $w -=   5*sin(2*($w4 - $w1) + $w5 - $w2);
   $w -=   5*sin($w5 - 2*$w1);
   $w -=   5*sin(2*($w3 + $w4) + $w5);
   $w +=   4*sin(2*($w3 - $w1) + $w5);
   $w +=   4*sin(2*($w4 - $w1) + $w2 + $w5);
   $w +=   4*sin($w3 - 2*$w4);
   $w -=   4*sin($w3 - $w1);
   $w -=   4*sin($w2 - 2*$w1);
   $w -=   4*sin($w1);
   $w +=   3*sin(2 * $w4 + $w3);
   $w -=   3*sin(2*($w4 + $w5 - $w3));
   $w -=   3*sin($w3 - $w1 - $w2);
   $w -=   3*sin($w2 + $w3);
   $w -=   3*sin(2*($w4 + $w5) + $w3 - $w2);
   $w -=   3*sin(2*($w1 + $w4 + $w5) - $w2 - $w3);
   $w -=   3*sin(2*($w4 + $w5) + 3*$w3);
   $w -=   3*sin(2*($w1 + $w4 + $w5) - $w2);

// ----------------------------------
// Done.  Return ecliptical longitude
// expressed in degrees.

   return $w / 36000000;

  } // End of  Delta_Psi()













// ----------------------------------------------------
// Compute nutation in right ascension at the celestial
// equator, also known as the equation of the equinoxes,
// in degrees.
//
// This is also called the equation of the equinoxes and
// is the difference (True Equinox - Mean Equinox).
//
// NOTE:
// Degrees  240 = Seconds of RA

   function Delta_Equinox ($JDArg)
{

// ---------------------------------------------
// Compute cosine of true obliquity of ecliptic.
   $CosEps = cos(Deg2Rad(Epsilon_Mean($JDArg) + Delta_Epsilon($JDArg)));

// ---------------------------------------
// Done.  Return equation of the equinoxes
// in degrees.
   return Delta_Psi($JDArg) * $CosEps;

} // End of  Delta_Equinox()














/*
   ==========================================
   This function returns the correction to be
   applied to the ecliptical longitude to
   reduce it from the dynamical system to
   the FK5 system of coordinates.

*/

   Function FK5_Lng_Corr ($JDArg, $EclLngDeg, $EclLatDeg)

{
// Compute ($t) in Julian centuries
// from J2000.0
   $t = ($JDArg - 2451545.0) / 36525;

// Read ecliptical latitude argument.
   $b = Deg2Rad($EclLatDeg);

   $Lprime = Deg2Rad($EclLngDeg - 1.397*$t - 0.00031*$t*$t);

// Compute longitude correction in degrees.
   $w = (0.03916*(cos($Lprime) + sin($Lprime))*tan($b) - 0.09033)
      / 3600;

// Return the longitude
// correction.
   return $w;

} // End of  FK5_Lng_Corr()







/*
   ==========================================
   This function returns the correction to be
   applied to the ecliptical latitude to
   reduce it from the dynamical system to
   the FK5 system of coordinates.
*/

   Function FK5_Lat_Corr ($JDArg, $EclLngDeg)
{
// -------------------------------------
// Compute Julian centuries from J2000.0
   $t = ($JDArg - 2451545) / 36525;

// ------------------------
// Compute auxiliary angle.
   $LPrime = Deg2Rad($EclLngDeg - 1.397*$t - 0.00031*$t*$t);

// -------------------------------------------------
// Done.  Return the latitude correction in degrees.
   return 0.03916*(cos($LPrime) - sin($LPrime)) / 3600;

} // End of  FK5_Lat_Corr()















/*

=======================================
This function is designed to modulate a
negative or positive angle expressed in
degrees to fall within the standard
range from 0 to +360 degrees.

*/

   function Deg_Mod_360 ($DegArg)
{

// ----------------------
// Read degrees argument.
   $x = trim($DegArg);

// --------------------
// Set internal working
// decimals limit.
   $d = 24;

// --------------------------------------
// Remember numerical  sign of argument.
   $sign = ($x < 0) ? -1 : 1;

// -------------------------------------
// Work with absolute value of argument.
   $x = bcMul($sign, $x, $d);

// ------------------------
// Compute (Angle MOD 360).
   $w = bcSub($x, bcMul(bcDiv($x, "360"), "360"), $d);

// ------------------------
// Recall numerical  sign.
   if ($sign < 0) {$w = bcSub("360", $w, $d);}

// ----------------------------
// Done.  Return result rounded
// to 16 decimals.

   return bcAdd($w, "0.00000000000000005", 16);


} // End of  Deg_Mod_360()











/*

========================================================
This function converts geocentric ecliptical coordinates
into the corresponding geocentric equatorial coordinates.

Angular arguments are in decimal degrees and results are
returned in decimal degrees.


---------
ARGUMENTS

$EclLngDegArg = Geocentric ecliptical longitude in degrees.
$EclLatDegArg = Geocentric ecliptical latitude in degrees.
$EpsDegArg    = Obliquity of the ecliptic in degrees.


------
ERRORS

This function does NOT check for invalid arguments.

*/



   function Ecl_To_Equ_Coords ($EclLngDegArg, $EclLatDegArg, $EpsDegArg)

  {

// ------------------------------
// Convert input ecliptical angle
// arguments into radians.

   $Lng = Deg2Rad(trim($EclLngDegArg));
   $Lat = Deg2Rad(trim($EclLatDegArg));
   $Eps = Deg2Rad(trim($EpsDegArg));

// -------------------------------------
// Compute the equatorial RA in degrees.

   $y = sin($Lng)*cos($Eps) - tan($Lat)*sin($Eps);
   $x = cos($Lng);
   $RADeg = Rad2Deg(ATan2($y, $x));
   if ($RADeg < 0) {$RADeg += 360;}

// -----------------------------------
// Compute the equatorial declination.

   $w = ASin(sin($Lat)*cos($Eps) + cos($Lat)*sin($Lng)*sin($Eps));
   $DeclDeg = Rad2Deg($w);

// ---------------------------------------------
// Done.  Return the RA and declination, both in
// degrees, in a space delimited data vector.

   return "$RADeg $DeclDeg";

  } // End of  Ecl_To_Equ_Coords()












   function EclLng_To_EquDecl ($EclLngDegArg, $EclLatDegArg, $EpsDegArg)

  {

// ------------------------------
// Convert input ecliptical angle
// arguments into radians.

   $Lng = Deg2Rad($EclLngDegArg);
   $Lat = Deg2Rad($EclLatDegArg);
   $Eps = Deg2Rad($EpsDegArg);

// ----------------------------------------------
// Compute the equatorial declination in radians.

   $w = ASin( sin($Lat)*cos($Eps) + cos($Lat)*sin($Lng)*sin($Eps) );

// -------------------
// Convert to degrees.

   $DeclDeg = Rad2Deg($w);

// -----
// Done.

   return "$DeclDeg";

  } // End of  EclLng_To_EquDecl()








   function EclLat_To_EquRA ($EclLngDegArg, $EclLatDegArg, $EpsDegArg)

  {

// ------------------------------
// Convert input ecliptical angle
// arguments into radians.

   $Lng = Deg2Rad(trim($EclLngDegArg));
   $Lat = Deg2Rad(trim($EclLatDegArg));
   $Eps = Deg2Rad(trim($EpsDegArg));

// -------------------------------------
// Compute the equatorial RA in degrees.

   $y = sin($Lng)*cos($Eps) - tan($Lat)*sin($Eps);
   $x = cos($Lng);

   $RADeg = Rad2Deg(ATan2($y, $x));

   if ($RADeg < 0) {$RADeg += 360;}

// -----
// Done.

   return "$RADeg";

  } // End of  EclLat_To_EquRA()






















// =================================================
// This BC math function adds a sequence of numbers
// to the specified decimals of precision.
//
// Data string format = "1.1  2.22  3.333"
// Series of space-delimited numerical arguments
// to be summed.

   function bcAdd_Seq ($SeqStrArg, $NumDecimalsArg)

{

// ----------------------------------------------
// Remove redundant white space from data vector.

   $SeqStr = preg_replace('/\s+/', ' ', trim($SeqStrArg));

// -----------------------
// Get number of decimals.

   $d = trim($NumDecimalsArg);

// -----------------------------------------------
// Separate items to be summed into working array.

   $item = preg_split ("[ ]", trim($SeqStr));

// -------------------------
// Count items to be summed.

   $ItemCount = count($item);

// --------------------------------
// Handle case of empty items list.

   if ($ItemCount == 0) {return "0";}

// -----------------------------------
// Handle case of single item in list.

   if ($ItemCount == 1) {return $item[0];}

// -------------------
// Sum the items list.

   $sum = "0";

   for ($i=0;  $i < $ItemCount;   $i++)
       {
        $sum = bcAdd($sum, $item[$i], $d+8);
       }

   return bcAdd("0", $sum, $d);

} // End of  bcAdd_Seq()











// ==============================================
// This BC math function multiplies a sequence of
// numbers to the specified decimals of precision.
//
// Data string format = "1.1  2.22  3.333"
// Series of space-delimited numerical arguments
// to be multiplied.

   function bcMul_Seq ($SeqStrArg, $NumDecimalsArg)

{
   $SeqStr = preg_replace('/\s+/', ' ', trim($SeqStrArg));

   $d = trim($NumDecimalsArg);

   $item = preg_split ("[ ]", $SeqStr);

   $ItemCount = count($item);

   $prod = "1";

   if ($ItemCount == 0) {return "0";}

   if ($ItemCount == 1) {return bcAdd("0", $item[0], $d);}

   for ($i=0;  $i < $ItemCount;   $i++)
       {
        $prod = bcMul($prod, $item[$i], $d+5);
       }

   $prod = bcAdd("0", $prod, $d);

   return $prod;

} // End of  bcMul_Seq()














/*

==============================================
This function computes the arbitrary precision
factorial value for any given positive integer.

It is not advised NOT to use extreme ($PosIntArg)
values.  They will time-out after 30 seconds.

This function can be used for most any practical
problems that involve factorials.

------
ERRORS

On error, FALSE is returned.
An error occurs if the argument is
not a valid positive integer value.

*/

   function bcFactorial ($PosIntArg)

{

// --------------------
// Read argument value.

   $j = trim($PosIntArg);

// --------------------------------------
// Set empty argument to zero by default.

   if ($j == "") {$j = "0";}

// ---------------------------------
// Error if argument is non-numeric.

   if (!Is_Numeric($j)) {return FALSE;}

// ----------------------------------------
// Trim off redundant zeros and/or decimal.

   if (StrPos($j, ".") !== FALSE)
      {
       $j = RTrim(RTrim($j, "0"), ".");
      }

// ----------------------------------------------------
// Error if modified argument still contains a decimal.

   if (StrPos($j, ".") !== FALSE) {return FALSE;}

// ---------------------
// Error if argument < 0

   if ($j < 0) {return FALSE;}

// ---------------------------------
// Handle cases of argument = 0 or 1

   if ($j <= 1)  {return "1";}

// --------------------------
// Compute factorial product.

   $prod = "1";

   for ($i=1;   $i <= $j;   $i++)
       {
        $prod = bcMul($prod, $i);
       }

// -----
// Done.

   return $prod;

} // End of  bcFactorial()









/*
   ********************************************************
   This function computes the greatest common divisor (GCD)
   of any two arbitrary precision integer strings (a and b).
   The integers can be any non-zero values and the GCD will
   also be a non-zero integer at least 1.


   Test:

   $A = "528";
   $B = "328";

   print bcGCD ($A, $B);


*/


   function bcGCD ($IntArgA, $IntArgB)

{
   $A = str_replace("-", "", $IntArgA);
   $B = str_replace("-", "", $IntArgB);

   while ($B != 0)

         {$C=$B;     $B = bcMod($A, $B);     $A=$C;}

   return $A;

} // End of  bcGCD()





/*
   *****************************************
   This function reduces an integer fraction
   (a/b) to its lowest possible terms.


   DEPENDENCY:

   This function calls:

   bcGCD()


*/

   function bcReduce_Frac ($FracStrArg)

{
   $frac = split ("[/]", $FracStrArg);

   $A = $frac[0];
   $B = $frac[1];

   if ($A < 0 && $B < 0)
      {
       $A = str_replace("-", "", $A);
       $B = str_replace("-", "", $B);
      }

   $GCD = bcGCD ($A, $B);

// ---------------------------------------
// Reduce the (A,B) values to lowest terms
// by dividing them each by the GCD value,
// replacing the original (A,B) values.
// If GCD=1,then the fraction cannot be
// reduced to any lower terms and the
// fraction string remains unchanged.

   $A = bcDiv($A, $GCD);
   $B = bcDiv($B, $GCD);

// ----
// Done.

   return "$A/$B";

} // End of  bcReduce_Frac()



















// ------------------------------------------------
// Short text version calendar for use with the PRE
// tag for W3C compliance.  For positive years only.
//
// CalModeArg = Calendar mode (0=Julian or 1=Gregorian)
//
// Default calendar mode = 1 = Gregorian


   function Calendar_Text ($YNumArg, $MNumArg, $CalModeArg=1)

{

// Read input arguments and construct
// encoded calendar date.

   $y = $YNumArg;
   $m = $MNumArg;  $mm = SPrintF("%02d", $m);
   $d = 1;      $dd = "01";

   $ymmdd = "$y$mm$dd";

   $CalMode = $CalModeArg;





// Get parameters needed to create calendar table.

   $JD1     = YYYYMMDD_To_JD12 ($ymmdd, $CalMode);
   $DoW1    = ($JD1 + 1) % 7;
   $MName   = Month_Str($m);
   $NumDays = Days_In_Month ($y, $m, $CalMode);


//   return "$ymmdd<BR>$mmdd";


   $PadSpaces = Str_Repeat(" ", 20 - (StrLen($MName)+StrLen($y)));
   $line = ""; // Str_Repeat("=", 20);

   $CalStr = "";
   $WDays = "Su Mo Tu We Th Fr Sa";

   for ($i=1;  $i <= $DoW1;   $i++) {$CalStr .= "&nbsp;&nbsp;&nbsp;";}

   for ($d=1;   $d <= $NumDays;   $d++)
       {
        if ($d < 10) {$p = "&nbsp;";} else {$p = "";}
            $CalStr .= $p . "$d ";
        if (($d + $DoW1 ) % 7 == 0) {$CalStr .= "\n";}
       }


// Done.

   return trim("$line\n$YNumArg$PadSpaces$MName\n\n$WDays\n$CalStr\n$line");

} // End of  Calendar_Text()









/*

   This function rounds off a given decimal
   value to a given number of decimals.



   The rounding is absolute.

   The argument ($xStrArg) should be a numeric
   string, like "1.23456789"

*/

   function bcRound ($xStrArg, $NumDecimalsArg)

{
   $SignStr = (substr($xStrArg,0,1) == "-" )? "-" : "";

   $x = str_replace("-", "", $xStrArg);

   $decimals = $NumDecimalsArg;

   $Zeros5 = "";

   for ($i=0;   $i < $decimals;   $i++)
       {$Zeros5 .= "0";} $Zeros5 .= "5";

   $x = bcAdd($x, "0.$Zeros5", $decimals);

   return "$SignStr$x";
}










































/*

This function converts decimal degrees into
a "DD MM SS.sss" formatted string with an
optional decimals setting for arc seconds.

The result will always have the general
format "DDD MM' SS.sss", with the degrees
part optionally padded with 1 or 2 zeros if
required.  The arc seconds part also allows
attached decimals which may also vary as needed.

This function is related to the Deg_To_HMS() function.

---------
ARGUMENTS

$DegArg         = Decimal degrees input argument

$PosSignFlag      = This flag allows an optional +
                  sign to be attached to positive
                  angles.

$DDDdigits      = This value is 1|2|3 and controls
                  the zero padding of the degrees
                  part of the output.

$ssDecimals     = This is the number of decimal places,
                  1|2|3|4, to use for the arc seconds
                  part of the returned result.

$SymbolsFlag    = This flag allows optional degrees,
                  minutes and arc seconds symbols to
                  be attached to the output string.

------
ERRORS

This function does NOT check for invalid input arguments.

*/
   function Deg_To_DMS ($PosSignFlag, $DegArg, $dddDigits, $ssDecimals, $SymbolsFlag=TRUE)
{

// Handle numerical sign of input.
   if ($DegArg < 0) {$sign = "-";} else {$sign = "";}
   if ($PosSignFlag === TRUE && $sign != "-")
      {$sign = "+";}

// Work with the absolute value
// of original argument.  The
// numerical sign will be
// reapplied to the output.
   $DegArg = abs($DegArg);

// Compute degrees.
   $ddd = floor($DegArg);

// Compute arc minutes.
   $w  = 60*($DegArg - $ddd);
   $mm = (int)$w;

// Compute arc seconds.
   $ss = 60*($w - $mm);

// Format degrees for output.  Left pad with up
// to 2 zeros to make a 1 to 3-digit string.

// Adjust redundant DDD padding zeros
// according to ($dddDigits) value.
   if ($dddDigits == 1)
      {$ddd = 0+$ddd;}

   if ($dddDigits == 2)
      {
       if (StrLen($ddd) < 2) {$ddd = "0$ddd";}
      }

   if ($dddDigits == 3)
      {
       if (StrLen($ddd) < 2)  {$ddd = "00$ddd";}
       if (StrLen($ddd) == 2) {$ddd = "0$ddd";}
      }

// Format arc minutes value ($mm).
// for output.
   if ($mm < 10) {$mm = "0$mm";}

// Construct output dms labels, if indicated.
   $DDDsymbol = $MMsymbol = $SSsymbol = "";
   if ($SymbolsFlag === TRUE)
      {
       $DDDsymbol = "";
       $MMsymbol  = "'";
       $SSsymbol  = '"';
      }

// Construct output format string
// for arc seconds ($ss).
   $SSformat = "%02." . "$ssDecimals" . "f";

// Format arc seconds value ($ss).
// for output.
   if ($ssDecimals == 0)
      {$ss = sPrintF("%02d", $ss);
      }
   else
      {
       $ss = sPrintF($SSformat, $ss);
       if ($ss < 10) {$ss = "0$ss";}
      }


// Patch for that blasted residual 60 problem.
   if ($ss == 60) {$mm = sprintf("%02d", $mm+1);   $ss = "00.000000";}
   if ($mm == 60) {$hh = sprintf("%02d", $hh+1);   $mm = "00";}


// Return the formatted "DDD MM SS.sss" angle string.
   return "$sign$ddd$DDDsymbol $mm$MMsymbol $ss$SSsymbol";

} // End of  Deg_To_DMS()









/*

This function converts decimal degrees into a
HH MM SS.sss formatted hour angle string with
an optional decimals setting and optional display
of the numerical sign for positive values.  In
terms of time angles, 15 degrees = 1 hour.

This function is related to the Deg_To_DMS() function.

---------
ARGUMENTS

$DegArg = Raw degrees argument input.

$hhDigits = Digits to use for the hours = 1|2
            1 = Return hours 0 to 9 as '0' to '9'
            2 = Return hours 0 to 9 as '00' to '09'


$ssDecimals = Decimal digits = 0|1|2|3|4  to be used
              for the SS part.


$PosSignFlag = Binary flag to indicate a + signed result.

              TRUE = Attach a + sign to ALL positive
                     values without exception.  This
                     does NOT effect negative values.

             FALSE = Do NOT attach a + sign to any
                     positive values.  This does NOT
                     effect negative values.

$SymbolsFlag  = Binary flag to indicate label mode.

                TRUE = Attach h,m,s labels
                       to returned result.

               FALSE = Do NOT attach any h,m,s labels
                       to returned result.

------
ERRORS
This function does NOT check for invalid input arguments.

*/

   function Deg_To_HMS ($PosSignFlag, $DegArg, $hhDigits, $ssDecimals, $SymbolsFlag=TRUE)
{

// Handle numerical sign of input.
   if ($DegArg < 0) {$sign = "-";} else {$sign = "";}
   if ($PosSignFlag === TRUE && $sign != "-")
      {$sign = "+";}

// Convert decimal degrees into
// decimal hours and work with
// the absolute value.  The
// sign will be reapplied to
// the output.
   $deg = abs($DegArg / 15);

// --------------
// Compute hours.

   $hh = floor($deg);

// ----------------
// Compute minutes.

   $w  = 60*($deg - $hh);
   $mm = floor($w);

// ----------------
// Compute seconds.

   $ss = 60*($w - $mm);

// --------------------------------
// Patch to handle the 60s problem.

   if ($ss == 60) {$mm += 1;  $ss = 0;}
   if ($mm == 60) {$hh += 1;  $mm = 0;}

// ---------------------------
// Format HH value for output.

   $hh = sPrintF("%02d", $hh);

// ---------------------------
// Format MM value for output.

   $mm = sPrintF("%02d", $mm);

// --------------------------------------------
// Attach output symbols (h m s), if indicated.

   $HHsymbol = $MMsymbol = $SSsymbol = "";
   if ($SymbolsFlag)
      {
       $HHsymbol = "h";
       $MMsymbol = "m";
       $SSsymbol = "s";
      }

// Format SS.sss value for output.
   $SSformat = "%02." . "$ssDecimals" . "f";
   $ss = sPrintF($SSformat, $ss);
         if ($ss < 10) {$ss = "0$ss";}

// Return the formatted "HH MM SS.sss" time string.
   return "$sign$hh" . "$HHsymbol $mm" .  "$MMsymbol $ss" . "$SSsymbol";

} // End of  Deg_To_HMS()


















/*

   This function solves a quadratic
   equation of the general form

   Ax + Bx + C = 0

   for numerical roots x1 and x2

   The solutions may be real or complex (imaginary).

   The computations are performed in arbitrary precision
   with output precision truncated to 16 decimals.




*/



   function Quad_Solve ($ArgA, $ArgB, $ArgC)

{

// ---------------------
// Read A,B,C arguments.

   $A = trim($ArgA);
   $B = trim($ArgB);
   $C = trim($ArgC);

// --------------------------------
// Set internal decimals precision.

   $DP = 20;

// ------------------------------------------
// Compute discriminant D.  If negative, then
// the solutions are complex (imaginary).

   $w1 = bcMul($B, $B, $DP); // B

   $w2 = bcMul("4", bcMul($A, $C, $DP), $DP); // 4AC

    $D = bcSub($w1, $w2, $DP);

// -----------------------------------------------
// Determine if quadratic root is real or complex.

   $i = "";

   if (bcComp($D, "0", $DP) < 0)

      {
       $i = "<B> i</B>";  $D = bcSub("0", $D, $DP);
      }

//  --------------------------
//  Compute values of u and v.

    $u = bcDiv(bcSub("0", $B, $DP), bcMul("2", $A, $DP), $DP); // -B/2A

   $w3 = bcSqRt($D, $DP); // SqRt(B - 4AC) = SqRt(D)

    $v = bcDiv($w3, bcMul("2", $A, $DP), $DP); // SqRt(B - 4AC) / 2A

// -----------------------------
// Do this if roots are complex.

   if ($i != "")
      {
       $u = RTrim(RTrim($u, "0"), ".");
       $v = RTrim(RTrim($v, "0"), ".");
       return "<PRE>x = $u &plusmn; $v$i</PRE>";
      }

// --------------------------
// Do this if roots are real.

   $x1 = RTrim(RTrim(bcSub($u, $v, 16), "0"), ".");
   $x2 = RTrim(RTrim(bcAdd($u, $v, 16), "0"), ".");
   return "<PRE>x1 = $x1\nx2 = $x2</PRE>";


} // End of  Quad_Solve()


















/*

********************************************************
This function converts any of several units of length or
distance into the equivalent in meters.  It makes use of
the metric length equivalents defined within this module.

It is the opposite of the Meters_To_Length() function.

----
NOTE

These computations are able to accomodate arbitrary
precision out to 105 decimals which is sufficient
for any practical usage.

---------
ARGUMENTS

$LengthVal = Numerical value of the length to be converted
$FromUnits = The units symbol corresponding to the value

The returned output is simply the equivalent length in meters.

-------------------------------------------------------
There are 15 units symbols recognized by this function.

They are:

 SYMBOL         UNITS

   A            Angstroms
   nm           Nanometers
 um or u        Micrometers (or Microns)
   mm           Millimeters
   cm           Centimeters
   in           Inches
   ft           Feet
   yd           Yards
   m            Meters
   km           Kilometers
   mi           Miles (Statute)
   nmi          Nautical miles
   AU           Astronomical Units
   LY           Light Years (Based on Julian year)
   psc          Parsecs

IMPORTANT NOTE:
The units symbols used here ARE case sensitive.

ERRORS:
An error occurs if the argument cannot be resolved
into a valid numerical value and units symbol.  On
error, FALSE is returned.

*/

   Function Length_To_Meters ($LengthVal, $FromUnits)
{

// ------------------------------
// Set arbitrary precision limit.

   $nDec = 105;

// -----------------------------
// Get the numerical value part.
// Error if not a numerical value.

   $x = trim($LengthVal);  If (!Is_Numeric($x)) {Return FALSE;}

// ----------------
// Get units symbol.

   $UnitsSymbol = trim($FromUnits);

// ---------------------------------------------------
// Initialize factor to empty string.   If it is still
// empty on exit from the switch block, below then it
// means that the units symbol was not recognized as
// valid.

   $f = "";

// --------------------------------------
// Determine the conversion factor to use
// from the units symbol argument.
//
// NOTE: These symbols ARE case sensitive.

   switch ($UnitsSymbol)
  {
   case "A"   : $f = M_PER_A;    break;
   case "um"  : $f = M_PER_UM;   break;
   case "u"   : $f = M_PER_UM;   break;
   case "nm"  : $f = M_PER_NM;   break;
   case "mm"  : $f = M_PER_MM;   break;
   case "cm"  : $f = M_PER_CM;   break;
   case "in"  : $f = M_PER_IN;   break;
   case "ft"  : $f = M_PER_FT;   break;
   case "yd"  : $f = M_PER_YD;   break;
   case "m"   : $f = M_PER_M;    break;
   case "km"  : $f = M_PER_KM;   break;
   case "mi"  : $f = M_PER_MI;   break;
   case "nmi" : $f = M_PER_NMI;  break;
   case "AU"  : $f = M_PER_AU;   break;
   case "LY"  : $f = M_PER_LY;   break;
   case "psc" : $f = M_PER_PSC;  break;
  }

// ------------------------------------------
// Error if conversion factor is still empty,
// which means that the units symbol is not
// found in the table of valid symbols.
   If ($f === FALSE) {Return $f;}

// ----------------------------------------------
// Otherwise, return the equivalent meters value
// by multiplying the value of the argument by
// the conversion factor $f.
   $w = bcMul($x, $f, $nDec);

// -----
// Done.
   return RTrim(RTrim($w, "0"), ".");

}  // End of Length_To_Meters()







/*

**********************************************************
This function converts a length or distance value given in
meters into its equivalent in any of the 15 length units
recognized by this module.

It is the opposite of the Length_To_Meters() function and
recognizes the same length units symbols.

NOTE:
The computations are able to accomodate arbitrary
precision out to 105 decimals which is sufficient
for any practical usage.

ARGUMENTS
$metersArg =  Value in meters that will be converted
              to the specified output units.

$toUnits   =  Units symbol for the output value.  This
              symbol will NOT be attached to the output.
              It will simply be a numeric value that is
              understood to be in the specified units.

There are 15 units symbols recognized by this function.
They are:

 SYMBOL         UNITS

   A            Angstroms
   nm           Nanometers
 um or         Micrometers (or Microns)
   mm           Millimeters
   cm           Centimeters
   in           Inches
   ft           Feet
   yd           Yards
   m            Meters
   km           Kilometers
   mi           Miles (Statute)
   nmi          Nautical miles
   AU           Astronomical Units
   LY           Light Years (Based on Julian year)
   psc          Parsecs

IMPORTANT NOTE
The units symbols used here ARE case sensitive.

ERRORS
An error occurs if the argument cannot be resolved
into a valid numerical value and units symbol.  On
error, FALSE is returned.

*/


   Function Meters_To_Length ($MetersVal, $ToUnits)

{

// ------------------------------
// Set arbitrary precision limit.

   $nDec = 105;

// ------------------------------
// Read value of meters argument.

   $m = Trim("$MetersVal");

// -------------------------------------
// Error if meters value is non-numeric.

   If (!Is_Numeric($m)) {Return FALSE;}

// --------------------------------------
// Read the output units symbol argument.

   $UnitsSymbol = Trim($ToUnits);

// ---------------------------------------------------
// Initialize factor to empty string.   If it is still
// empty on exit from the switch block below, then it
// means that the units symbol was not found in the
// table of valid units.

   $f = "";

// --------------------------------------
// Determine the conversion factor to use
// from the units symbol argument.
// THESE SYMBOLS ARE CASE SENSITIVE.

   switch ($UnitsSymbol)
  {
   case "A"   : $f = M_PER_A;    break;
   case "um"  : $f = M_PER_UM;   break;
   case "u"   : $f = M_PER_UM;   break;
   case "nm"  : $f = M_PER_NM;   break;
   case "mm"  : $f = M_PER_MM;   break;
   case "cm"  : $f = M_PER_CM;   break;
   case "in"  : $f = M_PER_IN;   break;
   case "ft"  : $f = M_PER_FT;   break;
   case "yd"  : $f = M_PER_YD;   break;
   case "m"   : $f = M_PER_M;    break;
   case "km"  : $f = M_PER_KM;   break;
   case "mi"  : $f = M_PER_MI;   break;
   case "nmi" : $f = M_PER_NMI;  break;
   case "AU"  : $f = M_PER_AU;   break;
   case "LY"  : $f = M_PER_LY;   break;
   case "psc" : $f = M_PER_PSC;  break;
  }

// ------------------------------------------
// Error if conversion factor is still empty.

   If ($f === "") {return FALSE;}

// ----------------------------------------------
// Otherwise, return the equivalent of the given
// meters value by dividing the value of the
// argument by the conversion factor $f.

   $w = bcDiv ($m, $f, $nDec);

   return RTrim(RTrim($w, "0"), ".");

}  // End of Meters_To_Length()














/*

   This function computes equivalent distances and optional light-time
   given AUs as an input argument.

   The light-time flag is optional and is set to FALSE by default.
   To enable light-time as part of the output, set it to TRUE.

*/


   function AU_To_AUKMMILT ($AUArg, $LTFlag=FALSE, $AUDecimalsArg=7)

{

   $AU = trim($AUArg);


   $km = $AU * KM_PER_AU;
   $mi = $km / KM_PER_MI;


// Select prefix: M=Millions or B=Billions of kilometers

   $factor = 1;
   $suffix = "";

   if ($km >= 1E6)  {$suffix = "M";  $factor=1000000;}

   if ($km >= 1E9)  {$suffix = "B";  $factor=1000000000;}

   $km /= $factor;

   $km = Format_Num (FALSE, $km, 3, 6) . " $suffix" . "km";

   $km = Str_Replace("| 0", "|  ", $km);


// Select prefix: M=Millions or B=Billions of miles

   $factor = 1;
   $suffix = "";

   if ($mi >= 1E6)  {$suffix = "M";  $factor=1000000;}

   if ($mi >= 1E9)  {$suffix = "B";  $factor=1000000000;}

   $mi /= $factor;

   $mi = Format_Num (FALSE, $mi, 3, 6) . " $suffix" . "mi";


   if ($AU >= 4) {$w = "7";} else {$w = "$AUDecimalsArg";}

       $AU = sPrintF("%1.$w" . "f", $AU) . " AU";


   $LT = FloatVal($AU) * 0.0057755183310895;

   $LTHMS = Days_To_HMS ($LT, 1);

   $LTHMS = Str_Replace("00h ", "", $LTHMS);


   if ($LTFlag === TRUE)
      {
       $w = "$AU =|$km =|$mi |$LTHMS";
      }
   else
      {
       $w = "$AU =|$km =|$mi";
      }

   $w = Str_Replace("AU|00",  "AU|  ",  $w);
   $w = Str_Replace("AU|0",   "AU| ",   $w);
   $w = Str_Replace("Mkm|00", "Mkm|  ", $w);
   $w = Str_Replace("Mkm|0",  "Mkm| ",  $w);
   $w = Str_Replace("Bkm|00", "Bkm|  ", $w);
   $w = Str_Replace("Bkm|0",  "Bkm| ",  $w);
   $w = Str_Replace("|", " ", $w);

   $w = Str_Replace("    ", "   ", $w);

// --------------
// Final filters.

   $w = Str_Replace("= 0",  "=  ", $w);
   $w = Str_Replace("=  0", "=   ", $w);

   return $w;

} // End of  AU_To_AUKMMILT()













// ************************************************************************
// Generate server query as complete HTML TABLE code (PHP5)


   function Server_Query ()
{

   $out = "";

   $x = $_SERVER;

   ForEach ($x as $key => $value)

           {
            $out .= "<TR><TD BGCOLOR=\"LightYellow\">$key</TD><TD BGCOLOR=\"white\">$value</TD></TR>\n";
           }

   return "<TABLE BGCOLOR=\"black\">$out</TABLE>";

}








// ************************************************************************
// Generate array query as complete HTML TABLE code (PHP5)


   function Array_Query ($ArrayNameArg)
{

   $out = '';

   $x = $ArrayNameArg;

   ForEach ($x as $key => $value)
           {
            $out .= "<TR><TD BGCOLOR=\"cyan\">$key</TD><TD BGCOLOR=\"white\">$value</TD></TR>\n";
           }

   return "<TABLE BGCOLOR=\"black\" CELLPADDING=\"2\" CELLSPACING=\"1\">$out</TABLE>";

}














// ************************************************************************

// This function returns a listing of all the custom definitions
// and functions defined within the Kore 2009.x module contained
// within this "core" folder.

   function Kore_Info ()

{

   global $ModuleVersion;

   $KoreLines      = file("Kore.php");
   $KoreLinesCount = count($KoreLines);




// =========================================================
// Generate list of all custom constants within Kore module.

   $ConstantsList  = "";
   $ConstantsCount = 0;

   for ($i=0;   $i < $KoreLinesCount;   $i++)
       {
        $KoreLine = trim($KoreLines[$i]);
        $KoreLine = Str_Replace(" ", "", $KoreLine);

        if (StrToLower(substr($KoreLine, 0, 8)) == 'define("')
           {
            $w = trim(substr($KoreLine, 8, strlen($KoreLine)));
            $w = Str_Replace('",', " = ", $w);

            $ParenthesisPointerR = strpos($w, ")");

            $w = substr($w, 0, $ParenthesisPointerR);

            $ConstantsList .= "$w\n";

            $ConstantsCount++;
           }
       }


// =========================================================
// Generate list of all custom functions within Kore module.

   $FunctionsList  = "";
   $FunctionsCount = 0;

   for ($i=0;   $i < $KoreLinesCount;   $i++)
       {
        $KoreLine = trim($KoreLines[$i]);

        if (StrToLower(substr($KoreLine, 0, 8)) == "function")
           {
            $w = trim(substr($KoreLine, 8, strlen($KoreLine)));

            $FunctionsList .= "$w\n";

            $FunctionsCount++;
           }
       }

   $FunctionsList = trim($FunctionsList);


// =================================
// Return final listing code string.

   return

"<B><PRE>
NeoProgrammics KORE MODULE $ModuleVersion - INFO

----------------------------------------------
The following $ConstantsCount custom constants are defined:

$ConstantsList
----------------------------------------------
The following $FunctionsCount custom functions are defined:

$FunctionsList
</PRE></B>";

} // End of  Kore_Info ()






























/*

  This function returns a mathematically encoded card deck string.

  The card deck refers to an American standard 52-card deck, minus
  any jokers.

  The card codes are in hexadecimal to allow encoding in 2-digit
  form.

  As a 2-digit number, the 1st digit represents the suit whilst
  the 2nd digit represents the rank.

  The four numerical suits are 1 to 4 in hexadecinal.

  The suit codes (1,2,3,4) can be assigned arbitrarily to mean
  diamonds, clubs, hearts and spades.

  The 13 numerical ranks (Ace to King) are 1 to D in hexadecimal.

  Every card can be encoded as a 2-digit hexadecimal number as

  SuitRank

*/






   Function Shuffled_Cards()

{

// --------------------------------------
// Define an array to hold the card deck.

   $CardDeck = array();

// ---------------------------------------
// Create an array of the numbers 1 to 52.

   for ($i=0;   $i < 52;   $i++)  {$CardDeck[$i] = $i+1;}
        shuffle($CardDeck);

// ----------------------------------------------------
// Collect shuffled deck as encoded string of 52 cards.

   $cards = "";

   for ($i=0;   $i < 52;   $i++)
       {$CardID = $CardDeck[$i];

// ----------------------------------
// Compute suit code number (1 to 4).

   $CardSuit= floor(($CardID-1) / 13) + 1;

// ----------------------------------
// Encode card rank number (1 to 13).

   $CardRank = StrToUpper(DecHex(($CardID % 13) + 1));

// ---------------------------------------------
// Construct card code from suit and rank codes.

   $CardCode = "$CardSuit$CardRank";

   $cards .= $CardCode;

       }

// -----
// Done.

   return $cards;

} // End of  Shuffled_Cards()





  Function String_To_Entities($TextStrArg)

{

   $x = $TextStrArg;

   $j = StrLen($x);

   $z = "";

   For ($i=0;   $i < $j;   $i++)

       {

        $y =  SubStr($x, $i, 1);

        $z .= "&#38;&#35;" . ord($y) . "&#59;";

       }

   Return $z;

}






/*

================================================
Compute decimal degrees corresponding to a given
space delimited DMS argument string.

This function is flexible and allows decimal
DMS values to be entered and non-standard
values outside the normal range, such as
90 seconds which would be treated as
equivalent to 00:01:30:00




=======
ERRORS:

On error, Boolean FALSE is returned.

*/


   function DMS_To_Deg ($SignNumArg, $DMSarg)

{

// Read numerical sign argument.
   $sign = ($SignNumArg < 0)? "-" : "+";

// ---------------------------
// Read input argument string.
   $DMS = trim($DMSarg);

// -------------------------------------------------
// Normalize spacing of the angle elements string to
// ensure only a single space between the elements.
   $DMS = preg_replace('/\s+/', ' ', trim($DMS));

// --------------------------------
// If input argument is empty, then
// use default "0 0 0" string.
   if ($DMS == "") {$DMS = "0 0 0";}

// -----------------------------
// Read angle elements and split
// into a work array.
   $DMS = preg_split ("[ ]", $DMS);

// ----------------------------------
// Count angle elements (n = 1 to 3).
   $n = count($DMS);

// ------------------------------------
// Fetch degrees value from work array.
   $dd = $DMS[0];

// ------------------------------------
// Fetch minutes value from work array.
   $mm = ($n > 1)?  $DMS[1] : 0;

// --------------------------------------
// Compute degrees equivalent to minutes.
   $mmd = bcDiv($mm, "60", 24);

// ------------------------------------
// Fetch seconds value from work array.
   if ($n > 2) {$ss = $DMS[2];} else {$ss = 0;}

// --------------------------------------
// Compute degrees equivalent to seconds.
   $ssd = bcDiv($ss, "3600", 24);

// -----------------------------------------------------
// Compute final decimal degrees rounded to
// 16 decimals, recalling numerical sign.
   $deg = $sign . bcRound(bcAdd_Seq("$dd $mmd $ssd", 24), 16);

// Patch to remove redundant negative sign.
   if ($deg == "-0") {$deg = "0";}

// -----
// Done.
   return $deg;

} // End of  DMS_To_Deg()















/*

   Lagrangian Interpolation for any number (n) of XY data pairs.

   ==========
   DATA RULES

   Both X and Y must represent continuous functions and may be
   linear or non-linear.

   No two values of X may be identical.

   --------------------------
   Test text data input table

   X Data    Y Data
   29.43     0.4913598528
   30.97     0.5145891926
   27.69     0.4646875083
   28.11     0.4711658342
   31.58     0.5236885653
   33.05     0.5453707057

   X = Angle in degrees
   Y = Sine of angle X

   ---------------------------------------------------------
   Test text  data table converts into the Lagrange_Interp()
   input XY data vector strings:

   $xDataVectorArg = "29.43 30.97 27.69 28.11 31.58 33.05";

   $yDataVectorArg = "0.4913598528 0.5145891926 0.4646875083 0.4711658342 0.5236885653 0.5453707057";

*/

   function Lagrange_Interp ($xDataVectorArg, $yDataVectorArg, $xArg)
{

// Read XY data vector strings, stripping all redundant
// white spaces from within the strings.
   $Xdv = preg_replace('/\s+/', ' ', trim($xDataVectorArg));
   $Ydv = preg_replace('/\s+/', ' ', trim($yDataVectorArg));

// Split XY data vector strings
// into matching indexed arrays.
   $xData = split("[ ]", $Xdv);
   $yData = split("[ ]", $Ydv);

// Count number of XY data elements.
// For every X value there must be a
// matching Y value, but no two X
// values can be the same.
   $xDataCount = count($xData);
   $yDataCount = count($yData);

// Return error message if data vector element count mismatch.
// For every X value there must be a corresponding Y value
// or an XY data count mismatch error occurs.
   if ($xDataCount != $yDataCount)   {return "ERROR: XY Data Count Mismatch";}

// Number of matching data pairs.
   $n = ($xDataCount + $yDataCount) / 2;

// Read X argument for which to
// interpolate the Y value based
// on the given XY data.
   $x = trim($xArg);

// Given X, interpolate the corresponding Y value.

// Initialise y series accumulator.
   $y=0;

// Compute product series.
   for ($i=0;   $i < $n;   $i++)
       {

        $c = "1";

        for ($j=0;   $j < $n;   $j++)
            {

             if ($j != $i)
                {
                 $P = bcMul($c, bcSub($x, $xData[$j], 32), 32);
                 $Q = bcSub($xData[$i], $xData[$j], 32);
                 $c = bcDiv($P, $Q, 32);
                }

            } // Next j

//      Accumulate sum of product terms.
        $y = bcAdd($y, bcMul($c, $yData[$i], 32), 32);

       } // Next i

// Remember numerical sign of
// interpolated y as 1
   $sign = ($y < 0)? -1 : +1;

// Round off interpolated y value to 16-decimal output limit.
   $y = bcMul($sign, bcAdd(bcMul($sign, $y, 32), "0.00000000000000005", 16), 16);

// Cut off any redundant zeros and/or decimal point from y value.
   if (StrPos($y, ".") !== FALSE) {$y = RTrim(RTrim($y, "0"), ".");}

// Done.
   return $y;

} // End of  Lagrange_Interp()









// This function generates a drop-down menu code for a given
// month with optional selection set.

   function Month_Selector($NameArg, $SelectedIndexArg)
  {

   $list = "";

   $Mmm = "JanFebMarAprMayJunJulAugSepOctNovDec";

   for ($i=0;   $i < 12;   $i++)

       {

        if ($i == $SelectedIndexArg) {$chk=" SELECTED";}else{$chk="";}

        $list .= "<OPTION$chk>" . substr($Mmm, $i*3, 3) . "</OPTION>\n";
       }

    return "<SELECT NAME=\"$NameArg\" CLASS=\"BgYellow\">\n" . $list . "</SELECT>\n";

  } // End of  Month_Selector()




// This function generates drop-down menu code to select a given year
// from within a range of years and with a given default selection.

   function Year_Selector($NameArg, $yStartArg, $yEndArg, $SelectedYearArg)
  {

   $list = "";

   $ySpan =  $yEndArg - $yStartArg + 1;

   for ($y=$yStartArg;   $y < $yEndArg+1;   $y++)

       {
        if ($y == $SelectedYearArg) {$chk=" SELECTED";}else{$chk="";}

        $list .= "<OPTION$chk>" . $y . "</OPTION>\n";
       }

   return "<SELECT NAME=\"$NameArg\" CLASS=\"BgYellow\">\n" . $list . "</SELECT>";

  }









// Function to generate a drop-down menu code for a given
// inner planet with selected index.

   function Inner_Planet_Selector($NameArg, $SelectedIndexArg)
  {

   $list = "";

   $bodies = "MercuryVenus  Mars";

   for ($i=0;   $i < 3;   $i++)

   {

    if ($i == $SelectedIndexArg) {$chk=" SELECTED";}else{$chk="";}

    $list .= "<OPTION$chk>" . trim(substr($bodies, $i*7, 7)) . "</OPTION>\n";
   }

    return "<SELECT NAME=\"$NameArg\" CLASS=\"BgYellow\">\n" . $list . "</SELECT>\n";

  }








// Function to generate a drop-down menu code for a given
// outer planet with selected index.

   function Outer_Planet_Selector($NameArg, $SelectedIndexArg)
  {

   $list = "";

   $bodies = "JupiterSaturn Uranus NeptunePluto  ";

   for ($i=0;   $i < 5;   $i++)

   {

    if ($i == $SelectedIndexArg) {$chk=" SELECTED";}else{$chk="";}

    $list .= "<OPTION$chk>" . trim(substr($bodies, $i*7, 7)) . "</OPTION>\n";
   }

    return "<SELECT NAME=\"$NameArg\" CLASS=\"BgYellow\">\n" . $list . "</SELECT>\n";

  }




// Function to generate a drop-down menu code for a given
// asteroid with selected index.

   function Asteroid_Selector($NameArg, $SelectedIndexArg)
  {

   $list = "";

   $bodies = "Ceres     Pallas    Juno      Vesta     Hebe      ";

   for ($i=0;   $i < 5;   $i++)

   {

    if ($i == $SelectedIndexArg) {$chk=" SELECTED";}else{$chk="";}

    $list .= "<OPTION$chk>" . trim(substr($bodies, 10*$i, 10)) . "</OPTION>\n";
   }

    return "<SELECT NAME=\"$NameArg\" CLASS=\"BgYellow\">\n" . $list . "</SELECT>\n";

  }











   function Spam_Text($TextStringArg)
   {

   $out = "";

   $TextLength = strlen($TextStringArg);

   for ($i=0;   $i < $TextLength;   $i++)
   {

   $char = substr($TextStringArg, $i, 1);

   $out .= $char . Rand_String(mt_rand(1, 20));


   }

   return Rand_String(mt_rand(1, 20)) . $out;


   }











   function Rand_String($NumCharsArg)
  {

   $chars1 = "qwertyuioasdfghjklzxcvbnmQWERTYUIOLKJHGFDSAZXCVBNM";
   $chars2 = " 1234567890 /*-!@#$%^&*()_+=,./?' qwertyuioasdfghjklzxcvbnmQWERTYUIOLKJHGFDSAZXCVBNM ";

   $Chars1Length = strlen($chars1);
   $Chars2Length = strlen($chars2);


   $out =substr($chars1, mt_rand(0, $Chars1Length - 1), 1);

    for ($i=0;   $i < $NumCharsArg;   $i++)
   {

    $out .= substr($chars2, mt_rand(0, $Chars2Length - 1), 1);

   }

    return "<$out>";

  }









   function Rand_Text($NumCharsArg)
  {

   $chars1 = "qwertyuiopasdfghjklzxcvbnmQWERTYUIOPLKJHGFDSAZXCVBNM";

   $Chars1Length = strlen($chars1);

   $out =substr($chars1, mt_rand(0, $Chars1Length - 1), 1);

    for ($i=0;   $i < $NumCharsArg;   $i++)
   {

    $out .= substr($chars1, mt_rand(0, $Chars1Length - 1), 1);

   }

    return "<$out>";

  }










// ================================================
// This BC math function adds a sequence of squared
// numbers to the specified decimals of precision.

   function bcAdd_Seq_Sqr ($SeqStrArg, $NumDecimals)

{

// ----------------------------------------------
// Remove redundant white space from data vector.
   $SeqStr = preg_replace('/\s+/', ' ', trim($SeqStrArg));

// -----------------------
// Get number of decimals.
   $d = trim($NumDecimals);

// -----------------------------------------------
// Separate items to be summed into working array.
   $item = split ("[ ]", trim($SeqStr));

// -------------------------
// Count items to be summed.
   $ItemCount = count($item);

// --------------------------------
// Handle case of empty items list.
   if ($ItemCount == 0) {return "0";}

// -----------------------------------
// Handle case of single item in list.
   if ($ItemCount == 1) {return $item[0];}

// -------------------
// Sum the items list.
   $sum = "0";

   for ($i=0;  $i < $ItemCount;   $i++)
       {
        $sq = bcMul($item[$i], $item[$i], $d+16);
        $sum = bcAdd($sum, $sq, $d+16);
       }

   return bcAdd("0", $sum, $d);

} // End of  bcAdd_Seq_Sqr()








   function Day_Selector($NameArg, $SelectedIndexArg=0)
  {

   $list = "";


   for ($d=0;   $d < 31;   $d++)

       {

        if ($d == $SelectedIndexArg) {$chk=" SELECTED";}else{$chk="";}

        $list .= "<OPTION$chk>" . sPrintF("%02d", $d+1) . "</OPTION>\n";
       }

    return "<SELECT NAME=\"$NameArg\" CLASS=\"BgYellow\">\n" . $list . "</SELECT>\n";

  } // End of  Day_Selector()















   function Question_Block($QuizDataLineArg)
{

   $QuizDataLine = trim($QuizDataLineArg);

   $w = split("[|]", $QuizDataLine);

   $DataCount = count($w);

   $AnswersCount = $DataCount - 1;

   $nQ = trim($w[0]);

   $n = floatval($nQ);   $nn = sPrintF("%02d", $n);

   $ip = strpos($nQ, ":");

   $question = substr($nQ, $ip+1, strlen($nQ));





// Collect answers into array.

   $answers = array();

   for ($i=1;   $i < $DataCount;   $i++)
   {
   $answers[$i-1] = $w[$i];
   }

   $AnswersCount = count($answers);

   shuffle($answers);





   $out = "";

   for ($i=0;   $i < $AnswersCount;   $i++)
   {

   $choice = chr($i + 65);

   $out .=
"<TR>
<TD BGCOLOR=\"cyan\" CLASS=\"Tx9\" ALIGN=\"center\"><B>$choice</B>&nbsp;<INPUT NAME=\"a$nn\" TYPE=\"radio\" VALUE=\"$i\"></TD>
<TD BGCOLOR=\"white\">$answers[$i]</TD>
</TR>\n"
;

   }




$TableStart =
"<TABLE BGCOLOR=\"black\" CLASS=\"Tx10\" BORDER=\"0\" CELLPADDING=\"3\" CELLSPACING=\"1\">
<TR>
<TD BGCOLOR=\"GreenYellow\" ALIGN=\"center\"><B>$n</B></TD><TD BGCOLOR=\"yellow\"><B>$question</B></TD>
</TR>"
;

$TableEnd =
"<TR><TD BGCOLOR=\"black\" COLSPAN=\"2\">&nbsp;</TD></TR>
</TABLE>"
;


   return $TableStart . $out . $TableEnd;


}













// This function swaps XY data elements in a string delimited paired data vector.
// The data vector is returned with the XY data element positions reversed.

   function Swap_XY_Data_Pairs($DataPairsArg)
{

   $xVector = $yVector = $dV = "";

   $PairedDataVector = str_replace("\n", " ",  $DataPairsArg);

   $PairedDataVector = Strip_WSpace($PairedDataVector);

   $PairedData = split("[ ]", $PairedDataVector);

   $PairedDataCount = count($PairedData);

   if ($PairedDataCount % 2 != 0 || $PairedDataCount < 4)  {return "ERROR";}

//   Get all (X,Y) data elements into separate
//   space delimited data string vectors.
     for ($i=0;   $i < $PairedDataCount;   $i+=2)
         {
          $xVector .= $PairedData[$i] . " ";
          $yVector .= $PairedData[$i+1] . " ";
         }
          $xVector  =  trim($xVector);
          $yVector  =  trim($yVector);

// Construct output data vector with swapped (X,Y) data elements.
   for ($i=0;   $i < $PairedDataCount; $i+=2)
       {
        $dV .= ($PairedData[$i+1] . " " . $PairedData[$i] . " ");
       }

// Done
   return trim($dV);

} // End of  Swap_XY_Data_Pairs()






// This function splits a data vector string into two separate
// space delimited (x, y) data vector strings delimited by a
// bar (|) character.

   function XY_Data_Pairs_To_Data_Vectors ($DataPairsArg)
{

  $xVector = $yVector = $dV = "";

   $PairedDataVector = str_replace("\n", " ",  $DataPairsArg);

   $PairedDataVector = Strip_WSpace($PairedDataVector);

   $PairedData = split("[ ]", $PairedDataVector);

   $PairedDataCount = count($PairedData);

   if ($PairedDataCount % 2 != 0 || $PairedDataCount < 4)  {return "ERROR";}

//   Get all (X,Y) data elements into separate
//   space delimited data string vectors.
     for ($i=0;   $i < $PairedDataCount;   $i+=2)
         {
          $xVector .= $PairedData[$i] . " ";
          $yVector .= $PairedData[$i+1] . " ";
         }
          $xVector  =  trim($xVector);
          $yVector  =  trim($yVector);

// Construct output data vector with swapped (X,Y) data elements.

// Done
   return trim("$xVector|$yVector");

} // End of  XY_Data_Pairs_To_Data_Vectors()











// Time addition function.
// To a positive time value, a positive or
// negative value is added.
//
// HMS strings are space delimited like
// 01 02 03.123

   function HMS_Add($HMSArg, $HMSSignNumArg, $HMSaddArg)
{

   $hms1 = trim($HMSArg);
   $hms1 = str_replace(":"," ", $hms1);
   $hms1 = trim(str_replace(",",".", $hms1));

   $hms1 = split("[ ]", trim($HMSArg));
    $hh1 = floatval($hms1[0]);
    $mm1 = floatval($hms1[1]);
    $ss1 = floatval($hms1[2]);

// Convert to decimal hours.
   $th1 = ($hh1*3600.0 + $mm1*60.0 + $ss1) / 3600.0;




   $SignNum = ($HMSSignNumArg < 0)? -1 : 1;




   $HMSAdd = trim($HMSaddArg);
   $hms2 = trim($HMSAdd);
   $hms2 = str_replace(":"," ", $hms2);
   $hms2 = trim(str_replace(",",".", $hms2));

   $hms2 = split("[ ]", $hms2);
    $hh2 = floatval($hms2[0]);
    $mm2 = floatval($hms2[1]);
    $ss2 = floatval($hms2[2]);

// Convert to decimal hours and apply sign.
   $th2 = $SignNum * ($hh2*3600.0 + $mm2*60.0 + $ss2) / 3600.0;

// Combine times according to the sign.
// Result must be non-negative or error.

   $thOut = $th1 + $th2;

   if ($thOut < 0) {return "";}

// Convert decimal hours to HMS for output.

// 12.3456789


   $hh = floor($thOut);

   $ww = 60*($thOut - $hh);

   $mm = floor($ww);

   $ss = 60*($ww - $mm);


   $hh = sPrintF("%02d", $hh);
   $mm = sPrintF("%02d", $mm);
   $ss = Format_Num (FALSE, $ss, 2, 3);

   return "$hh $mm $ss";



}



// preg_replace('/\s+/', ' ', trim($x));


   function Deg2DMS($DegArg)

{
   $deg = trim($DegArg);  if ($deg == ""){$deg = "0.0";}

   $sign = ($deg < 0)? '-' : '';

   $deg = abs($deg);

   $d = floor($deg);

   $w = 60*($deg - $d);

   $m = floor($w);

   $s = sPrintF("%2.3f", 60*($w - $m));

   $dd = sPrintF("%02d",  $d);
   $mm = sPrintF("%02d",  $m);
   $ss = sPrintF("%2.3f", $s);


// Correct for 60's curse.
   if ($ss == "60.000") {$m += 1;  $s = "0";}
   if ($mm == "60")     {$d += 1;  $m = "0";}

   $dd = sPrintF("%02d",  $d);
   $mm = sPrintF("%02d",  $m);
   $ss = sPrintF("%2.3f", $s);

   $ss = ($s < 10)? "0$ss" : $ss;


   return "$sign$dd $mm $ss";

}








/*

==============================================
This function computes the mean apparent local
sidereal time for the given JD number at any
longitude.

Longitudes are measured from above prime Greenwich meridian
with negative to left (west) and positive to right (east)




---------
ARGUMENTS

$JDArg     = JD Number argument for instant
$LngDegArg = Geographic longitude of location (Negative = West)


Default longitude = 0 degrees = Greenwich, UK.

----------
DEPENDENCY

Deg_Mod_360()


------
ERRORS
This function does NOT check for invalid arguments.


*/

   function Mean_Sidereal_Time ($JDArg, $LngDegArg=0)

{

// Test longitude Wash, DC at -77.0654166667


// --------------------------------
// Compute ($t) in Julian Millennia
// from J2000.0

   $t = ($JDArg - 2451545.0) / 36525.0;

// ------------------------------------------------
// Compute mean Greenwich Sidereal Time In Degrees.

   $GSTMeanDeg = 280.46061837 + 360.98564736629*($JDArg - 2451545)
               + ($t*$t * 0.000387993) - ($t*$t*$t / 38710000);

// -----------------------------------
// Compute local mean sidereal time at
// the given geographic longitude.

   $LSTMeanDeg = $GSTMeanDeg + $LngDegArg;

// -------------------------------
// Modulate sidereal time angle to
// fall between 0 and 360 degrees.

   $LSTMeanDeg = Deg_Mod_360($LSTMeanDeg);

// ----------------------------
// Done.  Return the mean local
// Sidereal Time in degrees.
   $LSTMeanDeg = RTrim(RTrim($LSTMeanDeg,"0"), ".");

   return $LSTMeanDeg;

} // End of  Mean_Sidereal_Time()









   function Distance_Between_Stars ($RADegArg1, $DeclDegArg1, $DistArg1, $RADegArg2, $DeclDegArg2, $DistArg2)
{

   $RA1   = Deg2Rad($RADegArg1);
   $decl1 = Deg2Rad($DeclDegArg1);
   $R1    = $DistArg1;
   $x1 = $R1 * cos($RA1)*cos($decl1);
   $y1 = $R1 * sin($RA1)*cos($decl1);
   $z1 = $R1 * sin($decl1);


   $RA2   = Deg2Rad($RADegArg2);
   $decl2 = Deg2Rad($DeclDegArg2);
   $R2    = $DistArg2;
   $x2 = $R2 * cos($RA2)*cos($decl2);
   $y2 = $R2 * sin($RA2)*cos($decl2);
   $z2 = $R2 * sin($decl2);


   $dx = $x2 - $x1;
   $dy = $y2 - $y1;
   $dz = $z2 - $z1;


   $D = SqRt($dx*$dx + $dy*$dy + $dz*$dz);


   $q  = "x1 = $x1<BR>y1 = $y1<BR>z1 = $z1<BR><BR>";

   $q .= "x2 = $x2<BR>y2 = $y2<BR>z2 = $z2<BR><BR>";

   $q .= "dx = $dx<BR>dy = $dy<BR>dz = $dz<BR><BR>";

   $q .= "D = $D";


   return $q;

}















/*

This function takes an hour angle given in
any of several string formats and returns
the equivalent in decimal degrees.

It is the opposite of the Deg_To_HMS() function.

Assuming that ($HMSarg) is an hour angle given
in any of the following formats:

---------------------------------------
FORMAT       EXAMPLE     INTERPRETED AS

HH MM SS     1 2 3    =  01h 02m 03s
HH MM      -05 03     = -05h 03m 00s
HH           2        =  2h
HH MM.mm    12 1.25   =  12h 01m 15s
HH.hhhhh    18.555    =  18h 33m 18s

Where:
HH=Hours,  MM=Minutes,  SS=Seconds

------
ERRORS
On error, FALSE is returned.
An error results if any of the
angular elements are non-numeric.

*/


   function HMS_To_Deg ($SignNumArg, $HMSarg)
{

// -----------------------------------
// Read and remember numerical  sign.
   $sign = ($SignNumArg < 0) ?  "-" : "";

// -------------------------------------------------
// Normalize the spacing of the hour angle elements.
// There must be at least one space between each of
// the HH MM SS arguments as separators.
   $hms = preg_replace('/\s+/', ' ', trim($HMSarg));

// ----------------------------------------------
// Set empty angle string to default = 0 degrees.
   if ($hms == "") {$hms = 0;}
   $w = preg_split("[ ]", $hms);
   $AngleCount = count($w);

// --------------------------------------------
// Error if any angular element is non-numeric.
   if ($AngleCount >= 1)
      {if (!Is_Numeric($w[0])) {return FALSE;}}

   if ($AngleCount >= 2)
      {if (!Is_Numeric($w[1])) {return FALSE;}}

   if ($AngleCount >= 3)
      {if (!Is_Numeric($w[2])) {return FALSE;}}

// ----------------
// Read HMS values.
   if ($AngleCount >= 1) {$hh = $w[0];} else {$hh=0;}
   if ($AngleCount >= 2) {$mm = $w[1];} else {$mm=0;}
   if ($AngleCount >= 3) {$ss = $w[2];} else {$ss=0;}

// --------------------------------------------
// Compute total seconds per each time element.
   $hhss = bcMul($hh, "3600", 24);
   $mmss = bcMul($mm, "60",   24);
   $Tsec = bcAdd_Seq("$hhss $mmss $ss", 24);

// ---------------------------------------
// Compute decimal degrees to 16 decimals,
// recalling numerical sign.
   $deg = $sign . bcRound(bcDiv($Tsec, "240", 24), 16);
   if ($deg == "-0") {$deg = "0";}

// -----
// Done.
   return  $deg;

} // End of  HMS_To_Deg()






/*

Make_File_Index()

AUTHOR : Jay Tanner
VERSION: 3.0

Written in PHP v4.3.9

This function creates a list of files with the
specified file extension ($testExt).  The file
extension IS case sensitive and must contain
the preceding dot (.) character.

The index listing will be saved as a text file
indicated by ($indexFileName) within the same
folder.

NOTE:
Program must be used inside a root folder and
operates only on files within that folder.

ERRORS:
If there are no files found matching the given
test file extension, then Boolean FALSE will be
returned by the function and no file written.

*/




   Function Make_File_Index ($indexFileName, $testExt)
{

// -------------------------
// Define files list string.

   $filesList = "";

// ------------------------------------------------------------
// Split URL of this PHP program into an array of its elements.

   $progURL = PathInfo($_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF']);

// -----------------------------------------------
// Isolate name of the root folder containing this
// program and the file names to be examined.

   $folder = Str_Replace($_SERVER['HTTP_HOST'], "", $progURL ["dirname"]);

//-------------------------------------------
// Define directory object for folder access.

   $filesDirectory = Dir("../$folder/");

// --------------------------------------------------
// Create a listing of all files in the folder ending
// with the given test file extension ($testExt).

   While (($fileName = $filesDirectory -> Read()) !== FALSE)
  {

// -----------------------------------------------------
// Isolate the file extension from the file name and add
// ($fileName) to list if ($fileExt) matches ($testExt).
// This test IS case sensitive.

   $fileExt = "";   $i = StrRPos(" " . $fileName, ".");

   If ($i > 0)
      {
       $fileExt = SubStr($fileName, $i-1, StrLen($fileName));

        If ($fileExt == $testExt) {$filesList .= "$fileName\n";}
      }

  }  // End of While()

// ----------------------------------------------
// AT THIS POINT, THERE SHOULD BE A LIST OF FILES
// WITH EXTENSION ($testExt) IN ($filesList).
//
// THIS STRING CAN BE WRITTEN TO A TEXT FILE AS AN
// INDEX OF FILES WITH THE GIVEN FILE EXTENSION
// CONTAINED WITHIN THE CURRENT FOLDER.
//
// IF THERE WERE NO MATCHING FILES FOUND, THEN
// ($filesList) WILL BE AN EMPTY STRING.

// ---------------------------
// Close the directory object.

   $filesDirectory -> Close();

// Each of the file names in the ($filesList)
// string is delimited with a \n code.


   If (Trim($filesList) == "") {Return FALSE;}

// -------------------------------------------------
// Write the file index to the file ($indexFileName)
// within the same folder.

   $fileHandle = FOpen($indexFileName, "w");
                 FWrite($fileHandle, $filesList);
                 FClose($fileHandle);

}



/*

   This function converts a value in seconds into
   its equivalent HH MM SS.sss  string rounded off
   to a given number of decimals (default=3).

*/

   function Seconds_To_HMS ($SecsArg=0, $DecimalsArg=3)
{

// Read seconds argument.
   $s = trim($SecsArg);

// Read decimals argument.
   $decimals = trim($DecimalsArg);


// Remember numerical sign and work with absolute
// value internally.  Sign will be reattached to
// the returned HH MM SS string.
   if ($s < 0)
      {$sign = '-';  $s = bcMul("-1", $s, 16);}
   else
      {$sign = '';}

// Compute hours.
   $hh = bcDiv($s, "3600");

   $s = bcSub($s, 3600*$hh, 16);

// Compute minutes.
   $mm = bcDiv($s, "60");

// Compute seconds.
   $ss = bcSub($s, 60*$mm, $decimals);

// Format the output time elements.
   $hh = sPrintF("%02d", $hh);
   $mm = sPrintF("%02d", $mm);

   if ($ss < 10) {$ss = "0$ss";}

// Done.
   return "$sign$hh $mm $ss";

} // End of  Seconds_To_HMS()








/*
   This function converts a  day fraction into its
   equivalent HH MM SS.sss string rounded off to a
   given number of decimals (default=3).

*/

   function DayFrac_To_HMS ($DayFracArg, $DecimalsArg=3)
{

// Read day fraction argument.
   $DayFrac = trim($DayFracArg);

// Compute seconds equivalent to day fraction.
   $s = bcMul("86400", $DayFrac, 16);

// Read decimals argument.
   $decimals = trim($DecimalsArg);

// Remember numerical sign and work with absolute
// value internally.  Sign will be reattached to
// the returned HH MM SS.sss string.
   if ($s < 0)
      {$sign = '-';  $s = bcMul("-1", $s, 16);}
   else
      {$sign = '';}

// Compute hours.
   $hh = bcDiv($s, "3600");

   $s = bcSub($s, 3600*$hh, 16);

// Compute minutes.
   $mm = bcDiv($s, "60");

// Compute seconds to 2 extra decimals.
   $ss = bcSub($s, 60*$mm, $decimals+2);

// Round off seconds to given number of decimals.
   $ss = bcAdd($ss, "0." . str_repeat("0", $decimals) . "5", $decimals);

// Format the output time elements.
   $hh = sPrintF("%02d", $hh);
   $mm = sPrintF("%02d", $mm);

   if ($ss < 10) {$ss = "0$ss";}

// Done.
   return "$sign$hh $mm $ss";

} // End of  DayFrac_To_HMS()








/*

   This function returns the encoded date for
   the nth Sunday of the given year and month.

   It was created to assist in determining the
   dates for the new start and end of daylight
   time.

   DEPENDENCY:
   YYYYMMDD_To_JD12()
   JD_To_YYYYMMDD()

*/

   function Nth_Sunday ($nArg, $yArg, $mArg, $CalModeArg)
{

   $sign = ($yArg < 0)? -1 : 1;
   $n = $nArg;
   $y = $sign * $yArg;
   $m = $mArg;
   $d = 1;

   $yyyymmdd = $sign * (($y * 10000) + ($m * 100) + $d);

   $JD12A = YYYYMMDD_To_JD12 ($yyyymmdd, $CalModeArg);

   $JD12B = $JD12A + (7 - ($JD12A + 1) % 7) % 7 + 7*($n-1);

   $nthSunday = JD_To_YYYYMMDD ($JD12B, $CalModeArg);

   return $nthSunday;

}





/*

   BEFORE 1986, THERE WAS NO FEDERAL MANDATE FOR
   DAYLIGHT SAVING TIME AND ITS USAGE WAS SPORADIC
   ACCORDING TO EACH STATE AND NOT ALWAYS IN SYNC.

   IN 1986, A FEDERAL MANDATE WAS IMPLEMENTED TO
   SYNCHRONIZE ANY STATES THAT DECIDED TO APPLY IT.


   =======================================
   OLD DAYLIGHT SAVING RULE (1986 to 2006)

   First Sunday in April to last Sunday in October


   =====================================
   NEW DAYLIGHT SAVING RULE (As of 2007)

   Second Sunday in March to first Sunday in November.




   THIS FUNCTION APPLIES THE NEW RULE FROM 2007
   AND THE OLD RULE FOR ALL DATES BEFORE 2007.


*/


   function Daylight_Dates ($yArg)

{

   if ($yArg < 2007)
      {
       $StartDate = Nth_Sunday  (1, $yArg, 4, 1);
       $EndDate   = Last_Sunday ($yArg, 10, 1);
      }

   else

      {
       $StartDate = Nth_Sunday (2, $yArg, 3, 1);
       $EndDate   = Nth_Sunday (1, $yArg, 11, 1);
      }

   return "$StartDate $EndDate";

}









/*

   function Last_Sunday ($yArg, $mArg, $CalModeArg)
{

   $sign = ($yArg < 0)? -1 : 1;
   $y = $sign * $yArg;
   $m = $mArg;
   $d = 1;

   $mTest = $m;
   $n = 0;

   while ($mTest == $m)
  {
   $n += 1;

   $yyyymmddA = $sign * (($y * 10000) + ($m * 100) + $d);

   $JD12A = Date_To_JD12 ($yyyymmddA, $CalModeArg);

   $JD12B = $JD12A + (7 - ($JD12A + 1) % 7) % 7 + 7*($n-1);

   $ymmddB = JD_To_Date ($JD12B, $CalModeArg);

   $w = floor($sign*$ymmddB / 10000);
   $mTest = floor(($sign*$ymmddB - 10000*$w) / 100);

  }

   $ymmdd = JD_To_YYYYMMDD ($JD12B-7, $CalModeArg);

   return $ymmdd;

}

*/




   function Last_Sunday ($yArg, $mArg, $CalModeArg)

{

   $sign = ($yArg < 0)? -1 : 1;
   $y    = $sign * $yArg;
   $m    = $mArg+1;
   $d    = 1;

   if ($m == 13) {$m=1;  $y = $y+1;}

   $yyyymmddA = $sign * (($y * 10000) + ($m * 100) + $d);

   $JD12A = Date_To_JD12($yyyymmddA, $CalModeArg) - 7;

   $JD12B = $JD12A + (7 - ($JD12A + 1) % 7) % 7;

   $yyyymmddB = JD_To_Date($JD12B, $CalModeArg);

   return $yyyymmddB;

}

?>

