<?PHP

/*

   THIS PROGRAM COMPUTES THE GEOCENTRIC DISTANCE
   OF MERCURY AT ANY TIME.

   VALIDITY TIME SPAN:  2000 BC > Date < 6000 AD

   THIS PROGRAM IS BUILT AROUND KORE v2009.x

*/


// -------------------------------
// Attach required custom modules.

   include_once("Kore.php");


   $cYear = date('Y');

   $out1 = $out2 = $RBState0 = $RBState1 = "";

   $OutBgClass = "BgWhite";
   $OutTxClass = "TxBlack";

   $_INTERNAL_DOC_TITLE_ = "Mercury Daily Distances Table";
   $_ACTION_FILE_NAME_   = $_SERVER['SCRIPT_NAME'];
   $_BGCOLOR_            = "#000022";

   $y = trim(@$_POST['y']);
   $m = trim(@$_POST['m']);
   $d = 1;

   if ($y == "" && $m == "" && $d == 1)
      {
       $y = date("Y");
       $m = date("m");
      }


// Force (m, d) into 2-digit format.
   $m = sPrintF ("%02d", $m);
   $d = sPrintF ("%02d", $d);


   $yyyymmdd = "$y$m$d";




// -------------------
// Read calendar mode.

// NOTE:
// OPPOSITE OF FUNCTIONS: Menu 0=Gregorian and 1=Julian

   $CalModeRB = @$_POST['CalModeRB'];

   if ($CalModeRB == 0)
      {$RBState0 = " CHECKED";  $CalMode = 1;}
   else
      {$RBState1 = " CHECKED";  $CalMode = 0;}


   $CalendarUsed = ($CalModeRB == 0)? "Gregorian" : "Julian";


   $IsValidDate = Is_Valid_Date($yyyymmdd, $CalMode);



// ***************************
// ***************************
// DO THIS BLOCK IF DATE IS OK


   if ($IsValidDate)

{

   $JD12 = YYYYMMDD_To_JD12 ($yyyymmdd, $CalMode);

   $JD00 = $JD12 - 0.5;

   $TimeOfDayFrac = 0;

   $JD = $JD00;
   $JD = bcRound($JD, 16);
   $JD = RTrim(RTrim($JD, "0"), ".");

   $DiM = Days_In_Month ($y, $m, $CalMode);

// ---------------------------------------
// Compute the DoW index (0 to 6) and then
// Get 3-letter DoW abbreviation string.

   $DoW = ($JD12 + 1) % 7;

   $DoWString = trim(substr("Sunday   Monday   Tuesday  WednesdayThursday Friday   Saturday ", 9*$DoW, 9));

// ---------------------------------------
// Get 3-letter month abbreviation string.

   $Mmm = Month_Str($m);

// ---------------
// Get era symbol.

   $era = ($y < 0)? "BC" : "AD";

// --------------------------------------------
// Construct calendar year + era output string.

   $yyyy = abs($y) . " $era";

// ------------------------------------------
// Construct abbreviated calendar date string
// and time-of-day and Delta-T strings.

   $DateString = $Mmm . " $yyyy ";

   $HSep = Str_Repeat("-", 66) . "\n";








// ---------------------------------------
// CONSTRUCT DATE OUTPUT HEADER TEXT BLOCK

   $out1 =

"MERCURY - Geocentric Distance And Light Time Table

 For month of $DateString      ($CalendarUsed Calendar)
 AT 00:00:00 TT on each date    (Delta T = 0)

DAY  ==================== DISTANCE ================  LIGHT_TIME
";



   $out2 = "";



// ***********************
// ***********************
// START COMPUTATIONS HERE


   $JDw = YYYYMMDD_To_JD12 ("$y" . sPrintF("%02d", $m) . "01", $CalMode) - 0.5;

   for ($i=0;   $i < $DiM;   $i++)

  {

   $j = sPrintF("%02d", $i+1);

   $MERCURY = preg_split("[ ]", Mercury_hXYZR($JDw + $i));
   $EARTH = preg_split("[ ]", Earth_hXYZR($JDw + $i));

   $hXp = $MERCURY[0];
   $hYp = $MERCURY[1];
   $hZp = $MERCURY[2];

   $hRp = SqRt($hXp*$hXp + $hYp*$hYp + $hZp*$hZp);

   $hXe = $EARTH[0];
   $hYe = $EARTH[1];
   $hZe = $EARTH[2];

   $gXp = $hXp - $hXe;
   $gYp = $hYp - $hYe;
   $gZp = $hZp - $hZe;

   $gRp = SqRt($gXp*$gXp + $gYp*$gYp + $gZp*$gZp);



// -----------------------------------
// FORMAT THE COMPUTATIONS FOR OUTPUT.

   $hRpAUKMMILT = AU_To_AUKMMILT($hRp, TRUE);
   $gRpAUKMMILT = AU_To_AUKMMILT($gRp, TRUE);

   $w1 = " $j  $gRpAUKMMILT\n";
   $w2 = "";

   $out1 .= $w1;
   $out2 .= $w2;

  }

   $out2 .= "\n";








} // END OF WHAT TO DO IF DATE IS OK


else


// --------------------------------------
// DO THIS BLOCK OF INSTRUCTIONS ON ERROR


  {

   $OutBgClass = "BgMdRed";
   $OutTxClass = "TxWhite";

   $out = "ERROR: Invalid Date";

  }






// ****************************************
// ****************************************


print <<< _HTML

<!DOCTYPE HTML>

<HTML lang='en'>

<HEAD>
<!-- Google tag (gtag.js) -->
<script async src="https://www.googletagmanager.com/gtag/js?id=G-WJ57EQVV4C"></script>
<script>
  window.dataLayer = window.dataLayer || [];
  function gtag(){dataLayer.push(arguments);}
  gtag('js', new Date());

  gtag('config', 'G-WJ57EQVV4C');
</script>

<TITLE>$_INTERNAL_DOC_TITLE_</TITLE>
<meta name='viewport' content='width=device-width, initial-scale=1'>
<meta name="description" content="This PHP program implements the VSOP87 theory to compute the geocentric distance and light time to the planet Mercury from 2000 BC to 6000 AD.">
<meta name="keywords" content="VSOP87,planet Mercury,Mercury distance,Mercury light time">
<meta name="author" content="Jay Tanner - https://www.PHPScienceLabs.com">
<meta http-equiv='pragma'  content='no-cache'>
<meta http-equiv='expires' content='-1'>
<meta name='robots'    content='index,follow'>
<meta name='googlebot' content='index,follow'>
<meta http-equiv='content-type' content='text/html; charset=UTF-8'>
<link rel="canonical" href="https://www.phpsciencelabs.com/vsop87-mercury-geocentric-distance-table/">

<style type='text/css'>

body {
      background:black; color:white: font-family:Verdana; font-size:105%;
      line-height:145%;
     }

div {
     background:white; color:black; font-family:Verdana; font-size:105%;
     line-height:145%; padding:6px;
    }

pre {
     background:white; color:black; font-family:monospace; font-size:150%;
     font-weight:bold; text-align:left; padding:6px;
    }

code {
      font-family:monospace; font-size:150%; font-weight:bold;
      color:black; background:LightYellow; padding:2px;
     }

hr {background:black; height:2px; border:0px;}

td {
    background:white; color:black; font-family:Verdana; text-align:center;
    padding:8px;
   }

img {
     border:1px solid #808080; box-shadow:4px 4px 3px #999999;
     border-radius:8px;
    }

input[type='text']::-ms-clear {width:0; height:0;}

 input[type='text']
{
 color:black; background:white; font-family:monospace;
 font-size:150%; font-weight:bold; text-align:center;
 box-shadow:2px 2px 3px #999999; border:2px solid black;
}

 input[type='text']:focus
{
 color:black; background:white;
 font-family:monospace; font-size:150%; font-weight:bold;
 text-align:center; box-shadow:2px 2px 3px #666666; border:2px solid blue;
}

 input[type='submit']
{
 font-weight:bold; border-radius:4px;
 border:1px solid black;
}

 input[type='submit']:hover
{
 box-shadow:2px 2px 2px #444444; background:cyan;
 font-weight:bold; border-radius:4px;
}

 textarea
{
 background:white; color:black; font-size:130%; font-family:monospace;
 font-weight:bold; border-radius: 8px; border:2px solid blue; padding:6px;
 word-wrap: break-word;
}

 A:link
{
 background:transparent; color:blue; font-size:90%; font-family:Verdana;
 font-weight:bold; text-decoration:none; line-height:175%; padding:2px;
 border:1px solid transparent;
}

A:visited {background:transparent; color:blue; font-size:90%;}

 A:hover
{
 background:yellow; color:black; font-size:90%; border:1px solid black;
 box-shadow:1px 1px 3px #222222;
}

A:active {background:yellow; color:black; font-size:90%;}

::selection      {background:yellow; color:black;}  /* For IE9+ and Safari     */
::-moz-selection {background:yellow; color:black;}  /* For FireFox and Mozilla */

</style>


</HEAD>





<BODY>



<FORM NAME="form1" METHOD="post" ACTION="$_ACTION_FILE_NAME_">

<TABLE WIDTH="648"  border="0" BGCOLOR="black">

<TR>
<TD COLSPAN="3" style='background:#000088; color:white; border:2px solid white; border-radius:8px 8px 0px 0px;'><B>MERCURY - Daily Geocentric Distance and Light-Time Tables<br>Based on the VSOP87 Theory</B><br><span style='font-size:80%;'>PHP Program by Jay Tanner</span></TD>
</TR>

<TR>
<TD COLSPAN="3" ALIGN="center" CLASS="TxGreen">Range: 2000 BC &lt; Date &lt; 6000 AD</TD>
</TR>

<TR>
<TD COLSPAN="3" ALIGN="center" CLASS="BgVLtGray">
&nbsp;<B>&plusmn;Year</B>:&nbsp;<INPUT NAME="y" TYPE="text" class='TxInpCtr' VALUE="$y" SIZE="6" MAXLENGTH="5" title=' Negative = BC Year '>
&nbsp;<B>Month</B>:&nbsp;<INPUT NAME="m" TYPE="text" class='TxInpCtr' VALUE="$m" SIZE="2" MAXLENGTH="2" title=' 01  to  12 '>
&nbsp; &nbsp;&nbsp; &nbsp;<B>Calendar Mode</B>:&nbsp;
<INPUT NAME="CalModeRB" TYPE="radio" VALUE="0"$RBState0>Gregorian&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
<INPUT NAME="CalModeRB" TYPE="radio" VALUE="1"$RBState1>Julian
</TD>
</TR>

<TR>
<TD COLSPAN="3">
<INPUT NAME="Compute_Button" TYPE="submit" VALUE="Compute" style='font-size:105%;'>
</TD>
</TR>

<TR><TD>
<span style='font-size:80%;'>Double-Click Within Text Area to Select ALL Text</span><br>
<textarea rows='38' cols='70' ReadOnly OnDblClick='this.select();' OnMouseUp='return true;'>
$out1$out2</textarea>
</TD></TR>
<tr><td><B style='font-size:75%;'>Jay Tanner - PHP Science Labs - $cYear</B></td></tr>

</TABLE>

</FORM>

</BODY>

</HTML>

_HTML;




/*

   ******************************************************
   MERCURY

   Heliocentric Coords Module - VSOP 87 Theory - Series C3

   Heliocentric rectangular X,Y,Z,R coordinates for the
   mean dynamical ecliptic and equinox of the date.

   Module Range: 2000 BC  to  6000 AD

   Written in PHP v5.2.5

   (c) Jay Tanner - NeoProgrammics - 2008 MAY 06 TUE

   Source code provided under the provisions of
   GNU Affero General Public License, version 3.
   http://www.fsf.org/licensing/licenses/agpl-3.0.html

   Jay Tanner
   VSOP87@NeoProgrammics.com

   ******************************************************

*/


   function Mercury_hXYZR($JDArg)
{

// Compute time in Julian millennia from J2000.0
   $t = ($JDArg - 2451545.0) / 365250.0;

// Compute the heliocentric coordinates for time ($t).
   $hX = Mercury_X0($t) + Mercury_X1($t) + Mercury_X2($t)
       + Mercury_X3($t) + Mercury_X4($t) + Mercury_X5($t);

   $hY = Mercury_Y0($t) + Mercury_Y1($t) + Mercury_Y2($t)
       + Mercury_Y3($t) + Mercury_Y4($t) + Mercury_Y5($t);

   $hZ = Mercury_Z0($t) + Mercury_Z1($t) + Mercury_Z2($t)
       + Mercury_Z3($t) + Mercury_Z4($t) + Mercury_Z5($t);

// Compute heliocentric radius vector (distance).
   $hR = SqRt($hX*$hX + $hY*$hY + $hZ*$hZ);

// Return the heliocentric XYZR coordinates
// in a space delimited data vector.
   return "$hX $hY $hZ $hR";

}





/*

   Mercury - VSOP87 Series Version C3 - Rectangular (X,Y,Z) Coordinates

   X = X-Coordinate in AU (Astronomical Units)
   Y = Y-Coordinate in AU
   Z = Z-Coordinate in AU

   R*R = (X*X + Y*Y + Z*Z)

   HELIOCENTRIC DYNAMICAL ECLIPTIC AND EQUINOX OF THE DATE

   PHP Language
   VSOP87 Functions Source Code
   Generated By NeoProgrammics VSOP87 Code Generator
   (c) Jay Tanner 2008

   Ref:
   Astronomy & Astrophysics
   vol. 202, p309-p315
   Pierre Bretagnon, Gerard Francou
   1988

   Source code provided under the provisions of
   GNU Affero General Public License, version 3.
   http://www.fsf.org/licensing/licenses/agpl-3.0.html

*/


   function Mercury_X0($t) // 1853 terms of order 0
{
   $X0  = 0.37749277893*cos(4.40259139579 + 26088.1469590577*$t);
   $X0 += 0.11918926148*cos(4.49027758439 + 0.2438174835*$t);
   $X0 += 0.03840153904*cos(1.17015646101 + 52176.0501006319*$t);
   $X0 += 0.00585979278*cos(4.22090402969 + 78263.95324220609*$t);
   $X0 += 0.00305833424*cos(2.10298673336 + 26087.65932409069*$t);
   $X0 += 0.00105974941*cos(0.98846517420 + 104351.85638378029*$t);
   $X0 += 0.00024906132*cos(5.26305668971 + 52175.56246566489*$t);
   $X0 += 0.00021056065*cos(4.03921104994 + 130439.75952535449*$t);
   $X0 += 0.00004441671*cos(0.80677167456 + 156527.66266692868*$t);
   $X0 += 0.00003347442*cos(2.10873209401 + 78263.46560723908*$t);
   $X0 += 0.00000976617*cos(3.85751708019 + 182615.56580850288*$t);
   $X0 += 0.00000703903*cos(4.46040321768 + 24978.7684069643*$t);
   $X0 += 0.00000640528*cos(1.20142454428 + 27197.5255111511*$t);
   $X0 += 0.00000558149*cos(5.21749759764 + 104351.36874881329*$t);
   $X0 += 0.00000447775*cos(2.97591075236 + 1059.6257476727*$t);
   $X0 += 0.00000400281*cos(1.86944903196 + 20426.8149099055*$t);
   $X0 += 0.00000283882*cos(3.79350724209 + 31749.4790082099*$t);
   $X0 += 0.00000191007*cos(4.27537754512 + 53285.4286527253*$t);
   $X0 += 0.00000181454*cos(4.95018365158 + 1109.1347346099*$t);
   $X0 += 0.00000193895*cos(0.67633415224 + 4552.1973145423*$t);
   $X0 += 0.00000221371*cos(0.62507677073 + 208703.46895007708*$t);
   $X0 += 0.00000139001*cos(1.22954955274 + 51066.6715485385*$t);
   $X0 += 0.00000183933*cos(1.19500306041 + 5661.0882316687*$t);
   $X0 += 0.00000153303*cos(2.68451868547 + 51116.6681704427*$t);
   $X0 += 0.00000104855*cos(2.02959651352 + 130439.27189038748*$t);
   $X0 += 0.00000115202*cos(0.56691147306 + 57837.3821497841*$t);
   $X0 += 0.00000090628*cos(0.25792152823 + 529.9347825781*$t);
   $X0 += 0.00000075318*cos(6.07215712399 + 27147.5288892469*$t);
   $X0 += 0.00000072957*cos(5.00412566043 + 21536.1934619989*$t);
   $X0 += 0.00000074803*cos(4.92914440147 + 46514.7180514797*$t);
   $X0 += 0.00000068710*cos(0.20379815554 + 25132.5472174491*$t);
   $X0 += 0.00000083504*cos(3.17939445535 + 10213.5293636945*$t);
   $X0 += 0.00000060985*cos(5.45128332631 + 27043.7467006663*$t);
   $X0 += 0.00000057987*cos(5.70479654236 + 14765.48286075331*$t);
   $X0 += 0.00000060953*cos(5.00244327341 + 47624.0966035731*$t);
   $X0 += 0.00000053356*cos(2.29756051926 + 26617.8379241523*$t);
   $X0 += 0.00000054964*cos(5.31144278536 + 12566.3955174663*$t);
   $X0 += 0.00000047881*cos(1.04626983477 + 79373.33179429949*$t);
   $X0 += 0.00000051263*cos(3.53744966658 + 426.8420083595*$t);
   $X0 += 0.00000051361*cos(3.67582188055 + 234791.37209165128*$t);
   $X0 += 0.00000050374*cos(3.60286953784 + 25558.4559939631*$t);
   $X0 += 0.00000048449*cos(3.51993896091 + 1589.3167127673*$t);
   $X0 += 0.00000048867*cos(5.73548681205 + 77204.57131201689*$t);
   $X0 += 0.00000034739*cos(5.68510489030 + 51646.3591355373*$t);
   $X0 += 0.00000032851*cos(2.99883640297 + 955.3559241251*$t);
   $X0 += 0.00000044998*cos(2.48247314947 + 41962.7645544209*$t);
   $X0 += 0.00000031618*cos(6.23993881626 + 37410.8110573621*$t);
   $X0 += 0.00000028634*cos(0.71509820566 + 30640.1004561165*$t);
   $X0 += 0.00000031480*cos(4.28075443320 + 77154.57469011268*$t);
   $X0 += 0.00000032102*cos(3.61877734434 + 83925.28529135829*$t);
   $X0 += 0.00000027670*cos(0.34889366405 + 39609.8984006491*$t);
   $X0 += 0.00000026872*cos(6.26000952694 + 36301.4325052687*$t);
   $X0 += 0.00000023054*cos(2.29052909388 + 53131.6498422405*$t);
   $X0 += 0.00000026193*cos(3.69823179514 + 11322.42028082089*$t);
   $X0 += 0.00000021235*cos(5.11548231564 + 156527.17503196168*$t);
   $X0 += 0.00000026946*cos(1.76252593609 + 6283.3196674749*$t);
   $X0 += 0.00000020875*cos(5.66887296875 + 15874.8614128467*$t);
   $X0 += 0.00000016454*cos(1.69895735267 + 72602.62119305388*$t);
   $X0 += 0.00000016101*cos(3.22620073160 + 9104.1508116011*$t);
   $X0 += 0.00000022241*cos(4.03643224642 + 13521.50762410789*$t);
   $X0 += 0.00000021188*cos(1.77076461428 + 73711.99974514729*$t);
   $X0 += 0.00000015123*cos(2.81544648697 + 18849.4713674577*$t);
   $X0 += 0.00000017255*cos(4.95346284309 + 27197.03787618409*$t);
   $X0 += 0.00000017287*cos(2.12451701243 + 51749.45190975589*$t);
   $X0 += 0.00000013862*cos(2.86403540216 + 53235.43203082109*$t);
   $X0 += 0.00000015239*cos(0.39591934360 + 7238.4317741165*$t);
   $X0 += 0.00000016863*cos(2.14228359969 + 50586.97720534809*$t);
   $X0 += 0.00000013546*cos(5.21986559185 + 52705.7410657265*$t);
   $X0 += 0.00000013134*cos(5.78471170881 + 25028.7650288685*$t);
   $X0 += 0.00000014145*cos(3.03635370573 + 63498.71419893629*$t);
   $X0 += 0.00000013081*cos(1.77889761864 + 26068.47719815791*$t);
   $X0 += 0.00000013058*cos(3.88464356096 + 26107.81671995749*$t);
   $X0 += 0.00000013609*cos(3.25855054515 + 51220.4503590233*$t);
   $X0 += 0.00000013539*cos(2.50293964820 + 103292.47445359109*$t);
   $X0 += 0.00000012138*cos(0.44338161953 + 260879.27523322542*$t);
   $X0 += 0.00000011433*cos(1.98105303514 + 19317.43635781211*$t);
   $X0 += 0.00000010174*cos(2.39402085953 + 77734.26227711148*$t);
   $X0 += 0.00000012016*cos(5.53519559373 + 68050.66769599509*$t);
   $X0 += 0.00000012094*cos(4.09914561799 + 105461.23493587368*$t);
   $X0 += 0.00000009215*cos(2.48225447254 + 40853.3860023275*$t);
   $X0 += 0.00000009157*cos(2.82670347755 + 33326.8225506577*$t);
   $X0 += 0.00000009552*cos(2.11034031277 + 38654.2986590405*$t);
   $X0 += 0.00000012500*cos(3.89244437690 + 45892.9742506405*$t);
   $X0 += 0.00000008978*cos(0.47617343198 + 26084.2656236997*$t);
   $X0 += 0.00000008975*cos(5.18743195952 + 26092.0282944157*$t);
   $X0 += 0.00000012160*cos(3.40664771625 + 65697.80154222329*$t);
   $X0 += 0.00000008880*cos(0.33806954264 + 26514.7451499337*$t);
   $X0 += 0.00000008892*cos(0.74770854519 + 3442.81876244891*$t);
   $X0 += 0.00000008503*cos(0.38676770328 + 110013.18843293248*$t);
   $X0 += 0.00000010244*cos(2.50326457964 + 21535.70582703189*$t);
   $X0 += 0.00000008816*cos(4.86930066154 + 32371.2228090491*$t);
   $X0 += 0.00000009705*cos(5.44815918458 + 529.44714761109*$t);
   $X0 += 0.00000007663*cos(3.73484444482 + 56728.0035976907*$t);
   $X0 += 0.00000007852*cos(0.65554683317 + 5661.5758666357*$t);
   $X0 += 0.00000008166*cos(0.32786623322 + 27677.2198543415*$t);
   $X0 += 0.00000007299*cos(1.14994821071 + 6770.46678376209*$t);
   $X0 += 0.00000007437*cos(3.67894554888 + 32858.8575603033*$t);
   $X0 += 0.00000007796*cos(1.18993742629 + 31748.99137324289*$t);
   $X0 += 0.00000006786*cos(2.43978460253 + 43072.1431065143*$t);
   $X0 += 0.00000007408*cos(4.54789639858 + 23869.3898548709*$t);
   $X0 += 0.00000005938*cos(5.34593352715 + 79219.55298381469*$t);
   $X0 += 0.00000007553*cos(1.04863475066 + 103242.47783168689*$t);
   $X0 += 0.00000005988*cos(3.04417882527 + 62389.33564684289*$t);
   $X0 += 0.00000006154*cos(5.99522472277 + 59414.7256922319*$t);
   $X0 += 0.00000006213*cos(0.75145729813 + 19805.0711090663*$t);
   $X0 += 0.00000006507*cos(1.06344526474 + 213.5429129215*$t);
   $X0 += 0.00000006284*cos(4.91987886888 + 22645.57201409229*$t);
   $X0 += 0.00000006068*cos(4.82277476434 + 99799.90288672148*$t);
   $X0 += 0.00000006276*cos(2.74748364541 + 640.1411037975*$t);
   $X0 += 0.00000006192*cos(1.11265105508 + 28306.90406324449*$t);
   $X0 += 0.00000006081*cos(5.23278439947 + 25874.8478636197*$t);
   $X0 += 0.00000004693*cos(6.19313162709 + 16983.75232997309*$t);
   $X0 += 0.00000004527*cos(1.91140424398 + 182615.07817353586*$t);
   $X0 += 0.00000005085*cos(0.33812112463 + 26301.4460544957*$t);
   $X0 += 0.00000004658*cos(4.11167846402 + 26095.2605060585*$t);
   $X0 += 0.00000005323*cos(2.15023457135 + 24978.28077199729*$t);
   $X0 += 0.00000004497*cos(1.59603827995 + 26081.03341205689*$t);
   $X0 += 0.00000005479*cos(5.17786692376 + 76674.88034692229*$t);
   $X0 += 0.00000003898*cos(4.75161525594 + 98690.52433462809*$t);
   $X0 += 0.00000005352*cos(5.17556723587 + 77837.35505133009*$t);
   $X0 += 0.00000003694*cos(0.48936321493 + 58946.76070187749*$t);
   $X0 += 0.00000003631*cos(5.55357343180 + 129380.37759516528*$t);
   $X0 += 0.00000004821*cos(3.64928863412 + 37410.32342239509*$t);
   $X0 += 0.00000003323*cos(1.95435021777 + 78793.64420730069*$t);
   $X0 += 0.00000003932*cos(0.76247078505 + 29530.7219040231*$t);
   $X0 += 0.00000004022*cos(4.06373546409 + 2119.00767786191*$t);
   $X0 += 0.00000003208*cos(5.89844713322 + 25557.96835899609*$t);
   $X0 += 0.00000003611*cos(0.17577280462 + 91785.70468379749*$t);
   $X0 += 0.00000003062*cos(5.92483558903 + 79323.33517239528*$t);
   $X0 += 0.00000003480*cos(0.66265476262 + 71980.87739221469*$t);
   $X0 += 0.00000003093*cos(2.30327965785 + 94138.57083756929*$t);
   $X0 += 0.00000003172*cos(3.00172025362 + 27043.2590656993*$t);
   $X0 += 0.00000002912*cos(3.49412631679 + 286967.17837479962*$t);
   $X0 += 0.00000003746*cos(6.08415745758 + 89586.61734051049*$t);
   $X0 += 0.00000002870*cos(5.51397391523 + 69160.04624808849*$t);
   $X0 += 0.00000002730*cos(5.88663339742 + 44937.3745090319*$t);
   $X0 += 0.00000002725*cos(5.42457603553 + 103822.16541868569*$t);
   $X0 += 0.00000003214*cos(4.87298940652 + 2218.51328670329*$t);
   $X0 += 0.00000002660*cos(4.82966654366 + 52156.38033973211*$t);
   $X0 += 0.00000003257*cos(2.71278189877 + 25934.3681485729*$t);
   $X0 += 0.00000002620*cos(5.77949301476 + 20426.32727493849*$t);
   $X0 += 0.00000003227*cos(2.95128548648 + 26241.9257695425*$t);
   $X0 += 0.00000003086*cos(0.02730531846 + 77308.35350059748*$t);
   $X0 += 0.00000003099*cos(0.86870364149 + 131549.13807744789*$t);
   $X0 += 0.00000002731*cos(0.19114675485 + 149.8070146181*$t);
   $X0 += 0.00000002486*cos(3.12136440583 + 1052.51220067191*$t);
   $X0 += 0.00000003167*cos(0.38944502659 + 33326.33491569069*$t);
   $X0 += 0.00000002680*cos(0.65295917827 + 52195.71986153169*$t);
   $X0 += 0.00000002231*cos(3.43792316659 + 136101.09157450669*$t);
   $X0 += 0.00000002653*cos(1.68316051523 + 7.3573644843*$t);
   $X0 += 0.00000002074*cos(4.96316067320 + 48733.47515566649*$t);
   $X0 += 0.00000001974*cos(5.16962188890 + 64742.2018006147*$t);
   $X0 += 0.00000002064*cos(0.21494181057 + 51116.18053547569*$t);
   $X0 += 0.00000002533*cos(6.02031593619 + 1066.7392946735*$t);
   $X0 += 0.00000001949*cos(1.68714181050 + 58459.1259506233*$t);
   $X0 += 0.00000002010*cos(2.44539246304 + 16984.2399649401*$t);
   $X0 += 0.00000001876*cos(0.49249036640 + 82815.90673926489*$t);
   $X0 += 0.00000001832*cos(1.95461512625 + 52179.9314359899*$t);
   $X0 += 0.00000002306*cos(4.20103483106 + 54394.80720481869*$t);
   $X0 += 0.00000001837*cos(5.53329054515 + 66941.2891439017*$t);
   $X0 += 0.00000001783*cos(1.94386918249 + 25620.1819687033*$t);
   $X0 += 0.00000001826*cos(3.52694014247 + 52172.1687652739*$t);
   $X0 += 0.00000001791*cos(5.32229718021 + 25661.5487681817*$t);
   $X0 += 0.00000002126*cos(2.91445652095 + 51536.15281431789*$t);
   $X0 += 0.00000001714*cos(3.72047839107 + 26556.11194941211*$t);
   $X0 += 0.00000001972*cos(5.03869626311 + 45405.3394993863*$t);
   $X0 += 0.00000001711*cos(1.11320004200 + 32858.36992533629*$t);
   $X0 += 0.00000001643*cos(3.40283361772 + 52602.6482915079*$t);
   $X0 += 0.00000001665*cos(3.83085129266 + 5327.7199258663*$t);
   $X0 += 0.00000001930*cos(5.78785787875 + 13656.10430865991*$t);
   $X0 += 0.00000001655*cos(1.59098456165 + 125887.80602829569*$t);
   $X0 += 0.00000001554*cos(3.40019334049 + 13521.9952590749*$t);
   $X0 += 0.00000001517*cos(3.39842653971 + 53765.1229959157*$t);
   $X0 += 0.00000001870*cos(4.09951350071 + 129330.38097326108*$t);
   $X0 += 0.00000001705*cos(6.14540598785 + 43071.65547154729*$t);
   $X0 += 0.00000001683*cos(4.32584442162 + 52168.93655363109*$t);
   $X0 += 0.00000001449*cos(6.10448204410 + 88477.23878841709*$t);
   $X0 += 0.00000001466*cos(5.27211735766 + 24499.0740637739*$t);
   $X0 += 0.00000001655*cos(6.12815645892 + 76.50988875911*$t);
   $X0 += 0.00000001539*cos(2.12176953420 + 105307.45612538888*$t);
   $X0 += 0.00000001542*cos(1.96595530367 + 102762.78348849648*$t);
   $X0 += 0.00000001400*cos(1.26185265808 + 26081.51840015769*$t);
   $X0 += 0.00000001400*cos(4.40173881619 + 26094.77551795771*$t);
   $X0 += 0.00000001415*cos(1.31785110641 + 49957.2929964451*$t);
   $X0 += 0.00000001545*cos(2.74460517130 + 85502.6288338061*$t);
   $X0 += 0.00000001259*cos(4.06949140437 + 51962.7510051939*$t);
   $X0 += 0.00000001464*cos(1.94328769190 + 103925.25819290428*$t);
   $X0 += 0.00000001265*cos(6.23687279994 + 35192.0539531753*$t);
   $X0 += 0.00000001563*cos(1.53248979312 + 6681.46867088311*$t);
   $X0 += 0.00000001517*cos(4.14158249910 + 213.0552779545*$t);
   $X0 += 0.00000001171*cos(4.44560816323 + 57836.89451481709*$t);
   $X0 += 0.00000001547*cos(3.43699337530 + 52389.3491960699*$t);
   $X0 += 0.00000001423*cos(1.60092130514 + 50057.2862402535*$t);
   $X0 += 0.00000001238*cos(0.84389836177 + 52183.1636476327*$t);
   $X0 += 0.00000001070*cos(5.82812348558 + 26728.0442453717*$t);
   $X0 += 0.00000001020*cos(3.63373915432 + 12431.79883291429*$t);
   $X0 += 0.00000000965*cos(2.32109036696 + 155468.28073673949*$t);
   $X0 += 0.00000001018*cos(5.37034692069 + 24498.58642880689*$t);
   $X0 += 0.00000000994*cos(3.22732624356 + 117873.60782537168*$t);
   $X0 += 0.00000001002*cos(4.98556762896 + 208702.98131511007*$t);
   $X0 += 0.00000001214*cos(2.58462840069 + 25661.06113321469*$t);
   $X0 += 0.00000000958*cos(1.52001998159 + 124778.42747620228*$t);
   $X0 += 0.00000000893*cos(3.52720868596 + 85034.66384345168*$t);
   $X0 += 0.00000000875*cos(1.72393184919 + 74821.37829724069*$t);
   $X0 += 0.00000000871*cos(0.27938200831 + 1059.1381127057*$t);
   $X0 += 0.00000000910*cos(3.71329662418 + 98068.78053378889*$t);
   $X0 += 0.00000000827*cos(2.44624289116 + 22645.08437912529*$t);
   $X0 += 0.00000000829*cos(4.98909621401 + 104881.54734887488*$t);
   $X0 += 0.00000000870*cos(5.31437509457 + 51066.18391357149*$t);
   $X0 += 0.00000000978*cos(6.15736380216 + 38520.1896094555*$t);
   $X0 += 0.00000000915*cos(1.34262350391 + 6.86972951729*$t);
   $X0 += 0.00000000855*cos(2.53346806421 + 51123.7817174435*$t);
   $X0 += 0.00000000795*cos(5.35307581508 + 120226.47397914348*$t);
   $X0 += 0.00000000778*cos(6.26214759185 + 11322.9079157879*$t);
   $X0 += 0.00000000913*cos(5.46819895663 + 52026.4869034973*$t);
   $X0 += 0.00000000930*cos(2.84523614939 + 115674.52048208468*$t);
   $X0 += 0.00000000932*cos(3.80512706428 + 55618.6250455973*$t);
   $X0 += 0.00000000707*cos(0.26168602915 + 313055.08151637383*$t);
   $X0 += 0.00000000803*cos(3.92107317901 + 157637.04121902208*$t);
   $X0 += 0.00000000730*cos(2.20347820113 + 53284.94101775829*$t);
   $X0 += 0.00000000725*cos(2.69780707537 + 105411.23831396949*$t);
   $X0 += 0.00000000788*cos(5.97840774266 + 3340.8562441833*$t);
   $X0 += 0.00000000867*cos(5.92376156240 + 51109.55462344189*$t);
   $X0 += 0.00000000718*cos(2.18164724460 + 129910.06856025988*$t);
   $X0 += 0.00000000688*cos(0.61823279820 + 419.72846135871*$t);
   $X0 += 0.00000000710*cos(2.26735831674 + 95247.94938966268*$t);
   $X0 += 0.00000000694*cos(5.40455859489 + 26617.35028918529*$t);
   $X0 += 0.00000000621*cos(1.24687546366 + 4551.7096795753*$t);
   $X0 += 0.00000000661*cos(4.35118955440 + 10213.0417287275*$t);
   $X0 += 0.00000000674*cos(0.86923628110 + 28206.9108194361*$t);
   $X0 += 0.00000000609*cos(1.59724799045 + 78244.28348130629*$t);
   $X0 += 0.00000000661*cos(5.76395663117 + 52022.2712901471*$t);
   $X0 += 0.00000000741*cos(3.07875547199 + 103396.25664217169*$t);
   $X0 += 0.00000000585*cos(0.20603891239 + 162188.99471608088*$t);
   $X0 += 0.00000000599*cos(1.83731791923 + 46848.5739922491*$t);
   $X0 += 0.00000000634*cos(4.05934263309 + 39609.41076568209*$t);
   $X0 += 0.00000000731*cos(3.42015997539 + 433.9555553603*$t);
   $X0 += 0.00000000591*cos(2.65948363134 + 71025.27765060609*$t);
   $X0 += 0.00000000576*cos(2.53160242412 + 47623.60896860609*$t);
   $X0 += 0.00000000545*cos(2.34985134231 + 48732.98752069949*$t);
   $X0 += 0.00000000528*cos(4.82834838426 + 23754.95056618569*$t);
   $X0 += 0.00000000531*cos(6.23166694924 + 41962.27691945389*$t);
   $X0 += 0.00000000525*cos(2.22737583220 + 10022.0810975829*$t);
   $X0 += 0.00000000578*cos(0.96705890639 + 80482.71034639288*$t);
   $X0 += 0.00000000675*cos(6.00003515273 + 52329.8289111167*$t);
   $X0 += 0.00000000664*cos(5.96665839431 + 77624.05595589208*$t);
   $X0 += 0.00000000541*cos(2.93745553127 + 6282.83203250789*$t);
   $X0 += 0.00000000639*cos(4.14642362107 + 45494.82524723229*$t);
   $X0 += 0.00000000594*cos(3.31128236497 + 7994.77225950771*$t);
   $X0 += 0.00000000467*cos(4.75719880131 + 84547.0290921975*$t);
   $X0 += 0.00000000552*cos(5.81449540251 + 52099.7840293563*$t);
   $X0 += 0.00000000490*cos(2.96490072888 + 64608.09275102969*$t);
   $X0 += 0.00000000553*cos(2.39742743080 + 1692.40948698591*$t);
   $X0 += 0.00000000456*cos(1.94416761157 + 90830.10494218889*$t);
   $X0 += 0.00000000470*cos(3.29329750332 + 26237.7101561923*$t);
   $X0 += 0.00000000501*cos(4.18799339117 + 25132.0595824821*$t);
   $X0 += 0.00000000615*cos(3.70395803529 + 78283.62300310588*$t);
   $X0 += 0.00000000441*cos(6.26869790938 + 27140.41534224609*$t);
   $X0 += 0.00000000445*cos(4.64236190444 + 151975.70916986988*$t);
   $X0 += 0.00000000437*cos(5.66855783361 + 20760.1832157079*$t);
   $X0 += 0.00000000472*cos(0.86711075193 + 155418.28411483529*$t);
   $X0 += 0.00000000428*cos(3.39576463667 + 77204.08367704989*$t);
   $X0 += 0.00000000469*cos(3.54106330325 + 108903.80988083909*$t);
   $X0 += 0.00000000478*cos(0.48558080454 + 63498.22656396929*$t);
   $X0 += 0.00000000448*cos(1.04430189969 + 78256.83969520529*$t);
   $X0 += 0.00000000591*cos(1.93028393360 + 853.4401992355*$t);
   $X0 += 0.00000000438*cos(3.00642339238 + 51645.87150057029*$t);
   $X0 += 0.00000000440*cos(4.84626648414 + 28306.41642827749*$t);
   $X0 += 0.00000000413*cos(3.60129277287 + 38519.70197448849*$t);
   $X0 += 0.00000000456*cos(2.01311166346 + 26038.1503371535*$t);
   $X0 += 0.00000000456*cos(3.65054925615 + 26138.1435809619*$t);
   $X0 += 0.00000000470*cos(1.08701476541 + 58946.27306691049*$t);
   $X0 += 0.00000000434*cos(2.31499427060 + 29428.7593857575*$t);
   $X0 += 0.00000000439*cos(0.83378397651 + 2333.44021035551*$t);
   $X0 += 0.00000000406*cos(2.57203150650 + 1582.2031657665*$t);
   $X0 += 0.00000000419*cos(5.03633652112 + 128850.68663007069*$t);
   $X0 += 0.00000000403*cos(0.84418373392 + 28421.3433519297*$t);
   $X0 += 0.00000000423*cos(4.33257670397 + 50586.48957038109*$t);
   $X0 += 0.00000000420*cos(5.00524088112 + 78267.83457756409*$t);
   $X0 += 0.00000000508*cos(0.82135338502 + 26720.93069837089*$t);
   $X0 += 0.00000000445*cos(4.23188832059 + 1109.6223695769*$t);
   $X0 += 0.00000000452*cos(5.82726780405 + 25985.0541848391*$t);
   $X0 += 0.00000000440*cos(1.02799745863 + 78050.65414676809*$t);
   $X0 += 0.00000000427*cos(2.83364183360 + 27154.6424362477*$t);
   $X0 += 0.00000000422*cos(2.30681425357 + 93029.19228547589*$t);
   $X0 += 0.00000000387*cos(3.49964872741 + 22747.5345323579*$t);
   $X0 += 0.00000000451*cos(6.18526475915 + 26191.23973327629*$t);
   $X0 += 0.00000000525*cos(4.94137283297 + 25455.3632197445*$t);
   $X0 += 0.00000000439*cos(2.68207148004 + 46514.23041651269*$t);
   $X0 += 0.00000000390*cos(4.99420452598 + 130013.16133447849*$t);
   $X0 += 0.00000000364*cos(0.17463222983 + 78690.55143308209*$t);
   $X0 += 0.00000000468*cos(4.61226407005 + 76145.18938182769*$t);
   $X0 += 0.00000000361*cos(2.87164735708 + 114565.14192999128*$t);
   $X0 += 0.00000000354*cos(0.48572193966 + 52644.0150909863*$t);
   $X0 += 0.00000000418*cos(0.29451759125 + 78260.07190684808*$t);
   $X0 += 0.00000000366*cos(0.57462664693 + 25028.27739390149*$t);
   $X0 += 0.00000000394*cos(0.21360682748 + 78477.25233764409*$t);
   $X0 += 0.00000000337*cos(0.17366623994 + 79853.02613748988*$t);
   $X0 += 0.00000000356*cos(5.33654783666 + 633.0275567967*$t);
   $X0 += 0.00000000427*cos(4.30697428113 + 19804.5834740993*$t);
   $X0 += 0.00000000360*cos(6.08031778998 + 48835.43767393209*$t);
   $X0 += 0.00000000431*cos(4.76277333184 + 30639.61282114949*$t);
   $X0 += 0.00000000336*cos(2.07023419375 + 18208.05780571871*$t);
   $X0 += 0.00000000317*cos(4.36862743851 + 76045.1961380193*$t);
   $X0 += 0.00000000403*cos(5.18083081721 + 131395.35926696309*$t);
   $X0 += 0.00000000300*cos(4.63629291392 + 25973.70767037247*$t);
   $X0 += 0.00000000365*cos(5.56868437730 + 25565.5695409639*$t);
   $X0 += 0.00000000417*cos(1.80624647547 + 71493.2426409605*$t);
   $X0 += 0.00000000298*cos(1.02394718540 + 26202.58624774291*$t);
   $X0 += 0.00000000411*cos(0.25672822810 + 1596.43025976811*$t);
   $X0 += 0.00000000286*cos(1.16931638049 + 52182.67865953191*$t);
   $X0 += 0.00000000304*cos(4.98265595281 + 51708.08511027749*$t);
   $X0 += 0.00000000288*cos(2.94996952897 + 26164.4130303333*$t);
   $X0 += 0.00000000326*cos(0.06254421557 + 26610.72437715149*$t);
   $X0 += 0.00000000375*cos(3.57829900047 + 26162.928557625*$t);
   $X0 += 0.00000000268*cos(6.27766671307 + 143961.51096694589*$t);
   $X0 += 0.00000000367*cos(5.77470435266 + 111590.53197538029*$t);
   $X0 += 0.00000000264*cos(4.93317721296 + 72936.4771338233*$t);
   $X0 += 0.00000000301*cos(2.58808348930 + 224.5886131854*$t);
   $X0 += 0.00000000286*cos(5.68119587232 + 46848.0863572821*$t);
   $X0 += 0.00000000257*cos(5.37176115578 + 181556.18387831368*$t);
   $X0 += 0.00000000352*cos(2.05335376553 + 26013.3653604904*$t);
   $X0 += 0.00000000274*cos(1.10717834982 + 956.53297345411*$t);
   $X0 += 0.00000000253*cos(0.21818001465 + 75.0254160508*$t);
   $X0 += 0.00000000256*cos(2.67592097640 + 537.0483295789*$t);
   $X0 += 0.00000000245*cos(4.84007767717 + 51756.5654567567*$t);
   $X0 += 0.00000000286*cos(4.31029470055 + 52169.42154173189*$t);
   $X0 += 0.00000000326*cos(1.57460855395 + 25771.7550894011*$t);
   $X0 += 0.00000000252*cos(0.65486372524 + 25668.6623151825*$t);
   $X0 += 0.00000000239*cos(3.45236126203 + 42154.2128205325*$t);
   $X0 += 0.00000000324*cos(4.09374409332 + 26404.5388287143*$t);
   $X0 += 0.00000000285*cos(1.43554886267 + 1162.7185218913*$t);
   $X0 += 0.00000000237*cos(0.47964640927 + 124156.68367536309*$t);
   $X0 += 0.00000000271*cos(1.47052171454 + 15874.37377787969*$t);
   $X0 += 0.00000000307*cos(3.88495459836 + 78271.06678920689*$t);
   $X0 += 0.00000000323*cos(1.43180313540 + 19406.43447069109*$t);
   $X0 += 0.00000000245*cos(0.30807019834 + 111122.56698502589*$t);
   $X0 += 0.00000000229*cos(3.58086611266 + 36300.94487030169*$t);
   $X0 += 0.00000000243*cos(5.83259460773 + 7238.9194090835*$t);
   $X0 += 0.00000000242*cos(5.89612108076 + 51748.96427478889*$t);
   $X0 += 0.00000000275*cos(5.58726658873 + 77211.68485901768*$t);
   $X0 += 0.00000000234*cos(1.35631291904 + 83924.79765639128*$t);
   $X0 += 0.00000000279*cos(2.69169755313 + 77197.45776501608*$t);
   $X0 += 0.00000000260*cos(4.48257071630 + 32769.3718124573*$t);
   $X0 += 0.00000000222*cos(4.75592425527 + 100909.28143881488*$t);
   $X0 += 0.00000000279*cos(4.66876901800 + 26507.63160293289*$t);
   $X0 += 0.00000000299*cos(4.60166208316 + 2648.6986429565*$t);
   $X0 += 0.00000000213*cos(0.53490943339 + 1479.11039154791*$t);
   $X0 += 0.00000000221*cos(2.46213816436 + 102.84895673509*$t);
   $X0 += 0.00000000231*cos(5.89019952812 + 141762.42362365888*$t);
   $X0 += 0.00000000208*cos(2.11068814642 + 146314.37712071768*$t);
   $X0 += 0.00000000236*cos(3.31514829524 + 14764.9952257863*$t);
   $X0 += 0.00000000219*cos(0.89003648017 + 110.45013870291*$t);
   $X0 += 0.00000000241*cos(4.57135620023 + 150866.33061777649*$t);
   $X0 += 0.00000000210*cos(0.69084085552 + 183724.94436059629*$t);
   $X0 += 0.00000000280*cos(2.23623280041 + 78114.39004507149*$t);
   $X0 += 0.00000000211*cos(1.74754356172 + 130969.45049044909*$t);
   $X0 += 0.00000000244*cos(1.23689090868 + 11610.7957758577*$t);
   $X0 += 0.00000000265*cos(2.55973299169 + 39744.0074502341*$t);
   $X0 += 0.00000000200*cos(0.58765256464 + 16065.82204399129*$t);
   $X0 += 0.00000000222*cos(0.56541187975 + 81706.5281871715*$t);
   $X0 += 0.00000000197*cos(2.60824419320 + 52815.9473869459*$t);
   $X0 += 0.00000000197*cos(2.88503643876 + 12565.9078824993*$t);
   $X0 += 0.00000000241*cos(3.88588376012 + 27147.04125427989*$t);
   $X0 += 0.00000000205*cos(3.58973790844 + 33968.23611239669*$t);
   $X0 += 0.00000000248*cos(0.88668879236 + 71582.72838880648*$t);
   $X0 += 0.00000000246*cos(2.24144190197 + 51742.33836275509*$t);
   $X0 += 0.00000000203*cos(2.34530658749 + 25881.9614106205*$t);
   $X0 += 0.00000000202*cos(6.11911124355 + 25448.2496727437*$t);
   $X0 += 0.00000000228*cos(1.77307991536 + 234790.88445668427*$t);
   $X0 += 0.00000000198*cos(3.28340120527 + 26294.33250749489*$t);
   $X0 += 0.00000000231*cos(2.35481338711 + 44181.52165860769*$t);
   $X0 += 0.00000000185*cos(3.58471190592 + 64607.60511606269*$t);
   $X0 += 0.00000000226*cos(1.07085750404 + 7879.84533585549*$t);
   $X0 += 0.00000000176*cos(2.16160994172 + 77154.08705514569*$t);
   $X0 += 0.00000000179*cos(1.96834324181 + 14477.1073657165*$t);
   $X0 += 0.00000000195*cos(3.51218600925 + 59414.23805726489*$t);
   $X0 += 0.00000000207*cos(5.81274289850 + 1376.0176173293*$t);
   $X0 += 0.00000000174*cos(3.31014261106 + 339142.98465794802*$t);
   $X0 += 0.00000000174*cos(0.97426408403 + 51543.2663613187*$t);
   $X0 += 0.00000000184*cos(6.12768134790 + 129484.15978374588*$t);
   $X0 += 0.00000000173*cos(4.63181171916 + 23968.89546371229*$t);
   $X0 += 0.00000000207*cos(1.74596827081 + 20760.6708506749*$t);
   $X0 += 0.00000000166*cos(5.44721121225 + 79372.84415933248*$t);
   $X0 += 0.00000000185*cos(2.73373869094 + 103711.95909746629*$t);
   $X0 += 0.00000000161*cos(5.70596905124 + 73711.51211018028*$t);
   $X0 += 0.00000000194*cos(3.26236269810 + 50483.8844311295*$t);
   $X0 += 0.00000000220*cos(6.11849377619 + 69159.55861312148*$t);
   $X0 += 0.00000000191*cos(5.22716664164 + 155997.97170183409*$t);
   $X0 += 0.00000000216*cos(3.03668046944 + 53131.16220727349*$t);
   $X0 += 0.00000000156*cos(4.03054314136 + 106570.61348796709*$t);
   $X0 += 0.00000000181*cos(6.12275012681 + 18093.13088206649*$t);
   $X0 += 0.00000000152*cos(2.52925607729 + 78110.17443172129*$t);
   $X0 += 0.00000000149*cos(1.42595419867 + 45892.48661567349*$t);
   $X0 += 0.00000000182*cos(5.31752226443 + 121335.85253123689*$t);
   $X0 += 0.00000000155*cos(3.25690697601 + 188276.89785765509*$t);
   $X0 += 0.00000000178*cos(5.74884910934 + 131499.14145554368*$t);
   $X0 += 0.00000000166*cos(3.75288461237 + 468.20880783791*$t);
   $X0 += 0.00000000157*cos(2.94486244891 + 61279.9570947495*$t);
   $X0 += 0.00000000203*cos(3.72989631697 + 51322.85371887989*$t);
   $X0 += 0.00000000190*cos(5.02157979409 + 25874.36022865269*$t);
   $X0 += 0.00000000142*cos(3.08736192154 + 50594.09075234889*$t);
   $X0 += 0.00000000175*cos(5.19081352879 + 26624.9514711531*$t);
   $X0 += 0.00000000148*cos(4.64578202487 + 104332.1866228805*$t);
   $X0 += 0.00000000153*cos(5.28337526633 + 52101.26850206459*$t);
   $X0 += 0.00000000164*cos(2.33110807572 + 74820.89066227368*$t);
   $X0 += 0.00000000158*cos(2.76429934042 + 78417.73205269089*$t);
   $X0 += 0.00000000140*cos(2.46992319261 + 52279.14287485049*$t);
   $X0 += 0.00000000163*cos(0.92047983596 + 25780.58933808811*$t);
   $X0 += 0.00000000160*cos(4.73851791777 + 26395.7045800273*$t);
   $X0 += 0.00000000132*cos(4.64443067312 + 22760.01130277751*$t);
   $X0 += 0.00000000129*cos(4.81820902062 + 54394.31956985169*$t);
   $X0 += 0.00000000150*cos(0.46939872247 + 104371.52614468009*$t);
   $X0 += 0.00000000166*cos(0.65776448592 + 522.8212355773*$t);
   $X0 += 0.00000000149*cos(4.09099834194 + 23889.0596157707*$t);
   $X0 += 0.00000000125*cos(0.99047205683 + 65697.31390725628*$t);
   $X0 += 0.00000000127*cos(3.92999950538 + 54294.81396101029*$t);
   $X0 += 0.00000000136*cos(1.42756921141 + 102233.09252340188*$t);
   $X0 += 0.00000000140*cos(3.82491395518 + 52808.83383994509*$t);
   $X0 += 0.00000000171*cos(2.58259586325 + 78187.68717093048*$t);
   $X0 += 0.00000000119*cos(1.41260374923 + 178063.61231144409*$t);
   $X0 += 0.00000000121*cos(3.91577422121 + 181506.18725640948*$t);
   $X0 += 0.00000000150*cos(2.94179740209 + 13362.6935242827*$t);
   $X0 += 0.00000000160*cos(2.07478360203 + 955.8435590921*$t);
   $X0 += 0.00000000122*cos(2.54246856782 + 52072.9573264133*$t);
   $X0 += 0.00000000139*cos(5.71618549148 + 97113.18079218028*$t);
   $X0 += 0.00000000116*cos(4.06977577347 + 104344.74283677948*$t);
   $X0 += 0.00000000127*cos(0.21852764054 + 26521.8586969345*$t);
   $X0 += 0.00000000113*cos(1.81808535780 + 154938.58977164488*$t);
   $X0 += 0.00000000116*cos(1.53733085408 + 110634.93223377169*$t);
   $X0 += 0.00000000120*cos(4.73916175135 + 23969.3830986793*$t);
   $X0 += 0.00000000117*cos(2.35333449828 + 18093.6185170335*$t);
   $X0 += 0.00000000114*cos(0.81259306552 + 25754.2910182883*$t);
   $X0 += 0.00000000150*cos(0.60833287071 + 25551.34244696229*$t);
   $X0 += 0.00000000109*cos(2.00086908622 + 40565.01050729069*$t);
   $X0 += 0.00000000110*cos(2.36264207110 + 24176.9474758405*$t);
   $X0 += 0.00000000107*cos(0.12185511336 + 68242.11596210669*$t);
   $X0 += 0.00000000112*cos(4.84462127363 + 26422.0028998271*$t);
   $X0 += 0.00000000107*cos(5.97982710676 + 90695.99589260388*$t);
   $X0 += 0.00000000104*cos(1.75659774658 + 156101.06447605268*$t);
   $X0 += 0.00000000111*cos(4.99889923860 + 116918.00808376308*$t);
   $X0 += 0.00000000121*cos(1.56849356350 + 28287.2343023447*$t);
   $X0 += 0.00000000108*cos(6.08227647204 + 44181.03402364069*$t);
   $X0 += 0.00000000102*cos(4.15125783923 + 26198.3532802771*$t);
   $X0 += 0.00000000143*cos(5.40448013475 + 50579.86365834729*$t);
   $X0 += 0.00000000108*cos(1.57765174204 + 26107.32908499049*$t);
   $X0 += 0.00000000103*cos(1.01910656324 + 29416.28261533789*$t);
   $X0 += 0.00000000102*cos(2.83999953225 + 74923.34081550628*$t);
   $X0 += 0.00000000111*cos(3.14675866322 + 51639.24558853649*$t);
   $X0 += 0.00000000100*cos(5.42391958785 + 25035.8785758693*$t);
   $X0 += 0.00000000097*cos(5.13819824333 + 323.74923414091*$t);
   $X0 += 0.00000000100*cos(3.26784777828 + 104565.15547921829*$t);
   $X0 += 0.00000000099*cos(0.40287203553 + 31415.6230674405*$t);
   $X0 += 0.00000000120*cos(0.30671438664 + 134991.71302241329*$t);
   $X0 += 0.00000000131*cos(2.63461906713 + 647.25465079831*$t);
   $X0 += 0.00000000128*cos(4.13013360989 + 104138.55728834229*$t);
   $X0 += 0.00000000110*cos(0.16249290358 + 24357.0246061251*$t);
   $X0 += 0.00000000105*cos(5.76370877998 + 26067.9895631909*$t);
   $X0 += 0.00000000095*cos(2.53553314645 + 52705.25343075949*$t);
   $X0 += 0.00000000105*cos(5.29014619681 + 55516.6625273317*$t);
   $X0 += 0.00000000093*cos(5.06220815245 + 52126.05347872769*$t);
   $X0 += 0.00000000092*cos(3.89022682970 + 54509.2464935039*$t);
   $X0 += 0.00000000096*cos(3.48344687819 + 743.23387801611*$t);
   $X0 += 0.00000000109*cos(4.33764824375 + 632.5399218297*$t);
   $X0 += 0.00000000090*cos(3.28421553573 + 27999.3464422749*$t);
   $X0 += 0.00000000094*cos(0.26566762837 + 103291.98681862408*$t);
   $X0 += 0.00000000111*cos(4.40219993403 + 40565.4981422577*$t);
   $X0 += 0.00000000101*cos(3.06854770182 + 51951.70530492999*$t);
   $X0 += 0.00000000102*cos(1.77060203080 + 104355.73771913828*$t);
   $X0 += 0.00000000088*cos(0.98000029962 + 1588.82907780029*$t);
   $X0 += 0.00000000102*cos(5.00530419072 + 26941.3433408097*$t);
   $X0 += 0.00000000109*cos(0.41184272678 + 60056.13925397089*$t);
   $X0 += 0.00000000093*cos(5.48203494654 + 27780.31262856009*$t);
   $X0 += 0.00000000089*cos(3.15455699832 + 68050.18006102808*$t);
   $X0 += 0.00000000106*cos(3.72898790593 + 6770.9544187291*$t);
   $X0 += 0.00000000101*cos(0.37716551293 + 52250.8316991992*$t);
   $X0 += 0.00000000086*cos(0.08251374650 + 52325.61329776649*$t);
   $X0 += 0.00000000092*cos(5.92652248722 + 140653.04507156549*$t);
   $X0 += 0.00000000107*cos(1.95514516893 + 157483.26240853728*$t);
   $X0 += 0.00000000087*cos(2.25938740860 + 25938.5837619231*$t);
   $X0 += 0.00000000094*cos(4.55110857685 + 51219.76094466129*$t);
   $X0 += 0.00000000089*cos(5.24168644118 + 36109.9842391571*$t);
   $X0 += 0.00000000093*cos(3.23654834978 + 103.3365917021*$t);
   $X0 += 0.00000000098*cos(5.50021939322 + 27819.2693119903*$t);
   $X0 += 0.00000000084*cos(3.07323794777 + 53228.31848382029*$t);
   $X0 += 0.00000000087*cos(4.71717400578 + 52065.84377941249*$t);
   $X0 += 0.00000000099*cos(4.86243388147 + 97581.14578253469*$t);
   $X0 += 0.00000000105*cos(5.43405944979 + 70269.42480018188*$t);
   $X0 += 0.00000000083*cos(1.07166783581 + 51219.96272405629*$t);
   $X0 += 0.00000000101*cos(0.37289734688 + 52226.0467225361*$t);
   $X0 += 0.00000000087*cos(3.22552375788 + 104778.45457465628*$t);
   $X0 += 0.00000000102*cos(6.17051419926 + 12432.2864678813*$t);
   $X0 += 0.00000000082*cos(3.53594042739 + 78731.91823256049*$t);
   $X0 += 0.00000000101*cos(5.35965052966 + 119117.0954270501*$t);
   $X0 += 0.00000000108*cos(1.06265370367 + 49527.59527515889*$t);
   $X0 += 0.00000000092*cos(4.58719441891 + 51859.65823097529*$t);
   $X0 += 0.00000000080*cos(5.90301150782 + 53242.5455778219*$t);
   $X0 += 0.00000000107*cos(3.33033819724 + 25447.76203777669*$t);
   $X0 += 0.00000000088*cos(6.01318518947 + 12725.20961729149*$t);
   $X0 += 0.00000000091*cos(3.91995839732 + 26300.95841952869*$t);
   $X0 += 0.00000000080*cos(3.22630056109 + 105940.92927906409*$t);
   $X0 += 0.00000000076*cos(1.13669532403 + 102133.09927959349*$t);
   $X0 += 0.00000000079*cos(4.83610406929 + 80482.22271142589*$t);
   $X0 += 0.00000000091*cos(4.23329610745 + 51013.57539622409*$t);
   $X0 += 0.00000000094*cos(2.96601881549 + 2222.1004520805*$t);
   $X0 += 0.00000000089*cos(2.52058786962 + 137678.43511695448*$t);
   $X0 += 0.00000000075*cos(4.78568609813 + 3327.89183879669*$t);
   $X0 += 0.00000000089*cos(0.85977762746 + 1910.9556657337*$t);
   $X0 += 0.00000000098*cos(3.32594040773 + 50056.79860528649*$t);
   $X0 += 0.00000000074*cos(1.41768963459 + 25977.94063783829*$t);
   $X0 += 0.00000000089*cos(0.85266701038 + 52492.4419702885*$t);
   $X0 += 0.00000000073*cos(5.13639330582 + 50697.1835265675*$t);
   $X0 += 0.00000000102*cos(3.34303114673 + 104347.97504842229*$t);
   $X0 += 0.00000000072*cos(3.04850836782 + 170049.41410852008*$t);
   $X0 += 0.00000000078*cos(5.73381249578 + 103285.36090659029*$t);
   $X0 += 0.00000000075*cos(1.55658438412 + 52595.53474450709*$t);
   $X0 += 0.00000000074*cos(2.73392405787 + 25021.6514818677*$t);
   $X0 += 0.00000000075*cos(1.65865555855 + 77844.46859833089*$t);
   $X0 += 0.00000000078*cos(1.07533201236 + 74.53778108379*$t);
   $X0 += 0.00000000094*cos(3.12911978302 + 52698.62751872569*$t);
   $X0 += 0.00000000095*cos(5.99575918602 + 25938.09612695609*$t);
   $X0 += 0.00000000087*cos(5.82357433280 + 72602.13355808688*$t);
   $X0 += 0.00000000070*cos(5.67887850233 + 27670.10630734069*$t);
   $X0 += 0.00000000076*cos(0.64559937719 + 104358.96993078108*$t);
   $X0 += 0.00000000074*cos(5.28706150347 + 25954.0379094727*$t);
   $X0 += 0.00000000070*cos(1.27402715914 + 66653.40128383189*$t);
   $X0 += 0.00000000071*cos(3.97675324486 + 24998.4381678641*$t);
   $X0 += 0.00000000073*cos(0.37426023469 + 26222.2560086427*$t);
   $X0 += 0.00000000081*cos(4.55346032047 + 8194.0315157251*$t);
   $X0 += 0.00000000066*cos(4.21780979859 + 78270.58180110609*$t);
   $X0 += 0.00000000072*cos(1.38206164812 + 426.3543733925*$t);
   $X0 += 0.00000000078*cos(5.28440082300 + 104202.29318664569*$t);
   $X0 += 0.00000000066*cos(3.65294907987 + 89586.12970554348*$t);
   $X0 += 0.00000000081*cos(4.51389667692 + 26080.54577708989*$t);
   $X0 += 0.00000000068*cos(2.13854484878 + 207644.08701988788*$t);
   $X0 += 0.00000000065*cos(5.87225172961 + 12546.72575656651*$t);
   $X0 += 0.00000000074*cos(3.91991873665 + 97670.63153038069*$t);
   $X0 += 0.00000000067*cos(2.82169598550 + 2111.8941308611*$t);
   $X0 += 0.00000000079*cos(5.28726952189 + 77830.24150432929*$t);
   $X0 += 0.00000000083*cos(3.78654945074 + 522.3336006103*$t);
   $X0 += 0.00000000079*cos(3.79951632035 + 25446.73339731871*$t);
   $X0 += 0.00000000063*cos(6.08370597700 + 70268.93716521488*$t);
   $X0 += 0.00000000074*cos(2.86346034860 + 95247.46175469569*$t);
   $X0 += 0.00000000085*cos(5.74305012713 + 72935.98949885629*$t);
   $X0 += 0.00000000062*cos(3.50617407914 + 150244.58681693728*$t);
   $X0 += 0.00000000081*cos(6.24341823549 + 77733.77464214448*$t);
   $X0 += 0.00000000077*cos(1.16703890175 + 316.6356871401*$t);
   $X0 += 0.00000000075*cos(1.04471353660 + 33967.74847742969*$t);
   $X0 += 0.00000000071*cos(3.33594132090 + 27684.3334013423*$t);
   $X0 += 0.00000000061*cos(4.37869496731 + 846.3266522347*$t);
   $X0 += 0.00000000064*cos(1.68211518159 + 27177.8557502513*$t);
   $X0 += 0.00000000073*cos(4.46095079744 + 26083.7779887327*$t);
   $X0 += 0.00000000075*cos(1.85897859977 + 26729.56052079669*$t);
   $X0 += 0.00000000071*cos(4.93586530137 + 17893.8716258491*$t);
   $X0 += 0.00000000063*cos(4.07467864823 + 52290.48938931711*$t);
   $X0 += 0.00000000072*cos(6.13433603432 + 50800.27630078609*$t);
   $X0 += 0.00000000059*cos(1.40204104749 + 52061.61081194667*$t);
   $X0 += 0.00000000077*cos(2.75393031467 + 38813.6003938327*$t);
   $X0 += 0.00000000080*cos(0.16011950416 + 26089.63143176599*$t);
   $X0 += 0.00000000069*cos(1.59730247279 + 56727.51596272369*$t);
   $X0 += 0.00000000080*cos(5.51239927463 + 26086.6624863494*$t);
   $X0 += 0.00000000058*cos(2.41520369138 + 419.2408263917*$t);
   $X0 += 0.00000000066*cos(1.67365233236 + 2008.8013566425*$t);
   $X0 += 0.00000000065*cos(2.06432984048 + 66652.91364886488*$t);
   $X0 += 0.00000000058*cos(1.62097861042 + 99024.38027539749*$t);
   $X0 += 0.00000000077*cos(2.34966674240 + 103299.58800059189*$t);
   $X0 += 0.00000000069*cos(1.46687065035 + 316.14805217309*$t);
   $X0 += 0.00000000061*cos(1.53694228847 + 126997.18458038908*$t);
   $X0 += 0.00000000068*cos(3.36042091101 + 137210.47012660008*$t);
   $X0 += 0.00000000058*cos(2.64478381351 + 167850.32676523308*$t);
   $X0 += 0.00000000057*cos(2.63258373208 + 26011.8808877821*$t);
   $X0 += 0.00000000062*cos(5.91411269940 + 19316.9487228451*$t);
   $X0 += 0.00000000073*cos(2.88502409591 + 26091.5406594487*$t);
   $X0 += 0.00000000055*cos(3.74371053813 + 209812.84750217047*$t);
   $X0 += 0.00000000055*cos(5.12214562228 + 44295.9609472929*$t);
   $X0 += 0.00000000058*cos(1.54167828285 + 25508.4593720589*$t);
   $X0 += 0.00000000073*cos(0.48335623767 + 18848.98373249069*$t);
   $X0 += 0.00000000056*cos(5.41229627249 + 22625.9022531925*$t);
   $X0 += 0.00000000064*cos(0.50377008046 + 77410.75686045409*$t);
   $X0 += 0.00000000054*cos(6.01642065343 + 52252.31617190749*$t);
   $X0 += 0.00000000066*cos(1.74674543912 + 77795.98825185168*$t);
   $X0 += 0.00000000055*cos(3.61457759506 + 107794.43132874569*$t);
   $X0 += 0.00000000057*cos(5.00011319395 + 24449.0774418697*$t);
   $X0 += 0.00000000055*cos(5.16272010710 + 172402.28026229188*$t);
   $X0 += 0.00000000067*cos(1.36451686419 + 19.42594341629*$t);
   $X0 += 0.00000000053*cos(2.13528819255 + 32370.73517408209*$t);
   $X0 += 0.00000000052*cos(5.50451319252 + 3340.36860921629*$t);
   $X0 += 0.00000000055*cos(5.18023594496 + 220.6564599223*$t);
   $X0 += 0.00000000072*cos(4.73998437139 + 24864.32911827909*$t);
   $X0 += 0.00000000052*cos(2.23631389572 + 23868.9022199039*$t);
   $X0 += 0.00000000060*cos(3.57826041100 + 90695.50825763689*$t);
   $X0 += 0.00000000052*cos(5.67658506171 + 26312.4917547596*$t);
   $X0 += 0.00000000053*cos(1.86041754394 + 19406.9221056581*$t);
   $X0 += 0.00000000053*cos(4.13410058005 + 26667.8345460565*$t);
   $X0 += 0.00000000070*cos(1.65225887205 + 2199.33116077051*$t);
   $X0 += 0.00000000059*cos(5.11557005640 + 38.3768531213*$t);
   $X0 += 0.00000000051*cos(1.23099026352 + 58857.2749540315*$t);
   $X0 += 0.00000000050*cos(5.77314673421 + 129799.86223904048*$t);
   $X0 += 0.00000000067*cos(1.08017341186 + 78257.32468330608*$t);
   $X0 += 0.00000000050*cos(0.63851425030 + 27727.2164762457*$t);
   $X0 += 0.00000000048*cos(4.17567122141 + 27044.4361150283*$t);
   $X0 += 0.00000000047*cos(5.67782985051 + 51969.8645521947*$t);
   $X0 += 0.00000000055*cos(4.78351928938 + 157057.35363202328*$t);
   $X0 += 0.00000000060*cos(1.14600976483 + 85034.17620848469*$t);
   $X0 += 0.00000000049*cos(3.86127420297 + 77631.16950289288*$t);
   $X0 += 0.00000000064*cos(0.02229686602 + 76571.78757270369*$t);
   $X0 += 0.00000000046*cos(2.81829667996 + 77836.86741636308*$t);
   $X0 += 0.00000000054*cos(5.78884650295 + 23919.3864767751*$t);
   $X0 += 0.00000000047*cos(2.89825161341 + 155572.06292532009*$t);
   $X0 += 0.00000000064*cos(0.92372709577 + 27311.96479983631*$t);
   $X0 += 0.00000000046*cos(6.08737593166 + 76681.99389392308*$t);
   $X0 += 0.00000000046*cos(4.76636361010 + 29530.2342690561*$t);
   $X0 += 0.00000000053*cos(5.00175567579 + 49842.85370775989*$t);
   $X0 += 0.00000000055*cos(0.24135443971 + 27037.32256802749*$t);
   $X0 += 0.00000000054*cos(3.68631996655 + 949.4194264533*$t);
   $X0 += 0.00000000047*cos(0.24536622662 + 29550.3916649229*$t);
   $X0 += 0.00000000056*cos(0.74089796514 + 53234.94439585409*$t);
   $X0 += 0.00000000062*cos(0.02840313751 + 26011.39325281509*$t);
   $X0 += 0.00000000051*cos(1.39538802358 + 28736.6017845307*$t);
   $X0 += 0.00000000045*cos(4.11327342358 + 55504.18575691209*$t);
   $X0 += 0.00000000062*cos(1.33928421875 + 176954.23375935068*$t);
   $X0 += 0.00000000043*cos(5.66151866873 + 78903.8505285201*$t);
   $X0 += 0.00000000044*cos(3.67807658881 + 16703.3059509825*$t);
   $X0 += 0.00000000052*cos(4.49985681008 + 110012.70079796549*$t);
   $X0 += 0.00000000050*cos(5.46692304514 + 25138.9713500879*$t);
   $X0 += 0.00000000043*cos(4.35523617525 + 1272.9248431107*$t);
   $X0 += 0.00000000044*cos(2.19769750104 + 105460.74730090669*$t);
   $X0 += 0.00000000058*cos(1.25267627496 + 26094.77287109149*$t);
   $X0 += 0.00000000042*cos(2.03273940863 + 1486.2239385487*$t);
   $X0 += 0.00000000042*cos(2.08186025120 + 78189.17164363878*$t);
   $X0 += 0.00000000048*cos(5.60414814999 + 65831.91059180829*$t);
   $X0 += 0.00000000054*cos(0.13039414614 + 52382.23564906909*$t);
   $X0 += 0.00000000041*cos(3.34731027231 + 1795.5022612045*$t);
   $X0 += 0.00000000043*cos(1.03324982448 + 60055.65161900389*$t);
   $X0 += 0.00000000046*cos(0.15692625477 + 54087.2495838491*$t);
   $X0 += 0.00000000053*cos(4.84593569859 + 260878.78759825832*$t);
   $X0 += 0.00000000048*cos(0.50977666152 + 25466.4031582185*$t);
   $X0 += 0.00000000048*cos(5.15384836969 + 26709.8907598969*$t);
   $X0 += 0.00000000048*cos(2.14565331737 + 76667.76679992149*$t);
   $X0 += 0.00000000047*cos(5.63034544251 + 104275.59031250469*$t);
   $X0 += 0.00000000039*cos(0.78144245702 + 132658.51662954128*$t);
   $X0 += 0.00000000040*cos(1.64169511886 + 26824.02347258949*$t);
   $X0 += 0.00000000040*cos(4.01282714257 + 25352.2704455259*$t);
   $X0 += 0.00000000043*cos(0.07999630612 + 365230.88779952223*$t);
   $X0 += 0.00000000044*cos(1.85242708019 + 52712.85461272729*$t);
   $X0 += 0.00000000045*cos(6.15705935255 + 28256.9074413403*$t);
   $X0 += 0.00000000039*cos(5.27554854731 + 103241.99019671988*$t);
   $X0 += 0.00000000051*cos(1.99101186951 + 182085.87484340828*$t);
   $X0 += 0.00000000050*cos(2.09130353046 + 147423.75567281108*$t);
   $X0 += 0.00000000043*cos(0.02949186925 + 214364.80099922928*$t);
   $X0 += 0.00000000039*cos(2.59117531794 + 99799.41525175449*$t);
   $X0 += 0.00000000044*cos(5.29675829214 + 25867.73431661889*$t);
   $X0 += 0.00000000043*cos(1.77519064144 + 1265.81129610991*$t);
   $X0 += 0.00000000038*cos(5.81703538519 + 104505.63519426509*$t);
   $X0 += 0.00000000037*cos(5.58241031795 + 104198.07757329549*$t);
   $X0 += 0.00000000037*cos(4.53399815239 + 128320.99566497609*$t);
   $X0 += 0.00000000042*cos(5.19727486409 + 25170.2166730759*$t);
   $X0 += 0.00000000040*cos(5.22993102913 + 206.42936592071*$t);
   $X0 += 0.00000000038*cos(3.52230881007 + 130459.42928625426*$t);
   $X0 += 0.00000000037*cos(3.93629350281 + 26102.3740530593*$t);
   $X0 += 0.00000000050*cos(0.24736089771 + 153.5349930013*$t);
   $X0 += 0.00000000037*cos(1.67080581846 + 24203.24579564031*$t);
   $X0 += 0.00000000041*cos(0.69210311059 + 34282.4222922663*$t);
   $X0 += 0.00000000036*cos(3.01321329685 + 26118.4738200621*$t);
   $X0 += 0.00000000036*cos(2.65041173436 + 26057.82009805329*$t);
   $X0 += 0.00000000036*cos(4.58551065069 + 34281.9346572993*$t);
   $X0 += 0.00000000042*cos(6.16382020852 + 38653.81102407349*$t);
   $X0 += 0.00000000035*cos(1.73963103644 + 26073.91986505609*$t);
   $X0 += 0.00000000039*cos(0.45985553144 + 27006.0772450395*$t);
   $X0 += 0.00000000039*cos(5.40193552573 + 151.2914873264*$t);
   $X0 += 0.00000000037*cos(5.43157910397 + 78367.04601642469*$t);
   $X0 += 0.00000000034*cos(3.56564955468 + 27567.01353312209*$t);
   $X0 += 0.00000000035*cos(0.56899615907 + 78896.73698151928*$t);
   $X0 += 0.00000000037*cos(4.00811594346 + 25773.96077918809*$t);
   $X0 += 0.00000000037*cos(1.65550908775 + 26402.33313892731*$t);
   $X0 += 0.00000000045*cos(0.79814686706 + 2126.1212248627*$t);
   $X0 += 0.00000000037*cos(5.12681406758 + 9384.59719059169*$t);
   $X0 += 0.00000000033*cos(2.80453550266 + 116783.89903417809*$t);
   $X0 += 0.00000000035*cos(2.62159531521 + 27463.9207589035*$t);
   $X0 += 0.00000000033*cos(0.53015735881 + 51852.54468397449*$t);
   $X0 += 0.00000000037*cos(1.41559511533 + 130420.08976445469*$t);
   $X0 += 0.00000000035*cos(0.57763840050 + 62388.84801187589*$t);
   $X0 += 0.00000000033*cos(1.99538084924 + 26190.1094773233*$t);
   $X0 += 0.00000000033*cos(3.66824418197 + 25986.18444079209*$t);
   $X0 += 0.00000000035*cos(3.17259770035 + 51653.47268253809*$t);
   $X0 += 0.00000000032*cos(6.04952225858 + 39629.56816154889*$t);
   $X0 += 0.00000000040*cos(4.40766389323 + 148.32254190981*$t);
   $X0 += 0.00000000043*cos(2.52314764326 + 157587.04459711789*$t);
   $X0 += 0.00000000034*cos(3.99284017002 + 27973.04812247509*$t);
   $X0 += 0.00000000043*cos(0.54047908222 + 26308.5596014965*$t);
   $X0 += 0.00000000040*cos(0.99428555932 + 205.9417309537*$t);
   $X0 += 0.00000000034*cos(3.54482897451 + 13541.17738500769*$t);
   $X0 += 0.00000000031*cos(0.68583013925 + 207594.09039798369*$t);
   $X0 += 0.00000000032*cos(4.46011355827 + 204151.51545301828*$t);
   $X0 += 0.00000000034*cos(3.97349842615 + 51868.4924796623*$t);
   $X0 += 0.00000000043*cos(3.01518313757 + 51529.03926731709*$t);
   $X0 += 0.00000000031*cos(5.58184574820 + 78160.86046798748*$t);
   $X0 += 0.00000000036*cos(0.93475827193 + 130226.46042991648*$t);
   $X0 += 0.00000000030*cos(0.82681675219 + 130432.64597835368*$t);
   $X0 += 0.00000000039*cos(5.59499527090 + 64901.5035354069*$t);
   $X0 += 0.00000000031*cos(2.16793872992 + 51433.0600400993*$t);
   $X0 += 0.00000000041*cos(0.15772285796 + 40852.8983673605*$t);
   $X0 += 0.00000000039*cos(1.91317014888 + 76674.39271195528*$t);
   $X0 += 0.00000000031*cos(3.23141999011 + 94330.01910368088*$t);
   $X0 += 0.00000000030*cos(4.87996329072 + 181026.49291321907*$t);
   $X0 += 0.00000000030*cos(6.07201296464 + 96356.84030678908*$t);
   $X0 += 0.00000000029*cos(4.81126203138 + 182188.96761762688*$t);
   $X0 += 0.00000000034*cos(4.11459561196 + 111122.07935005888*$t);
   $X0 += 0.00000000031*cos(4.20399330248 + 26514.25751496669*$t);
   $X0 += 0.00000000036*cos(4.04094788403 + 75615.49841673308*$t);
   $X0 += 0.00000000028*cos(1.03310392380 + 86143.55476057807*$t);
   $X0 += 0.00000000028*cos(0.70757951529 + 80382.71710258449*$t);
   $X0 += 0.00000000037*cos(4.30143869314 + 37698.6989174319*$t);
   $X0 += 0.00000000030*cos(0.97985183975 + 77101.47853779829*$t);
   $X0 += 0.00000000029*cos(4.41729316787 + 114.6429243969*$t);
   $X0 += 0.00000000030*cos(4.17789908809 + 26084.21480579459*$t);
   $X0 += 0.00000000027*cos(0.61831878834 + 26241.4381345755*$t);
   $X0 += 0.00000000027*cos(4.67046992186 + 60369.83779887349*$t);
   $X0 += 0.00000000027*cos(5.92007475132 + 101011.24395708049*$t);
   $X0 += 0.00000000027*cos(1.62628759375 + 52509.9060414013*$t);
   $X0 += 0.00000000029*cos(1.49521060767 + 26092.07911232079*$t);
   $X0 += 0.00000000033*cos(6.04618436459 + 42153.72518556549*$t);
   $X0 += 0.00000000033*cos(2.68337298749 + 49954.19346603489*$t);
   $X0 += 0.00000000027*cos(0.40084478333 + 25933.8805136059*$t);
   $X0 += 0.00000000027*cos(4.17930801308 + 91785.21704883048*$t);
   $X0 += 0.00000000026*cos(2.02354598068 + 35472.9879671329*$t);
   $X0 += 0.00000000034*cos(1.25359434355 + 22004.1584523533*$t);
   $X0 += 0.00000000026*cos(4.60481238290 + 71980.38975724769*$t);
   $X0 += 0.00000000026*cos(0.04548898361 + 130653.05862079248*$t);
   $X0 += 0.00000000029*cos(4.59660921219 + 136722.83537534589*$t);
   $X0 += 0.00000000035*cos(1.50172306531 + 52483.6077216015*$t);
   $X0 += 0.00000000027*cos(4.62222063183 + 54375.1374439189*$t);
   $X0 += 0.00000000026*cos(5.92161181229 + 26050.0139234199*$t);
   $X0 += 0.00000000033*cos(6.20772055762 + 77727.14873011068*$t);
   $X0 += 0.00000000031*cos(6.01164257984 + 87367.86023632369*$t);
   $X0 += 0.00000000028*cos(5.00871771816 + 183571.16555011149*$t);
   $X0 += 0.00000000034*cos(0.00412760548 + 1905.70858242391*$t);
   $X0 += 0.00000000028*cos(2.05766266145 + 51226.87449166209*$t);
   $X0 += 0.00000000032*cos(1.32626829577 + 77307.66408623548*$t);
   $X0 += 0.00000000033*cos(6.12204372905 + 78039.60844650418*$t);
   $X0 += 0.00000000031*cos(3.88641044819 + 60370.3254338405*$t);
   $X0 += 0.00000000033*cos(2.48500437212 + 123201.08393375449*$t);
   $X0 += 0.00000000027*cos(0.91898929060 + 1224.06165826211*$t);
   $X0 += 0.00000000025*cos(3.43998049238 + 78338.73484077338*$t);
   $X0 += 0.00000000029*cos(2.28677197427 + 49842.36607279289*$t);
   $X0 += 0.00000000025*cos(1.89094361261 + 76785.08666814168*$t);
   $X0 += 0.00000000024*cos(2.66952173476 + 24824.9895964795*$t);
   $X0 += 0.00000000031*cos(3.35325440781 + 161079.61616398748*$t);
   $X0 += 0.00000000031*cos(1.70259252563 + 75930.75684933408*$t);
   $X0 += 0.00000000024*cos(3.24402392538 + 1045.3986536711*$t);
   $X0 += 0.00000000029*cos(5.58510707424 + 25600.5122078035*$t);
   $X0 += 0.00000000029*cos(0.07851795697 + 26575.7817103119*$t);
   $X0 += 0.00000000026*cos(5.97420147293 + 39450.5966658569*$t);
   $X0 += 0.00000000027*cos(0.73776364996 + 27171.2271913513*$t);
   $X0 += 0.00000000027*cos(4.92586138125 + 25005.0667267641*$t);
   $X0 += 0.00000000032*cos(0.79345370764 + 9103.6631766341*$t);
   $X0 += 0.00000000024*cos(2.49242891465 + 129373.26404816448*$t);
   $X0 += 0.00000000023*cos(0.40602877121 + 85502.14119883909*$t);
   $X0 += 0.00000000024*cos(3.27434836834 + 52609.7618385087*$t);
   $X0 += 0.00000000025*cos(4.99949553531 + 299.37021175271*$t);
   $X0 += 0.00000000027*cos(1.76554597629 + 143005.91122533729*$t);
   $X0 += 0.00000000027*cos(1.29828079538 + 24073.1652872599*$t);
   $X0 += 0.00000000025*cos(5.44789800662 + 25654.43522118089*$t);
   $X0 += 0.00000000024*cos(0.97156468916 + 52286.25642185129*$t);
   $X0 += 0.00000000023*cos(5.20850734570 + 38813.11275886569*$t);
   $X0 += 0.00000000026*cos(4.38403477094 + 27676.73221937449*$t);
   $X0 += 0.00000000023*cos(2.99428107393 + 27351.3043216359*$t);
   $X0 += 0.00000000025*cos(1.41127895754 + 48847.9144443517*$t);
   $X0 += 0.00000000028*cos(4.21226965873 + 24925.6722546499*$t);
   $X0 += 0.00000000029*cos(4.41166263353 + 30172.1354657621*$t);
   $X0 += 0.00000000026*cos(1.50614892755 + 78153.74692098668*$t);
   $X0 += 0.00000000023*cos(2.83196594205 + 50064.3997872543*$t);
   $X0 += 0.00000000024*cos(1.97684419653 + 51962.26337022689*$t);
   $X0 += 0.00000000023*cos(1.32747738833 + 77947.56137254949*$t);
   $X0 += 0.00000000023*cos(1.63082397395 + 123669.04892410889*$t);
   $X0 += 0.00000000022*cos(1.83245731848 + 78213.95662030189*$t);
   $X0 += 0.00000000025*cos(5.93871501925 + 26725.1432310819*$t);
   $X0 += 0.00000000025*cos(6.00809531914 + 25451.1506870335*$t);
   $X0 += 0.00000000022*cos(5.41900898206 + 50264.8506174147*$t);
   $X0 += 0.00000000030*cos(2.27535725604 + 53907.1724535645*$t);
   $X0 += 0.00000000030*cos(2.41447471924 + 23754.46293121869*$t);
   $X0 += 0.00000000025*cos(3.33106077360 + 1.7282901918*$t);
   $X0 += 0.00000000025*cos(3.33106077360 + 1.24065522479*$t);
   $X0 += 0.00000000024*cos(6.19145581405 + 78786.53066029989*$t);
   $X0 += 0.00000000022*cos(3.40212459403 + 6885.39370741431*$t);
   $X0 += 0.00000000025*cos(0.11282707964 + 130435.87818999648*$t);
   $X0 += 0.00000000022*cos(3.21338836945 + 50444.9277476993*$t);
   $X0 += 0.00000000028*cos(6.25386153205 + 6129.5408569901*$t);
   $X0 += 0.00000000025*cos(4.36482847694 + 28103.1286308555*$t);
   $X0 += 0.00000000023*cos(2.05346110066 + 103918.14464590348*$t);
   $X0 += 0.00000000025*cos(2.29706808754 + 52174.56562792359*$t);
   $X0 += 0.00000000022*cos(1.18783541749 + 2219.0009216703*$t);
   $X0 += 0.00000000022*cos(3.88895575058 + 78580.34511186268*$t);
   $X0 += 0.00000000029*cos(2.91258588959 + 1730.87853544909*$t);
   $X0 += 0.00000000022*cos(0.65836013072 + 80597.14963507809*$t);
   $X0 += 0.00000000024*cos(3.86546209537 + 51842.1941598625*$t);
   $X0 += 0.00000000020*cos(0.93295366770 + 51955.63745819309*$t);
   $X0 += 0.00000000028*cos(2.18732774825 + 536.5606946119*$t);
   $X0 += 0.00000000020*cos(3.94160971202 + 23439.20449861769*$t);
   $X0 += 0.00000000023*cos(5.40527491438 + 129387.49114216608*$t);
   $X0 += 0.00000000021*cos(5.54505025605 + 163766.33825852869*$t);
   $X0 += 0.00000000025*cos(2.01245845269 + 81604.56566890588*$t);
   $X0 += 0.00000000021*cos(3.01232876030 + 1063.55790093581*$t);
   $X0 += 0.00000000023*cos(4.96870238168 + 16342.8264032011*$t);
   $X0 += 0.00000000022*cos(2.15856505754 + 96357.32794175608*$t);
   $X0 += 0.00000000023*cos(1.14266322173 + 76144.70174686068*$t);
   $X0 += 0.00000000020*cos(1.29887924947 + 51329.9672658807*$t);
   $X0 += 0.00000000020*cos(1.71833485025 + 1089.95260867711*$t);
   $X0 += 0.00000000020*cos(0.66958086764 + 123758.53467195488*$t);
   $X0 += 0.00000000019*cos(0.72592280877 + 51535.66517935089*$t);
   $X0 += 0.00000000022*cos(3.99083726125 + 3462.0008883817*$t);
   $X0 += 0.00000000021*cos(2.05460557007 + 130290.19632821988*$t);
   $X0 += 0.00000000023*cos(3.93119144363 + 50167.49256147289*$t);
   $X0 += 0.00000000020*cos(6.09924257691 + 196137.31725009429*$t);
   $X0 += 0.00000000023*cos(2.69769554120 + 166740.94821313968*$t);
   $X0 += 0.00000000026*cos(6.04075681178 + 22747.04689739089*$t);
   $X0 += 0.00000000022*cos(0.05797120861 + 26126.2799946955*$t);
   $X0 += 0.00000000022*cos(5.71570675867 + 26735.1577923725*$t);
   $X0 += 0.00000000019*cos(4.66884897067 + 78683.43788608129*$t);
   $X0 += 0.00000000020*cos(2.25947325898 + 75930.26921436709*$t);
   $X0 += 0.00000000024*cos(2.12831639297 + 145204.99856862429*$t);
   $X0 += 0.00000000019*cos(3.69323819425 + 130446.87307235529*$t);
   $X0 += 0.00000000024*cos(3.04675754545 + 52177.53457334019*$t);
   $X0 += 0.00000000025*cos(4.82366657336 + 130443.64086071249*$t);
   $X0 += 0.00000000020*cos(2.64704504078 + 98690.03669966108*$t);
   $X0 += 0.00000000019*cos(3.52306398899 + 103498.66000202828*$t);
   $X0 += 0.00000000025*cos(2.89149847485 + 76888.17944236028*$t);
   $X0 += 0.00000000022*cos(0.83649888960 + 24952.4700871645*$t);
   $X0 += 0.00000000022*cos(4.82712614161 + 27223.8238309509*$t);
   $X0 += 0.00000000018*cos(3.15171375774 + 78413.51643934069*$t);
   $X0 += 0.00000000020*cos(0.30552664552 + 104819.8213741347*$t);
   $X0 += 0.00000000022*cos(3.38793394354 + 129379.88996019828*$t);
   $X0 += 0.00000000019*cos(6.23315092064 + 26248.5543284425*$t);
   $X0 += 0.00000000019*cos(5.71365941775 + 25927.7395896729*$t);
   $X0 += 0.00000000019*cos(1.78991551728 + 53029.2464823839*$t);
   $X0 += 0.00000000018*cos(4.18932552893 + 128221.00242116769*$t);
   $X0 += 0.00000000020*cos(5.93600662958 + 25908.0698287731*$t);
   $X0 += 0.00000000020*cos(6.01080370881 + 26268.2240893423*$t);
   $X0 += 0.00000000019*cos(4.93432108959 + 52817.46366237089*$t);
   $X0 += 0.00000000018*cos(1.87899794614 + 26411.6523757151*$t);
   $X0 += 0.00000000022*cos(4.75027274504 + 103932.37173990508*$t);
   $X0 += 0.00000000018*cos(4.74269631350 + 52195.23222656469*$t);
   $X0 += 0.00000000024*cos(3.42780024885 + 86144.04239554508*$t);
   $X0 += 0.00000000020*cos(0.38316404093 + 12412.6167069815*$t);
   $X0 += 0.00000000025*cos(3.36113877175 + 78313.94986411028*$t);
   $X0 += 0.00000000020*cos(5.18325339676 + 233731.99016146208*$t);
   $X0 += 0.00000000020*cos(0.60562813850 + 25234.9505773057*$t);
   $X0 += 0.00000000017*cos(2.29216844390 + 53868.21577013429*$t);
   $X0 += 0.00000000017*cos(4.19571093375 + 77307.86586563049*$t);
   $X0 += 0.00000000017*cos(0.77039059907 + 24602.8562523545*$t);
   $X0 += 0.00000000017*cos(4.89323443214 + 27573.4376657609*$t);
   $X0 += 0.00000000023*cos(1.65926065951 + 14.47091148511*$t);
   $X0 += 0.00000000018*cos(0.80516343452 + 52137.9170649941*$t);
   $X0 += 0.00000000020*cos(6.28264844209 + 130866.35771623049*$t);
   $X0 += 0.00000000016*cos(2.69293004860 + 79330.44871939608*$t);
   $X0 += 0.00000000018*cos(6.15817325038 + 79316.22162539448*$t);
   $X0 += 0.00000000016*cos(2.44285814230 + 3.6375178745*$t);
   $X0 += 0.00000000017*cos(5.95313986190 + 121335.36489626988*$t);
   $X0 += 0.00000000019*cos(0.00239064860 + 132028.83242063828*$t);
   $X0 += 0.00000000018*cos(3.07016978555 + 102659.69071427788*$t);
   $X0 += 0.00000000018*cos(2.48538976177 + 99023.89264043048*$t);
   $X0 += 0.00000000016*cos(1.30832465047 + 50903.3690750047*$t);
   $X0 += 0.00000000019*cos(0.09235363676 + 94138.08320260228*$t);
   $X0 += 0.00000000016*cos(6.10168410633 + 28310.00359365469*$t);
   $X0 += 0.00000000016*cos(5.36118674347 + 25668.17468021549*$t);
   $X0 += 0.00000000016*cos(0.06029661634 + 24395.9812895553*$t);
   $X0 += 0.00000000019*cos(5.79930577989 + 78793.15657233368*$t);
   $X0 += 0.00000000015*cos(2.51997196564 + 155887.76538061468*$t);
   $X0 += 0.00000000022*cos(6.08296146382 + 1055.6935944096*$t);
   $X0 += 0.00000000016*cos(2.88270090422 + 65717.47130312308*$t);
   $X0 += 0.00000000021*cos(5.12943933290 + 3178.38960805111*$t);
   $X0 += 0.00000000019*cos(4.86287967507 + 106570.12585300008*$t);
   $X0 += 0.00000000015*cos(3.64115942517 + 50690.06997956669*$t);
   $X0 += 0.00000000017*cos(3.32848323709 + 55638.2948064971*$t);
   $X0 += 0.00000000016*cos(0.98987078340 + 104358.4849426803*$t);
   $X0 += 0.00000000015*cos(4.27283735340 + 6043.9847638919*$t);
   $X0 += 0.00000000016*cos(0.28894936273 + 176332.48995851149*$t);
   $X0 += 0.00000000017*cos(2.00324514114 + 62197.88738073129*$t);
   $X0 += 0.00000000017*cos(4.71989777168 + 86457.74094044768*$t);
   $X0 += 0.00000000015*cos(3.42730813171 + 52310.1591502169*$t);
   $X0 += 0.00000000015*cos(0.50958115781 + 235900.75064374466*$t);
   $X0 += 0.00000000015*cos(4.72190436199 + 61560.8911087071*$t);
   $X0 += 0.00000000017*cos(3.12766472647 + 103821.67778371868*$t);
   $X0 += 0.00000000017*cos(4.57987044915 + 153085.08772196328*$t);
   $X0 += 0.00000000020*cos(4.57785189845 + 3.6883357796*$t);
   $X0 += 0.00000000014*cos(5.68581925640 + 193938.22990680728*$t);
   $X0 += 0.00000000015*cos(2.61658035144 + 50007.2896183493*$t);
   $X0 += 0.00000000015*cos(0.06488742456 + 26831.1370195903*$t);
   $X0 += 0.00000000014*cos(0.38204630080 + 133882.3344703199*$t);
   $X0 += 0.00000000017*cos(3.50167044497 + 39629.08052658189*$t);
   $X0 += 0.00000000016*cos(1.91399253711 + 77.9943614674*$t);
   $X0 += 0.00000000015*cos(4.72920790161 + 125112.2834169717*$t);
   $X0 += 0.00000000014*cos(2.29306969321 + 50380.79165691089*$t);
   $X0 += 0.00000000016*cos(4.13091799370 + 104345.22782488028*$t);
   $X0 += 0.00000000014*cos(1.95123243708 + 26247.4486938499*$t);
   $X0 += 0.00000000014*cos(3.71239259413 + 25928.8452242655*$t);
   $X0 += 0.00000000014*cos(0.78821341455 + 16066.3096789583*$t);
   $X0 += 0.00000000016*cos(3.86619726000 + 50910.4826220055*$t);
   $X0 += 0.00000000018*cos(2.12623271525 + 92740.81679043909*$t);
   $X0 += 0.00000000014*cos(5.78782811767 + 1169.83206889211*$t);
   $X0 += 0.00000000016*cos(3.71525448527 + 53815.1196178199*$t);
   $X0 += 0.00000000014*cos(4.45337551699 + 78149.51395352086*$t);
   $X0 += 0.00000000016*cos(4.39061350798 + 203042.13690092487*$t);
   $X0 += 0.00000000013*cos(0.37425161178 + 639.6534688305*$t);
   $X0 += 0.00000000015*cos(6.07909581296 + 77616.94240889128*$t);
   $X0 += 0.00000000014*cos(4.68939663529 + 82815.41910429788*$t);
   $X0 += 0.00000000014*cos(4.74773700930 + 29415.79498037089*$t);
   $X0 += 0.00000000014*cos(5.20331959316 + 102755.66994149568*$t);
   $X0 += 0.00000000014*cos(1.72247029033 + 109.9625037359*$t);
   $X0 += 0.00000000017*cos(0.16324703337 + 163298.37326817428*$t);
   $X0 += 0.00000000015*cos(1.68035003786 + 11.2895177474*$t);
   $X0 += 0.00000000014*cos(5.16225967454 + 173511.65881438529*$t);
   $X0 += 0.00000000014*cos(4.39150540314 + 52161.82300663029*$t);
   $X0 += 0.00000000012*cos(2.27730262051 + 102018.17235594128*$t);
   $X0 += 0.00000000017*cos(2.64158525193 + 52155.8927047651*$t);
   $X0 += 0.00000000013*cos(2.88005600662 + 102769.89703549728*$t);
   $X0 += 0.00000000014*cos(0.84211332161 + 78378.3925308913*$t);
   $X0 += 0.00000000014*cos(3.21771971652 + 26077.1012587938*$t);
   $X0 += 0.00000000014*cos(2.44590531469 + 26099.1926593216*$t);
   $X0 += 0.00000000013*cos(3.84393935941 + 79322.84753742829*$t);
   $X0 += 0.00000000016*cos(0.56654525193 + 51534.63653889291*$t);
   $X0 += 0.00000000014*cos(4.45047217505 + 20043.9183776823*$t);
   $X0 += 0.00000000016*cos(4.85429966913 + 50050.17269325269*$t);
   $X0 += 0.00000000013*cos(3.70788935281 + 44936.88687406489*$t);
   $X0 += 0.00000000014*cos(6.05588092072 + 24506.18761077469*$t);
   $X0 += 0.00000000014*cos(0.68454442549 + 35833.4675149143*$t);
   $X0 += 0.00000000012*cos(0.96763097373 + 52388.86156110289*$t);
   $X0 += 0.00000000014*cos(1.93515089311 + 198490.18340386607*$t);
   $X0 += 0.00000000013*cos(0.54258695046 + 103719.07264446709*$t);
   $X0 += 0.00000000013*cos(2.24721025792 + 24491.96051677309*$t);
   $X0 += 0.00000000013*cos(2.99089102997 + 52025.99926853029*$t);
   $X0 += 0.00000000012*cos(1.26244334627 + 25984.3647704771*$t);
   $X0 += 0.00000000012*cos(4.40118168494 + 26191.9291476383*$t);
   $X0 += 0.00000000014*cos(4.74013545104 + 53265.7588918255*$t);
   $X0 += 0.00000000016*cos(2.69508248660 + 25653.94758621389*$t);
   $X0 += 0.00000000012*cos(5.94982180077 + 181659.96606689428*$t);
   $X0 += 0.00000000014*cos(3.98127038415 + 53399.86794141051*$t);
   $X0 += 0.00000000012*cos(0.13992189685 + 53772.23654291649*$t);
   $X0 += 0.00000000012*cos(1.72523320675 + 43981.7747674233*$t);
   $X0 += 0.00000000012*cos(5.92436505942 + 28199.79727243529*$t);
   $X0 += 0.00000000015*cos(4.44881823724 + 1638.82569970449*$t);
   $X0 += 0.00000000016*cos(6.01974460542 + 34082.6754010819*$t);
   $X0 += 0.00000000012*cos(4.59110069577 + 51596.3625136331*$t);
   $X0 += 0.00000000013*cos(0.25880243151 + 52025.00243078899*$t);
   $X0 += 0.00000000011*cos(4.64769916430 + 52911.92661416369*$t);
   $X0 += 0.00000000011*cos(1.89874077048 + 70383.86408886709*$t);
   $X0 += 0.00000000012*cos(1.02764449556 + 112231.45790215228*$t);
   $X0 += 0.00000000015*cos(4.79472245666 + 103883.89139342589*$t);
   $X0 += 0.00000000012*cos(1.34514817706 + 52171.68113030689*$t);
   $X0 += 0.00000000014*cos(1.43559106475 + 26555.6243144451*$t);
   $X0 += 0.00000000013*cos(1.61506412907 + 286966.69073983253*$t);
   $X0 += 0.00000000013*cos(5.01690772702 + 208173.77798498247*$t);
   $X0 += 0.00000000013*cos(5.62315849993 + 79219.06534884768*$t);
   $X0 += 0.00000000011*cos(1.70747594337 + 52168.44891866409*$t);
   $X0 += 0.00000000011*cos(5.83137810614 + 26012.87772552339*$t);
   $X0 += 0.00000000011*cos(2.17980505276 + 48713.8053947667*$t);
   $X0 += 0.00000000014*cos(0.87464188435 + 27998.85880730789*$t);
   $X0 += 0.00000000014*cos(1.00856446032 + 52755.7376876307*$t);
   $X0 += 0.00000000014*cos(2.05272684647 + 52041.9410510469*$t);
   $X0 += 0.00000000012*cos(5.20664534748 + 131548.65044248088*$t);
   $X0 += 0.00000000012*cos(2.40230884933 + 130363.49345407888*$t);
   $X0 += 0.00000000011*cos(5.14348161985 + 104277.07478521299*$t);
   $X0 += 0.00000000014*cos(4.30522472177 + 27250.62166346549*$t);
   $X0 += 0.00000000011*cos(3.57984794643 + 51109.06698847489*$t);
   $X0 += 0.00000000011*cos(0.50291413498 + 115674.03284711768*$t);
   $X0 += 0.00000000014*cos(2.57900377541 + 49527.10764019189*$t);
   $X0 += 0.00000000013*cos(1.25530162700 + 52027.97137620559*$t);
   $X0 += 0.00000000011*cos(3.18085491379 + 57503.5262090147*$t);
   $X0 += 0.00000000013*cos(3.20863649406 + 78470.13879064328*$t);
   $X0 += 0.00000000014*cos(1.85519169440 + 860.55374623631*$t);
   $X0 += 0.00000000011*cos(4.15960190462 + 92741.3044254061*$t);
   $X0 += 0.00000000013*cos(5.32460666801 + 126996.69694542208*$t);
   $X0 += 0.00000000011*cos(1.25079104877 + 20894.7799002599*$t);
   $X0 += 0.00000000012*cos(0.71489552500 + 51440.17358710009*$t);
   $X0 += 0.00000000015*cos(0.61708921425 + 24395.49365458829*$t);
   $X0 += 0.00000000012*cos(4.22401388053 + 52179.98225389499*$t);
   $X0 += 0.00000000013*cos(3.48954455220 + 65716.98366815608*$t);
   $X0 += 0.00000000014*cos(4.39584924296 + 3442.3311274819*$t);
   $X0 += 0.00000000011*cos(5.05191127458 + 736.1203310153*$t);
   $X0 += 0.00000000010*cos(2.32271342326 + 333.6121232859*$t);
   $X0 += 0.00000000011*cos(1.70456192528 + 50536.98058344389*$t);
   $X0 += 0.00000000010*cos(2.59099487043 + 26190.75209830929*$t);
   $X0 += 0.00000000012*cos(2.51295739282 + 53758.00944891489*$t);
   $X0 += 0.00000000011*cos(5.25664726449 + 26081.03076519069*$t);
   $X0 += 0.00000000010*cos(2.79700804589 + 78340.21931348169*$t);
   $X0 += 0.00000000011*cos(4.88405989087 + 78800.75775430149*$t);
   $X0 += 0.00000000012*cos(6.04572661714 + 52179.4438010229*$t);
   $X0 += 0.00000000010*cos(4.25864792126 + 84945.17809560569*$t);
   $X0 += 0.00000000011*cos(0.86477927749 + 81592.08889848628*$t);
   $X0 += 0.00000000011*cos(5.65005720167 + 641.16974425549*$t);
   $X0 += 0.00000000014*cos(1.53374794610 + 183145.25677359749*$t);
   $X0 += 0.00000000010*cos(6.17476996345 + 25863.8021633558*$t);
   $X0 += 0.00000000010*cos(3.55930973128 + 53093.98038661369*$t);
   $X0 += 0.00000000011*cos(0.09944117691 + 51123.29408247649*$t);
   $X0 += 0.00000000014*cos(0.87408194721 + 25242.0641243065*$t);
   $X0 += 0.00000000010*cos(1.40003580429 + 154408.89880655028*$t);
   $X0 += 0.00000000014*cos(2.93953436186 + 54344.81058291449*$t);
   $X0 += 0.00000000011*cos(2.53121281826 + 78057.76769376888*$t);
   $X0 += 0.00000000012*cos(1.32462443773 + 26149.87293379789*$t);
   $X0 += 0.00000000012*cos(4.33900059348 + 26026.4209843175*$t);
   $X0 += 0.00000000010*cos(0.70338242506 + 637.24008950771*$t);
   $X0 += 0.00000000010*cos(2.36288136655 + 91919.81373338249*$t);
   $X0 += 0.00000000014*cos(3.26945543294 + 53125.22570960169*$t);
   $X0 += 0.00000000011*cos(2.11182562121 + 26094.2878829907*$t);
   $X0 += 0.00000000012*cos(1.35040938945 + 136100.60393953968*$t);
   $X0 += 0.00000000010*cos(4.17716450281 + 6751.28465782931*$t);
   $X0 += 0.00000000010*cos(0.88468292764 + 137209.98249163308*$t);
   $X0 += 0.00000000010*cos(3.56286217757 + 77940.44782554869*$t);
   $X0 += 0.00000000010*cos(2.79243370962 + 45404.8518644193*$t);
   $X0 += 0.00000000013*cos(5.65469308747 + 50270.5853356915*$t);
   $X0 += 0.00000000011*cos(5.25078615822 + 77520.96318167349*$t);
   $X0 += 0.00000000011*cos(5.80424854653 + 19958.8499195511*$t);
   $X0 += 0.00000000010*cos(6.13036479176 + 25619.69433373629*$t);
   $X0 += 0.00000000011*cos(5.27912616615 + 39763.6772111339*$t);
   $X0 += 0.00000000010*cos(1.03100691224 + 54086.76194888209*$t);
   $X0 += 0.00000000009*cos(2.58296270636 + 130593.53833583929*$t);
   $X0 += 0.00000000009*cos(1.93570258198 + 26514.0557355717*$t);
   $X0 += 0.00000000009*cos(3.72792244923 + 25662.2381825437*$t);
   $X0 += 0.00000000010*cos(0.87681259763 + 101703.40155830729*$t);
   $X0 += 0.00000000013*cos(4.75978960525 + 26161.44408491669*$t);
   $X0 += 0.00000000013*cos(0.90383542596 + 26014.8498331987*$t);
   $X0 += 0.00000000010*cos(2.43967897596 + 104991.75367009429*$t);
   $X0 += 0.00000000010*cos(1.24342292300 + 52061.65099371849*$t);
   $X0 += 0.00000000009*cos(1.43295817370 + 25131.85780308709*$t);
   $X0 += 0.00000000011*cos(5.57723740117 + 183674.94773869208*$t);
   $X0 += 0.00000000010*cos(0.49259119482 + 28791.7631137333*$t);
   $X0 += 0.00000000010*cos(5.98998894020 + 4372.1201842577*$t);
   $X0 += 0.00000000010*cos(3.84228475284 + 158746.41977111546*$t);
   $X0 += 0.00000000009*cos(2.25433851888 + 45290.90021070109*$t);
   $X0 += 0.00000000010*cos(5.16716938460 + 23384.5308043821*$t);
   $X0 += 0.00000000012*cos(5.74392096253 + 76042.09660760908*$t);
   $X0 += 0.00000000012*cos(3.47156983874 + 52396.4627430707*$t);
   $X0 += 0.00000000013*cos(5.95557437514 + 1073.85284167431*$t);
   $X0 += 0.00000000010*cos(6.00837282954 + 103924.77055793729*$t);
   $X0 += 0.00000000010*cos(3.46837882181 + 91804.88680973028*$t);
   $X0 += 0.00000000009*cos(3.96963705288 + 103189.38167937248*$t);
   $X0 += 0.00000000009*cos(0.21796861386 + 156547.33242782846*$t);
   $X0 += 0.00000000009*cos(5.23543221261 + 735.6326960483*$t);
   $X0 += 0.00000000009*cos(1.95816420727 + 45494.33761226529*$t);
   $X0 += 0.00000000009*cos(1.00178607588 + 114.68310616872*$t);
   $X0 += 0.00000000009*cos(2.34791148855 + 130285.98071486969*$t);
   $X0 += 0.00000000013*cos(3.54674823405 + 2168.5166647991*$t);
   $X0 += 0.00000000010*cos(1.19765211170 + 32132.3755404331*$t);
   $X0 += 0.00000000009*cos(3.61118357787 + 104984.64012309349*$t);
   $X0 += 0.00000000011*cos(3.38074365009 + 19367.4329797163*$t);
   $X0 += 0.00000000010*cos(2.44976316483 + 90989.40667698109*$t);
   $X0 += 0.00000000009*cos(3.11869996984 + 25550.85481199529*$t);
   $X0 += 0.00000000009*cos(0.74322450432 + 76255.39570304709*$t);
   $X0 += 0.00000000009*cos(4.41624596522 + 31281.5140178555*$t);
   $X0 += 0.00000000012*cos(5.37072201396 + 1699.5230339867*$t);
   $X0 += 0.00000000011*cos(3.93547727008 + 220.16882495529*$t);
   $X0 += 0.00000000009*cos(2.41712554824 + 51130.89526444429*$t);
   $X0 += 0.00000000011*cos(1.25172069529 + 4022.50634944771*$t);
   $X0 += 0.00000000009*cos(1.96969875107 + 52797.7939014711*$t);
   $X0 += 0.00000000009*cos(5.38585404056 + 24844.6593573793*$t);
   $X0 += 0.00000000009*cos(0.27777099065 + 27331.6345607361*$t);
   $X0 += 0.00000000010*cos(1.25844371970 + 230239.41859459245*$t);
   $X0 += 0.00000000009*cos(4.03973091884 + 156314.36357149068*$t);
   $X0 += 0.00000000011*cos(3.17025202968 + 80175.15272542329*$t);
   $X0 += 0.00000000009*cos(2.05859040472 + 129329.89333829407*$t);
   $X0 += 0.00000000008*cos(2.65734511175 + 38634.6288981407*$t);
   $X0 += 0.00000000011*cos(5.69260296788 + 46046.7530611253*$t);
   $X0 += 0.00000000009*cos(0.74288994112 + 52190.2771946335*$t);
   $X0 += 0.00000000009*cos(1.39536424747 + 25454.87558477749*$t);
   $X0 += 0.00000000009*cos(4.15458502385 + 23439.6921335847*$t);
   $X0 += 0.00000000008*cos(5.65796860157 + 102762.29585352947*$t);
   $X0 += 0.00000000011*cos(2.58616852636 + 2538.4923217371*$t);
   $X0 += 0.00000000010*cos(3.10787144924 + 391318.79094109643*$t);
   $X0 += 0.00000000008*cos(1.00986052839 + 80174.66509045628*$t);
   $X0 += 0.00000000008*cos(5.80399155144 + 142871.80217575228*$t);
   $X0 += 0.00000000009*cos(5.83837700511 + 76152.30292882849*$t);
   $X0 += 0.00000000010*cos(0.76586105008 + 54060.95126404929*$t);
   $X0 += 0.00000000009*cos(4.80200590567 + 4.1759707466*$t);
   $X0 += 0.00000000010*cos(4.30161237035 + 35472.50033216589*$t);
   $X0 += 0.00000000010*cos(5.71617817105 + 125887.31839332868*$t);
   $X0 += 0.00000000008*cos(1.10337825429 + 27360.8279846849*$t);
   $X0 += 0.00000000009*cos(2.54194822126 + 1685.2959399851*$t);
   $X0 += 0.00000000009*cos(5.51043601697 + 58458.63831565629*$t);
   $X0 += 0.00000000009*cos(5.26195704910 + 25977.45300287129*$t);
   $X0 += 0.00000000009*cos(2.40795090884 + 1485.0468892197*$t);
   $X0 += 0.00000000008*cos(2.74586842231 + 2325.1932262991*$t);
   $X0 += 0.00000000009*cos(3.83628763541 + 22595.5753921881*$t);
   $X0 += 0.00000000009*cos(0.89908538700 + 52172.11794736879*$t);
   $X0 += 0.00000000008*cos(5.40598309652 + 26934.2297938089*$t);
   $X0 += 0.00000000008*cos(2.16624725476 + 1575.08961876571*$t);
   $X0 += 0.00000000008*cos(0.66150337713 + 51876.9237063627*$t);
   $X0 += 0.00000000010*cos(3.52742187765 + 2751.79141717511*$t);
   $X0 += 0.00000000008*cos(2.46382310966 + 10681.49435404891*$t);
   $X0 += 0.00000000011*cos(4.78783764732 + 28096.7044982167*$t);
   $X0 += 0.00000000008*cos(0.74099606624 + 77956.3956212365*$t);
   $X0 += 0.00000000008*cos(2.46215429910 + 52400.3948963338*$t);
   $X0 += 0.00000000008*cos(4.46259452267 + 54824.5049261049*$t);
   $X0 += 0.00000000010*cos(3.07618265192 + 240452.70414080346*$t);
   $X0 += 0.00000000008*cos(0.55741619385 + 77741.37582411229*$t);
   $X0 += 0.00000000009*cos(2.33738686148 + 24491.47288180609*$t);
   $X0 += 0.00000000008*cos(0.42516315180 + 42791.2090925567*$t);
   $X0 += 0.00000000008*cos(4.70625369395 + 52490.23628050151*$t);
   $X0 += 0.00000000008*cos(3.86744165099 + 156520.54911992788*$t);
   $X0 += 0.00000000007*cos(6.06395790305 + 52206.3769616363*$t);
   $X0 += 0.00000000008*cos(2.15830939123 + 17098.67925362531*$t);
   $X0 += 0.00000000007*cos(4.72162069749 + 50291.1489372145*$t);
   $X0 += 0.00000000010*cos(4.49723907752 + 22065.8844270935*$t);
   $X0 += 0.00000000008*cos(2.65028306976 + 51112.73601717959*$t);
   $X0 += 0.00000000009*cos(2.14235988572 + 104454.94915799888*$t);
   $X0 += 0.00000000010*cos(3.27688619254 + 441.06910236111*$t);
   $X0 += 0.00000000010*cos(1.07651496998 + 49.7528044207*$t);
   $X0 += 0.00000000007*cos(3.67002518147 + 7880.3329708225*$t);
   $X0 += 0.00000000007*cos(4.65878049481 + 52182.67601266569*$t);
   $X0 += 0.00000000008*cos(3.31050910615 + 52099.29639438929*$t);
   $X0 += 0.00000000007*cos(5.29571821879 + 122445.23108333028*$t);
   $X0 += 0.00000000008*cos(3.30784894855 + 66940.80150893469*$t);
   $X0 += 0.00000000008*cos(0.11825864367 + 187167.51930556168*$t);
   $X0 += 0.00000000009*cos(4.39445007595 + 156507.9929060289*$t);
   $X0 += 0.00000000007*cos(0.74222476670 + 51086.3413094383*$t);
   $X0 += 0.00000000008*cos(0.44253932824 + 430.7741616226*$t);
   $X0 += 0.00000000008*cos(5.53617232430 + 149288.98707532868*$t);
   $X0 += 0.00000000008*cos(5.41205762148 + 49956.8053614781*$t);
   $X0 += 0.00000000007*cos(0.95302460841 + 53132.33925660249*$t);
   $X0 += 0.00000000008*cos(2.62377772169 + 100908.79380384788*$t);
   $X0 += 0.00000000007*cos(3.70441477670 + 233681.99353955788*$t);
   $X0 += 0.00000000007*cos(2.34002720858 + 104248.76360956169*$t);
   $X0 += 0.00000000008*cos(5.03600399329 + 77314.77763323628*$t);
   $X0 += 0.00000000008*cos(0.25407509038 + 112231.94553711927*$t);
   $X0 += 0.00000000007*cos(1.59729628741 + 29396.6128544381*$t);
   $X0 += 0.00000000007*cos(4.06632874380 + 22779.6810636773*$t);
   $X0 += 0.00000000009*cos(3.00366222190 + 64741.71416564769*$t);
   $X0 += 0.00000000007*cos(5.95614295602 + 26290.4003542318*$t);
   $X0 += 0.00000000007*cos(5.99066738237 + 25885.89356388359*$t);
   $X0 += 0.00000000009*cos(3.58307674710 + 25863.31452838879*$t);
   $X0 += 0.00000000007*cos(3.41090615216 + 13655.6166736929*$t);
   $X0 += 0.00000000007*cos(3.03792135842 + 156740.96176236669*$t);
   $X0 += 0.00000000008*cos(4.55619991306 + 78571.51086317569*$t);
   $X0 += 0.00000000007*cos(3.14378763001 + 52663.6848518861*$t);
   $X0 += 0.00000000007*cos(1.56777604890 + 2118.5200428949*$t);
   $X0 += 0.00000000008*cos(4.91321996165 + 102872.98980971589*$t);
   $X0 += 0.00000000010*cos(3.40349349020 + 2703.3723371921*$t);
   $X0 += 0.00000000007*cos(0.43580348099 + 52074.08758236629*$t);
   $X0 += 0.00000000007*cos(2.04943751114 + 209659.06869168568*$t);
   $X0 += 0.00000000007*cos(3.67671851452 + 149846.43781352908*$t);
   $X0 += 0.00000000007*cos(4.30875978873 + 77417.87040745489*$t);
   $X0 += 0.00000000009*cos(1.15423490298 + 56260.03860733629*$t);
   $X0 += 0.00000000007*cos(3.50070425163 + 117892.78995130445*$t);
   $X0 += 0.00000000009*cos(2.88385440366 + 18073.9487561337*$t);
   $X0 += 0.00000000007*cos(4.35051921098 + 917.6864684983*$t);
   $X0 += 0.00000000008*cos(2.28326188901 + 32808.8609383991*$t);
   $X0 += 0.00000000008*cos(1.76307802021 + 207114.39605479328*$t);
   $X0 += 0.00000000008*cos(5.32581127623 + 3308.22207789689*$t);
   $X0 += 0.00000000007*cos(2.91658615206 + 23267.3158149315*$t);
   $X0 += 0.00000000007*cos(2.74703887915 + 28908.9781031839*$t);
   $X0 += 0.00000000007*cos(3.11128439659 + 37.88921815429*$t);
   $X0 += 0.00000000007*cos(6.13048599151 + 19202.50943415989*$t);
   $X0 += 0.00000000007*cos(5.70115634056 + 52145.72323962749*$t);
   $X0 += 0.00000000008*cos(4.37424371597 + 103395.56722780968*$t);
   $X0 += 0.00000000007*cos(1.57841023127 + 208276.87075920108*$t);
   $X0 += 0.00000000009*cos(1.81069664930 + 2014.73785431429*$t);
   $X0 += 0.00000000007*cos(1.08271855123 + 63786.6020590061*$t);
   $X0 += 0.00000000007*cos(2.83741634124 + 24712.3731592119*$t);
   $X0 += 0.00000000007*cos(2.64521183634 + 127099.14709865468*$t);
   $X0 += 0.00000000008*cos(0.10378895366 + 120417.92224525509*$t);
   $X0 += 0.00000000008*cos(2.96158619363 + 103815.05187168489*$t);
   $X0 += 0.00000000007*cos(0.99819510291 + 8989.22388794889*$t);
   $X0 += 0.00000000008*cos(1.62417504708 + 579.4437695153*$t);
   $X0 += 0.00000000007*cos(2.35955703224 + 1135.4330544097*$t);
   $X0 += 0.00000000006*cos(5.07797134858 + 27574.1270801229*$t);
   $X0 += 0.00000000007*cos(5.91237630109 + 102976.08258393449*$t);
   $X0 += 0.00000000006*cos(3.83307059240 + 28774.8690535989*$t);
   $X0 += 0.00000000006*cos(1.83055443881 + 23401.4248645165*$t);
   $X0 += 0.00000000006*cos(1.38927552842 + 80463.04058549309*$t);
   $X0 += 0.00000000008*cos(1.96535036231 + 51258.1198146501*$t);
   $X0 += 0.00000000006*cos(3.61870647781 + 23704.95394428149*$t);
   $X0 += 0.00000000006*cos(2.04491855340 + 28471.3399738339*$t);
   $X0 += 0.00000000006*cos(1.68248146387 + 55618.1374106303*$t);
   $X0 += 0.00000000007*cos(1.61837202878 + 53311.7269725251*$t);
   $X0 += 0.00000000007*cos(5.53141035478 + 1802.6158082053*$t);
   $X0 += 0.00000000008*cos(4.68334848992 + 102018.65999090829*$t);
   $X0 += 0.00000000007*cos(5.32851714437 + 79995.0755951387*$t);
   $X0 += 0.00000000007*cos(5.04612545544 + 52278.0126188975*$t);
   $X0 += 0.00000000008*cos(4.88698976103 + 1162.23088692429*$t);
   $X0 += 0.00000000006*cos(6.06855779639 + 53439.2074632101*$t);
   $X0 += 0.00000000007*cos(1.80548735062 + 29580.7185259273*$t);
   $X0 += 0.00000000007*cos(3.67914527744 + 27388.97377726269*$t);
   $X0 += 0.00000000007*cos(1.98447975377 + 24787.3201408527*$t);
   $X0 += 0.00000000006*cos(3.77150246945 + 106470.62024415868*$t);
   $X0 += 0.00000000007*cos(2.88869753965 + 104127.51158807839*$t);
   $X0 += 0.00000000007*cos(0.34091991598 + 27624.1237020271*$t);
   $X0 += 0.00000000007*cos(5.32270511523 + 24552.1702160883*$t);
   $X0 += 0.00000000008*cos(3.83070609952 + 948.9317914863*$t);
   $X0 += 0.00000000007*cos(5.94865065189 + 6680.98103591609*$t);
   $X0 += 0.00000000008*cos(0.77567524248 + 51861.86392076229*$t);
   $X0 += 0.00000000008*cos(3.58552572007 + 51554.3062997927*$t);
   $X0 += 0.00000000007*cos(4.23028089495 + 26720.44306340389*$t);
   $X0 += 0.00000000007*cos(2.84651949085 + 52214.1831362697*$t);
   $X0 += 0.00000000007*cos(5.58797516267 + 25572.68308796469*$t);
   $X0 += 0.00000000007*cos(0.07564986854 + 26603.6108301507*$t);
   $X0 += 0.00000000007*cos(5.38283352699 + 415.7963080956*$t);
   $X0 += 0.00000000008*cos(1.12257158980 + 54191.0317724297*$t);
   $X0 += 0.00000000006*cos(0.30285518092 + 24505.69997580769*$t);
   $X0 += 0.00000000007*cos(4.29992052709 + 2974.85377209451*$t);
   $X0 += 0.00000000007*cos(4.30362880096 + 48092.0615939275*$t);
   $X0 += 0.00000000006*cos(3.84286985256 + 78043.54059976728*$t);
   $X0 += 0.00000000006*cos(2.33701938638 + 19202.99706912689*$t);
   $X0 += 0.00000000006*cos(1.70009289223 + 53661.3408073351*$t);
   $X0 += 0.00000000006*cos(0.21405293391 + 104426.63798234759*$t);
   $X0 += 0.00000000006*cos(0.26081525757 + 27883.4054027787*$t);
   $X0 += 0.00000000006*cos(3.09307209285 + 156523.78133157069*$t);
   $X0 += 0.00000000008*cos(5.86283230643 + 51120.60032370579*$t);
   $X0 += 0.00000000006*cos(1.52305102135 + 226.07308589371*$t);
   $X0 += 0.00000000008*cos(0.51879015762 + 48997.9043100643*$t);
   $X0 += 0.00000000006*cos(4.68111037881 + 149756.95206568309*$t);
   $X0 += 0.00000000007*cos(1.82650871102 + 4083.74468922089*$t);
   $X0 += 0.00000000008*cos(3.80478888311 + 53259.1303329255*$t);
   $X0 += 0.00000000006*cos(5.07036834716 + 130006.04778747768*$t);
   $X0 += 0.00000000007*cos(1.36856874001 + 162810.73851692009*$t);
   $X0 += 0.00000000006*cos(2.21234472657 + 27441.8957040745*$t);
   $X0 += 0.00000000006*cos(3.45128030464 + 24734.3982140409*$t);
   $X0 += 0.00000000007*cos(4.80045013467 + 55503.69812194508*$t);
   $X0 += 0.00000000008*cos(3.49135266112 + 422.9098550964*$t);
   $X0 += 0.00000000006*cos(0.40283917391 + 76777.97312114088*$t);
   $X0 += 0.00000000006*cos(5.52356103113 + 155461.16718973868*$t);
   $X0 += 0.00000000006*cos(4.38638116269 + 50161.0684288341*$t);
   $X0 += 0.00000000006*cos(5.35041562795 + 78262.46876949779*$t);
   $X0 += 0.00000000006*cos(3.67530335628 + 223.1041404771*$t);
   $X0 += 0.00000000006*cos(3.76321444127 + 78225.82020656829*$t);
   $X0 += 0.00000000006*cos(5.36628440371 + 76468.69479848508*$t);
   $X0 += 0.00000000006*cos(4.67707783228 + 78597.8091829755*$t);
   $X0 += 0.00000000007*cos(1.16259543363 + 30110.4094910219*$t);
   $X0 += 0.00000000006*cos(3.49467754329 + 35077.61466449009*$t);
   $X0 += 0.00000000007*cos(1.41920132868 + 1884.65734593389*$t);
   $X0 += 0.00000000007*cos(4.54853659672 + 1082.8364148101*$t);
   $X0 += 0.00000000006*cos(1.96577053321 + 5193.12324131429*$t);
   $X0 += 0.00000000006*cos(5.17864942773 + 171292.90171019848*$t);
   $X0 += 0.00000000006*cos(2.86805602516 + 25984.56654987209*$t);
   $X0 += 0.00000000006*cos(4.20102757189 + 29864.5778447925*$t);
   $X0 += 0.00000000006*cos(1.46259745932 + 22311.7160733229*$t);
   $X0 += 0.00000000006*cos(4.38736983699 + 76991.27221657889*$t);
   $X0 += 0.00000000005*cos(0.68949263605 + 27410.8246065891*$t);
   $X0 += 0.00000000005*cos(4.97413239516 + 24765.4693115263*$t);
   $X0 += 0.00000000006*cos(1.69366740663 + 51092.9698683383*$t);
   $X0 += 0.00000000007*cos(3.84496680361 + 88476.75115345008*$t);
   $X0 += 0.00000000007*cos(2.78025711521 + 34102.3451619817*$t);
   $X0 += 0.00000000005*cos(1.26084797472 + 2686.4782770577*$t);
   $X0 += 0.00000000006*cos(5.55737889155 + 31903.2578186947*$t);
   $X0 += 0.00000000006*cos(0.10624613966 + 20273.0360994207*$t);
   $X0 += 0.00000000006*cos(2.86083846875 + 104874.43380187408*$t);
   $X0 += 0.00000000005*cos(2.16809698297 + 155475.39428374029*$t);
   $X0 += 0.00000000005*cos(6.14186631974 + 128747.59385585209*$t);
   $X0 += 0.00000000005*cos(5.05100044910 + 26500.51805593209*$t);
   $X0 += 0.00000000005*cos(0.61262458211 + 25675.7758621833*$t);
   $X0 += 0.00000000007*cos(4.13768447366 + 25234.46294233869*$t);
   $X0 += 0.00000000005*cos(5.69197138572 + 53551.8239004777*$t);
   $X0 += 0.00000000005*cos(0.34616736613 + 60170.5785426561*$t);
   $X0 += 0.00000000005*cos(6.05655614068 + 27353.71443768409*$t);
   $X0 += 0.00000000007*cos(1.18001608430 + 52602.16065654089*$t);
   $X0 += 0.00000000005*cos(2.35031471718 + 128106.07549751546*$t);
   $X0 += 0.00000000005*cos(2.02403333135 + 629.0954035336*$t);
   $X0 += 0.00000000005*cos(1.31146516422 + 35191.56631820829*$t);
   $X0 += 0.00000000006*cos(2.18238318065 + 3492.8153843531*$t);
   $X0 += 0.00000000007*cos(4.81830760175 + 169093.81436691148*$t);
   $X0 += 0.00000000006*cos(4.71794828628 + 81591.60126351927*$t);
   $X0 += 0.00000000006*cos(4.84978213740 + 25462.4767667453*$t);
   $X0 += 0.00000000006*cos(0.81384289381 + 26713.81715137009*$t);
   $X0 += 0.00000000007*cos(2.97261820220 + 2641.5850959557*$t);
   $X0 += 0.00000000005*cos(3.79651203289 + 58220.2786820073*$t);
   $X0 += 0.00000000005*cos(2.70097411082 + 52813.0463726561*$t);
   $X0 += 0.00000000006*cos(1.52077024889 + 156531.54400228668*$t);
   $X0 += 0.00000000005*cos(1.19912051466 + 26236.225683484*$t);
   $X0 += 0.00000000007*cos(4.44424756295 + 9745.07673837309*$t);
   $X0 += 0.00000000005*cos(4.73755202618 + 26035.0508067433*$t);
   $X0 += 0.00000000005*cos(0.92607300503 + 26141.24311137209*$t);
   $X0 += 0.00000000006*cos(1.08638725845 + 117873.12019040468*$t);
   $X0 += 0.00000000006*cos(3.87149168929 + 28214.02436643689*$t);
   $X0 += 0.00000000005*cos(3.94775717381 + 487.39093377069*$t);
   $X0 += 0.00000000006*cos(1.27113821429 + 53764.63536094869*$t);
   $X0 += 0.00000000005*cos(4.36934946235 + 104035.46451412368*$t);
   $X0 += 0.00000000006*cos(4.83390346226 + 437.8877086234*$t);
   $X0 += 0.00000000005*cos(3.74810256898 + 52098.29955664799*$t);
   $X0 += 0.00000000006*cos(1.53446440230 + 98068.29289882188*$t);
   $X0 += 0.00000000005*cos(2.74138703040 + 65538.49980743109*$t);
   $X0 += 0.00000000005*cos(2.17866438309 + 189854.24140010288*$t);
   $X0 += 0.00000000005*cos(1.65056263900 + 76138.07583482689*$t);
   $X0 += 0.00000000005*cos(4.88320109198 + 104301.85976187608*$t);
   $X0 += 0.00000000005*cos(0.46719543123 + 1243.73141916191*$t);
   $X0 += 0.00000000005*cos(1.20837858717 + 118829.20756698029*$t);
   $X0 += 0.00000000006*cos(2.74486813288 + 13675.2864345927*$t);
   $X0 += 0.00000000005*cos(2.13216570647 + 153084.60008699627*$t);
   $X0 += 0.00000000006*cos(1.50606009977 + 13.9832765181*$t);
   $X0 += 0.00000000005*cos(0.04883294046 + 78697.66498008289*$t);
   $X0 += 0.00000000005*cos(3.63277479562 + 26610.23674218449*$t);
   $X0 += 0.00000000005*cos(5.00629124334 + 1052.02456570489*$t);
   $X0 += 0.00000000005*cos(3.70422350101 + 149.3193796511*$t);
   $X0 += 0.00000000006*cos(6.13258483711 + 122444.74344836328*$t);
   $X0 += 0.00000000006*cos(5.32441467055 + 24822.5794804313*$t);
   $X0 += 0.00000000005*cos(0.80608286136 + 26162.44092265799*$t);
   $X0 += 0.00000000005*cos(6.05508383397 + 154.0226279683*$t);
   $X0 += 0.00000000005*cos(0.11277607968 + 104401.85300568449*$t);
   $X0 += 0.00000000005*cos(0.32519606443 + 53654.91667469629*$t);
   $X0 += 0.00000000005*cos(5.10038810847 + 118828.71993201328*$t);
   $X0 += 0.00000000005*cos(6.11943807574 + 191.2044486281*$t);
   $X0 += 0.00000000005*cos(5.77124771600 + 124777.93984123527*$t);
   $X0 += 0.00000000005*cos(6.14333001423 + 51006.46184922329*$t);
   $X0 += 0.00000000006*cos(4.56143654817 + 104241.65006256089*$t);
   $X0 += 0.00000000005*cos(0.65495415751 + 104668.24825343689*$t);
   $X0 += 0.00000000004*cos(6.26437463075 + 76532.83088927349*$t);
   $X0 += 0.00000000006*cos(0.69389985894 + 75615.01078176608*$t);
   $X0 += 0.00000000004*cos(2.34685033934 + 24608.79275002629*$t);
   $X0 += 0.00000000004*cos(2.34685033934 + 24609.2803849933*$t);
   $X0 += 0.00000000005*cos(5.16993727796 + 23389.6955116805*$t);
   $X0 += 0.00000000005*cos(0.49368775325 + 28786.5984064349*$t);
   $X0 += 0.00000000004*cos(1.13517998278 + 31775.7773280097*$t);
   $X0 += 0.00000000004*cos(4.52844504843 + 20400.5165901057*$t);
   $X0 += 0.00000000005*cos(4.70021525766 + 107679.50440509345*$t);
   $X0 += 0.00000000004*cos(6.12765105459 + 13541.6650199747*$t);
   $X0 += 0.00000000005*cos(5.57977129989 + 29428.27175079049*$t);
   $X0 += 0.00000000005*cos(5.30763173110 + 26.0545023163*$t);
   $X0 += 0.00000000004*cos(3.86073160191 + 25764.64154240029*$t);
   $X0 += 0.00000000005*cos(6.09620933760 + 78265.43771491438*$t);
   $X0 += 0.00000000006*cos(2.35286352659 + 51688.4153493777*$t);
   $X0 += 0.00000000006*cos(6.24688491106 + 23865.80268949369*$t);
   $X0 += 0.00000000004*cos(2.73813240429 + 35211.72371407509*$t);
   $X0 += 0.00000000004*cos(2.92549262692 + 16964.57020404031*$t);
   $X0 += 0.00000000004*cos(5.93215370895 + 47804.1737338577*$t);
   $X0 += 0.00000000004*cos(3.83250941463 + 77623.56832092509*$t);
   $X0 += 0.00000000005*cos(0.50234970795 + 86458.22857541469*$t);
   $X0 += 0.00000000004*cos(1.08485860111 + 27040.2235823173*$t);
   $X0 += 0.00000000004*cos(4.57876643010 + 25136.0703357981*$t);
   $X0 += 0.00000000005*cos(4.59959229753 + 25778.86863640189*$t);
   $X0 += 0.00000000005*cos(1.06403273368 + 26397.42528171349*$t);
   $X0 += 0.00000000005*cos(4.02283478855 + 78374.15956342549*$t);
   $X0 += 0.00000000004*cos(3.47793310153 + 48834.95003896509*$t);
   $X0 += 0.00000000005*cos(5.01835542656 + 25771.26745443409*$t);
   $X0 += 0.00000000004*cos(1.67917095725 + 33480.6013611425*$t);
   $X0 += 0.00000000004*cos(3.98445407396 + 18695.6925569729*$t);
   $X0 += 0.00000000005*cos(0.59989929417 + 703.8770021009*$t);
   $X0 += 0.00000000004*cos(5.10698405361 + 156378.09946979408*$t);
   $X0 += 0.00000000005*cos(0.28534518128 + 155467.79310177248*$t);
   $X0 += 0.00000000005*cos(1.29884621572 + 26404.05119374729*$t);
   $X0 += 0.00000000004*cos(0.18983170026 + 9123.33293753389*$t);
   $X0 += 0.00000000005*cos(2.32620215234 + 2015.9149036433*$t);
   $X0 += 0.00000000004*cos(0.61980953649 + 76998.38576357969*$t);
   $X0 += 0.00000000004*cos(0.13065109470 + 129586.56314360249*$t);
   $X0 += 0.00000000004*cos(2.99427045342 + 1368.9040703285*$t);
   $X0 += 0.00000000004*cos(1.49149882620 + 45425.0092602861*$t);
   $X0 += 0.00000000004*cos(5.39873919994 + 28928.6478640837*$t);
   $X0 += 0.00000000004*cos(0.26488583127 + 23247.6460540317*$t);
   $X0 += 0.00000000004*cos(5.36778217078 + 23866.2903244607*$t);
   $X0 += 0.00000000004*cos(2.35188051852 + 20453.1132297053*$t);
   $X0 += 0.00000000004*cos(3.31174451269 + 31723.1806884101*$t);
   $X0 += 0.00000000004*cos(2.30331784549 + 26239.19462890059*$t);
   $X0 += 0.00000000004*cos(1.97810159548 + 199599.56195595946*$t);
   $X0 += 0.00000000004*cos(2.77836300783 + 52356.1272309165*$t);
   $X0 += 0.00000000004*cos(1.24931397565 + 26137.65594599489*$t);
   $X0 += 0.00000000004*cos(3.46342382133 + 51539.0538286077*$t);
   $X0 += 0.00000000004*cos(3.69944590805 + 32279.1699733045*$t);
   $X0 += 0.00000000004*cos(1.96417912316 + 19897.1239448109*$t);
   $X0 += 0.00000000004*cos(1.70188026990 + 78905.36680394509*$t);
   $X0 += 0.00000000004*cos(3.00071021966 + 52336.4574700167*$t);
   $X0 += 0.00000000004*cos(5.68676014339 + 50912.8927380537*$t);
   $X0 += 0.00000000005*cos(0.53917560687 + 26085.17801364109*$t);
   $X0 += 0.00000000005*cos(5.12444942434 + 26091.1159044743*$t);
   $X0 += 0.00000000004*cos(6.12827810146 + 25441.13612574289*$t);
   $X0 += 0.00000000004*cos(3.34711834324 + 25923.71104846829*$t);
   $X0 += 0.00000000004*cos(2.31650668797 + 26252.58286964711*$t);
   $X0 += 0.00000000004*cos(0.93197083282 + 24925.18461968289*$t);
   $X0 += 0.00000000005*cos(0.84223070790 + 13706.1009305641*$t);
   $X0 += 0.00000000005*cos(4.82139432331 + 38470.19298755129*$t);
   $X0 += 0.00000000005*cos(4.69896171289 + 18227.7275666185*$t);
   $X0 += 0.00000000005*cos(0.96466331832 + 33948.5663514969*$t);
   $X0 += 0.00000000005*cos(6.00043896187 + 10.8018827804*$t);
   $X0 += 0.00000000004*cos(2.80981383368 + 147423.26803784407*$t);
   $X0 += 0.00000000004*cos(3.37539797378 + 120225.98634417648*$t);
   $X0 += 0.00000000004*cos(2.65298230712 + 95.7354097343*$t);
   $X0 += 0.00000000004*cos(4.35824496747 + 74935.81758592589*$t);
   $X0 += 0.00000000004*cos(0.27953957902 + 142871.31454078527*$t);
   $X0 += 0.00000000004*cos(0.37589110708 + 26131.0300339611*$t);
   $X0 += 0.00000000004*cos(5.28773392413 + 26045.2638841543*$t);
   $X0 += 0.00000000004*cos(3.24717162624 + 1066.2516597065*$t);
   $X0 += 0.00000000004*cos(0.73794406026 + 25035.39094090229*$t);
   $X0 += 0.00000000005*cos(4.22425932829 + 1577.0997249643*$t);
   $X0 += 0.00000000004*cos(0.56307156026 + 1262.91354509469*$t);
   $X0 += 0.00000000005*cos(6.00635216618 + 51102.44107644109*$t);
   $X0 += 0.00000000004*cos(1.12630290328 + 106262.56823203048*$t);
   $X0 += 0.00000000004*cos(1.48479477953 + 104771.34102765548*$t);
   $X0 += 0.00000000005*cos(1.39791854337 + 179172.99086353746*$t);
   $X0 += 0.00000000004*cos(3.88144405304 + 52329.34127614969*$t);
   $X0 += 0.00000000005*cos(1.62731568621 + 151200.18655854589*$t);
   $X0 += 0.00000000004*cos(3.34025458568 + 18319.7804023631*$t);
   $X0 += 0.00000000004*cos(2.32337044553 + 33856.5135157523*$t);
   $X0 += 0.00000000005*cos(3.80744702100 + 51315.74017187909*$t);
   $X0 += 0.00000000004*cos(3.26438690958 + 159970.23761189409*$t);
   $X0 += 0.00000000005*cos(5.02554220823 + 107692.46881048009*$t);
   $X0 += 0.00000000004*cos(3.12725019808 + 24182.19455915029*$t);
   $X0 += 0.00000000004*cos(3.79384310544 + 25021.16384690069*$t);
   $X0 += 0.00000000005*cos(3.05069290377 + 49637.8015963783*$t);
   $X0 += 0.00000000004*cos(3.62578024170 + 64901.01590043989*$t);
   $X0 += 0.00000000005*cos(0.63333030208 + 77930.09730143669*$t);
   $X0 += 0.00000000004*cos(3.70617022011 + 106685.05277665229*$t);
   $X0 += 0.00000000004*cos(2.87321007380 + 25249.1776713073*$t);
   $X0 += 0.00000000004*cos(2.79041495741 + 26927.11624680809*$t);
   $X0 += 0.00000000005*cos(1.54666149626 + 130020.27488147929*$t);
   $X0 += 0.00000000004*cos(1.73976987963 + 42430.7295447753*$t);
   $X0 += 0.00000000004*cos(3.38176805408 + 189386.27640974846*$t);
   $X0 += 0.00000000004*cos(3.25018015096 + 130907.72451570889*$t);
   $X0 += 0.00000000004*cos(2.06353106693 + 76352.7537589889*$t);
   $X0 += 0.00000000004*cos(5.31959837801 + 78050.16651180109*$t);
   $X0 += 0.00000000004*cos(2.24224128580 + 26507.14396796589*$t);
   $X0 += 0.00000000004*cos(3.53906955839 + 21715.78295731649*$t);
   $X0 += 0.00000000004*cos(3.82039111865 + 50690.7593939287*$t);
   $X0 += 0.00000000003*cos(5.68926517353 + 76095.19275992349*$t);
   $X0 += 0.00000000004*cos(0.40278258619 + 27047.2698190153*$t);
   $X0 += 0.00000000004*cos(5.26084244502 + 25129.0240991001*$t);
   $X0 += 0.00000000003*cos(4.44502926835 + 25980.8416521281*$t);
   $X0 += 0.00000000003*cos(1.21859576286 + 26195.4522659873*$t);
   $X0 += 0.00000000004*cos(4.63561794007 + 52165.00440036799*$t);
   $X0 += 0.00000000004*cos(2.69531678715 + 104881.05971390787*$t);
   $X0 += 0.00000000004*cos(4.60182936648 + 8044.7688814119*$t);
   $X0 += 0.00000000005*cos(0.15112505254 + 45455.3361212905*$t);
   $X0 += 0.00000000004*cos(5.01042912978 + 25565.08190599689*$t);
   $X0 += 0.00000000004*cos(2.55785749371 + 215942.14454167709*$t);
   $X0 += 0.00000000004*cos(5.41986884632 + 25992.1677318399*$t);
   $X0 += 0.00000000004*cos(0.24375618489 + 26184.1261862755*$t);
   $X0 += 0.00000000003*cos(0.06938261642 + 129909.58092529287*$t);
   $X0 += 0.00000000004*cos(4.35391878884 + 48153.7875686677*$t);
   $X0 += 0.00000000005*cos(1.35637771566 + 10059.7505532097*$t);
   $X0 += 0.00000000004*cos(0.44819649652 + 16496.6052136859*$t);
   $X0 += 0.00000000004*cos(5.21542853469 + 35679.6887044295*$t);
   $X0 += 0.00000000004*cos(1.03867130112 + 103395.76900720468*$t);
   $X0 += 0.00000000004*cos(3.88762025023 + 51040.3732287387*$t);
   $X0 += 0.00000000003*cos(5.63440461335 + 125111.79578200469*$t);
   $X0 += 0.00000000004*cos(2.46419290723 + 128843.57308306987*$t);
   $X0 += 0.00000000004*cos(2.48121871677 + 52015.64273124709*$t);
   $X0 += 0.00000000004*cos(5.17960970329 + 621.9876183227*$t);
   $X0 += 0.00000000003*cos(1.35017943249 + 23113.5370044467*$t);
   $X0 += 0.00000000003*cos(4.31344559872 + 29062.7569136687*$t);
   $X0 += 0.00000000003*cos(0.13992300676 + 25794.7361746805*$t);
   $X0 += 0.00000000003*cos(5.52370202445 + 26381.5577434349*$t);
   $X0 += 0.00000000003*cos(1.65047629311 + 78283.13536813887*$t);
   $X0 += 0.00000000004*cos(6.01757036855 + 26037.66270218649*$t);
   $X0 += 0.00000000004*cos(0.78738841944 + 23543.4743221653*$t);
   $X0 += 0.00000000004*cos(4.87623661177 + 28632.8195959501*$t);
   $X0 += 0.00000000003*cos(2.34677335126 + 26279.5952251693*$t);
   $X0 += 0.00000000003*cos(3.31685167995 + 25896.6986929461*$t);
   $X0 += 0.00000000004*cos(4.17686572959 + 25998.6612112117*$t);
   $X0 += 0.00000000004*cos(1.48675930162 + 26177.63270690369*$t);
   $X0 += 0.00000000003*cos(1.45345885107 + 24079.10178493169*$t);
   $X0 += 0.00000000003*cos(2.83722079462 + 49851.10069181629*$t);
   $X0 += 0.00000000004*cos(0.04334468913 + 14235.79189565871*$t);
   $X0 += 0.00000000004*cos(5.62028034208 + 37940.50202245669*$t);
   $X0 += 0.00000000003*cos(3.19680435173 + 41494.79956406649*$t);
   $X0 += 0.00000000004*cos(3.72413363777 + 261988.65378531882*$t);
   $X0 += 0.00000000003*cos(3.23280579449 + 61560.40347374009*$t);
   $X0 += 0.00000000004*cos(6.14807840422 + 32217.4439985643*$t);
   $X0 += 0.00000000003*cos(5.81361645381 + 78243.79584633929*$t);
   $X0 += 0.00000000004*cos(2.70356592860 + 51995.9729703473*$t);
   $X0 += 0.00000000003*cos(0.89067609209 + 154308.90556274189*$t);
   $X0 += 0.00000000003*cos(0.19486754591 + 78398.0622917911*$t);
   $X0 += 0.00000000003*cos(3.22218438408 + 202420.39310008568*$t);
   $X0 += 0.00000000003*cos(4.85336780642 + 79117.14962395809*$t);
   $X0 += 0.00000000004*cos(4.81266956480 + 28420.8557169627*$t);
   $X0 += 0.00000000003*cos(4.64236443125 + 313054.59388140672*$t);
   $X0 += 0.00000000003*cos(1.82348628585 + 26114.4452788575*$t);
   $X0 += 0.00000000003*cos(3.84013874536 + 26061.8486392579*$t);
   $X0 += 0.00000000003*cos(6.16611553760 + 116783.41139921108*$t);
   $X0 += 0.00000000003*cos(6.22418249204 + 2169.0042997661*$t);
   $X0 += 0.00000000003*cos(3.34714723948 + 32131.88790546609*$t);
   $X0 += 0.00000000003*cos(2.95057028322 + 68241.62832713968*$t);
   $X0 += 0.00000000004*cos(2.12743436214 + 49424.5025009403*$t);
   $X0 += 0.00000000003*cos(3.11077916313 + 27993.6117239981*$t);
   $X0 += 0.00000000004*cos(3.05439250241 + 156954.26085780468*$t);
   $X0 += 0.00000000004*cos(3.67142845852 + 52021.78365518009*$t);
   $X0 += 0.00000000004*cos(5.73212182199 + 192828.85135471389*$t);
   $X0 += 0.00000000003*cos(0.27361838208 + 25345.1568985251*$t);
   $X0 += 0.00000000003*cos(1.81581655351 + 87648.79425028129*$t);
   $X0 += 0.00000000003*cos(0.68500664823 + 105410.75067900248*$t);
   $X0 += 0.00000000003*cos(3.93077791188 + 130446.38808425449*$t);
   $X0 += 0.00000000003*cos(6.05546250987 + 45290.41257573408*$t);
   $X0 += 0.00000000003*cos(1.95812396188 + 251775.36823910786*$t);
   $X0 += 0.00000000004*cos(1.27498750708 + 50952.23225985329*$t);
   $X0 += 0.00000000003*cos(1.43670342990 + 23421.09462541629*$t);
   $X0 += 0.00000000003*cos(4.22692160131 + 28755.19929269911*$t);
   $X0 += 0.00000000003*cos(3.98270762104 + 163297.88563320728*$t);
   $X0 += 0.00000000003*cos(3.47547087612 + 26727.55661040469*$t);
   $X0 += 0.00000000004*cos(3.43694245255 + 24711.88552424489*$t);
   $X0 += 0.00000000003*cos(3.68091678909 + 111590.04434041328*$t);
   $X0 += 0.00000000004*cos(0.29131403040 + 50476.77088412869*$t);
   $X0 += 0.00000000003*cos(4.21457884017 + 52072.2679120513*$t);
   $X0 += 0.00000000003*cos(3.61792324623 + 77622.5396804671*$t);
   $X0 += 0.00000000003*cos(0.47995189315 + 52016.7483658397*$t);
   $X0 += 0.00000000003*cos(1.68763185836 + 25970.8270908375*$t);
   $X0 += 0.00000000003*cos(3.97599317285 + 26205.4668272779*$t);
   $X0 += 0.00000000004*cos(5.27937253398 + 27511.7116910207*$t);
   $X0 += 0.00000000004*cos(0.38425249723 + 24664.5822270947*$t);
   $X0 += 0.00000000003*cos(2.30032754148 + 33918.23949049249*$t);
   $X0 += 0.00000000003*cos(3.36329748973 + 18258.0544276229*$t);
   $X0 += 0.00000000004*cos(4.83285053411 + 112545.64408202188*$t);
   $X0 += 0.00000000004*cos(3.76448219227 + 61921.3706564885*$t);
   $X0 += 0.00000000003*cos(6.06735340898 + 51741.85072778809*$t);
   $X0 += 0.00000000003*cos(2.88367503068 + 222225.22039166848*$t);
   $X0 += 0.00000000003*cos(1.15891911019 + 78249.72614820449*$t);
   $X0 += 0.00000000004*cos(3.05981714265 + 158116.73556221249*$t);
   $X0 += 0.00000000003*cos(3.68443191743 + 73891.58924046489*$t);
   $X0 += 0.00000000003*cos(1.88338582885 + 26387.2733533269*$t);
   $X0 += 0.00000000003*cos(6.09657221455 + 12725.6972522585*$t);
   $X0 += 0.00000000003*cos(5.04796024742 + 26984.2264157131*$t);
   $X0 += 0.00000000003*cos(0.61566478379 + 25192.0675024023*$t);
   $X0 += 0.00000000003*cos(3.48855867987 + 103498.17236706128*$t);
   $X0 += 0.00000000003*cos(0.74835060715 + 79487.77108298469*$t);
   $X0 += 0.00000000003*cos(1.44150895108 + 50579.37602338029*$t);
   $X0 += 0.00000000003*cos(3.44346861237 + 143980.69309287866*$t);
   $X0 += 0.00000000003*cos(5.63232630472 + 91805.37444469729*$t);
   $X0 += 0.00000000003*cos(3.11959949863 + 50490.99797813029*$t);
   $X0 += 0.00000000003*cos(2.99543245253 + 54879.6662553075*$t);
   $X0 += 0.00000000003*cos(4.69667538764 + 55484.5159960123*$t);
   $X0 += 0.00000000003*cos(4.64572031653 + 2332.9525753885*$t);
   $X0 += 0.00000000003*cos(1.39528143088 + 1383.13116433011*$t);
   $X0 += 0.00000000003*cos(1.78559566583 + 128106.56313248246*$t);
   $X0 += 0.00000000003*cos(0.70542302818 + 1471.9968445471*$t);
   $X0 += 0.00000000003*cos(4.72601280222 + 146505.82538682928*$t);
   $X0 += 0.00000000004*cos(4.23386749461 + 102232.60488843489*$t);
   $X0 += 0.00000000003*cos(5.04493378788 + 24510.8034166099*$t);
   $X0 += 0.00000000003*cos(0.61869124333 + 27665.4905015055*$t);
   $X0 += 0.00000000004*cos(4.60583023308 + 26365.1819527991*$t);
   $X0 += 0.00000000004*cos(1.05779479813 + 25811.11196531629*$t);
   $X0 += 0.00000000003*cos(0.02744721053 + 106263.05586699749*$t);
   $X0 += 0.00000000003*cos(3.49736618564 + 50601.2042993497*$t);
   $X0 += 0.00000000003*cos(0.30011567778 + 179967.11098302988*$t);
   $X0 += 0.00000000003*cos(1.53690934138 + 210921.73841929682*$t);
   $X0 += 0.00000000003*cos(1.31624723631 + 26202.5460659711*$t);
   $X0 += 0.00000000003*cos(2.75823464337 + 2821.07496160971*$t);
   $X0 += 0.00000000003*cos(6.26421306637 + 5020.16230489671*$t);
   $X0 += 0.00000000003*cos(4.80001740407 + 229463.89598326846*$t);
   $X0 += 0.00000000003*cos(6.02378503125 + 71378.31571730827*$t);
   $X0 += 0.00000000003*cos(1.42673173544 + 29001.0309389285*$t);
   $X0 += 0.00000000003*cos(4.23689329577 + 23175.26297918689*$t);
   $X0 += 0.00000000003*cos(0.68304283752 + 15406.40878752529*$t);
   $X0 += 0.00000000003*cos(4.32693015716 + 25940.06823463139*$t);
   $X0 += 0.00000000003*cos(2.38990502393 + 220026.13304838145*$t);
   $X0 += 0.00000000003*cos(0.59649942463 + 963.6465204549*$t);
   $X0 += 0.00000000003*cos(2.87116210123 + 113455.76337789789*$t);
   $X0 += 0.00000000003*cos(1.35908128792 + 49201.4401460209*$t);
   $X0 += 0.00000000003*cos(5.82324458614 + 143165.21296012949*$t);
   $X0 += 0.00000000003*cos(2.38673232157 + 51735.22481575429*$t);
   $X0 += 0.00000000003*cos(2.84532132702 + 25914.69838767311*$t);
   $X0 += 0.00000000003*cos(2.81830370419 + 26261.59553044229*$t);
   $X0 += 0.00000000003*cos(5.22041687007 + 51745.51975649279*$t);
   $X0 += 0.00000000002*cos(3.03566163825 + 25554.52384069999*$t);
   $X0 += 0.00000000002*cos(2.62796339296 + 26621.77007741539*$t);
   $X0 += 0.00000000003*cos(5.20753478644 + 25530.6962208573*$t);
   $X0 += 0.00000000003*cos(0.45609024477 + 26645.5976972581*$t);
   $X0 += 0.00000000002*cos(3.73081171564 + 99979.49238203908*$t);
   $X0 += 0.00000000003*cos(5.03565808597 + 25626.81052760331*$t);
   $X0 += 0.00000000003*cos(0.62796694524 + 26549.48339051209*$t);
   $X0 += 0.00000000003*cos(5.59916996136 + 23976.4966456801*$t);
   $X0 += 0.00000000003*cos(5.34361577728 + 20956.5058750001*$t);
   $X0 += 0.00000000003*cos(0.32000925393 + 31219.7880431153*$t);
   $X0 += 0.00000000002*cos(0.74211300250 + 242985.64742485987*$t);
   $X0 += 0.00000000002*cos(4.68727528203 + 133767.40754666766*$t);
   $X0 += 0.00000000003*cos(4.44342016483 + 283524.6034298342*$t);
   $X0 += 0.00000000002*cos(1.15603737137 + 127790.81706491447*$t);
   $X0 += 0.00000000002*cos(4.29763002496 + 127791.30469988148*$t);
   $X0 += 0.00000000002*cos(5.43119958754 + 71378.80335227528*$t);
   $X0 += 0.00000000002*cos(4.58931827360 + 21202.3375212295*$t);
   $X0 += 0.00000000002*cos(1.07430675761 + 30973.9563968859*$t);
   $X0 += 0.00000000003*cos(3.47073853524 + 2353.1099712553*$t);
   $X0 += 0.00000000003*cos(2.48443590939 + 76358.48847726568*$t);
   $X0 += 0.00000000003*cos(0.03897228819 + 24292.40088036969*$t);
   $X0 += 0.00000000003*cos(2.04739291424 + 157636.55358405507*$t);
   $X0 += 0.00000000003*cos(0.83289676584 + 25361.6649391875*$t);
   $X0 += 0.00000000003*cos(4.83072826537 + 26814.6289789279*$t);
   $X0 += 0.00000000002*cos(1.24497717708 + 57369.41715942969*$t);
   $X0 += 0.00000000003*cos(5.41525530380 + 52187.0958008958*$t);
   $X0 += 0.00000000003*cos(4.02922617142 + 49953.70583106789*$t);
   $X0 += 0.00000000002*cos(1.46800292953 + 110396.08496515569*$t);
   $X0 += 0.00000000002*cos(2.56755430966 + 273311.31788362318*$t);
   $X0 += 0.00000000002*cos(0.95339846585 + 7994.2846245407*$t);
   $X0 += 0.00000000002*cos(2.63620968801 + 31127.7352073707*$t);
   $X0 += 0.00000000002*cos(3.02741534320 + 21048.5587107447*$t);
   $X0 += 0.00000000002*cos(0.16837305460 + 27133.30179524529*$t);
   $X0 += 0.00000000002*cos(3.81308180912 + 289185.93547898643*$t);
   $X0 += 0.00000000002*cos(0.71138534841 + 5634.7899118689*$t);
   $X0 += 0.00000000002*cos(1.76112647723 + 108903.32224587208*$t);
   $X0 += 0.00000000003*cos(6.06940283285 + 128857.80017707148*$t);
   $X0 += 0.00000000002*cos(4.81801153794 + 5687.3865514685*$t);
   $X0 += 0.00000000003*cos(4.73128149211 + 21650.63275068411*$t);
   $X0 += 0.00000000003*cos(0.93234353910 + 30525.66116743129*$t);
   $X0 += 0.00000000003*cos(0.49698769500 + 117.5636857037*$t);
   $X0 += 0.00000000002*cos(2.18169836096 + 29266.2927496253*$t);
   $X0 += 0.00000000002*cos(5.80090003502 + 24176.45984087349*$t);
   $X0 += 0.00000000002*cos(3.25477992512 + 102129.99974918329*$t);
   $X0 += 0.00000000003*cos(4.34402660372 + 53338.52480503969*$t);
   $X0 += 0.00000000003*cos(6.27379601640 + 67608.51339047348*$t);
   $X0 += 0.00000000002*cos(3.21341452307 + 267649.98583447101*$t);
   $X0 += 0.00000000002*cos(1.92153809084 + 544.1618765797*$t);
   $X0 += 0.00000000002*cos(0.63318713248 + 54862.7721951731*$t);
   $X0 += 0.00000000002*cos(4.71897270316 + 162188.50708111389*$t);
   $X0 += 0.00000000002*cos(5.95465982913 + 123554.12200045666*$t);
   $X0 += 0.00000000003*cos(3.65120791864 + 47803.68609889069*$t);
   $X0 += 0.00000000002*cos(3.82189241243 + 112546.13171698889*$t);
   $X0 += 0.00000000002*cos(0.77739174044 + 71024.79001563908*$t);
   $X0 += 0.00000000003*cos(3.39676430082 + 24971.65485996349*$t);
   $X0 += 0.00000000003*cos(2.26686073039 + 27204.6390581519*$t);
   $X0 += 0.00000000002*cos(5.01817813007 + 26165.89750304159*$t);
   $X0 += 0.00000000003*cos(4.38762823834 + 14477.5950006835*$t);
   $X0 += 0.00000000002*cos(6.05328448229 + 97466.21885888245*$t);
   $X0 += 0.00000000002*cos(5.24033215662 + 8482.40701076191*$t);
   $X0 += 0.00000000003*cos(3.13012289596 + 206584.70508969868*$t);
   $X0 += 0.00000000003*cos(0.28818731494 + 25881.47377565349*$t);
   $X0 += 0.00000000002*cos(0.63582632095 + 105306.96849042189*$t);
   $X0 += 0.00000000003*cos(6.00821622727 + 7768.12273921109*$t);
   $X0 += 0.00000000002*cos(0.75081825828 + 263097.54470244513*$t);
   $X0 += 0.00000000002*cos(0.75081825828 + 263098.03233741222*$t);
   $X0 += 0.00000000002*cos(0.39253998165 + 90988.91904201408*$t);
   $X0 += 0.00000000003*cos(3.01604861450 + 5814.86704215349*$t);
   $X0 += 0.00000000003*cos(0.86557994215 + 49976.9627573449*$t);
   $X0 += 0.00000000003*cos(2.68824003168 + 27161.7559832485*$t);
   $X0 += 0.00000000002*cos(6.12457483767 + 27151.46104250999*$t);
   $X0 += 0.00000000003*cos(1.61229350276 + 29051.0275608327*$t);
   $X0 += 0.00000000003*cos(4.05133152845 + 23125.2663572827*$t);
   $X0 += 0.00000000002*cos(3.41958084674 + 32973.29684898851*$t);
   $X0 += 0.00000000003*cos(0.44711181004 + 134.35286706851*$t);
   $X0 += 0.00000000002*cos(0.64120353129 + 78484.36588464488*$t);
   $X0 += 0.00000000002*cos(0.13220595262 + 50373.67810991009*$t);
   $X0 += 0.00000000003*cos(2.58526525309 + 246114.03618995566*$t);
   $X0 += 0.00000000002*cos(5.24756751342 + 19336.61848374489*$t);
   $X0 += 0.00000000002*cos(2.97251385048 + 25014.53793486689*$t);
   $X0 += 0.00000000002*cos(0.46042924320 + 156534.77621392948*$t);
   $X0 += 0.00000000003*cos(4.30654322236 + 78115.87451777978*$t);
   $X0 += 0.00000000002*cos(0.28012264229 + 51760.49761001979*$t);
   $X0 += 0.00000000002*cos(1.24434330997 + 25867.24668165189*$t);
   $X0 += 0.00000000002*cos(3.40478353479 + 26089.14379679899*$t);
   $X0 += 0.00000000002*cos(5.12603604161 + 30486.3216456317*$t);
   $X0 += 0.00000000002*cos(0.53758898960 + 21689.9722724837*$t);
   $X0 += 0.00000000003*cos(3.31169456914 + 78112.90557236319*$t);
   $X0 += 0.00000000002*cos(4.36305168844 + 1898.5950354231*$t);
   $X0 += 0.00000000002*cos(5.21464317983 + 235746.97183325986*$t);
   $X0 += 0.00000000002*cos(4.71956373810 + 26151.8828573611*$t);
   $X0 += 0.00000000002*cos(0.94406129310 + 26024.4110607543*$t);
   $X0 += 0.00000000002*cos(5.77039379556 + 50049.68505828569*$t);
   $X0 += 0.00000000003*cos(4.30724193035 + 42116.5433649057*$t);
   $X0 += 0.00000000002*cos(0.39973619044 + 61165.51780606429*$t);
   $X0 += 0.00000000002*cos(1.17288571933 + 249268.72327485126*$t);
   $X0 += 0.00000000002*cos(4.26445898022 + 184526.76529172009*$t);
   $X0 += 0.00000000003*cos(6.06473923651 + 26226.6644559284*$t);
   $X0 += 0.00000000003*cos(5.88207110188 + 25949.629462187*$t);
   $X0 += 0.00000000002*cos(4.41949123763 + 223180.33249831005*$t);
   $X0 += 0.00000000002*cos(4.41949123763 + 223180.82013327705*$t);
   $X0 += 0.00000000003*cos(2.07289744094 + 65851.58035270809*$t);
   $X0 += 0.00000000002*cos(3.33696976419 + 4525.8989947425*$t);
   $X0 += 0.00000000003*cos(1.90326122005 + 26376.0348191275*$t);
   $X0 += 0.00000000003*cos(3.76036381116 + 25800.2590989879*$t);
   $X0 += 0.00000000002*cos(1.19438265065 + 26049.52628845289*$t);
   $X0 += 0.00000000002*cos(5.14117998003 + 4398.41850405751*$t);
   $X0 += 0.00000000003*cos(3.79796015832 + 23581.5019948011*$t);
   $X0 += 0.00000000003*cos(1.86566487289 + 28594.7919233143*$t);
   $X0 += 0.00000000002*cos(3.55520237633 + 1593.2488660304*$t);
   $X0 += 0.00000000002*cos(3.13962245506 + 78267.34694259708*$t);
   $X0 += 0.00000000002*cos(0.92505130149 + 54508.7588585369*$t);
   $X0 += 0.00000000002*cos(4.31177771871 + 78476.76470267709*$t);
   $X0 += 0.00000000002*cos(0.41611414366 + 1280.0383901115*$t);
   $X0 += 0.00000000002*cos(1.10174173283 + 35077.12702952309*$t);
   $X0 += 0.00000000003*cos(1.43403842688 + 26124.79552198719*$t);
   $X0 += 0.00000000003*cos(4.22958660433 + 26051.4983961282*$t);
   $X0 += 0.00000000002*cos(1.72941826067 + 158745.93213614845*$t);
   $X0 += 0.00000000002*cos(5.48576882408 + 52367.49836674349*$t);
   $X0 += 0.00000000002*cos(6.20502032580 + 41520.61024889929*$t);
   $X0 += 0.00000000002*cos(4.14056597313 + 51950.2208322217*$t);
   $X0 += 0.00000000002*cos(2.55940503302 + 28133.4554918599*$t);
   $X0 += 0.00000000002*cos(3.10421999819 + 24042.8384262555*$t);
   $X0 += 0.00000000003*cos(2.17160326158 + 51753.38406301899*$t);
   $X0 += 0.00000000003*cos(6.15199422925 + 25633.23759253039*$t);
   $X0 += 0.00000000003*cos(5.79481610914 + 26543.05632558499*$t);
   $X0 += 0.00000000002*cos(3.97512757833 + 24999.54380245671*$t);
   $X0 += 0.00000000002*cos(1.68849745288 + 27176.75011565869*$t);
   $X0 += 0.00000000002*cos(0.30844272162 + 309.0345051723*$t);
   $X0 += 0.00000000002*cos(2.40438367720 + 1155.6049748905*$t);
   $X0 += 0.00000000002*cos(0.34200603730 + 236702.57157486846*$t);
   $X0 += 0.00000000002*cos(3.00390396144 + 76887.69180739328*$t);
   $X0 += 0.00000000002*cos(0.98682277986 + 4084.2323241879*$t);
   $X0 += 0.00000000002*cos(1.59074157843 + 24601.67920302549*$t);
   $X0 += 0.00000000002*cos(2.80906034932 + 27818.78167702329*$t);
   $X0 += 0.00000000002*cos(2.26687572256 + 2198.8435258035*$t);
   $X0 += 0.00000000002*cos(4.53134788435 + 24292.8885153367*$t);
   $X0 += 0.00000000002*cos(1.58580178730 + 22759.5236678105*$t);
   $X0 += 0.00000000002*cos(2.91202347140 + 27143.5967359838*$t);
   $X0 += 0.00000000002*cos(1.70541081745 + 25973.22003540546*$t);
   $X0 += 0.00000000002*cos(0.14177660173 + 10021.5934626159*$t);
   $X0 += 0.00000000002*cos(1.51917223670 + 20618.2631760171*$t);
   $X0 += 0.00000000002*cos(4.14445279451 + 31558.0307420983*$t);
   $X0 += 0.00000000002*cos(2.33949764052 + 2854.8841913937*$t);
   $X0 += 0.00000000002*cos(1.98831285254 + 51953.18977763829*$t);
   $X0 += 0.00000000002*cos(0.41604857841 + 51951.21766996299*$t);
   $X0 += 0.00000000002*cos(1.57183044674 + 52249.34722649089*$t);
   $X0 += 0.00000000002*cos(1.70818461045 + 138633.54722359608*$t);
   $X0 += 0.00000000002*cos(5.13575806013 + 225687.46509753366*$t);
   $X0 += 0.00000000002*cos(1.56258965853 + 138319.36104372646*$t);
   $X0 += 0.00000000002*cos(2.18966503711 + 32768.88417749029*$t);
   $X0 += 0.00000000002*cos(0.33098015159 + 24079.5894198987*$t);
   $X0 += 0.00000000002*cos(4.71436622522 + 52202.3484204317*$t);
   $X0 += 0.00000000002*cos(4.96992436419 + 28206.42318446909*$t);
   $X0 += 0.00000000002*cos(4.96216423435 + 180496.80194812448*$t);
   $X0 += 0.00000000002*cos(4.73660629226 + 78259.58427188109*$t);
   $X0 += 0.00000000002*cos(2.28401532959 + 26624.46383618609*$t);
   $X0 += 0.00000000002*cos(0.07960474111 + 23549.8984548041*$t);
   $X0 += 0.00000000002*cos(0.37327493336 + 26293.84487252789*$t);
   $X0 += 0.00000000002*cos(5.83913193896 + 4601.70630147949*$t);
   $X0 += 0.00000000002*cos(6.23015270329 + 21716.2705922835*$t);
   $X0 += 0.00000000002*cos(3.66936461651 + 330.8627811417*$t);
   $X0 += 0.00000000002*cos(1.51579465920 + 51219.27330969429*$t);
   $X0 += 0.00000000002*cos(5.05016618926 + 78256.35206023828*$t);
   $X0 += 0.00000000002*cos(3.58281342347 + 50063.91215228729*$t);
   $X0 += 0.00000000002*cos(1.42199745004 + 132658.02899457427*$t);
   $X0 += 0.00000000002*cos(3.28152370063 + 96.2230447013*$t);
   $X0 += 0.00000000002*cos(4.38858567715 + 1265.32366114289*$t);
   $X0 += 0.00000000002*cos(2.49109566786 + 52823.0609339467*$t);
   $X0 += 0.00000000002*cos(1.26800894276 + 78270.57915423988*$t);
   $X0 += 0.00000000002*cos(5.41071683840 + 7392.21058460129*$t);
   $X0 += 0.00000000002*cos(1.10270458425 + 61165.03017109729*$t);
   $X0 += 0.00000000002*cos(2.14950038663 + 2544.4288194089*$t);
   $X0 += 0.00000000002*cos(1.97162097641 + 26421.5152648601*$t);
   $X0 += 0.00000000002*cos(1.33257889355 + 2655.8121899573*$t);
   $X0 += 0.00000000002*cos(0.79025165001 + 24705.9490265731*$t);
   $X0 += 0.00000000002*cos(4.87337338120 + 27470.3448915423*$t);
   $X0 += 0.00000000002*cos(6.14740336072 + 1353.5049275333*$t);
   $X0 += 0.00000000002*cos(0.11906484226 + 1603.5438067689*$t);
   $X0 += 0.00000000002*cos(3.46701006002 + 40738.45907867528*$t);
   $X0 += 0.00000000002*cos(5.71504799272 + 30460.0233258319*$t);
   $X0 += 0.00000000002*cos(1.16042497962 + 4578.4956343421*$t);
   $X0 += 0.00000000002*cos(0.14527391572 + 78113.90241010448*$t);
   $X0 += 0.00000000002*cos(4.19897532608 + 32839.1877994035*$t);
   $X0 += 0.00000000002*cos(0.13374340654 + 14297.5178703989*$t);
   $X0 += 0.00000000002*cos(5.52988162467 + 37878.7760477165*$t);
   $X0 += 0.00000000002*cos(6.20542886765 + 104501.41958091488*$t);
   $X0 += 0.00000000002*cos(2.61097814223 + 25138.48371512089*$t);
   $X0 += 0.00000000002*cos(1.06395351693 + 44131.52503670349*$t);
   $X0 += 0.00000000002*cos(4.10426443523 + 104358.48229581409*$t);
   $X0 += 0.00000000002*cos(5.39518200455 + 23336.5993593661*$t);
   $X0 += 0.00000000002*cos(5.39518200455 + 23336.11172439909*$t);
   $X0 += 0.00000000002*cos(6.04499536111 + 224.1009782184*$t);
   $X0 += 0.00000000002*cos(3.63958668308 + 51547.19851458179*$t);
   $X0 += 0.00000000002*cos(3.16690823789 + 202.0095776906*$t);
   $X0 += 0.00000000002*cos(3.16690823789 + 202.4972126576*$t);
   $X0 += 0.00000000002*cos(6.25775046727 + 31108.06544647091*$t);
   $X0 += 0.00000000002*cos(5.68905987112 + 21068.2284716445*$t);
   $X0 += 0.00000000002*cos(2.96587029379 + 117076.82218358829*$t);
   $X0 += 0.00000000002*cos(6.10746294738 + 117077.30981855528*$t);
   $X0 += 0.00000000002*cos(5.38291113550 + 24814.97829846349*$t);
   $X0 += 0.00000000002*cos(1.98570847468 + 27299.4880294167*$t);
   $X0 += 0.00000000002*cos(3.67791655653 + 24876.8058886987*$t);
   $X0 += 0.00000000002*cos(2.83769263825 + 26086.17485138239*$t);
   $X0 += 0.00000000002*cos(5.74298703182 + 105418.35186097029*$t);
   $X0 += 0.00000000002*cos(2.69099617398 + 49534.7088221597*$t);
   $X0 += 0.00000000002*cos(4.25460776712 + 107679.99204006046*$t);
   $X0 += 0.00000000002*cos(1.43038533012 + 19337.1061187119*$t);
   $X0 += 0.00000000002*cos(1.80708255079 + 24957.9930114719*$t);
   $X0 += 0.00000000002*cos(3.85654248042 + 27218.3009066435*$t);
   $X0 += 0.00000000002*cos(3.46898572008 + 66826.36222024946*$t);
   $X0 += 0.00000000002*cos(3.11677200697 + 27257.7352104663*$t);
   $X0 += 0.00000000002*cos(1.54813681705 + 26197.86564531009*$t);
   $X0 += 0.00000000002*cos(2.26084282341 + 151199.69892357888*$t);
   $X0 += 0.00000000002*cos(2.41505887981 + 426.1525939975*$t);
   $X0 += 0.00000000002*cos(2.05501019547 + 76784.59903317469*$t);
   $X0 += 0.00000000002*cos(5.17891248404 + 179172.50322857044*$t);
   $X0 += 0.00000000002*cos(3.03839569379 + 168959.21768235945*$t);
   $X0 += 0.00000000002*cos(3.03839569378 + 168959.70531732646*$t);
   $X0 += 0.00000000002*cos(6.05144227865 + 7829.84871395129*$t);
   $X0 += 0.00000000002*cos(2.93845777192 + 151975.22153490288*$t);
   $X0 += 0.00000000002*cos(5.56610117919 + 20235.36664379391*$t);
   $X0 += 0.00000000002*cos(0.09752385202 + 31940.9272743215*$t);
   $X0 += 0.00000000002*cos(0.34264055281 + 1585.38455950421*$t);
   $X0 += 0.00000000002*cos(3.25664376155 + 76.0222537921*$t);
   $X0 += 0.00000000002*cos(4.50673995632 + 1581.7155307995*$t);
   $X0 += 0.00000000002*cos(4.26102899883 + 52643.5274560193*$t);
   $X0 += 0.00000000002*cos(0.89847733804 + 130433.13096645445*$t);
   $X0 += 0.00000000002*cos(5.35295047368 + 79956.11891170849*$t);
   $X0 += 0.00000000002*cos(1.38123247936 + 190809.35350674443*$t);
   $X0 += 0.00000000002*cos(5.22748899717 + 155417.79647986828*$t);
   $X0 += 0.00000000002*cos(2.25472465461 + 58219.79104704029*$t);
   $X0 += 0.00000000002*cos(5.67352931908 + 299399.22102519739*$t);
   $X0 += 0.00000000002*cos(0.82971507937 + 51738.40620949199*$t);
   $X0 += 0.00000000002*cos(2.97072309926 + 2045.5523502857*$t);
   $X0 += 0.00000000002*cos(4.40886857181 + 27953.3783615753*$t);
   $X0 += 0.00000000002*cos(1.25475645940 + 24222.9155565401*$t);
   $X0 += 0.00000000002*cos(4.61765528518 + 164721.45036517028*$t);
   $X0 += 0.00000000002*cos(0.02963994293 + 28441.0131128295*$t);
   $X0 += 0.00000000002*cos(5.63398508828 + 23735.2808052859*$t);
   $X0 += 0.00000000002*cos(4.94052786966 + 26948.4568878105*$t);
   $X0 += 0.00000000002*cos(0.72309716155 + 25227.83703030489*$t);
   $X0 += 0.00000000002*cos(2.26852923431 + 148532.64658993746*$t);
   $X0 += 0.00000000002*cos(2.26852923431 + 148533.13422490445*$t);
   $X0 += 0.00000000002*cos(1.51852948979 + 77307.17645126847*$t);
   $X0 += 0.00000000002*cos(6.10588157940 + 191.6920835951*$t);
   $X0 += 0.00000000002*cos(0.34186830904 + 23976.00901071309*$t);
   $X0 += 0.00000000002*cos(4.08150026692 + 206370.27255720507*$t);
   $X0 += 0.00000000002*cos(5.05627960945 + 4268.3379956771*$t);
   $X0 += 0.00000000002*cos(1.84151604889 + 23962.2695516785*$t);
   $X0 += 0.00000000002*cos(1.84151604889 + 23961.78191671149*$t);
   $X0 += 0.00000000002*cos(4.35044814692 + 3776.6747032183*$t);
   $X0 += 0.00000000002*cos(2.20524904392 + 24918.55870764909*$t);
   $X0 += 0.00000000002*cos(3.21408526505 + 102769.40940053028*$t);
   $X0 += 0.00000000002*cos(2.12858110666 + 25889.0749576213*$t);
   $X0 += 0.00000000002*cos(3.53504392455 + 26287.21896049409*$t);
   $X0 += 0.00000000002*cos(1.66367572246 + 2648.2110079895*$t);
   $X0 += 0.00000000002*cos(5.71672028658 + 80432.71372448868*$t);
   $X0 += 0.00000000002*cos(2.91629438354 + 13989.9602494293*$t);
   $X0 += 0.00000000002*cos(2.74733064767 + 38186.3336686861*$t);
   $X0 += 0.00000000002*cos(3.61656667123 + 8194.5191506921*$t);
   $X0 += 0.00000000002*cos(4.13822438475 + 237009.64156087102*$t);
   $X0 += 0.00000000002*cos(2.25489644524 + 24863.84148331209*$t);
   $X0 += 0.00000000002*cos(0.79435355035 + 76064.86589891909*$t);
   $X0 += 0.00000000002*cos(6.11153635623 + 71492.75500599349*$t);
   $X0 += 0.00000000002*cos(1.47457311729 + 30831.5487222281*$t);
   $X0 += 0.00000000002*cos(4.18905191391 + 21344.7451958873*$t);
   $X0 += 0.00000000002*cos(2.64103330733 + 124663.98818751707*$t);
   $X0 += 0.00000000001*cos(4.41626377495 + 101702.91392334028*$t);
   $X0 += 0.00000000002*cos(2.92553614920 + 105404.12476696869*$t);
   $X0 += 0.00000000002*cos(1.90962776233 + 307.31380348609*$t);
   $X0 += 0.00000000002*cos(1.20163121188 + 184834.32291268966*$t);
   $X0 += 0.00000000002*cos(4.34322386547 + 184833.83527772261*$t);
   $X0 += 0.00000000001*cos(5.20673509792 + 124156.19604039608*$t);
   $X0 += 0.00000000001*cos(3.29097711242 + 22909.51353352309*$t);
   $X0 += 0.00000000001*cos(3.29097711242 + 22910.0011684901*$t);
   $X0 += 0.00000000001*cos(4.22627101320 + 958.87904247409*$t);
   $X0 += 0.00000000002*cos(0.74624553136 + 77196.97013004909*$t);
   $X0 += 0.00000000002*cos(2.61684173604 + 9591.29792788829*$t);
   $X0 += 0.00000000002*cos(1.03105335724 + 189385.78877478145*$t);
   $X0 += 0.00000000001*cos(0.57809610749 + 24919.2481220111*$t);
   $X0 += 0.00000000001*cos(5.08552892372 + 27257.0457961043*$t);
   $X0 += 0.00000000002*cos(3.46440981619 + 25440.64849077589*$t);
   $X0 += 0.00000000001*cos(0.45556161499 + 26010.3964150738*$t);
   $X0 += 0.00000000002*cos(0.42518108376 + 43693.8869073535*$t);
   $X0 += 0.00000000002*cos(3.47315264300 + 25014.05029989989*$t);
   $X0 += 0.00000000001*cos(5.34981963981 + 9385.0848255587*$t);
   $X0 += 0.00000000002*cos(4.60292662632 + 26202.09861277591*$t);
   $X0 += 0.00000000001*cos(4.92214692877 + 52499.55551728929*$t);
   $X0 += 0.00000000002*cos(3.46155131960 + 14650.55593710108*$t);
   $X0 += 0.00000000002*cos(5.08759156576 + 78690.06379811508*$t);
   $X0 += 0.00000000001*cos(0.08764164508 + 5039.34443082949*$t);
   $X0 += 0.00000000002*cos(4.10611692906 + 2214.98690507971*$t);
   $X0 += 0.00000000001*cos(3.50305053765 + 77211.19722405069*$t);
   $X0 += 0.00000000001*cos(5.68985540916 + 233202.29919636747*$t);
   $X0 += 0.00000000002*cos(5.49174220323 + 21516.52370109911*$t);
   $X0 += 0.00000000002*cos(0.17188282798 + 30659.77021701629*$t);
   $X0 += 0.00000000002*cos(5.30439852095 + 621.49998335569*$t);
   $X0 += 0.00000000002*cos(1.13720629039 + 25764.15390743329*$t);
   $X0 += 0.00000000001*cos(5.32131294248 + 26.5421372833*$t);
   $X0 += 0.00000000001*cos(2.30545913825 + 226796.35601466001*$t);
   $X0 += 0.00000000001*cos(5.44705179184 + 226796.84364962706*$t);
   $X0 += 0.00000000002*cos(0.33446329498 + 28839.69455874929*$t);
   $X0 += 0.00000000002*cos(5.50347633955 + 102225.97897640108*$t);
   $X0 += 0.00000000002*cos(1.56059892812 + 129971.79453500008*$t);
   $X0 += 0.00000000002*cos(4.06465171282 + 27139.92770727909*$t);
   $X0 += 0.00000000001*cos(2.62468643294 + 50807.38984778689*$t);
   $X0 += 0.00000000001*cos(6.16306543539 + 78187.19953596349*$t);
   $X0 += 0.00000000002*cos(5.06371993416 + 51472.41691601449*$t);
   return $X0;
}



   function Mercury_X1($t) // 1023 terms of order 1
{
   $X1  = 0.00328639517*cos(6.04028758995 + 0.2438174835*$t);
   $X1 += 0.00106107047*cos(5.91538469937 + 52176.0501006319*$t);
   $X1 += 0.00032448440*cos(2.68404164136 + 78263.95324220609*$t);
   $X1 += 0.00009699418*cos(5.42935843059 + 26087.65932409069*$t);
   $X1 += 0.00008808490*cos(5.73515025839 + 104351.85638378029*$t);
   $X1 += 0.00004833058*cos(0.21935428474 + 26088.1469590577*$t);
   $X1 += 0.00002334329*cos(2.50289115433 + 130439.75952535449*$t);
   $X1 += 0.00000890250*cos(2.61758665086 + 52175.56246566489*$t);
   $X1 += 0.00000615643*cos(5.55374456387 + 156527.66266692868*$t);
   $X1 += 0.00000147225*cos(6.09730240676 + 78263.46560723908*$t);
   $X1 += 0.00000162460*cos(2.32137690587 + 182615.56580850288*$t);
   $X1 += 0.00000069502*cos(2.91939587909 + 24978.7684069643*$t);
   $X1 += 0.00000063529*cos(2.74219954754 + 27197.5255111511*$t);
   $X1 += 0.00000042967*cos(5.37217235288 + 208703.46895007708*$t);
   $X1 += 0.00000032514*cos(3.18436046111 + 104351.36874881329*$t);
   $X1 += 0.00000017597*cos(5.97213886087 + 51066.6715485385*$t);
   $X1 += 0.00000013287*cos(5.79146748296 + 53285.4286527253*$t);
   $X1 += 0.00000011758*cos(0.18693736842 + 1109.1347346099*$t);
   $X1 += 0.00000012502*cos(1.40596439354 + 1059.6257476727*$t);
   $X1 += 0.00000011468*cos(0.35008608561 + 20426.8149099055*$t);
   $X1 += 0.00000011394*cos(2.13977164486 + 234791.37209165128*$t);
   $X1 += 0.00000008221*cos(5.31229385932 + 31749.4790082099*$t);
   $X1 += 0.00000008067*cos(0.15951495713 + 130439.27189038748*$t);
   $X1 += 0.00000006176*cos(5.46098436501 + 4552.1973145423*$t);
   $X1 += 0.00000004784*cos(0.26415468148 + 21536.1934619989*$t);
   $X1 += 0.00000004858*cos(2.74077664990 + 77154.57469011268*$t);
   $X1 += 0.00000004269*cos(3.39637722107 + 46514.7180514797*$t);
   $X1 += 0.00000004412*cos(5.10134190424 + 25132.5472174491*$t);
   $X1 += 0.00000003922*cos(2.37978972370 + 1589.3167127673*$t);
   $X1 += 0.00000004017*cos(4.41672907758 + 27147.5288892469*$t);
   $X1 += 0.00000003763*cos(4.17909693196 + 14765.48286075331*$t);
   $X1 += 0.00000004285*cos(4.27106878562 + 51116.6681704427*$t);
   $X1 += 0.00000003901*cos(0.54893072973 + 27043.7467006663*$t);
   $X1 += 0.00000003221*cos(1.09802577229 + 26617.8379241523*$t);
   $X1 += 0.00000003080*cos(4.68247133566 + 25558.4559939631*$t);
   $X1 += 0.00000003030*cos(5.19056049817 + 260879.27523322542*$t);
   $X1 += 0.00000002907*cos(0.23044150312 + 26068.47719815791*$t);
   $X1 += 0.00000002906*cos(5.43307929185 + 26107.81671995749*$t);
   $X1 += 0.00000002383*cos(2.07830748691 + 26084.2656236997*$t);
   $X1 += 0.00000002381*cos(3.58525530417 + 26092.0282944157*$t);
   $X1 += 0.00000002069*cos(1.48127323869 + 37410.8110573621*$t);
   $X1 += 0.00000002028*cos(5.44501246598 + 30640.1004561165*$t);
   $X1 += 0.00000002088*cos(3.34536117513 + 156527.17503196168*$t);
   $X1 += 0.00000001998*cos(2.50305248201 + 79373.33179429949*$t);
   $X1 += 0.00000001880*cos(0.22580120983 + 47624.0966035731*$t);
   $X1 += 0.00000001580*cos(1.69158612681 + 9104.1508116011*$t);
   $X1 += 0.00000001575*cos(4.31668154619 + 955.3559241251*$t);
   $X1 += 0.00000001508*cos(0.43559873501 + 19317.43635781211*$t);
   $X1 += 0.00000001397*cos(0.16380111329 + 72602.62119305388*$t);
   $X1 += 0.00000001363*cos(3.28693470602 + 50586.97720534809*$t);
   $X1 += 0.00000001495*cos(3.77822350663 + 12566.3955174663*$t);
   $X1 += 0.00000001401*cos(2.04862276452 + 426.8420083595*$t);
   $X1 += 0.00000001215*cos(3.93785769383 + 52705.7410657265*$t);
   $X1 += 0.00000001310*cos(2.48605199154 + 26095.2605060585*$t);
   $X1 += 0.00000001272*cos(3.22244702438 + 26081.03341205689*$t);
   $X1 += 0.00000001476*cos(2.99369021823 + 23869.3898548709*$t);
   $X1 += 0.00000001169*cos(5.49111664316 + 3442.81876244891*$t);
   $X1 += 0.00000001103*cos(1.21903918539 + 53235.43203082109*$t);
   $X1 += 0.00000001375*cos(5.79233621902 + 103242.47783168689*$t);
   $X1 += 0.00000001244*cos(1.82501834169 + 51220.4503590233*$t);
   $X1 += 0.00000001238*cos(2.66650381891 + 28306.90406324449*$t);
   $X1 += 0.00000000967*cos(3.57175258487 + 53131.6498422405*$t);
   $X1 += 0.00000000986*cos(5.22314307595 + 32858.8575603033*$t);
   $X1 += 0.00000000911*cos(4.29268130103 + 529.44714761109*$t);
   $X1 += 0.00000000859*cos(0.94524893916 + 40853.3860023275*$t);
   $X1 += 0.00000000855*cos(5.33250267421 + 27677.2198543415*$t);
   $X1 += 0.00000000844*cos(1.03648635199 + 25874.8478636197*$t);
   $X1 += 0.00000001042*cos(0.19040742290 + 22645.57201409229*$t);
   $X1 += 0.00000000924*cos(0.74224500219 + 640.1411037975*$t);
   $X1 += 0.00000000810*cos(4.64191507416 + 26301.4460544957*$t);
   $X1 += 0.00000000807*cos(1.95815243056 + 286967.17837479962*$t);
   $X1 += 0.00000000875*cos(2.13143145763 + 83925.28529135829*$t);
   $X1 += 0.00000000874*cos(5.55044306137 + 529.9347825781*$t);
   $X1 += 0.00000000821*cos(4.77032702914 + 36301.4325052687*$t);
   $X1 += 0.00000000823*cos(5.19902587368 + 11322.42028082089*$t);
   $X1 += 0.00000000738*cos(2.19981448222 + 56728.0035976907*$t);
   $X1 += 0.00000000724*cos(1.50055745214 + 18849.4713674577*$t);
   $X1 += 0.00000000714*cos(2.67898291505 + 6770.46678376209*$t);
   $X1 += 0.00000000667*cos(3.97361924647 + 43072.1431065143*$t);
   $X1 += 0.00000000665*cos(3.28272656551 + 52156.38033973211*$t);
   $X1 += 0.00000000893*cos(0.42706797774 + 27197.03787618409*$t);
   $X1 += 0.00000000752*cos(1.88765944457 + 39609.8984006491*$t);
   $X1 += 0.00000000673*cos(4.81316066279 + 1052.51220067191*$t);
   $X1 += 0.00000000688*cos(0.86035048942 + 15874.8614128467*$t);
   $X1 += 0.00000000618*cos(2.12715472204 + 5661.5758666357*$t);
   $X1 += 0.00000000808*cos(4.33645956081 + 1066.7392946735*$t);
   $X1 += 0.00000000616*cos(0.70456047849 + 51646.3591355373*$t);
   $X1 += 0.00000000650*cos(1.39099602371 + 25028.7650288685*$t);
   $X1 += 0.00000000550*cos(0.20488240066 + 182615.07817353586*$t);
   $X1 += 0.00000000554*cos(3.01484203719 + 2119.00767786191*$t);
   $X1 += 0.00000000535*cos(0.35919946298 + 52179.9314359899*$t);
   $X1 += 0.00000000657*cos(5.49249173952 + 29530.7219040231*$t);
   $X1 += 0.00000000527*cos(4.93321108764 + 26514.7451499337*$t);
   $X1 += 0.00000000502*cos(0.52469020589 + 38654.2986590405*$t);
   $X1 += 0.00000000568*cos(0.32194599748 + 24978.28077199729*$t);
   $X1 += 0.00000000498*cos(4.51537827269 + 63498.71419893629*$t);
   $X1 += 0.00000000469*cos(5.15675930919 + 110013.18843293248*$t);
   $X1 += 0.00000000521*cos(2.19480576214 + 52195.71986153169*$t);
   $X1 += 0.00000000440*cos(3.21479581797 + 98690.52433462809*$t);
   $X1 += 0.00000000580*cos(6.26369012947 + 7.3573644843*$t);
   $X1 += 0.00000000438*cos(4.12030207268 + 33326.8225506577*$t);
   $X1 += 0.00000000530*cos(0.13282096675 + 2218.51328670329*$t);
   $X1 += 0.00000000483*cos(3.64743298930 + 51749.45190975589*$t);
   $X1 += 0.00000000434*cos(5.12850749811 + 52172.1687652739*$t);
   $X1 += 0.00000000408*cos(1.65432828623 + 7238.4317741165*$t);
   $X1 += 0.00000000390*cos(0.61831253528 + 78793.64420730069*$t);
   $X1 += 0.00000000374*cos(2.01855068333 + 58946.76070187749*$t);
   $X1 += 0.00000000404*cos(6.01175659056 + 52168.93655363109*$t);
   $X1 += 0.00000000356*cos(1.51807956509 + 62389.33564684289*$t);
   $X1 += 0.00000000373*cos(0.98588361546 + 103292.47445359109*$t);
   $X1 += 0.00000000326*cos(4.29903268783 + 79323.33517239528*$t);
   $X1 += 0.00000000393*cos(2.56051977629 + 129330.38097326108*$t);
   $X1 += 0.00000000385*cos(5.51554890357 + 52183.1636476327*$t);
   $X1 += 0.00000000316*cos(6.11147866851 + 76674.88034692229*$t);
   $X1 += 0.00000000390*cos(5.74481892138 + 54394.80720481869*$t);
   $X1 += 0.00000000323*cos(6.04898382500 + 49957.2929964451*$t);
   $X1 += 0.00000000299*cos(1.43100773503 + 16983.75232997309*$t);
   $X1 += 0.00000000328*cos(4.04864136289 + 68050.66769599509*$t);
   $X1 += 0.00000000314*cos(4.92007418019 + 51536.15281431789*$t);
   $X1 += 0.00000000367*cos(4.85256595117 + 77308.35350059748*$t);
   $X1 += 0.00000000316*cos(3.49203616890 + 45405.3394993863*$t);
   $X1 += 0.00000000319*cos(4.23988546277 + 13656.10430865991*$t);
   $X1 += 0.00000000266*cos(0.22278100987 + 48733.47515566649*$t);
   $X1 += 0.00000000289*cos(6.05400760825 + 6.86972951729*$t);
   $X1 += 0.00000000261*cos(4.00184287590 + 16984.2399649401*$t);
   $X1 += 0.00000000261*cos(1.49214547675 + 52389.3491960699*$t);
   $X1 += 0.00000000233*cos(0.84231286084 + 51123.7817174435*$t);
   $X1 += 0.00000000231*cos(5.24319932827 + 82815.90673926489*$t);
   $X1 += 0.00000000239*cos(2.80376783399 + 213.5429129215*$t);
   $X1 += 0.00000000222*cos(3.99420764167 + 66941.2891439017*$t);
   $X1 += 0.00000000216*cos(5.00937403559 + 313055.08151637383*$t);
   $X1 += 0.00000000276*cos(1.32498030782 + 51109.55462344189*$t);
   $X1 += 0.00000000222*cos(3.36848698260 + 32371.2228090491*$t);
   $X1 += 0.00000000198*cos(2.05297081511 + 53765.1229959157*$t);
   $X1 += 0.00000000197*cos(0.74093273246 + 69160.04624808849*$t);
   $X1 += 0.00000000196*cos(2.65111539002 + 50057.2862402535*$t);
   $X1 += 0.00000000202*cos(4.02603981938 + 129380.37759516528*$t);
   $X1 += 0.00000000201*cos(4.46760354980 + 44937.3745090319*$t);
   $X1 += 0.00000000216*cos(2.15128003627 + 213.0552779545*$t);
   $X1 += 0.00000000185*cos(1.91579542249 + 136101.09157450669*$t);
   $X1 += 0.00000000183*cos(2.26311522221 + 419.72846135871*$t);
   $X1 += 0.00000000186*cos(1.00434945598 + 57837.3821497841*$t);
   $X1 += 0.00000000178*cos(5.27169711768 + 105461.23493587368*$t);
   $X1 += 0.00000000177*cos(3.75548067168 + 51962.7510051939*$t);
   $X1 += 0.00000000170*cos(0.05181048718 + 78244.28348130629*$t);
   $X1 += 0.00000000229*cos(1.74351509136 + 433.9555553603*$t);
   $X1 += 0.00000000173*cos(0.79032586326 + 94138.57083756929*$t);
   $X1 += 0.00000000157*cos(3.59089906997 + 64742.2018006147*$t);
   $X1 += 0.00000000153*cos(0.87597586259 + 59414.7256922319*$t);
   $X1 += 0.00000000159*cos(6.16899315901 + 853.4401992355*$t);
   $X1 += 0.00000000155*cos(4.68975805923 + 35192.0539531753*$t);
   $X1 += 0.00000000149*cos(0.33674359016 + 24499.0740637739*$t);
   $X1 += 0.00000000152*cos(6.14951984841 + 4551.7096795753*$t);
   $X1 += 0.00000000147*cos(1.70740828389 + 52602.6482915079*$t);
   $X1 += 0.00000000147*cos(2.15567882351 + 19805.0711090663*$t);
   $X1 += 0.00000000182*cos(2.25669630329 + 55618.6250455973*$t);
   $X1 += 0.00000000140*cos(0.10429379711 + 23754.95056618569*$t);
   $X1 += 0.00000000146*cos(3.32146505695 + 208702.98131511007*$t);
   $X1 += 0.00000000154*cos(3.39064691795 + 99799.90288672148*$t);
   $X1 += 0.00000000139*cos(6.20098571989 + 25973.70767037247*$t);
   $X1 += 0.00000000138*cos(5.74451336242 + 26202.58624774291*$t);
   $X1 += 0.00000000191*cos(3.87484611458 + 26728.0442453717*$t);
   $X1 += 0.00000000152*cos(5.74228778845 + 1109.6223695769*$t);
   $X1 += 0.00000000162*cos(1.42094114156 + 38520.1896094555*$t);
   $X1 += 0.00000000141*cos(1.24257990116 + 77734.26227711148*$t);
   $X1 += 0.00000000135*cos(3.41542832234 + 78267.83457756409*$t);
   $X1 += 0.00000000128*cos(4.56986923193 + 88477.23878841709*$t);
   $X1 += 0.00000000147*cos(5.33327922722 + 31748.99137324289*$t);
   $X1 += 0.00000000135*cos(6.26697445915 + 124778.42747620228*$t);
   $X1 += 0.00000000148*cos(1.14618449835 + 27154.6424362477*$t);
   $X1 += 0.00000000134*cos(5.16840701859 + 12431.79883291429*$t);
   $X1 += 0.00000000121*cos(3.61695346173 + 104881.54734887488*$t);
   $X1 += 0.00000000116*cos(4.76247090725 + 25934.3681485729*$t);
   $X1 += 0.00000000116*cos(2.64947988255 + 32858.36992533629*$t);
   $X1 += 0.00000000115*cos(0.89972324521 + 26241.9257695425*$t);
   $X1 += 0.00000000148*cos(1.02099168260 + 25661.5487681817*$t);
   $X1 += 0.00000000110*cos(5.99966246087 + 28206.9108194361*$t);
   $X1 += 0.00000000108*cos(1.68778367228 + 27140.41534224609*$t);
   $X1 += 0.00000000115*cos(4.01860742052 + 103822.16541868569*$t);
   $X1 += 0.00000000109*cos(3.92589684326 + 53284.94101775829*$t);
   $X1 += 0.00000000107*cos(5.56945579461 + 28421.3433519297*$t);
   $X1 += 0.00000000143*cos(4.93475595866 + 1596.43025976811*$t);
   $X1 += 0.00000000112*cos(5.60947833730 + 155418.28411483529*$t);
   $X1 += 0.00000000113*cos(2.46236993488 + 5327.7199258663*$t);
   $X1 += 0.00000000101*cos(3.51555637450 + 25620.1819687033*$t);
   $X1 += 0.00000000122*cos(3.03742531641 + 10213.5293636945*$t);
   $X1 += 0.00000000102*cos(5.56482383222 + 2333.44021035551*$t);
   $X1 += 0.00000000097*cos(2.14556254447 + 26556.11194941211*$t);
   $X1 += 0.00000000118*cos(1.76061169391 + 7994.77225950771*$t);
   $X1 += 0.00000000109*cos(1.60651799859 + 103396.25664217169*$t);
   $X1 += 0.00000000109*cos(6.11285140847 + 79219.55298381469*$t);
   $X1 += 0.00000000095*cos(0.10552520847 + 58459.1259506233*$t);
   $X1 += 0.00000000095*cos(2.74369425701 + 78256.83969520529*$t);
   $X1 += 0.00000000097*cos(1.08624310756 + 105411.23831396949*$t);
   $X1 += 0.00000000092*cos(2.26275916512 + 10213.0417287275*$t);
   $X1 += 0.00000000094*cos(4.41890699093 + 1582.2031657665*$t);
   $X1 += 0.00000000103*cos(4.94789948755 + 91785.70468379749*$t);
   $X1 += 0.00000000101*cos(3.48509343222 + 20426.32727493849*$t);
   $X1 += 0.00000000089*cos(3.24400432046 + 74821.37829724069*$t);
   $X1 += 0.00000000089*cos(0.10163739667 + 125887.80602829569*$t);
   $X1 += 0.00000000103*cos(3.50686067995 + 51066.18391357149*$t);
   $X1 += 0.00000000087*cos(4.36360162781 + 27043.2590656993*$t);
   $X1 += 0.00000000083*cos(2.62141457558 + 73711.99974514729*$t);
   $X1 += 0.00000000083*cos(1.77797295395 + 77624.05595589208*$t);
   $X1 += 0.00000000097*cos(5.36704254243 + 71980.87739221469*$t);
   $X1 += 0.00000000081*cos(0.79181721859 + 155468.28073673949*$t);
   $X1 += 0.00000000080*cos(4.00356912047 + 22645.08437912529*$t);
   $X1 += 0.00000000081*cos(2.81631624727 + 76045.1961380193*$t);
   $X1 += 0.00000000081*cos(5.04291780878 + 26138.1435809619*$t);
   $X1 += 0.00000000080*cos(0.61687993534 + 26038.1503371535*$t);
   $X1 += 0.00000000104*cos(2.28163680118 + 78271.06678920689*$t);
   $X1 += 0.00000000103*cos(5.23374561959 + 78283.62300310588*$t);
   $X1 += 0.00000000083*cos(2.50636376987 + 80482.71034639288*$t);
   $X1 += 0.00000000089*cos(1.89337139570 + 78260.07190684808*$t);
   $X1 += 0.00000000073*cos(4.21987024947 + 26617.35028918529*$t);
   $X1 += 0.00000000077*cos(0.51899049759 + 18208.05780571871*$t);
   $X1 += 0.00000000082*cos(3.91217027287 + 77211.68485901768*$t);
   $X1 += 0.00000000077*cos(2.88917318730 + 41962.7645544209*$t);
   $X1 += 0.00000000082*cos(4.38267763386 + 77197.45776501608*$t);
   $X1 += 0.00000000067*cos(5.02424385988 + 85034.66384345168*$t);
   $X1 += 0.00000000070*cos(1.09572627877 + 537.0483295789*$t);
   $X1 += 0.00000000067*cos(0.33161014347 + 5661.0882316687*$t);
   $X1 += 0.00000000067*cos(3.82822220045 + 120226.47397914348*$t);
   $X1 += 0.00000000065*cos(4.95852563263 + 162188.99471608088*$t);
   $X1 += 0.00000000075*cos(4.93326519789 + 149.8070146181*$t);
   $X1 += 0.00000000071*cos(2.00854094820 + 108903.80988083909*$t);
   $X1 += 0.00000000063*cos(3.29036116867 + 51756.5654567567*$t);
   $X1 += 0.00000000062*cos(2.43463735809 + 102762.78348849648*$t);
   $X1 += 0.00000000067*cos(4.50074668009 + 64608.09275102969*$t);
   $X1 += 0.00000000064*cos(5.16392570404 + 13521.9952590749*$t);
   $X1 += 0.00000000062*cos(0.12968651638 + 28306.41642827749*$t);
   $X1 += 0.00000000058*cos(1.62746286468 + 11322.9079157879*$t);
   $X1 += 0.00000000058*cos(0.27966370230 + 24498.58642880689*$t);
   $X1 += 0.00000000077*cos(3.79522760165 + 25565.5695409639*$t);
   $X1 += 0.00000000076*cos(4.60149373597 + 78477.25233764409*$t);
   $X1 += 0.00000000063*cos(0.77089662682 + 93029.19228547589*$t);
   $X1 += 0.00000000078*cos(0.26117589680 + 71493.2426409605*$t);
   $X1 += 0.00000000077*cos(3.91821281814 + 51742.33836275509*$t);
   $X1 += 0.00000000057*cos(3.59228773257 + 2648.6986429565*$t);
   $X1 += 0.00000000058*cos(1.77431779111 + 339142.98465794802*$t);
   $X1 += 0.00000000056*cos(5.53301773648 + 76145.18938182769*$t);
   $X1 += 0.00000000059*cos(5.94806683967 + 21535.70582703189*$t);
   $X1 += 0.00000000056*cos(1.70277932228 + 117873.60782537168*$t);
   $X1 += 0.00000000053*cos(4.64718015139 + 25557.96835899609*$t);
   $X1 += 0.00000000060*cos(1.21018656116 + 71025.27765060609*$t);
   $X1 += 0.00000000052*cos(0.56762366419 + 10022.0810975829*$t);
   $X1 += 0.00000000055*cos(5.77897482747 + 51322.85371887989*$t);
   $X1 += 0.00000000058*cos(1.40115688303 + 43071.65547154729*$t);
   $X1 += 0.00000000053*cos(5.07195556849 + 79853.02613748988*$t);
   $X1 += 0.00000000053*cos(0.21642667454 + 78050.65414676809*$t);
   $X1 += 0.00000000053*cos(2.16306876234 + 98068.78053378889*$t);
   $X1 += 0.00000000057*cos(1.24363255483 + 633.0275567967*$t);
   $X1 += 0.00000000049*cos(0.37407469111 + 90830.10494218889*$t);
   $X1 += 0.00000000067*cos(1.84374645383 + 26610.72437715149*$t);
   $X1 += 0.00000000050*cos(6.00585950729 + 131549.13807744789*$t);
   $X1 += 0.00000000049*cos(5.14290001765 + 33968.23611239669*$t);
   $X1 += 0.00000000050*cos(5.30537322308 + 81706.5281871715*$t);
   $X1 += 0.00000000051*cos(0.73226920314 + 129910.06856025988*$t);
   $X1 += 0.00000000057*cos(3.56358445708 + 26624.9514711531*$t);
   $X1 += 0.00000000045*cos(3.36854845214 + 51116.18053547569*$t);
   $X1 += 0.00000000045*cos(3.10088248442 + 104332.1866228805*$t);
   $X1 += 0.00000000051*cos(0.85422591600 + 647.25465079831*$t);
   $X1 += 0.00000000043*cos(4.78281150158 + 78690.55143308209*$t);
   $X1 += 0.00000000054*cos(2.25324121444 + 522.8212355773*$t);
   $X1 += 0.00000000042*cos(1.33828179439 + 114565.14192999128*$t);
   $X1 += 0.00000000041*cos(5.13406465270 + 38519.70197448849*$t);
   $X1 += 0.00000000041*cos(3.95146540461 + 25028.27739390149*$t);
   $X1 += 0.00000000052*cos(1.00660903298 + 39744.0074502341*$t);
   $X1 += 0.00000000043*cos(0.49413326170 + 103925.25819290428*$t);
   $X1 += 0.00000000045*cos(4.58276908488 + 76.50988875911*$t);
   $X1 += 0.00000000042*cos(3.21789877684 + 46848.5739922491*$t);
   $X1 += 0.00000000047*cos(3.90827664098 + 44181.52165860769*$t);
   $X1 += 0.00000000038*cos(3.08592038431 + 22760.01130277751*$t);
   $X1 += 0.00000000037*cos(3.12927309026 + 151975.70916986988*$t);
   $X1 += 0.00000000050*cos(2.23521826581 + 25551.34244696229*$t);
   $X1 += 0.00000000050*cos(0.72721768819 + 50579.86365834729*$t);
   $X1 += 0.00000000045*cos(2.61026174635 + 7879.84533585549*$t);
   $X1 += 0.00000000043*cos(4.81034248928 + 26521.8586969345*$t);
   $X1 += 0.00000000037*cos(2.72331919328 + 30639.61282114949*$t);
   $X1 += 0.00000000037*cos(3.88891756517 + 48732.98752069949*$t);
   $X1 += 0.00000000035*cos(3.18195761886 + 84547.0290921975*$t);
   $X1 += 0.00000000040*cos(0.70421187083 + 52815.9473869459*$t);
   $X1 += 0.00000000041*cos(4.14956775757 + 1059.1381127057*$t);
   $X1 += 0.00000000041*cos(3.03500918971 + 150866.33061777649*$t);
   $X1 += 0.00000000040*cos(5.01484347149 + 26294.33250749489*$t);
   $X1 += 0.00000000041*cos(0.58688251264 + 25881.9614106205*$t);
   $X1 += 0.00000000034*cos(0.93821904321 + 26191.23973327629*$t);
   $X1 += 0.00000000034*cos(4.73103289822 + 25985.0541848391*$t);
   $X1 += 0.00000000034*cos(0.83186142058 + 89586.61734051049*$t);
   $X1 += 0.00000000041*cos(4.17323142245 + 52329.8289111167*$t);
   $X1 += 0.00000000035*cos(2.52595229567 + 157637.04121902208*$t);
   $X1 += 0.00000000042*cos(6.02261134030 + 26507.63160293289*$t);
   $X1 += 0.00000000039*cos(0.14372316696 + 234790.88445668427*$t);
   $X1 += 0.00000000037*cos(1.88707685753 + 57836.89451481709*$t);
   $X1 += 0.00000000036*cos(0.34370159925 + 130969.45049044909*$t);
   $X1 += 0.00000000032*cos(2.37775558754 + 181506.18725640948*$t);
   $X1 += 0.00000000033*cos(1.24575523402 + 50594.09075234889*$t);
   $X1 += 0.00000000044*cos(6.11332634879 + 6681.46867088311*$t);
   $X1 += 0.00000000036*cos(0.18721130545 + 104355.73771913828*$t);
   $X1 += 0.00000000032*cos(4.64549491762 + 129484.15978374588*$t);
   $X1 += 0.00000000030*cos(5.20724500900 + 52644.0150909863*$t);
   $X1 += 0.00000000031*cos(2.56009147472 + 29416.28261533789*$t);
   $X1 += 0.00000000031*cos(1.30643526573 + 105307.45612538888*$t);
   $X1 += 0.00000000030*cos(3.71640911346 + 95247.94938966268*$t);
   $X1 += 0.00000000031*cos(2.51344352354 + 52290.48938931711*$t);
   $X1 += 0.00000000029*cos(3.83470620242 + 181556.18387831368*$t);
   $X1 += 0.00000000039*cos(1.79502663396 + 25448.2496727437*$t);
   $X1 += 0.00000000032*cos(4.74231453232 + 51639.24558853649*$t);
   $X1 += 0.00000000029*cos(4.21913805837 + 53242.5455778219*$t);
   $X1 += 0.00000000029*cos(5.39418594332 + 25668.6623151825*$t);
   $X1 += 0.00000000028*cos(6.18231601923 + 77204.57131201689*$t);
   $X1 += 0.00000000036*cos(2.35417973488 + 25132.0595824821*$t);
   $X1 += 0.00000000027*cos(2.33120714494 + 54509.2464935039*$t);
   $X1 += 0.00000000026*cos(2.96005545403 + 52061.61081194667*$t);
   $X1 += 0.00000000025*cos(0.44641100649 + 29428.7593857575*$t);
   $X1 += 0.00000000031*cos(1.36909644374 + 18093.13088206649*$t);
   $X1 += 0.00000000030*cos(2.99318128420 + 26941.3433408097*$t);
   $X1 += 0.00000000025*cos(0.01970755306 + 956.53297345411*$t);
   $X1 += 0.00000000027*cos(3.92481990123 + 18093.6185170335*$t);
   $X1 += 0.00000000028*cos(5.33056586298 + 104358.96993078108*$t);
   $X1 += 0.00000000026*cos(1.68523834841 + 26237.7101561923*$t);
   $X1 += 0.00000000024*cos(5.34360042933 + 22747.5345323579*$t);
   $X1 += 0.00000000024*cos(0.60759776379 + 46514.23041651269*$t);
   $X1 += 0.00000000029*cos(4.14852842199 + 131499.14145554368*$t);
   $X1 += 0.00000000024*cos(5.11295227085 + 42154.2128205325*$t);
   $X1 += 0.00000000024*cos(2.72205818316 + 54294.81396101029*$t);
   $X1 += 0.00000000023*cos(0.59370019139 + 146314.37712071768*$t);
   $X1 += 0.00000000024*cos(3.26900809848 + 26107.32908499049*$t);
   $X1 += 0.00000000026*cos(4.45438431206 + 25021.6514818677*$t);
   $X1 += 0.00000000023*cos(0.68577711601 + 20760.1832157079*$t);
   $X1 += 0.00000000023*cos(0.40104725527 + 77154.08705514569*$t);
   $X1 += 0.00000000024*cos(4.07999331304 + 26067.9895631909*$t);
   $X1 += 0.00000000023*cos(3.75077334681 + 25035.8785758693*$t);
   $X1 += 0.00000000023*cos(4.73851847693 + 143961.51096694589*$t);
   $X1 += 0.00000000026*cos(1.72805748634 + 27684.3334013423*$t);
   $X1 += 0.00000000022*cos(3.46952873618 + 19804.5834740993*$t);
   $X1 += 0.00000000025*cos(6.21468652611 + 26080.54577708989*$t);
   $X1 += 0.00000000023*cos(3.49763456236 + 130013.16133447849*$t);
   $X1 += 0.00000000022*cos(0.73153517786 + 79372.84415933248*$t);
   $X1 += 0.00000000025*cos(0.72123980539 + 52026.4869034973*$t);
   $X1 += 0.00000000021*cos(5.79201875326 + 104344.74283677948*$t);
   $X1 += 0.00000000021*cos(5.87156389227 + 102133.09927959349*$t);
   $X1 += 0.00000000023*cos(1.39284020284 + 61279.9570947495*$t);
   $X1 += 0.00000000021*cos(2.36405581509 + 85502.6288338061*$t);
   $X1 += 0.00000000023*cos(0.10713155060 + 77844.46859833089*$t);
   $X1 += 0.00000000022*cos(1.72894324326 + 188276.89785765509*$t);
   $X1 += 0.00000000021*cos(5.21984877761 + 124156.68367536309*$t);
   $X1 += 0.00000000027*cos(0.68325003204 + 103299.58800059189*$t);
   $X1 += 0.00000000020*cos(5.77389955341 + 23968.89546371229*$t);
   $X1 += 0.00000000023*cos(0.69010991788 + 77830.24150432929*$t);
   $X1 += 0.00000000021*cos(1.14745744394 + 103285.36090659029*$t);
   $X1 += 0.00000000020*cos(2.07158691051 + 49527.59527515889*$t);
   $X1 += 0.00000000020*cos(2.29184774399 + 1479.11039154791*$t);
   $X1 += 0.00000000020*cos(5.92355125832 + 23969.3830986793*$t);
   $X1 += 0.00000000020*cos(4.95913484029 + 103711.95909746629*$t);
   $X1 += 0.00000000019*cos(3.65190813330 + 52126.05347872769*$t);
   $X1 += 0.00000000019*cos(0.04924351217 + 3327.89183879669*$t);
   $X1 += 0.00000000022*cos(1.42243873408 + 104565.15547921829*$t);
   $X1 += 0.00000000026*cos(1.25974103229 + 13362.6935242827*$t);
   $X1 += 0.00000000025*cos(0.01235262119 + 24864.32911827909*$t);
   $X1 += 0.00000000023*cos(4.22016173229 + 15874.37377787969*$t);
   $X1 += 0.00000000022*cos(4.72905553277 + 6283.3196674749*$t);
   $X1 += 0.00000000025*cos(5.64927135524 + 27311.96479983631*$t);
   $X1 += 0.00000000019*cos(4.78388065302 + 53228.31848382029*$t);
   $X1 += 0.00000000021*cos(3.31277231518 + 97581.14578253469*$t);
   $X1 += 0.00000000018*cos(5.56704384876 + 106570.61348796709*$t);
   $X1 += 0.00000000021*cos(1.92247469083 + 60056.13925397089*$t);
   $X1 += 0.00000000019*cos(6.17553462363 + 26083.7779887327*$t);
   $X1 += 0.00000000021*cos(1.98793084523 + 104371.52614468009*$t);
   $X1 += 0.00000000017*cos(2.47269230880 + 1162.7185218913*$t);
   $X1 += 0.00000000020*cos(4.30765634281 + 41962.27691945389*$t);
   $X1 += 0.00000000020*cos(1.17324939187 + 26091.5406594487*$t);
   $X1 += 0.00000000017*cos(2.58918808487 + 77410.75686045409*$t);
   $X1 += 0.00000000018*cos(1.56564486854 + 115674.52048208468*$t);
   $X1 += 0.00000000021*cos(5.05205604193 + 134991.71302241329*$t);
   $X1 += 0.00000000017*cos(3.10172524257 + 104138.55728834229*$t);
   $X1 += 0.00000000017*cos(5.66948357862 + 37410.32342239509*$t);
   $X1 += 0.00000000019*cos(5.53464997520 + 522.3336006103*$t);
   $X1 += 0.00000000019*cos(4.91833775610 + 52698.62751872569*$t);
   $X1 += 0.00000000019*cos(3.92340038849 + 131395.35926696309*$t);
   $X1 += 0.00000000016*cos(4.33041124879 + 12546.72575656651*$t);
   $X1 += 0.00000000016*cos(6.22611768107 + 100909.28143881488*$t);
   $X1 += 0.00000000016*cos(0.82606858430 + 25867.73431661889*$t);
   $X1 += 0.00000000020*cos(5.89802475411 + 26094.77287109149*$t);
   $X1 += 0.00000000018*cos(5.08722928383 + 47623.60896860609*$t);
   $X1 += 0.00000000018*cos(1.45687313804 + 12432.2864678813*$t);
   $X1 += 0.00000000020*cos(6.13261748755 + 11610.7957758577*$t);
   $X1 += 0.00000000015*cos(5.51634191387 + 183724.94436059629*$t);
   $X1 += 0.00000000017*cos(4.79626245128 + 51529.03926731709*$t);
   $X1 += 0.00000000020*cos(1.77003688612 + 426.3543733925*$t);
   $X1 += 0.00000000016*cos(4.90993271420 + 26308.5596014965*$t);
   $X1 += 0.00000000016*cos(4.79764512586 + 128850.68663007069*$t);
   $X1 += 0.00000000019*cos(4.56495311793 + 51543.2663613187*$t);
   $X1 += 0.00000000019*cos(3.74798808655 + 155997.97170183409*$t);
   $X1 += 0.00000000015*cos(1.34224435786 + 26164.4130303333*$t);
   $X1 += 0.00000000018*cos(3.66440627266 + 25874.36022865269*$t);
   $X1 += 0.00000000014*cos(2.04202638738 + 36300.94487030169*$t);
   $X1 += 0.00000000019*cos(5.88442577075 + 45494.82524723229*$t);
   $X1 += 0.00000000014*cos(3.57916105055 + 44295.9609472929*$t);
   $X1 += 0.00000000018*cos(3.82094554949 + 119117.0954270501*$t);
   $X1 += 0.00000000016*cos(3.76272474790 + 76667.76679992149*$t);
   $X1 += 0.00000000014*cos(2.07119234799 + 107794.43132874569*$t);
   $X1 += 0.00000000014*cos(2.36814985653 + 16065.82204399129*$t);
   $X1 += 0.00000000014*cos(1.33222225896 + 44181.03402364069*$t);
   $X1 += 0.00000000014*cos(4.76386004796 + 2111.8941308611*$t);
   $X1 += 0.00000000015*cos(1.74258697688 + 52226.0467225361*$t);
   $X1 += 0.00000000014*cos(4.76443919805 + 27999.3464422749*$t);
   $X1 += 0.00000000019*cos(4.94519146654 + 104347.97504842229*$t);
   $X1 += 0.00000000014*cos(3.43033111839 + 116918.00808376308*$t);
   $X1 += 0.00000000015*cos(4.82510142643 + 365230.88779952223*$t);
   $X1 += 0.00000000016*cos(0.25720185897 + 52712.85461272729*$t);
   $X1 += 0.00000000017*cos(4.24342377575 + 97113.18079218028*$t);
   $X1 += 0.00000000013*cos(0.92750711335 + 24176.9474758405*$t);
   $X1 += 0.00000000017*cos(3.10108098380 + 20760.6708506749*$t);
   $X1 += 0.00000000014*cos(0.49576253379 + 25954.0379094727*$t);
   $X1 += 0.00000000014*cos(5.16786249742 + 26222.2560086427*$t);
   $X1 += 0.00000000017*cos(0.70506585057 + 70269.42480018188*$t);
   $X1 += 0.00000000013*cos(6.17002721316 + 178063.61231144409*$t);
   $X1 += 0.00000000015*cos(1.07987898155 + 52099.7840293563*$t);
   $X1 += 0.00000000013*cos(3.68965474753 + 323.74923414091*$t);
   $X1 += 0.00000000013*cos(1.62156829158 + 39609.41076568209*$t);
   $X1 += 0.00000000013*cos(0.38134163255 + 206.42936592071*$t);
   $X1 += 0.00000000014*cos(1.82134412060 + 105940.92927906409*$t);
   $X1 += 0.00000000013*cos(2.20656460343 + 102233.09252340188*$t);
   $X1 += 0.00000000014*cos(0.61259865461 + 77204.08367704989*$t);
   $X1 += 0.00000000014*cos(0.58031216793 + 13521.50762410789*$t);
   $X1 += 0.00000000013*cos(4.69319019775 + 111590.53197538029*$t);
   $X1 += 0.00000000012*cos(4.27702379029 + 76681.99389392308*$t);
   $X1 += 0.00000000015*cos(1.28652822525 + 27670.10630734069*$t);
   $X1 += 0.00000000013*cos(2.50362342037 + 58946.27306691049*$t);
   $X1 += 0.00000000015*cos(2.00793708614 + 26300.95841952869*$t);
   $X1 += 0.00000000013*cos(3.31492062191 + 14477.1073657165*$t);
   $X1 += 0.00000000013*cos(1.07027376830 + 1692.40948698591*$t);
   $X1 += 0.00000000013*cos(4.39268185141 + 140653.04507156549*$t);
   $X1 += 0.00000000012*cos(1.34267704514 + 52072.9573264133*$t);
   $X1 += 0.00000000012*cos(6.26475550018 + 54394.31956985169*$t);
   $X1 += 0.00000000012*cos(1.17105305254 + 7238.9194090835*$t);
   $X1 += 0.00000000012*cos(5.40922931059 + 25447.76203777669*$t);
   $X1 += 0.00000000012*cos(6.20873329955 + 72936.4771338233*$t);
   $X1 += 0.00000000015*cos(5.49315509340 + 2126.1212248627*$t);
   $X1 += 0.00000000012*cos(5.66744055627 + 55504.18575691209*$t);
   $X1 += 0.00000000012*cos(3.61008361501 + 220.6564599223*$t);
   $X1 += 0.00000000012*cos(1.76003505598 + 111122.56698502589*$t);
   $X1 += 0.00000000012*cos(1.22583703139 + 90695.99589260388*$t);
   $X1 += 0.00000000012*cos(1.56652249391 + 104778.45457465628*$t);
   $X1 += 0.00000000012*cos(5.12697804116 + 64607.60511606269*$t);
   $X1 += 0.00000000012*cos(6.24535629479 + 110634.93223377169*$t);
   $X1 += 0.00000000011*cos(3.57080417047 + 36109.9842391571*$t);
   $X1 += 0.00000000013*cos(0.24854762034 + 49842.85370775989*$t);
   $X1 += 0.00000000014*cos(1.03533656892 + 78417.73205269089*$t);
   $X1 += 0.00000000011*cos(2.67619239702 + 26198.3532802771*$t);
   $X1 += 0.00000000011*cos(2.98743263419 + 25977.94063783829*$t);
   $X1 += 0.00000000012*cos(5.20745322961 + 31415.6230674405*$t);
   $X1 += 0.00000000012*cos(5.36057333309 + 6770.9544187291*$t);
   $X1 += 0.00000000011*cos(4.12038634021 + 51969.8645521947*$t);
   $X1 += 0.00000000010*cos(0.53057090724 + 23868.9022199039*$t);
   $X1 += 0.00000000012*cos(6.15491126193 + 130420.08976445469*$t);
   $X1 += 0.00000000014*cos(2.66131661996 + 32769.3718124573*$t);
   $X1 += 0.00000000013*cos(2.57684550665 + 33967.74847742969*$t);
   $X1 += 0.00000000011*cos(0.63840837632 + 51645.87150057029*$t);
   $X1 += 0.00000000010*cos(0.30214091711 + 28736.6017845307*$t);
   $X1 += 0.00000000010*cos(1.03579590518 + 52705.25343075949*$t);
   $X1 += 0.00000000014*cos(5.20517620734 + 77837.35505133009*$t);
   $X1 += 0.00000000010*cos(4.48203927159 + 141762.42362365888*$t);
   $X1 += 0.00000000012*cos(0.89324430712 + 224.5886131854*$t);
   $X1 += 0.00000000010*cos(1.51553715870 + 170049.41410852008*$t);
   $X1 += 0.00000000013*cos(4.39783434076 + 38813.6003938327*$t);
   $X1 += 0.00000000010*cos(1.45494488062 + 27147.04125427989*$t);
   $X1 += 0.00000000010*cos(3.80634126779 + 78903.8505285201*$t);
   $X1 += 0.00000000013*cos(4.94493446189 + 6282.83203250789*$t);
   $X1 += 0.00000000010*cos(0.60091290149 + 207644.08701988788*$t);
   $X1 += 0.00000000012*cos(0.27544235334 + 26395.7045800273*$t);
   $X1 += 0.00000000012*cos(5.38818267787 + 25780.58933808811*$t);
   $X1 += 0.00000000010*cos(0.84101636752 + 12565.9078824993*$t);
   $X1 += 0.00000000012*cos(4.18148179610 + 26725.1432310819*$t);
   $X1 += 0.00000000012*cos(1.48214323511 + 25451.1506870335*$t);
   $X1 += 0.00000000009*cos(5.62764518642 + 51219.76094466129*$t);
   $X1 += 0.00000000009*cos(1.40970085491 + 155572.06292532009*$t);
   $X1 += 0.00000000009*cos(1.98340877635 + 78731.91823256049*$t);
   $X1 += 0.00000000009*cos(1.03302022359 + 14764.9952257863*$t);
   $X1 += 0.00000000011*cos(3.25069263252 + 65697.80154222329*$t);
   $X1 += 0.00000000012*cos(2.28213947586 + 25446.73339731871*$t);
   $X1 += 0.00000000012*cos(3.38148555535 + 26729.56052079669*$t);
   $X1 += 0.00000000012*cos(1.48681981161 + 12725.20961729149*$t);
   $X1 += 0.00000000010*cos(4.04518614257 + 65831.91059180829*$t);
   $X1 += 0.00000000009*cos(1.98008802000 + 16703.3059509825*$t);
   $X1 += 0.00000000009*cos(0.19460006823 + 51708.08511027749*$t);
   $X1 += 0.00000000009*cos(5.42812340959 + 207594.09039798369*$t);
   $X1 += 0.00000000009*cos(0.25321773369 + 156101.06447605268*$t);
   $X1 += 0.00000000012*cos(6.08676982525 + 176954.23375935068*$t);
   $X1 += 0.00000000009*cos(1.85021451264 + 68242.11596210669*$t);
   $X1 += 0.00000000009*cos(2.19168596252 + 1910.9556657337*$t);
   $X1 += 0.00000000009*cos(1.01460352352 + 50586.48957038109*$t);
   $X1 += 0.00000000009*cos(1.60908201613 + 52609.7618385087*$t);
   $X1 += 0.00000000008*cos(3.02342551822 + 29530.2342690561*$t);
   $X1 += 0.00000000009*cos(2.83792438251 + 205.9417309537*$t);
   $X1 += 0.00000000009*cos(3.48790336223 + 55516.6625273317*$t);
   $X1 += 0.00000000011*cos(2.35570208491 + 52022.2712901471*$t);
   $X1 += 0.00000000011*cos(3.36653560664 + 157057.35363202328*$t);
   $X1 += 0.00000000008*cos(1.31427690069 + 39629.56816154889*$t);
   $X1 += 0.00000000010*cos(1.95365576783 + 52382.23564906909*$t);
   $X1 += 0.00000000008*cos(5.91790300447 + 52182.67865953191*$t);
   $X1 += 0.00000000010*cos(5.89081572614 + 40565.4981422577*$t);
   $X1 += 0.00000000008*cos(4.44673072353 + 103.3365917021*$t);
   $X1 += 0.00000000008*cos(0.60254149340 + 157483.26240853728*$t);
   $X1 += 0.00000000008*cos(0.07994652584 + 27177.8557502513*$t);
   $X1 += 0.00000000008*cos(5.58367850537 + 24998.4381678641*$t);
   $X1 += 0.00000000009*cos(3.23707766893 + 19.42594341629*$t);
   $X1 += 0.00000000010*cos(0.69943263197 + 536.5606946119*$t);
   $X1 += 0.00000000010*cos(3.22927666969 + 260878.78759825832*$t);
   $X1 += 0.00000000008*cos(6.13315569962 + 48847.9144443517*$t);
   $X1 += 0.00000000008*cos(6.23628865773 + 860.55374623631*$t);
   $X1 += 0.00000000008*cos(4.33455466105 + 26720.93069837089*$t);
   $X1 += 0.00000000009*cos(0.87327236491 + 25654.43522118089*$t);
   $X1 += 0.00000000009*cos(4.71644893845 + 63498.22656396929*$t);
   $X1 += 0.00000000008*cos(2.95337805438 + 52595.53474450709*$t);
   $X1 += 0.00000000008*cos(2.09085109942 + 130446.87307235529*$t);
   $X1 += 0.00000000007*cos(3.35858694831 + 50697.1835265675*$t);
   $X1 += 0.00000000008*cos(5.66351050239 + 1224.06165826211*$t);
   $X1 += 0.00000000009*cos(4.15418404000 + 19316.9487228451*$t);
   $X1 += 0.00000000007*cos(5.10075848971 + 13541.17738500769*$t);
   $X1 += 0.00000000009*cos(3.23441439127 + 130443.64086071249*$t);
   $X1 += 0.00000000007*cos(2.30584818184 + 209812.84750217047*$t);
   $X1 += 0.00000000007*cos(0.64823715471 + 9384.59719059169*$t);
   $X1 += 0.00000000007*cos(0.61661379288 + 46848.0863572821*$t);
   $X1 += 0.00000000007*cos(3.64199179530 + 172402.28026229188*$t);
   $X1 += 0.00000000007*cos(0.75404668288 + 846.3266522347*$t);
   $X1 += 0.00000000007*cos(3.05134673809 + 27044.4361150283*$t);
   $X1 += 0.00000000007*cos(1.99335698169 + 150244.58681693728*$t);
   $X1 += 0.00000000009*cos(3.93094161276 + 26735.1577923725*$t);
   $X1 += 0.00000000007*cos(4.75721708287 + 52325.61329776649*$t);
   $X1 += 0.00000000007*cos(1.46600264602 + 45892.9742506405*$t);
   $X1 += 0.00000000008*cos(2.77799241232 + 52169.42154173189*$t);
   $X1 += 0.00000000008*cos(5.13696206564 + 83924.79765639128*$t);
   $X1 += 0.00000000007*cos(5.38422375135 + 80597.14963507809*$t);
   $X1 += 0.00000000006*cos(3.78360025001 + 52279.14287485049*$t);
   $X1 += 0.00000000006*cos(3.14185350605 + 51013.57539622409*$t);
   $X1 += 0.00000000008*cos(3.74001326185 + 129387.49114216608*$t);
   $X1 += 0.00000000009*cos(3.71121213994 + 955.8435590921*$t);
   $X1 += 0.00000000008*cos(1.50347216209 + 77727.14873011068*$t);
   $X1 += 0.00000000006*cos(1.00543740922 + 79330.44871939608*$t);
   $X1 += 0.00000000007*cos(5.56521824483 + 78378.3925308913*$t);
   $X1 += 0.00000000007*cos(5.54385794189 + 51219.96272405629*$t);
   $X1 += 0.00000000006*cos(4.20431417253 + 129373.26404816448*$t);
   $X1 += 0.00000000007*cos(1.63093386109 + 26162.928557625*$t);
   $X1 += 0.00000000006*cos(3.53855282447 + 103241.99019671988*$t);
   $X1 += 0.00000000006*cos(3.70872609374 + 3340.36860921629*$t);
   $X1 += 0.00000000006*cos(3.67291216777 + 35472.9879671329*$t);
   $X1 += 0.00000000006*cos(4.50914445356 + 130653.05862079248*$t);
   $X1 += 0.00000000006*cos(6.20953170755 + 110.45013870291*$t);
   $X1 += 0.00000000007*cos(1.43696950392 + 77631.16950289288*$t);
   $X1 += 0.00000000006*cos(1.84528711156 + 53131.16220727349*$t);
   $X1 += 0.00000000006*cos(4.02329719726 + 26013.3653604904*$t);
   $X1 += 0.00000000006*cos(0.25065144064 + 25508.4593720589*$t);
   $X1 += 0.00000000006*cos(5.41297359057 + 26667.8345460565*$t);
   $X1 += 0.00000000006*cos(1.27873892607 + 70268.93716521488*$t);
   $X1 += 0.00000000006*cos(3.75153393501 + 103918.14464590348*$t);
   $X1 += 0.00000000008*cos(0.93245740025 + 157587.04459711789*$t);
   $X1 += 0.00000000006*cos(1.85606762274 + 6885.39370741431*$t);
   $X1 += 0.00000000007*cos(4.77971152102 + 214364.80099922928*$t);
   $X1 += 0.00000000007*cos(4.36588080270 + 25754.2910182883*$t);
   $X1 += 0.00000000007*cos(1.29774422851 + 26422.0028998271*$t);
   $X1 += 0.00000000006*cos(2.52395990409 + 60055.65161900389*$t);
   $X1 += 0.00000000006*cos(6.00893693971 + 78149.51395352086*$t);
   $X1 += 0.00000000005*cos(0.89617169225 + 50064.3997872543*$t);
   $X1 += 0.00000000006*cos(2.63212801251 + 128221.00242116769*$t);
   $X1 += 0.00000000006*cos(4.95615912967 + 75615.49841673308*$t);
   $X1 += 0.00000000005*cos(1.27204675441 + 137678.43511695448*$t);
   $X1 += 0.00000000007*cos(5.96534970508 + 56727.51596272369*$t);
   $X1 += 0.00000000007*cos(5.86123151663 + 8194.0315157251*$t);
   $X1 += 0.00000000005*cos(6.05965648339 + 130226.46042991648*$t);
   $X1 += 0.00000000006*cos(5.74195245077 + 80382.71710258449*$t);
   $X1 += 0.00000000006*cos(2.26934702805 + 51653.47268253809*$t);
   $X1 += 0.00000000006*cos(0.06833462464 + 123669.04892410889*$t);
   $X1 += 0.00000000006*cos(3.85232597863 + 72602.13355808688*$t);
   $X1 += 0.00000000006*cos(4.43482215622 + 50056.79860528649*$t);
   $X1 += 0.00000000005*cos(1.17789065193 + 102769.89703549728*$t);
   $X1 += 0.00000000006*cos(3.77265893918 + 74820.89066227368*$t);
   $X1 += 0.00000000006*cos(1.81837030655 + 161079.61616398748*$t);
   $X1 += 0.00000000006*cos(3.17899498144 + 103932.37173990508*$t);
   $X1 += 0.00000000006*cos(2.61088164463 + 25234.9505773057*$t);
   $X1 += 0.00000000007*cos(1.11412129297 + 154938.58977164488*$t);
   $X1 += 0.00000000005*cos(3.94989870298 + 419.2408263917*$t);
   $X1 += 0.00000000005*cos(3.88779112513 + 19406.9221056581*$t);
   $X1 += 0.00000000006*cos(3.53467301913 + 1265.81129610991*$t);
   $X1 += 0.00000000007*cos(0.99187081299 + 64901.5035354069*$t);
   $X1 += 0.00000000005*cos(2.45437001625 + 73711.51211018028*$t);
   $X1 += 0.00000000005*cos(0.40751082829 + 78213.95662030189*$t);
   $X1 += 0.00000000006*cos(0.47609620784 + 182085.87484340828*$t);
   $X1 += 0.00000000005*cos(0.72963478402 + 102755.66994149568*$t);
   $X1 += 0.00000000005*cos(3.23310371062 + 40565.01050729069*$t);
   $X1 += 0.00000000005*cos(4.56090628241 + 102.84895673509*$t);
   $X1 += 0.00000000005*cos(2.73032624940 + 51748.96427478889*$t);
   $X1 += 0.00000000005*cos(2.42117857215 + 53399.86794141051*$t);
   $X1 += 0.00000000005*cos(3.99699244254 + 27780.31262856009*$t);
   $X1 += 0.00000000005*cos(6.27879128389 + 80482.22271142589*$t);
   $X1 += 0.00000000005*cos(4.04842386586 + 25938.5837619231*$t);
   $X1 += 0.00000000005*cos(0.51237606895 + 26402.33313892731*$t);
   $X1 += 0.00000000005*cos(5.15124896226 + 25773.96077918809*$t);
   $X1 += 0.00000000005*cos(1.58549672991 + 77616.94240889128*$t);
   $X1 += 0.00000000005*cos(2.07804016284 + 34282.4222922663*$t);
   $X1 += 0.00000000006*cos(0.18266035721 + 50050.17269325269*$t);
   $X1 += 0.00000000005*cos(3.56668263605 + 17893.8716258491*$t);
   $X1 += 0.00000000004*cos(2.54682197432 + 130432.64597835368*$t);
   $X1 += 0.00000000004*cos(1.97821363637 + 51852.54468397449*$t);
   $X1 += 0.00000000004*cos(0.66387666740 + 32370.73517408209*$t);
   $X1 += 0.00000000005*cos(2.92531330514 + 204151.51545301828*$t);
   $X1 += 0.00000000006*cos(3.31682565623 + 75930.75684933408*$t);
   $X1 += 0.00000000004*cos(3.69733037417 + 25668.17468021549*$t);
   $X1 += 0.00000000005*cos(6.08000873981 + 6043.9847638919*$t);
   $X1 += 0.00000000005*cos(4.31437334805 + 39450.5966658569*$t);
   $X1 += 0.00000000005*cos(3.85449175904 + 24491.96051677309*$t);
   $X1 += 0.00000000004*cos(2.80245159203 + 51955.63745819309*$t);
   $X1 += 0.00000000004*cos(5.66172545911 + 103498.66000202828*$t);
   $X1 += 0.00000000005*cos(1.63974900109 + 52396.4627430707*$t);
   $X1 += 0.00000000004*cos(4.13434256469 + 104505.63519426509*$t);
   $X1 += 0.00000000005*cos(4.01690246624 + 23754.46293121869*$t);
   $X1 += 0.00000000005*cos(0.99791835393 + 123201.08393375449*$t);
   $X1 += 0.00000000004*cos(1.49755025165 + 54087.2495838491*$t);
   $X1 += 0.00000000004*cos(5.20879448063 + 637.24008950771*$t);
   $X1 += 0.00000000004*cos(2.16539647390 + 28791.7631137333*$t);
   $X1 += 0.00000000004*cos(3.49822855731 + 23384.5308043821*$t);
   $X1 += 0.00000000005*cos(6.09748705519 + 53029.2464823839*$t);
   $X1 += 0.00000000005*cos(4.96740570225 + 18848.98373249069*$t);
   $X1 += 0.00000000004*cos(3.38348270644 + 26824.02347258949*$t);
   $X1 += 0.00000000004*cos(2.28014232477 + 25352.2704455259*$t);
   $X1 += 0.00000000004*cos(5.11054976948 + 133882.3344703199*$t);
   $X1 += 0.00000000004*cos(2.44432694792 + 132658.51662954128*$t);
   $X1 += 0.00000000004*cos(4.79464066723 + 53772.23654291649*$t);
   $X1 += 0.00000000004*cos(3.58245667544 + 183571.16555011149*$t);
   $X1 += 0.00000000004*cos(1.18282836563 + 167850.32676523308*$t);
   $X1 += 0.00000000005*cos(0.57469466163 + 145204.99856862429*$t);
   $X1 += 0.00000000004*cos(2.91942328210 + 27171.2271913513*$t);
   $X1 += 0.00000000004*cos(2.74420174911 + 25005.0667267641*$t);
   $X1 += 0.00000000004*cos(4.57300561972 + 50483.8844311295*$t);
   $X1 += 0.00000000004*cos(1.71297602638 + 78786.53066029989*$t);
   $X1 += 0.00000000004*cos(4.98597716653 + 949.4194264533*$t);
   $X1 += 0.00000000005*cos(5.48294820542 + 45892.48661567349*$t);
   $X1 += 0.00000000004*cos(5.09042463265 + 1795.5022612045*$t);
   $X1 += 0.00000000005*cos(3.41506026848 + 26709.8907598969*$t);
   $X1 += 0.00000000005*cos(2.24856476273 + 25466.4031582185*$t);
   $X1 += 0.00000000004*cos(5.84172747790 + 52065.84377941249*$t);
   $X1 += 0.00000000004*cos(1.67956658507 + 2222.1004520805*$t);
   $X1 += 0.00000000004*cos(2.68517686010 + 78270.58180110609*$t);
   $X1 += 0.00000000004*cos(4.37036821346 + 25653.94758621389*$t);
   $X1 += 0.00000000004*cos(3.79249696812 + 49842.36607279289*$t);
   $X1 += 0.00000000004*cos(0.19068653380 + 143005.91122533729*$t);
   $X1 += 0.00000000004*cos(3.27752582001 + 78800.75775430149*$t);
   $X1 += 0.00000000004*cos(4.85511136724 + 65697.31390725628*$t);
   $X1 += 0.00000000003*cos(0.19189003895 + 52195.23222656469*$t);
   $X1 += 0.00000000004*cos(5.02476065705 + 130459.42928625426*$t);
   $X1 += 0.00000000004*cos(3.97588615914 + 24491.47288180609*$t);
   $X1 += 0.00000000004*cos(4.14710532879 + 3178.38960805111*$t);
   $X1 += 0.00000000004*cos(2.12747516147 + 220.16882495529*$t);
   $X1 += 0.00000000004*cos(4.96500381777 + 52250.8316991992*$t);
   $X1 += 0.00000000003*cos(4.30211830006 + 78160.86046798748*$t);
   $X1 += 0.00000000004*cos(4.46270461199 + 87367.86023632369*$t);
   $X1 += 0.00000000004*cos(1.72859821695 + 130435.87818999648*$t);
   $X1 += 0.00000000004*cos(4.85546199905 + 53234.94439585409*$t);
   $X1 += 0.00000000004*cos(1.63790933088 + 26575.7817103119*$t);
   $X1 += 0.00000000004*cos(4.02571570033 + 25600.5122078035*$t);
   $X1 += 0.00000000003*cos(0.66645810174 + 1486.2239385487*$t);
   $X1 += 0.00000000005*cos(4.92448477691 + 86144.04239554508*$t);
   $X1 += 0.00000000004*cos(2.91181818574 + 32132.3755404331*$t);
   $X1 += 0.00000000004*cos(2.61773549272 + 66653.40128383189*$t);
   $X1 += 0.00000000003*cos(1.93014248251 + 52310.1591502169*$t);
   $X1 += 0.00000000003*cos(3.81387230068 + 45290.90021070109*$t);
   $X1 += 0.00000000003*cos(0.34402480925 + 70383.86408886709*$t);
   $X1 += 0.00000000003*cos(4.41765442359 + 52252.31617190749*$t);
   $X1 += 0.00000000005*cos(1.98227688514 + 52808.83383994509*$t);
   $X1 += 0.00000000003*cos(5.60337796614 + 1588.82907780029*$t);
   $X1 += 0.00000000004*cos(5.75007070234 + 58857.2749540315*$t);
   $X1 += 0.00000000004*cos(4.77171683373 + 51951.70530492999*$t);
   $X1 += 0.00000000003*cos(3.97223691848 + 50264.8506174147*$t);
   $X1 += 0.00000000004*cos(2.74364830901 + 20043.9183776823*$t);
   $X1 += 0.00000000004*cos(5.35376721002 + 128320.99566497609*$t);
   $X1 += 0.00000000003*cos(5.53627007155 + 25986.18444079209*$t);
   $X1 += 0.00000000003*cos(0.12735495966 + 26190.1094773233*$t);
   $X1 += 0.00000000004*cos(3.03031240281 + 136722.83537534589*$t);
   $X1 += 0.00000000003*cos(4.41334655990 + 65717.47130312308*$t);
   $X1 += 0.00000000003*cos(4.19581525920 + 181026.49291321907*$t);
   $X1 += 0.00000000003*cos(2.72755670550 + 51535.66517935089*$t);
   $X1 += 0.00000000004*cos(1.89805960768 + 129799.86223904048*$t);
   $X1 += 0.00000000003*cos(0.20996976206 + 26073.91986505609*$t);
   $X1 += 0.00000000003*cos(5.45365526915 + 26102.3740530593*$t);
   $X1 += 0.00000000003*cos(3.40818167058 + 52168.44891866409*$t);
   $X1 += 0.00000000004*cos(0.98011046066 + 52155.8927047651*$t);
   $X1 += 0.00000000004*cos(2.86756899369 + 37698.6989174319*$t);
   $X1 += 0.00000000003*cos(5.28107423901 + 51109.06698847489*$t);
   $X1 += 0.00000000003*cos(3.44421079458 + 26247.4486938499*$t);
   $X1 += 0.00000000003*cos(2.21941423663 + 25928.8452242655*$t);
   $X1 += 0.00000000004*cos(1.88058917739 + 468.20880783791*$t);
   $X1 += 0.00000000004*cos(4.84952063164 + 132028.83242063828*$t);
   $X1 += 0.00000000004*cos(4.04256757305 + 25661.06113321469*$t);
   $X1 += 0.00000000003*cos(5.18635157447 + 2703.3723371921*$t);
   $X1 += 0.00000000004*cos(1.14854014554 + 166740.94821313968*$t);
   $X1 += 0.00000000004*cos(1.66492426174 + 59414.23805726489*$t);
   $X1 += 0.00000000003*cos(0.11263836231 + 61560.8911087071*$t);
   $X1 += 0.00000000003*cos(1.57772720091 + 79316.22162539448*$t);
   $X1 += 0.00000000003*cos(4.62863855126 + 130866.35771623049*$t);
   $X1 += 0.00000000003*cos(4.83516317774 + 79219.06534884768*$t);
   $X1 += 0.00000000003*cos(4.03532450201 + 24506.18761077469*$t);
   $X1 += 0.00000000003*cos(2.23216290209 + 51868.4924796623*$t);
   $X1 += 0.00000000003*cos(1.28804069436 + 19202.50943415989*$t);
   $X1 += 0.00000000003*cos(4.56968797112 + 40852.8983673605*$t);
   $X1 += 0.00000000003*cos(4.38565150092 + 116783.89903417809*$t);
   $X1 += 0.00000000003*cos(4.07787007514 + 105460.74730090669*$t);
   $X1 += 0.00000000003*cos(1.85194703747 + 48835.43767393209*$t);
   $X1 += 0.00000000003*cos(5.69146662946 + 114.68310616872*$t);
   $X1 += 0.00000000003*cos(0.19144773585 + 76785.08666814168*$t);
   $X1 += 0.00000000003*cos(2.42893576806 + 81592.08889848628*$t);
   $X1 += 0.00000000003*cos(1.24692506910 + 95247.46175469569*$t);
   $X1 += 0.00000000003*cos(3.12006782424 + 52171.68113030689*$t);
   $X1 += 0.00000000004*cos(0.40583501169 + 78110.17443172129*$t);
   $X1 += 0.00000000003*cos(0.88564181183 + 68050.18006102808*$t);
   $X1 += 0.00000000003*cos(4.02600027527 + 38653.81102407349*$t);
   $X1 += 0.00000000003*cos(2.89741869348 + 2219.0009216703*$t);
   $X1 += 0.00000000003*cos(4.35261728870 + 52179.4438010229*$t);
   $X1 += 0.00000000003*cos(5.72167155686 + 52286.25642185129*$t);
   $X1 += 0.00000000003*cos(4.88587761079 + 76674.39271195528*$t);
   $X1 += 0.00000000003*cos(2.50139495886 + 126997.18458038908*$t);
   $X1 += 0.00000000003*cos(6.06197330359 + 77101.47853779829*$t);
   $X1 += 0.00000000003*cos(5.97751979418 + 77741.37582411229*$t);
   $X1 += 0.00000000003*cos(2.52437891388 + 8989.22388794889*$t);
   $X1 += 0.00000000003*cos(3.70897028008 + 77733.77464214448*$t);
   $X1 += 0.00000000003*cos(6.24188448720 + 286966.69073983253*$t);
   $X1 += 0.00000000003*cos(5.68828427997 + 74.53778108379*$t);
   $X1 += 0.00000000003*cos(2.64767376731 + 632.5399218297*$t);
   $X1 += 0.00000000004*cos(6.05038593627 + 28287.2343023447*$t);
   $X1 += 0.00000000004*cos(5.89642440212 + 23889.0596157707*$t);
   $X1 += 0.00000000003*cos(3.64902489843 + 75930.26921436709*$t);
   $X1 += 0.00000000002*cos(3.26392643567 + 182188.96761762688*$t);
   $X1 += 0.00000000003*cos(6.24797687447 + 27250.62166346549*$t);
   $X1 += 0.00000000003*cos(5.71051457818 + 51315.74017187909*$t);
   $X1 += 0.00000000003*cos(2.95050506838 + 52182.67601266569*$t);
   $X1 += 0.00000000002*cos(0.14560269876 + 121335.85253123689*$t);
   $X1 += 0.00000000002*cos(2.31563547899 + 110012.70079796549*$t);
   $X1 += 0.00000000002*cos(2.79146825409 + 2008.8013566425*$t);
   $X1 += 0.00000000002*cos(5.33939376534 + 24925.6722546499*$t);
   $X1 += 0.00000000003*cos(5.02309471101 + 39629.08052658189*$t);
   $X1 += 0.00000000003*cos(4.81867653228 + 3340.8562441833*$t);
   $X1 += 0.00000000002*cos(5.06629871936 + 23439.20449861769*$t);
   $X1 += 0.00000000002*cos(6.21328369214 + 29415.79498037089*$t);
   $X1 += 0.00000000003*cos(2.40182139304 + 25131.85780308709*$t);
   $X1 += 0.00000000003*cos(4.68865672548 + 51123.29408247649*$t);
   $X1 += 0.00000000003*cos(5.00160560591 + 90695.50825763689*$t);
   $X1 += 0.00000000002*cos(4.84919130926 + 24505.69997580769*$t);
   $X1 += 0.00000000003*cos(4.38371673981 + 26011.8808877821*$t);
   $X1 += 0.00000000002*cos(4.91686748502 + 51329.9672658807*$t);
   $X1 += 0.00000000003*cos(0.23735872795 + 81604.56566890588*$t);
   $X1 += 0.00000000003*cos(0.95743283729 + 52813.0463726561*$t);
   $X1 += 0.00000000003*cos(1.59253054642 + 391318.79094109643*$t);
   $X1 += 0.00000000002*cos(4.54043739808 + 196137.31725009429*$t);
   $X1 += 0.00000000003*cos(2.85252650926 + 156507.9929060289*$t);
   $X1 += 0.00000000003*cos(2.50847406676 + 27727.2164762457*$t);
   $X1 += 0.00000000003*cos(3.15515096445 + 24449.0774418697*$t);
   $X1 += 0.00000000002*cos(1.06183386008 + 78057.76769376888*$t);
   $X1 += 0.00000000003*cos(4.75030126782 + 78313.94986411028*$t);
   $X1 += 0.00000000003*cos(2.83857270203 + 203042.13690092487*$t);
   $X1 += 0.00000000003*cos(4.41522279829 + 34082.6754010819*$t);
   $X1 += 0.00000000002*cos(1.11682523208 + 38634.6288981407*$t);
   $X1 += 0.00000000003*cos(5.82930728472 + 78257.32468330608*$t);
   $X1 += 0.00000000002*cos(3.32397187418 + 49527.10764019189*$t);
   $X1 += 0.00000000003*cos(3.52707759552 + 52041.9410510469*$t);
   $X1 += 0.00000000002*cos(5.03728139459 + 104819.8213741347*$t);
   $X1 += 0.00000000002*cos(0.34249656430 + 62197.88738073129*$t);
   $X1 += 0.00000000003*cos(3.81629136441 + 103291.98681862408*$t);
   $X1 += 0.00000000002*cos(0.38902235709 + 51962.26337022689*$t);
   $X1 += 0.00000000002*cos(5.06187057536 + 35077.61466449009*$t);
   $X1 += 0.00000000003*cos(3.50433403816 + 96357.32794175608*$t);
   $X1 += 0.00000000002*cos(5.63169833454 + 25021.16384690069*$t);
   $X1 += 0.00000000002*cos(3.17912670229 + 66652.91364886488*$t);
   $X1 += 0.00000000002*cos(0.74572876767 + 199599.0743209924*$t);
   $X1 += 0.00000000002*cos(2.95582870605 + 9103.6631766341*$t);
   $X1 += 0.00000000002*cos(1.09917701972 + 74923.34081550628*$t);
   $X1 += 0.00000000002*cos(0.60560213674 + 17098.67925362531*$t);
   $X1 += 0.00000000002*cos(5.11946015221 + 78470.13879064328*$t);
   $X1 += 0.00000000002*cos(2.53118536507 + 101703.40155830729*$t);
   $X1 += 0.00000000002*cos(2.20613703147 + 77307.66408623548*$t);
   $X1 += 0.00000000002*cos(0.59446302071 + 104991.75367009429*$t);
   $X1 += 0.00000000002*cos(2.42950804559 + 86143.55476057807*$t);
   $X1 += 0.00000000002*cos(4.03918468742 + 26312.4917547596*$t);
   $X1 += 0.00000000003*cos(4.60683796484 + 69159.55861312148*$t);
   $X1 += 0.00000000002*cos(2.79539090828 + 27676.73221937449*$t);
   $X1 += 0.00000000002*cos(0.80954348332 + 91919.81373338249*$t);
   $X1 += 0.00000000002*cos(3.64345901161 + 225687.46509753366*$t);
   $X1 += 0.00000000002*cos(6.09474483307 + 60369.83779887349*$t);
   $X1 += 0.00000000002*cos(4.02958262076 + 26184.1261862755*$t);
   $X1 += 0.00000000002*cos(1.63404241045 + 25992.1677318399*$t);
   $X1 += 0.00000000002*cos(4.78056230516 + 62388.84801187589*$t);
   $X1 += 0.00000000002*cos(5.29281787520 + 7880.3329708225*$t);
   $X1 += 0.00000000003*cos(5.23217095654 + 51534.63653889291*$t);
   $X1 += 0.00000000002*cos(1.02304147154 + 24608.79275002629*$t);
   $X1 += 0.00000000002*cos(0.08655833344 + 183145.25677359749*$t);
   $X1 += 0.00000000002*cos(3.38094681034 + 177288.08970012008*$t);
   $X1 += 0.00000000002*cos(2.73964259090 + 26397.42528171349*$t);
   $X1 += 0.00000000002*cos(2.92398244031 + 25778.86863640189*$t);
   $X1 += 0.00000000002*cos(5.05032388283 + 42791.2090925567*$t);
   $X1 += 0.00000000002*cos(0.32496221869 + 38813.11275886569*$t);
   $X1 += 0.00000000002*cos(5.11101145665 + 51539.0538286077*$t);
   $X1 += 0.00000000002*cos(4.02707586748 + 24952.4700871645*$t);
   $X1 += 0.00000000002*cos(1.63654916373 + 27223.8238309509*$t);
   $X1 += 0.00000000002*cos(4.56989494681 + 103719.07264446709*$t);
   $X1 += 0.00000000002*cos(3.21686304521 + 25984.3647704771*$t);
   $X1 += 0.00000000002*cos(2.44676198600 + 26191.9291476383*$t);
   $X1 += 0.00000000002*cos(0.73451437899 + 25927.7395896729*$t);
   $X1 += 0.00000000002*cos(4.92911065222 + 26248.5543284425*$t);
   $X1 += 0.00000000002*cos(5.25833412712 + 113341.32408921266*$t);
   $X1 += 0.00000000002*cos(3.14486706475 + 53265.7588918255*$t);
   $X1 += 0.00000000002*cos(1.46628877500 + 51842.1941598625*$t);
   $X1 += 0.00000000002*cos(2.47984490464 + 16066.3096789583*$t);
   $X1 += 0.00000000002*cos(2.09863898672 + 29550.3916649229*$t);
   $X1 += 0.00000000002*cos(3.56498604449 + 22625.9022531925*$t);
   $X1 += 0.00000000002*cos(1.44463204026 + 27037.32256802749*$t);
   $X1 += 0.00000000002*cos(4.21899299095 + 25138.9713500879*$t);
   $X1 += 0.00000000002*cos(0.93669101848 + 157482.77477357029*$t);
   $X1 += 0.00000000002*cos(0.12919974221 + 52817.46366237089*$t);
   $X1 += 0.00000000002*cos(3.79805429136 + 104202.29318664569*$t);
   $X1 += 0.00000000002*cos(4.69592084098 + 25550.85481199529*$t);
   $X1 += 0.00000000002*cos(5.21767038269 + 52388.86156110289*$t);
   $X1 += 0.00000000002*cos(3.65031025117 + 233731.99016146208*$t);
   $X1 += 0.00000000002*cos(3.29338443989 + 54824.5049261049*$t);
   $X1 += 0.00000000002*cos(1.73836504660 + 13655.6166736929*$t);
   $X1 += 0.00000000002*cos(4.20276068024 + 78793.15657233368*$t);
   $X1 += 0.00000000002*cos(4.96143878613 + 168959.21768235945*$t);
   $X1 += 0.00000000002*cos(6.25349918418 + 205260.89400511167*$t);
   $X1 += 0.00000000002*cos(3.21379506167 + 128106.07549751546*$t);
   $X1 += 0.00000000002*cos(2.12894187827 + 50910.4826220055*$t);
   $X1 += 0.00000000002*cos(3.08974476939 + 51859.65823097529*$t);
   $X1 += 0.00000000002*cos(1.51881438178 + 25863.8021633558*$t);
   $X1 += 0.00000000002*cos(4.40672336509 + 53758.00944891489*$t);
   $X1 += 0.00000000002*cos(5.60768237774 + 52492.4419702885*$t);
   $X1 += 0.00000000002*cos(0.61004720971 + 735.6326960483*$t);
   $X1 += 0.00000000002*cos(0.61004720971 + 736.1203310153*$t);
   $X1 += 0.00000000002*cos(1.54306118494 + 1066.2516597065*$t);
   $X1 += 0.00000000002*cos(1.65860835610 + 215474.17955132265*$t);
   $X1 += 0.00000000002*cos(5.50882009325 + 26077.1012587938*$t);
   $X1 += 0.00000000002*cos(0.15480493796 + 26099.1926593216*$t);
   $X1 += 0.00000000002*cos(1.15953086967 + 96356.84030678908*$t);
   $X1 += 0.00000000002*cos(2.82477485660 + 50579.37602338029*$t);
   $X1 += 0.00000000002*cos(4.95075276458 + 176332.48995851149*$t);
   $X1 += 0.00000000002*cos(5.17774922517 + 78896.73698151928*$t);
   $X1 += 0.00000000002*cos(0.26673303771 + 26411.6523757151*$t);
   $X1 += 0.00000000002*cos(5.49340805923 + 87253.42094763846*$t);
   $X1 += 0.00000000002*cos(4.42579128459 + 181659.96606689428*$t);
   $X1 += 0.00000000002*cos(4.56681824452 + 23976.00901071309*$t);
   $X1 += 0.00000000002*cos(2.69925276343 + 102232.60488843489*$t);
   $X1 += 0.00000000002*cos(3.44678527119 + 52483.6077216015*$t);
   $X1 += 0.00000000002*cos(0.27890618815 + 199599.56195595946*$t);
   $X1 += 0.00000000002*cos(3.49834714879 + 25933.8805136059*$t);
   $X1 += 0.00000000002*cos(5.12143437332 + 23439.6921335847*$t);
   $X1 += 0.00000000001*cos(4.63971494884 + 27351.3043216359*$t);
   $X1 += 0.00000000001*cos(1.02391008237 + 24824.9895964795*$t);
   $X1 += 0.00000000001*cos(4.66880968358 + 78697.66498008289*$t);
   $X1 += 0.00000000002*cos(5.51544937611 + 60370.3254338405*$t);
   $X1 += 0.00000000001*cos(3.24905067191 + 51596.3625136331*$t);
   $X1 += 0.00000000002*cos(4.53665275519 + 103815.05187168489*$t);
   $X1 += 0.00000000002*cos(1.48707290600 + 33326.33491569069*$t);
   $X1 += 0.00000000002*cos(0.70194343478 + 51226.87449166209*$t);
   $X1 += 0.00000000001*cos(2.48229161835 + 77307.86586563049*$t);
   $X1 += 0.00000000002*cos(2.89900338624 + 50952.23225985329*$t);
   $X1 += 0.00000000002*cos(5.06785546369 + 65716.98366815608*$t);
   $X1 += 0.00000000001*cos(3.03622093333 + 231348.79714668586*$t);
   $X1 += 0.00000000001*cos(1.22871462641 + 163765.85062356168*$t);
   $X1 += 0.00000000002*cos(5.81358049715 + 34281.9346572993*$t);
   $X1 += 0.00000000001*cos(0.01895053315 + 25234.46294233869*$t);
   $X1 += 0.00000000001*cos(5.99824107896 + 147423.75567281108*$t);
   $X1 += 0.00000000002*cos(1.73430601364 + 26514.25751496669*$t);
   $X1 += 0.00000000001*cos(0.46522663694 + 3442.3311274819*$t);
   $X1 += 0.00000000001*cos(2.66766894498 + 102762.29585352947*$t);
   $X1 += 0.00000000001*cos(4.19740391458 + 189854.24140010288*$t);
   $X1 += 0.00000000001*cos(4.71704307559 + 54879.6662553075*$t);
   $X1 += 0.00000000002*cos(4.94622107659 + 94330.01910368088*$t);
   $X1 += 0.00000000002*cos(3.22708232430 + 22747.04689739089*$t);
   $X1 += 0.00000000001*cos(5.03032609104 + 138319.84867869345*$t);
   $X1 += 0.00000000001*cos(4.79191528728 + 52509.9060414013*$t);
   $X1 += 0.00000000001*cos(5.71338507031 + 35191.56631820829*$t);
   $X1 += 0.00000000002*cos(5.64922837609 + 26713.81715137009*$t);
   $X1 += 0.00000000002*cos(0.01439665512 + 25462.4767667453*$t);
   $X1 += 0.00000000001*cos(6.11047772238 + 76667.27916495448*$t);
   $X1 += 0.00000000001*cos(1.01940165259 + 116783.41139921108*$t);
   $X1 += 0.00000000002*cos(4.11674730434 + 23919.3864767751*$t);
   $X1 += 0.00000000002*cos(1.54687772687 + 28256.9074413403*$t);
   $X1 += 0.00000000002*cos(2.55506634692 + 99024.38027539749*$t);
   $X1 += 0.00000000001*cos(5.53588784644 + 112546.13171698889*$t);
   $X1 += 0.00000000002*cos(4.36940598830 + 26241.4381345755*$t);
   $X1 += 0.00000000001*cos(0.47478664591 + 129329.89333829407*$t);
   $X1 += 0.00000000002*cos(3.32047464917 + 118828.71993201328*$t);
   $X1 += 0.00000000001*cos(3.81811027049 + 49956.8053614781*$t);
   $X1 += 0.00000000001*cos(1.58300591770 + 28199.79727243529*$t);
   $X1 += 0.00000000001*cos(1.36639947851 + 68241.62832713968*$t);
   $X1 += 0.00000000001*cos(5.16896085902 + 103821.67778371868*$t);
   $X1 += 0.00000000002*cos(1.67830961245 + 25441.13612574289*$t);
   $X1 += 0.00000000001*cos(4.61230342742 + 80175.15272542329*$t);
   $X1 += 0.00000000002*cos(0.48315631972 + 23962.2695516785*$t);
   $X1 += 0.00000000001*cos(3.50023943727 + 26934.2297938089*$t);
   $X1 += 0.00000000001*cos(5.56740631565 + 42153.72518556549*$t);
   $X1 += 0.00000000001*cos(0.57318968993 + 50380.79165691089*$t);
   $X1 += 0.00000000001*cos(4.87428065409 + 241562.08269289686*$t);
   $X1 += 0.00000000001*cos(6.24255788428 + 55618.1374106303*$t);
   $X1 += 0.00000000001*cos(4.91389387543 + 92740.81679043909*$t);
   $X1 += 0.00000000002*cos(5.62002771737 + 27567.01353312209*$t);
   $X1 += 0.00000000002*cos(0.86522458907 + 98690.03669966108*$t);
   $X1 += 0.00000000001*cos(3.83786759867 + 173511.17117941826*$t);
   $X1 += 0.00000000001*cos(3.83786759867 + 173511.65881438529*$t);
   $X1 += 0.00000000001*cos(3.73710353066 + 102018.17235594128*$t);
   $X1 += 0.00000000001*cos(3.98058009404 + 49954.19346603489*$t);
   $X1 += 0.00000000001*cos(0.70035139387 + 155461.16718973868*$t);
   $X1 += 0.00000000001*cos(1.23780419541 + 193937.74227184022*$t);
   $X1 += 0.00000000001*cos(4.37321488294 + 26927.11624680809*$t);
   $X1 += 0.00000000001*cos(1.29041014827 + 25249.1776713073*$t);
   $X1 += 0.00000000001*cos(0.70197977264 + 149642.51277699786*$t);
   $X1 += 0.00000000001*cos(2.49619638334 + 112231.45790215228*$t);
   $X1 += 0.00000000001*cos(3.31600584150 + 639.6534688305*$t);
   $X1 += 0.00000000001*cos(2.81858522024 + 28420.8557169627*$t);
   $X1 += 0.00000000002*cos(0.06574072735 + 55503.69812194508*$t);
   $X1 += 0.00000000001*cos(5.90451405265 + 35472.50033216589*$t);
   $X1 += 0.00000000001*cos(1.51650944248 + 48997.9043100643*$t);
   $X1 += 0.00000000001*cos(5.22070652557 + 27006.0772450395*$t);
   $X1 += 0.00000000001*cos(0.44291850564 + 25170.2166730759*$t);
   $X1 += 0.00000000001*cos(1.78735146281 + 106263.05586699749*$t);
   $X1 += 0.00000000001*cos(4.72084316953 + 90989.40667698109*$t);
   $X1 += 0.00000000001*cos(4.99715761034 + 50690.06997956669*$t);
   $X1 += 0.00000000001*cos(0.33557225431 + 77836.86741636308*$t);
   $X1 += 0.00000000001*cos(3.31386459990 + 125112.2834169717*$t);
   $X1 += 0.00000000001*cos(5.55100305938 + 26610.23674218449*$t);
   $X1 += 0.00000000001*cos(4.62894445056 + 143980.69309287866*$t);
   $X1 += 0.00000000001*cos(0.73494247156 + 153084.60008699627*$t);
   $X1 += 0.00000000001*cos(1.17231831682 + 45404.8518644193*$t);
   $X1 += 0.00000000001*cos(2.54701077766 + 2332.9525753885*$t);
   $X1 += 0.00000000001*cos(4.07473618347 + 76152.30292882849*$t);
   $X1 += 0.00000000001*cos(2.13363904714 + 45494.33761226529*$t);
   $X1 += 0.00000000001*cos(0.33894588198 + 24925.18461968289*$t);
   $X1 += 0.00000000001*cos(2.22391897375 + 50593.60311738189*$t);
   $X1 += 0.00000000001*cos(5.06402826535 + 115674.03284711768*$t);
   $X1 += 0.00000000001*cos(6.21839548284 + 151200.18655854589*$t);
   $X1 += 0.00000000001*cos(1.41453859000 + 84308.18182358149*$t);
   $X1 += 0.00000000001*cos(0.92714278131 + 129379.88996019828*$t);
   $X1 += 0.00000000001*cos(5.82040259792 + 24073.1652872599*$t);
   $X1 += 0.00000000001*cos(6.12640774047 + 28103.1286308555*$t);
   $X1 += 0.00000000001*cos(2.07741344436 + 19406.43447069109*$t);
   $X1 += 0.00000000001*cos(0.51650041444 + 24844.6593573793*$t);
   $X1 += 0.00000000001*cos(5.14712461677 + 27331.6345607361*$t);
   $X1 += 0.00000000001*cos(1.95998075094 + 154408.89880655028*$t);
   $X1 += 0.00000000001*cos(6.15204641708 + 163298.37326817428*$t);
   $X1 += 0.00000000001*cos(5.62631785594 + 58220.2786820073*$t);
   $X1 += 0.00000000001*cos(2.38135876415 + 97670.63153038069*$t);
   $X1 += 0.00000000001*cos(0.13326662566 + 22759.5236678105*$t);
   $X1 += 0.00000000001*cos(1.74239852457 + 24395.9812895553*$t);
   $X1 += 0.00000000001*cos(5.76996425564 + 92741.3044254061*$t);
   $X1 += 0.00000000001*cos(5.18001187713 + 128850.19899510368*$t);
   $X1 += 0.00000000001*cos(1.70882749408 + 25984.56654987209*$t);
   $X1 += 0.00000000001*cos(4.63169895688 + 24357.0246061251*$t);
   $X1 += 0.00000000001*cos(1.03192607433 + 27819.2693119903*$t);
   $X1 += 0.00000000001*cos(6.18803487660 + 183724.45672562926*$t);
   $X1 += 0.00000000001*cos(0.75326620837 + 26624.46383618609*$t);
   $X1 += 0.00000000001*cos(1.73910591832 + 25455.3632197445*$t);
   $X1 += 0.00000000001*cos(1.73910591832 + 25454.87558477749*$t);
   $X1 += 0.00000000001*cos(4.91320505829 + 189385.78877478145*$t);
   $X1 += 0.00000000001*cos(4.91320505829 + 189386.27640974846*$t);
   $X1 += 0.00000000001*cos(0.33962058759 + 209659.06869168568*$t);
   $X1 += 0.00000000001*cos(0.57831111391 + 109.9625037359*$t);
   $X1 += 0.00000000001*cos(3.01192529140 + 82815.41910429788*$t);
   $X1 += 0.00000000001*cos(5.28068518753 + 147423.26803784407*$t);
   $X1 += 0.00000000001*cos(5.41436422726 + 23704.95394428149*$t);
   $X1 += 0.00000000001*cos(0.24926080395 + 28471.3399738339*$t);
   $X1 += 0.00000000001*cos(5.21985223980 + 51440.17358710009*$t);
   $X1 += 0.00000000001*cos(5.81513934032 + 25035.39094090229*$t);
   $X1 += 0.00000000001*cos(5.63545263237 + 26555.6243144451*$t);
   $X1 += 0.00000000001*cos(4.02649464248 + 26177.63270690369*$t);
   $X1 += 0.00000000001*cos(1.63713038873 + 25998.6612112117*$t);
   $X1 += 0.00000000001*cos(1.98713499450 + 233681.99353955788*$t);
   $X1 += 0.00000000001*cos(3.17318181579 + 21650.63275068411*$t);
   $X1 += 0.00000000001*cos(2.49044321542 + 30525.66116743129*$t);
   $X1 += 0.00000000001*cos(1.93284114667 + 111590.04434041328*$t);
   $X1 += 0.00000000001*cos(4.15055550488 + 24203.24579564031*$t);
   $X1 += 0.00000000001*cos(1.51306952632 + 27973.04812247509*$t);
   $X1 += 0.00000000001*cos(4.04308594118 + 99799.41525175449*$t);
   $X1 += 0.00000000001*cos(2.47194170898 + 128106.56313248246*$t);
   $X1 += 0.00000000001*cos(3.82198016519 + 104198.07757329549*$t);
   $X1 += 0.00000000001*cos(1.80561843054 + 132350.95900857169*$t);
   $X1 += 0.00000000001*cos(3.89409837101 + 6680.98103591609*$t);
   $X1 += 0.00000000001*cos(3.02028039042 + 26202.09861277591*$t);
   $X1 += 0.00000000001*cos(2.86028052056 + 50167.49256147289*$t);
   $X1 += 0.00000000001*cos(3.13915436914 + 313054.59388140672*$t);
   $X1 += 0.00000000001*cos(0.04271369503 + 131548.65044248088*$t);
   $X1 += 0.00000000001*cos(4.92373111036 + 85502.14119883909*$t);
   $X1 += 0.00000000001*cos(2.02772617744 + 51542.77872635169*$t);
   $X1 += 0.00000000001*cos(5.92130408913 + 102018.65999090829*$t);
   $X1 += 0.00000000001*cos(3.43111189593 + 51322.36608391289*$t);
   $X1 += 0.00000000001*cos(1.54854247637 + 78413.51643934069*$t);
   $X1 += 0.00000000001*cos(6.28162689960 + 156531.54400228668*$t);
   $X1 += 0.00000000001*cos(1.20531548605 + 105306.96849042189*$t);
   $X1 += 0.00000000001*cos(2.58426030730 + 35077.12702952309*$t);
   $X1 += 0.00000000001*cos(1.09812311120 + 155467.79310177248*$t);
   $X1 += 0.00000000001*cos(1.33162407420 + 45290.41257573408*$t);
   $X1 += 0.00000000001*cos(3.52374631003 + 1162.23088692429*$t);
   $X1 += 0.00000000001*cos(4.28752586027 + 77211.19722405069*$t);
   $X1 += 0.00000000001*cos(4.51338458149 + 25152.2169783489*$t);
   $X1 += 0.00000000001*cos(1.15024044972 + 27024.07693976651*$t);
   $X1 += 0.00000000001*cos(4.26343278626 + 247223.41474204906*$t);
   $X1 += 0.00000000001*cos(3.96461816746 + 183674.94773869208*$t);
   $X1 += 0.00000000001*cos(3.14345130679 + 25131.37016812009*$t);
   $X1 += 0.00000000001*cos(6.06920679029 + 26126.2799946955*$t);
   $X1 += 0.00000000001*cos(5.87760354810 + 26050.0139234199*$t);
   $X1 += 0.00000000001*cos(0.97484682364 + 71980.38975724769*$t);
   $X1 += 0.00000000001*cos(5.34892363416 + 27388.97377726269*$t);
   $X1 += 0.00000000001*cos(0.31470139705 + 24787.3201408527*$t);
   $X1 += 0.00000000001*cos(0.51689218530 + 155475.39428374029*$t);
   $X1 += 0.00000000001*cos(0.66134795423 + 51756.07782178969*$t);
   $X1 += 0.00000000001*cos(2.18850969113 + 54086.76194888209*$t);
   $X1 += 0.00000000001*cos(4.14165011369 + 94138.08320260228*$t);
   $X1 += 0.00000000001*cos(5.17339556279 + 24602.8562523545*$t);
   $X1 += 0.00000000001*cos(0.49022946842 + 27573.4376657609*$t);
   $X1 += 0.00000000001*cos(4.40553516948 + 121335.36489626988*$t);
   $X1 += 0.00000000001*cos(0.40614537067 + 198490.18340386607*$t);
   $X1 += 0.00000000001*cos(0.84588819348 + 181555.69624334667*$t);
   $X1 += 0.00000000001*cos(5.13509102157 + 156534.77621392948*$t);
   $X1 += 0.00000000001*cos(1.73614423183 + 12725.6972522585*$t);
   $X1 += 0.00000000001*cos(0.52115552294 + 130006.04778747768*$t);
   $X1 += 0.00000000001*cos(1.59903474540 + 89586.12970554348*$t);
   $X1 += 0.00000000001*cos(3.74413194011 + 131394.87163199608*$t);
   $X1 += 0.00000000001*cos(0.61669479944 + 77039.64776646046*$t);
   return $X1*$t;
}



   function Mercury_X2($t) // 413 terms of order 2
{
   $X2  = 0.00020000263*cos(5.96893489541 + 26088.1469590577*$t);
   $X2 += 0.00008268782*cos(0.41593027178 + 0.2438174835*$t);
   $X2 += 0.00002498175*cos(3.38864668295 + 52176.0501006319*$t);
   $X2 += 0.00000949347*cos(0.79533142414 + 78263.95324220609*$t);
   $X2 += 0.00000776108*cos(5.52747825416 + 26087.65932409069*$t);
   $X2 += 0.00000371109*cos(4.03419264198 + 104351.85638378029*$t);
   $X2 += 0.00000130259*cos(0.87132938107 + 130439.75952535449*$t);
   $X2 += 0.00000069902*cos(2.02066051600 + 52175.56246566489*$t);
   $X2 += 0.00000042865*cos(3.95497707255 + 156527.66266692868*$t);
   $X2 += 0.00000013564*cos(0.74070016642 + 182615.56580850288*$t);
   $X2 += 0.00000011344*cos(4.89785767709 + 78263.46560723908*$t);
   $X2 += 0.00000004184*cos(3.80256859065 + 208703.46895007708*$t);
   $X2 += 0.00000002952*cos(1.21119140823 + 24978.7684069643*$t);
   $X2 += 0.00000002715*cos(4.19935661957 + 27197.5255111511*$t);
   $X2 += 0.00000002412*cos(1.63037150116 + 104351.36874881329*$t);
   $X2 += 0.00000001268*cos(0.57748866225 + 234791.37209165128*$t);
   $X2 += 0.00000001017*cos(4.33432055001 + 51066.6715485385*$t);
   $X2 += 0.00000000591*cos(4.73978384246 + 130439.27189038748*$t);
   $X2 += 0.00000000372*cos(0.82364022383 + 53285.4286527253*$t);
   $X2 += 0.00000000379*cos(3.63308186625 + 260879.27523322542*$t);
   $X2 += 0.00000000353*cos(1.13633330939 + 77154.57469011268*$t);
   $X2 += 0.00000000353*cos(2.07182576415 + 1109.1347346099*$t);
   $X2 += 0.00000000259*cos(4.03007203572 + 20426.8149099055*$t);
   $X2 += 0.00000000259*cos(5.19867290786 + 1059.6257476727*$t);
   $X2 += 0.00000000198*cos(1.36980571393 + 27043.7467006663*$t);
   $X2 += 0.00000000178*cos(5.96725527175 + 31749.4790082099*$t);
   $X2 += 0.00000000161*cos(1.00641443984 + 1589.3167127673*$t);
   $X2 += 0.00000000148*cos(2.97453779319 + 4552.1973145423*$t);
   $X2 += 0.00000000145*cos(1.59448029290 + 21536.1934619989*$t);
   $X2 += 0.00000000158*cos(1.60582346371 + 156527.17503196168*$t);
   $X2 += 0.00000000169*cos(4.00782299083 + 25132.5472174491*$t);
   $X2 += 0.00000000134*cos(5.93157203334 + 25558.4559939631*$t);
   $X2 += 0.00000000112*cos(0.40462785686 + 286967.17837479962*$t);
   $X2 += 0.00000000111*cos(5.65704047760 + 26617.8379241523*$t);
   $X2 += 0.00000000122*cos(1.49577379421 + 46514.7180514797*$t);
   $X2 += 0.00000000125*cos(1.39636100360 + 23869.3898548709*$t);
   $X2 += 0.00000000111*cos(2.28972892610 + 14765.48286075331*$t);
   $X2 += 0.00000000106*cos(4.77037689879 + 51116.6681704427*$t);
   $X2 += 0.00000000120*cos(4.20635563605 + 103242.47783168689*$t);
   $X2 += 0.00000000113*cos(2.57148364836 + 27147.5288892469*$t);
   $X2 += 0.00000000105*cos(4.19895688280 + 28306.90406324449*$t);
   $X2 += 0.00000000086*cos(5.07571410391 + 19317.43635781211*$t);
   $X2 += 0.00000000103*cos(5.90310312996 + 5661.0882316687*$t);
   $X2 += 0.00000000067*cos(6.26109568890 + 9104.1508116011*$t);
   $X2 += 0.00000000076*cos(1.89488008069 + 529.9347825781*$t);
   $X2 += 0.00000000066*cos(4.44479405797 + 50586.97720534809*$t);
   $X2 += 0.00000000066*cos(3.84543572024 + 3442.81876244891*$t);
   $X2 += 0.00000000061*cos(2.83683010519 + 37410.8110573621*$t);
   $X2 += 0.00000000059*cos(4.74002403670 + 72602.62119305388*$t);
   $X2 += 0.00000000074*cos(1.71392170541 + 22645.57201409229*$t);
   $X2 += 0.00000000056*cos(2.44778877932 + 52705.7410657265*$t);
   $X2 += 0.00000000061*cos(3.55810846013 + 30640.1004561165*$t);
   $X2 += 0.00000000053*cos(5.29509726913 + 640.1411037975*$t);
   $X2 += 0.00000000056*cos(0.44039665586 + 32858.8575603033*$t);
   $X2 += 0.00000000062*cos(0.55305609443 + 51220.4503590233*$t);
   $X2 += 0.00000000050*cos(4.18169929034 + 53131.6498422405*$t);
   $X2 += 0.00000000046*cos(3.94824823145 + 27677.2198543415*$t);
   $X2 += 0.00000000045*cos(1.59652029747 + 25973.70767037247*$t);
   $X2 += 0.00000000045*cos(4.05512833234 + 26202.58624774291*$t);
   $X2 += 0.00000000044*cos(4.75905220926 + 182615.07817353586*$t);
   $X2 += 0.00000000046*cos(2.53699777188 + 27197.03787618409*$t);
   $X2 += 0.00000000046*cos(5.78816990516 + 53235.43203082109*$t);
   $X2 += 0.00000000058*cos(2.08536631307 + 57837.3821497841*$t);
   $X2 += 0.00000000043*cos(3.15018018958 + 529.44714761109*$t);
   $X2 += 0.00000000042*cos(1.10095861133 + 47624.0966035731*$t);
   $X2 += 0.00000000041*cos(2.92440512287 + 26301.4460544957*$t);
   $X2 += 0.00000000039*cos(5.52297144186 + 40853.3860023275*$t);
   $X2 += 0.00000000038*cos(2.59002993610 + 25874.8478636197*$t);
   $X2 += 0.00000000045*cos(3.87761344478 + 29530.7219040231*$t);
   $X2 += 0.00000000045*cos(4.83071642182 + 10213.5293636945*$t);
   $X2 += 0.00000000035*cos(1.77114967266 + 2119.00767786191*$t);
   $X2 += 0.00000000034*cos(0.50537233993 + 56728.0035976907*$t);
   $X2 += 0.00000000040*cos(4.99855315823 + 24978.28077199729*$t);
   $X2 += 0.00000000040*cos(0.98737142456 + 129330.38097326108*$t);
   $X2 += 0.00000000033*cos(3.45515093268 + 313055.08151637383*$t);
   $X2 += 0.00000000039*cos(5.42516034342 + 955.3559241251*$t);
   $X2 += 0.00000000031*cos(4.38256920252 + 6770.46678376209*$t);
   $X2 += 0.00000000038*cos(1.73621782583 + 2218.51328670329*$t);
   $X2 += 0.00000000031*cos(1.18390533268 + 12566.3955174663*$t);
   $X2 += 0.00000000032*cos(4.46582914668 + 49957.2929964451*$t);
   $X2 += 0.00000000029*cos(5.65827133315 + 426.8420083595*$t);
   $X2 += 0.00000000028*cos(5.46107883558 + 43072.1431065143*$t);
   $X2 += 0.00000000025*cos(1.58123453631 + 98690.52433462809*$t);
   $X2 += 0.00000000030*cos(3.15402941421 + 79373.33179429949*$t);
   $X2 += 0.00000000023*cos(5.44646488235 + 78793.64420730069*$t);
   $X2 += 0.00000000030*cos(0.97936081436 + 77204.57131201689*$t);
   $X2 += 0.00000000020*cos(1.34579383399 + 11322.42028082089*$t);
   $X2 += 0.00000000027*cos(0.95498900102 + 54394.80720481869*$t);
   $X2 += 0.00000000023*cos(3.49479332748 + 77308.35350059748*$t);
   $X2 += 0.00000000019*cos(5.90875785254 + 83925.28529135829*$t);
   $X2 += 0.00000000022*cos(1.88106368245 + 45405.3394993863*$t);
   $X2 += 0.00000000020*cos(2.33312866023 + 36301.4325052687*$t);
   $X2 += 0.00000000018*cos(2.63764225914 + 79323.33517239528*$t);
   $X2 += 0.00000000021*cos(2.63713620085 + 13656.10430865991*$t);
   $X2 += 0.00000000020*cos(4.83742385337 + 33326.8225506577*$t);
   $X2 += 0.00000000018*cos(1.65998530501 + 52156.38033973211*$t);
   $X2 += 0.00000000023*cos(4.13597342584 + 41962.7645544209*$t);
   $X2 += 0.00000000018*cos(6.07003180743 + 52389.3491960699*$t);
   $X2 += 0.00000000017*cos(3.47243675916 + 58946.76070187749*$t);
   $X2 += 0.00000000016*cos(1.64800690653 + 23754.95056618569*$t);
   $X2 += 0.00000000019*cos(2.36847359761 + 39609.8984006491*$t);
   $X2 += 0.00000000017*cos(0.25513432917 + 51536.15281431789*$t);
   $X2 += 0.00000000018*cos(0.34702974356 + 18849.4713674577*$t);
   $X2 += 0.00000000015*cos(0.95866304490 + 76674.88034692229*$t);
   $X2 += 0.00000000015*cos(1.75514605179 + 48733.47515566649*$t);
   $X2 += 0.00000000015*cos(4.34995902728 + 853.4401992355*$t);
   $X2 += 0.00000000014*cos(3.61473044000 + 82815.90673926489*$t);
   $X2 += 0.00000000014*cos(5.02500731242 + 52179.9314359899*$t);
   $X2 += 0.00000000014*cos(1.69891784853 + 15874.8614128467*$t);
   $X2 += 0.00000000014*cos(2.36729572184 + 66941.2891439017*$t);
   $X2 += 0.00000000015*cos(5.49473150242 + 16984.2399649401*$t);
   $X2 += 0.00000000015*cos(5.05704278229 + 38654.2986590405*$t);
   $X2 += 0.00000000013*cos(0.64687702919 + 53765.1229959157*$t);
   $X2 += 0.00000000013*cos(1.62167132693 + 208702.98131511007*$t);
   $X2 += 0.00000000013*cos(3.82178316362 + 50057.2862402535*$t);
   $X2 += 0.00000000016*cos(2.05228361219 + 51646.3591355373*$t);
   $X2 += 0.00000000012*cos(3.97912977913 + 28421.3433519297*$t);
   $X2 += 0.00000000013*cos(4.04493324307 + 155418.28411483529*$t);
   $X2 += 0.00000000016*cos(0.65996554595 + 55618.6250455973*$t);
   $X2 += 0.00000000012*cos(3.62569930460 + 5661.5758666357*$t);
   $X2 += 0.00000000015*cos(0.75515577986 + 52195.71986153169*$t);
   $X2 += 0.00000000014*cos(2.81046707561 + 26514.7451499337*$t);
   $X2 += 0.00000000014*cos(3.30095837282 + 110013.18843293248*$t);
   $X2 += 0.00000000013*cos(0.56442607871 + 213.0552779545*$t);
   $X2 += 0.00000000014*cos(2.10561918260 + 26728.0442453717*$t);
   $X2 += 0.00000000013*cos(3.66951257967 + 52172.1687652739*$t);
   $X2 += 0.00000000011*cos(5.46295418561 + 63498.71419893629*$t);
   $X2 += 0.00000000011*cos(0.61230507574 + 79219.55298381469*$t);
   $X2 += 0.00000000010*cos(4.66712610746 + 103292.47445359109*$t);
   $X2 += 0.00000000011*cos(5.20556952105 + 31748.99137324289*$t);
   $X2 += 0.00000000014*cos(3.33453776277 + 6283.3196674749*$t);
   $X2 += 0.00000000010*cos(0.83705988710 + 52290.48938931711*$t);
   $X2 += 0.00000000012*cos(2.60449391926 + 13521.50762410789*$t);
   $X2 += 0.00000000009*cos(3.03188275218 + 35192.0539531753*$t);
   $X2 += 0.00000000009*cos(1.24305000805 + 76045.1961380193*$t);
   $X2 += 0.00000000011*cos(5.89042281055 + 62389.33564684289*$t);
   $X2 += 0.00000000010*cos(4.16418363493 + 51749.45190975589*$t);
   $X2 += 0.00000000009*cos(4.66321261687 + 124778.42747620228*$t);
   $X2 += 0.00000000009*cos(0.22808750229 + 339142.98465794802*$t);
   $X2 += 0.00000000009*cos(4.75513146544 + 78244.28348130629*$t);
   $X2 += 0.00000000009*cos(2.64262688750 + 20426.32727493849*$t);
   $X2 += 0.00000000011*cos(2.92721365751 + 38520.1896094555*$t);
   $X2 += 0.00000000010*cos(3.30244477369 + 16983.75232997309*$t);
   $X2 += 0.00000000009*cos(1.16009282528 + 1109.6223695769*$t);
   $X2 += 0.00000000009*cos(4.65609022876 + 52061.61081194667*$t);
   $X2 += 0.00000000009*cos(2.15470374859 + 104881.54734887488*$t);
   $X2 += 0.00000000008*cos(1.58129311338 + 68050.66769599509*$t);
   $X2 += 0.00000000008*cos(4.01456932062 + 2333.44021035551*$t);
   $X2 += 0.00000000008*cos(0.50473636519 + 12431.79883291429*$t);
   $X2 += 0.00000000008*cos(4.71148146957 + 28206.9108194361*$t);
   $X2 += 0.00000000008*cos(0.22428318331 + 136101.09157450669*$t);
   $X2 += 0.00000000008*cos(5.13855904986 + 213.5429129215*$t);
   $X2 += 0.00000000008*cos(1.53446312549 + 24499.0740637739*$t);
   $X2 += 0.00000000008*cos(0.19632874417 + 103396.25664217169*$t);
   $X2 += 0.00000000007*cos(4.83084149979 + 52168.93655363109*$t);
   $X2 += 0.00000000009*cos(0.68229010552 + 21535.70582703189*$t);
   $X2 += 0.00000000007*cos(5.67305147777 + 53284.94101775829*$t);
   $X2 += 0.00000000007*cos(4.91493686906 + 105461.23493587368*$t);
   $X2 += 0.00000000007*cos(2.13257749573 + 69160.04624808849*$t);
   $X2 += 0.00000000009*cos(0.17767669866 + 7994.77225950771*$t);
   $X2 += 0.00000000009*cos(3.44284310428 + 73711.99974514729*$t);
   $X2 += 0.00000000008*cos(1.40334354171 + 59414.7256922319*$t);
   $X2 += 0.00000000008*cos(3.92015227538 + 52183.1636476327*$t);
   $X2 += 0.00000000007*cos(1.95913527567 + 64742.2018006147*$t);
   $X2 += 0.00000000007*cos(1.83770637593 + 78267.83457756409*$t);
   $X2 += 0.00000000007*cos(5.50637575808 + 26107.81671995749*$t);
   $X2 += 0.00000000007*cos(3.29290677599 + 26068.47719815791*$t);
   $X2 += 0.00000000006*cos(2.48089512320 + 25028.7650288685*$t);
   $X2 += 0.00000000007*cos(6.10010912620 + 51962.7510051939*$t);
   $X2 += 0.00000000007*cos(5.20852269117 + 18208.05780571871*$t);
   $X2 += 0.00000000008*cos(3.20036201627 + 44937.3745090319*$t);
   $X2 += 0.00000000008*cos(1.81720177562 + 51066.18391357149*$t);
   $X2 += 0.00000000006*cos(5.72654755255 + 105411.23831396949*$t);
   $X2 += 0.00000000007*cos(4.86085983191 + 65697.80154222329*$t);
   $X2 += 0.00000000006*cos(2.84309765149 + 88477.23878841709*$t);
   $X2 += 0.00000000006*cos(2.90025697766 + 78477.25233764409*$t);
   $X2 += 0.00000000006*cos(6.10721529063 + 52602.6482915079*$t);
   $X2 += 0.00000000007*cos(3.76564310534 + 78283.62300310588*$t);
   $X2 += 0.00000000006*cos(2.54939515992 + 19805.0711090663*$t);
   $X2 += 0.00000000005*cos(0.40790961084 + 108903.80988083909*$t);
   $X2 += 0.00000000006*cos(2.08187740275 + 129380.37759516528*$t);
   $X2 += 0.00000000005*cos(4.74517813371 + 74821.37829724069*$t);
   $X2 += 0.00000000007*cos(5.47163902652 + 45892.9742506405*$t);
   $X2 += 0.00000000005*cos(3.96385649366 + 1059.1381127057*$t);
   $X2 += 0.00000000005*cos(2.11024903764 + 26084.2656236997*$t);
   $X2 += 0.00000000005*cos(0.40769424239 + 26092.0282944157*$t);
   $X2 += 0.00000000007*cos(4.95181465375 + 71493.2426409605*$t);
   $X2 += 0.00000000005*cos(3.72498845601 + 81706.5281871715*$t);
   $X2 += 0.00000000005*cos(1.28192740378 + 51322.85371887989*$t);
   $X2 += 0.00000000005*cos(0.38902168011 + 33968.23611239669*$t);
   $X2 += 0.00000000005*cos(1.36259031496 + 78256.83969520529*$t);
   $X2 += 0.00000000005*cos(2.94315074391 + 26617.35028918529*$t);
   $X2 += 0.00000000006*cos(0.40666986327 + 78260.07190684808*$t);
   $X2 += 0.00000000004*cos(3.59234736829 + 25661.5487681817*$t);
   $X2 += 0.00000000005*cos(5.20687247513 + 94138.57083756929*$t);
   $X2 += 0.00000000004*cos(5.73436372945 + 22645.08437912529*$t);
   $X2 += 0.00000000005*cos(5.43911125012 + 93029.19228547589*$t);
   $X2 += 0.00000000004*cos(1.51966869369 + 22760.01130277751*$t);
   $X2 += 0.00000000004*cos(4.11367481672 + 29416.28261533789*$t);
   $X2 += 0.00000000004*cos(0.33419603419 + 76145.18938182769*$t);
   $X2 += 0.00000000004*cos(0.81789657863 + 181506.18725640948*$t);
   $X2 += 0.00000000004*cos(5.87548494754 + 13521.9952590749*$t);
   $X2 += 0.00000000004*cos(0.98530676575 + 99799.90288672148*$t);
   $X2 += 0.00000000004*cos(3.64991710996 + 79853.02613748988*$t);
   $X2 += 0.00000000005*cos(5.51168134551 + 44181.52165860769*$t);
   $X2 += 0.00000000004*cos(4.17969593700 + 4551.7096795753*$t);
   $X2 += 0.00000000004*cos(1.07013306418 + 32371.2228090491*$t);
   $X2 += 0.00000000004*cos(4.56317466224 + 32858.36992533629*$t);
   $X2 += 0.00000000004*cos(0.76064444657 + 54509.2464935039*$t);
   $X2 += 0.00000000004*cos(5.34376558738 + 155468.28073673949*$t);
   $X2 += 0.00000000003*cos(3.33102397097 + 162188.99471608088*$t);
   $X2 += 0.00000000003*cos(1.90784099869 + 24498.58642880689*$t);
   $X2 += 0.00000000005*cos(0.70628118340 + 78271.06678920689*$t);
   $X2 += 0.00000000004*cos(4.75154050928 + 234790.88445668427*$t);
   $X2 += 0.00000000003*cos(5.20128667447 + 77734.26227711148*$t);
   $X2 += 0.00000000004*cos(4.16379409199 + 7879.84533585549*$t);
   $X2 += 0.00000000004*cos(5.89383677298 + 64608.09275102969*$t);
   $X2 += 0.00000000004*cos(4.51512332677 + 51116.18053547569*$t);
   $X2 += 0.00000000003*cos(1.82486201671 + 28306.41642827749*$t);
   $X2 += 0.00000000004*cos(5.71282679191 + 39744.0074502341*$t);
   $X2 += 0.00000000003*cos(1.53805667122 + 104332.1866228805*$t);
   $X2 += 0.00000000004*cos(3.89140851351 + 80482.71034639288*$t);
   $X2 += 0.00000000003*cos(1.85783050998 + 37410.32342239509*$t);
   $X2 += 0.00000000004*cos(1.55416627058 + 24864.32911827909*$t);
   $X2 += 0.00000000003*cos(2.39011669525 + 103822.16541868569*$t);
   $X2 += 0.00000000004*cos(4.09022608915 + 27311.96479983631*$t);
   $X2 += 0.00000000003*cos(2.12318372488 + 120226.47397914348*$t);
   $X2 += 0.00000000003*cos(4.30318589016 + 102133.09927959349*$t);
   $X2 += 0.00000000003*cos(1.44971105205 + 150866.33061777649*$t);
   $X2 += 0.00000000003*cos(5.04167948013 + 90830.10494218889*$t);
   $X2 += 0.00000000003*cos(3.27662875658 + 77624.05595589208*$t);
   $X2 += 0.00000000003*cos(3.21324303917 + 129484.15978374588*$t);
   $X2 += 0.00000000003*cos(4.58517957507 + 125887.80602829569*$t);
   $X2 += 0.00000000003*cos(5.16830743449 + 130969.45049044909*$t);
   $X2 += 0.00000000003*cos(4.95343110449 + 58459.1259506233*$t);
   $X2 += 0.00000000002*cos(0.76903109719 + 6282.83203250789*$t);
   $X2 += 0.00000000003*cos(1.18091352454 + 26941.3433408097*$t);
   $X2 += 0.00000000002*cos(1.64968376160 + 3327.89183879669*$t);
   $X2 += 0.00000000003*cos(2.92429834126 + 18093.13088206649*$t);
   $X2 += 0.00000000002*cos(1.24139720028 + 25132.0595824821*$t);
   $X2 += 0.00000000003*cos(0.05117365042 + 26095.2605060585*$t);
   $X2 += 0.00000000003*cos(6.12547181968 + 71025.27765060609*$t);
   $X2 += 0.00000000003*cos(3.14802602217 + 43071.65547154729*$t);
   $X2 += 0.00000000002*cos(5.96655122637 + 114565.14192999128*$t);
   $X2 += 0.00000000002*cos(2.80186013990 + 71980.87739221469*$t);
   $X2 += 0.00000000002*cos(2.32214550421 + 91785.70468379749*$t);
   $X2 += 0.00000000002*cos(2.48797331613 + 2648.6986429565*$t);
   $X2 += 0.00000000002*cos(3.89856574340 + 78378.3925308913*$t);
   $X2 += 0.00000000003*cos(4.90084849261 + 104355.73771913828*$t);
   $X2 += 0.00000000002*cos(2.45748414015 + 26081.03341205689*$t);
   $X2 += 0.00000000002*cos(2.90684371055 + 77197.45776501608*$t);
   $X2 += 0.00000000002*cos(3.27787980454 + 365230.88779952223*$t);
   $X2 += 0.00000000002*cos(2.96609297657 + 78690.55143308209*$t);
   $X2 += 0.00000000003*cos(5.20911364004 + 52815.9473869459*$t);
   $X2 += 0.00000000002*cos(3.65607552967 + 25448.2496727437*$t);
   $X2 += 0.00000000003*cos(0.46560289429 + 77837.35505133009*$t);
   $X2 += 0.00000000002*cos(0.51377942249 + 104371.52614468009*$t);
   $X2 += 0.00000000002*cos(1.04438989611 + 25028.27739390149*$t);
   $X2 += 0.00000000002*cos(2.95995089299 + 11322.9079157879*$t);
   $X2 += 0.00000000002*cos(2.29166292494 + 77211.68485901768*$t);
   $X2 += 0.00000000002*cos(3.65747965646 + 105307.45612538888*$t);
   $X2 += 0.00000000002*cos(3.59068071614 + 78050.65414676809*$t);
   $X2 += 0.00000000002*cos(0.09837196170 + 27043.2590656993*$t);
   $X2 += 0.00000000002*cos(4.33096627724 + 104344.74283677948*$t);
   $X2 += 0.00000000002*cos(5.54465700187 + 46514.23041651269*$t);
   $X2 += 0.00000000002*cos(4.94013298829 + 77154.08705514569*$t);
   $X2 += 0.00000000002*cos(3.18110797568 + 25451.1506870335*$t);
   $X2 += 0.00000000002*cos(2.48251705553 + 26725.1432310819*$t);
   $X2 += 0.00000000002*cos(2.52179042190 + 131499.14145554368*$t);
   $X2 += 0.00000000002*cos(6.07056602539 + 61279.9570947495*$t);
   $X2 += 0.00000000002*cos(1.27106707156 + 89586.61734051049*$t);
   $X2 += 0.00000000002*cos(1.28866377354 + 30639.61282114949*$t);
   $X2 += 0.00000000002*cos(1.72917997527 + 97581.14578253469*$t);
   $X2 += 0.00000000002*cos(1.43340926720 + 78149.51395352086*$t);
   $X2 += 0.00000000002*cos(4.01850243495 + 7238.4317741165*$t);
   $X2 += 0.00000000002*cos(5.54249111609 + 426.3543733925*$t);
   $X2 += 0.00000000002*cos(0.49218153901 + 107794.43132874569*$t);
   $X2 += 0.00000000002*cos(0.34191520237 + 47623.60896860609*$t);
   $X2 += 0.00000000002*cos(5.99315014021 + 104565.15547921829*$t);
   $X2 += 0.00000000002*cos(1.99101623607 + 44295.9609472929*$t);
   $X2 += 0.00000000002*cos(5.47567807736 + 18093.6185170335*$t);
   $X2 += 0.00000000002*cos(5.22941762292 + 10213.0417287275*$t);
   $X2 += 0.00000000002*cos(3.76589759239 + 104358.96993078108*$t);
   $X2 += 0.00000000002*cos(3.44277374163 + 104347.97504842229*$t);
   $X2 += 0.00000000002*cos(3.46735728702 + 134991.71302241329*$t);
   $X2 += 0.00000000002*cos(5.38147823295 + 129910.06856025988*$t);
   $X2 += 0.00000000002*cos(1.91253822385 + 57836.89451481709*$t);
   $X2 += 0.00000000002*cos(3.81440405932 + 25557.96835899609*$t);
   $X2 += 0.00000000002*cos(1.41016954122 + 54294.81396101029*$t);
   $X2 += 0.00000000002*cos(3.48149032535 + 102762.78348849648*$t);
   $X2 += 0.00000000001*cos(1.23200675306 + 7238.9194090835*$t);
   $X2 += 0.00000000002*cos(4.28600144274 + 25934.3681485729*$t);
   $X2 += 0.00000000001*cos(1.40803647408 + 151975.70916986988*$t);
   $X2 += 0.00000000002*cos(4.52450590998 + 26241.9257695425*$t);
   $X2 += 0.00000000001*cos(1.50678411527 + 131549.13807744789*$t);
   $X2 += 0.00000000001*cos(1.06085484841 + 23968.89546371229*$t);
   $X2 += 0.00000000002*cos(2.95379110730 + 25021.6514818677*$t);
   $X2 += 0.00000000002*cos(5.72041106584 + 27154.6424362477*$t);
   $X2 += 0.00000000001*cos(2.16358753423 + 181556.18387831368*$t);
   $X2 += 0.00000000001*cos(0.24053830744 + 27140.41534224609*$t);
   $X2 += 0.00000000001*cos(1.76435464195 + 149.8070146181*$t);
   $X2 += 0.00000000001*cos(3.46637889336 + 50586.48957038109*$t);
   $X2 += 0.00000000001*cos(4.03245836538 + 85502.6288338061*$t);
   $X2 += 0.00000000001*cos(4.69456883146 + 1052.51220067191*$t);
   $X2 += 0.00000000001*cos(1.69300525500 + 84547.0290921975*$t);
   $X2 += 0.00000000001*cos(3.42418885871 + 41962.27691945389*$t);
   $X2 += 0.00000000002*cos(1.10596292069 + 1066.7392946735*$t);
   $X2 += 0.00000000001*cos(4.90928501927 + 33326.33491569069*$t);
   $X2 += 0.00000000001*cos(5.93281924072 + 103285.36090659029*$t);
   $X2 += 0.00000000001*cos(0.91877297183 + 55504.18575691209*$t);
   $X2 += 0.00000000001*cos(6.04736307912 + 117873.60782537168*$t);
   $X2 += 0.00000000001*cos(2.53958781081 + 25035.8785758693*$t);
   $X2 += 0.00000000001*cos(4.09517185988 + 58946.27306691049*$t);
   $X2 += 0.00000000001*cos(0.11578152669 + 188276.89785765509*$t);
   $X2 += 0.00000000001*cos(1.73884599121 + 49842.85370775989*$t);
   $X2 += 0.00000000001*cos(4.77533799776 + 29428.7593857575*$t);
   $X2 += 0.00000000001*cos(5.22199381468 + 146314.37712071768*$t);
   $X2 += 0.00000000001*cos(2.12230022464 + 119117.0954270501*$t);
   $X2 += 0.00000000001*cos(2.86720953628 + 39629.56816154889*$t);
   $X2 += 0.00000000001*cos(2.74763511093 + 12546.72575656651*$t);
   $X2 += 0.00000000001*cos(0.25887140846 + 64607.60511606269*$t);
   $X2 += 0.00000000001*cos(5.22258225356 + 23868.9022199039*$t);
   $X2 += 0.00000000001*cos(4.58859350492 + 48847.9144443517*$t);
   $X2 += 0.00000000001*cos(1.51099098110 + 260878.78759825832*$t);
   $X2 += 0.00000000001*cos(3.51708181436 + 25620.1819687033*$t);
   $X2 += 0.00000000001*cos(0.18832551491 + 98068.78053378889*$t);
   $X2 += 0.00000000001*cos(0.48744281934 + 22747.5345323579*$t);
   $X2 += 0.00000000001*cos(5.29369767440 + 26556.11194941211*$t);
   $X2 += 0.00000000001*cos(5.89974555271 + 85034.66384345168*$t);
   $X2 += 0.00000000001*cos(5.40405552306 + 5327.7199258663*$t);
   $X2 += 0.00000000001*cos(3.99805833912 + 63498.22656396929*$t);
   $X2 += 0.00000000001*cos(0.48047428009 + 26300.95841952869*$t);
   $X2 += 0.00000000001*cos(0.36034529603 + 105940.92927906409*$t);
   $X2 += 0.00000000001*cos(2.48936428540 + 65831.91059180829*$t);
   $X2 += 0.00000000001*cos(2.46490999297 + 79372.84415933248*$t);
   $X2 += 0.00000000001*cos(3.19576682217 + 7.3573644843*$t);
   $X2 += 0.00000000001*cos(0.86957384492 + 53399.86794141051*$t);
   $X2 += 0.00000000001*cos(3.82636106300 + 80597.14963507809*$t);
   $X2 += 0.00000000001*cos(3.10426657434 + 49527.59527515889*$t);
   $X2 += 0.00000000001*cos(0.58848037656 + 36300.94487030169*$t);
   $X2 += 0.00000000001*cos(1.70956045888 + 116918.00808376308*$t);
   $X2 += 0.00000000001*cos(4.85073507517 + 26080.54577708989*$t);
   $X2 += 0.00000000001*cos(3.44422946655 + 60056.13925397089*$t);
   $X2 += 0.00000000001*cos(4.81916140005 + 10022.0810975829*$t);
   $X2 += 0.00000000001*cos(5.11488921531 + 51123.7817174435*$t);
   $X2 += 0.00000000001*cos(0.47588667079 + 51645.87150057029*$t);
   $X2 += 0.00000000001*cos(1.41817736905 + 76.50988875911*$t);
   $X2 += 0.00000000001*cos(0.99594891808 + 106570.61348796709*$t);
   $X2 += 0.00000000001*cos(0.28396033479 + 25021.16384690069*$t);
   $X2 += 0.00000000001*cos(1.51089793343 + 15874.37377787969*$t);
   $X2 += 0.00000000001*cos(0.30377336401 + 86144.04239554508*$t);
   $X2 += 0.00000000001*cos(3.39681294015 + 52644.0150909863*$t);
   $X2 += 0.00000000001*cos(5.97495759877 + 26094.77551795771*$t);
   $X2 += 0.00000000001*cos(2.83507136702 + 26081.51840015769*$t);
   $X2 += 0.00000000001*cos(5.89040270497 + 6.86972951729*$t);
   $X2 += 0.00000000001*cos(0.29019267678 + 16703.3059509825*$t);
   $X2 += 0.00000000001*cos(5.32811862019 + 35472.9879671329*$t);
   $X2 += 0.00000000001*cos(0.32321972304 + 42154.2128205325*$t);
   $X2 += 0.00000000001*cos(4.39149562944 + 77410.75686045409*$t);
   $X2 += 0.00000000001*cos(3.51650331906 + 103925.25819290428*$t);
   $X2 += 0.00000000001*cos(2.00607418956 + 52329.8289111167*$t);
   $X2 += 0.00000000001*cos(3.10569882865 + 6681.46867088311*$t);
   $X2 += 0.00000000001*cos(0.56719469804 + 6770.9544187291*$t);
   $X2 += 0.00000000001*cos(4.66201946613 + 955.8435590921*$t);
   $X2 += 0.00000000001*cos(0.75784126065 + 51109.55462344189*$t);
   $X2 += 0.00000000001*cos(1.25573296054 + 29530.2342690561*$t);
   $X2 += 0.00000000001*cos(1.77919762464 + 26067.9895631909*$t);
   $X2 += 0.00000000001*cos(4.03762558727 + 25447.76203777669*$t);
   $X2 += 0.00000000001*cos(5.95598082912 + 52705.25343075949*$t);
   $X2 += 0.00000000001*cos(1.01747250713 + 33967.74847742969*$t);
   $X2 += 0.00000000001*cos(0.01664299877 + 25035.39094090229*$t);
   $X2 += 0.00000000001*cos(6.20560095721 + 26107.32908499049*$t);
   $X2 += 0.00000000001*cos(0.93578815320 + 23969.3830986793*$t);
   $X2 += 0.00000000001*cos(2.52616943216 + 19316.9487228451*$t);
   $X2 += 0.00000000001*cos(5.09896744500 + 26091.5406594487*$t);
   $X2 += 0.00000000001*cos(2.39439324238 + 53131.16220727349*$t);
   $X2 += 0.00000000001*cos(2.74115190658 + 95247.94938966268*$t);
   $X2 += 0.00000000001*cos(5.36396008776 + 103299.58800059189*$t);
   $X2 += 0.00000000001*cos(4.59387367112 + 26094.77287109149*$t);
   $X2 += 0.00000000001*cos(3.12740178962 + 102233.09252340188*$t);
   $X2 += 0.00000000001*cos(2.15814286033 + 25668.17468021549*$t);
   $X2 += 0.00000000000*cos(6.02426301935 + 12565.9078824993*$t);
   $X2 += 0.00000000000*cos(5.08027895309 + 54394.31956985169*$t);
   $X2 += 0.00000000000*cos(2.97863205244 + 111122.56698502589*$t);
   $X2 += 0.00000000001*cos(3.29290475293 + 12432.2864678813*$t);
   $X2 += 0.00000000001*cos(4.11678654325 + 115674.52048208468*$t);
   $X2 += 0.00000000000*cos(1.52900740701 + 25234.46294233869*$t);
   $X2 += 0.00000000001*cos(0.81921278001 + 25661.06113321469*$t);
   $X2 += 0.00000000001*cos(1.94160256445 + 39609.41076568209*$t);
   $X2 += 0.00000000000*cos(4.43222686144 + 45892.48661567349*$t);
   $X2 += 0.00000000000*cos(5.52190845642 + 77830.24150432929*$t);
   $X2 += 0.00000000000*cos(5.87054542331 + 116783.89903417809*$t);
   $X2 += 0.00000000000*cos(0.73723465850 + 28420.8557169627*$t);
   $X2 += 0.00000000000*cos(1.91678340872 + 103241.99019671988*$t);
   $X2 += 0.00000000000*cos(0.75824711372 + 52026.4869034973*$t);
   $X2 += 0.00000000000*cos(1.73983373880 + 19804.5834740993*$t);
   $X2 += 0.00000000001*cos(2.85651249975 + 26083.7779887327*$t);
   $X2 += 0.00000000000*cos(2.54986446702 + 53242.5455778219*$t);
   $X2 += 0.00000000000*cos(4.12464845193 + 1588.82907780029*$t);
   $X2 += 0.00000000000*cos(4.10463489089 + 51219.96272405629*$t);
   $X2 += 0.00000000000*cos(5.49428153801 + 157637.04121902208*$t);
   $X2 += 0.00000000000*cos(2.31218688576 + 72602.13355808688*$t);
   $X2 += 0.00000000000*cos(4.67530659291 + 83924.79765639128*$t);
   $X2 += 0.00000000000*cos(4.56610835896 + 130420.08976445469*$t);
   $X2 += 0.00000000000*cos(5.25102871130 + 25668.6623151825*$t);
   $X2 += 0.00000000000*cos(1.26830048676 + 3340.8562441833*$t);
   $X2 += 0.00000000000*cos(1.02801034430 + 77204.08367704989*$t);
   $X2 += 0.00000000000*cos(3.83991637789 + 207594.09039798369*$t);
   $X2 += 0.00000000000*cos(2.19144576095 + 419.72846135871*$t);
   $X2 += 0.00000000000*cos(5.62747197310 + 32370.73517408209*$t);
   $X2 += 0.00000000000*cos(1.99592158964 + 78903.8505285201*$t);
   $X2 += 0.00000000000*cos(0.06549387620 + 44936.88687406489*$t);
   $X2 += 0.00000000000*cos(4.55067254270 + 286966.69073983253*$t);
   return $X2*$t*$t;
}



   function Mercury_X3($t) // 135 terms of order 3
{
   $X3  = 0.00000180660*cos(1.53524873089 + 0.2438174835*$t);
   $X3 += 0.00000056109*cos(1.50368825372 + 52176.0501006319*$t);
   $X3 += 0.00000023909*cos(5.09285389011 + 78263.95324220609*$t);
   $X3 += 0.00000011317*cos(2.22514154728 + 104351.85638378029*$t);
   $X3 += 0.00000011202*cos(6.20552374893 + 26088.1469590577*$t);
   $X3 += 0.00000007321*cos(1.49034347852 + 26087.65932409069*$t);
   $X3 += 0.00000005011*cos(5.45015928644 + 130439.75952535449*$t);
   $X3 += 0.00000002025*cos(2.30601224164 + 156527.66266692868*$t);
   $X3 += 0.00000001152*cos(0.12743555397 + 52175.56246566489*$t);
   $X3 += 0.00000000763*cos(5.40668920792 + 182615.56580850288*$t);
   $X3 += 0.00000000387*cos(3.30772457389 + 78263.46560723908*$t);
   $X3 += 0.00000000274*cos(2.20509049018 + 208703.46895007708*$t);
   $X3 += 0.00000000112*cos(0.07801111545 + 104351.36874881329*$t);
   $X3 += 0.00000000095*cos(5.27592058522 + 234791.37209165128*$t);
   $X3 += 0.00000000037*cos(4.50900105290 + 24978.7684069643*$t);
   $X3 += 0.00000000034*cos(4.28000177876 + 27197.5255111511*$t);
   $X3 += 0.00000000032*cos(2.05675254224 + 260879.27523322542*$t);
   $X3 += 0.00000000024*cos(2.45956768858 + 51066.6715485385*$t);
   $X3 += 0.00000000032*cos(3.14716159333 + 130439.27189038748*$t);
   $X3 += 0.00000000013*cos(5.72061024737 + 77154.57469011268*$t);
   $X3 += 0.00000000010*cos(5.11454206540 + 286967.17837479962*$t);
   $X3 += 0.00000000013*cos(0.11908599274 + 53285.4286527253*$t);
   $X3 += 0.00000000010*cos(6.24570451816 + 156527.17503196168*$t);
   $X3 += 0.00000000006*cos(4.82972887929 + 1109.1347346099*$t);
   $X3 += 0.00000000007*cos(2.99175820133 + 1059.6257476727*$t);
   $X3 += 0.00000000006*cos(1.93767779760 + 20426.8149099055*$t);
   $X3 += 0.00000000006*cos(2.56259446748 + 103242.47783168689*$t);
   $X3 += 0.00000000004*cos(0.57540517960 + 31749.4790082099*$t);
   $X3 += 0.00000000004*cos(2.89306042580 + 79373.33179429949*$t);
   $X3 += 0.00000000003*cos(1.89223297598 + 313055.08151637383*$t);
   $X3 += 0.00000000003*cos(0.79688696210 + 4552.1973145423*$t);
   $X3 += 0.00000000003*cos(3.08934162665 + 182615.07817353586*$t);
   $X3 += 0.00000000003*cos(5.67790423166 + 46514.7180514797*$t);
   $X3 += 0.00000000003*cos(1.82353994272 + 21536.1934619989*$t);
   $X3 += 0.00000000002*cos(0.08756125776 + 27147.5288892469*$t);
   $X3 += 0.00000000002*cos(3.96079675679 + 1589.3167127673*$t);
   $X3 += 0.00000000002*cos(0.44816937719 + 25132.5472174491*$t);
   $X3 += 0.00000000002*cos(5.80684121758 + 14765.48286075331*$t);
   $X3 += 0.00000000002*cos(5.81670079373 + 51116.6681704427*$t);
   $X3 += 0.00000000002*cos(5.64942685187 + 129330.38097326108*$t);
   $X3 += 0.00000000002*cos(4.47727329739 + 27197.03787618409*$t);
   $X3 += 0.00000000002*cos(2.76458797335 + 26617.8379241523*$t);
   $X3 += 0.00000000002*cos(2.01320187052 + 27043.7467006663*$t);
   $X3 += 0.00000000002*cos(6.24606226113 + 25558.4559939631*$t);
   $X3 += 0.00000000002*cos(2.88488990350 + 72602.62119305388*$t);
   $X3 += 0.00000000002*cos(1.80402690775 + 26068.47719815791*$t);
   $X3 += 0.00000000002*cos(0.72272570426 + 26107.81671995749*$t);
   $X3 += 0.00000000001*cos(5.15880419190 + 26092.0282944157*$t);
   $X3 += 0.00000000001*cos(3.65122349702 + 26084.2656236997*$t);
   $X3 += 0.00000000001*cos(3.47204379646 + 24978.28077199729*$t);
   $X3 += 0.00000000001*cos(3.04599592472 + 37410.8110573621*$t);
   $X3 += 0.00000000001*cos(0.31369621903 + 30640.1004561165*$t);
   $X3 += 0.00000000001*cos(0.38979376906 + 52705.7410657265*$t);
   $X3 += 0.00000000001*cos(5.01792294793 + 51220.4503590233*$t);
   $X3 += 0.00000000001*cos(1.77400727778 + 47624.0966035731*$t);
   $X3 += 0.00000000001*cos(6.21364236354 + 208702.98131511007*$t);
   $X3 += 0.00000000001*cos(3.26982842147 + 9104.1508116011*$t);
   $X3 += 0.00000000001*cos(6.10085930771 + 98690.52433462809*$t);
   $X3 += 0.00000000001*cos(2.72881790021 + 955.3559241251*$t);
   $X3 += 0.00000000001*cos(2.74013903592 + 1109.6223695769*$t);
   $X3 += 0.00000000001*cos(2.00927169251 + 19317.43635781211*$t);
   $X3 += 0.00000000001*cos(4.85871407247 + 50586.97720534809*$t);
   $X3 += 0.00000000001*cos(5.37897907969 + 12566.3955174663*$t);
   $X3 += 0.00000000001*cos(3.51249866668 + 40853.3860023275*$t);
   $X3 += 0.00000000001*cos(3.74067924304 + 53235.43203082109*$t);
   $X3 += 0.00000000001*cos(4.93930525226 + 339142.98465794802*$t);
   $X3 += 0.00000000001*cos(3.64879489376 + 426.8420083595*$t);
   $X3 += 0.00000000001*cos(2.70663061614 + 49957.2929964451*$t);
   $X3 += 0.00000000001*cos(4.05940415323 + 26095.2605060585*$t);
   $X3 += 0.00000000001*cos(4.79523771458 + 26081.03341205689*$t);
   $X3 += 0.00000000001*cos(4.56727257443 + 23869.3898548709*$t);
   $X3 += 0.00000000001*cos(1.91316161387 + 77308.35350059748*$t);
   $X3 += 0.00000000001*cos(5.81582715174 + 105461.23493587368*$t);
   $X3 += 0.00000000001*cos(1.67826436610 + 51646.3591355373*$t);
   $X3 += 0.00000000001*cos(2.42681597375 + 155418.28411483529*$t);
   $X3 += 0.00000000001*cos(4.76066610745 + 56728.0035976907*$t);
   $X3 += 0.00000000001*cos(3.51433188972 + 53131.6498422405*$t);
   $X3 += 0.00000000001*cos(0.78156168692 + 3442.81876244891*$t);
   $X3 += 0.00000000001*cos(3.65095652022 + 78793.64420730069*$t);
   $X3 += 0.00000000001*cos(6.11514198482 + 54394.80720481869*$t);
   $X3 += 0.00000000001*cos(0.70996446104 + 79323.33517239528*$t);
   $X3 += 0.00000000001*cos(4.23955129941 + 28306.90406324449*$t);
   $X3 += 0.00000000001*cos(0.04814737416 + 45405.3394993863*$t);
   $X3 += 0.00000000001*cos(0.51322200896 + 32858.8575603033*$t);
   $X3 += 0.00000000000*cos(2.70568366640 + 529.44714761109*$t);
   $X3 += 0.00000000001*cos(1.65918694650 + 25028.7650288685*$t);
   $X3 += 0.00000000000*cos(2.60942411635 + 25874.8478636197*$t);
   $X3 += 0.00000000001*cos(1.76282246473 + 22645.57201409229*$t);
   $X3 += 0.00000000000*cos(2.31611990566 + 640.1411037975*$t);
   $X3 += 0.00000000000*cos(0.89355979270 + 529.9347825781*$t);
   $X3 += 0.00000000000*cos(6.21531765450 + 26301.4460544957*$t);
   $X3 += 0.00000000000*cos(3.72335130305 + 83925.28529135829*$t);
   $X3 += 0.00000000000*cos(1.81919615354 + 82815.90673926489*$t);
   $X3 += 0.00000000001*cos(1.59769152646 + 27677.2198543415*$t);
   $X3 += 0.00000000000*cos(3.60725162991 + 11322.42028082089*$t);
   $X3 += 0.00000000000*cos(0.07486495309 + 36301.4325052687*$t);
   $X3 += 0.00000000000*cos(4.52680880738 + 5661.5758666357*$t);
   $X3 += 0.00000000000*cos(3.07463170530 + 18849.4713674577*$t);
   $X3 += 0.00000000000*cos(1.10484790510 + 6770.46678376209*$t);
   $X3 += 0.00000000000*cos(5.54664261005 + 43072.1431065143*$t);
   $X3 += 0.00000000000*cos(4.85626455897 + 52156.38033973211*$t);
   $X3 += 0.00000000000*cos(3.44123385645 + 39609.8984006491*$t);
   $X3 += 0.00000000000*cos(0.10298074121 + 1052.51220067191*$t);
   $X3 += 0.00000000000*cos(2.42274340940 + 15874.8614128467*$t);
   $X3 += 0.00000000000*cos(5.90990701318 + 1066.7392946735*$t);
   $X3 += 0.00000000000*cos(0.47240363594 + 24499.0740637739*$t);
   $X3 += 0.00000000000*cos(4.58808593083 + 2119.00767786191*$t);
   $X3 += 0.00000000000*cos(1.93271006548 + 52179.9314359899*$t);
   $X3 += 0.00000000000*cos(2.71699794579 + 27043.2590656993*$t);
   $X3 += 0.00000000000*cos(0.78321130217 + 29530.7219040231*$t);
   $X3 += 0.00000000000*cos(0.87395244120 + 20426.32727493849*$t);
   $X3 += 0.00000000000*cos(0.22412464998 + 26514.7451499337*$t);
   $X3 += 0.00000000000*cos(2.09910746178 + 38654.2986590405*$t);
   $X3 += 0.00000000000*cos(0.37782872130 + 51066.18391357149*$t);
   $X3 += 0.00000000000*cos(6.08588015961 + 63498.71419893629*$t);
   $X3 += 0.00000000000*cos(3.06180705468 + 234790.88445668427*$t);
   $X3 += 0.00000000000*cos(0.44816354598 + 110013.18843293248*$t);
   $X3 += 0.00000000000*cos(3.76757290021 + 52195.71986153169*$t);
   $X3 += 0.00000000000*cos(0.26388289924 + 76674.88034692229*$t);
   $X3 += 0.00000000000*cos(3.06411835882 + 4551.7096795753*$t);
   $X3 += 0.00000000000*cos(1.55289005756 + 7.3573644843*$t);
   $X3 += 0.00000000000*cos(5.69100090266 + 33326.8225506577*$t);
   $X3 += 0.00000000000*cos(4.84238249166 + 2218.51328670329*$t);
   $X3 += 0.00000000000*cos(5.21804461989 + 51749.45190975589*$t);
   $X3 += 0.00000000000*cos(0.07928724753 + 7238.4317741165*$t);
   $X3 += 0.00000000000*cos(0.41819802690 + 52172.1687652739*$t);
   $X3 += 0.00000000000*cos(1.04835331170 + 53284.94101775829*$t);
   $X3 += 0.00000000000*cos(0.31150101949 + 66941.2891439017*$t);
   $X3 += 0.00000000000*cos(1.30179196383 + 52168.93655363109*$t);
   $X3 += 0.00000000000*cos(3.59073128026 + 58946.76070187749*$t);
   $X3 += 0.00000000000*cos(5.69460606394 + 30639.61282114949*$t);
   $X3 += 0.00000000000*cos(3.09211354360 + 62389.33564684289*$t);
   $X3 += 0.00000000000*cos(3.64742441695 + 105411.23831396949*$t);
   $X3 += 0.00000000000*cos(2.56174511433 + 103292.47445359109*$t);
   $X3 += 0.00000000000*cos(3.50117859238 + 23868.9022199039*$t);
   return $X3*$t*$t*$t;
}



   function Mercury_X4($t) // 42 terms of order 4
{
   $X4  = 0.00000043303*cos(2.70854317703 + 26088.1469590577*$t);
   $X4 += 0.00000016746*cos(2.85109602051 + 0.2438174835*$t);
   $X4 += 0.00000005097*cos(5.82035608585 + 52176.0501006319*$t);
   $X4 += 0.00000001110*cos(2.85416500039 + 78263.95324220609*$t);
   $X4 += 0.00000000968*cos(3.18837996543 + 26087.65932409069*$t);
   $X4 += 0.00000000362*cos(0.10660634045 + 104351.85638378029*$t);
   $X4 += 0.00000000161*cos(3.56341896847 + 130439.75952535449*$t);
   $X4 += 0.00000000075*cos(0.55603774289 + 156527.66266692868*$t);
   $X4 += 0.00000000076*cos(6.07252064875 + 52175.56246566489*$t);
   $X4 += 0.00000000033*cos(3.72826875982 + 182615.56580850288*$t);
   $X4 += 0.00000000014*cos(2.40716156772 + 78263.46560723908*$t);
   $X4 += 0.00000000014*cos(0.56399976721 + 208703.46895007708*$t);
   $X4 += 0.00000000005*cos(3.66056233199 + 234791.37209165128*$t);
   $X4 += 0.00000000004*cos(5.13174489459 + 104351.36874881329*$t);
   $X4 += 0.00000000002*cos(2.87490875788 + 24978.7684069643*$t);
   $X4 += 0.00000000002*cos(5.84989349950 + 27197.5255111511*$t);
   $X4 += 0.00000000002*cos(0.45146054812 + 260879.27523322542*$t);
   $X4 += 0.00000000001*cos(1.74376603369 + 130439.27189038748*$t);
   $X4 += 0.00000000001*cos(3.51635178690 + 286967.17837479962*$t);
   $X4 += 0.00000000001*cos(0.06797049294 + 51066.6715485385*$t);
   $X4 += 0.00000000001*cos(1.33488273309 + 1059.6257476727*$t);
   $X4 += 0.00000000001*cos(0.19743580633 + 20426.8149099055*$t);
   $X4 += 0.00000000000*cos(4.73344633686 + 156527.17503196168*$t);
   $X4 += 0.00000000000*cos(3.03361722015 + 53285.4286527253*$t);
   $X4 += 0.00000000000*cos(0.28697199826 + 1109.1347346099*$t);
   $X4 += 0.00000000000*cos(2.12246327493 + 31749.4790082099*$t);
   $X4 += 0.00000000000*cos(3.62335756262 + 77154.57469011268*$t);
   $X4 += 0.00000000000*cos(5.27675905248 + 4552.1973145423*$t);
   $X4 += 0.00000000000*cos(0.97938730115 + 51116.6681704427*$t);
   $X4 += 0.00000000000*cos(2.90146638781 + 5661.0882316687*$t);
   $X4 += 0.00000000000*cos(3.37193339699 + 21536.1934619989*$t);
   $X4 += 0.00000000000*cos(1.47858093383 + 182615.07817353586*$t);
   $X4 += 0.00000000000*cos(3.30664239042 + 46514.7180514797*$t);
   $X4 += 0.00000000000*cos(2.38442876069 + 1589.3167127673*$t);
   $X4 += 0.00000000000*cos(4.43080795748 + 27147.5288892469*$t);
   $X4 += 0.00000000000*cos(5.33267823849 + 25132.5472174491*$t);
   $X4 += 0.00000000000*cos(0.89978296758 + 26617.8379241523*$t);
   $X4 += 0.00000000000*cos(4.07365069575 + 14765.48286075331*$t);
   $X4 += 0.00000000000*cos(1.63297628773 + 25558.4559939631*$t);
   $X4 += 0.00000000000*cos(3.35368510638 + 27043.7467006663*$t);
   $X4 += 0.00000000000*cos(4.71304947972 + 529.9347825781*$t);
   $X4 += 0.00000000000*cos(5.15326778299 + 57837.3821497841*$t);
   return $X4*$t*$t*$t*$t;
}



   function Mercury_X5($t) // 16 terms of order 5
{
   $X5  = 0.00000000414*cos(4.09017660105 + 0.2438174835*$t);
   $X5 += 0.00000000327*cos(2.83894329980 + 26088.1469590577*$t);
   $X5 += 0.00000000134*cos(4.51536199764 + 52176.0501006319*$t);
   $X5 += 0.00000000046*cos(1.23267980717 + 78263.95324220609*$t);
   $X5 += 0.00000000016*cos(4.45794773259 + 104351.85638378029*$t);
   $X5 += 0.00000000007*cos(5.57199695746 + 26087.65932409069*$t);
   $X5 += 0.00000000006*cos(1.55726840151 + 130439.75952535449*$t);
   $X5 += 0.00000000003*cos(4.94192470876 + 156527.66266692868*$t);
   $X5 += 0.00000000001*cos(1.93929522999 + 182615.56580850288*$t);
   $X5 += 0.00000000001*cos(4.16271885947 + 52175.56246566489*$t);
   $X5 += 0.00000000001*cos(5.10948519211 + 208703.46895007708*$t);
   $X5 += 0.00000000000*cos(0.99915185773 + 78263.46560723908*$t);
   $X5 += 0.00000000000*cos(1.90802069686 + 234791.37209165128*$t);
   $X5 += 0.00000000000*cos(3.92135630671 + 104351.36874881329*$t);
   $X5 += 0.00000000000*cos(1.30030594321 + 24978.7684069643*$t);
   $X5 += 0.00000000000*cos(0.96944549328 + 27197.5255111511*$t);
   return $X5*$t*$t*$t*$t*$t;
}



   function Mercury_Y0($t) // 1853 terms of order 0
{
   $Y0  = 0.37749277893*cos(2.83179506899 + 26088.1469590577*$t);
   $Y0 += 0.11918926148*cos(2.91948125760 + 0.2438174835*$t);
   $Y0 += 0.03840153904*cos(5.88254544140 + 52176.0501006319*$t);
   $Y0 += 0.00585979278*cos(2.65010770289 + 78263.95324220609*$t);
   $Y0 += 0.00305833424*cos(3.67378306016 + 26087.65932409069*$t);
   $Y0 += 0.00105974941*cos(5.70085415459 + 104351.85638378029*$t);
   $Y0 += 0.00024906132*cos(0.55066770933 + 52175.56246566489*$t);
   $Y0 += 0.00021056065*cos(2.46841472315 + 130439.75952535449*$t);
   $Y0 += 0.00004441671*cos(5.51916065495 + 156527.66266692868*$t);
   $Y0 += 0.00003347442*cos(3.67952842081 + 78263.46560723908*$t);
   $Y0 += 0.00000976617*cos(2.28672075339 + 182615.56580850288*$t);
   $Y0 += 0.00000703903*cos(2.88960689088 + 24978.7684069643*$t);
   $Y0 += 0.00000640528*cos(5.91381352466 + 27197.5255111511*$t);
   $Y0 += 0.00000558149*cos(0.50510861725 + 104351.36874881329*$t);
   $Y0 += 0.00000447775*cos(1.40511442556 + 1059.6257476727*$t);
   $Y0 += 0.00000400281*cos(0.29865270517 + 20426.8149099055*$t);
   $Y0 += 0.00000283882*cos(2.22271091529 + 31749.4790082099*$t);
   $Y0 += 0.00000191007*cos(2.70458121832 + 53285.4286527253*$t);
   $Y0 += 0.00000181454*cos(0.23779467119 + 1109.1347346099*$t);
   $Y0 += 0.00000193895*cos(5.38872313263 + 4552.1973145423*$t);
   $Y0 += 0.00000221371*cos(5.33746575111 + 208703.46895007708*$t);
   $Y0 += 0.00000139001*cos(5.94193853313 + 51066.6715485385*$t);
   $Y0 += 0.00000183933*cos(2.76579938720 + 5661.0882316687*$t);
   $Y0 += 0.00000153303*cos(1.11372235868 + 51116.6681704427*$t);
   $Y0 += 0.00000104855*cos(3.60039284031 + 130439.27189038748*$t);
   $Y0 += 0.00000115202*cos(5.27930045344 + 57837.3821497841*$t);
   $Y0 += 0.00000090628*cos(4.97031050861 + 529.9347825781*$t);
   $Y0 += 0.00000075318*cos(4.50136079719 + 27147.5288892469*$t);
   $Y0 += 0.00000072957*cos(3.43332933363 + 21536.1934619989*$t);
   $Y0 += 0.00000074803*cos(3.35834807468 + 46514.7180514797*$t);
   $Y0 += 0.00000068710*cos(4.91618713593 + 25132.5472174491*$t);
   $Y0 += 0.00000083504*cos(1.60859812856 + 10213.5293636945*$t);
   $Y0 += 0.00000060985*cos(3.88048699951 + 27043.7467006663*$t);
   $Y0 += 0.00000057987*cos(4.13400021557 + 14765.48286075331*$t);
   $Y0 += 0.00000060953*cos(3.43164694662 + 47624.0966035731*$t);
   $Y0 += 0.00000053356*cos(0.72676419246 + 26617.8379241523*$t);
   $Y0 += 0.00000054964*cos(3.74064645856 + 12566.3955174663*$t);
   $Y0 += 0.00000047881*cos(5.75865881516 + 79373.33179429949*$t);
   $Y0 += 0.00000051263*cos(1.96665333979 + 426.8420083595*$t);
   $Y0 += 0.00000051361*cos(2.10502555376 + 234791.37209165128*$t);
   $Y0 += 0.00000050374*cos(2.03207321104 + 25558.4559939631*$t);
   $Y0 += 0.00000048449*cos(1.94914263411 + 1589.3167127673*$t);
   $Y0 += 0.00000048867*cos(4.16469048525 + 77204.57131201689*$t);
   $Y0 += 0.00000034739*cos(4.11430856351 + 51646.3591355373*$t);
   $Y0 += 0.00000032851*cos(4.56963272977 + 955.3559241251*$t);
   $Y0 += 0.00000044998*cos(0.91167682267 + 41962.7645544209*$t);
   $Y0 += 0.00000031618*cos(4.66914248946 + 37410.8110573621*$t);
   $Y0 += 0.00000028634*cos(5.42748718604 + 30640.1004561165*$t);
   $Y0 += 0.00000031480*cos(2.70995810641 + 77154.57469011268*$t);
   $Y0 += 0.00000032102*cos(2.04798101755 + 83925.28529135829*$t);
   $Y0 += 0.00000027670*cos(5.06128264443 + 39609.8984006491*$t);
   $Y0 += 0.00000026872*cos(4.68921320015 + 36301.4325052687*$t);
   $Y0 += 0.00000023054*cos(0.71973276708 + 53131.6498422405*$t);
   $Y0 += 0.00000026193*cos(5.26902812193 + 11322.42028082089*$t);
   $Y0 += 0.00000021235*cos(0.40309333526 + 156527.17503196168*$t);
   $Y0 += 0.00000026946*cos(0.19172960930 + 6283.3196674749*$t);
   $Y0 += 0.00000020875*cos(4.09807664196 + 15874.8614128467*$t);
   $Y0 += 0.00000016454*cos(0.12816102587 + 72602.62119305388*$t);
   $Y0 += 0.00000016101*cos(1.65540440480 + 9104.1508116011*$t);
   $Y0 += 0.00000022241*cos(5.60722857321 + 13521.50762410789*$t);
   $Y0 += 0.00000021188*cos(0.19996828748 + 73711.99974514729*$t);
   $Y0 += 0.00000015123*cos(1.24465016017 + 18849.4713674577*$t);
   $Y0 += 0.00000017255*cos(0.24107386270 + 27197.03787618409*$t);
   $Y0 += 0.00000017287*cos(0.55372068563 + 51749.45190975589*$t);
   $Y0 += 0.00000013862*cos(1.29323907537 + 53235.43203082109*$t);
   $Y0 += 0.00000015239*cos(1.96671567039 + 7238.4317741165*$t);
   $Y0 += 0.00000016863*cos(0.57148727289 + 50586.97720534809*$t);
   $Y0 += 0.00000013546*cos(3.64906926506 + 52705.7410657265*$t);
   $Y0 += 0.00000013134*cos(4.21391538202 + 25028.7650288685*$t);
   $Y0 += 0.00000014145*cos(1.46555737893 + 63498.71419893629*$t);
   $Y0 += 0.00000013081*cos(0.20810129185 + 26068.47719815791*$t);
   $Y0 += 0.00000013058*cos(2.31384723416 + 26107.81671995749*$t);
   $Y0 += 0.00000013609*cos(1.68775421835 + 51220.4503590233*$t);
   $Y0 += 0.00000013539*cos(0.93214332140 + 103292.47445359109*$t);
   $Y0 += 0.00000012138*cos(5.15577059992 + 260879.27523322542*$t);
   $Y0 += 0.00000011433*cos(0.41025670835 + 19317.43635781211*$t);
   $Y0 += 0.00000010174*cos(0.82322453273 + 77734.26227711148*$t);
   $Y0 += 0.00000012016*cos(3.96439926693 + 68050.66769599509*$t);
   $Y0 += 0.00000012094*cos(2.52834929119 + 105461.23493587368*$t);
   $Y0 += 0.00000009215*cos(0.91145814574 + 40853.3860023275*$t);
   $Y0 += 0.00000009157*cos(1.25590715075 + 33326.8225506577*$t);
   $Y0 += 0.00000009552*cos(0.53954398597 + 38654.2986590405*$t);
   $Y0 += 0.00000012500*cos(2.32164805011 + 45892.9742506405*$t);
   $Y0 += 0.00000008978*cos(5.18856241237 + 26084.2656236997*$t);
   $Y0 += 0.00000008975*cos(3.61663563272 + 26092.0282944157*$t);
   $Y0 += 0.00000012160*cos(1.83585138946 + 65697.80154222329*$t);
   $Y0 += 0.00000008880*cos(5.05045852302 + 26514.7451499337*$t);
   $Y0 += 0.00000008892*cos(5.46009752557 + 3442.81876244891*$t);
   $Y0 += 0.00000008503*cos(5.09915668367 + 110013.18843293248*$t);
   $Y0 += 0.00000010244*cos(4.07406090643 + 21535.70582703189*$t);
   $Y0 += 0.00000008816*cos(3.29850433474 + 32371.2228090491*$t);
   $Y0 += 0.00000009705*cos(0.73577020420 + 529.44714761109*$t);
   $Y0 += 0.00000007663*cos(2.16404811803 + 56728.0035976907*$t);
   $Y0 += 0.00000007852*cos(5.36793581356 + 5661.5758666357*$t);
   $Y0 += 0.00000008166*cos(5.04025521360 + 27677.2198543415*$t);
   $Y0 += 0.00000007299*cos(2.72074453751 + 6770.46678376209*$t);
   $Y0 += 0.00000007437*cos(2.10814922209 + 32858.8575603033*$t);
   $Y0 += 0.00000007796*cos(2.76073375309 + 31748.99137324289*$t);
   $Y0 += 0.00000006786*cos(0.86898827574 + 43072.1431065143*$t);
   $Y0 += 0.00000007408*cos(2.97710007179 + 23869.3898548709*$t);
   $Y0 += 0.00000005938*cos(3.77513720036 + 79219.55298381469*$t);
   $Y0 += 0.00000007553*cos(5.76102373104 + 103242.47783168689*$t);
   $Y0 += 0.00000005988*cos(1.47338249847 + 62389.33564684289*$t);
   $Y0 += 0.00000006154*cos(4.42442839598 + 59414.7256922319*$t);
   $Y0 += 0.00000006213*cos(5.46384627852 + 19805.0711090663*$t);
   $Y0 += 0.00000006507*cos(5.77583424512 + 213.5429129215*$t);
   $Y0 += 0.00000006284*cos(3.34908254208 + 22645.57201409229*$t);
   $Y0 += 0.00000006068*cos(3.25197843754 + 99799.90288672148*$t);
   $Y0 += 0.00000006276*cos(1.17668731862 + 640.1411037975*$t);
   $Y0 += 0.00000006192*cos(5.82504003546 + 28306.90406324449*$t);
   $Y0 += 0.00000006081*cos(3.66198807267 + 25874.8478636197*$t);
   $Y0 += 0.00000004693*cos(1.48074264670 + 16983.75232997309*$t);
   $Y0 += 0.00000004527*cos(3.48220057078 + 182615.07817353586*$t);
   $Y0 += 0.00000005085*cos(5.05051010501 + 26301.4460544957*$t);
   $Y0 += 0.00000004658*cos(2.54088213723 + 26095.2605060585*$t);
   $Y0 += 0.00000005323*cos(3.72103089814 + 24978.28077199729*$t);
   $Y0 += 0.00000004497*cos(0.02524195316 + 26081.03341205689*$t);
   $Y0 += 0.00000005479*cos(3.60707059696 + 76674.88034692229*$t);
   $Y0 += 0.00000003898*cos(3.18081892914 + 98690.52433462809*$t);
   $Y0 += 0.00000005352*cos(3.60477090908 + 77837.35505133009*$t);
   $Y0 += 0.00000003694*cos(5.20175219531 + 58946.76070187749*$t);
   $Y0 += 0.00000003631*cos(3.98277710501 + 129380.37759516528*$t);
   $Y0 += 0.00000004821*cos(5.22008496092 + 37410.32342239509*$t);
   $Y0 += 0.00000003323*cos(0.38355389097 + 78793.64420730069*$t);
   $Y0 += 0.00000003932*cos(5.47485976543 + 29530.7219040231*$t);
   $Y0 += 0.00000004022*cos(2.49293913730 + 2119.00767786191*$t);
   $Y0 += 0.00000003208*cos(1.18605815284 + 25557.96835899609*$t);
   $Y0 += 0.00000003611*cos(4.88816178501 + 91785.70468379749*$t);
   $Y0 += 0.00000003062*cos(4.35403926224 + 79323.33517239528*$t);
   $Y0 += 0.00000003480*cos(5.37504374301 + 71980.87739221469*$t);
   $Y0 += 0.00000003093*cos(0.73248333106 + 94138.57083756929*$t);
   $Y0 += 0.00000003172*cos(4.57251658041 + 27043.2590656993*$t);
   $Y0 += 0.00000002912*cos(1.92332998999 + 286967.17837479962*$t);
   $Y0 += 0.00000003746*cos(4.51336113078 + 89586.61734051049*$t);
   $Y0 += 0.00000002870*cos(3.94317758843 + 69160.04624808849*$t);
   $Y0 += 0.00000002730*cos(4.31583707063 + 44937.3745090319*$t);
   $Y0 += 0.00000002725*cos(3.85377970873 + 103822.16541868569*$t);
   $Y0 += 0.00000003214*cos(0.16060042614 + 2218.51328670329*$t);
   $Y0 += 0.00000002660*cos(3.25887021687 + 52156.38033973211*$t);
   $Y0 += 0.00000003257*cos(1.14198557197 + 25934.3681485729*$t);
   $Y0 += 0.00000002620*cos(1.06710403438 + 20426.32727493849*$t);
   $Y0 += 0.00000003227*cos(1.38048915969 + 26241.9257695425*$t);
   $Y0 += 0.00000003086*cos(4.73969429884 + 77308.35350059748*$t);
   $Y0 += 0.00000003099*cos(5.58109262188 + 131549.13807744789*$t);
   $Y0 += 0.00000002731*cos(4.90353573524 + 149.8070146181*$t);
   $Y0 += 0.00000002486*cos(1.55056807904 + 1052.51220067191*$t);
   $Y0 += 0.00000003167*cos(1.96024135338 + 33326.33491569069*$t);
   $Y0 += 0.00000002680*cos(5.36534815866 + 52195.71986153169*$t);
   $Y0 += 0.00000002231*cos(1.86712683980 + 136101.09157450669*$t);
   $Y0 += 0.00000002653*cos(0.11236418844 + 7.3573644843*$t);
   $Y0 += 0.00000002074*cos(3.39236434641 + 48733.47515566649*$t);
   $Y0 += 0.00000001974*cos(3.59882556210 + 64742.2018006147*$t);
   $Y0 += 0.00000002064*cos(1.78573813736 + 51116.18053547569*$t);
   $Y0 += 0.00000002533*cos(4.44951960940 + 1066.7392946735*$t);
   $Y0 += 0.00000001949*cos(0.11634548370 + 58459.1259506233*$t);
   $Y0 += 0.00000002010*cos(0.87459613624 + 16984.2399649401*$t);
   $Y0 += 0.00000001876*cos(5.20487934678 + 82815.90673926489*$t);
   $Y0 += 0.00000001832*cos(0.38381879945 + 52179.9314359899*$t);
   $Y0 += 0.00000002306*cos(2.63023850427 + 54394.80720481869*$t);
   $Y0 += 0.00000001837*cos(3.96249421836 + 66941.2891439017*$t);
   $Y0 += 0.00000001783*cos(0.37307285570 + 25620.1819687033*$t);
   $Y0 += 0.00000001826*cos(1.95614381567 + 52172.1687652739*$t);
   $Y0 += 0.00000001791*cos(3.75150085341 + 25661.5487681817*$t);
   $Y0 += 0.00000002126*cos(1.34366019415 + 51536.15281431789*$t);
   $Y0 += 0.00000001714*cos(2.14968206427 + 26556.11194941211*$t);
   $Y0 += 0.00000001972*cos(3.46789993631 + 45405.3394993863*$t);
   $Y0 += 0.00000001711*cos(2.68399636879 + 32858.36992533629*$t);
   $Y0 += 0.00000001643*cos(1.83203729093 + 52602.6482915079*$t);
   $Y0 += 0.00000001665*cos(2.26005496586 + 5327.7199258663*$t);
   $Y0 += 0.00000001930*cos(4.21706155195 + 13656.10430865991*$t);
   $Y0 += 0.00000001655*cos(0.02018823486 + 125887.80602829569*$t);
   $Y0 += 0.00000001554*cos(1.82939701370 + 13521.9952590749*$t);
   $Y0 += 0.00000001517*cos(1.82763021291 + 53765.1229959157*$t);
   $Y0 += 0.00000001870*cos(2.52871717392 + 129330.38097326108*$t);
   $Y0 += 0.00000001705*cos(1.43301700747 + 43071.65547154729*$t);
   $Y0 += 0.00000001683*cos(2.75504809483 + 52168.93655363109*$t);
   $Y0 += 0.00000001449*cos(4.53368571731 + 88477.23878841709*$t);
   $Y0 += 0.00000001466*cos(3.70132103086 + 24499.0740637739*$t);
   $Y0 += 0.00000001655*cos(4.55736013213 + 76.50988875911*$t);
   $Y0 += 0.00000001539*cos(0.55097320740 + 105307.45612538888*$t);
   $Y0 += 0.00000001542*cos(0.39515897687 + 102762.78348849648*$t);
   $Y0 += 0.00000001400*cos(5.97424163846 + 26081.51840015769*$t);
   $Y0 += 0.00000001400*cos(2.83094248939 + 26094.77551795771*$t);
   $Y0 += 0.00000001415*cos(6.03024008680 + 49957.2929964451*$t);
   $Y0 += 0.00000001545*cos(1.17380884450 + 85502.6288338061*$t);
   $Y0 += 0.00000001259*cos(2.49869507757 + 51962.7510051939*$t);
   $Y0 += 0.00000001464*cos(0.37249136511 + 103925.25819290428*$t);
   $Y0 += 0.00000001265*cos(4.66607647315 + 35192.0539531753*$t);
   $Y0 += 0.00000001563*cos(6.24487877351 + 6681.46867088311*$t);
   $Y0 += 0.00000001517*cos(5.71237882590 + 213.0552779545*$t);
   $Y0 += 0.00000001171*cos(6.01640449002 + 57836.89451481709*$t);
   $Y0 += 0.00000001547*cos(1.86619704850 + 52389.3491960699*$t);
   $Y0 += 0.00000001423*cos(0.03012497835 + 50057.2862402535*$t);
   $Y0 += 0.00000001238*cos(5.55628734215 + 52183.1636476327*$t);
   $Y0 += 0.00000001070*cos(4.25732715878 + 26728.0442453717*$t);
   $Y0 += 0.00000001020*cos(5.20453548111 + 12431.79883291429*$t);
   $Y0 += 0.00000000965*cos(0.75029404016 + 155468.28073673949*$t);
   $Y0 += 0.00000001018*cos(0.65795794031 + 24498.58642880689*$t);
   $Y0 += 0.00000000994*cos(1.65652991676 + 117873.60782537168*$t);
   $Y0 += 0.00000001002*cos(0.27317864857 + 208702.98131511007*$t);
   $Y0 += 0.00000001214*cos(4.15542472749 + 25661.06113321469*$t);
   $Y0 += 0.00000000958*cos(6.23240896197 + 124778.42747620228*$t);
   $Y0 += 0.00000000893*cos(1.95641235916 + 85034.66384345168*$t);
   $Y0 += 0.00000000875*cos(0.15313552239 + 74821.37829724069*$t);
   $Y0 += 0.00000000871*cos(1.85017833510 + 1059.1381127057*$t);
   $Y0 += 0.00000000910*cos(2.14250029739 + 98068.78053378889*$t);
   $Y0 += 0.00000000827*cos(4.01703921795 + 22645.08437912529*$t);
   $Y0 += 0.00000000829*cos(3.41829988722 + 104881.54734887488*$t);
   $Y0 += 0.00000000870*cos(0.60198611419 + 51066.18391357149*$t);
   $Y0 += 0.00000000978*cos(4.58656747537 + 38520.1896094555*$t);
   $Y0 += 0.00000000915*cos(2.91341983071 + 6.86972951729*$t);
   $Y0 += 0.00000000855*cos(0.96267173742 + 51123.7817174435*$t);
   $Y0 += 0.00000000795*cos(3.78227948828 + 120226.47397914348*$t);
   $Y0 += 0.00000000778*cos(4.69135126506 + 11322.9079157879*$t);
   $Y0 += 0.00000000913*cos(3.89740262984 + 52026.4869034973*$t);
   $Y0 += 0.00000000930*cos(1.27443982259 + 115674.52048208468*$t);
   $Y0 += 0.00000000932*cos(2.23433073748 + 55618.6250455973*$t);
   $Y0 += 0.00000000707*cos(4.97407500954 + 313055.08151637383*$t);
   $Y0 += 0.00000000803*cos(2.35027685221 + 157637.04121902208*$t);
   $Y0 += 0.00000000730*cos(3.77427452793 + 53284.94101775829*$t);
   $Y0 += 0.00000000725*cos(1.12701074857 + 105411.23831396949*$t);
   $Y0 += 0.00000000788*cos(4.40761141586 + 3340.8562441833*$t);
   $Y0 += 0.00000000867*cos(4.35296523561 + 51109.55462344189*$t);
   $Y0 += 0.00000000718*cos(0.61085091780 + 129910.06856025988*$t);
   $Y0 += 0.00000000688*cos(5.33062177858 + 419.72846135871*$t);
   $Y0 += 0.00000000710*cos(0.69656198994 + 95247.94938966268*$t);
   $Y0 += 0.00000000694*cos(0.69216961450 + 26617.35028918529*$t);
   $Y0 += 0.00000000621*cos(2.81767179045 + 4551.7096795753*$t);
   $Y0 += 0.00000000661*cos(5.92198588119 + 10213.0417287275*$t);
   $Y0 += 0.00000000674*cos(5.58162526148 + 28206.9108194361*$t);
   $Y0 += 0.00000000609*cos(0.02645166365 + 78244.28348130629*$t);
   $Y0 += 0.00000000661*cos(4.19316030438 + 52022.2712901471*$t);
   $Y0 += 0.00000000741*cos(1.50795914520 + 103396.25664217169*$t);
   $Y0 += 0.00000000585*cos(4.91842789278 + 162188.99471608088*$t);
   $Y0 += 0.00000000599*cos(0.26652159243 + 46848.5739922491*$t);
   $Y0 += 0.00000000634*cos(5.63013895989 + 39609.41076568209*$t);
   $Y0 += 0.00000000731*cos(1.84936364859 + 433.9555553603*$t);
   $Y0 += 0.00000000591*cos(1.08868730454 + 71025.27765060609*$t);
   $Y0 += 0.00000000576*cos(4.10239875091 + 47623.60896860609*$t);
   $Y0 += 0.00000000545*cos(3.92064766910 + 48732.98752069949*$t);
   $Y0 += 0.00000000528*cos(3.25755205747 + 23754.95056618569*$t);
   $Y0 += 0.00000000531*cos(1.51927796885 + 41962.27691945389*$t);
   $Y0 += 0.00000000525*cos(0.65657950540 + 10022.0810975829*$t);
   $Y0 += 0.00000000578*cos(5.67944788678 + 80482.71034639288*$t);
   $Y0 += 0.00000000675*cos(4.42923882594 + 52329.8289111167*$t);
   $Y0 += 0.00000000664*cos(4.39586206751 + 77624.05595589208*$t);
   $Y0 += 0.00000000541*cos(4.50825185806 + 6282.83203250789*$t);
   $Y0 += 0.00000000639*cos(2.57562729428 + 45494.82524723229*$t);
   $Y0 += 0.00000000594*cos(1.74048603817 + 7994.77225950771*$t);
   $Y0 += 0.00000000467*cos(3.18640247451 + 84547.0290921975*$t);
   $Y0 += 0.00000000552*cos(4.24369907572 + 52099.7840293563*$t);
   $Y0 += 0.00000000490*cos(1.39410440209 + 64608.09275102969*$t);
   $Y0 += 0.00000000553*cos(0.82663110400 + 1692.40948698591*$t);
   $Y0 += 0.00000000456*cos(0.37337128477 + 90830.10494218889*$t);
   $Y0 += 0.00000000470*cos(1.72250117653 + 26237.7101561923*$t);
   $Y0 += 0.00000000501*cos(5.75878971797 + 25132.0595824821*$t);
   $Y0 += 0.00000000615*cos(2.13316170849 + 78283.62300310588*$t);
   $Y0 += 0.00000000441*cos(4.69790158258 + 27140.41534224609*$t);
   $Y0 += 0.00000000445*cos(3.07156557764 + 151975.70916986988*$t);
   $Y0 += 0.00000000437*cos(0.95616885323 + 20760.1832157079*$t);
   $Y0 += 0.00000000472*cos(5.57949973231 + 155418.28411483529*$t);
   $Y0 += 0.00000000428*cos(4.96656096346 + 77204.08367704989*$t);
   $Y0 += 0.00000000469*cos(1.97026697646 + 108903.80988083909*$t);
   $Y0 += 0.00000000478*cos(2.05637713133 + 63498.22656396929*$t);
   $Y0 += 0.00000000448*cos(5.75669088007 + 78256.83969520529*$t);
   $Y0 += 0.00000000591*cos(0.35948760681 + 853.4401992355*$t);
   $Y0 += 0.00000000438*cos(4.57721971918 + 51645.87150057029*$t);
   $Y0 += 0.00000000440*cos(0.13387750376 + 28306.41642827749*$t);
   $Y0 += 0.00000000413*cos(5.17208909967 + 38519.70197448849*$t);
   $Y0 += 0.00000000456*cos(0.44231533666 + 26038.1503371535*$t);
   $Y0 += 0.00000000456*cos(2.07975292935 + 26138.1435809619*$t);
   $Y0 += 0.00000000470*cos(2.65781109220 + 58946.27306691049*$t);
   $Y0 += 0.00000000434*cos(0.74419794380 + 29428.7593857575*$t);
   $Y0 += 0.00000000439*cos(5.54617295689 + 2333.44021035551*$t);
   $Y0 += 0.00000000406*cos(1.00123517971 + 1582.2031657665*$t);
   $Y0 += 0.00000000419*cos(3.46554019433 + 128850.68663007069*$t);
   $Y0 += 0.00000000403*cos(5.55657271431 + 28421.3433519297*$t);
   $Y0 += 0.00000000423*cos(5.90337303076 + 50586.48957038109*$t);
   $Y0 += 0.00000000420*cos(3.43444455433 + 78267.83457756409*$t);
   $Y0 += 0.00000000508*cos(5.53374236541 + 26720.93069837089*$t);
   $Y0 += 0.00000000445*cos(2.66109199380 + 1109.6223695769*$t);
   $Y0 += 0.00000000452*cos(4.25647147725 + 25985.0541848391*$t);
   $Y0 += 0.00000000440*cos(5.74038643901 + 78050.65414676809*$t);
   $Y0 += 0.00000000427*cos(1.26284550681 + 27154.6424362477*$t);
   $Y0 += 0.00000000422*cos(0.73601792678 + 93029.19228547589*$t);
   $Y0 += 0.00000000387*cos(1.92885240062 + 22747.5345323579*$t);
   $Y0 += 0.00000000451*cos(4.61446843236 + 26191.23973327629*$t);
   $Y0 += 0.00000000525*cos(3.37057650618 + 25455.3632197445*$t);
   $Y0 += 0.00000000439*cos(4.25286780683 + 46514.23041651269*$t);
   $Y0 += 0.00000000390*cos(3.42340819919 + 130013.16133447849*$t);
   $Y0 += 0.00000000364*cos(4.88702121021 + 78690.55143308209*$t);
   $Y0 += 0.00000000468*cos(3.04146774326 + 76145.18938182769*$t);
   $Y0 += 0.00000000361*cos(1.30085103028 + 114565.14192999128*$t);
   $Y0 += 0.00000000354*cos(5.19811092004 + 52644.0150909863*$t);
   $Y0 += 0.00000000418*cos(5.00690657163 + 78260.07190684808*$t);
   $Y0 += 0.00000000366*cos(2.14542297373 + 25028.27739390149*$t);
   $Y0 += 0.00000000394*cos(4.92599580786 + 78477.25233764409*$t);
   $Y0 += 0.00000000337*cos(4.88605522032 + 79853.02613748988*$t);
   $Y0 += 0.00000000356*cos(3.76575150987 + 633.0275567967*$t);
   $Y0 += 0.00000000427*cos(5.87777060793 + 19804.5834740993*$t);
   $Y0 += 0.00000000360*cos(4.50952146319 + 48835.43767393209*$t);
   $Y0 += 0.00000000431*cos(0.05038435146 + 30639.61282114949*$t);
   $Y0 += 0.00000000336*cos(0.49943786696 + 18208.05780571871*$t);
   $Y0 += 0.00000000317*cos(2.79783111172 + 76045.1961380193*$t);
   $Y0 += 0.00000000403*cos(3.61003449042 + 131395.35926696309*$t);
   $Y0 += 0.00000000300*cos(3.06549658713 + 25973.70767037247*$t);
   $Y0 += 0.00000000365*cos(3.99788805051 + 25565.5695409639*$t);
   $Y0 += 0.00000000417*cos(0.23545014867 + 71493.2426409605*$t);
   $Y0 += 0.00000000298*cos(5.73633616578 + 26202.58624774291*$t);
   $Y0 += 0.00000000411*cos(4.96911720848 + 1596.43025976811*$t);
   $Y0 += 0.00000000286*cos(5.88170536088 + 52182.67865953191*$t);
   $Y0 += 0.00000000304*cos(3.41185962602 + 51708.08511027749*$t);
   $Y0 += 0.00000000288*cos(1.37917320218 + 26164.4130303333*$t);
   $Y0 += 0.00000000326*cos(4.77493319595 + 26610.72437715149*$t);
   $Y0 += 0.00000000375*cos(2.00750267367 + 26162.928557625*$t);
   $Y0 += 0.00000000268*cos(4.70687038628 + 143961.51096694589*$t);
   $Y0 += 0.00000000367*cos(4.20390802587 + 111590.53197538029*$t);
   $Y0 += 0.00000000264*cos(3.36238088617 + 72936.4771338233*$t);
   $Y0 += 0.00000000301*cos(1.01728716251 + 224.5886131854*$t);
   $Y0 += 0.00000000286*cos(0.96880689193 + 46848.0863572821*$t);
   $Y0 += 0.00000000257*cos(3.80096482898 + 181556.18387831368*$t);
   $Y0 += 0.00000000352*cos(0.48255743874 + 26013.3653604904*$t);
   $Y0 += 0.00000000274*cos(5.81956733021 + 956.53297345411*$t);
   $Y0 += 0.00000000253*cos(4.93056899504 + 75.0254160508*$t);
   $Y0 += 0.00000000256*cos(1.10512464961 + 537.0483295789*$t);
   $Y0 += 0.00000000245*cos(3.26928135038 + 51756.5654567567*$t);
   $Y0 += 0.00000000286*cos(2.73949837375 + 52169.42154173189*$t);
   $Y0 += 0.00000000326*cos(0.00381222715 + 25771.7550894011*$t);
   $Y0 += 0.00000000252*cos(5.36725270563 + 25668.6623151825*$t);
   $Y0 += 0.00000000239*cos(1.88156493524 + 42154.2128205325*$t);
   $Y0 += 0.00000000324*cos(2.52294776653 + 26404.5388287143*$t);
   $Y0 += 0.00000000285*cos(6.14793784305 + 1162.7185218913*$t);
   $Y0 += 0.00000000237*cos(5.19203538966 + 124156.68367536309*$t);
   $Y0 += 0.00000000271*cos(3.04131804133 + 15874.37377787969*$t);
   $Y0 += 0.00000000307*cos(2.31415827156 + 78271.06678920689*$t);
   $Y0 += 0.00000000323*cos(3.00259946219 + 19406.43447069109*$t);
   $Y0 += 0.00000000245*cos(5.02045917872 + 111122.56698502589*$t);
   $Y0 += 0.00000000229*cos(5.15166243945 + 36300.94487030169*$t);
   $Y0 += 0.00000000243*cos(4.26179828093 + 7238.9194090835*$t);
   $Y0 += 0.00000000242*cos(1.18373210037 + 51748.96427478889*$t);
   $Y0 += 0.00000000275*cos(4.01647026193 + 77211.68485901768*$t);
   $Y0 += 0.00000000234*cos(2.92710924584 + 83924.79765639128*$t);
   $Y0 += 0.00000000279*cos(1.12090122634 + 77197.45776501608*$t);
   $Y0 += 0.00000000260*cos(2.91177438950 + 32769.3718124573*$t);
   $Y0 += 0.00000000222*cos(3.18512792848 + 100909.28143881488*$t);
   $Y0 += 0.00000000279*cos(3.09797269121 + 26507.63160293289*$t);
   $Y0 += 0.00000000299*cos(3.03086575636 + 2648.6986429565*$t);
   $Y0 += 0.00000000213*cos(5.24729841378 + 1479.11039154791*$t);
   $Y0 += 0.00000000221*cos(4.03293449116 + 102.84895673509*$t);
   $Y0 += 0.00000000231*cos(4.31940320133 + 141762.42362365888*$t);
   $Y0 += 0.00000000208*cos(0.53989181963 + 146314.37712071768*$t);
   $Y0 += 0.00000000236*cos(4.88594462204 + 14764.9952257863*$t);
   $Y0 += 0.00000000219*cos(5.60242546056 + 110.45013870291*$t);
   $Y0 += 0.00000000241*cos(3.00055987343 + 150866.33061777649*$t);
   $Y0 += 0.00000000210*cos(5.40322983591 + 183724.94436059629*$t);
   $Y0 += 0.00000000280*cos(0.66543647361 + 78114.39004507149*$t);
   $Y0 += 0.00000000211*cos(0.17674723492 + 130969.45049044909*$t);
   $Y0 += 0.00000000244*cos(5.94927988906 + 11610.7957758577*$t);
   $Y0 += 0.00000000265*cos(0.98893666490 + 39744.0074502341*$t);
   $Y0 += 0.00000000200*cos(2.15844889143 + 16065.82204399129*$t);
   $Y0 += 0.00000000222*cos(5.27780086013 + 81706.5281871715*$t);
   $Y0 += 0.00000000197*cos(1.03744786641 + 52815.9473869459*$t);
   $Y0 += 0.00000000197*cos(4.45583276555 + 12565.9078824993*$t);
   $Y0 += 0.00000000241*cos(5.45668008692 + 27147.04125427989*$t);
   $Y0 += 0.00000000205*cos(2.01894158165 + 33968.23611239669*$t);
   $Y0 += 0.00000000248*cos(5.59907777274 + 71582.72838880648*$t);
   $Y0 += 0.00000000246*cos(0.67064557518 + 51742.33836275509*$t);
   $Y0 += 0.00000000203*cos(0.77451026070 + 25881.9614106205*$t);
   $Y0 += 0.00000000202*cos(4.54831491675 + 25448.2496727437*$t);
   $Y0 += 0.00000000228*cos(3.34387624216 + 234790.88445668427*$t);
   $Y0 += 0.00000000198*cos(1.71260487848 + 26294.33250749489*$t);
   $Y0 += 0.00000000231*cos(0.78401706031 + 44181.52165860769*$t);
   $Y0 += 0.00000000185*cos(5.15550823271 + 64607.60511606269*$t);
   $Y0 += 0.00000000226*cos(2.64165383084 + 7879.84533585549*$t);
   $Y0 += 0.00000000176*cos(3.73240626851 + 77154.08705514569*$t);
   $Y0 += 0.00000000179*cos(3.53913956860 + 14477.1073657165*$t);
   $Y0 += 0.00000000195*cos(5.08298233604 + 59414.23805726489*$t);
   $Y0 += 0.00000000207*cos(4.24194657171 + 1376.0176173293*$t);
   $Y0 += 0.00000000174*cos(1.73934628427 + 339142.98465794802*$t);
   $Y0 += 0.00000000174*cos(5.68665306442 + 51543.2663613187*$t);
   $Y0 += 0.00000000184*cos(4.55688502110 + 129484.15978374588*$t);
   $Y0 += 0.00000000173*cos(6.20260804596 + 23968.89546371229*$t);
   $Y0 += 0.00000000207*cos(0.17517194401 + 20760.6708506749*$t);
   $Y0 += 0.00000000166*cos(0.73482223187 + 79372.84415933248*$t);
   $Y0 += 0.00000000185*cos(1.16294236415 + 103711.95909746629*$t);
   $Y0 += 0.00000000161*cos(0.99358007086 + 73711.51211018028*$t);
   $Y0 += 0.00000000194*cos(1.69156637131 + 50483.8844311295*$t);
   $Y0 += 0.00000000220*cos(1.40610479581 + 69159.55861312148*$t);
   $Y0 += 0.00000000191*cos(3.65637031484 + 155997.97170183409*$t);
   $Y0 += 0.00000000216*cos(4.60747679624 + 53131.16220727349*$t);
   $Y0 += 0.00000000156*cos(2.45974681457 + 106570.61348796709*$t);
   $Y0 += 0.00000000181*cos(1.41036114643 + 18093.13088206649*$t);
   $Y0 += 0.00000000152*cos(0.95845975049 + 78110.17443172129*$t);
   $Y0 += 0.00000000149*cos(2.99675052546 + 45892.48661567349*$t);
   $Y0 += 0.00000000182*cos(3.74672593763 + 121335.85253123689*$t);
   $Y0 += 0.00000000155*cos(1.68611064922 + 188276.89785765509*$t);
   $Y0 += 0.00000000178*cos(4.17805278254 + 131499.14145554368*$t);
   $Y0 += 0.00000000166*cos(2.18208828557 + 468.20880783791*$t);
   $Y0 += 0.00000000157*cos(1.37406612212 + 61279.9570947495*$t);
   $Y0 += 0.00000000203*cos(2.15909999017 + 51322.85371887989*$t);
   $Y0 += 0.00000000190*cos(0.30919081370 + 25874.36022865269*$t);
   $Y0 += 0.00000000142*cos(1.51656559474 + 50594.09075234889*$t);
   $Y0 += 0.00000000175*cos(3.62001720200 + 26624.9514711531*$t);
   $Y0 += 0.00000000148*cos(3.07498569808 + 104332.1866228805*$t);
   $Y0 += 0.00000000153*cos(3.71257893953 + 52101.26850206459*$t);
   $Y0 += 0.00000000164*cos(3.90190440252 + 74820.89066227368*$t);
   $Y0 += 0.00000000158*cos(1.19350301362 + 78417.73205269089*$t);
   $Y0 += 0.00000000140*cos(0.89912686582 + 52279.14287485049*$t);
   $Y0 += 0.00000000163*cos(5.63286881634 + 25780.58933808811*$t);
   $Y0 += 0.00000000160*cos(3.16772159097 + 26395.7045800273*$t);
   $Y0 += 0.00000000132*cos(3.07363434632 + 22760.01130277751*$t);
   $Y0 += 0.00000000129*cos(0.10582004023 + 54394.31956985169*$t);
   $Y0 += 0.00000000150*cos(5.18178770285 + 104371.52614468009*$t);
   $Y0 += 0.00000000166*cos(5.37015346630 + 522.8212355773*$t);
   $Y0 += 0.00000000149*cos(2.52020201514 + 23889.0596157707*$t);
   $Y0 += 0.00000000125*cos(2.56126838362 + 65697.31390725628*$t);
   $Y0 += 0.00000000127*cos(2.35920317858 + 54294.81396101029*$t);
   $Y0 += 0.00000000136*cos(6.13995819179 + 102233.09252340188*$t);
   $Y0 += 0.00000000140*cos(2.25411762839 + 52808.83383994509*$t);
   $Y0 += 0.00000000171*cos(1.01179953646 + 78187.68717093048*$t);
   $Y0 += 0.00000000119*cos(6.12499272961 + 178063.61231144409*$t);
   $Y0 += 0.00000000121*cos(2.34497789442 + 181506.18725640948*$t);
   $Y0 += 0.00000000150*cos(1.37100107530 + 13362.6935242827*$t);
   $Y0 += 0.00000000160*cos(0.50398727524 + 955.8435590921*$t);
   $Y0 += 0.00000000122*cos(0.97167224102 + 52072.9573264133*$t);
   $Y0 += 0.00000000139*cos(4.14538916469 + 97113.18079218028*$t);
   $Y0 += 0.00000000116*cos(2.49897944668 + 104344.74283677948*$t);
   $Y0 += 0.00000000127*cos(4.93091662093 + 26521.8586969345*$t);
   $Y0 += 0.00000000113*cos(0.24728903100 + 154938.58977164488*$t);
   $Y0 += 0.00000000116*cos(6.24971983447 + 110634.93223377169*$t);
   $Y0 += 0.00000000120*cos(3.16836542455 + 23969.3830986793*$t);
   $Y0 += 0.00000000117*cos(0.78253817148 + 18093.6185170335*$t);
   $Y0 += 0.00000000114*cos(5.52498204591 + 25754.2910182883*$t);
   $Y0 += 0.00000000150*cos(5.32072185109 + 25551.34244696229*$t);
   $Y0 += 0.00000000109*cos(3.57166541301 + 40565.01050729069*$t);
   $Y0 += 0.00000000110*cos(0.79184574430 + 24176.9474758405*$t);
   $Y0 += 0.00000000107*cos(4.83424409375 + 68242.11596210669*$t);
   $Y0 += 0.00000000112*cos(3.27382494684 + 26422.0028998271*$t);
   $Y0 += 0.00000000107*cos(4.40903077996 + 90695.99589260388*$t);
   $Y0 += 0.00000000104*cos(0.18580141979 + 156101.06447605268*$t);
   $Y0 += 0.00000000111*cos(3.42810291181 + 116918.00808376308*$t);
   $Y0 += 0.00000000121*cos(6.28088254389 + 28287.2343023447*$t);
   $Y0 += 0.00000000108*cos(1.36988749165 + 44181.03402364069*$t);
   $Y0 += 0.00000000102*cos(2.58046151243 + 26198.3532802771*$t);
   $Y0 += 0.00000000143*cos(3.83368380796 + 50579.86365834729*$t);
   $Y0 += 0.00000000108*cos(3.14844806883 + 26107.32908499049*$t);
   $Y0 += 0.00000000103*cos(5.73149554363 + 29416.28261533789*$t);
   $Y0 += 0.00000000102*cos(1.26920320545 + 74923.34081550628*$t);
   $Y0 += 0.00000000111*cos(1.57596233643 + 51639.24558853649*$t);
   $Y0 += 0.00000000100*cos(3.85312326105 + 25035.8785758693*$t);
   $Y0 += 0.00000000097*cos(3.56740191654 + 323.74923414091*$t);
   $Y0 += 0.00000000100*cos(1.69705145148 + 104565.15547921829*$t);
   $Y0 += 0.00000000099*cos(5.11526101592 + 31415.6230674405*$t);
   $Y0 += 0.00000000120*cos(5.01910336702 + 134991.71302241329*$t);
   $Y0 += 0.00000000131*cos(1.06382274034 + 647.25465079831*$t);
   $Y0 += 0.00000000128*cos(2.55933728310 + 104138.55728834229*$t);
   $Y0 += 0.00000000110*cos(4.87488188396 + 24357.0246061251*$t);
   $Y0 += 0.00000000105*cos(1.05131979960 + 26067.9895631909*$t);
   $Y0 += 0.00000000095*cos(4.10632947324 + 52705.25343075949*$t);
   $Y0 += 0.00000000105*cos(3.71934987002 + 55516.6625273317*$t);
   $Y0 += 0.00000000093*cos(3.49141182565 + 52126.05347872769*$t);
   $Y0 += 0.00000000092*cos(2.31943050291 + 54509.2464935039*$t);
   $Y0 += 0.00000000096*cos(1.91265055140 + 743.23387801611*$t);
   $Y0 += 0.00000000109*cos(5.90844457054 + 632.5399218297*$t);
   $Y0 += 0.00000000090*cos(1.71341920893 + 27999.3464422749*$t);
   $Y0 += 0.00000000094*cos(1.83646395516 + 103291.98681862408*$t);
   $Y0 += 0.00000000111*cos(2.83140360723 + 40565.4981422577*$t);
   $Y0 += 0.00000000101*cos(1.49775137503 + 51951.70530492999*$t);
   $Y0 += 0.00000000102*cos(0.19980570400 + 104355.73771913828*$t);
   $Y0 += 0.00000000088*cos(2.55079662642 + 1588.82907780029*$t);
   $Y0 += 0.00000000102*cos(3.43450786393 + 26941.3433408097*$t);
   $Y0 += 0.00000000109*cos(5.12423170717 + 60056.13925397089*$t);
   $Y0 += 0.00000000093*cos(3.91123861975 + 27780.31262856009*$t);
   $Y0 += 0.00000000089*cos(4.72535332511 + 68050.18006102808*$t);
   $Y0 += 0.00000000106*cos(2.15819157914 + 6770.9544187291*$t);
   $Y0 += 0.00000000101*cos(5.08955449331 + 52250.8316991992*$t);
   $Y0 += 0.00000000086*cos(4.79490272689 + 52325.61329776649*$t);
   $Y0 += 0.00000000092*cos(4.35572616043 + 140653.04507156549*$t);
   $Y0 += 0.00000000107*cos(0.38434884214 + 157483.26240853728*$t);
   $Y0 += 0.00000000087*cos(0.68859108180 + 25938.5837619231*$t);
   $Y0 += 0.00000000094*cos(2.98031225005 + 51219.76094466129*$t);
   $Y0 += 0.00000000089*cos(3.67089011438 + 36109.9842391571*$t);
   $Y0 += 0.00000000093*cos(1.66575202299 + 103.3365917021*$t);
   $Y0 += 0.00000000098*cos(3.92942306643 + 27819.2693119903*$t);
   $Y0 += 0.00000000084*cos(1.50244162097 + 53228.31848382029*$t);
   $Y0 += 0.00000000087*cos(3.14637767899 + 52065.84377941249*$t);
   $Y0 += 0.00000000099*cos(3.29163755467 + 97581.14578253469*$t);
   $Y0 += 0.00000000105*cos(3.86326312300 + 70269.42480018188*$t);
   $Y0 += 0.00000000083*cos(2.64246416261 + 51219.96272405629*$t);
   $Y0 += 0.00000000101*cos(5.08528632726 + 52226.0467225361*$t);
   $Y0 += 0.00000000087*cos(1.65472743108 + 104778.45457465628*$t);
   $Y0 += 0.00000000102*cos(4.59971787247 + 12432.2864678813*$t);
   $Y0 += 0.00000000082*cos(1.96514410059 + 78731.91823256049*$t);
   $Y0 += 0.00000000101*cos(3.78885420287 + 119117.0954270501*$t);
   $Y0 += 0.00000000108*cos(5.77504268406 + 49527.59527515889*$t);
   $Y0 += 0.00000000092*cos(3.01639809211 + 51859.65823097529*$t);
   $Y0 += 0.00000000080*cos(4.33221518102 + 53242.5455778219*$t);
   $Y0 += 0.00000000107*cos(4.90113452404 + 25447.76203777669*$t);
   $Y0 += 0.00000000088*cos(1.30079620909 + 12725.20961729149*$t);
   $Y0 += 0.00000000091*cos(5.49075472411 + 26300.95841952869*$t);
   $Y0 += 0.00000000080*cos(1.65550423430 + 105940.92927906409*$t);
   $Y0 += 0.00000000076*cos(5.84908430441 + 102133.09927959349*$t);
   $Y0 += 0.00000000079*cos(0.12371508890 + 80482.22271142589*$t);
   $Y0 += 0.00000000091*cos(2.66249978065 + 51013.57539622409*$t);
   $Y0 += 0.00000000094*cos(1.39522248869 + 2222.1004520805*$t);
   $Y0 += 0.00000000089*cos(0.94979154283 + 137678.43511695448*$t);
   $Y0 += 0.00000000075*cos(0.07329711774 + 3327.89183879669*$t);
   $Y0 += 0.00000000089*cos(2.43057395425 + 1910.9556657337*$t);
   $Y0 += 0.00000000098*cos(4.89673673453 + 50056.79860528649*$t);
   $Y0 += 0.00000000074*cos(6.13007861498 + 25977.94063783829*$t);
   $Y0 += 0.00000000089*cos(5.56505599077 + 52492.4419702885*$t);
   $Y0 += 0.00000000073*cos(3.56559697903 + 50697.1835265675*$t);
   $Y0 += 0.00000000102*cos(1.77223481993 + 104347.97504842229*$t);
   $Y0 += 0.00000000072*cos(1.47771204102 + 170049.41410852008*$t);
   $Y0 += 0.00000000078*cos(4.16301616898 + 103285.36090659029*$t);
   $Y0 += 0.00000000075*cos(6.26897336450 + 52595.53474450709*$t);
   $Y0 += 0.00000000074*cos(1.16312773108 + 25021.6514818677*$t);
   $Y0 += 0.00000000075*cos(0.08785923175 + 77844.46859833089*$t);
   $Y0 += 0.00000000078*cos(2.64612833915 + 74.53778108379*$t);
   $Y0 += 0.00000000094*cos(1.55832345623 + 52698.62751872569*$t);
   $Y0 += 0.00000000095*cos(1.28337020564 + 25938.09612695609*$t);
   $Y0 += 0.00000000087*cos(1.11118535241 + 72602.13355808688*$t);
   $Y0 += 0.00000000070*cos(4.10808217554 + 27670.10630734069*$t);
   $Y0 += 0.00000000076*cos(5.35798835757 + 104358.96993078108*$t);
   $Y0 += 0.00000000074*cos(3.71626517668 + 25954.0379094727*$t);
   $Y0 += 0.00000000070*cos(5.98641613952 + 66653.40128383189*$t);
   $Y0 += 0.00000000071*cos(2.40595691806 + 24998.4381678641*$t);
   $Y0 += 0.00000000073*cos(5.08664921507 + 26222.2560086427*$t);
   $Y0 += 0.00000000081*cos(6.12425664727 + 8194.0315157251*$t);
   $Y0 += 0.00000000066*cos(2.64701347180 + 78270.58180110609*$t);
   $Y0 += 0.00000000072*cos(2.95285797491 + 426.3543733925*$t);
   $Y0 += 0.00000000078*cos(3.71360449621 + 104202.29318664569*$t);
   $Y0 += 0.00000000066*cos(5.22374540666 + 89586.12970554348*$t);
   $Y0 += 0.00000000081*cos(6.08469300372 + 26080.54577708989*$t);
   $Y0 += 0.00000000068*cos(0.56774852198 + 207644.08701988788*$t);
   $Y0 += 0.00000000065*cos(4.30145540281 + 12546.72575656651*$t);
   $Y0 += 0.00000000074*cos(2.34912240985 + 97670.63153038069*$t);
   $Y0 += 0.00000000067*cos(1.25089965871 + 2111.8941308611*$t);
   $Y0 += 0.00000000079*cos(3.71647319510 + 77830.24150432929*$t);
   $Y0 += 0.00000000083*cos(5.35734577753 + 522.3336006103*$t);
   $Y0 += 0.00000000079*cos(2.22871999356 + 25446.73339731871*$t);
   $Y0 += 0.00000000063*cos(1.37131699662 + 70268.93716521488*$t);
   $Y0 += 0.00000000074*cos(4.43425667540 + 95247.46175469569*$t);
   $Y0 += 0.00000000085*cos(1.03066114675 + 72935.98949885629*$t);
   $Y0 += 0.00000000062*cos(1.93537775234 + 150244.58681693728*$t);
   $Y0 += 0.00000000081*cos(1.53102925511 + 77733.77464214448*$t);
   $Y0 += 0.00000000077*cos(5.87942788213 + 316.6356871401*$t);
   $Y0 += 0.00000000075*cos(2.61550986340 + 33967.74847742969*$t);
   $Y0 += 0.00000000071*cos(1.76514499411 + 27684.3334013423*$t);
   $Y0 += 0.00000000061*cos(2.80789864051 + 846.3266522347*$t);
   $Y0 += 0.00000000064*cos(0.11131885479 + 27177.8557502513*$t);
   $Y0 += 0.00000000073*cos(6.03174712424 + 26083.7779887327*$t);
   $Y0 += 0.00000000075*cos(0.28818227298 + 26729.56052079669*$t);
   $Y0 += 0.00000000071*cos(3.36506897458 + 17893.8716258491*$t);
   $Y0 += 0.00000000063*cos(2.50388232144 + 52290.48938931711*$t);
   $Y0 += 0.00000000072*cos(4.56353970752 + 50800.27630078609*$t);
   $Y0 += 0.00000000059*cos(6.11443002788 + 52061.61081194667*$t);
   $Y0 += 0.00000000077*cos(1.18313398787 + 38813.6003938327*$t);
   $Y0 += 0.00000000080*cos(4.87250848454 + 26089.63143176599*$t);
   $Y0 += 0.00000000069*cos(3.16809879958 + 56727.51596272369*$t);
   $Y0 += 0.00000000080*cos(3.94160294784 + 26086.6624863494*$t);
   $Y0 += 0.00000000058*cos(3.98600001818 + 419.2408263917*$t);
   $Y0 += 0.00000000066*cos(0.10285600557 + 2008.8013566425*$t);
   $Y0 += 0.00000000065*cos(3.63512616728 + 66652.91364886488*$t);
   $Y0 += 0.00000000058*cos(0.05018228362 + 99024.38027539749*$t);
   $Y0 += 0.00000000077*cos(0.77887041561 + 103299.58800059189*$t);
   $Y0 += 0.00000000069*cos(3.03766697714 + 316.14805217309*$t);
   $Y0 += 0.00000000061*cos(6.24933126885 + 126997.18458038908*$t);
   $Y0 += 0.00000000068*cos(1.78962458422 + 137210.47012660008*$t);
   $Y0 += 0.00000000058*cos(1.07398748672 + 167850.32676523308*$t);
   $Y0 += 0.00000000057*cos(1.06178740528 + 26011.8808877821*$t);
   $Y0 += 0.00000000062*cos(1.20172371901 + 19316.9487228451*$t);
   $Y0 += 0.00000000073*cos(4.45582042270 + 26091.5406594487*$t);
   $Y0 += 0.00000000055*cos(2.17291421134 + 209812.84750217047*$t);
   $Y0 += 0.00000000055*cos(3.55134929549 + 44295.9609472929*$t);
   $Y0 += 0.00000000058*cos(6.25406726323 + 25508.4593720589*$t);
   $Y0 += 0.00000000073*cos(2.05415256446 + 18848.98373249069*$t);
   $Y0 += 0.00000000056*cos(3.84149994570 + 22625.9022531925*$t);
   $Y0 += 0.00000000064*cos(5.21615906084 + 77410.75686045409*$t);
   $Y0 += 0.00000000054*cos(4.44562432663 + 52252.31617190749*$t);
   $Y0 += 0.00000000066*cos(0.17594911232 + 77795.98825185168*$t);
   $Y0 += 0.00000000055*cos(2.04378126827 + 107794.43132874569*$t);
   $Y0 += 0.00000000057*cos(3.42931686715 + 24449.0774418697*$t);
   $Y0 += 0.00000000055*cos(3.59192378031 + 172402.28026229188*$t);
   $Y0 += 0.00000000067*cos(2.93531319099 + 19.42594341629*$t);
   $Y0 += 0.00000000053*cos(3.70608451935 + 32370.73517408209*$t);
   $Y0 += 0.00000000052*cos(0.79212421214 + 3340.36860921629*$t);
   $Y0 += 0.00000000055*cos(3.60943961817 + 220.6564599223*$t);
   $Y0 += 0.00000000072*cos(3.16918804459 + 24864.32911827909*$t);
   $Y0 += 0.00000000052*cos(3.80711022251 + 23868.9022199039*$t);
   $Y0 += 0.00000000060*cos(5.14905673779 + 90695.50825763689*$t);
   $Y0 += 0.00000000052*cos(4.10578873491 + 26312.4917547596*$t);
   $Y0 += 0.00000000053*cos(0.28962121715 + 19406.9221056581*$t);
   $Y0 += 0.00000000053*cos(2.56330425325 + 26667.8345460565*$t);
   $Y0 += 0.00000000070*cos(0.08146254525 + 2199.33116077051*$t);
   $Y0 += 0.00000000059*cos(3.54477372961 + 38.3768531213*$t);
   $Y0 += 0.00000000051*cos(5.94337924391 + 58857.2749540315*$t);
   $Y0 += 0.00000000050*cos(4.20235040741 + 129799.86223904048*$t);
   $Y0 += 0.00000000067*cos(5.79256239224 + 78257.32468330608*$t);
   $Y0 += 0.00000000050*cos(5.35090323068 + 27727.2164762457*$t);
   $Y0 += 0.00000000048*cos(2.60487489461 + 27044.4361150283*$t);
   $Y0 += 0.00000000047*cos(4.10703352371 + 51969.8645521947*$t);
   $Y0 += 0.00000000055*cos(3.21272296258 + 157057.35363202328*$t);
   $Y0 += 0.00000000060*cos(2.71680609162 + 85034.17620848469*$t);
   $Y0 += 0.00000000049*cos(2.29047787617 + 77631.16950289288*$t);
   $Y0 += 0.00000000064*cos(4.73468584640 + 76571.78757270369*$t);
   $Y0 += 0.00000000046*cos(4.38909300675 + 77836.86741636308*$t);
   $Y0 += 0.00000000054*cos(4.21805017615 + 23919.3864767751*$t);
   $Y0 += 0.00000000047*cos(1.32745528662 + 155572.06292532009*$t);
   $Y0 += 0.00000000064*cos(5.63611607616 + 27311.96479983631*$t);
   $Y0 += 0.00000000046*cos(4.51657960486 + 76681.99389392308*$t);
   $Y0 += 0.00000000046*cos(0.05397462972 + 29530.2342690561*$t);
   $Y0 += 0.00000000053*cos(3.43095934900 + 49842.85370775989*$t);
   $Y0 += 0.00000000055*cos(4.95374342010 + 27037.32256802749*$t);
   $Y0 += 0.00000000054*cos(2.11552363976 + 949.4194264533*$t);
   $Y0 += 0.00000000047*cos(4.95775520700 + 29550.3916649229*$t);
   $Y0 += 0.00000000056*cos(2.31169429193 + 53234.94439585409*$t);
   $Y0 += 0.00000000062*cos(1.59919946431 + 26011.39325281509*$t);
   $Y0 += 0.00000000051*cos(6.10777700397 + 28736.6017845307*$t);
   $Y0 += 0.00000000045*cos(2.54247709679 + 55504.18575691209*$t);
   $Y0 += 0.00000000062*cos(6.05167319913 + 176954.23375935068*$t);
   $Y0 += 0.00000000043*cos(4.09072234194 + 78903.8505285201*$t);
   $Y0 += 0.00000000044*cos(2.10728026202 + 16703.3059509825*$t);
   $Y0 += 0.00000000052*cos(6.07065313687 + 110012.70079796549*$t);
   $Y0 += 0.00000000050*cos(3.89612671834 + 25138.9713500879*$t);
   $Y0 += 0.00000000043*cos(2.78443984846 + 1272.9248431107*$t);
   $Y0 += 0.00000000044*cos(3.76849382783 + 105460.74730090669*$t);
   $Y0 += 0.00000000058*cos(2.82347260176 + 26094.77287109149*$t);
   $Y0 += 0.00000000042*cos(0.46194308183 + 1486.2239385487*$t);
   $Y0 += 0.00000000042*cos(0.51106392440 + 78189.17164363878*$t);
   $Y0 += 0.00000000048*cos(4.03335182320 + 65831.91059180829*$t);
   $Y0 += 0.00000000054*cos(4.84278312652 + 52382.23564906909*$t);
   $Y0 += 0.00000000041*cos(1.77651394552 + 1795.5022612045*$t);
   $Y0 += 0.00000000043*cos(2.60404615127 + 60055.65161900389*$t);
   $Y0 += 0.00000000046*cos(4.86931523516 + 54087.2495838491*$t);
   $Y0 += 0.00000000053*cos(0.13354671820 + 260878.78759825832*$t);
   $Y0 += 0.00000000048*cos(5.22216564191 + 25466.4031582185*$t);
   $Y0 += 0.00000000048*cos(3.58305204289 + 26709.8907598969*$t);
   $Y0 += 0.00000000048*cos(0.57485699057 + 76667.76679992149*$t);
   $Y0 += 0.00000000047*cos(4.05954911572 + 104275.59031250469*$t);
   $Y0 += 0.00000000039*cos(5.49383143740 + 132658.51662954128*$t);
   $Y0 += 0.00000000040*cos(0.07089879206 + 26824.02347258949*$t);
   $Y0 += 0.00000000040*cos(2.44203081578 + 25352.2704455259*$t);
   $Y0 += 0.00000000043*cos(4.79238528651 + 365230.88779952223*$t);
   $Y0 += 0.00000000044*cos(0.28163075339 + 52712.85461272729*$t);
   $Y0 += 0.00000000045*cos(4.58626302576 + 28256.9074413403*$t);
   $Y0 += 0.00000000039*cos(0.56315956692 + 103241.99019671988*$t);
   $Y0 += 0.00000000051*cos(0.42021554272 + 182085.87484340828*$t);
   $Y0 += 0.00000000050*cos(0.52050720367 + 147423.75567281108*$t);
   $Y0 += 0.00000000043*cos(4.74188084964 + 214364.80099922928*$t);
   $Y0 += 0.00000000039*cos(4.16197164473 + 99799.41525175449*$t);
   $Y0 += 0.00000000044*cos(3.72596196534 + 25867.73431661889*$t);
   $Y0 += 0.00000000043*cos(0.20439431464 + 1265.81129610991*$t);
   $Y0 += 0.00000000038*cos(4.24623905840 + 104505.63519426509*$t);
   $Y0 += 0.00000000037*cos(4.01161399115 + 104198.07757329549*$t);
   $Y0 += 0.00000000037*cos(2.96320182560 + 128320.99566497609*$t);
   $Y0 += 0.00000000042*cos(3.62647853730 + 25170.2166730759*$t);
   $Y0 += 0.00000000040*cos(3.65913470233 + 206.42936592071*$t);
   $Y0 += 0.00000000038*cos(1.95151248327 + 130459.42928625426*$t);
   $Y0 += 0.00000000037*cos(2.36549717602 + 26102.3740530593*$t);
   $Y0 += 0.00000000050*cos(1.81815722451 + 153.5349930013*$t);
   $Y0 += 0.00000000037*cos(0.10000949167 + 24203.24579564031*$t);
   $Y0 += 0.00000000041*cos(5.40449209098 + 34282.4222922663*$t);
   $Y0 += 0.00000000036*cos(1.44241697006 + 26118.4738200621*$t);
   $Y0 += 0.00000000036*cos(1.07961540756 + 26057.82009805329*$t);
   $Y0 += 0.00000000036*cos(6.15630697749 + 34281.9346572993*$t);
   $Y0 += 0.00000000042*cos(1.45143122814 + 38653.81102407349*$t);
   $Y0 += 0.00000000035*cos(0.16883470965 + 26073.91986505609*$t);
   $Y0 += 0.00000000039*cos(5.17224451182 + 27006.0772450395*$t);
   $Y0 += 0.00000000039*cos(3.83113919893 + 151.2914873264*$t);
   $Y0 += 0.00000000037*cos(3.86078277718 + 78367.04601642469*$t);
   $Y0 += 0.00000000034*cos(1.99485322788 + 27567.01353312209*$t);
   $Y0 += 0.00000000035*cos(5.28138513945 + 78896.73698151928*$t);
   $Y0 += 0.00000000037*cos(2.43731961667 + 25773.96077918809*$t);
   $Y0 += 0.00000000037*cos(0.08471276095 + 26402.33313892731*$t);
   $Y0 += 0.00000000045*cos(5.51053584744 + 2126.1212248627*$t);
   $Y0 += 0.00000000037*cos(0.41442508719 + 9384.59719059169*$t);
   $Y0 += 0.00000000033*cos(1.23373917586 + 116783.89903417809*$t);
   $Y0 += 0.00000000035*cos(1.05079898841 + 27463.9207589035*$t);
   $Y0 += 0.00000000033*cos(5.24254633919 + 51852.54468397449*$t);
   $Y0 += 0.00000000037*cos(6.12798409572 + 130420.08976445469*$t);
   $Y0 += 0.00000000035*cos(2.14843472729 + 62388.84801187589*$t);
   $Y0 += 0.00000000033*cos(0.42458452244 + 26190.1094773233*$t);
   $Y0 += 0.00000000033*cos(2.09744785518 + 25986.18444079209*$t);
   $Y0 += 0.00000000035*cos(1.60180137355 + 51653.47268253809*$t);
   $Y0 += 0.00000000032*cos(4.47872593179 + 39629.56816154889*$t);
   $Y0 += 0.00000000040*cos(2.83686756644 + 148.32254190981*$t);
   $Y0 += 0.00000000043*cos(0.95235131647 + 157587.04459711789*$t);
   $Y0 += 0.00000000034*cos(2.42204384323 + 27973.04812247509*$t);
   $Y0 += 0.00000000043*cos(5.25286806261 + 26308.5596014965*$t);
   $Y0 += 0.00000000040*cos(2.56508188611 + 205.9417309537*$t);
   $Y0 += 0.00000000034*cos(5.11562530130 + 13541.17738500769*$t);
   $Y0 += 0.00000000031*cos(5.39821911963 + 207594.09039798369*$t);
   $Y0 += 0.00000000032*cos(2.88931723147 + 204151.51545301828*$t);
   $Y0 += 0.00000000034*cos(2.40270209936 + 51868.4924796623*$t);
   $Y0 += 0.00000000043*cos(1.44438681078 + 51529.03926731709*$t);
   $Y0 += 0.00000000031*cos(4.01104942140 + 78160.86046798748*$t);
   $Y0 += 0.00000000036*cos(5.64714725231 + 130226.46042991648*$t);
   $Y0 += 0.00000000030*cos(5.53920573257 + 130432.64597835368*$t);
   $Y0 += 0.00000000039*cos(4.02419894411 + 64901.5035354069*$t);
   $Y0 += 0.00000000031*cos(0.59714240312 + 51433.0600400993*$t);
   $Y0 += 0.00000000041*cos(1.72851918476 + 40852.8983673605*$t);
   $Y0 += 0.00000000039*cos(3.48396647568 + 76674.39271195528*$t);
   $Y0 += 0.00000000031*cos(1.66062366332 + 94330.01910368088*$t);
   $Y0 += 0.00000000030*cos(3.30916696393 + 181026.49291321907*$t);
   $Y0 += 0.00000000030*cos(1.35962398425 + 96356.84030678908*$t);
   $Y0 += 0.00000000029*cos(3.24046570458 + 182188.96761762688*$t);
   $Y0 += 0.00000000034*cos(5.68539193875 + 111122.07935005888*$t);
   $Y0 += 0.00000000031*cos(5.77478962928 + 26514.25751496669*$t);
   $Y0 += 0.00000000036*cos(2.47015155723 + 75615.49841673308*$t);
   $Y0 += 0.00000000028*cos(2.60390025059 + 86143.55476057807*$t);
   $Y0 += 0.00000000028*cos(5.41996849568 + 80382.71710258449*$t);
   $Y0 += 0.00000000037*cos(2.73064236635 + 37698.6989174319*$t);
   $Y0 += 0.00000000030*cos(5.69224082014 + 77101.47853779829*$t);
   $Y0 += 0.00000000029*cos(2.84649684108 + 114.6429243969*$t);
   $Y0 += 0.00000000030*cos(2.60710276130 + 26084.21480579459*$t);
   $Y0 += 0.00000000027*cos(2.18911511513 + 26241.4381345755*$t);
   $Y0 += 0.00000000027*cos(6.24126624865 + 60369.83779887349*$t);
   $Y0 += 0.00000000027*cos(4.34927842452 + 101011.24395708049*$t);
   $Y0 += 0.00000000027*cos(0.05549126696 + 52509.9060414013*$t);
   $Y0 += 0.00000000029*cos(6.20759958805 + 26092.07911232079*$t);
   $Y0 += 0.00000000033*cos(1.33379538421 + 42153.72518556549*$t);
   $Y0 += 0.00000000033*cos(1.11257666070 + 49954.19346603489*$t);
   $Y0 += 0.00000000027*cos(1.97164111012 + 25933.8805136059*$t);
   $Y0 += 0.00000000027*cos(5.75010433988 + 91785.21704883048*$t);
   $Y0 += 0.00000000026*cos(0.45274965388 + 35472.9879671329*$t);
   $Y0 += 0.00000000034*cos(5.96598332393 + 22004.1584523533*$t);
   $Y0 += 0.00000000026*cos(6.17560870969 + 71980.38975724769*$t);
   $Y0 += 0.00000000026*cos(4.75787796399 + 130653.05862079248*$t);
   $Y0 += 0.00000000029*cos(3.02581288540 + 136722.83537534589*$t);
   $Y0 += 0.00000000035*cos(6.21411204570 + 52483.6077216015*$t);
   $Y0 += 0.00000000027*cos(3.05142430504 + 54375.1374439189*$t);
   $Y0 += 0.00000000026*cos(4.35081548550 + 26050.0139234199*$t);
   $Y0 += 0.00000000033*cos(4.63692423082 + 77727.14873011068*$t);
   $Y0 += 0.00000000031*cos(4.44084625304 + 87367.86023632369*$t);
   $Y0 += 0.00000000028*cos(3.43792139137 + 183571.16555011149*$t);
   $Y0 += 0.00000000034*cos(4.71651658587 + 1905.70858242391*$t);
   $Y0 += 0.00000000028*cos(0.48686633466 + 51226.87449166209*$t);
   $Y0 += 0.00000000032*cos(6.03865727615 + 77307.66408623548*$t);
   $Y0 += 0.00000000033*cos(4.55124740226 + 78039.60844650418*$t);
   $Y0 += 0.00000000031*cos(2.31561412140 + 60370.3254338405*$t);
   $Y0 += 0.00000000033*cos(0.91420804533 + 123201.08393375449*$t);
   $Y0 += 0.00000000027*cos(5.63137827099 + 1224.06165826211*$t);
   $Y0 += 0.00000000025*cos(1.86918416559 + 78338.73484077338*$t);
   $Y0 += 0.00000000029*cos(3.85756830106 + 49842.36607279289*$t);
   $Y0 += 0.00000000025*cos(0.32014728581 + 76785.08666814168*$t);
   $Y0 += 0.00000000024*cos(1.09872540797 + 24824.9895964795*$t);
   $Y0 += 0.00000000031*cos(1.78245808102 + 161079.61616398748*$t);
   $Y0 += 0.00000000031*cos(0.13179619883 + 75930.75684933408*$t);
   $Y0 += 0.00000000024*cos(1.67322759859 + 1045.3986536711*$t);
   $Y0 += 0.00000000029*cos(4.01431074745 + 25600.5122078035*$t);
   $Y0 += 0.00000000029*cos(4.79090693735 + 26575.7817103119*$t);
   $Y0 += 0.00000000026*cos(4.40340514614 + 39450.5966658569*$t);
   $Y0 += 0.00000000027*cos(5.45015263034 + 27171.2271913513*$t);
   $Y0 += 0.00000000027*cos(3.35506505446 + 25005.0667267641*$t);
   $Y0 += 0.00000000032*cos(2.36425003443 + 9103.6631766341*$t);
   $Y0 += 0.00000000024*cos(0.92163258785 + 129373.26404816448*$t);
   $Y0 += 0.00000000023*cos(1.97682509801 + 85502.14119883909*$t);
   $Y0 += 0.00000000024*cos(1.70355204155 + 52609.7618385087*$t);
   $Y0 += 0.00000000025*cos(3.42869920852 + 299.37021175271*$t);
   $Y0 += 0.00000000027*cos(0.19474964949 + 143005.91122533729*$t);
   $Y0 += 0.00000000027*cos(6.01066977576 + 24073.1652872599*$t);
   $Y0 += 0.00000000025*cos(3.87710167982 + 25654.43522118089*$t);
   $Y0 += 0.00000000024*cos(5.68395366955 + 52286.25642185129*$t);
   $Y0 += 0.00000000023*cos(0.49611836531 + 38813.11275886569*$t);
   $Y0 += 0.00000000026*cos(5.95483109773 + 27676.73221937449*$t);
   $Y0 += 0.00000000023*cos(1.42348474713 + 27351.3043216359*$t);
   $Y0 += 0.00000000025*cos(6.12366793793 + 48847.9144443517*$t);
   $Y0 += 0.00000000028*cos(2.64147333193 + 24925.6722546499*$t);
   $Y0 += 0.00000000029*cos(2.84086630674 + 30172.1354657621*$t);
   $Y0 += 0.00000000026*cos(6.21853790794 + 78153.74692098668*$t);
   $Y0 += 0.00000000023*cos(1.26116961525 + 50064.3997872543*$t);
   $Y0 += 0.00000000024*cos(3.54764052332 + 51962.26337022689*$t);
   $Y0 += 0.00000000023*cos(6.03986636871 + 77947.56137254949*$t);
   $Y0 += 0.00000000023*cos(0.06002764715 + 123669.04892410889*$t);
   $Y0 += 0.00000000022*cos(0.26166099168 + 78213.95662030189*$t);
   $Y0 += 0.00000000025*cos(4.36791869246 + 26725.1432310819*$t);
   $Y0 += 0.00000000025*cos(4.43729899234 + 25451.1506870335*$t);
   $Y0 += 0.00000000022*cos(3.84821265527 + 50264.8506174147*$t);
   $Y0 += 0.00000000030*cos(0.70456092924 + 53907.1724535645*$t);
   $Y0 += 0.00000000030*cos(3.98527104604 + 23754.46293121869*$t);
   $Y0 += 0.00000000025*cos(1.76026444680 + 1.7282901918*$t);
   $Y0 += 0.00000000025*cos(4.90185710039 + 1.24065522479*$t);
   $Y0 += 0.00000000024*cos(4.62065948726 + 78786.53066029989*$t);
   $Y0 += 0.00000000022*cos(1.83132826724 + 6885.39370741431*$t);
   $Y0 += 0.00000000025*cos(4.82521606002 + 130435.87818999648*$t);
   $Y0 += 0.00000000022*cos(1.64259204266 + 50444.9277476993*$t);
   $Y0 += 0.00000000028*cos(4.68306520525 + 6129.5408569901*$t);
   $Y0 += 0.00000000025*cos(2.79403215015 + 28103.1286308555*$t);
   $Y0 += 0.00000000023*cos(0.48266477387 + 103918.14464590348*$t);
   $Y0 += 0.00000000025*cos(0.72627176075 + 52174.56562792359*$t);
   $Y0 += 0.00000000022*cos(5.90022439787 + 2219.0009216703*$t);
   $Y0 += 0.00000000022*cos(2.31815942379 + 78580.34511186268*$t);
   $Y0 += 0.00000000029*cos(4.48338221638 + 1730.87853544909*$t);
   $Y0 += 0.00000000022*cos(5.37074911110 + 80597.14963507809*$t);
   $Y0 += 0.00000000024*cos(2.29466576857 + 51842.1941598625*$t);
   $Y0 += 0.00000000020*cos(5.64534264808 + 51955.63745819309*$t);
   $Y0 += 0.00000000028*cos(3.75812407504 + 536.5606946119*$t);
   $Y0 += 0.00000000020*cos(5.51240603881 + 23439.20449861769*$t);
   $Y0 += 0.00000000023*cos(3.83447858759 + 129387.49114216608*$t);
   $Y0 += 0.00000000021*cos(3.97425392925 + 163766.33825852869*$t);
   $Y0 += 0.00000000025*cos(0.44166212589 + 81604.56566890588*$t);
   $Y0 += 0.00000000021*cos(1.44153243351 + 1063.55790093581*$t);
   $Y0 += 0.00000000023*cos(3.39790605489 + 16342.8264032011*$t);
   $Y0 += 0.00000000022*cos(0.58776873075 + 96357.32794175608*$t);
   $Y0 += 0.00000000023*cos(2.71345954853 + 76144.70174686068*$t);
   $Y0 += 0.00000000020*cos(6.01126822986 + 51329.9672658807*$t);
   $Y0 += 0.00000000020*cos(0.14753852345 + 1089.95260867711*$t);
   $Y0 += 0.00000000020*cos(5.38196984803 + 123758.53467195488*$t);
   $Y0 += 0.00000000019*cos(2.29671913556 + 51535.66517935089*$t);
   $Y0 += 0.00000000022*cos(5.56163358805 + 3462.0008883817*$t);
   $Y0 += 0.00000000021*cos(0.48380924328 + 130290.19632821988*$t);
   $Y0 += 0.00000000023*cos(2.36039511683 + 50167.49256147289*$t);
   $Y0 += 0.00000000020*cos(4.52844625012 + 196137.31725009429*$t);
   $Y0 += 0.00000000023*cos(1.12689921440 + 166740.94821313968*$t);
   $Y0 += 0.00000000026*cos(1.32836783140 + 22747.04689739089*$t);
   $Y0 += 0.00000000022*cos(4.77036018900 + 26126.2799946955*$t);
   $Y0 += 0.00000000022*cos(4.14491043187 + 26735.1577923725*$t);
   $Y0 += 0.00000000019*cos(3.09805264388 + 78683.43788608129*$t);
   $Y0 += 0.00000000020*cos(3.83026958578 + 75930.26921436709*$t);
   $Y0 += 0.00000000024*cos(0.55752006618 + 145204.99856862429*$t);
   $Y0 += 0.00000000019*cos(2.12244186746 + 130446.87307235529*$t);
   $Y0 += 0.00000000024*cos(1.47596121866 + 52177.53457334019*$t);
   $Y0 += 0.00000000025*cos(3.25287024656 + 130443.64086071249*$t);
   $Y0 += 0.00000000020*cos(4.21784136757 + 98690.03669966108*$t);
   $Y0 += 0.00000000019*cos(1.95226766220 + 103498.66000202828*$t);
   $Y0 += 0.00000000025*cos(1.32070214805 + 76888.17944236028*$t);
   $Y0 += 0.00000000022*cos(5.54888786998 + 24952.4700871645*$t);
   $Y0 += 0.00000000022*cos(3.25632981482 + 27223.8238309509*$t);
   $Y0 += 0.00000000018*cos(1.58091743094 + 78413.51643934069*$t);
   $Y0 += 0.00000000020*cos(5.01791562590 + 104819.8213741347*$t);
   $Y0 += 0.00000000022*cos(4.95873027034 + 129379.88996019828*$t);
   $Y0 += 0.00000000019*cos(4.66235459384 + 26248.5543284425*$t);
   $Y0 += 0.00000000019*cos(4.14286309096 + 25927.7395896729*$t);
   $Y0 += 0.00000000019*cos(0.21911919049 + 53029.2464823839*$t);
   $Y0 += 0.00000000018*cos(2.61852920213 + 128221.00242116769*$t);
   $Y0 += 0.00000000020*cos(4.36521030279 + 25908.0698287731*$t);
   $Y0 += 0.00000000020*cos(4.44000738201 + 26268.2240893423*$t);
   $Y0 += 0.00000000019*cos(3.36352476280 + 52817.46366237089*$t);
   $Y0 += 0.00000000018*cos(0.30820161935 + 26411.6523757151*$t);
   $Y0 += 0.00000000022*cos(3.17947641824 + 103932.37173990508*$t);
   $Y0 += 0.00000000018*cos(0.03030733311 + 52195.23222656469*$t);
   $Y0 += 0.00000000024*cos(1.85700392205 + 86144.04239554508*$t);
   $Y0 += 0.00000000020*cos(5.09555302131 + 12412.6167069815*$t);
   $Y0 += 0.00000000025*cos(1.79034244495 + 78313.94986411028*$t);
   $Y0 += 0.00000000020*cos(3.61245706996 + 233731.99016146208*$t);
   $Y0 += 0.00000000020*cos(5.31801711889 + 25234.9505773057*$t);
   $Y0 += 0.00000000017*cos(0.72137211711 + 53868.21577013429*$t);
   $Y0 += 0.00000000017*cos(5.76650726054 + 77307.86586563049*$t);
   $Y0 += 0.00000000017*cos(5.48277957945 + 24602.8562523545*$t);
   $Y0 += 0.00000000017*cos(3.32243810535 + 27573.4376657609*$t);
   $Y0 += 0.00000000023*cos(0.08846433271 + 14.47091148511*$t);
   $Y0 += 0.00000000018*cos(5.51755241491 + 52137.9170649941*$t);
   $Y0 += 0.00000000020*cos(4.71185211530 + 130866.35771623049*$t);
   $Y0 += 0.00000000016*cos(1.12213372181 + 79330.44871939608*$t);
   $Y0 += 0.00000000018*cos(4.58737692359 + 79316.22162539448*$t);
   $Y0 += 0.00000000016*cos(4.01365446910 + 3.6375178745*$t);
   $Y0 += 0.00000000017*cos(1.24075088152 + 121335.36489626988*$t);
   $Y0 += 0.00000000019*cos(4.71477962898 + 132028.83242063828*$t);
   $Y0 += 0.00000000018*cos(1.49937345876 + 102659.69071427788*$t);
   $Y0 += 0.00000000018*cos(4.05618608857 + 99023.89264043048*$t);
   $Y0 += 0.00000000016*cos(6.02071363086 + 50903.3690750047*$t);
   $Y0 += 0.00000000019*cos(1.66314996355 + 94138.08320260228*$t);
   $Y0 += 0.00000000016*cos(4.53088777953 + 28310.00359365469*$t);
   $Y0 += 0.00000000016*cos(0.64879776309 + 25668.17468021549*$t);
   $Y0 += 0.00000000016*cos(4.77268559673 + 24395.9812895553*$t);
   $Y0 += 0.00000000019*cos(1.08691679951 + 78793.15657233368*$t);
   $Y0 += 0.00000000015*cos(0.94917563884 + 155887.76538061468*$t);
   $Y0 += 0.00000000022*cos(4.51216513703 + 1055.6935944096*$t);
   $Y0 += 0.00000000016*cos(1.31190457742 + 65717.47130312308*$t);
   $Y0 += 0.00000000021*cos(3.55864300611 + 3178.38960805111*$t);
   $Y0 += 0.00000000019*cos(0.15049069469 + 106570.12585300008*$t);
   $Y0 += 0.00000000015*cos(2.07036309837 + 50690.06997956669*$t);
   $Y0 += 0.00000000017*cos(1.75768691030 + 55638.2948064971*$t);
   $Y0 += 0.00000000016*cos(5.70225976378 + 104358.4849426803*$t);
   $Y0 += 0.00000000015*cos(5.84363368019 + 6043.9847638919*$t);
   $Y0 += 0.00000000016*cos(5.00133834312 + 176332.48995851149*$t);
   $Y0 += 0.00000000017*cos(0.43244881435 + 62197.88738073129*$t);
   $Y0 += 0.00000000017*cos(0.00750879129 + 86457.74094044768*$t);
   $Y0 += 0.00000000015*cos(1.85651180492 + 52310.1591502169*$t);
   $Y0 += 0.00000000015*cos(5.22197013820 + 235900.75064374466*$t);
   $Y0 += 0.00000000015*cos(3.15110803520 + 61560.8911087071*$t);
   $Y0 += 0.00000000017*cos(4.69846105326 + 103821.67778371868*$t);
   $Y0 += 0.00000000017*cos(3.00907412235 + 153085.08772196328*$t);
   $Y0 += 0.00000000020*cos(6.14864822525 + 3.6883357796*$t);
   $Y0 += 0.00000000014*cos(4.11502292960 + 193938.22990680728*$t);
   $Y0 += 0.00000000015*cos(1.04578402465 + 50007.2896183493*$t);
   $Y0 += 0.00000000015*cos(4.77727640495 + 26831.1370195903*$t);
   $Y0 += 0.00000000014*cos(5.09443528119 + 133882.3344703199*$t);
   $Y0 += 0.00000000017*cos(5.07246677176 + 39629.08052658189*$t);
   $Y0 += 0.00000000016*cos(0.34319621032 + 77.9943614674*$t);
   $Y0 += 0.00000000015*cos(3.15841157482 + 125112.2834169717*$t);
   $Y0 += 0.00000000014*cos(0.72227336641 + 50380.79165691089*$t);
   $Y0 += 0.00000000016*cos(2.56012166691 + 104345.22782488028*$t);
   $Y0 += 0.00000000014*cos(0.38043611029 + 26247.4486938499*$t);
   $Y0 += 0.00000000014*cos(2.14159626733 + 25928.8452242655*$t);
   $Y0 += 0.00000000014*cos(5.50060239494 + 16066.3096789583*$t);
   $Y0 += 0.00000000016*cos(2.29540093320 + 50910.4826220055*$t);
   $Y0 += 0.00000000018*cos(3.69702904204 + 92740.81679043909*$t);
   $Y0 += 0.00000000014*cos(4.21703179087 + 1169.83206889211*$t);
   $Y0 += 0.00000000016*cos(2.14445815847 + 53815.1196178199*$t);
   $Y0 += 0.00000000014*cos(2.88257919019 + 78149.51395352086*$t);
   $Y0 += 0.00000000016*cos(2.81981718118 + 203042.13690092487*$t);
   $Y0 += 0.00000000013*cos(1.94504793857 + 639.6534688305*$t);
   $Y0 += 0.00000000015*cos(4.50829948617 + 77616.94240889128*$t);
   $Y0 += 0.00000000014*cos(6.26019296208 + 82815.41910429788*$t);
   $Y0 += 0.00000000014*cos(0.03534802891 + 29415.79498037089*$t);
   $Y0 += 0.00000000014*cos(3.63252326636 + 102755.66994149568*$t);
   $Y0 += 0.00000000014*cos(3.29326661713 + 109.9625037359*$t);
   $Y0 += 0.00000000017*cos(4.87563601376 + 163298.37326817428*$t);
   $Y0 += 0.00000000015*cos(0.10955371106 + 11.2895177474*$t);
   $Y0 += 0.00000000014*cos(3.59146334775 + 173511.65881438529*$t);
   $Y0 += 0.00000000014*cos(2.82070907634 + 52161.82300663029*$t);
   $Y0 += 0.00000000012*cos(3.84809894730 + 102018.17235594128*$t);
   $Y0 += 0.00000000017*cos(4.21238157873 + 52155.8927047651*$t);
   $Y0 += 0.00000000013*cos(1.30925967982 + 102769.89703549728*$t);
   $Y0 += 0.00000000014*cos(5.55450230200 + 78378.3925308913*$t);
   $Y0 += 0.00000000014*cos(1.64692338972 + 26077.1012587938*$t);
   $Y0 += 0.00000000014*cos(0.87510898790 + 26099.1926593216*$t);
   $Y0 += 0.00000000013*cos(5.41473568620 + 79322.84753742829*$t);
   $Y0 += 0.00000000016*cos(5.27893423231 + 51534.63653889291*$t);
   $Y0 += 0.00000000014*cos(2.87967584826 + 20043.9183776823*$t);
   $Y0 += 0.00000000016*cos(3.28350334233 + 50050.17269325269*$t);
   $Y0 += 0.00000000013*cos(5.27868567961 + 44936.88687406489*$t);
   $Y0 += 0.00000000014*cos(4.48508459393 + 24506.18761077469*$t);
   $Y0 += 0.00000000014*cos(5.39693340588 + 35833.4675149143*$t);
   $Y0 += 0.00000000012*cos(2.53842730053 + 52388.86156110289*$t);
   $Y0 += 0.00000000014*cos(0.36435456631 + 198490.18340386607*$t);
   $Y0 += 0.00000000013*cos(5.25497593084 + 103719.07264446709*$t);
   $Y0 += 0.00000000013*cos(0.67641393113 + 24491.96051677309*$t);
   $Y0 += 0.00000000013*cos(4.56168735677 + 52025.99926853029*$t);
   $Y0 += 0.00000000012*cos(5.97483232665 + 25984.3647704771*$t);
   $Y0 += 0.00000000012*cos(2.83038535815 + 26191.9291476383*$t);
   $Y0 += 0.00000000014*cos(3.16933912424 + 53265.7588918255*$t);
   $Y0 += 0.00000000016*cos(4.26587881339 + 25653.94758621389*$t);
   $Y0 += 0.00000000012*cos(4.37902547397 + 181659.96606689428*$t);
   $Y0 += 0.00000000014*cos(2.41047405735 + 53399.86794141051*$t);
   $Y0 += 0.00000000012*cos(4.85231087723 + 53772.23654291649*$t);
   $Y0 += 0.00000000012*cos(0.15443687995 + 43981.7747674233*$t);
   $Y0 += 0.00000000012*cos(4.35356873262 + 28199.79727243529*$t);
   $Y0 += 0.00000000015*cos(6.01961456404 + 1638.82569970449*$t);
   $Y0 += 0.00000000016*cos(4.44894827862 + 34082.6754010819*$t);
   $Y0 += 0.00000000012*cos(3.02030436898 + 51596.3625136331*$t);
   $Y0 += 0.00000000013*cos(4.97119141190 + 52025.00243078899*$t);
   $Y0 += 0.00000000011*cos(3.07690283750 + 52911.92661416369*$t);
   $Y0 += 0.00000000011*cos(0.32794444369 + 70383.86408886709*$t);
   $Y0 += 0.00000000012*cos(2.59844082236 + 112231.45790215228*$t);
   $Y0 += 0.00000000015*cos(3.22392612987 + 103883.89139342589*$t);
   $Y0 += 0.00000000012*cos(2.91594450386 + 52171.68113030689*$t);
   $Y0 += 0.00000000014*cos(3.00638739155 + 26555.6243144451*$t);
   $Y0 += 0.00000000013*cos(3.18586045586 + 286966.69073983253*$t);
   $Y0 += 0.00000000013*cos(3.44611140023 + 208173.77798498247*$t);
   $Y0 += 0.00000000013*cos(0.91076951955 + 79219.06534884768*$t);
   $Y0 += 0.00000000011*cos(3.27827227016 + 52168.44891866409*$t);
   $Y0 += 0.00000000011*cos(1.11898912575 + 26012.87772552339*$t);
   $Y0 += 0.00000000011*cos(0.60900872596 + 48713.8053947667*$t);
   $Y0 += 0.00000000014*cos(2.44543821114 + 27998.85880730789*$t);
   $Y0 += 0.00000000014*cos(5.72095344070 + 52755.7376876307*$t);
   $Y0 += 0.00000000014*cos(0.48193051968 + 52041.9410510469*$t);
   $Y0 += 0.00000000012*cos(0.49425636709 + 131548.65044248088*$t);
   $Y0 += 0.00000000012*cos(0.83151252253 + 130363.49345407888*$t);
   $Y0 += 0.00000000011*cos(3.57268529306 + 104277.07478521299*$t);
   $Y0 += 0.00000000014*cos(2.73442839497 + 27250.62166346549*$t);
   $Y0 += 0.00000000011*cos(5.15064427323 + 51109.06698847489*$t);
   $Y0 += 0.00000000011*cos(2.07371046178 + 115674.03284711768*$t);
   $Y0 += 0.00000000014*cos(4.14980010220 + 49527.10764019189*$t);
   $Y0 += 0.00000000013*cos(5.96769060739 + 52027.97137620559*$t);
   $Y0 += 0.00000000011*cos(1.61005858699 + 57503.5262090147*$t);
   $Y0 += 0.00000000013*cos(1.63784016727 + 78470.13879064328*$t);
   $Y0 += 0.00000000014*cos(0.28439536761 + 860.55374623631*$t);
   $Y0 += 0.00000000011*cos(2.58880557782 + 92741.3044254061*$t);
   $Y0 += 0.00000000013*cos(0.61221768763 + 126996.69694542208*$t);
   $Y0 += 0.00000000011*cos(5.96318002915 + 20894.7799002599*$t);
   $Y0 += 0.00000000012*cos(5.42728450538 + 51440.17358710009*$t);
   $Y0 += 0.00000000015*cos(2.18788554104 + 24395.49365458829*$t);
   $Y0 += 0.00000000012*cos(2.65321755374 + 52179.98225389499*$t);
   $Y0 += 0.00000000013*cos(5.06034087900 + 65716.98366815608*$t);
   $Y0 += 0.00000000014*cos(5.96664556976 + 3442.3311274819*$t);
   $Y0 += 0.00000000011*cos(3.48111494779 + 736.1203310153*$t);
   $Y0 += 0.00000000010*cos(3.89350975005 + 333.6121232859*$t);
   $Y0 += 0.00000000011*cos(0.13376559849 + 50536.98058344389*$t);
   $Y0 += 0.00000000010*cos(4.16179119723 + 26190.75209830929*$t);
   $Y0 += 0.00000000012*cos(0.94216106603 + 53758.00944891489*$t);
   $Y0 += 0.00000000011*cos(0.54425828410 + 26081.03076519069*$t);
   $Y0 += 0.00000000010*cos(1.22621171910 + 78340.21931348169*$t);
   $Y0 += 0.00000000011*cos(3.31326356407 + 78800.75775430149*$t);
   $Y0 += 0.00000000012*cos(1.33333763675 + 52179.4438010229*$t);
   $Y0 += 0.00000000010*cos(2.68785159447 + 84945.17809560569*$t);
   $Y0 += 0.00000000011*cos(5.57716825787 + 81592.08889848628*$t);
   $Y0 += 0.00000000011*cos(0.93766822129 + 641.16974425549*$t);
   $Y0 += 0.00000000014*cos(6.24613692649 + 183145.25677359749*$t);
   $Y0 += 0.00000000010*cos(4.60397363665 + 25863.8021633558*$t);
   $Y0 += 0.00000000010*cos(1.98851340448 + 53093.98038661369*$t);
   $Y0 += 0.00000000011*cos(1.67023750371 + 51123.29408247649*$t);
   $Y0 += 0.00000000014*cos(5.58647092759 + 25242.0641243065*$t);
   $Y0 += 0.00000000010*cos(6.11242478468 + 154408.89880655028*$t);
   $Y0 += 0.00000000014*cos(1.36873803506 + 54344.81058291449*$t);
   $Y0 += 0.00000000011*cos(0.96041649146 + 78057.76769376888*$t);
   $Y0 += 0.00000000012*cos(6.03701341811 + 26149.87293379789*$t);
   $Y0 += 0.00000000012*cos(2.76820426669 + 26026.4209843175*$t);
   $Y0 += 0.00000000010*cos(5.41577140545 + 637.24008950771*$t);
   $Y0 += 0.00000000010*cos(0.79208503976 + 91919.81373338249*$t);
   $Y0 += 0.00000000014*cos(1.69865910615 + 53125.22570960169*$t);
   $Y0 += 0.00000000011*cos(3.68262194800 + 26094.2878829907*$t);
   $Y0 += 0.00000000012*cos(2.92120571625 + 136100.60393953968*$t);
   $Y0 += 0.00000000010*cos(2.60636817601 + 6751.28465782931*$t);
   $Y0 += 0.00000000010*cos(2.45547925443 + 137209.98249163308*$t);
   $Y0 += 0.00000000010*cos(1.99206585077 + 77940.44782554869*$t);
   $Y0 += 0.00000000010*cos(4.36323003642 + 45404.8518644193*$t);
   $Y0 += 0.00000000013*cos(4.08389676067 + 50270.5853356915*$t);
   $Y0 += 0.00000000011*cos(3.67998983142 + 77520.96318167349*$t);
   $Y0 += 0.00000000011*cos(4.23345221973 + 19958.8499195511*$t);
   $Y0 += 0.00000000010*cos(1.41797581138 + 25619.69433373629*$t);
   $Y0 += 0.00000000011*cos(3.70832983935 + 39763.6772111339*$t);
   $Y0 += 0.00000000010*cos(2.60180323904 + 54086.76194888209*$t);
   $Y0 += 0.00000000009*cos(1.01216637956 + 130593.53833583929*$t);
   $Y0 += 0.00000000009*cos(0.36490625518 + 26514.0557355717*$t);
   $Y0 += 0.00000000009*cos(2.15712612244 + 25662.2381825437*$t);
   $Y0 += 0.00000000010*cos(5.58920157802 + 101703.40155830729*$t);
   $Y0 += 0.00000000013*cos(3.18899327846 + 26161.44408491669*$t);
   $Y0 += 0.00000000013*cos(5.61622440634 + 26014.8498331987*$t);
   $Y0 += 0.00000000010*cos(0.86888264917 + 104991.75367009429*$t);
   $Y0 += 0.00000000010*cos(5.95581190338 + 52061.65099371849*$t);
   $Y0 += 0.00000000009*cos(6.14534715408 + 25131.85780308709*$t);
   $Y0 += 0.00000000011*cos(4.00644107438 + 183674.94773869208*$t);
   $Y0 += 0.00000000010*cos(5.20498017520 + 28791.7631137333*$t);
   $Y0 += 0.00000000010*cos(4.41919261340 + 4372.1201842577*$t);
   $Y0 += 0.00000000010*cos(2.27148842605 + 158746.41977111546*$t);
   $Y0 += 0.00000000009*cos(0.68354219208 + 45290.90021070109*$t);
   $Y0 += 0.00000000010*cos(3.59637305781 + 23384.5308043821*$t);
   $Y0 += 0.00000000012*cos(4.17312463574 + 76042.09660760908*$t);
   $Y0 += 0.00000000012*cos(1.90077351195 + 52396.4627430707*$t);
   $Y0 += 0.00000000013*cos(4.38477804835 + 1073.85284167431*$t);
   $Y0 += 0.00000000010*cos(1.29598384915 + 103924.77055793729*$t);
   $Y0 += 0.00000000010*cos(5.03917514860 + 91804.88680973028*$t);
   $Y0 += 0.00000000009*cos(2.39884072609 + 103189.38167937248*$t);
   $Y0 += 0.00000000009*cos(4.93035759424 + 156547.33242782846*$t);
   $Y0 += 0.00000000009*cos(0.52304323222 + 735.6326960483*$t);
   $Y0 += 0.00000000009*cos(3.52896053406 + 45494.33761226529*$t);
   $Y0 += 0.00000000009*cos(5.71417505627 + 114.68310616872*$t);
   $Y0 += 0.00000000009*cos(0.77711516175 + 130285.98071486969*$t);
   $Y0 += 0.00000000013*cos(5.11754456085 + 2168.5166647991*$t);
   $Y0 += 0.00000000010*cos(5.91004109209 + 32132.3755404331*$t);
   $Y0 += 0.00000000009*cos(2.04038725108 + 104984.64012309349*$t);
   $Y0 += 0.00000000011*cos(1.80994732330 + 19367.4329797163*$t);
   $Y0 += 0.00000000010*cos(0.87896683804 + 90989.40667698109*$t);
   $Y0 += 0.00000000009*cos(4.68949629663 + 25550.85481199529*$t);
   $Y0 += 0.00000000009*cos(5.45561348471 + 76255.39570304709*$t);
   $Y0 += 0.00000000009*cos(2.84544963842 + 31281.5140178555*$t);
   $Y0 += 0.00000000012*cos(3.79992568717 + 1699.5230339867*$t);
   $Y0 += 0.00000000011*cos(5.50627359687 + 220.16882495529*$t);
   $Y0 += 0.00000000009*cos(0.84632922145 + 51130.89526444429*$t);
   $Y0 += 0.00000000011*cos(5.96410967567 + 4022.50634944771*$t);
   $Y0 += 0.00000000009*cos(0.39890242427 + 52797.7939014711*$t);
   $Y0 += 0.00000000009*cos(3.81505771377 + 24844.6593573793*$t);
   $Y0 += 0.00000000009*cos(4.99015997103 + 27331.6345607361*$t);
   $Y0 += 0.00000000010*cos(5.97083270008 + 230239.41859459245*$t);
   $Y0 += 0.00000000009*cos(2.46893459204 + 156314.36357149068*$t);
   $Y0 += 0.00000000011*cos(1.59945570289 + 80175.15272542329*$t);
   $Y0 += 0.00000000009*cos(3.62938673151 + 129329.89333829407*$t);
   $Y0 += 0.00000000008*cos(1.08654878495 + 38634.6288981407*$t);
   $Y0 += 0.00000000011*cos(4.12180664109 + 46046.7530611253*$t);
   $Y0 += 0.00000000009*cos(5.45527892151 + 52190.2771946335*$t);
   $Y0 += 0.00000000009*cos(2.96616057426 + 25454.87558477749*$t);
   $Y0 += 0.00000000009*cos(2.58378869705 + 23439.6921335847*$t);
   $Y0 += 0.00000000008*cos(0.94557962119 + 102762.29585352947*$t);
   $Y0 += 0.00000000011*cos(1.01537219957 + 2538.4923217371*$t);
   $Y0 += 0.00000000010*cos(1.53707512245 + 391318.79094109643*$t);
   $Y0 += 0.00000000008*cos(2.58065685518 + 80174.66509045628*$t);
   $Y0 += 0.00000000008*cos(4.23319522465 + 142871.80217575228*$t);
   $Y0 += 0.00000000009*cos(4.26758067831 + 76152.30292882849*$t);
   $Y0 += 0.00000000010*cos(5.47825003046 + 54060.95126404929*$t);
   $Y0 += 0.00000000009*cos(3.23120957887 + 4.1759707466*$t);
   $Y0 += 0.00000000010*cos(5.87240869715 + 35472.50033216589*$t);
   $Y0 += 0.00000000010*cos(1.00378919066 + 125887.31839332868*$t);
   $Y0 += 0.00000000008*cos(5.81576723468 + 27360.8279846849*$t);
   $Y0 += 0.00000000009*cos(0.97115189447 + 1685.2959399851*$t);
   $Y0 += 0.00000000009*cos(0.79804703659 + 58458.63831565629*$t);
   $Y0 += 0.00000000009*cos(0.54956806871 + 25977.45300287129*$t);
   $Y0 += 0.00000000009*cos(3.97874723563 + 1485.0468892197*$t);
   $Y0 += 0.00000000008*cos(1.17507209551 + 2325.1932262991*$t);
   $Y0 += 0.00000000009*cos(2.26549130862 + 22595.5753921881*$t);
   $Y0 += 0.00000000009*cos(5.61147436738 + 52172.11794736879*$t);
   $Y0 += 0.00000000008*cos(3.83518676972 + 26934.2297938089*$t);
   $Y0 += 0.00000000008*cos(0.59545092796 + 1575.08961876571*$t);
   $Y0 += 0.00000000008*cos(5.37389235751 + 51876.9237063627*$t);
   $Y0 += 0.00000000010*cos(1.95662555086 + 2751.79141717511*$t);
   $Y0 += 0.00000000008*cos(0.89302678287 + 10681.49435404891*$t);
   $Y0 += 0.00000000011*cos(3.21704132053 + 28096.7044982167*$t);
   $Y0 += 0.00000000008*cos(5.45338504662 + 77956.3956212365*$t);
   $Y0 += 0.00000000008*cos(0.89135797231 + 52400.3948963338*$t);
   $Y0 += 0.00000000008*cos(2.89179819588 + 54824.5049261049*$t);
   $Y0 += 0.00000000010*cos(1.50538632513 + 240452.70414080346*$t);
   $Y0 += 0.00000000008*cos(5.26980517423 + 77741.37582411229*$t);
   $Y0 += 0.00000000009*cos(3.90818318828 + 24491.47288180609*$t);
   $Y0 += 0.00000000008*cos(5.13755213219 + 42791.2090925567*$t);
   $Y0 += 0.00000000008*cos(3.13545736715 + 52490.23628050151*$t);
   $Y0 += 0.00000000008*cos(2.29664532419 + 156520.54911992788*$t);
   $Y0 += 0.00000000007*cos(4.49316157626 + 52206.3769616363*$t);
   $Y0 += 0.00000000008*cos(0.58751306444 + 17098.67925362531*$t);
   $Y0 += 0.00000000007*cos(3.15082437070 + 50291.1489372145*$t);
   $Y0 += 0.00000000010*cos(2.92644275072 + 22065.8844270935*$t);
   $Y0 += 0.00000000008*cos(1.07948674297 + 51112.73601717959*$t);
   $Y0 += 0.00000000009*cos(0.57156355893 + 104454.94915799888*$t);
   $Y0 += 0.00000000010*cos(1.70608986575 + 441.06910236111*$t);
   $Y0 += 0.00000000010*cos(2.64731129677 + 49.7528044207*$t);
   $Y0 += 0.00000000007*cos(2.09922885467 + 7880.3329708225*$t);
   $Y0 += 0.00000000007*cos(6.22957682161 + 52182.67601266569*$t);
   $Y0 += 0.00000000008*cos(4.88130543295 + 52099.29639438929*$t);
   $Y0 += 0.00000000007*cos(3.72492189200 + 122445.23108333028*$t);
   $Y0 += 0.00000000008*cos(4.87864527535 + 66940.80150893469*$t);
   $Y0 += 0.00000000008*cos(4.83064762406 + 187167.51930556168*$t);
   $Y0 += 0.00000000009*cos(2.82365374916 + 156507.9929060289*$t);
   $Y0 += 0.00000000007*cos(5.45461374708 + 51086.3413094383*$t);
   $Y0 += 0.00000000008*cos(5.15492830863 + 430.7741616226*$t);
   $Y0 += 0.00000000008*cos(3.96537599750 + 149288.98707532868*$t);
   $Y0 += 0.00000000008*cos(0.69966864109 + 49956.8053614781*$t);
   $Y0 += 0.00000000007*cos(5.66541358879 + 53132.33925660249*$t);
   $Y0 += 0.00000000008*cos(4.19457404849 + 100908.79380384788*$t);
   $Y0 += 0.00000000007*cos(2.13361844991 + 233681.99353955788*$t);
   $Y0 += 0.00000000007*cos(0.76923088178 + 104248.76360956169*$t);
   $Y0 += 0.00000000008*cos(3.46520766649 + 77314.77763323628*$t);
   $Y0 += 0.00000000008*cos(4.96646407077 + 112231.94553711927*$t);
   $Y0 += 0.00000000007*cos(0.02649996061 + 29396.6128544381*$t);
   $Y0 += 0.00000000007*cos(2.49553241701 + 22779.6810636773*$t);
   $Y0 += 0.00000000009*cos(4.57445854869 + 64741.71416564769*$t);
   $Y0 += 0.00000000007*cos(4.38534662923 + 26290.4003542318*$t);
   $Y0 += 0.00000000007*cos(4.41987105557 + 25885.89356388359*$t);
   $Y0 += 0.00000000009*cos(5.15387307390 + 25863.31452838879*$t);
   $Y0 += 0.00000000007*cos(4.98170247896 + 13655.6166736929*$t);
   $Y0 += 0.00000000007*cos(1.46712503162 + 156740.96176236669*$t);
   $Y0 += 0.00000000008*cos(2.98540358627 + 78571.51086317569*$t);
   $Y0 += 0.00000000007*cos(1.57299130322 + 52663.6848518861*$t);
   $Y0 += 0.00000000007*cos(3.13857237569 + 2118.5200428949*$t);
   $Y0 += 0.00000000008*cos(3.34242363485 + 102872.98980971589*$t);
   $Y0 += 0.00000000010*cos(4.97428981699 + 2703.3723371921*$t);
   $Y0 += 0.00000000007*cos(5.14819246138 + 52074.08758236629*$t);
   $Y0 += 0.00000000007*cos(0.47864118434 + 209659.06869168568*$t);
   $Y0 += 0.00000000007*cos(2.10592218772 + 149846.43781352908*$t);
   $Y0 += 0.00000000007*cos(2.73796346194 + 77417.87040745489*$t);
   $Y0 += 0.00000000009*cos(5.86662388336 + 56260.03860733629*$t);
   $Y0 += 0.00000000007*cos(5.07150057843 + 117892.78995130445*$t);
   $Y0 += 0.00000000009*cos(1.31305807687 + 18073.9487561337*$t);
   $Y0 += 0.00000000007*cos(5.92131553777 + 917.6864684983*$t);
   $Y0 += 0.00000000008*cos(0.71246556222 + 32808.8609383991*$t);
   $Y0 += 0.00000000008*cos(0.19228169341 + 207114.39605479328*$t);
   $Y0 += 0.00000000008*cos(0.61342229584 + 3308.22207789689*$t);
   $Y0 += 0.00000000007*cos(1.34578982527 + 23267.3158149315*$t);
   $Y0 += 0.00000000007*cos(1.17624255235 + 28908.9781031839*$t);
   $Y0 += 0.00000000007*cos(4.68208072339 + 37.88921815429*$t);
   $Y0 += 0.00000000007*cos(1.41809701113 + 19202.50943415989*$t);
   $Y0 += 0.00000000007*cos(4.13036001377 + 52145.72323962749*$t);
   $Y0 += 0.00000000008*cos(2.80344738918 + 103395.56722780968*$t);
   $Y0 += 0.00000000007*cos(0.00761390447 + 208276.87075920108*$t);
   $Y0 += 0.00000000009*cos(3.38149297609 + 2014.73785431429*$t);
   $Y0 += 0.00000000007*cos(5.79510753162 + 63786.6020590061*$t);
   $Y0 += 0.00000000007*cos(1.26662001445 + 24712.3731592119*$t);
   $Y0 += 0.00000000007*cos(1.07441550954 + 127099.14709865468*$t);
   $Y0 += 0.00000000008*cos(4.81617793405 + 120417.92224525509*$t);
   $Y0 += 0.00000000008*cos(1.39078986683 + 103815.05187168489*$t);
   $Y0 += 0.00000000007*cos(2.56899142970 + 8989.22388794889*$t);
   $Y0 += 0.00000000008*cos(3.19497137388 + 579.4437695153*$t);
   $Y0 += 0.00000000007*cos(3.93035335903 + 1135.4330544097*$t);
   $Y0 += 0.00000000006*cos(3.50717502178 + 27574.1270801229*$t);
   $Y0 += 0.00000000007*cos(4.34157997430 + 102976.08258393449*$t);
   $Y0 += 0.00000000006*cos(2.26227426560 + 28774.8690535989*$t);
   $Y0 += 0.00000000006*cos(0.25975811202 + 23401.4248645165*$t);
   $Y0 += 0.00000000006*cos(6.10166450881 + 80463.04058549309*$t);
   $Y0 += 0.00000000008*cos(0.39455403552 + 51258.1198146501*$t);
   $Y0 += 0.00000000006*cos(2.04791015101 + 23704.95394428149*$t);
   $Y0 += 0.00000000006*cos(0.47412222661 + 28471.3399738339*$t);
   $Y0 += 0.00000000006*cos(3.25327779066 + 55618.1374106303*$t);
   $Y0 += 0.00000000007*cos(0.04757570199 + 53311.7269725251*$t);
   $Y0 += 0.00000000007*cos(3.96061402799 + 1802.6158082053*$t);
   $Y0 += 0.00000000008*cos(3.11255216313 + 102018.65999090829*$t);
   $Y0 += 0.00000000007*cos(3.75772081758 + 79995.0755951387*$t);
   $Y0 += 0.00000000007*cos(3.47532912865 + 52278.0126188975*$t);
   $Y0 += 0.00000000008*cos(0.17460078065 + 1162.23088692429*$t);
   $Y0 += 0.00000000006*cos(4.49776146960 + 53439.2074632101*$t);
   $Y0 += 0.00000000007*cos(0.23469102383 + 29580.7185259273*$t);
   $Y0 += 0.00000000007*cos(2.10834895065 + 27388.97377726269*$t);
   $Y0 += 0.00000000007*cos(0.41368342697 + 24787.3201408527*$t);
   $Y0 += 0.00000000006*cos(2.20070614265 + 106470.62024415868*$t);
   $Y0 += 0.00000000007*cos(1.31790121285 + 104127.51158807839*$t);
   $Y0 += 0.00000000007*cos(5.05330889636 + 27624.1237020271*$t);
   $Y0 += 0.00000000007*cos(3.75190878844 + 24552.1702160883*$t);
   $Y0 += 0.00000000008*cos(5.40150242632 + 948.9317914863*$t);
   $Y0 += 0.00000000007*cos(1.23626167150 + 6680.98103591609*$t);
   $Y0 += 0.00000000008*cos(5.48806422287 + 51861.86392076229*$t);
   $Y0 += 0.00000000008*cos(2.01472939327 + 51554.3062997927*$t);
   $Y0 += 0.00000000007*cos(5.80107722174 + 26720.44306340389*$t);
   $Y0 += 0.00000000007*cos(1.27572316405 + 52214.1831362697*$t);
   $Y0 += 0.00000000007*cos(4.01717883587 + 25572.68308796469*$t);
   $Y0 += 0.00000000007*cos(4.78803884893 + 26603.6108301507*$t);
   $Y0 += 0.00000000007*cos(3.81203720019 + 415.7963080956*$t);
   $Y0 += 0.00000000008*cos(5.83496057018 + 54191.0317724297*$t);
   $Y0 += 0.00000000006*cos(1.87365150772 + 24505.69997580769*$t);
   $Y0 += 0.00000000007*cos(2.72912420029 + 2974.85377209451*$t);
   $Y0 += 0.00000000007*cos(2.73283247417 + 48092.0615939275*$t);
   $Y0 += 0.00000000006*cos(2.27207352577 + 78043.54059976728*$t);
   $Y0 += 0.00000000006*cos(0.76622305958 + 19202.99706912689*$t);
   $Y0 += 0.00000000006*cos(0.12929656543 + 53661.3408073351*$t);
   $Y0 += 0.00000000006*cos(4.92644191429 + 104426.63798234759*$t);
   $Y0 += 0.00000000006*cos(4.97320423795 + 27883.4054027787*$t);
   $Y0 += 0.00000000006*cos(1.52227576605 + 156523.78133157069*$t);
   $Y0 += 0.00000000008*cos(4.29203597963 + 51120.60032370579*$t);
   $Y0 += 0.00000000006*cos(6.23544000173 + 226.07308589371*$t);
   $Y0 += 0.00000000008*cos(5.23117913800 + 48997.9043100643*$t);
   $Y0 += 0.00000000006*cos(3.11031405202 + 149756.95206568309*$t);
   $Y0 += 0.00000000007*cos(3.39730503781 + 4083.74468922089*$t);
   $Y0 += 0.00000000008*cos(2.23399255632 + 53259.1303329255*$t);
   $Y0 += 0.00000000006*cos(3.49957202036 + 130006.04778747768*$t);
   $Y0 += 0.00000000007*cos(6.08095772039 + 162810.73851692009*$t);
   $Y0 += 0.00000000006*cos(0.64154839978 + 27441.8957040745*$t);
   $Y0 += 0.00000000006*cos(1.88048397784 + 24734.3982140409*$t);
   $Y0 += 0.00000000007*cos(0.08806115428 + 55503.69812194508*$t);
   $Y0 += 0.00000000008*cos(1.92055633433 + 422.9098550964*$t);
   $Y0 += 0.00000000006*cos(5.11522815430 + 76777.97312114088*$t);
   $Y0 += 0.00000000006*cos(3.95276470433 + 155461.16718973868*$t);
   $Y0 += 0.00000000006*cos(2.81558483589 + 50161.0684288341*$t);
   $Y0 += 0.00000000006*cos(3.77961930116 + 78262.46876949779*$t);
   $Y0 += 0.00000000006*cos(2.10450702949 + 223.1041404771*$t);
   $Y0 += 0.00000000006*cos(2.19241811447 + 78225.82020656829*$t);
   $Y0 += 0.00000000006*cos(3.79548807692 + 76468.69479848508*$t);
   $Y0 += 0.00000000006*cos(3.10628150548 + 78597.8091829755*$t);
   $Y0 += 0.00000000007*cos(5.87498441401 + 30110.4094910219*$t);
   $Y0 += 0.00000000006*cos(1.92388121650 + 35077.61466449009*$t);
   $Y0 += 0.00000000007*cos(2.98999765548 + 1884.65734593389*$t);
   $Y0 += 0.00000000007*cos(6.11933292351 + 1082.8364148101*$t);
   $Y0 += 0.00000000006*cos(3.53656686000 + 5193.12324131429*$t);
   $Y0 += 0.00000000006*cos(3.60785310094 + 171292.90171019848*$t);
   $Y0 += 0.00000000006*cos(4.43885235196 + 25984.56654987209*$t);
   $Y0 += 0.00000000006*cos(2.63023124509 + 29864.5778447925*$t);
   $Y0 += 0.00000000006*cos(6.17498643970 + 22311.7160733229*$t);
   $Y0 += 0.00000000006*cos(2.81657351020 + 76991.27221657889*$t);
   $Y0 += 0.00000000005*cos(5.40188161644 + 27410.8246065891*$t);
   $Y0 += 0.00000000005*cos(3.40333606836 + 24765.4693115263*$t);
   $Y0 += 0.00000000006*cos(0.12287107983 + 51092.9698683383*$t);
   $Y0 += 0.00000000007*cos(5.41576313041 + 88476.75115345008*$t);
   $Y0 += 0.00000000007*cos(1.20946078842 + 34102.3451619817*$t);
   $Y0 += 0.00000000005*cos(2.83164430151 + 2686.4782770577*$t);
   $Y0 += 0.00000000006*cos(3.98658256476 + 31903.2578186947*$t);
   $Y0 += 0.00000000006*cos(4.81863512004 + 20273.0360994207*$t);
   $Y0 += 0.00000000006*cos(1.29004214195 + 104874.43380187408*$t);
   $Y0 += 0.00000000005*cos(0.59730065618 + 155475.39428374029*$t);
   $Y0 += 0.00000000005*cos(4.57106999295 + 128747.59385585209*$t);
   $Y0 += 0.00000000005*cos(3.48020412230 + 26500.51805593209*$t);
   $Y0 += 0.00000000005*cos(5.32501356250 + 25675.7758621833*$t);
   $Y0 += 0.00000000007*cos(5.70848080046 + 25234.46294233869*$t);
   $Y0 += 0.00000000005*cos(4.12117505892 + 53551.8239004777*$t);
   $Y0 += 0.00000000005*cos(5.05855634652 + 60170.5785426561*$t);
   $Y0 += 0.00000000005*cos(4.48575981388 + 27353.71443768409*$t);
   $Y0 += 0.00000000007*cos(2.75081241109 + 52602.16065654089*$t);
   $Y0 += 0.00000000005*cos(3.92111104398 + 128106.07549751546*$t);
   $Y0 += 0.00000000005*cos(0.45323700456 + 629.0954035336*$t);
   $Y0 += 0.00000000005*cos(2.88226149102 + 35191.56631820829*$t);
   $Y0 += 0.00000000006*cos(0.61158685386 + 3492.8153843531*$t);
   $Y0 += 0.00000000007*cos(3.24751127495 + 169093.81436691148*$t);
   $Y0 += 0.00000000006*cos(0.00555930589 + 81591.60126351927*$t);
   $Y0 += 0.00000000006*cos(3.27898581061 + 25462.4767667453*$t);
   $Y0 += 0.00000000006*cos(5.52623187419 + 26713.81715137009*$t);
   $Y0 += 0.00000000007*cos(1.40182187541 + 2641.5850959557*$t);
   $Y0 += 0.00000000005*cos(2.22571570610 + 58220.2786820073*$t);
   $Y0 += 0.00000000005*cos(1.13017778403 + 52813.0463726561*$t);
   $Y0 += 0.00000000006*cos(6.23315922928 + 156531.54400228668*$t);
   $Y0 += 0.00000000005*cos(5.91150949505 + 26236.225683484*$t);
   $Y0 += 0.00000000007*cos(6.01504388975 + 9745.07673837309*$t);
   $Y0 += 0.00000000005*cos(3.16675569939 + 26035.0508067433*$t);
   $Y0 += 0.00000000005*cos(5.63846198541 + 26141.24311137209*$t);
   $Y0 += 0.00000000006*cos(2.65718358525 + 117873.12019040468*$t);
   $Y0 += 0.00000000006*cos(2.30069536250 + 28214.02436643689*$t);
   $Y0 += 0.00000000005*cos(5.51855350061 + 487.39093377069*$t);
   $Y0 += 0.00000000006*cos(2.84193454109 + 53764.63536094869*$t);
   $Y0 += 0.00000000005*cos(2.79855313555 + 104035.46451412368*$t);
   $Y0 += 0.00000000006*cos(3.26310713546 + 437.8877086234*$t);
   $Y0 += 0.00000000005*cos(2.17730624219 + 52098.29955664799*$t);
   $Y0 += 0.00000000006*cos(3.10526072910 + 98068.29289882188*$t);
   $Y0 += 0.00000000005*cos(1.17059070361 + 65538.49980743109*$t);
   $Y0 += 0.00000000005*cos(0.60786805630 + 189854.24140010288*$t);
   $Y0 += 0.00000000005*cos(0.07976631221 + 76138.07583482689*$t);
   $Y0 += 0.00000000005*cos(3.31240476519 + 104301.85976187608*$t);
   $Y0 += 0.00000000005*cos(5.17958441161 + 1243.73141916191*$t);
   $Y0 += 0.00000000005*cos(5.92076756755 + 118829.20756698029*$t);
   $Y0 += 0.00000000006*cos(4.31566445968 + 13675.2864345927*$t);
   $Y0 += 0.00000000005*cos(3.70296203327 + 153084.60008699627*$t);
   $Y0 += 0.00000000006*cos(3.07685642656 + 13.9832765181*$t);
   $Y0 += 0.00000000005*cos(4.76122192085 + 78697.66498008289*$t);
   $Y0 += 0.00000000005*cos(5.20357112241 + 26610.23674218449*$t);
   $Y0 += 0.00000000005*cos(0.29390226295 + 1052.02456570489*$t);
   $Y0 += 0.00000000005*cos(5.27501982781 + 149.3193796511*$t);
   $Y0 += 0.00000000006*cos(1.42019585673 + 122444.74344836328*$t);
   $Y0 += 0.00000000006*cos(3.75361834376 + 24822.5794804313*$t);
   $Y0 += 0.00000000005*cos(2.37687918815 + 26162.44092265799*$t);
   $Y0 += 0.00000000005*cos(4.48428750718 + 154.0226279683*$t);
   $Y0 += 0.00000000005*cos(4.82516506006 + 104401.85300568449*$t);
   $Y0 += 0.00000000005*cos(5.03758504482 + 53654.91667469629*$t);
   $Y0 += 0.00000000005*cos(0.38799912808 + 118828.71993201328*$t);
   $Y0 += 0.00000000005*cos(1.40704909535 + 191.2044486281*$t);
   $Y0 += 0.00000000005*cos(1.05885873562 + 124777.93984123527*$t);
   $Y0 += 0.00000000005*cos(4.57253368744 + 51006.46184922329*$t);
   $Y0 += 0.00000000006*cos(2.99064022137 + 104241.65006256089*$t);
   $Y0 += 0.00000000005*cos(5.36734313790 + 104668.24825343689*$t);
   $Y0 += 0.00000000004*cos(4.69357830395 + 76532.83088927349*$t);
   $Y0 += 0.00000000006*cos(2.26469618573 + 75615.01078176608*$t);
   $Y0 += 0.00000000004*cos(3.91764666613 + 24608.79275002629*$t);
   $Y0 += 0.00000000004*cos(0.77605401254 + 24609.2803849933*$t);
   $Y0 += 0.00000000005*cos(3.59914095117 + 23389.6955116805*$t);
   $Y0 += 0.00000000005*cos(5.20607673363 + 28786.5984064349*$t);
   $Y0 += 0.00000000004*cos(5.84756896317 + 31775.7773280097*$t);
   $Y0 += 0.00000000004*cos(2.95764872163 + 20400.5165901057*$t);
   $Y0 += 0.00000000005*cos(6.27101158445 + 107679.50440509345*$t);
   $Y0 += 0.00000000004*cos(4.55685472779 + 13541.6650199747*$t);
   $Y0 += 0.00000000005*cos(0.86738231950 + 29428.27175079049*$t);
   $Y0 += 0.00000000005*cos(0.59524275071 + 26.0545023163*$t);
   $Y0 += 0.00000000004*cos(2.28993527511 + 25764.64154240029*$t);
   $Y0 += 0.00000000005*cos(4.52541301080 + 78265.43771491438*$t);
   $Y0 += 0.00000000006*cos(0.78206719980 + 51688.4153493777*$t);
   $Y0 += 0.00000000006*cos(1.53449593067 + 23865.80268949369*$t);
   $Y0 += 0.00000000004*cos(1.16733607750 + 35211.72371407509*$t);
   $Y0 += 0.00000000004*cos(1.35469630012 + 16964.57020404031*$t);
   $Y0 += 0.00000000004*cos(4.36135738215 + 47804.1737338577*$t);
   $Y0 += 0.00000000004*cos(5.40330574142 + 77623.56832092509*$t);
   $Y0 += 0.00000000005*cos(5.21473868834 + 86458.22857541469*$t);
   $Y0 += 0.00000000004*cos(5.79724758149 + 27040.2235823173*$t);
   $Y0 += 0.00000000004*cos(3.00797010331 + 25136.0703357981*$t);
   $Y0 += 0.00000000005*cos(3.02879597074 + 25778.86863640189*$t);
   $Y0 += 0.00000000005*cos(5.77642171406 + 26397.42528171349*$t);
   $Y0 += 0.00000000005*cos(2.45203846176 + 78374.15956342549*$t);
   $Y0 += 0.00000000004*cos(5.04872942833 + 48834.95003896509*$t);
   $Y0 += 0.00000000005*cos(0.30596644618 + 25771.26745443409*$t);
   $Y0 += 0.00000000004*cos(0.10837463046 + 33480.6013611425*$t);
   $Y0 += 0.00000000004*cos(2.41365774716 + 18695.6925569729*$t);
   $Y0 += 0.00000000005*cos(5.31228827456 + 703.8770021009*$t);
   $Y0 += 0.00000000004*cos(3.53618772681 + 156378.09946979408*$t);
   $Y0 += 0.00000000005*cos(1.85614150808 + 155467.79310177248*$t);
   $Y0 += 0.00000000005*cos(2.86964254252 + 26404.05119374729*$t);
   $Y0 += 0.00000000004*cos(1.76062802705 + 9123.33293753389*$t);
   $Y0 += 0.00000000005*cos(0.75540582554 + 2015.9149036433*$t);
   $Y0 += 0.00000000004*cos(5.33219851688 + 76998.38576357969*$t);
   $Y0 += 0.00000000004*cos(4.84304007509 + 129586.56314360249*$t);
   $Y0 += 0.00000000004*cos(1.42347412662 + 1368.9040703285*$t);
   $Y0 += 0.00000000004*cos(6.20388780658 + 45425.0092602861*$t);
   $Y0 += 0.00000000004*cos(3.82794287315 + 28928.6478640837*$t);
   $Y0 += 0.00000000004*cos(4.97727481165 + 23247.6460540317*$t);
   $Y0 += 0.00000000004*cos(3.79698584398 + 23866.2903244607*$t);
   $Y0 += 0.00000000004*cos(0.78108419173 + 20453.1132297053*$t);
   $Y0 += 0.00000000004*cos(1.74094818589 + 31723.1806884101*$t);
   $Y0 += 0.00000000004*cos(0.73252151870 + 26239.19462890059*$t);
   $Y0 += 0.00000000004*cos(0.40730526869 + 199599.56195595946*$t);
   $Y0 += 0.00000000004*cos(1.20756668103 + 52356.1272309165*$t);
   $Y0 += 0.00000000004*cos(2.82011030245 + 26137.65594599489*$t);
   $Y0 += 0.00000000004*cos(1.89262749454 + 51539.0538286077*$t);
   $Y0 += 0.00000000004*cos(2.12864958126 + 32279.1699733045*$t);
   $Y0 += 0.00000000004*cos(0.39338279636 + 19897.1239448109*$t);
   $Y0 += 0.00000000004*cos(0.13108394311 + 78905.36680394509*$t);
   $Y0 += 0.00000000004*cos(1.42991389286 + 52336.4574700167*$t);
   $Y0 += 0.00000000004*cos(4.11596381659 + 50912.8927380537*$t);
   $Y0 += 0.00000000005*cos(5.25156458726 + 26085.17801364109*$t);
   $Y0 += 0.00000000005*cos(3.55365309754 + 26091.1159044743*$t);
   $Y0 += 0.00000000004*cos(4.55748177466 + 25441.13612574289*$t);
   $Y0 += 0.00000000004*cos(1.77632201645 + 25923.71104846829*$t);
   $Y0 += 0.00000000004*cos(0.74571036117 + 26252.58286964711*$t);
   $Y0 += 0.00000000004*cos(2.50276715961 + 24925.18461968289*$t);
   $Y0 += 0.00000000005*cos(5.55461968829 + 13706.1009305641*$t);
   $Y0 += 0.00000000005*cos(3.25059799651 + 38470.19298755129*$t);
   $Y0 += 0.00000000005*cos(3.12816538610 + 18227.7275666185*$t);
   $Y0 += 0.00000000005*cos(5.67705229870 + 33948.5663514969*$t);
   $Y0 += 0.00000000005*cos(1.28804998148 + 10.8018827804*$t);
   $Y0 += 0.00000000004*cos(4.38061016047 + 147423.26803784407*$t);
   $Y0 += 0.00000000004*cos(4.94619430058 + 120225.98634417648*$t);
   $Y0 += 0.00000000004*cos(4.22377863391 + 95.7354097343*$t);
   $Y0 += 0.00000000004*cos(2.78744864067 + 74935.81758592589*$t);
   $Y0 += 0.00000000004*cos(1.85033590581 + 142871.31454078527*$t);
   $Y0 += 0.00000000004*cos(5.08828008747 + 26131.0300339611*$t);
   $Y0 += 0.00000000004*cos(3.71693759733 + 26045.2638841543*$t);
   $Y0 += 0.00000000004*cos(4.81796795304 + 1066.2516597065*$t);
   $Y0 += 0.00000000004*cos(2.30874038706 + 25035.39094090229*$t);
   $Y0 += 0.00000000005*cos(5.79505565508 + 1577.0997249643*$t);
   $Y0 += 0.00000000004*cos(2.13386788705 + 1262.91354509469*$t);
   $Y0 += 0.00000000005*cos(4.43555583939 + 51102.44107644109*$t);
   $Y0 += 0.00000000004*cos(2.69709923008 + 106262.56823203048*$t);
   $Y0 += 0.00000000004*cos(6.19718375992 + 104771.34102765548*$t);
   $Y0 += 0.00000000005*cos(6.11030752375 + 179172.99086353746*$t);
   $Y0 += 0.00000000004*cos(5.45224037983 + 52329.34127614969*$t);
   $Y0 += 0.00000000005*cos(0.05651935942 + 151200.18655854589*$t);
   $Y0 += 0.00000000004*cos(1.76945825888 + 18319.7804023631*$t);
   $Y0 += 0.00000000004*cos(0.75257411874 + 33856.5135157523*$t);
   $Y0 += 0.00000000005*cos(2.23665069420 + 51315.74017187909*$t);
   $Y0 += 0.00000000004*cos(1.69359058279 + 159970.23761189409*$t);
   $Y0 += 0.00000000005*cos(3.45474588143 + 107692.46881048009*$t);
   $Y0 += 0.00000000004*cos(4.69804652488 + 24182.19455915029*$t);
   $Y0 += 0.00000000004*cos(5.36463943224 + 25021.16384690069*$t);
   $Y0 += 0.00000000005*cos(1.47989657697 + 49637.8015963783*$t);
   $Y0 += 0.00000000004*cos(5.19657656850 + 64901.01590043989*$t);
   $Y0 += 0.00000000005*cos(5.34571928246 + 77930.09730143669*$t);
   $Y0 += 0.00000000004*cos(2.13537389331 + 106685.05277665229*$t);
   $Y0 += 0.00000000004*cos(1.30241374700 + 25249.1776713073*$t);
   $Y0 += 0.00000000004*cos(1.21961863062 + 26927.11624680809*$t);
   $Y0 += 0.00000000005*cos(6.25905047664 + 130020.27488147929*$t);
   $Y0 += 0.00000000004*cos(0.16897355283 + 42430.7295447753*$t);
   $Y0 += 0.00000000004*cos(1.81097172729 + 189386.27640974846*$t);
   $Y0 += 0.00000000004*cos(1.67938382417 + 130907.72451570889*$t);
   $Y0 += 0.00000000004*cos(0.49273474013 + 76352.7537589889*$t);
   $Y0 += 0.00000000004*cos(0.60720939762 + 78050.16651180109*$t);
   $Y0 += 0.00000000004*cos(3.81303761260 + 26507.14396796589*$t);
   $Y0 += 0.00000000004*cos(5.10986588518 + 21715.78295731649*$t);
   $Y0 += 0.00000000004*cos(2.24959479186 + 50690.7593939287*$t);
   $Y0 += 0.00000000003*cos(4.11846884673 + 76095.19275992349*$t);
   $Y0 += 0.00000000004*cos(5.11517156658 + 27047.2698190153*$t);
   $Y0 += 0.00000000004*cos(3.69004611822 + 25129.0240991001*$t);
   $Y0 += 0.00000000003*cos(2.87423294156 + 25980.8416521281*$t);
   $Y0 += 0.00000000003*cos(5.93098474324 + 26195.4522659873*$t);
   $Y0 += 0.00000000004*cos(3.06482161328 + 52165.00440036799*$t);
   $Y0 += 0.00000000004*cos(4.26611311394 + 104881.05971390787*$t);
   $Y0 += 0.00000000004*cos(3.03103303969 + 8044.7688814119*$t);
   $Y0 += 0.00000000005*cos(4.86351403293 + 45455.3361212905*$t);
   $Y0 += 0.00000000004*cos(0.29804014939 + 25565.08190599689*$t);
   $Y0 += 0.00000000004*cos(0.98706116692 + 215942.14454167709*$t);
   $Y0 += 0.00000000004*cos(3.84907251953 + 25992.1677318399*$t);
   $Y0 += 0.00000000004*cos(4.95614516527 + 26184.1261862755*$t);
   $Y0 += 0.00000000003*cos(1.64017894322 + 129909.58092529287*$t);
   $Y0 += 0.00000000004*cos(2.78312246204 + 48153.7875686677*$t);
   $Y0 += 0.00000000005*cos(6.06876669605 + 10059.7505532097*$t);
   $Y0 += 0.00000000004*cos(5.16058547691 + 16496.6052136859*$t);
   $Y0 += 0.00000000004*cos(3.64463220789 + 35679.6887044295*$t);
   $Y0 += 0.00000000004*cos(2.60946762791 + 103395.76900720468*$t);
   $Y0 += 0.00000000004*cos(2.31682392343 + 51040.3732287387*$t);
   $Y0 += 0.00000000003*cos(0.92201563297 + 125111.79578200469*$t);
   $Y0 += 0.00000000004*cos(0.89339658044 + 128843.57308306987*$t);
   $Y0 += 0.00000000004*cos(0.91042238998 + 52015.64273124709*$t);
   $Y0 += 0.00000000004*cos(3.60881337650 + 621.9876183227*$t);
   $Y0 += 0.00000000003*cos(6.06256841288 + 23113.5370044467*$t);
   $Y0 += 0.00000000003*cos(2.74264927192 + 29062.7569136687*$t);
   $Y0 += 0.00000000003*cos(4.85231198714 + 25794.7361746805*$t);
   $Y0 += 0.00000000003*cos(3.95290569766 + 26381.5577434349*$t);
   $Y0 += 0.00000000003*cos(3.22127261990 + 78283.13536813887*$t);
   $Y0 += 0.00000000004*cos(1.30518138816 + 26037.66270218649*$t);
   $Y0 += 0.00000000004*cos(5.49977739982 + 23543.4743221653*$t);
   $Y0 += 0.00000000004*cos(3.30544028498 + 28632.8195959501*$t);
   $Y0 += 0.00000000003*cos(0.77597702447 + 26279.5952251693*$t);
   $Y0 += 0.00000000003*cos(1.74605535315 + 25896.6986929461*$t);
   $Y0 += 0.00000000004*cos(2.60606940279 + 25998.6612112117*$t);
   $Y0 += 0.00000000004*cos(6.19914828201 + 26177.63270690369*$t);
   $Y0 += 0.00000000003*cos(3.02425517786 + 24079.10178493169*$t);
   $Y0 += 0.00000000003*cos(1.26642446783 + 49851.10069181629*$t);
   $Y0 += 0.00000000004*cos(4.75573366952 + 14235.79189565871*$t);
   $Y0 += 0.00000000004*cos(4.04948401528 + 37940.50202245669*$t);
   $Y0 += 0.00000000003*cos(1.62600802494 + 41494.79956406649*$t);
   $Y0 += 0.00000000004*cos(2.15333731098 + 261988.65378531882*$t);
   $Y0 += 0.00000000003*cos(4.80360212129 + 61560.40347374009*$t);
   $Y0 += 0.00000000004*cos(4.57728207743 + 32217.4439985643*$t);
   $Y0 += 0.00000000003*cos(1.10122747342 + 78243.79584633929*$t);
   $Y0 += 0.00000000004*cos(1.13276960181 + 51995.9729703473*$t);
   $Y0 += 0.00000000003*cos(5.60306507247 + 154308.90556274189*$t);
   $Y0 += 0.00000000003*cos(4.90725652630 + 78398.0622917911*$t);
   $Y0 += 0.00000000003*cos(1.65138805728 + 202420.39310008568*$t);
   $Y0 += 0.00000000003*cos(3.28257147962 + 79117.14962395809*$t);
   $Y0 += 0.00000000004*cos(0.10028058442 + 28420.8557169627*$t);
   $Y0 += 0.00000000003*cos(6.21316075804 + 313054.59388140672*$t);
   $Y0 += 0.00000000003*cos(0.25268995906 + 26114.4452788575*$t);
   $Y0 += 0.00000000003*cos(2.26934241856 + 26061.8486392579*$t);
   $Y0 += 0.00000000003*cos(1.45372655722 + 116783.41139921108*$t);
   $Y0 += 0.00000000003*cos(4.65338616524 + 2169.0042997661*$t);
   $Y0 += 0.00000000003*cos(4.91794356627 + 32131.88790546609*$t);
   $Y0 += 0.00000000003*cos(4.52136661001 + 68241.62832713968*$t);
   $Y0 += 0.00000000004*cos(0.55663803534 + 49424.5025009403*$t);
   $Y0 += 0.00000000003*cos(1.53998283633 + 27993.6117239981*$t);
   $Y0 += 0.00000000004*cos(1.48359617562 + 156954.26085780468*$t);
   $Y0 += 0.00000000004*cos(5.24222478531 + 52021.78365518009*$t);
   $Y0 += 0.00000000004*cos(4.16132549519 + 192828.85135471389*$t);
   $Y0 += 0.00000000003*cos(4.98600736246 + 25345.1568985251*$t);
   $Y0 += 0.00000000003*cos(0.24502022671 + 87648.79425028129*$t);
   $Y0 += 0.00000000003*cos(2.25580297503 + 105410.75067900248*$t);
   $Y0 += 0.00000000003*cos(2.35998158508 + 130446.38808425449*$t);
   $Y0 += 0.00000000003*cos(1.34307352948 + 45290.41257573408*$t);
   $Y0 += 0.00000000003*cos(0.38732763509 + 251775.36823910786*$t);
   $Y0 += 0.00000000004*cos(5.98737648746 + 50952.23225985329*$t);
   $Y0 += 0.00000000003*cos(6.14909241029 + 23421.09462541629*$t);
   $Y0 += 0.00000000003*cos(2.65612527451 + 28755.19929269911*$t);
   $Y0 += 0.00000000003*cos(5.55350394783 + 163297.88563320728*$t);
   $Y0 += 0.00000000003*cos(5.04626720292 + 26727.55661040469*$t);
   $Y0 += 0.00000000004*cos(5.00773877934 + 24711.88552424489*$t);
   $Y0 += 0.00000000003*cos(5.25171311588 + 111590.04434041328*$t);
   $Y0 += 0.00000000004*cos(5.00370301079 + 50476.77088412869*$t);
   $Y0 += 0.00000000003*cos(2.64378251337 + 52072.2679120513*$t);
   $Y0 += 0.00000000003*cos(2.04712691944 + 77622.5396804671*$t);
   $Y0 += 0.00000000003*cos(5.19234087353 + 52016.7483658397*$t);
   $Y0 += 0.00000000003*cos(0.11683553157 + 25970.8270908375*$t);
   $Y0 += 0.00000000003*cos(2.40519684605 + 26205.4668272779*$t);
   $Y0 += 0.00000000004*cos(3.70857620718 + 27511.7116910207*$t);
   $Y0 += 0.00000000004*cos(5.09664147762 + 24664.5822270947*$t);
   $Y0 += 0.00000000003*cos(0.72953121469 + 33918.23949049249*$t);
   $Y0 += 0.00000000003*cos(1.79250116293 + 18258.0544276229*$t);
   $Y0 += 0.00000000004*cos(0.12046155373 + 112545.64408202188*$t);
   $Y0 += 0.00000000004*cos(2.19368586548 + 61921.3706564885*$t);
   $Y0 += 0.00000000003*cos(1.35496442859 + 51741.85072778809*$t);
   $Y0 += 0.00000000003*cos(1.31287870388 + 222225.22039166848*$t);
   $Y0 += 0.00000000003*cos(5.87130809057 + 78249.72614820449*$t);
   $Y0 += 0.00000000004*cos(1.48902081585 + 158116.73556221249*$t);
   $Y0 += 0.00000000003*cos(5.25522824422 + 73891.58924046489*$t);
   $Y0 += 0.00000000003*cos(0.31258950206 + 26387.2733533269*$t);
   $Y0 += 0.00000000003*cos(4.52577588776 + 12725.6972522585*$t);
   $Y0 += 0.00000000003*cos(3.47716392063 + 26984.2264157131*$t);
   $Y0 += 0.00000000003*cos(5.32805376417 + 25192.0675024023*$t);
   $Y0 += 0.00000000003*cos(5.05935500666 + 103498.17236706128*$t);
   $Y0 += 0.00000000003*cos(5.46073958753 + 79487.77108298469*$t);
   $Y0 += 0.00000000003*cos(3.01230527787 + 50579.37602338029*$t);
   $Y0 += 0.00000000003*cos(5.01426493917 + 143980.69309287866*$t);
   $Y0 += 0.00000000003*cos(4.06152997793 + 91805.37444469729*$t);
   $Y0 += 0.00000000003*cos(1.54880317183 + 50490.99797813029*$t);
   $Y0 += 0.00000000003*cos(1.42463612574 + 54879.6662553075*$t);
   $Y0 += 0.00000000003*cos(3.12587906085 + 55484.5159960123*$t);
   $Y0 += 0.00000000003*cos(6.21651664332 + 2332.9525753885*$t);
   $Y0 += 0.00000000003*cos(6.10767041127 + 1383.13116433011*$t);
   $Y0 += 0.00000000003*cos(0.21479933904 + 128106.56313248246*$t);
   $Y0 += 0.00000000003*cos(5.41781200857 + 1471.9968445471*$t);
   $Y0 += 0.00000000003*cos(3.15521647543 + 146505.82538682928*$t);
   $Y0 += 0.00000000004*cos(5.80466382140 + 102232.60488843489*$t);
   $Y0 += 0.00000000003*cos(3.47413746108 + 24510.8034166099*$t);
   $Y0 += 0.00000000003*cos(5.33108022372 + 27665.4905015055*$t);
   $Y0 += 0.00000000004*cos(3.03503390629 + 26365.1819527991*$t);
   $Y0 += 0.00000000004*cos(5.77018377851 + 25811.11196531629*$t);
   $Y0 += 0.00000000003*cos(4.73983619092 + 106263.05586699749*$t);
   $Y0 += 0.00000000003*cos(1.92656985885 + 50601.2042993497*$t);
   $Y0 += 0.00000000003*cos(5.01250465817 + 179967.11098302988*$t);
   $Y0 += 0.00000000003*cos(3.10770566817 + 210921.73841929682*$t);
   $Y0 += 0.00000000003*cos(6.02863621669 + 26202.5460659711*$t);
   $Y0 += 0.00000000003*cos(1.18743831658 + 2821.07496160971*$t);
   $Y0 += 0.00000000003*cos(4.69341673957 + 5020.16230489671*$t);
   $Y0 += 0.00000000003*cos(3.22922107727 + 229463.89598326846*$t);
   $Y0 += 0.00000000003*cos(1.31139605087 + 71378.31571730827*$t);
   $Y0 += 0.00000000003*cos(6.13912071583 + 29001.0309389285*$t);
   $Y0 += 0.00000000003*cos(2.66609696897 + 23175.26297918689*$t);
   $Y0 += 0.00000000003*cos(2.25383916432 + 15406.40878752529*$t);
   $Y0 += 0.00000000003*cos(2.75613383037 + 25940.06823463139*$t);
   $Y0 += 0.00000000003*cos(0.81910869714 + 220026.13304838145*$t);
   $Y0 += 0.00000000003*cos(5.30888840501 + 963.6465204549*$t);
   $Y0 += 0.00000000003*cos(1.30036577443 + 113455.76337789789*$t);
   $Y0 += 0.00000000003*cos(6.07147026831 + 49201.4401460209*$t);
   $Y0 += 0.00000000003*cos(4.25244825934 + 143165.21296012949*$t);
   $Y0 += 0.00000000003*cos(0.81593599477 + 51735.22481575429*$t);
   $Y0 += 0.00000000003*cos(1.27452500023 + 25914.69838767311*$t);
   $Y0 += 0.00000000003*cos(1.24750737739 + 26261.59553044229*$t);
   $Y0 += 0.00000000003*cos(3.64962054328 + 51745.51975649279*$t);
   $Y0 += 0.00000000002*cos(1.46486531145 + 25554.52384069999*$t);
   $Y0 += 0.00000000002*cos(1.05716706617 + 26621.77007741539*$t);
   $Y0 += 0.00000000003*cos(3.63673845964 + 25530.6962208573*$t);
   $Y0 += 0.00000000003*cos(5.16847922516 + 26645.5976972581*$t);
   $Y0 += 0.00000000002*cos(5.30160804243 + 99979.49238203908*$t);
   $Y0 += 0.00000000003*cos(3.46486175917 + 25626.81052760331*$t);
   $Y0 += 0.00000000003*cos(5.34035592563 + 26549.48339051209*$t);
   $Y0 += 0.00000000003*cos(4.02837363457 + 23976.4966456801*$t);
   $Y0 += 0.00000000003*cos(3.77281945049 + 20956.5058750001*$t);
   $Y0 += 0.00000000003*cos(5.03239823431 + 31219.7880431153*$t);
   $Y0 += 0.00000000002*cos(5.45450198288 + 242985.64742485987*$t);
   $Y0 += 0.00000000002*cos(6.25807160883 + 133767.40754666766*$t);
   $Y0 += 0.00000000003*cos(2.87262383804 + 283524.6034298342*$t);
   $Y0 += 0.00000000002*cos(2.72683369817 + 127790.81706491447*$t);
   $Y0 += 0.00000000002*cos(2.72683369816 + 127791.30469988148*$t);
   $Y0 += 0.00000000002*cos(3.86040326075 + 71378.80335227528*$t);
   $Y0 += 0.00000000002*cos(3.01852194681 + 21202.3375212295*$t);
   $Y0 += 0.00000000002*cos(5.78669573799 + 30973.9563968859*$t);
   $Y0 += 0.00000000003*cos(1.89994220844 + 2353.1099712553*$t);
   $Y0 += 0.00000000003*cos(0.91363958260 + 76358.48847726568*$t);
   $Y0 += 0.00000000003*cos(1.60976861498 + 24292.40088036969*$t);
   $Y0 += 0.00000000003*cos(3.61818924103 + 157636.55358405507*$t);
   $Y0 += 0.00000000003*cos(5.54528574622 + 25361.6649391875*$t);
   $Y0 += 0.00000000003*cos(3.25993193858 + 26814.6289789279*$t);
   $Y0 += 0.00000000002*cos(5.95736615747 + 57369.41715942969*$t);
   $Y0 += 0.00000000003*cos(3.84445897700 + 52187.0958008958*$t);
   $Y0 += 0.00000000003*cos(5.60002249822 + 49953.70583106789*$t);
   $Y0 += 0.00000000002*cos(6.18039190992 + 110396.08496515569*$t);
   $Y0 += 0.00000000002*cos(0.99675798287 + 273311.31788362318*$t);
   $Y0 += 0.00000000002*cos(2.52419479264 + 7994.2846245407*$t);
   $Y0 += 0.00000000002*cos(1.06541336121 + 31127.7352073707*$t);
   $Y0 += 0.00000000002*cos(1.45661901641 + 21048.5587107447*$t);
   $Y0 += 0.00000000002*cos(4.88076203498 + 27133.30179524529*$t);
   $Y0 += 0.00000000002*cos(2.24228548233 + 289185.93547898643*$t);
   $Y0 += 0.00000000002*cos(2.28218167520 + 5634.7899118689*$t);
   $Y0 += 0.00000000002*cos(3.33192280402 + 108903.32224587208*$t);
   $Y0 += 0.00000000003*cos(4.49860650606 + 128857.80017707148*$t);
   $Y0 += 0.00000000002*cos(0.10562255755 + 5687.3865514685*$t);
   $Y0 += 0.00000000003*cos(3.16048516532 + 21650.63275068411*$t);
   $Y0 += 0.00000000003*cos(5.64473251948 + 30525.66116743129*$t);
   $Y0 += 0.00000000003*cos(5.20937667539 + 117.5636857037*$t);
   $Y0 += 0.00000000002*cos(0.61090203417 + 29266.2927496253*$t);
   $Y0 += 0.00000000002*cos(1.08851105464 + 24176.45984087349*$t);
   $Y0 += 0.00000000002*cos(1.68398359832 + 102129.99974918329*$t);
   $Y0 += 0.00000000003*cos(2.77323027693 + 53338.52480503969*$t);
   $Y0 += 0.00000000003*cos(1.56140703602 + 67608.51339047348*$t);
   $Y0 += 0.00000000002*cos(1.64261819628 + 267649.98583447101*$t);
   $Y0 += 0.00000000002*cos(0.35074176404 + 544.1618765797*$t);
   $Y0 += 0.00000000002*cos(5.34557611286 + 54862.7721951731*$t);
   $Y0 += 0.00000000002*cos(0.00658372278 + 162188.50708111389*$t);
   $Y0 += 0.00000000002*cos(1.24227084874 + 123554.12200045666*$t);
   $Y0 += 0.00000000003*cos(5.22200424544 + 47803.68609889069*$t);
   $Y0 += 0.00000000002*cos(2.25109608564 + 112546.13171698889*$t);
   $Y0 += 0.00000000002*cos(2.34818806724 + 71024.79001563908*$t);
   $Y0 += 0.00000000003*cos(1.82596797402 + 24971.65485996349*$t);
   $Y0 += 0.00000000003*cos(0.69606440360 + 27204.6390581519*$t);
   $Y0 += 0.00000000002*cos(3.44738180328 + 26165.89750304159*$t);
   $Y0 += 0.00000000003*cos(2.81683191154 + 14477.5950006835*$t);
   $Y0 += 0.00000000002*cos(1.34089550191 + 97466.21885888245*$t);
   $Y0 += 0.00000000002*cos(3.66953582983 + 8482.40701076191*$t);
   $Y0 += 0.00000000003*cos(1.55932656916 + 206584.70508969868*$t);
   $Y0 += 0.00000000003*cos(1.85898364174 + 25881.47377565349*$t);
   $Y0 += 0.00000000002*cos(2.20662264774 + 105306.96849042189*$t);
   $Y0 += 0.00000000003*cos(1.29582724688 + 7768.12273921109*$t);
   $Y0 += 0.00000000002*cos(2.32161458508 + 263097.54470244513*$t);
   $Y0 += 0.00000000002*cos(5.46320723867 + 263098.03233741222*$t);
   $Y0 += 0.00000000002*cos(1.96333630845 + 90988.91904201408*$t);
   $Y0 += 0.00000000003*cos(4.58684494130 + 5814.86704215349*$t);
   $Y0 += 0.00000000003*cos(5.57796892253 + 49976.9627573449*$t);
   $Y0 += 0.00000000003*cos(1.11744370489 + 27161.7559832485*$t);
   $Y0 += 0.00000000002*cos(4.55377851088 + 27151.46104250999*$t);
   $Y0 += 0.00000000003*cos(0.04149717596 + 29051.0275608327*$t);
   $Y0 += 0.00000000003*cos(2.48053520166 + 23125.2663572827*$t);
   $Y0 += 0.00000000002*cos(1.84878451995 + 32973.29684898851*$t);
   $Y0 += 0.00000000003*cos(5.15950079042 + 134.35286706851*$t);
   $Y0 += 0.00000000002*cos(5.35359251167 + 78484.36588464488*$t);
   $Y0 += 0.00000000002*cos(4.84459493300 + 50373.67810991009*$t);
   $Y0 += 0.00000000003*cos(1.01446892629 + 246114.03618995566*$t);
   $Y0 += 0.00000000002*cos(0.53517853304 + 19336.61848374489*$t);
   $Y0 += 0.00000000002*cos(1.40171752369 + 25014.53793486689*$t);
   $Y0 += 0.00000000002*cos(5.17281822358 + 156534.77621392948*$t);
   $Y0 += 0.00000000003*cos(2.73574689557 + 78115.87451777978*$t);
   $Y0 += 0.00000000002*cos(4.99251162267 + 51760.49761001979*$t);
   $Y0 += 0.00000000002*cos(2.81513963677 + 25867.24668165189*$t);
   $Y0 += 0.00000000002*cos(4.97557986159 + 26089.14379679899*$t);
   $Y0 += 0.00000000002*cos(3.55523971481 + 30486.3216456317*$t);
   $Y0 += 0.00000000002*cos(5.24997796999 + 21689.9722724837*$t);
   $Y0 += 0.00000000003*cos(1.74089824235 + 78112.90557236319*$t);
   $Y0 += 0.00000000002*cos(2.79225536164 + 1898.5950354231*$t);
   $Y0 += 0.00000000002*cos(3.64384685304 + 235746.97183325986*$t);
   $Y0 += 0.00000000002*cos(3.14876741131 + 26151.8828573611*$t);
   $Y0 += 0.00000000002*cos(5.65645027349 + 26024.4110607543*$t);
   $Y0 += 0.00000000002*cos(1.05800481517 + 50049.68505828569*$t);
   $Y0 += 0.00000000003*cos(2.73644560356 + 42116.5433649057*$t);
   $Y0 += 0.00000000002*cos(5.11212517082 + 61165.51780606429*$t);
   $Y0 += 0.00000000002*cos(5.88527469971 + 249268.72327485126*$t);
   $Y0 += 0.00000000002*cos(2.69366265342 + 184526.76529172009*$t);
   $Y0 += 0.00000000003*cos(4.49394290972 + 26226.6644559284*$t);
   $Y0 += 0.00000000003*cos(4.31127477508 + 25949.629462187*$t);
   $Y0 += 0.00000000002*cos(5.99028756442 + 223180.33249831005*$t);
   $Y0 += 0.00000000002*cos(2.84869491083 + 223180.82013327705*$t);
   $Y0 += 0.00000000003*cos(0.50210111414 + 65851.58035270809*$t);
   $Y0 += 0.00000000002*cos(1.76617343740 + 4525.8989947425*$t);
   $Y0 += 0.00000000003*cos(0.33246489326 + 26376.0348191275*$t);
   $Y0 += 0.00000000003*cos(2.18956748436 + 25800.2590989879*$t);
   $Y0 += 0.00000000002*cos(2.76517897745 + 26049.52628845289*$t);
   $Y0 += 0.00000000002*cos(3.57038365323 + 4398.41850405751*$t);
   $Y0 += 0.00000000003*cos(2.22716383153 + 23581.5019948011*$t);
   $Y0 += 0.00000000003*cos(0.29486854609 + 28594.7919233143*$t);
   $Y0 += 0.00000000002*cos(1.98440604953 + 1593.2488660304*$t);
   $Y0 += 0.00000000002*cos(4.71041878186 + 78267.34694259708*$t);
   $Y0 += 0.00000000002*cos(2.49584762828 + 54508.7588585369*$t);
   $Y0 += 0.00000000002*cos(5.88257404550 + 78476.76470267709*$t);
   $Y0 += 0.00000000002*cos(5.12850312405 + 1280.0383901115*$t);
   $Y0 += 0.00000000002*cos(2.67253805962 + 35077.12702952309*$t);
   $Y0 += 0.00000000003*cos(6.14642740727 + 26124.79552198719*$t);
   $Y0 += 0.00000000003*cos(2.65879027753 + 26051.4983961282*$t);
   $Y0 += 0.00000000002*cos(3.30021458746 + 158745.93213614845*$t);
   $Y0 += 0.00000000002*cos(3.91497249728 + 52367.49836674349*$t);
   $Y0 += 0.00000000002*cos(1.49263134542 + 41520.61024889929*$t);
   $Y0 += 0.00000000002*cos(2.56976964633 + 51950.2208322217*$t);
   $Y0 += 0.00000000002*cos(0.98860870622 + 28133.4554918599*$t);
   $Y0 += 0.00000000002*cos(1.53342367140 + 24042.8384262555*$t);
   $Y0 += 0.00000000003*cos(0.60080693478 + 51753.38406301899*$t);
   $Y0 += 0.00000000003*cos(4.58119790245 + 25633.23759253039*$t);
   $Y0 += 0.00000000003*cos(4.22401978235 + 26543.05632558499*$t);
   $Y0 += 0.00000000002*cos(2.40433125154 + 24999.54380245671*$t);
   $Y0 += 0.00000000002*cos(0.11770112608 + 27176.75011565869*$t);
   $Y0 += 0.00000000002*cos(1.87923904841 + 309.0345051723*$t);
   $Y0 += 0.00000000002*cos(0.83358735040 + 1155.6049748905*$t);
   $Y0 += 0.00000000002*cos(5.05439501768 + 236702.57157486846*$t);
   $Y0 += 0.00000000002*cos(4.57470028824 + 76887.69180739328*$t);
   $Y0 += 0.00000000002*cos(5.69921176024 + 4084.2323241879*$t);
   $Y0 += 0.00000000002*cos(3.16153790522 + 24601.67920302549*$t);
   $Y0 += 0.00000000002*cos(4.37985667611 + 27818.78167702329*$t);
   $Y0 += 0.00000000002*cos(3.83767204936 + 2198.8435258035*$t);
   $Y0 += 0.00000000002*cos(2.96055155756 + 24292.8885153367*$t);
   $Y0 += 0.00000000002*cos(3.15659811410 + 22759.5236678105*$t);
   $Y0 += 0.00000000002*cos(1.34122714460 + 27143.5967359838*$t);
   $Y0 += 0.00000000002*cos(3.27620714425 + 25973.22003540546*$t);
   $Y0 += 0.00000000002*cos(1.71257292852 + 10021.5934626159*$t);
   $Y0 += 0.00000000002*cos(6.23156121709 + 20618.2631760171*$t);
   $Y0 += 0.00000000002*cos(2.57365646771 + 31558.0307420983*$t);
   $Y0 += 0.00000000002*cos(0.76870131372 + 2854.8841913937*$t);
   $Y0 += 0.00000000002*cos(0.41751652575 + 51953.18977763829*$t);
   $Y0 += 0.00000000002*cos(1.98684490521 + 51951.21766996299*$t);
   $Y0 += 0.00000000002*cos(0.00103411994 + 52249.34722649089*$t);
   $Y0 += 0.00000000002*cos(3.27898093725 + 138633.54722359608*$t);
   $Y0 += 0.00000000002*cos(3.56496173333 + 225687.46509753366*$t);
   $Y0 += 0.00000000002*cos(3.13338598533 + 138319.36104372646*$t);
   $Y0 += 0.00000000002*cos(3.76046136390 + 32768.88417749029*$t);
   $Y0 += 0.00000000002*cos(5.04336913197 + 24079.5894198987*$t);
   $Y0 += 0.00000000002*cos(3.14356989843 + 52202.3484204317*$t);
   $Y0 += 0.00000000002*cos(0.25753538380 + 28206.42318446909*$t);
   $Y0 += 0.00000000002*cos(3.39136790755 + 180496.80194812448*$t);
   $Y0 += 0.00000000002*cos(0.02421731187 + 78259.58427188109*$t);
   $Y0 += 0.00000000002*cos(3.85481165639 + 26624.46383618609*$t);
   $Y0 += 0.00000000002*cos(4.79199372149 + 23549.8984548041*$t);
   $Y0 += 0.00000000002*cos(1.94407126016 + 26293.84487252789*$t);
   $Y0 += 0.00000000002*cos(1.12674295858 + 4601.70630147949*$t);
   $Y0 += 0.00000000002*cos(4.65935637649 + 21716.2705922835*$t);
   $Y0 += 0.00000000002*cos(2.09856828971 + 330.8627811417*$t);
   $Y0 += 0.00000000002*cos(3.08659098600 + 51219.27330969429*$t);
   $Y0 += 0.00000000002*cos(0.33777720887 + 78256.35206023828*$t);
   $Y0 += 0.00000000002*cos(5.15360975027 + 50063.91215228729*$t);
   $Y0 += 0.00000000002*cos(2.99279377683 + 132658.02899457427*$t);
   $Y0 += 0.00000000002*cos(1.71072737383 + 96.2230447013*$t);
   $Y0 += 0.00000000002*cos(5.95938200395 + 1265.32366114289*$t);
   $Y0 += 0.00000000002*cos(0.92029934107 + 52823.0609339467*$t);
   $Y0 += 0.00000000002*cos(2.83880526955 + 78270.57915423988*$t);
   $Y0 += 0.00000000002*cos(0.69832785802 + 7392.21058460129*$t);
   $Y0 += 0.00000000002*cos(2.67350091104 + 61165.03017109729*$t);
   $Y0 += 0.00000000002*cos(3.72029671343 + 2544.4288194089*$t);
   $Y0 += 0.00000000002*cos(3.54241730321 + 26421.5152648601*$t);
   $Y0 += 0.00000000002*cos(6.04496787393 + 2655.8121899573*$t);
   $Y0 += 0.00000000002*cos(5.50264063039 + 24705.9490265731*$t);
   $Y0 += 0.00000000002*cos(3.30257705441 + 27470.3448915423*$t);
   $Y0 += 0.00000000002*cos(1.43501438033 + 1353.5049275333*$t);
   $Y0 += 0.00000000002*cos(4.83145382265 + 1603.5438067689*$t);
   $Y0 += 0.00000000002*cos(5.03780638681 + 40738.45907867528*$t);
   $Y0 += 0.00000000002*cos(4.14425166593 + 30460.0233258319*$t);
   $Y0 += 0.00000000002*cos(5.87281396000 + 4578.4956343421*$t);
   $Y0 += 0.00000000002*cos(1.71607024251 + 78113.90241010448*$t);
   $Y0 += 0.00000000002*cos(2.62817899928 + 32839.1877994035*$t);
   $Y0 += 0.00000000002*cos(4.84613238692 + 14297.5178703989*$t);
   $Y0 += 0.00000000002*cos(3.95908529788 + 37878.7760477165*$t);
   $Y0 += 0.00000000002*cos(4.63463254085 + 104501.41958091488*$t);
   $Y0 += 0.00000000002*cos(4.18177446903 + 25138.48371512089*$t);
   $Y0 += 0.00000000002*cos(5.77634249731 + 44131.52503670349*$t);
   $Y0 += 0.00000000002*cos(5.67506076203 + 104358.48229581409*$t);
   $Y0 += 0.00000000002*cos(3.82438567775 + 23336.5993593661*$t);
   $Y0 += 0.00000000002*cos(0.68279302416 + 23336.11172439909*$t);
   $Y0 += 0.00000000002*cos(1.33260638073 + 224.1009782184*$t);
   $Y0 += 0.00000000002*cos(2.06879035628 + 51547.19851458179*$t);
   $Y0 += 0.00000000002*cos(4.73770456468 + 202.0095776906*$t);
   $Y0 += 0.00000000002*cos(1.59611191109 + 202.4972126576*$t);
   $Y0 += 0.00000000002*cos(4.68695414047 + 31108.06544647091*$t);
   $Y0 += 0.00000000002*cos(4.11826354432 + 21068.2284716445*$t);
   $Y0 += 0.00000000002*cos(4.53666662059 + 117076.82218358829*$t);
   $Y0 += 0.00000000002*cos(4.53666662059 + 117077.30981855528*$t);
   $Y0 += 0.00000000002*cos(0.67052215512 + 24814.97829846349*$t);
   $Y0 += 0.00000000002*cos(0.41491214788 + 27299.4880294167*$t);
   $Y0 += 0.00000000002*cos(2.10712022974 + 24876.8058886987*$t);
   $Y0 += 0.00000000002*cos(4.40848896505 + 26086.17485138239*$t);
   $Y0 += 0.00000000002*cos(4.17219070503 + 105418.35186097029*$t);
   $Y0 += 0.00000000002*cos(1.12019984719 + 49534.7088221597*$t);
   $Y0 += 0.00000000002*cos(2.68381144033 + 107679.99204006046*$t);
   $Y0 += 0.00000000002*cos(6.14277431051 + 19337.1061187119*$t);
   $Y0 += 0.00000000002*cos(0.23628622399 + 24957.9930114719*$t);
   $Y0 += 0.00000000002*cos(2.28574615363 + 27218.3009066435*$t);
   $Y0 += 0.00000000002*cos(5.03978204688 + 66826.36222024946*$t);
   $Y0 += 0.00000000002*cos(1.54597568017 + 27257.7352104663*$t);
   $Y0 += 0.00000000002*cos(3.11893314385 + 26197.86564531009*$t);
   $Y0 += 0.00000000002*cos(3.83163915021 + 151199.69892357888*$t);
   $Y0 += 0.00000000002*cos(0.84426255302 + 426.1525939975*$t);
   $Y0 += 0.00000000002*cos(3.62580652226 + 76784.59903317469*$t);
   $Y0 += 0.00000000002*cos(0.46652350366 + 179172.50322857044*$t);
   $Y0 += 0.00000000002*cos(4.60919202058 + 168959.21768235945*$t);
   $Y0 += 0.00000000002*cos(1.46759936699 + 168959.70531732646*$t);
   $Y0 += 0.00000000002*cos(1.33905329827 + 7829.84871395129*$t);
   $Y0 += 0.00000000002*cos(4.50925409872 + 151975.22153490288*$t);
   $Y0 += 0.00000000002*cos(3.99530485240 + 20235.36664379391*$t);
   $Y0 += 0.00000000002*cos(4.80991283240 + 31940.9272743215*$t);
   $Y0 += 0.00000000002*cos(5.05502953319 + 1585.38455950421*$t);
   $Y0 += 0.00000000002*cos(4.82744008834 + 76.0222537921*$t);
   $Y0 += 0.00000000002*cos(6.07753628311 + 1581.7155307995*$t);
   $Y0 += 0.00000000002*cos(5.83182532562 + 52643.5274560193*$t);
   $Y0 += 0.00000000002*cos(5.61086631842 + 130433.13096645445*$t);
   $Y0 += 0.00000000002*cos(3.78215414688 + 79956.11891170849*$t);
   $Y0 += 0.00000000002*cos(2.95202880616 + 190809.35350674443*$t);
   $Y0 += 0.00000000002*cos(0.51510001678 + 155417.79647986828*$t);
   $Y0 += 0.00000000002*cos(3.82552098141 + 58219.79104704029*$t);
   $Y0 += 0.00000000002*cos(4.10273299229 + 299399.22102519739*$t);
   $Y0 += 0.00000000002*cos(5.54210405976 + 51738.40620949199*$t);
   $Y0 += 0.00000000002*cos(1.39992677247 + 2045.5523502857*$t);
   $Y0 += 0.00000000002*cos(2.83807224501 + 27953.3783615753*$t);
   $Y0 += 0.00000000002*cos(5.96714543979 + 24222.9155565401*$t);
   $Y0 += 0.00000000002*cos(6.18845161198 + 164721.45036517028*$t);
   $Y0 += 0.00000000002*cos(4.74202892331 + 28441.0131128295*$t);
   $Y0 += 0.00000000002*cos(4.06318876149 + 23735.2808052859*$t);
   $Y0 += 0.00000000002*cos(3.36973154287 + 26948.4568878105*$t);
   $Y0 += 0.00000000002*cos(5.43548614193 + 25227.83703030489*$t);
   $Y0 += 0.00000000002*cos(3.83932556111 + 148532.64658993746*$t);
   $Y0 += 0.00000000002*cos(0.69773290752 + 148533.13422490445*$t);
   $Y0 += 0.00000000002*cos(3.08932581659 + 77307.17645126847*$t);
   $Y0 += 0.00000000002*cos(4.53508525261 + 191.6920835951*$t);
   $Y0 += 0.00000000002*cos(1.91266463584 + 23976.00901071309*$t);
   $Y0 += 0.00000000002*cos(2.51070394013 + 206370.27255720507*$t);
   $Y0 += 0.00000000002*cos(3.48548328265 + 4268.3379956771*$t);
   $Y0 += 0.00000000002*cos(0.27071972210 + 23962.2695516785*$t);
   $Y0 += 0.00000000002*cos(3.41231237569 + 23961.78191671149*$t);
   $Y0 += 0.00000000002*cos(2.77965182012 + 3776.6747032183*$t);
   $Y0 += 0.00000000002*cos(0.63445271713 + 24918.55870764909*$t);
   $Y0 += 0.00000000002*cos(4.78488159185 + 102769.40940053028*$t);
   $Y0 += 0.00000000002*cos(0.55778477986 + 25889.0749576213*$t);
   $Y0 += 0.00000000002*cos(1.96424759776 + 26287.21896049409*$t);
   $Y0 += 0.00000000002*cos(3.23447204926 + 2648.2110079895*$t);
   $Y0 += 0.00000000002*cos(4.14592395979 + 80432.71372448868*$t);
   $Y0 += 0.00000000002*cos(1.34549805675 + 13989.9602494293*$t);
   $Y0 += 0.00000000002*cos(1.17653432087 + 38186.3336686861*$t);
   $Y0 += 0.00000000002*cos(2.04577034444 + 8194.5191506921*$t);
   $Y0 += 0.00000000002*cos(5.70902071155 + 237009.64156087102*$t);
   $Y0 += 0.00000000002*cos(3.82569277203 + 24863.84148331209*$t);
   $Y0 += 0.00000000002*cos(5.50674253073 + 76064.86589891909*$t);
   $Y0 += 0.00000000002*cos(1.39914737585 + 71492.75500599349*$t);
   $Y0 += 0.00000000002*cos(6.18696209768 + 30831.5487222281*$t);
   $Y0 += 0.00000000002*cos(2.61825558712 + 21344.7451958873*$t);
   $Y0 += 0.00000000002*cos(1.07023698053 + 124663.98818751707*$t);
   $Y0 += 0.00000000001*cos(5.98706010174 + 101702.91392334028*$t);
   $Y0 += 0.00000000002*cos(1.35473982240 + 105404.12476696869*$t);
   $Y0 += 0.00000000002*cos(3.48042408913 + 307.31380348609*$t);
   $Y0 += 0.00000000002*cos(5.91402019226 + 184834.32291268966*$t);
   $Y0 += 0.00000000002*cos(5.91402019226 + 184833.83527772261*$t);
   $Y0 += 0.00000000001*cos(0.49434611754 + 124156.19604039608*$t);
   $Y0 += 0.00000000001*cos(4.86177343922 + 22909.51353352309*$t);
   $Y0 += 0.00000000001*cos(1.72018078563 + 22910.0011684901*$t);
   $Y0 += 0.00000000001*cos(5.79706734000 + 958.87904247409*$t);
   $Y0 += 0.00000000002*cos(2.31704185815 + 77196.97013004909*$t);
   $Y0 += 0.00000000002*cos(4.18763806284 + 9591.29792788829*$t);
   $Y0 += 0.00000000002*cos(2.60184968404 + 189385.78877478145*$t);
   $Y0 += 0.00000000001*cos(5.29048508788 + 24919.2481220111*$t);
   $Y0 += 0.00000000001*cos(3.51473259692 + 27257.0457961043*$t);
   $Y0 += 0.00000000002*cos(5.03520614299 + 25440.64849077589*$t);
   $Y0 += 0.00000000001*cos(5.16795059538 + 26010.3964150738*$t);
   $Y0 += 0.00000000002*cos(5.13757006415 + 43693.8869073535*$t);
   $Y0 += 0.00000000002*cos(5.04394896979 + 25014.05029989989*$t);
   $Y0 += 0.00000000001*cos(3.77902331302 + 9385.0848255587*$t);
   $Y0 += 0.00000000002*cos(6.17372295311 + 26202.09861277591*$t);
   $Y0 += 0.00000000001*cos(3.35135060198 + 52499.55551728929*$t);
   $Y0 += 0.00000000002*cos(5.03234764639 + 14650.55593710108*$t);
   $Y0 += 0.00000000002*cos(0.37520258538 + 78690.06379811508*$t);
   $Y0 += 0.00000000001*cos(1.65843797188 + 5039.34443082949*$t);
   $Y0 += 0.00000000002*cos(2.53532060227 + 2214.98690507971*$t);
   $Y0 += 0.00000000001*cos(5.07384686444 + 77211.19722405069*$t);
   $Y0 += 0.00000000001*cos(4.11905908237 + 233202.29919636747*$t);
   $Y0 += 0.00000000002*cos(3.92094587643 + 21516.52370109911*$t);
   $Y0 += 0.00000000002*cos(4.88427180837 + 30659.77021701629*$t);
   $Y0 += 0.00000000002*cos(0.59200954056 + 621.49998335569*$t);
   $Y0 += 0.00000000002*cos(2.70800261719 + 25764.15390743329*$t);
   $Y0 += 0.00000000001*cos(3.75051661568 + 26.5421372833*$t);
   $Y0 += 0.00000000001*cos(3.87625546504 + 226796.35601466001*$t);
   $Y0 += 0.00000000001*cos(3.87625546504 + 226796.84364962706*$t);
   $Y0 += 0.00000000002*cos(5.04685227536 + 28839.69455874929*$t);
   $Y0 += 0.00000000002*cos(3.93268001276 + 102225.97897640108*$t);
   $Y0 += 0.00000000002*cos(6.27298790851 + 129971.79453500008*$t);
   $Y0 += 0.00000000002*cos(5.63544803962 + 27139.92770727909*$t);
   $Y0 += 0.00000000001*cos(1.05389010615 + 50807.38984778689*$t);
   $Y0 += 0.00000000001*cos(1.45067645500 + 78187.19953596349*$t);
   $Y0 += 0.00000000002*cos(3.49292360736 + 51472.41691601449*$t);
   return $Y0;
}



   function Mercury_Y1($t) // 1023 terms of order 1
{
   $Y1  = 0.00328639517*cos(4.46949126315 + 0.2438174835*$t);
   $Y1 += 0.00106107047*cos(4.34458837257 + 52176.0501006319*$t);
   $Y1 += 0.00032448440*cos(1.11324531456 + 78263.95324220609*$t);
   $Y1 += 0.00009699418*cos(0.71696945020 + 26087.65932409069*$t);
   $Y1 += 0.00008808490*cos(4.16435393159 + 104351.85638378029*$t);
   $Y1 += 0.00004833058*cos(4.93174326512 + 26088.1469590577*$t);
   $Y1 += 0.00002334329*cos(0.93209482754 + 130439.75952535449*$t);
   $Y1 += 0.00000890250*cos(4.18838297765 + 52175.56246566489*$t);
   $Y1 += 0.00000615643*cos(3.98294823708 + 156527.66266692868*$t);
   $Y1 += 0.00000147225*cos(1.38491342638 + 78263.46560723908*$t);
   $Y1 += 0.00000162460*cos(0.75058057907 + 182615.56580850288*$t);
   $Y1 += 0.00000069502*cos(1.34859955230 + 24978.7684069643*$t);
   $Y1 += 0.00000063529*cos(1.17140322075 + 27197.5255111511*$t);
   $Y1 += 0.00000042967*cos(3.80137602608 + 208703.46895007708*$t);
   $Y1 += 0.00000032514*cos(4.75515678790 + 104351.36874881329*$t);
   $Y1 += 0.00000017597*cos(4.40134253408 + 51066.6715485385*$t);
   $Y1 += 0.00000013287*cos(4.22067115617 + 53285.4286527253*$t);
   $Y1 += 0.00000011758*cos(1.75773369521 + 1109.1347346099*$t);
   $Y1 += 0.00000012502*cos(6.11835337393 + 1059.6257476727*$t);
   $Y1 += 0.00000011468*cos(5.06247506599 + 20426.8149099055*$t);
   $Y1 += 0.00000011394*cos(0.56897531807 + 234791.37209165128*$t);
   $Y1 += 0.00000008221*cos(3.74149753253 + 31749.4790082099*$t);
   $Y1 += 0.00000008067*cos(1.73031128392 + 130439.27189038748*$t);
   $Y1 += 0.00000006176*cos(3.89018803821 + 4552.1973145423*$t);
   $Y1 += 0.00000004784*cos(4.97654366187 + 21536.1934619989*$t);
   $Y1 += 0.00000004858*cos(1.16998032311 + 77154.57469011268*$t);
   $Y1 += 0.00000004269*cos(1.82558089427 + 46514.7180514797*$t);
   $Y1 += 0.00000004412*cos(3.53054557745 + 25132.5472174491*$t);
   $Y1 += 0.00000003922*cos(0.80899339690 + 1589.3167127673*$t);
   $Y1 += 0.00000004017*cos(2.84593275079 + 27147.5288892469*$t);
   $Y1 += 0.00000003763*cos(2.60830060517 + 14765.48286075331*$t);
   $Y1 += 0.00000004285*cos(2.70027245882 + 51116.6681704427*$t);
   $Y1 += 0.00000003901*cos(5.26131971011 + 27043.7467006663*$t);
   $Y1 += 0.00000003221*cos(5.81041475267 + 26617.8379241523*$t);
   $Y1 += 0.00000003080*cos(3.11167500886 + 25558.4559939631*$t);
   $Y1 += 0.00000003030*cos(3.61976417137 + 260879.27523322542*$t);
   $Y1 += 0.00000002907*cos(4.94283048350 + 26068.47719815791*$t);
   $Y1 += 0.00000002906*cos(3.86228296506 + 26107.81671995749*$t);
   $Y1 += 0.00000002383*cos(0.50751116011 + 26084.2656236997*$t);
   $Y1 += 0.00000002381*cos(2.01445897738 + 26092.0282944157*$t);
   $Y1 += 0.00000002069*cos(6.19366221907 + 37410.8110573621*$t);
   $Y1 += 0.00000002028*cos(3.87421613919 + 30640.1004561165*$t);
   $Y1 += 0.00000002088*cos(4.91615750193 + 156527.17503196168*$t);
   $Y1 += 0.00000001998*cos(0.93225615522 + 79373.33179429949*$t);
   $Y1 += 0.00000001880*cos(4.93819019021 + 47624.0966035731*$t);
   $Y1 += 0.00000001580*cos(0.12078980001 + 9104.1508116011*$t);
   $Y1 += 0.00000001575*cos(5.88747787298 + 955.3559241251*$t);
   $Y1 += 0.00000001508*cos(5.14798771540 + 19317.43635781211*$t);
   $Y1 += 0.00000001397*cos(4.87619009368 + 72602.62119305388*$t);
   $Y1 += 0.00000001363*cos(1.71613837922 + 50586.97720534809*$t);
   $Y1 += 0.00000001495*cos(2.20742717983 + 12566.3955174663*$t);
   $Y1 += 0.00000001401*cos(0.47782643773 + 426.8420083595*$t);
   $Y1 += 0.00000001215*cos(2.36706136703 + 52705.7410657265*$t);
   $Y1 += 0.00000001310*cos(0.91525566474 + 26095.2605060585*$t);
   $Y1 += 0.00000001272*cos(1.65165069758 + 26081.03341205689*$t);
   $Y1 += 0.00000001476*cos(1.42289389144 + 23869.3898548709*$t);
   $Y1 += 0.00000001169*cos(3.92032031637 + 3442.81876244891*$t);
   $Y1 += 0.00000001103*cos(5.93142816578 + 53235.43203082109*$t);
   $Y1 += 0.00000001375*cos(4.22153989222 + 103242.47783168689*$t);
   $Y1 += 0.00000001244*cos(0.25422201490 + 51220.4503590233*$t);
   $Y1 += 0.00000001238*cos(1.09570749211 + 28306.90406324449*$t);
   $Y1 += 0.00000000967*cos(2.00095625807 + 53131.6498422405*$t);
   $Y1 += 0.00000000986*cos(3.65234674915 + 32858.8575603033*$t);
   $Y1 += 0.00000000911*cos(5.86347762782 + 529.44714761109*$t);
   $Y1 += 0.00000000859*cos(5.65763791955 + 40853.3860023275*$t);
   $Y1 += 0.00000000855*cos(3.76170634741 + 27677.2198543415*$t);
   $Y1 += 0.00000000844*cos(5.74887533238 + 25874.8478636197*$t);
   $Y1 += 0.00000001042*cos(4.90279640329 + 22645.57201409229*$t);
   $Y1 += 0.00000000924*cos(5.45463398257 + 640.1411037975*$t);
   $Y1 += 0.00000000810*cos(3.07111874736 + 26301.4460544957*$t);
   $Y1 += 0.00000000807*cos(0.38735610376 + 286967.17837479962*$t);
   $Y1 += 0.00000000875*cos(0.56063513084 + 83925.28529135829*$t);
   $Y1 += 0.00000000874*cos(3.97964673458 + 529.9347825781*$t);
   $Y1 += 0.00000000821*cos(3.19953070235 + 36301.4325052687*$t);
   $Y1 += 0.00000000823*cos(0.48663689329 + 11322.42028082089*$t);
   $Y1 += 0.00000000738*cos(0.62901815542 + 56728.0035976907*$t);
   $Y1 += 0.00000000724*cos(6.21294643252 + 18849.4713674577*$t);
   $Y1 += 0.00000000714*cos(4.24977924185 + 6770.46678376209*$t);
   $Y1 += 0.00000000667*cos(2.40282291967 + 43072.1431065143*$t);
   $Y1 += 0.00000000665*cos(1.71193023872 + 52156.38033973211*$t);
   $Y1 += 0.00000000893*cos(1.99786430454 + 27197.03787618409*$t);
   $Y1 += 0.00000000752*cos(0.31686311777 + 39609.8984006491*$t);
   $Y1 += 0.00000000673*cos(3.24236433600 + 1052.51220067191*$t);
   $Y1 += 0.00000000688*cos(5.57273946980 + 15874.8614128467*$t);
   $Y1 += 0.00000000618*cos(0.55635839524 + 5661.5758666357*$t);
   $Y1 += 0.00000000808*cos(2.76566323402 + 1066.7392946735*$t);
   $Y1 += 0.00000000616*cos(5.41694945887 + 51646.3591355373*$t);
   $Y1 += 0.00000000650*cos(6.10338500409 + 25028.7650288685*$t);
   $Y1 += 0.00000000550*cos(1.77567872746 + 182615.07817353586*$t);
   $Y1 += 0.00000000554*cos(1.44404571039 + 2119.00767786191*$t);
   $Y1 += 0.00000000535*cos(5.07158844336 + 52179.9314359899*$t);
   $Y1 += 0.00000000657*cos(3.92169541273 + 29530.7219040231*$t);
   $Y1 += 0.00000000527*cos(3.36241476084 + 26514.7451499337*$t);
   $Y1 += 0.00000000502*cos(5.23707918628 + 38654.2986590405*$t);
   $Y1 += 0.00000000568*cos(1.89274232427 + 24978.28077199729*$t);
   $Y1 += 0.00000000498*cos(2.94458194590 + 63498.71419893629*$t);
   $Y1 += 0.00000000469*cos(3.58596298240 + 110013.18843293248*$t);
   $Y1 += 0.00000000521*cos(0.62400943535 + 52195.71986153169*$t);
   $Y1 += 0.00000000440*cos(1.64399949118 + 98690.52433462809*$t);
   $Y1 += 0.00000000580*cos(4.69289380268 + 7.3573644843*$t);
   $Y1 += 0.00000000438*cos(2.54950574588 + 33326.8225506577*$t);
   $Y1 += 0.00000000530*cos(1.70361729355 + 2218.51328670329*$t);
   $Y1 += 0.00000000483*cos(2.07663666251 + 51749.45190975589*$t);
   $Y1 += 0.00000000434*cos(3.55771117131 + 52172.1687652739*$t);
   $Y1 += 0.00000000408*cos(3.22512461303 + 7238.4317741165*$t);
   $Y1 += 0.00000000390*cos(5.33070151567 + 78793.64420730069*$t);
   $Y1 += 0.00000000374*cos(0.44775435653 + 58946.76070187749*$t);
   $Y1 += 0.00000000404*cos(4.44096026376 + 52168.93655363109*$t);
   $Y1 += 0.00000000356*cos(6.23046854548 + 62389.33564684289*$t);
   $Y1 += 0.00000000373*cos(5.69827259585 + 103292.47445359109*$t);
   $Y1 += 0.00000000326*cos(2.72823636104 + 79323.33517239528*$t);
   $Y1 += 0.00000000393*cos(0.98972344950 + 129330.38097326108*$t);
   $Y1 += 0.00000000385*cos(3.94475257678 + 52183.1636476327*$t);
   $Y1 += 0.00000000316*cos(4.54068234172 + 76674.88034692229*$t);
   $Y1 += 0.00000000390*cos(4.17402259459 + 54394.80720481869*$t);
   $Y1 += 0.00000000323*cos(4.47818749821 + 49957.2929964451*$t);
   $Y1 += 0.00000000299*cos(3.00180406182 + 16983.75232997309*$t);
   $Y1 += 0.00000000328*cos(2.47784503610 + 68050.66769599509*$t);
   $Y1 += 0.00000000314*cos(3.34927785340 + 51536.15281431789*$t);
   $Y1 += 0.00000000367*cos(3.28176962437 + 77308.35350059748*$t);
   $Y1 += 0.00000000316*cos(1.92123984210 + 45405.3394993863*$t);
   $Y1 += 0.00000000319*cos(2.66908913597 + 13656.10430865991*$t);
   $Y1 += 0.00000000266*cos(4.93516999025 + 48733.47515566649*$t);
   $Y1 += 0.00000000289*cos(1.34161862787 + 6.86972951729*$t);
   $Y1 += 0.00000000261*cos(2.43104654910 + 16984.2399649401*$t);
   $Y1 += 0.00000000261*cos(6.20453445714 + 52389.3491960699*$t);
   $Y1 += 0.00000000233*cos(5.55470184122 + 51123.7817174435*$t);
   $Y1 += 0.00000000231*cos(3.67240300147 + 82815.90673926489*$t);
   $Y1 += 0.00000000239*cos(1.23297150719 + 213.5429129215*$t);
   $Y1 += 0.00000000222*cos(2.42341131487 + 66941.2891439017*$t);
   $Y1 += 0.00000000216*cos(3.43857770880 + 313055.08151637383*$t);
   $Y1 += 0.00000000276*cos(6.03736928820 + 51109.55462344189*$t);
   $Y1 += 0.00000000222*cos(1.79769065581 + 32371.2228090491*$t);
   $Y1 += 0.00000000198*cos(0.48217448832 + 53765.1229959157*$t);
   $Y1 += 0.00000000197*cos(5.45332171285 + 69160.04624808849*$t);
   $Y1 += 0.00000000196*cos(1.08031906323 + 50057.2862402535*$t);
   $Y1 += 0.00000000202*cos(2.45524349258 + 129380.37759516528*$t);
   $Y1 += 0.00000000201*cos(2.89680722301 + 44937.3745090319*$t);
   $Y1 += 0.00000000216*cos(3.72207636306 + 213.0552779545*$t);
   $Y1 += 0.00000000185*cos(0.34499909570 + 136101.09157450669*$t);
   $Y1 += 0.00000000183*cos(0.69231889541 + 419.72846135871*$t);
   $Y1 += 0.00000000186*cos(5.71673843636 + 57837.3821497841*$t);
   $Y1 += 0.00000000178*cos(3.70090079088 + 105461.23493587368*$t);
   $Y1 += 0.00000000177*cos(2.18468434488 + 51962.7510051939*$t);
   $Y1 += 0.00000000170*cos(4.76419946756 + 78244.28348130629*$t);
   $Y1 += 0.00000000229*cos(0.17271876456 + 433.9555553603*$t);
   $Y1 += 0.00000000173*cos(5.50271484365 + 94138.57083756929*$t);
   $Y1 += 0.00000000157*cos(2.02010274317 + 64742.2018006147*$t);
   $Y1 += 0.00000000153*cos(5.58836484297 + 59414.7256922319*$t);
   $Y1 += 0.00000000159*cos(4.59819683221 + 853.4401992355*$t);
   $Y1 += 0.00000000155*cos(3.11896173244 + 35192.0539531753*$t);
   $Y1 += 0.00000000149*cos(5.04913257055 + 24499.0740637739*$t);
   $Y1 += 0.00000000152*cos(1.43713086803 + 4551.7096795753*$t);
   $Y1 += 0.00000000147*cos(0.13661195710 + 52602.6482915079*$t);
   $Y1 += 0.00000000147*cos(0.58488249671 + 19805.0711090663*$t);
   $Y1 += 0.00000000182*cos(0.68589997649 + 55618.6250455973*$t);
   $Y1 += 0.00000000140*cos(4.81668277749 + 23754.95056618569*$t);
   $Y1 += 0.00000000146*cos(4.89226138375 + 208702.98131511007*$t);
   $Y1 += 0.00000000154*cos(1.81985059116 + 99799.90288672148*$t);
   $Y1 += 0.00000000139*cos(4.63018939309 + 25973.70767037247*$t);
   $Y1 += 0.00000000138*cos(4.17371703563 + 26202.58624774291*$t);
   $Y1 += 0.00000000191*cos(2.30404978778 + 26728.0442453717*$t);
   $Y1 += 0.00000000152*cos(4.17149146165 + 1109.6223695769*$t);
   $Y1 += 0.00000000162*cos(6.13333012195 + 38520.1896094555*$t);
   $Y1 += 0.00000000141*cos(5.95496888155 + 77734.26227711148*$t);
   $Y1 += 0.00000000135*cos(1.84463199555 + 78267.83457756409*$t);
   $Y1 += 0.00000000128*cos(2.99907290513 + 88477.23878841709*$t);
   $Y1 += 0.00000000147*cos(0.62089024684 + 31748.99137324289*$t);
   $Y1 += 0.00000000135*cos(4.69617813235 + 124778.42747620228*$t);
   $Y1 += 0.00000000148*cos(5.85857347873 + 27154.6424362477*$t);
   $Y1 += 0.00000000134*cos(0.45601803820 + 12431.79883291429*$t);
   $Y1 += 0.00000000121*cos(2.04615713494 + 104881.54734887488*$t);
   $Y1 += 0.00000000116*cos(3.19167458045 + 25934.3681485729*$t);
   $Y1 += 0.00000000116*cos(4.22027620935 + 32858.36992533629*$t);
   $Y1 += 0.00000000115*cos(5.61211222559 + 26241.9257695425*$t);
   $Y1 += 0.00000000148*cos(5.73338066299 + 25661.5487681817*$t);
   $Y1 += 0.00000000110*cos(4.42886613408 + 28206.9108194361*$t);
   $Y1 += 0.00000000108*cos(0.11698734549 + 27140.41534224609*$t);
   $Y1 += 0.00000000115*cos(2.44781109373 + 103822.16541868569*$t);
   $Y1 += 0.00000000109*cos(5.49669317005 + 53284.94101775829*$t);
   $Y1 += 0.00000000107*cos(3.99865946781 + 28421.3433519297*$t);
   $Y1 += 0.00000000143*cos(3.36395963187 + 1596.43025976811*$t);
   $Y1 += 0.00000000112*cos(4.03868201051 + 155418.28411483529*$t);
   $Y1 += 0.00000000113*cos(0.89157360809 + 5327.7199258663*$t);
   $Y1 += 0.00000000101*cos(1.94476004771 + 25620.1819687033*$t);
   $Y1 += 0.00000000122*cos(1.46662898961 + 10213.5293636945*$t);
   $Y1 += 0.00000000102*cos(3.99402750542 + 2333.44021035551*$t);
   $Y1 += 0.00000000097*cos(0.57476621767 + 26556.11194941211*$t);
   $Y1 += 0.00000000118*cos(0.18981536711 + 7994.77225950771*$t);
   $Y1 += 0.00000000109*cos(0.03572167179 + 103396.25664217169*$t);
   $Y1 += 0.00000000109*cos(4.54205508167 + 79219.55298381469*$t);
   $Y1 += 0.00000000095*cos(4.81791418885 + 58459.1259506233*$t);
   $Y1 += 0.00000000095*cos(1.17289793021 + 78256.83969520529*$t);
   $Y1 += 0.00000000097*cos(5.79863208794 + 105411.23831396949*$t);
   $Y1 += 0.00000000092*cos(3.83355549192 + 10213.0417287275*$t);
   $Y1 += 0.00000000094*cos(2.84811066413 + 1582.2031657665*$t);
   $Y1 += 0.00000000103*cos(3.37710316075 + 91785.70468379749*$t);
   $Y1 += 0.00000000101*cos(5.05588975902 + 20426.32727493849*$t);
   $Y1 += 0.00000000089*cos(1.67320799366 + 74821.37829724069*$t);
   $Y1 += 0.00000000089*cos(4.81402637705 + 125887.80602829569*$t);
   $Y1 += 0.00000000103*cos(5.07765700675 + 51066.18391357149*$t);
   $Y1 += 0.00000000087*cos(5.93439795460 + 27043.2590656993*$t);
   $Y1 += 0.00000000083*cos(1.05061824879 + 73711.99974514729*$t);
   $Y1 += 0.00000000083*cos(0.20717662715 + 77624.05595589208*$t);
   $Y1 += 0.00000000097*cos(3.79624621564 + 71980.87739221469*$t);
   $Y1 += 0.00000000081*cos(5.50420619898 + 155468.28073673949*$t);
   $Y1 += 0.00000000080*cos(5.57436544726 + 22645.08437912529*$t);
   $Y1 += 0.00000000081*cos(1.24551992048 + 76045.1961380193*$t);
   $Y1 += 0.00000000081*cos(3.47212148199 + 26138.1435809619*$t);
   $Y1 += 0.00000000080*cos(5.32926891572 + 26038.1503371535*$t);
   $Y1 += 0.00000000104*cos(0.71084047438 + 78271.06678920689*$t);
   $Y1 += 0.00000000103*cos(3.66294929279 + 78283.62300310588*$t);
   $Y1 += 0.00000000083*cos(0.93556744308 + 80482.71034639288*$t);
   $Y1 += 0.00000000089*cos(0.32257506891 + 78260.07190684808*$t);
   $Y1 += 0.00000000073*cos(5.79066657626 + 26617.35028918529*$t);
   $Y1 += 0.00000000077*cos(5.23137947798 + 18208.05780571871*$t);
   $Y1 += 0.00000000082*cos(2.34137394607 + 77211.68485901768*$t);
   $Y1 += 0.00000000077*cos(1.31837686051 + 41962.7645544209*$t);
   $Y1 += 0.00000000082*cos(2.81188130706 + 77197.45776501608*$t);
   $Y1 += 0.00000000067*cos(3.45344753309 + 85034.66384345168*$t);
   $Y1 += 0.00000000070*cos(5.80811525915 + 537.0483295789*$t);
   $Y1 += 0.00000000067*cos(1.90240647027 + 5661.0882316687*$t);
   $Y1 += 0.00000000067*cos(2.25742587365 + 120226.47397914348*$t);
   $Y1 += 0.00000000065*cos(3.38772930583 + 162188.99471608088*$t);
   $Y1 += 0.00000000075*cos(3.36246887110 + 149.8070146181*$t);
   $Y1 += 0.00000000071*cos(0.43774462141 + 108903.80988083909*$t);
   $Y1 += 0.00000000063*cos(1.71956484187 + 51756.5654567567*$t);
   $Y1 += 0.00000000062*cos(0.86384103129 + 102762.78348849648*$t);
   $Y1 += 0.00000000067*cos(2.92995035330 + 64608.09275102969*$t);
   $Y1 += 0.00000000064*cos(3.59312937724 + 13521.9952590749*$t);
   $Y1 += 0.00000000062*cos(1.70048284317 + 28306.41642827749*$t);
   $Y1 += 0.00000000058*cos(0.05666653789 + 11322.9079157879*$t);
   $Y1 += 0.00000000058*cos(1.85046002910 + 24498.58642880689*$t);
   $Y1 += 0.00000000077*cos(2.22443127486 + 25565.5695409639*$t);
   $Y1 += 0.00000000076*cos(3.03069740917 + 78477.25233764409*$t);
   $Y1 += 0.00000000063*cos(5.48328560721 + 93029.19228547589*$t);
   $Y1 += 0.00000000078*cos(4.97356487718 + 71493.2426409605*$t);
   $Y1 += 0.00000000077*cos(2.34741649134 + 51742.33836275509*$t);
   $Y1 += 0.00000000057*cos(2.02149140578 + 2648.6986429565*$t);
   $Y1 += 0.00000000058*cos(0.20352146431 + 339142.98465794802*$t);
   $Y1 += 0.00000000056*cos(3.96222140968 + 76145.18938182769*$t);
   $Y1 += 0.00000000059*cos(1.23567785929 + 21535.70582703189*$t);
   $Y1 += 0.00000000056*cos(0.13198299548 + 117873.60782537168*$t);
   $Y1 += 0.00000000053*cos(6.21797647818 + 25557.96835899609*$t);
   $Y1 += 0.00000000060*cos(5.92257554155 + 71025.27765060609*$t);
   $Y1 += 0.00000000052*cos(5.28001264458 + 10022.0810975829*$t);
   $Y1 += 0.00000000055*cos(4.20817850068 + 51322.85371887989*$t);
   $Y1 += 0.00000000058*cos(2.97195320983 + 43071.65547154729*$t);
   $Y1 += 0.00000000053*cos(3.50115924170 + 79853.02613748988*$t);
   $Y1 += 0.00000000053*cos(4.92881565492 + 78050.65414676809*$t);
   $Y1 += 0.00000000053*cos(0.59227243554 + 98068.78053378889*$t);
   $Y1 += 0.00000000057*cos(5.95602153522 + 633.0275567967*$t);
   $Y1 += 0.00000000049*cos(5.08646367150 + 90830.10494218889*$t);
   $Y1 += 0.00000000067*cos(0.27295012704 + 26610.72437715149*$t);
   $Y1 += 0.00000000050*cos(4.43506318049 + 131549.13807744789*$t);
   $Y1 += 0.00000000049*cos(3.57210369085 + 33968.23611239669*$t);
   $Y1 += 0.00000000050*cos(3.73457689628 + 81706.5281871715*$t);
   $Y1 += 0.00000000051*cos(5.44465818352 + 129910.06856025988*$t);
   $Y1 += 0.00000000057*cos(1.99278813028 + 26624.9514711531*$t);
   $Y1 += 0.00000000045*cos(4.93934477894 + 51116.18053547569*$t);
   $Y1 += 0.00000000045*cos(1.53008615762 + 104332.1866228805*$t);
   $Y1 += 0.00000000051*cos(5.56661489639 + 647.25465079831*$t);
   $Y1 += 0.00000000043*cos(3.21201517479 + 78690.55143308209*$t);
   $Y1 += 0.00000000054*cos(0.68244488765 + 522.8212355773*$t);
   $Y1 += 0.00000000042*cos(6.05067077478 + 114565.14192999128*$t);
   $Y1 += 0.00000000041*cos(0.42167567232 + 38519.70197448849*$t);
   $Y1 += 0.00000000041*cos(5.52226173141 + 25028.27739390149*$t);
   $Y1 += 0.00000000052*cos(5.71899801337 + 39744.0074502341*$t);
   $Y1 += 0.00000000043*cos(5.20652224208 + 103925.25819290428*$t);
   $Y1 += 0.00000000045*cos(3.01197275808 + 76.50988875911*$t);
   $Y1 += 0.00000000042*cos(1.64710245004 + 46848.5739922491*$t);
   $Y1 += 0.00000000047*cos(2.33748031419 + 44181.52165860769*$t);
   $Y1 += 0.00000000038*cos(1.51512405751 + 22760.01130277751*$t);
   $Y1 += 0.00000000037*cos(1.55847676346 + 151975.70916986988*$t);
   $Y1 += 0.00000000050*cos(0.66442193902 + 25551.34244696229*$t);
   $Y1 += 0.00000000050*cos(5.43960666857 + 50579.86365834729*$t);
   $Y1 += 0.00000000045*cos(4.18105807315 + 7879.84533585549*$t);
   $Y1 += 0.00000000043*cos(3.23954616248 + 26521.8586969345*$t);
   $Y1 += 0.00000000037*cos(4.29411552008 + 30639.61282114949*$t);
   $Y1 += 0.00000000037*cos(5.45971389197 + 48732.98752069949*$t);
   $Y1 += 0.00000000035*cos(1.61116129207 + 84547.0290921975*$t);
   $Y1 += 0.00000000040*cos(5.41660085121 + 52815.9473869459*$t);
   $Y1 += 0.00000000041*cos(5.72036408436 + 1059.1381127057*$t);
   $Y1 += 0.00000000041*cos(1.46421286291 + 150866.33061777649*$t);
   $Y1 += 0.00000000040*cos(3.44404714469 + 26294.33250749489*$t);
   $Y1 += 0.00000000041*cos(5.29927149302 + 25881.9614106205*$t);
   $Y1 += 0.00000000034*cos(5.65060802359 + 26191.23973327629*$t);
   $Y1 += 0.00000000034*cos(3.16023657143 + 25985.0541848391*$t);
   $Y1 += 0.00000000034*cos(5.54425040097 + 89586.61734051049*$t);
   $Y1 += 0.00000000041*cos(2.60243509566 + 52329.8289111167*$t);
   $Y1 += 0.00000000035*cos(0.95515596888 + 157637.04121902208*$t);
   $Y1 += 0.00000000042*cos(4.45181501350 + 26507.63160293289*$t);
   $Y1 += 0.00000000039*cos(1.71451949376 + 234790.88445668427*$t);
   $Y1 += 0.00000000037*cos(3.45787318433 + 57836.89451481709*$t);
   $Y1 += 0.00000000036*cos(5.05609057964 + 130969.45049044909*$t);
   $Y1 += 0.00000000032*cos(0.80695926075 + 181506.18725640948*$t);
   $Y1 += 0.00000000033*cos(5.95814421441 + 50594.09075234889*$t);
   $Y1 += 0.00000000044*cos(4.54253002200 + 6681.46867088311*$t);
   $Y1 += 0.00000000036*cos(4.89960028584 + 104355.73771913828*$t);
   $Y1 += 0.00000000032*cos(3.07469859082 + 129484.15978374588*$t);
   $Y1 += 0.00000000030*cos(3.63644868220 + 52644.0150909863*$t);
   $Y1 += 0.00000000031*cos(0.98929514792 + 29416.28261533789*$t);
   $Y1 += 0.00000000031*cos(6.01882424612 + 105307.45612538888*$t);
   $Y1 += 0.00000000030*cos(2.14561278666 + 95247.94938966268*$t);
   $Y1 += 0.00000000031*cos(0.94264719675 + 52290.48938931711*$t);
   $Y1 += 0.00000000029*cos(2.26390987562 + 181556.18387831368*$t);
   $Y1 += 0.00000000039*cos(0.22423030716 + 25448.2496727437*$t);
   $Y1 += 0.00000000032*cos(3.17151820552 + 51639.24558853649*$t);
   $Y1 += 0.00000000029*cos(2.64834173157 + 53242.5455778219*$t);
   $Y1 += 0.00000000029*cos(3.82338961652 + 25668.6623151825*$t);
   $Y1 += 0.00000000028*cos(4.61151969244 + 77204.57131201689*$t);
   $Y1 += 0.00000000036*cos(3.92497606168 + 25132.0595824821*$t);
   $Y1 += 0.00000000027*cos(0.76041081814 + 54509.2464935039*$t);
   $Y1 += 0.00000000026*cos(1.38925912724 + 52061.61081194667*$t);
   $Y1 += 0.00000000025*cos(5.15879998687 + 29428.7593857575*$t);
   $Y1 += 0.00000000031*cos(2.93989277053 + 18093.13088206649*$t);
   $Y1 += 0.00000000030*cos(1.42238495741 + 26941.3433408097*$t);
   $Y1 += 0.00000000025*cos(4.73209653345 + 956.53297345411*$t);
   $Y1 += 0.00000000027*cos(2.35402357443 + 18093.6185170335*$t);
   $Y1 += 0.00000000028*cos(3.75976953619 + 104358.96993078108*$t);
   $Y1 += 0.00000000026*cos(0.11444202162 + 26237.7101561923*$t);
   $Y1 += 0.00000000024*cos(3.77280410253 + 22747.5345323579*$t);
   $Y1 += 0.00000000024*cos(2.17839409058 + 46514.23041651269*$t);
   $Y1 += 0.00000000029*cos(2.57773209519 + 131499.14145554368*$t);
   $Y1 += 0.00000000024*cos(3.54215594406 + 42154.2128205325*$t);
   $Y1 += 0.00000000024*cos(1.15126185636 + 54294.81396101029*$t);
   $Y1 += 0.00000000023*cos(5.30608917177 + 146314.37712071768*$t);
   $Y1 += 0.00000000024*cos(4.83980442527 + 26107.32908499049*$t);
   $Y1 += 0.00000000026*cos(2.88358798527 + 25021.6514818677*$t);
   $Y1 += 0.00000000023*cos(2.25657344280 + 20760.1832157079*$t);
   $Y1 += 0.00000000023*cos(1.97184358207 + 77154.08705514569*$t);
   $Y1 += 0.00000000024*cos(5.65078963984 + 26067.9895631909*$t);
   $Y1 += 0.00000000023*cos(2.17997702001 + 25035.8785758693*$t);
   $Y1 += 0.00000000023*cos(3.16772215014 + 143961.51096694589*$t);
   $Y1 += 0.00000000026*cos(0.15726115955 + 27684.3334013423*$t);
   $Y1 += 0.00000000022*cos(5.04032506297 + 19804.5834740993*$t);
   $Y1 += 0.00000000025*cos(1.50229754572 + 26080.54577708989*$t);
   $Y1 += 0.00000000023*cos(1.92683823556 + 130013.16133447849*$t);
   $Y1 += 0.00000000022*cos(2.30233150466 + 79372.84415933248*$t);
   $Y1 += 0.00000000025*cos(5.43362878578 + 52026.4869034973*$t);
   $Y1 += 0.00000000021*cos(4.22122242647 + 104344.74283677948*$t);
   $Y1 += 0.00000000021*cos(4.30076756548 + 102133.09927959349*$t);
   $Y1 += 0.00000000023*cos(6.10522918323 + 61279.9570947495*$t);
   $Y1 += 0.00000000021*cos(0.79325948829 + 85502.6288338061*$t);
   $Y1 += 0.00000000023*cos(4.81952053099 + 77844.46859833089*$t);
   $Y1 += 0.00000000022*cos(0.15814691647 + 188276.89785765509*$t);
   $Y1 += 0.00000000021*cos(3.64905245082 + 124156.68367536309*$t);
   $Y1 += 0.00000000027*cos(5.39563901242 + 103299.58800059189*$t);
   $Y1 += 0.00000000020*cos(1.06151057303 + 23968.89546371229*$t);
   $Y1 += 0.00000000023*cos(5.40249889826 + 77830.24150432929*$t);
   $Y1 += 0.00000000021*cos(5.85984642433 + 103285.36090659029*$t);
   $Y1 += 0.00000000020*cos(0.50079058372 + 49527.59527515889*$t);
   $Y1 += 0.00000000020*cos(0.72105141719 + 1479.11039154791*$t);
   $Y1 += 0.00000000020*cos(4.35275493153 + 23969.3830986793*$t);
   $Y1 += 0.00000000020*cos(3.38833851349 + 103711.95909746629*$t);
   $Y1 += 0.00000000019*cos(2.08111180650 + 52126.05347872769*$t);
   $Y1 += 0.00000000019*cos(1.62003983896 + 3327.89183879669*$t);
   $Y1 += 0.00000000022*cos(6.13482771447 + 104565.15547921829*$t);
   $Y1 += 0.00000000026*cos(5.97213001267 + 13362.6935242827*$t);
   $Y1 += 0.00000000025*cos(4.72474160158 + 24864.32911827909*$t);
   $Y1 += 0.00000000023*cos(5.79095805909 + 15874.37377787969*$t);
   $Y1 += 0.00000000022*cos(3.15825920598 + 6283.3196674749*$t);
   $Y1 += 0.00000000025*cos(4.07847502844 + 27311.96479983631*$t);
   $Y1 += 0.00000000019*cos(3.21308432623 + 53228.31848382029*$t);
   $Y1 += 0.00000000021*cos(1.74197598839 + 97581.14578253469*$t);
   $Y1 += 0.00000000018*cos(3.99624752196 + 106570.61348796709*$t);
   $Y1 += 0.00000000021*cos(0.35167836404 + 60056.13925397089*$t);
   $Y1 += 0.00000000019*cos(1.46314564325 + 26083.7779887327*$t);
   $Y1 += 0.00000000021*cos(0.41713451843 + 104371.52614468009*$t);
   $Y1 += 0.00000000017*cos(0.90189598200 + 1162.7185218913*$t);
   $Y1 += 0.00000000020*cos(5.87845266961 + 41962.27691945389*$t);
   $Y1 += 0.00000000020*cos(2.74404571866 + 26091.5406594487*$t);
   $Y1 += 0.00000000017*cos(1.01839175808 + 77410.75686045409*$t);
   $Y1 += 0.00000000018*cos(6.27803384892 + 115674.52048208468*$t);
   $Y1 += 0.00000000021*cos(3.48125971513 + 134991.71302241329*$t);
   $Y1 += 0.00000000017*cos(1.53092891578 + 104138.55728834229*$t);
   $Y1 += 0.00000000017*cos(0.95709459823 + 37410.32342239509*$t);
   $Y1 += 0.00000000019*cos(0.82226099482 + 522.3336006103*$t);
   $Y1 += 0.00000000019*cos(3.34754142930 + 52698.62751872569*$t);
   $Y1 += 0.00000000019*cos(2.35260406170 + 131395.35926696309*$t);
   $Y1 += 0.00000000016*cos(2.75961492200 + 12546.72575656651*$t);
   $Y1 += 0.00000000016*cos(4.65532135428 + 100909.28143881488*$t);
   $Y1 += 0.00000000016*cos(5.53845756469 + 25867.73431661889*$t);
   $Y1 += 0.00000000020*cos(1.18563577372 + 26094.77287109149*$t);
   $Y1 += 0.00000000018*cos(0.37484030344 + 47623.60896860609*$t);
   $Y1 += 0.00000000018*cos(6.16926211843 + 12432.2864678813*$t);
   $Y1 += 0.00000000020*cos(4.56182116076 + 11610.7957758577*$t);
   $Y1 += 0.00000000015*cos(3.94554558708 + 183724.94436059629*$t);
   $Y1 += 0.00000000017*cos(3.22546612448 + 51529.03926731709*$t);
   $Y1 += 0.00000000020*cos(3.34083321292 + 426.3543733925*$t);
   $Y1 += 0.00000000016*cos(3.33913638740 + 26308.5596014965*$t);
   $Y1 += 0.00000000016*cos(3.22684879906 + 128850.68663007069*$t);
   $Y1 += 0.00000000019*cos(2.99415679113 + 51543.2663613187*$t);
   $Y1 += 0.00000000019*cos(2.17719175975 + 155997.97170183409*$t);
   $Y1 += 0.00000000015*cos(6.05463333824 + 26164.4130303333*$t);
   $Y1 += 0.00000000018*cos(5.23520259945 + 25874.36022865269*$t);
   $Y1 += 0.00000000014*cos(3.61282271417 + 36300.94487030169*$t);
   $Y1 += 0.00000000019*cos(4.31362944396 + 45494.82524723229*$t);
   $Y1 += 0.00000000014*cos(2.00836472375 + 44295.9609472929*$t);
   $Y1 += 0.00000000018*cos(2.25014922269 + 119117.0954270501*$t);
   $Y1 += 0.00000000016*cos(2.19192842111 + 76667.76679992149*$t);
   $Y1 += 0.00000000014*cos(0.50039602119 + 107794.43132874569*$t);
   $Y1 += 0.00000000014*cos(3.93894618332 + 16065.82204399129*$t);
   $Y1 += 0.00000000014*cos(2.90301858576 + 44181.03402364069*$t);
   $Y1 += 0.00000000014*cos(3.19306372116 + 2111.8941308611*$t);
   $Y1 += 0.00000000015*cos(0.17179065008 + 52226.0467225361*$t);
   $Y1 += 0.00000000014*cos(3.19364287125 + 27999.3464422749*$t);
   $Y1 += 0.00000000019*cos(3.37439513975 + 104347.97504842229*$t);
   $Y1 += 0.00000000014*cos(1.85953479160 + 116918.00808376308*$t);
   $Y1 += 0.00000000015*cos(3.25430509963 + 365230.88779952223*$t);
   $Y1 += 0.00000000016*cos(4.96959083935 + 52712.85461272729*$t);
   $Y1 += 0.00000000017*cos(2.67262744895 + 97113.18079218028*$t);
   $Y1 += 0.00000000013*cos(5.63989609373 + 24176.9474758405*$t);
   $Y1 += 0.00000000017*cos(1.53028465700 + 20760.6708506749*$t);
   $Y1 += 0.00000000014*cos(5.20815151417 + 25954.0379094727*$t);
   $Y1 += 0.00000000014*cos(3.59706617063 + 26222.2560086427*$t);
   $Y1 += 0.00000000017*cos(5.41745483095 + 70269.42480018188*$t);
   $Y1 += 0.00000000013*cos(4.59923088636 + 178063.61231144409*$t);
   $Y1 += 0.00000000015*cos(5.79226796194 + 52099.7840293563*$t);
   $Y1 += 0.00000000013*cos(2.11885842073 + 323.74923414091*$t);
   $Y1 += 0.00000000013*cos(3.19236461837 + 39609.41076568209*$t);
   $Y1 += 0.00000000013*cos(5.09373061293 + 206.42936592071*$t);
   $Y1 += 0.00000000014*cos(0.25054779380 + 105940.92927906409*$t);
   $Y1 += 0.00000000013*cos(0.63576827663 + 102233.09252340188*$t);
   $Y1 += 0.00000000014*cos(2.18339498140 + 77204.08367704989*$t);
   $Y1 += 0.00000000014*cos(2.15110849472 + 13521.50762410789*$t);
   $Y1 += 0.00000000013*cos(3.12239387096 + 111590.53197538029*$t);
   $Y1 += 0.00000000012*cos(2.70622746349 + 76681.99389392308*$t);
   $Y1 += 0.00000000015*cos(5.99891720563 + 27670.10630734069*$t);
   $Y1 += 0.00000000013*cos(4.07441974716 + 58946.27306691049*$t);
   $Y1 += 0.00000000015*cos(3.57873341293 + 26300.95841952869*$t);
   $Y1 += 0.00000000013*cos(4.88571694871 + 14477.1073657165*$t);
   $Y1 += 0.00000000013*cos(5.78266274868 + 1692.40948698591*$t);
   $Y1 += 0.00000000013*cos(2.82188552462 + 140653.04507156549*$t);
   $Y1 += 0.00000000012*cos(6.05506602553 + 52072.9573264133*$t);
   $Y1 += 0.00000000012*cos(1.55236651979 + 54394.31956985169*$t);
   $Y1 += 0.00000000012*cos(5.88344203293 + 7238.9194090835*$t);
   $Y1 += 0.00000000012*cos(0.69684033021 + 25447.76203777669*$t);
   $Y1 += 0.00000000012*cos(4.63793697276 + 72936.4771338233*$t);
   $Y1 += 0.00000000015*cos(3.92235876661 + 2126.1212248627*$t);
   $Y1 += 0.00000000012*cos(4.09664422947 + 55504.18575691209*$t);
   $Y1 += 0.00000000012*cos(2.03928728821 + 220.6564599223*$t);
   $Y1 += 0.00000000012*cos(0.18923872919 + 111122.56698502589*$t);
   $Y1 += 0.00000000012*cos(5.93822601178 + 90695.99589260388*$t);
   $Y1 += 0.00000000012*cos(6.27891147429 + 104778.45457465628*$t);
   $Y1 += 0.00000000012*cos(0.41458906078 + 64607.60511606269*$t);
   $Y1 += 0.00000000012*cos(4.67455996800 + 110634.93223377169*$t);
   $Y1 += 0.00000000011*cos(2.00000784368 + 36109.9842391571*$t);
   $Y1 += 0.00000000013*cos(4.96093660073 + 49842.85370775989*$t);
   $Y1 += 0.00000000014*cos(5.74772554930 + 78417.73205269089*$t);
   $Y1 += 0.00000000011*cos(1.10539607022 + 26198.3532802771*$t);
   $Y1 += 0.00000000011*cos(1.41663630740 + 25977.94063783829*$t);
   $Y1 += 0.00000000012*cos(3.63665690282 + 31415.6230674405*$t);
   $Y1 += 0.00000000012*cos(3.78977700630 + 6770.9544187291*$t);
   $Y1 += 0.00000000011*cos(2.54959001341 + 51969.8645521947*$t);
   $Y1 += 0.00000000010*cos(2.10136723403 + 23868.9022199039*$t);
   $Y1 += 0.00000000012*cos(4.58411493514 + 130420.08976445469*$t);
   $Y1 += 0.00000000014*cos(1.09052029316 + 32769.3718124573*$t);
   $Y1 += 0.00000000013*cos(4.14764183344 + 33967.74847742969*$t);
   $Y1 += 0.00000000011*cos(2.20920470312 + 51645.87150057029*$t);
   $Y1 += 0.00000000010*cos(5.01452989749 + 28736.6017845307*$t);
   $Y1 += 0.00000000010*cos(2.60659223197 + 52705.25343075949*$t);
   $Y1 += 0.00000000014*cos(3.63437988054 + 77837.35505133009*$t);
   $Y1 += 0.00000000010*cos(2.91124294480 + 141762.42362365888*$t);
   $Y1 += 0.00000000012*cos(5.60563328751 + 224.5886131854*$t);
   $Y1 += 0.00000000010*cos(6.22792613909 + 170049.41410852008*$t);
   $Y1 += 0.00000000013*cos(2.82703801397 + 38813.6003938327*$t);
   $Y1 += 0.00000000010*cos(3.02574120742 + 27147.04125427989*$t);
   $Y1 += 0.00000000010*cos(2.23554494099 + 78903.8505285201*$t);
   $Y1 += 0.00000000013*cos(0.23254548151 + 6282.83203250789*$t);
   $Y1 += 0.00000000010*cos(5.31330188188 + 207644.08701988788*$t);
   $Y1 += 0.00000000012*cos(4.98783133373 + 26395.7045800273*$t);
   $Y1 += 0.00000000012*cos(3.81738635107 + 25780.58933808811*$t);
   $Y1 += 0.00000000010*cos(2.41181269432 + 12565.9078824993*$t);
   $Y1 += 0.00000000012*cos(2.61068546931 + 26725.1432310819*$t);
   $Y1 += 0.00000000012*cos(6.19453221549 + 25451.1506870335*$t);
   $Y1 += 0.00000000009*cos(4.05684885963 + 51219.76094466129*$t);
   $Y1 += 0.00000000009*cos(6.12208983529 + 155572.06292532009*$t);
   $Y1 += 0.00000000009*cos(0.41261244956 + 78731.91823256049*$t);
   $Y1 += 0.00000000009*cos(2.60381655038 + 14764.9952257863*$t);
   $Y1 += 0.00000000011*cos(1.67989630572 + 65697.80154222329*$t);
   $Y1 += 0.00000000012*cos(0.71134314907 + 25446.73339731871*$t);
   $Y1 += 0.00000000012*cos(1.81068922855 + 26729.56052079669*$t);
   $Y1 += 0.00000000012*cos(3.05761613841 + 12725.20961729149*$t);
   $Y1 += 0.00000000010*cos(2.47438981577 + 65831.91059180829*$t);
   $Y1 += 0.00000000009*cos(0.40929169320 + 16703.3059509825*$t);
   $Y1 += 0.00000000009*cos(4.90698904861 + 51708.08511027749*$t);
   $Y1 += 0.00000000009*cos(3.85732708279 + 207594.09039798369*$t);
   $Y1 += 0.00000000009*cos(4.96560671408 + 156101.06447605268*$t);
   $Y1 += 0.00000000012*cos(4.51597349846 + 176954.23375935068*$t);
   $Y1 += 0.00000000009*cos(0.27941818584 + 68242.11596210669*$t);
   $Y1 += 0.00000000009*cos(3.76248228932 + 1910.9556657337*$t);
   $Y1 += 0.00000000009*cos(2.58539985032 + 50586.48957038109*$t);
   $Y1 += 0.00000000009*cos(0.03828568933 + 52609.7618385087*$t);
   $Y1 += 0.00000000008*cos(4.59422184502 + 29530.2342690561*$t);
   $Y1 += 0.00000000009*cos(4.40872070931 + 205.9417309537*$t);
   $Y1 += 0.00000000009*cos(1.91710703544 + 55516.6625273317*$t);
   $Y1 += 0.00000000011*cos(0.78490575811 + 52022.2712901471*$t);
   $Y1 += 0.00000000011*cos(1.79573927984 + 157057.35363202328*$t);
   $Y1 += 0.00000000008*cos(6.02666588108 + 39629.56816154889*$t);
   $Y1 += 0.00000000010*cos(0.38285944104 + 52382.23564906909*$t);
   $Y1 += 0.00000000008*cos(4.34710667767 + 52182.67865953191*$t);
   $Y1 += 0.00000000010*cos(4.32001939934 + 40565.4981422577*$t);
   $Y1 += 0.00000000008*cos(2.87593439674 + 103.3365917021*$t);
   $Y1 += 0.00000000008*cos(5.31493047379 + 157483.26240853728*$t);
   $Y1 += 0.00000000008*cos(4.79233550623 + 27177.8557502513*$t);
   $Y1 += 0.00000000008*cos(4.01288217857 + 24998.4381678641*$t);
   $Y1 += 0.00000000009*cos(4.80787399572 + 19.42594341629*$t);
   $Y1 += 0.00000000010*cos(2.27022895876 + 536.5606946119*$t);
   $Y1 += 0.00000000010*cos(4.80007299648 + 260878.78759825832*$t);
   $Y1 += 0.00000000008*cos(4.56235937282 + 48847.9144443517*$t);
   $Y1 += 0.00000000008*cos(4.66549233094 + 860.55374623631*$t);
   $Y1 += 0.00000000008*cos(2.76375833425 + 26720.93069837089*$t);
   $Y1 += 0.00000000009*cos(5.58566134530 + 25654.43522118089*$t);
   $Y1 += 0.00000000009*cos(0.00405995806 + 63498.22656396929*$t);
   $Y1 += 0.00000000008*cos(1.38258172759 + 52595.53474450709*$t);
   $Y1 += 0.00000000008*cos(0.52005477263 + 130446.87307235529*$t);
   $Y1 += 0.00000000007*cos(1.78779062151 + 50697.1835265675*$t);
   $Y1 += 0.00000000008*cos(4.09271417559 + 1224.06165826211*$t);
   $Y1 += 0.00000000009*cos(5.72498036679 + 19316.9487228451*$t);
   $Y1 += 0.00000000007*cos(0.38836950932 + 13541.17738500769*$t);
   $Y1 += 0.00000000009*cos(1.66361806448 + 130443.64086071249*$t);
   $Y1 += 0.00000000007*cos(0.73505185505 + 209812.84750217047*$t);
   $Y1 += 0.00000000007*cos(2.21903348151 + 9384.59719059169*$t);
   $Y1 += 0.00000000007*cos(2.18741011968 + 46848.0863572821*$t);
   $Y1 += 0.00000000007*cos(2.07119546850 + 172402.28026229188*$t);
   $Y1 += 0.00000000007*cos(5.46643566327 + 846.3266522347*$t);
   $Y1 += 0.00000000007*cos(1.48055041129 + 27044.4361150283*$t);
   $Y1 += 0.00000000007*cos(0.42256065489 + 150244.58681693728*$t);
   $Y1 += 0.00000000009*cos(2.36014528596 + 26735.1577923725*$t);
   $Y1 += 0.00000000007*cos(3.18642075607 + 52325.61329776649*$t);
   $Y1 += 0.00000000007*cos(6.17839162640 + 45892.9742506405*$t);
   $Y1 += 0.00000000008*cos(1.20719608552 + 52169.42154173189*$t);
   $Y1 += 0.00000000008*cos(0.42457308525 + 83924.79765639128*$t);
   $Y1 += 0.00000000007*cos(3.81342742455 + 80597.14963507809*$t);
   $Y1 += 0.00000000006*cos(2.21280392322 + 52279.14287485049*$t);
   $Y1 += 0.00000000006*cos(1.57105717926 + 51013.57539622409*$t);
   $Y1 += 0.00000000008*cos(2.16921693506 + 129387.49114216608*$t);
   $Y1 += 0.00000000009*cos(2.14041581315 + 955.8435590921*$t);
   $Y1 += 0.00000000008*cos(6.21586114247 + 77727.14873011068*$t);
   $Y1 += 0.00000000006*cos(5.71782638960 + 79330.44871939608*$t);
   $Y1 += 0.00000000007*cos(3.99442191803 + 78378.3925308913*$t);
   $Y1 += 0.00000000007*cos(0.83146896151 + 51219.96272405629*$t);
   $Y1 += 0.00000000006*cos(2.63351784574 + 129373.26404816448*$t);
   $Y1 += 0.00000000007*cos(0.06013753430 + 26162.928557625*$t);
   $Y1 += 0.00000000006*cos(5.10934915127 + 103241.99019671988*$t);
   $Y1 += 0.00000000006*cos(5.27952242053 + 3340.36860921629*$t);
   $Y1 += 0.00000000006*cos(2.10211584098 + 35472.9879671329*$t);
   $Y1 += 0.00000000006*cos(2.93834812676 + 130653.05862079248*$t);
   $Y1 += 0.00000000006*cos(4.63873538075 + 110.45013870291*$t);
   $Y1 += 0.00000000007*cos(6.14935848430 + 77631.16950289288*$t);
   $Y1 += 0.00000000006*cos(3.41608343835 + 53131.16220727349*$t);
   $Y1 += 0.00000000006*cos(2.45250087046 + 26013.3653604904*$t);
   $Y1 += 0.00000000006*cos(4.96304042103 + 25508.4593720589*$t);
   $Y1 += 0.00000000006*cos(3.84217726377 + 26667.8345460565*$t);
   $Y1 += 0.00000000006*cos(2.84953525287 + 70268.93716521488*$t);
   $Y1 += 0.00000000006*cos(2.18073760821 + 103918.14464590348*$t);
   $Y1 += 0.00000000008*cos(5.64484638063 + 157587.04459711789*$t);
   $Y1 += 0.00000000006*cos(0.28527129594 + 6885.39370741431*$t);
   $Y1 += 0.00000000007*cos(3.20891519423 + 214364.80099922928*$t);
   $Y1 += 0.00000000007*cos(2.79508447590 + 25754.2910182883*$t);
   $Y1 += 0.00000000007*cos(6.01013320890 + 26422.0028998271*$t);
   $Y1 += 0.00000000006*cos(4.09475623089 + 60055.65161900389*$t);
   $Y1 += 0.00000000006*cos(4.43814061291 + 78149.51395352086*$t);
   $Y1 += 0.00000000005*cos(5.60856067263 + 50064.3997872543*$t);
   $Y1 += 0.00000000006*cos(1.06133168572 + 128221.00242116769*$t);
   $Y1 += 0.00000000006*cos(3.38536280287 + 75615.49841673308*$t);
   $Y1 += 0.00000000005*cos(5.98443573479 + 137678.43511695448*$t);
   $Y1 += 0.00000000007*cos(1.25296072469 + 56727.51596272369*$t);
   $Y1 += 0.00000000007*cos(1.14884253624 + 8194.0315157251*$t);
   $Y1 += 0.00000000005*cos(4.48886015660 + 130226.46042991648*$t);
   $Y1 += 0.00000000006*cos(4.17115612398 + 80382.71710258449*$t);
   $Y1 += 0.00000000006*cos(0.69855070126 + 51653.47268253809*$t);
   $Y1 += 0.00000000006*cos(4.78072360502 + 123669.04892410889*$t);
   $Y1 += 0.00000000006*cos(5.42312230542 + 72602.13355808688*$t);
   $Y1 += 0.00000000006*cos(6.00561848302 + 50056.79860528649*$t);
   $Y1 += 0.00000000005*cos(5.89027963231 + 102769.89703549728*$t);
   $Y1 += 0.00000000006*cos(5.34345526598 + 74820.89066227368*$t);
   $Y1 += 0.00000000006*cos(0.24757397975 + 161079.61616398748*$t);
   $Y1 += 0.00000000006*cos(1.60819865465 + 103932.37173990508*$t);
   $Y1 += 0.00000000006*cos(1.04008531783 + 25234.9505773057*$t);
   $Y1 += 0.00000000007*cos(5.82651027335 + 154938.58977164488*$t);
   $Y1 += 0.00000000005*cos(5.52069502978 + 419.2408263917*$t);
   $Y1 += 0.00000000005*cos(2.31699479833 + 19406.9221056581*$t);
   $Y1 += 0.00000000006*cos(1.96387669233 + 1265.81129610991*$t);
   $Y1 += 0.00000000007*cos(5.70425979338 + 64901.5035354069*$t);
   $Y1 += 0.00000000005*cos(4.02516634304 + 73711.51211018028*$t);
   $Y1 += 0.00000000005*cos(5.11989980868 + 78213.95662030189*$t);
   $Y1 += 0.00000000006*cos(5.18848518823 + 182085.87484340828*$t);
   $Y1 += 0.00000000005*cos(5.44202376441 + 102755.66994149568*$t);
   $Y1 += 0.00000000005*cos(4.80390003741 + 40565.01050729069*$t);
   $Y1 += 0.00000000005*cos(6.13170260920 + 102.84895673509*$t);
   $Y1 += 0.00000000005*cos(4.30112257620 + 51748.96427478889*$t);
   $Y1 += 0.00000000005*cos(0.85038224536 + 53399.86794141051*$t);
   $Y1 += 0.00000000005*cos(2.42619611574 + 27780.31262856009*$t);
   $Y1 += 0.00000000005*cos(1.56640230350 + 80482.22271142589*$t);
   $Y1 += 0.00000000005*cos(2.47762753907 + 25938.5837619231*$t);
   $Y1 += 0.00000000005*cos(5.22476504933 + 26402.33313892731*$t);
   $Y1 += 0.00000000005*cos(3.58045263546 + 25773.96077918809*$t);
   $Y1 += 0.00000000005*cos(0.01470040312 + 77616.94240889128*$t);
   $Y1 += 0.00000000005*cos(0.50724383605 + 34282.4222922663*$t);
   $Y1 += 0.00000000006*cos(4.89504933760 + 50050.17269325269*$t);
   $Y1 += 0.00000000005*cos(1.99588630926 + 17893.8716258491*$t);
   $Y1 += 0.00000000004*cos(0.97602564752 + 130432.64597835368*$t);
   $Y1 += 0.00000000004*cos(0.40741730958 + 51852.54468397449*$t);
   $Y1 += 0.00000000004*cos(2.23467299420 + 32370.73517408209*$t);
   $Y1 += 0.00000000005*cos(1.35451697835 + 204151.51545301828*$t);
   $Y1 += 0.00000000006*cos(1.74602932944 + 75930.75684933408*$t);
   $Y1 += 0.00000000004*cos(5.26812670096 + 25668.17468021549*$t);
   $Y1 += 0.00000000005*cos(1.36761975943 + 6043.9847638919*$t);
   $Y1 += 0.00000000005*cos(2.74357702126 + 39450.5966658569*$t);
   $Y1 += 0.00000000005*cos(2.28369543224 + 24491.96051677309*$t);
   $Y1 += 0.00000000004*cos(1.23165526523 + 51955.63745819309*$t);
   $Y1 += 0.00000000004*cos(4.09092913231 + 103498.66000202828*$t);
   $Y1 += 0.00000000005*cos(0.06895267430 + 52396.4627430707*$t);
   $Y1 += 0.00000000004*cos(2.56354623790 + 104505.63519426509*$t);
   $Y1 += 0.00000000005*cos(5.58769879303 + 23754.46293121869*$t);
   $Y1 += 0.00000000005*cos(5.71030733431 + 123201.08393375449*$t);
   $Y1 += 0.00000000004*cos(6.20993923204 + 54087.2495838491*$t);
   $Y1 += 0.00000000004*cos(3.63799815383 + 637.24008950771*$t);
   $Y1 += 0.00000000004*cos(0.59460014711 + 28791.7631137333*$t);
   $Y1 += 0.00000000004*cos(1.92743223051 + 23384.5308043821*$t);
   $Y1 += 0.00000000005*cos(4.52669072840 + 53029.2464823839*$t);
   $Y1 += 0.00000000005*cos(0.25501672186 + 18848.98373249069*$t);
   $Y1 += 0.00000000004*cos(1.81268637965 + 26824.02347258949*$t);
   $Y1 += 0.00000000004*cos(0.70934599797 + 25352.2704455259*$t);
   $Y1 += 0.00000000004*cos(3.53975344268 + 133882.3344703199*$t);
   $Y1 += 0.00000000004*cos(0.87353062113 + 132658.51662954128*$t);
   $Y1 += 0.00000000004*cos(3.22384434044 + 53772.23654291649*$t);
   $Y1 += 0.00000000004*cos(2.01166034864 + 183571.16555011149*$t);
   $Y1 += 0.00000000004*cos(5.89521734601 + 167850.32676523308*$t);
   $Y1 += 0.00000000005*cos(5.28708364201 + 145204.99856862429*$t);
   $Y1 += 0.00000000004*cos(1.34862695530 + 27171.2271913513*$t);
   $Y1 += 0.00000000004*cos(1.17340542232 + 25005.0667267641*$t);
   $Y1 += 0.00000000004*cos(3.00220929293 + 50483.8844311295*$t);
   $Y1 += 0.00000000004*cos(0.14217969958 + 78786.53066029989*$t);
   $Y1 += 0.00000000004*cos(3.41518083973 + 949.4194264533*$t);
   $Y1 += 0.00000000005*cos(0.77055922503 + 45892.48661567349*$t);
   $Y1 += 0.00000000004*cos(3.51962830585 + 1795.5022612045*$t);
   $Y1 += 0.00000000005*cos(1.84426394169 + 26709.8907598969*$t);
   $Y1 += 0.00000000005*cos(0.67776843593 + 25466.4031582185*$t);
   $Y1 += 0.00000000004*cos(4.27093115110 + 52065.84377941249*$t);
   $Y1 += 0.00000000004*cos(0.10877025827 + 2222.1004520805*$t);
   $Y1 += 0.00000000004*cos(1.11438053331 + 78270.58180110609*$t);
   $Y1 += 0.00000000004*cos(5.94116454026 + 25653.94758621389*$t);
   $Y1 += 0.00000000004*cos(5.36329329492 + 49842.36607279289*$t);
   $Y1 += 0.00000000004*cos(4.90307551419 + 143005.91122533729*$t);
   $Y1 += 0.00000000004*cos(1.70672949321 + 78800.75775430149*$t);
   $Y1 += 0.00000000004*cos(0.14272238686 + 65697.31390725628*$t);
   $Y1 += 0.00000000003*cos(1.76268636575 + 52195.23222656469*$t);
   $Y1 += 0.00000000004*cos(3.45396433025 + 130459.42928625426*$t);
   $Y1 += 0.00000000004*cos(5.54668248593 + 24491.47288180609*$t);
   $Y1 += 0.00000000004*cos(2.57630900199 + 3178.38960805111*$t);
   $Y1 += 0.00000000004*cos(3.69827148826 + 220.16882495529*$t);
   $Y1 += 0.00000000004*cos(3.39420749098 + 52250.8316991992*$t);
   $Y1 += 0.00000000003*cos(2.73132197327 + 78160.86046798748*$t);
   $Y1 += 0.00000000004*cos(2.89190828519 + 87367.86023632369*$t);
   $Y1 += 0.00000000004*cos(0.15780189015 + 130435.87818999648*$t);
   $Y1 += 0.00000000004*cos(0.14307301866 + 53234.94439585409*$t);
   $Y1 += 0.00000000004*cos(0.06711300408 + 26575.7817103119*$t);
   $Y1 += 0.00000000004*cos(2.45491937354 + 25600.5122078035*$t);
   $Y1 += 0.00000000003*cos(5.37884708213 + 1486.2239385487*$t);
   $Y1 += 0.00000000005*cos(3.35368845011 + 86144.04239554508*$t);
   $Y1 += 0.00000000004*cos(1.34102185895 + 32132.3755404331*$t);
   $Y1 += 0.00000000004*cos(1.04693916592 + 66653.40128383189*$t);
   $Y1 += 0.00000000003*cos(0.35934615572 + 52310.1591502169*$t);
   $Y1 += 0.00000000003*cos(2.24307597389 + 45290.90021070109*$t);
   $Y1 += 0.00000000003*cos(5.05641378963 + 70383.86408886709*$t);
   $Y1 += 0.00000000003*cos(2.84685809679 + 52252.31617190749*$t);
   $Y1 += 0.00000000005*cos(0.41148055834 + 52808.83383994509*$t);
   $Y1 += 0.00000000003*cos(0.89098898576 + 1588.82907780029*$t);
   $Y1 += 0.00000000004*cos(4.17927437554 + 58857.2749540315*$t);
   $Y1 += 0.00000000004*cos(3.20092050693 + 51951.70530492999*$t);
   $Y1 += 0.00000000003*cos(2.40144059169 + 50264.8506174147*$t);
   $Y1 += 0.00000000004*cos(1.17285198222 + 20043.9183776823*$t);
   $Y1 += 0.00000000004*cos(3.78297088323 + 128320.99566497609*$t);
   $Y1 += 0.00000000003*cos(3.96547374476 + 25986.18444079209*$t);
   $Y1 += 0.00000000003*cos(4.83974394004 + 26190.1094773233*$t);
   $Y1 += 0.00000000004*cos(1.45951607601 + 136722.83537534589*$t);
   $Y1 += 0.00000000003*cos(2.84255023310 + 65717.47130312308*$t);
   $Y1 += 0.00000000003*cos(2.62501893240 + 181026.49291321907*$t);
   $Y1 += 0.00000000003*cos(4.29835303229 + 51535.66517935089*$t);
   $Y1 += 0.00000000004*cos(0.32726328089 + 129799.86223904048*$t);
   $Y1 += 0.00000000003*cos(4.92235874244 + 26073.91986505609*$t);
   $Y1 += 0.00000000003*cos(3.88285894236 + 26102.3740530593*$t);
   $Y1 += 0.00000000003*cos(4.97897799737 + 52168.44891866409*$t);
   $Y1 += 0.00000000004*cos(2.55090678746 + 52155.8927047651*$t);
   $Y1 += 0.00000000004*cos(1.29677266690 + 37698.6989174319*$t);
   $Y1 += 0.00000000003*cos(0.56868525863 + 51109.06698847489*$t);
   $Y1 += 0.00000000003*cos(1.87341446779 + 26247.4486938499*$t);
   $Y1 += 0.00000000003*cos(0.64861790983 + 25928.8452242655*$t);
   $Y1 += 0.00000000004*cos(0.30979285060 + 468.20880783791*$t);
   $Y1 += 0.00000000004*cos(3.27872430484 + 132028.83242063828*$t);
   $Y1 += 0.00000000004*cos(5.61336389985 + 25661.06113321469*$t);
   $Y1 += 0.00000000003*cos(0.47396259409 + 2703.3723371921*$t);
   $Y1 += 0.00000000004*cos(5.86092912592 + 166740.94821313968*$t);
   $Y1 += 0.00000000004*cos(3.23572058854 + 59414.23805726489*$t);
   $Y1 += 0.00000000003*cos(4.82502734269 + 61560.8911087071*$t);
   $Y1 += 0.00000000003*cos(0.00693087411 + 79316.22162539448*$t);
   $Y1 += 0.00000000003*cos(3.05784222447 + 130866.35771623049*$t);
   $Y1 += 0.00000000003*cos(0.12277419735 + 79219.06534884768*$t);
   $Y1 += 0.00000000003*cos(2.46452817521 + 24506.18761077469*$t);
   $Y1 += 0.00000000003*cos(0.66136657530 + 51868.4924796623*$t);
   $Y1 += 0.00000000003*cos(2.85883702115 + 19202.50943415989*$t);
   $Y1 += 0.00000000003*cos(6.14048429792 + 40852.8983673605*$t);
   $Y1 += 0.00000000003*cos(2.81485517413 + 116783.89903417809*$t);
   $Y1 += 0.00000000003*cos(5.64866640193 + 105460.74730090669*$t);
   $Y1 += 0.00000000003*cos(0.28115071068 + 48835.43767393209*$t);
   $Y1 += 0.00000000003*cos(4.12067030266 + 114.68310616872*$t);
   $Y1 += 0.00000000003*cos(4.90383671624 + 76785.08666814168*$t);
   $Y1 += 0.00000000003*cos(0.85813944126 + 81592.08889848628*$t);
   $Y1 += 0.00000000003*cos(2.81772139589 + 95247.46175469569*$t);
   $Y1 += 0.00000000003*cos(4.69086415103 + 52171.68113030689*$t);
   $Y1 += 0.00000000004*cos(5.11822399207 + 78110.17443172129*$t);
   $Y1 += 0.00000000003*cos(2.45643813863 + 68050.18006102808*$t);
   $Y1 += 0.00000000003*cos(5.59679660207 + 38653.81102407349*$t);
   $Y1 += 0.00000000003*cos(1.32662236669 + 2219.0009216703*$t);
   $Y1 += 0.00000000003*cos(5.92341361550 + 52179.4438010229*$t);
   $Y1 += 0.00000000003*cos(4.15087523006 + 52286.25642185129*$t);
   $Y1 += 0.00000000003*cos(0.17348863040 + 76674.39271195528*$t);
   $Y1 += 0.00000000003*cos(0.93059863207 + 126997.18458038908*$t);
   $Y1 += 0.00000000003*cos(4.49117697679 + 77101.47853779829*$t);
   $Y1 += 0.00000000003*cos(4.40672346738 + 77741.37582411229*$t);
   $Y1 += 0.00000000003*cos(4.09517524067 + 8989.22388794889*$t);
   $Y1 += 0.00000000003*cos(5.27976660688 + 77733.77464214448*$t);
   $Y1 += 0.00000000003*cos(1.52949550682 + 286966.69073983253*$t);
   $Y1 += 0.00000000003*cos(0.97589529959 + 74.53778108379*$t);
   $Y1 += 0.00000000003*cos(4.21847009410 + 632.5399218297*$t);
   $Y1 += 0.00000000004*cos(4.47958960947 + 28287.2343023447*$t);
   $Y1 += 0.00000000004*cos(4.32562807533 + 23889.0596157707*$t);
   $Y1 += 0.00000000003*cos(5.21982122522 + 75930.26921436709*$t);
   $Y1 += 0.00000000002*cos(1.69313010887 + 182188.96761762688*$t);
   $Y1 += 0.00000000003*cos(4.67718054767 + 27250.62166346549*$t);
   $Y1 += 0.00000000003*cos(4.13971825138 + 51315.74017187909*$t);
   $Y1 += 0.00000000003*cos(4.52130139518 + 52182.67601266569*$t);
   $Y1 += 0.00000000002*cos(4.85799167915 + 121335.85253123689*$t);
   $Y1 += 0.00000000002*cos(3.88643180579 + 110012.70079796549*$t);
   $Y1 += 0.00000000002*cos(1.22067192730 + 2008.8013566425*$t);
   $Y1 += 0.00000000002*cos(3.76859743855 + 24925.6722546499*$t);
   $Y1 += 0.00000000003*cos(0.31070573063 + 39629.08052658189*$t);
   $Y1 += 0.00000000003*cos(3.24788020549 + 3340.8562441833*$t);
   $Y1 += 0.00000000002*cos(0.35390973897 + 23439.20449861769*$t);
   $Y1 += 0.00000000002*cos(1.50089471176 + 29415.79498037089*$t);
   $Y1 += 0.00000000003*cos(0.83102506625 + 25131.85780308709*$t);
   $Y1 += 0.00000000003*cos(6.25945305227 + 51123.29408247649*$t);
   $Y1 += 0.00000000003*cos(0.28921662553 + 90695.50825763689*$t);
   $Y1 += 0.00000000002*cos(0.13680232887 + 24505.69997580769*$t);
   $Y1 += 0.00000000003*cos(2.81292041302 + 26011.8808877821*$t);
   $Y1 += 0.00000000002*cos(3.34607115822 + 51329.9672658807*$t);
   $Y1 += 0.00000000003*cos(4.94974770833 + 81604.56566890588*$t);
   $Y1 += 0.00000000003*cos(5.66982181767 + 52813.0463726561*$t);
   $Y1 += 0.00000000003*cos(0.02173421963 + 391318.79094109643*$t);
   $Y1 += 0.00000000002*cos(2.96964107128 + 196137.31725009429*$t);
   $Y1 += 0.00000000003*cos(1.28173018247 + 156507.9929060289*$t);
   $Y1 += 0.00000000003*cos(0.93767773997 + 27727.2164762457*$t);
   $Y1 += 0.00000000003*cos(1.58435463765 + 24449.0774418697*$t);
   $Y1 += 0.00000000002*cos(5.77422284047 + 78057.76769376888*$t);
   $Y1 += 0.00000000003*cos(3.17950494103 + 78313.94986411028*$t);
   $Y1 += 0.00000000003*cos(1.26777637523 + 203042.13690092487*$t);
   $Y1 += 0.00000000003*cos(2.84442647149 + 34082.6754010819*$t);
   $Y1 += 0.00000000002*cos(5.82921421247 + 38634.6288981407*$t);
   $Y1 += 0.00000000003*cos(4.25851095793 + 78257.32468330608*$t);
   $Y1 += 0.00000000002*cos(4.89476820098 + 49527.10764019189*$t);
   $Y1 += 0.00000000003*cos(1.95628126873 + 52041.9410510469*$t);
   $Y1 += 0.00000000002*cos(3.46648506780 + 104819.8213741347*$t);
   $Y1 += 0.00000000002*cos(5.05488554468 + 62197.88738073129*$t);
   $Y1 += 0.00000000003*cos(5.38708769121 + 103291.98681862408*$t);
   $Y1 += 0.00000000002*cos(1.95981868389 + 51962.26337022689*$t);
   $Y1 += 0.00000000002*cos(3.49107424857 + 35077.61466449009*$t);
   $Y1 += 0.00000000003*cos(1.93353771137 + 96357.32794175608*$t);
   $Y1 += 0.00000000002*cos(0.91930935416 + 25021.16384690069*$t);
   $Y1 += 0.00000000002*cos(4.74992302909 + 66652.91364886488*$t);
   $Y1 += 0.00000000002*cos(2.31652509447 + 199599.0743209924*$t);
   $Y1 += 0.00000000002*cos(4.52662503285 + 9103.6631766341*$t);
   $Y1 += 0.00000000002*cos(5.81156600010 + 74923.34081550628*$t);
   $Y1 += 0.00000000002*cos(5.31799111712 + 17098.67925362531*$t);
   $Y1 += 0.00000000002*cos(3.54866382542 + 78470.13879064328*$t);
   $Y1 += 0.00000000002*cos(0.96038903828 + 101703.40155830729*$t);
   $Y1 += 0.00000000002*cos(0.63534070468 + 77307.66408623548*$t);
   $Y1 += 0.00000000002*cos(5.30685200110 + 104991.75367009429*$t);
   $Y1 += 0.00000000002*cos(4.00030437238 + 86143.55476057807*$t);
   $Y1 += 0.00000000002*cos(2.46838836063 + 26312.4917547596*$t);
   $Y1 += 0.00000000003*cos(6.17763429163 + 69159.55861312148*$t);
   $Y1 += 0.00000000002*cos(4.36618723507 + 27676.73221937449*$t);
   $Y1 += 0.00000000002*cos(5.52193246370 + 91919.81373338249*$t);
   $Y1 += 0.00000000002*cos(2.07266268481 + 225687.46509753366*$t);
   $Y1 += 0.00000000002*cos(1.38235585269 + 60369.83779887349*$t);
   $Y1 += 0.00000000002*cos(2.45878629397 + 26184.1261862755*$t);
   $Y1 += 0.00000000002*cos(0.06324608365 + 25992.1677318399*$t);
   $Y1 += 0.00000000002*cos(0.06817332477 + 62388.84801187589*$t);
   $Y1 += 0.00000000002*cos(3.72202154841 + 7880.3329708225*$t);
   $Y1 += 0.00000000003*cos(3.66137462974 + 51534.63653889291*$t);
   $Y1 += 0.00000000002*cos(2.59383779834 + 24608.79275002629*$t);
   $Y1 += 0.00000000002*cos(4.79894731382 + 183145.25677359749*$t);
   $Y1 += 0.00000000002*cos(1.81015048354 + 177288.08970012008*$t);
   $Y1 += 0.00000000002*cos(1.16884626410 + 26397.42528171349*$t);
   $Y1 += 0.00000000002*cos(1.35318611352 + 25778.86863640189*$t);
   $Y1 += 0.00000000002*cos(3.47952755604 + 42791.2090925567*$t);
   $Y1 += 0.00000000002*cos(1.89575854549 + 38813.11275886569*$t);
   $Y1 += 0.00000000002*cos(3.54021512986 + 51539.0538286077*$t);
   $Y1 += 0.00000000002*cos(2.45627954068 + 24952.4700871645*$t);
   $Y1 += 0.00000000002*cos(0.06575283694 + 27223.8238309509*$t);
   $Y1 += 0.00000000002*cos(2.99909862001 + 103719.07264446709*$t);
   $Y1 += 0.00000000002*cos(1.64606671842 + 25984.3647704771*$t);
   $Y1 += 0.00000000002*cos(0.87596565920 + 26191.9291476383*$t);
   $Y1 += 0.00000000002*cos(5.44690335938 + 25927.7395896729*$t);
   $Y1 += 0.00000000002*cos(3.35831432542 + 26248.5543284425*$t);
   $Y1 += 0.00000000002*cos(3.68753780033 + 113341.32408921266*$t);
   $Y1 += 0.00000000002*cos(1.57407073796 + 53265.7588918255*$t);
   $Y1 += 0.00000000002*cos(6.17867775538 + 51842.1941598625*$t);
   $Y1 += 0.00000000002*cos(0.90904857785 + 16066.3096789583*$t);
   $Y1 += 0.00000000002*cos(0.52784265993 + 29550.3916649229*$t);
   $Y1 += 0.00000000002*cos(1.99418971769 + 22625.9022531925*$t);
   $Y1 += 0.00000000002*cos(6.15702102064 + 27037.32256802749*$t);
   $Y1 += 0.00000000002*cos(2.64819666416 + 25138.9713500879*$t);
   $Y1 += 0.00000000002*cos(2.50748734527 + 157482.77477357029*$t);
   $Y1 += 0.00000000002*cos(4.84158872259 + 52817.46366237089*$t);
   $Y1 += 0.00000000002*cos(2.22725796457 + 104202.29318664569*$t);
   $Y1 += 0.00000000002*cos(6.26671716778 + 25550.85481199529*$t);
   $Y1 += 0.00000000002*cos(0.50528140230 + 52388.86156110289*$t);
   $Y1 += 0.00000000002*cos(2.07951392438 + 233731.99016146208*$t);
   $Y1 += 0.00000000002*cos(1.72258811309 + 54824.5049261049*$t);
   $Y1 += 0.00000000002*cos(3.30916137339 + 13655.6166736929*$t);
   $Y1 += 0.00000000002*cos(5.77355700704 + 78793.15657233368*$t);
   $Y1 += 0.00000000002*cos(0.24904980575 + 168959.21768235945*$t);
   $Y1 += 0.00000000002*cos(4.68270285738 + 205260.89400511167*$t);
   $Y1 += 0.00000000002*cos(4.78459138847 + 128106.07549751546*$t);
   $Y1 += 0.00000000002*cos(0.55814555148 + 50910.4826220055*$t);
   $Y1 += 0.00000000002*cos(1.51894844260 + 51859.65823097529*$t);
   $Y1 += 0.00000000002*cos(6.23120336216 + 25863.8021633558*$t);
   $Y1 += 0.00000000002*cos(2.83592703830 + 53758.00944891489*$t);
   $Y1 += 0.00000000002*cos(4.03688605095 + 52492.4419702885*$t);
   $Y1 += 0.00000000002*cos(2.18084353651 + 735.6326960483*$t);
   $Y1 += 0.00000000002*cos(5.32243619010 + 736.1203310153*$t);
   $Y1 += 0.00000000002*cos(3.11385751174 + 1066.2516597065*$t);
   $Y1 += 0.00000000002*cos(0.08781202930 + 215474.17955132265*$t);
   $Y1 += 0.00000000002*cos(3.93802376645 + 26077.1012587938*$t);
   $Y1 += 0.00000000002*cos(4.86719391834 + 26099.1926593216*$t);
   $Y1 += 0.00000000002*cos(2.73032719646 + 96356.84030678908*$t);
   $Y1 += 0.00000000002*cos(4.39557118339 + 50579.37602338029*$t);
   $Y1 += 0.00000000002*cos(3.37995643779 + 176332.48995851149*$t);
   $Y1 += 0.00000000002*cos(3.60695289838 + 78896.73698151928*$t);
   $Y1 += 0.00000000002*cos(4.97912201809 + 26411.6523757151*$t);
   $Y1 += 0.00000000002*cos(3.92261173243 + 87253.42094763846*$t);
   $Y1 += 0.00000000002*cos(2.85499495780 + 181659.96606689428*$t);
   $Y1 += 0.00000000002*cos(6.13761457132 + 23976.00901071309*$t);
   $Y1 += 0.00000000002*cos(4.27004909023 + 102232.60488843489*$t);
   $Y1 += 0.00000000002*cos(1.87598894439 + 52483.6077216015*$t);
   $Y1 += 0.00000000002*cos(4.99129516853 + 199599.56195595946*$t);
   $Y1 += 0.00000000002*cos(5.06914347559 + 25933.8805136059*$t);
   $Y1 += 0.00000000002*cos(3.55063804653 + 23439.6921335847*$t);
   $Y1 += 0.00000000001*cos(3.06891862205 + 27351.3043216359*$t);
   $Y1 += 0.00000000001*cos(5.73629906275 + 24824.9895964795*$t);
   $Y1 += 0.00000000001*cos(3.09801335678 + 78697.66498008289*$t);
   $Y1 += 0.00000000002*cos(3.94465304931 + 60370.3254338405*$t);
   $Y1 += 0.00000000001*cos(1.67825434512 + 51596.3625136331*$t);
   $Y1 += 0.00000000002*cos(2.96585642839 + 103815.05187168489*$t);
   $Y1 += 0.00000000002*cos(3.05786923280 + 33326.33491569069*$t);
   $Y1 += 0.00000000002*cos(5.41433241516 + 51226.87449166209*$t);
   $Y1 += 0.00000000001*cos(4.05308794514 + 77307.86586563049*$t);
   $Y1 += 0.00000000002*cos(1.32820705945 + 50952.23225985329*$t);
   $Y1 += 0.00000000002*cos(0.35546648330 + 65716.98366815608*$t);
   $Y1 += 0.00000000001*cos(1.46542460653 + 231348.79714668586*$t);
   $Y1 += 0.00000000001*cos(2.79951095321 + 163765.85062356168*$t);
   $Y1 += 0.00000000002*cos(1.10119151677 + 34281.9346572993*$t);
   $Y1 += 0.00000000001*cos(1.58974685995 + 25234.46294233869*$t);
   $Y1 += 0.00000000001*cos(4.42744475216 + 147423.75567281108*$t);
   $Y1 += 0.00000000002*cos(3.30510234043 + 26514.25751496669*$t);
   $Y1 += 0.00000000001*cos(2.03602296374 + 3442.3311274819*$t);
   $Y1 += 0.00000000001*cos(4.23846527178 + 102762.29585352947*$t);
   $Y1 += 0.00000000001*cos(2.62660758779 + 189854.24140010288*$t);
   $Y1 += 0.00000000001*cos(3.14624674879 + 54879.6662553075*$t);
   $Y1 += 0.00000000002*cos(3.37542474980 + 94330.01910368088*$t);
   $Y1 += 0.00000000002*cos(4.79787865109 + 22747.04689739089*$t);
   $Y1 += 0.00000000001*cos(3.45952976424 + 138319.84867869345*$t);
   $Y1 += 0.00000000001*cos(3.22111896048 + 52509.9060414013*$t);
   $Y1 += 0.00000000001*cos(1.00099608993 + 35191.56631820829*$t);
   $Y1 += 0.00000000002*cos(4.07843204929 + 26713.81715137009*$t);
   $Y1 += 0.00000000002*cos(4.72678563551 + 25462.4767667453*$t);
   $Y1 += 0.00000000001*cos(1.39808874199 + 76667.27916495448*$t);
   $Y1 += 0.00000000001*cos(2.59019797939 + 116783.41139921108*$t);
   $Y1 += 0.00000000002*cos(2.54595097754 + 23919.3864767751*$t);
   $Y1 += 0.00000000002*cos(6.25926670726 + 28256.9074413403*$t);
   $Y1 += 0.00000000002*cos(0.98427002013 + 99024.38027539749*$t);
   $Y1 += 0.00000000001*cos(3.96509151964 + 112546.13171698889*$t);
   $Y1 += 0.00000000002*cos(5.94020231509 + 26241.4381345755*$t);
   $Y1 += 0.00000000001*cos(2.04558297271 + 129329.89333829407*$t);
   $Y1 += 0.00000000002*cos(4.89127097596 + 118828.71993201328*$t);
   $Y1 += 0.00000000001*cos(5.38890659728 + 49956.8053614781*$t);
   $Y1 += 0.00000000001*cos(0.01220959090 + 28199.79727243529*$t);
   $Y1 += 0.00000000001*cos(2.93719580530 + 68241.62832713968*$t);
   $Y1 += 0.00000000001*cos(0.45657187864 + 103821.67778371868*$t);
   $Y1 += 0.00000000002*cos(0.10751328565 + 25441.13612574289*$t);
   $Y1 += 0.00000000001*cos(3.04150710062 + 80175.15272542329*$t);
   $Y1 += 0.00000000002*cos(5.19554530010 + 23962.2695516785*$t);
   $Y1 += 0.00000000001*cos(1.92944311048 + 26934.2297938089*$t);
   $Y1 += 0.00000000001*cos(0.85501733527 + 42153.72518556549*$t);
   $Y1 += 0.00000000001*cos(5.28557867032 + 50380.79165691089*$t);
   $Y1 += 0.00000000001*cos(3.30348432730 + 241562.08269289686*$t);
   $Y1 += 0.00000000001*cos(1.53016890390 + 55618.1374106303*$t);
   $Y1 += 0.00000000001*cos(0.20150489504 + 92740.81679043909*$t);
   $Y1 += 0.00000000002*cos(4.04923139057 + 27567.01353312209*$t);
   $Y1 += 0.00000000002*cos(2.43602091587 + 98690.03669966108*$t);
   $Y1 += 0.00000000001*cos(5.40866392547 + 173511.17117941826*$t);
   $Y1 += 0.00000000001*cos(2.26707127188 + 173511.65881438529*$t);
   $Y1 += 0.00000000001*cos(5.30789985746 + 102018.17235594128*$t);
   $Y1 += 0.00000000001*cos(2.40978376724 + 49954.19346603489*$t);
   $Y1 += 0.00000000001*cos(5.41274037425 + 155461.16718973868*$t);
   $Y1 += 0.00000000001*cos(2.80860052221 + 193937.74227184022*$t);
   $Y1 += 0.00000000001*cos(2.80241855615 + 26927.11624680809*$t);
   $Y1 += 0.00000000001*cos(6.00279912865 + 25249.1776713073*$t);
   $Y1 += 0.00000000001*cos(5.41436875302 + 149642.51277699786*$t);
   $Y1 += 0.00000000001*cos(4.06699271013 + 112231.45790215228*$t);
   $Y1 += 0.00000000001*cos(4.88680216829 + 639.6534688305*$t);
   $Y1 += 0.00000000001*cos(4.38938154703 + 28420.8557169627*$t);
   $Y1 += 0.00000000002*cos(1.63653705415 + 55503.69812194508*$t);
   $Y1 += 0.00000000001*cos(1.19212507226 + 35472.50033216589*$t);
   $Y1 += 0.00000000001*cos(6.22889842286 + 48997.9043100643*$t);
   $Y1 += 0.00000000001*cos(3.64991019877 + 27006.0772450395*$t);
   $Y1 += 0.00000000001*cos(5.15530748603 + 25170.2166730759*$t);
   $Y1 += 0.00000000001*cos(0.21655513602 + 106263.05586699749*$t);
   $Y1 += 0.00000000001*cos(3.15004684273 + 90989.40667698109*$t);
   $Y1 += 0.00000000001*cos(3.42636128355 + 50690.06997956669*$t);
   $Y1 += 0.00000000001*cos(1.90636858110 + 77836.86741636308*$t);
   $Y1 += 0.00000000001*cos(1.74306827310 + 125112.2834169717*$t);
   $Y1 += 0.00000000001*cos(0.83861407900 + 26610.23674218449*$t);
   $Y1 += 0.00000000001*cos(6.19974077735 + 143980.69309287866*$t);
   $Y1 += 0.00000000001*cos(2.30573879835 + 153084.60008699627*$t);
   $Y1 += 0.00000000001*cos(2.74311464361 + 45404.8518644193*$t);
   $Y1 += 0.00000000001*cos(4.11780710445 + 2332.9525753885*$t);
   $Y1 += 0.00000000001*cos(2.50393985667 + 76152.30292882849*$t);
   $Y1 += 0.00000000001*cos(3.70443537393 + 45494.33761226529*$t);
   $Y1 += 0.00000000001*cos(1.90974220877 + 24925.18461968289*$t);
   $Y1 += 0.00000000001*cos(3.79471530054 + 50593.60311738189*$t);
   $Y1 += 0.00000000001*cos(0.35163928497 + 115674.03284711768*$t);
   $Y1 += 0.00000000001*cos(4.64759915605 + 151200.18655854589*$t);
   $Y1 += 0.00000000001*cos(6.12692757039 + 84308.18182358149*$t);
   $Y1 += 0.00000000001*cos(2.49793910811 + 129379.88996019828*$t);
   $Y1 += 0.00000000001*cos(4.24960627112 + 24073.1652872599*$t);
   $Y1 += 0.00000000001*cos(4.55561141368 + 28103.1286308555*$t);
   $Y1 += 0.00000000001*cos(3.64820977115 + 19406.43447069109*$t);
   $Y1 += 0.00000000001*cos(5.22888939483 + 24844.6593573793*$t);
   $Y1 += 0.00000000001*cos(3.57632828997 + 27331.6345607361*$t);
   $Y1 += 0.00000000001*cos(0.38918442414 + 154408.89880655028*$t);
   $Y1 += 0.00000000001*cos(4.58125009028 + 163298.37326817428*$t);
   $Y1 += 0.00000000001*cos(4.05552152915 + 58220.2786820073*$t);
   $Y1 += 0.00000000001*cos(0.81056243735 + 97670.63153038069*$t);
   $Y1 += 0.00000000001*cos(1.70406295246 + 22759.5236678105*$t);
   $Y1 += 0.00000000001*cos(0.17160219777 + 24395.9812895553*$t);
   $Y1 += 0.00000000001*cos(4.19916792884 + 92741.3044254061*$t);
   $Y1 += 0.00000000001*cos(0.46762289675 + 128850.19899510368*$t);
   $Y1 += 0.00000000001*cos(3.27962382087 + 25984.56654987209*$t);
   $Y1 += 0.00000000001*cos(3.06090263008 + 24357.0246061251*$t);
   $Y1 += 0.00000000001*cos(5.74431505472 + 27819.2693119903*$t);
   $Y1 += 0.00000000001*cos(1.47564589622 + 183724.45672562926*$t);
   $Y1 += 0.00000000001*cos(2.32406253516 + 26624.46383618609*$t);
   $Y1 += 0.00000000001*cos(0.16830959152 + 25455.3632197445*$t);
   $Y1 += 0.00000000001*cos(3.30990224511 + 25454.87558477749*$t);
   $Y1 += 0.00000000001*cos(0.20081607790 + 189385.78877478145*$t);
   $Y1 += 0.00000000001*cos(3.34240873149 + 189386.27640974846*$t);
   $Y1 += 0.00000000001*cos(5.05200956798 + 209659.06869168568*$t);
   $Y1 += 0.00000000001*cos(2.14910744070 + 109.9625037359*$t);
   $Y1 += 0.00000000001*cos(4.58272161819 + 82815.41910429788*$t);
   $Y1 += 0.00000000001*cos(0.56829620714 + 147423.26803784407*$t);
   $Y1 += 0.00000000001*cos(3.84356790046 + 23704.95394428149*$t);
   $Y1 += 0.00000000001*cos(4.96164978434 + 28471.3399738339*$t);
   $Y1 += 0.00000000001*cos(3.64905591301 + 51440.17358710009*$t);
   $Y1 += 0.00000000001*cos(1.10275035993 + 25035.39094090229*$t);
   $Y1 += 0.00000000001*cos(0.92306365199 + 26555.6243144451*$t);
   $Y1 += 0.00000000001*cos(2.45569831568 + 26177.63270690369*$t);
   $Y1 += 0.00000000001*cos(0.06633406194 + 25998.6612112117*$t);
   $Y1 += 0.00000000001*cos(0.41633866770 + 233681.99353955788*$t);
   $Y1 += 0.00000000001*cos(1.60238548900 + 21650.63275068411*$t);
   $Y1 += 0.00000000001*cos(0.91964688862 + 30525.66116743129*$t);
   $Y1 += 0.00000000001*cos(3.50363747347 + 111590.04434041328*$t);
   $Y1 += 0.00000000001*cos(2.57975917809 + 24203.24579564031*$t);
   $Y1 += 0.00000000001*cos(6.22545850671 + 27973.04812247509*$t);
   $Y1 += 0.00000000001*cos(5.61388226798 + 99799.41525175449*$t);
   $Y1 += 0.00000000001*cos(0.90114538219 + 128106.56313248246*$t);
   $Y1 += 0.00000000001*cos(2.25118383839 + 104198.07757329549*$t);
   $Y1 += 0.00000000001*cos(0.23482210375 + 132350.95900857169*$t);
   $Y1 += 0.00000000001*cos(5.46489469781 + 6680.98103591609*$t);
   $Y1 += 0.00000000001*cos(4.59107671722 + 26202.09861277591*$t);
   $Y1 += 0.00000000001*cos(1.28948419377 + 50167.49256147289*$t);
   $Y1 += 0.00000000001*cos(4.70995069593 + 313054.59388140672*$t);
   $Y1 += 0.00000000001*cos(1.61351002182 + 131548.65044248088*$t);
   $Y1 += 0.00000000001*cos(0.21134212997 + 85502.14119883909*$t);
   $Y1 += 0.00000000001*cos(3.59852250423 + 51542.77872635169*$t);
   $Y1 += 0.00000000001*cos(4.35050776234 + 102018.65999090829*$t);
   $Y1 += 0.00000000001*cos(5.00190822272 + 51322.36608391289*$t);
   $Y1 += 0.00000000001*cos(6.26093145675 + 78413.51643934069*$t);
   $Y1 += 0.00000000001*cos(4.71083057280 + 156531.54400228668*$t);
   $Y1 += 0.00000000001*cos(2.77611181285 + 105306.96849042189*$t);
   $Y1 += 0.00000000001*cos(4.15505663410 + 35077.12702952309*$t);
   $Y1 += 0.00000000001*cos(2.66891943800 + 155467.79310177248*$t);
   $Y1 += 0.00000000001*cos(2.90242040099 + 45290.41257573408*$t);
   $Y1 += 0.00000000001*cos(5.09454263682 + 1162.23088692429*$t);
   $Y1 += 0.00000000001*cos(5.85832218706 + 77211.19722405069*$t);
   $Y1 += 0.00000000001*cos(2.94258825469 + 25152.2169783489*$t);
   $Y1 += 0.00000000001*cos(5.86262943011 + 27024.07693976651*$t);
   $Y1 += 0.00000000001*cos(2.69263645947 + 247223.41474204906*$t);
   $Y1 += 0.00000000001*cos(2.39382184066 + 183674.94773869208*$t);
   $Y1 += 0.00000000001*cos(4.71424763358 + 25131.37016812009*$t);
   $Y1 += 0.00000000001*cos(4.49841046349 + 26126.2799946955*$t);
   $Y1 += 0.00000000001*cos(4.30680722131 + 26050.0139234199*$t);
   $Y1 += 0.00000000001*cos(2.54564315044 + 71980.38975724769*$t);
   $Y1 += 0.00000000001*cos(3.77812730736 + 27388.97377726269*$t);
   $Y1 += 0.00000000001*cos(5.02709037744 + 24787.3201408527*$t);
   $Y1 += 0.00000000001*cos(5.22928116569 + 155475.39428374029*$t);
   $Y1 += 0.00000000001*cos(2.23214428102 + 51756.07782178969*$t);
   $Y1 += 0.00000000001*cos(3.75930601793 + 54086.76194888209*$t);
   $Y1 += 0.00000000001*cos(5.71244644048 + 94138.08320260228*$t);
   $Y1 += 0.00000000001*cos(3.60259923599 + 24602.8562523545*$t);
   $Y1 += 0.00000000001*cos(5.20261844881 + 27573.4376657609*$t);
   $Y1 += 0.00000000001*cos(5.97633149627 + 121335.36489626988*$t);
   $Y1 += 0.00000000001*cos(5.11853435105 + 198490.18340386607*$t);
   $Y1 += 0.00000000001*cos(2.41668452027 + 181555.69624334667*$t);
   $Y1 += 0.00000000001*cos(3.56429469477 + 156534.77621392948*$t);
   $Y1 += 0.00000000001*cos(0.16534790503 + 12725.6972522585*$t);
   $Y1 += 0.00000000001*cos(5.23354450332 + 130006.04778747768*$t);
   $Y1 += 0.00000000001*cos(3.16983107219 + 89586.12970554348*$t);
   $Y1 += 0.00000000001*cos(5.31492826690 + 131394.87163199608*$t);
   $Y1 += 0.00000000001*cos(2.18749112623 + 77039.64776646046*$t);
   return $Y1*$t;
}



   function Mercury_Y2($t) // 413 terms of order 2
{
   $Y2  = 0.00020000263*cos(4.39813856862 + 26088.1469590577*$t);
   $Y2 += 0.00008268782*cos(5.12831925217 + 0.2438174835*$t);
   $Y2 += 0.00002498175*cos(1.81785035615 + 52176.0501006319*$t);
   $Y2 += 0.00000949347*cos(5.50772040452 + 78263.95324220609*$t);
   $Y2 += 0.00000776108*cos(0.81508927377 + 26087.65932409069*$t);
   $Y2 += 0.00000371109*cos(2.46339631518 + 104351.85638378029*$t);
   $Y2 += 0.00000130259*cos(5.58371836146 + 130439.75952535449*$t);
   $Y2 += 0.00000069902*cos(3.59145684280 + 52175.56246566489*$t);
   $Y2 += 0.00000042865*cos(2.38418074575 + 156527.66266692868*$t);
   $Y2 += 0.00000013564*cos(5.45308914681 + 182615.56580850288*$t);
   $Y2 += 0.00000011344*cos(0.18546869670 + 78263.46560723908*$t);
   $Y2 += 0.00000004184*cos(2.23177226385 + 208703.46895007708*$t);
   $Y2 += 0.00000002952*cos(5.92358038862 + 24978.7684069643*$t);
   $Y2 += 0.00000002715*cos(2.62856029278 + 27197.5255111511*$t);
   $Y2 += 0.00000002412*cos(3.20116782795 + 104351.36874881329*$t);
   $Y2 += 0.00000001268*cos(5.28987764263 + 234791.37209165128*$t);
   $Y2 += 0.00000001017*cos(2.76352422322 + 51066.6715485385*$t);
   $Y2 += 0.00000000591*cos(0.02739486208 + 130439.27189038748*$t);
   $Y2 += 0.00000000372*cos(5.53602920422 + 53285.4286527253*$t);
   $Y2 += 0.00000000379*cos(2.06228553945 + 260879.27523322542*$t);
   $Y2 += 0.00000000353*cos(5.84872228977 + 77154.57469011268*$t);
   $Y2 += 0.00000000353*cos(3.64262209095 + 1109.1347346099*$t);
   $Y2 += 0.00000000259*cos(2.45927570893 + 20426.8149099055*$t);
   $Y2 += 0.00000000259*cos(3.62787658106 + 1059.6257476727*$t);
   $Y2 += 0.00000000198*cos(6.08219469431 + 27043.7467006663*$t);
   $Y2 += 0.00000000178*cos(4.39645894496 + 31749.4790082099*$t);
   $Y2 += 0.00000000161*cos(5.71880342023 + 1589.3167127673*$t);
   $Y2 += 0.00000000148*cos(1.40374146640 + 4552.1973145423*$t);
   $Y2 += 0.00000000145*cos(0.02368396611 + 21536.1934619989*$t);
   $Y2 += 0.00000000158*cos(3.17661979051 + 156527.17503196168*$t);
   $Y2 += 0.00000000169*cos(2.43702666403 + 25132.5472174491*$t);
   $Y2 += 0.00000000134*cos(4.36077570655 + 25558.4559939631*$t);
   $Y2 += 0.00000000112*cos(5.11701683724 + 286967.17837479962*$t);
   $Y2 += 0.00000000111*cos(4.08624415081 + 26617.8379241523*$t);
   $Y2 += 0.00000000122*cos(6.20816277459 + 46514.7180514797*$t);
   $Y2 += 0.00000000125*cos(6.10874998399 + 23869.3898548709*$t);
   $Y2 += 0.00000000111*cos(0.71893259930 + 14765.48286075331*$t);
   $Y2 += 0.00000000106*cos(3.19958057200 + 51116.6681704427*$t);
   $Y2 += 0.00000000120*cos(2.63555930925 + 103242.47783168689*$t);
   $Y2 += 0.00000000113*cos(1.00068732157 + 27147.5288892469*$t);
   $Y2 += 0.00000000105*cos(2.62816055600 + 28306.90406324449*$t);
   $Y2 += 0.00000000086*cos(3.50491777711 + 19317.43635781211*$t);
   $Y2 += 0.00000000103*cos(1.19071414957 + 5661.0882316687*$t);
   $Y2 += 0.00000000067*cos(4.69029936211 + 9104.1508116011*$t);
   $Y2 += 0.00000000076*cos(0.32408375390 + 529.9347825781*$t);
   $Y2 += 0.00000000066*cos(2.87399773118 + 50586.97720534809*$t);
   $Y2 += 0.00000000066*cos(2.27463939345 + 3442.81876244891*$t);
   $Y2 += 0.00000000061*cos(1.26603377840 + 37410.8110573621*$t);
   $Y2 += 0.00000000059*cos(3.16922770991 + 72602.62119305388*$t);
   $Y2 += 0.00000000074*cos(0.14312537861 + 22645.57201409229*$t);
   $Y2 += 0.00000000056*cos(0.87699245253 + 52705.7410657265*$t);
   $Y2 += 0.00000000061*cos(1.98731213334 + 30640.1004561165*$t);
   $Y2 += 0.00000000053*cos(3.72430094233 + 640.1411037975*$t);
   $Y2 += 0.00000000056*cos(5.15278563625 + 32858.8575603033*$t);
   $Y2 += 0.00000000062*cos(5.26544507482 + 51220.4503590233*$t);
   $Y2 += 0.00000000050*cos(2.61090296354 + 53131.6498422405*$t);
   $Y2 += 0.00000000046*cos(2.37745190465 + 27677.2198543415*$t);
   $Y2 += 0.00000000045*cos(0.02572397068 + 25973.70767037247*$t);
   $Y2 += 0.00000000045*cos(2.48433200554 + 26202.58624774291*$t);
   $Y2 += 0.00000000044*cos(0.04666322888 + 182615.07817353586*$t);
   $Y2 += 0.00000000046*cos(4.10779409868 + 27197.03787618409*$t);
   $Y2 += 0.00000000046*cos(4.21737357836 + 53235.43203082109*$t);
   $Y2 += 0.00000000058*cos(0.51456998627 + 57837.3821497841*$t);
   $Y2 += 0.00000000043*cos(4.72097651637 + 529.44714761109*$t);
   $Y2 += 0.00000000042*cos(5.81334759172 + 47624.0966035731*$t);
   $Y2 += 0.00000000041*cos(1.35360879608 + 26301.4460544957*$t);
   $Y2 += 0.00000000039*cos(3.95217511506 + 40853.3860023275*$t);
   $Y2 += 0.00000000038*cos(1.01923360930 + 25874.8478636197*$t);
   $Y2 += 0.00000000045*cos(2.30681711799 + 29530.7219040231*$t);
   $Y2 += 0.00000000045*cos(3.25992009502 + 10213.5293636945*$t);
   $Y2 += 0.00000000035*cos(0.20035334587 + 2119.00767786191*$t);
   $Y2 += 0.00000000034*cos(5.21776132031 + 56728.0035976907*$t);
   $Y2 += 0.00000000040*cos(0.28616417785 + 24978.28077199729*$t);
   $Y2 += 0.00000000040*cos(5.69976040495 + 129330.38097326108*$t);
   $Y2 += 0.00000000033*cos(1.88435460588 + 313055.08151637383*$t);
   $Y2 += 0.00000000039*cos(0.71277136304 + 955.3559241251*$t);
   $Y2 += 0.00000000031*cos(5.95336552931 + 6770.46678376209*$t);
   $Y2 += 0.00000000038*cos(3.30701415262 + 2218.51328670329*$t);
   $Y2 += 0.00000000031*cos(5.89629431306 + 12566.3955174663*$t);
   $Y2 += 0.00000000032*cos(2.89503281988 + 49957.2929964451*$t);
   $Y2 += 0.00000000029*cos(4.08747500636 + 426.8420083595*$t);
   $Y2 += 0.00000000028*cos(3.89028250879 + 43072.1431065143*$t);
   $Y2 += 0.00000000025*cos(0.01043820952 + 98690.52433462809*$t);
   $Y2 += 0.00000000030*cos(1.58323308741 + 79373.33179429949*$t);
   $Y2 += 0.00000000023*cos(3.87566855556 + 78793.64420730069*$t);
   $Y2 += 0.00000000030*cos(5.69174979475 + 77204.57131201689*$t);
   $Y2 += 0.00000000020*cos(2.91659016078 + 11322.42028082089*$t);
   $Y2 += 0.00000000027*cos(5.66737798141 + 54394.80720481869*$t);
   $Y2 += 0.00000000023*cos(1.92399700068 + 77308.35350059748*$t);
   $Y2 += 0.00000000019*cos(4.33796152575 + 83925.28529135829*$t);
   $Y2 += 0.00000000022*cos(0.31026735565 + 45405.3394993863*$t);
   $Y2 += 0.00000000020*cos(0.76233233344 + 36301.4325052687*$t);
   $Y2 += 0.00000000018*cos(1.06684593234 + 79323.33517239528*$t);
   $Y2 += 0.00000000021*cos(1.06633987406 + 13656.10430865991*$t);
   $Y2 += 0.00000000020*cos(3.26662752657 + 33326.8225506577*$t);
   $Y2 += 0.00000000018*cos(0.08918897822 + 52156.38033973211*$t);
   $Y2 += 0.00000000023*cos(2.56517709904 + 41962.7645544209*$t);
   $Y2 += 0.00000000018*cos(4.49923548064 + 52389.3491960699*$t);
   $Y2 += 0.00000000017*cos(1.90164043236 + 58946.76070187749*$t);
   $Y2 += 0.00000000016*cos(0.07721057973 + 23754.95056618569*$t);
   $Y2 += 0.00000000019*cos(0.79767727082 + 39609.8984006491*$t);
   $Y2 += 0.00000000017*cos(4.96752330955 + 51536.15281431789*$t);
   $Y2 += 0.00000000018*cos(5.05941872394 + 18849.4713674577*$t);
   $Y2 += 0.00000000015*cos(5.67105202528 + 76674.88034692229*$t);
   $Y2 += 0.00000000015*cos(0.18434972499 + 48733.47515566649*$t);
   $Y2 += 0.00000000015*cos(2.77916270049 + 853.4401992355*$t);
   $Y2 += 0.00000000014*cos(2.04393411320 + 82815.90673926489*$t);
   $Y2 += 0.00000000014*cos(3.45421098562 + 52179.9314359899*$t);
   $Y2 += 0.00000000014*cos(0.12812152173 + 15874.8614128467*$t);
   $Y2 += 0.00000000014*cos(0.79649939505 + 66941.2891439017*$t);
   $Y2 += 0.00000000015*cos(3.92393517562 + 16984.2399649401*$t);
   $Y2 += 0.00000000015*cos(3.48624645549 + 38654.2986590405*$t);
   $Y2 += 0.00000000013*cos(5.35926600957 + 53765.1229959157*$t);
   $Y2 += 0.00000000013*cos(3.19246765373 + 208702.98131511007*$t);
   $Y2 += 0.00000000013*cos(2.25098683682 + 50057.2862402535*$t);
   $Y2 += 0.00000000016*cos(0.48148728540 + 51646.3591355373*$t);
   $Y2 += 0.00000000012*cos(2.40833345233 + 28421.3433519297*$t);
   $Y2 += 0.00000000013*cos(2.47413691628 + 155418.28411483529*$t);
   $Y2 += 0.00000000016*cos(5.37235452634 + 55618.6250455973*$t);
   $Y2 += 0.00000000012*cos(2.05490297780 + 5661.5758666357*$t);
   $Y2 += 0.00000000015*cos(5.46754476025 + 52195.71986153169*$t);
   $Y2 += 0.00000000014*cos(1.23967074882 + 26514.7451499337*$t);
   $Y2 += 0.00000000014*cos(1.73016204603 + 110013.18843293248*$t);
   $Y2 += 0.00000000013*cos(2.13522240551 + 213.0552779545*$t);
   $Y2 += 0.00000000014*cos(0.53482285581 + 26728.0442453717*$t);
   $Y2 += 0.00000000013*cos(2.09871625287 + 52172.1687652739*$t);
   $Y2 += 0.00000000011*cos(3.89215785881 + 63498.71419893629*$t);
   $Y2 += 0.00000000011*cos(5.32469405612 + 79219.55298381469*$t);
   $Y2 += 0.00000000010*cos(3.09632978067 + 103292.47445359109*$t);
   $Y2 += 0.00000000011*cos(0.49318054067 + 31748.99137324289*$t);
   $Y2 += 0.00000000014*cos(1.76374143598 + 6283.3196674749*$t);
   $Y2 += 0.00000000010*cos(5.54944886749 + 52290.48938931711*$t);
   $Y2 += 0.00000000012*cos(4.17529024605 + 13521.50762410789*$t);
   $Y2 += 0.00000000009*cos(1.46108642538 + 35192.0539531753*$t);
   $Y2 += 0.00000000009*cos(5.95543898844 + 76045.1961380193*$t);
   $Y2 += 0.00000000011*cos(4.31962648376 + 62389.33564684289*$t);
   $Y2 += 0.00000000010*cos(2.59338730813 + 51749.45190975589*$t);
   $Y2 += 0.00000000009*cos(3.09241629007 + 124778.42747620228*$t);
   $Y2 += 0.00000000009*cos(4.94047648268 + 339142.98465794802*$t);
   $Y2 += 0.00000000009*cos(3.18433513864 + 78244.28348130629*$t);
   $Y2 += 0.00000000009*cos(4.21342321430 + 20426.32727493849*$t);
   $Y2 += 0.00000000011*cos(1.35641733071 + 38520.1896094555*$t);
   $Y2 += 0.00000000010*cos(4.87324110048 + 16983.75232997309*$t);
   $Y2 += 0.00000000009*cos(5.87248180566 + 1109.6223695769*$t);
   $Y2 += 0.00000000009*cos(3.08529390196 + 52061.61081194667*$t);
   $Y2 += 0.00000000009*cos(0.58390742179 + 104881.54734887488*$t);
   $Y2 += 0.00000000008*cos(0.01049678658 + 68050.66769599509*$t);
   $Y2 += 0.00000000008*cos(2.44377299382 + 2333.44021035551*$t);
   $Y2 += 0.00000000008*cos(2.07553269199 + 12431.79883291429*$t);
   $Y2 += 0.00000000008*cos(3.14068514277 + 28206.9108194361*$t);
   $Y2 += 0.00000000008*cos(4.93667216370 + 136101.09157450669*$t);
   $Y2 += 0.00000000008*cos(3.56776272307 + 213.5429129215*$t);
   $Y2 += 0.00000000008*cos(6.24685210587 + 24499.0740637739*$t);
   $Y2 += 0.00000000008*cos(4.90871772456 + 103396.25664217169*$t);
   $Y2 += 0.00000000007*cos(3.26004517300 + 52168.93655363109*$t);
   $Y2 += 0.00000000009*cos(2.25308643231 + 21535.70582703189*$t);
   $Y2 += 0.00000000007*cos(0.96066249738 + 53284.94101775829*$t);
   $Y2 += 0.00000000007*cos(3.34414054226 + 105461.23493587368*$t);
   $Y2 += 0.00000000007*cos(0.56178116893 + 69160.04624808849*$t);
   $Y2 += 0.00000000009*cos(4.89006567905 + 7994.77225950771*$t);
   $Y2 += 0.00000000009*cos(1.87204677749 + 73711.99974514729*$t);
   $Y2 += 0.00000000008*cos(6.11573252209 + 59414.7256922319*$t);
   $Y2 += 0.00000000008*cos(2.34935594859 + 52183.1636476327*$t);
   $Y2 += 0.00000000007*cos(0.38833894888 + 64742.2018006147*$t);
   $Y2 += 0.00000000007*cos(0.26691004914 + 78267.83457756409*$t);
   $Y2 += 0.00000000007*cos(3.93557943129 + 26107.81671995749*$t);
   $Y2 += 0.00000000007*cos(1.72211044919 + 26068.47719815791*$t);
   $Y2 += 0.00000000006*cos(0.91009879641 + 25028.7650288685*$t);
   $Y2 += 0.00000000007*cos(4.52931279940 + 51962.7510051939*$t);
   $Y2 += 0.00000000007*cos(3.63772636438 + 18208.05780571871*$t);
   $Y2 += 0.00000000008*cos(1.62956568948 + 44937.3745090319*$t);
   $Y2 += 0.00000000008*cos(3.38799810242 + 51066.18391357149*$t);
   $Y2 += 0.00000000006*cos(4.15575122576 + 105411.23831396949*$t);
   $Y2 += 0.00000000007*cos(3.29006350511 + 65697.80154222329*$t);
   $Y2 += 0.00000000006*cos(1.27230132469 + 88477.23878841709*$t);
   $Y2 += 0.00000000006*cos(1.32946065086 + 78477.25233764409*$t);
   $Y2 += 0.00000000006*cos(4.53641896383 + 52602.6482915079*$t);
   $Y2 += 0.00000000007*cos(2.19484677855 + 78283.62300310588*$t);
   $Y2 += 0.00000000006*cos(0.97859883313 + 19805.0711090663*$t);
   $Y2 += 0.00000000005*cos(5.12029859122 + 108903.80988083909*$t);
   $Y2 += 0.00000000006*cos(0.51108107596 + 129380.37759516528*$t);
   $Y2 += 0.00000000005*cos(3.17438180692 + 74821.37829724069*$t);
   $Y2 += 0.00000000007*cos(3.90084269972 + 45892.9742506405*$t);
   $Y2 += 0.00000000005*cos(5.53465282046 + 1059.1381127057*$t);
   $Y2 += 0.00000000005*cos(0.53945271085 + 26084.2656236997*$t);
   $Y2 += 0.00000000005*cos(5.12008322277 + 26092.0282944157*$t);
   $Y2 += 0.00000000007*cos(3.38101832696 + 71493.2426409605*$t);
   $Y2 += 0.00000000005*cos(2.15419212921 + 81706.5281871715*$t);
   $Y2 += 0.00000000005*cos(5.99431638416 + 51322.85371887989*$t);
   $Y2 += 0.00000000005*cos(5.10141066049 + 33968.23611239669*$t);
   $Y2 += 0.00000000005*cos(6.07497929534 + 78256.83969520529*$t);
   $Y2 += 0.00000000005*cos(4.51394707071 + 26617.35028918529*$t);
   $Y2 += 0.00000000006*cos(5.11905884366 + 78260.07190684808*$t);
   $Y2 += 0.00000000004*cos(2.02155104150 + 25661.5487681817*$t);
   $Y2 += 0.00000000005*cos(3.63607614834 + 94138.57083756929*$t);
   $Y2 += 0.00000000004*cos(1.02197474906 + 22645.08437912529*$t);
   $Y2 += 0.00000000005*cos(3.86831492333 + 93029.19228547589*$t);
   $Y2 += 0.00000000004*cos(6.23205767407 + 22760.01130277751*$t);
   $Y2 += 0.00000000004*cos(2.54287848992 + 29416.28261533789*$t);
   $Y2 += 0.00000000004*cos(5.04658501458 + 76145.18938182769*$t);
   $Y2 += 0.00000000004*cos(5.53028555901 + 181506.18725640948*$t);
   $Y2 += 0.00000000004*cos(4.30468862075 + 13521.9952590749*$t);
   $Y2 += 0.00000000004*cos(5.69769574613 + 99799.90288672148*$t);
   $Y2 += 0.00000000004*cos(2.07912078316 + 79853.02613748988*$t);
   $Y2 += 0.00000000005*cos(3.94088501872 + 44181.52165860769*$t);
   $Y2 += 0.00000000004*cos(5.75049226379 + 4551.7096795753*$t);
   $Y2 += 0.00000000004*cos(5.78252204456 + 32371.2228090491*$t);
   $Y2 += 0.00000000004*cos(6.13397098904 + 32858.36992533629*$t);
   $Y2 += 0.00000000004*cos(5.47303342696 + 54509.2464935039*$t);
   $Y2 += 0.00000000004*cos(3.77296926059 + 155468.28073673949*$t);
   $Y2 += 0.00000000003*cos(1.76022764417 + 162188.99471608088*$t);
   $Y2 += 0.00000000003*cos(3.47863732548 + 24498.58642880689*$t);
   $Y2 += 0.00000000005*cos(5.41867016378 + 78271.06678920689*$t);
   $Y2 += 0.00000000004*cos(0.03915152889 + 234790.88445668427*$t);
   $Y2 += 0.00000000003*cos(3.63049034768 + 77734.26227711148*$t);
   $Y2 += 0.00000000004*cos(5.73459041878 + 7879.84533585549*$t);
   $Y2 += 0.00000000004*cos(4.32304044619 + 64608.09275102969*$t);
   $Y2 += 0.00000000004*cos(6.08591965357 + 51116.18053547569*$t);
   $Y2 += 0.00000000003*cos(3.39565834350 + 28306.41642827749*$t);
   $Y2 += 0.00000000004*cos(4.14203046511 + 39744.0074502341*$t);
   $Y2 += 0.00000000003*cos(6.25044565160 + 104332.1866228805*$t);
   $Y2 += 0.00000000004*cos(2.32061218672 + 80482.71034639288*$t);
   $Y2 += 0.00000000003*cos(3.42862683678 + 37410.32342239509*$t);
   $Y2 += 0.00000000004*cos(6.26655525096 + 24864.32911827909*$t);
   $Y2 += 0.00000000003*cos(0.81932036845 + 103822.16541868569*$t);
   $Y2 += 0.00000000004*cos(2.51942976235 + 27311.96479983631*$t);
   $Y2 += 0.00000000003*cos(0.55238739809 + 120226.47397914348*$t);
   $Y2 += 0.00000000003*cos(2.73238956336 + 102133.09927959349*$t);
   $Y2 += 0.00000000003*cos(6.16210003244 + 150866.33061777649*$t);
   $Y2 += 0.00000000003*cos(3.47088315334 + 90830.10494218889*$t);
   $Y2 += 0.00000000003*cos(1.70583242978 + 77624.05595589208*$t);
   $Y2 += 0.00000000003*cos(1.64244671238 + 129484.15978374588*$t);
   $Y2 += 0.00000000003*cos(3.01438324827 + 125887.80602829569*$t);
   $Y2 += 0.00000000003*cos(3.59751110770 + 130969.45049044909*$t);
   $Y2 += 0.00000000003*cos(3.38263477770 + 58459.1259506233*$t);
   $Y2 += 0.00000000002*cos(2.33982742399 + 6282.83203250789*$t);
   $Y2 += 0.00000000003*cos(5.89330250493 + 26941.3433408097*$t);
   $Y2 += 0.00000000002*cos(3.22048008840 + 3327.89183879669*$t);
   $Y2 += 0.00000000003*cos(4.49509466805 + 18093.13088206649*$t);
   $Y2 += 0.00000000002*cos(2.81219352708 + 25132.0595824821*$t);
   $Y2 += 0.00000000003*cos(4.76356263080 + 26095.2605060585*$t);
   $Y2 += 0.00000000003*cos(4.55467549289 + 71025.27765060609*$t);
   $Y2 += 0.00000000003*cos(4.71882234896 + 43071.65547154729*$t);
   $Y2 += 0.00000000002*cos(4.39575489958 + 114565.14192999128*$t);
   $Y2 += 0.00000000002*cos(1.23106381310 + 71980.87739221469*$t);
   $Y2 += 0.00000000002*cos(0.75134917742 + 91785.70468379749*$t);
   $Y2 += 0.00000000002*cos(0.91717698933 + 2648.6986429565*$t);
   $Y2 += 0.00000000002*cos(2.32776941660 + 78378.3925308913*$t);
   $Y2 += 0.00000000003*cos(3.33005216581 + 104355.73771913828*$t);
   $Y2 += 0.00000000002*cos(0.88668781336 + 26081.03341205689*$t);
   $Y2 += 0.00000000002*cos(1.33604738376 + 77197.45776501608*$t);
   $Y2 += 0.00000000002*cos(1.70708347774 + 365230.88779952223*$t);
   $Y2 += 0.00000000002*cos(1.39529664977 + 78690.55143308209*$t);
   $Y2 += 0.00000000003*cos(3.63831731324 + 52815.9473869459*$t);
   $Y2 += 0.00000000002*cos(2.08527920288 + 25448.2496727437*$t);
   $Y2 += 0.00000000003*cos(5.17799187468 + 77837.35505133009*$t);
   $Y2 += 0.00000000002*cos(5.22616840288 + 104371.52614468009*$t);
   $Y2 += 0.00000000002*cos(2.61518622291 + 25028.27739390149*$t);
   $Y2 += 0.00000000002*cos(1.38915456619 + 11322.9079157879*$t);
   $Y2 += 0.00000000002*cos(0.72086659814 + 77211.68485901768*$t);
   $Y2 += 0.00000000002*cos(2.08668332966 + 105307.45612538888*$t);
   $Y2 += 0.00000000002*cos(2.01988438935 + 78050.65414676809*$t);
   $Y2 += 0.00000000002*cos(1.66916828850 + 27043.2590656993*$t);
   $Y2 += 0.00000000002*cos(2.76016995044 + 104344.74283677948*$t);
   $Y2 += 0.00000000002*cos(0.83226802149 + 46514.23041651269*$t);
   $Y2 += 0.00000000002*cos(0.22774400791 + 77154.08705514569*$t);
   $Y2 += 0.00000000002*cos(0.91172072873 + 26725.1432310819*$t);
   $Y2 += 0.00000000002*cos(1.61031164889 + 25451.1506870335*$t);
   $Y2 += 0.00000000002*cos(0.95099409511 + 131499.14145554368*$t);
   $Y2 += 0.00000000002*cos(4.49976969859 + 61279.9570947495*$t);
   $Y2 += 0.00000000002*cos(5.98345605195 + 89586.61734051049*$t);
   $Y2 += 0.00000000002*cos(2.85946010033 + 30639.61282114949*$t);
   $Y2 += 0.00000000002*cos(0.15838364848 + 97581.14578253469*$t);
   $Y2 += 0.00000000002*cos(6.14579824758 + 78149.51395352086*$t);
   $Y2 += 0.00000000002*cos(5.58929876175 + 7238.4317741165*$t);
   $Y2 += 0.00000000002*cos(0.83010213570 + 426.3543733925*$t);
   $Y2 += 0.00000000002*cos(5.20457051939 + 107794.43132874569*$t);
   $Y2 += 0.00000000002*cos(1.91271152917 + 47623.60896860609*$t);
   $Y2 += 0.00000000002*cos(4.42235381341 + 104565.15547921829*$t);
   $Y2 += 0.00000000002*cos(0.42021990927 + 44295.9609472929*$t);
   $Y2 += 0.00000000002*cos(3.90488175057 + 18093.6185170335*$t);
   $Y2 += 0.00000000002*cos(0.51702864254 + 10213.0417287275*$t);
   $Y2 += 0.00000000002*cos(2.19510126559 + 104358.96993078108*$t);
   $Y2 += 0.00000000002*cos(1.87197741484 + 104347.97504842229*$t);
   $Y2 += 0.00000000002*cos(1.89656096023 + 134991.71302241329*$t);
   $Y2 += 0.00000000002*cos(3.81068190615 + 129910.06856025988*$t);
   $Y2 += 0.00000000002*cos(3.48333455064 + 57836.89451481709*$t);
   $Y2 += 0.00000000002*cos(5.38520038611 + 25557.96835899609*$t);
   $Y2 += 0.00000000002*cos(6.12255852160 + 54294.81396101029*$t);
   $Y2 += 0.00000000002*cos(1.91069399856 + 102762.78348849648*$t);
   $Y2 += 0.00000000001*cos(5.94439573345 + 7238.9194090835*$t);
   $Y2 += 0.00000000002*cos(2.71520511595 + 25934.3681485729*$t);
   $Y2 += 0.00000000001*cos(6.12042545447 + 151975.70916986988*$t);
   $Y2 += 0.00000000002*cos(2.95370958319 + 26241.9257695425*$t);
   $Y2 += 0.00000000001*cos(6.21917309565 + 131549.13807744789*$t);
   $Y2 += 0.00000000001*cos(2.63165117521 + 23968.89546371229*$t);
   $Y2 += 0.00000000002*cos(1.38299478051 + 25021.6514818677*$t);
   $Y2 += 0.00000000002*cos(4.14961473905 + 27154.6424362477*$t);
   $Y2 += 0.00000000001*cos(0.59279120743 + 181556.18387831368*$t);
   $Y2 += 0.00000000001*cos(4.95292728782 + 27140.41534224609*$t);
   $Y2 += 0.00000000001*cos(0.19355831516 + 149.8070146181*$t);
   $Y2 += 0.00000000001*cos(5.03717522016 + 50586.48957038109*$t);
   $Y2 += 0.00000000001*cos(2.46166203858 + 85502.6288338061*$t);
   $Y2 += 0.00000000001*cos(3.12377250466 + 1052.51220067191*$t);
   $Y2 += 0.00000000001*cos(0.12220892820 + 84547.0290921975*$t);
   $Y2 += 0.00000000001*cos(4.99498518550 + 41962.27691945389*$t);
   $Y2 += 0.00000000002*cos(5.81835190108 + 1066.7392946735*$t);
   $Y2 += 0.00000000001*cos(0.19689603888 + 33326.33491569069*$t);
   $Y2 += 0.00000000001*cos(4.36202291393 + 103285.36090659029*$t);
   $Y2 += 0.00000000001*cos(5.63116195221 + 55504.18575691209*$t);
   $Y2 += 0.00000000001*cos(4.47656675233 + 117873.60782537168*$t);
   $Y2 += 0.00000000001*cos(0.96879148402 + 25035.8785758693*$t);
   $Y2 += 0.00000000001*cos(5.66596818667 + 58946.27306691049*$t);
   $Y2 += 0.00000000001*cos(4.82817050708 + 188276.89785765509*$t);
   $Y2 += 0.00000000001*cos(0.16804966441 + 49842.85370775989*$t);
   $Y2 += 0.00000000001*cos(3.20454167097 + 29428.7593857575*$t);
   $Y2 += 0.00000000001*cos(3.65119748789 + 146314.37712071768*$t);
   $Y2 += 0.00000000001*cos(0.55150389784 + 119117.0954270501*$t);
   $Y2 += 0.00000000001*cos(1.29641320948 + 39629.56816154889*$t);
   $Y2 += 0.00000000001*cos(1.17683878413 + 12546.72575656651*$t);
   $Y2 += 0.00000000001*cos(1.82966773526 + 64607.60511606269*$t);
   $Y2 += 0.00000000001*cos(0.51019327318 + 23868.9022199039*$t);
   $Y2 += 0.00000000001*cos(3.01779717813 + 48847.9144443517*$t);
   $Y2 += 0.00000000001*cos(3.08178730790 + 260878.78759825832*$t);
   $Y2 += 0.00000000001*cos(1.94628548757 + 25620.1819687033*$t);
   $Y2 += 0.00000000001*cos(4.90071449530 + 98068.78053378889*$t);
   $Y2 += 0.00000000001*cos(5.19983179973 + 22747.5345323579*$t);
   $Y2 += 0.00000000001*cos(3.72290134761 + 26556.11194941211*$t);
   $Y2 += 0.00000000001*cos(4.32894922591 + 85034.66384345168*$t);
   $Y2 += 0.00000000001*cos(3.83325919627 + 5327.7199258663*$t);
   $Y2 += 0.00000000001*cos(5.56885466592 + 63498.22656396929*$t);
   $Y2 += 0.00000000001*cos(2.05127060689 + 26300.95841952869*$t);
   $Y2 += 0.00000000001*cos(5.07273427642 + 105940.92927906409*$t);
   $Y2 += 0.00000000001*cos(0.91856795860 + 65831.91059180829*$t);
   $Y2 += 0.00000000001*cos(4.03570631976 + 79372.84415933248*$t);
   $Y2 += 0.00000000001*cos(1.62497049537 + 7.3573644843*$t);
   $Y2 += 0.00000000001*cos(5.58196282531 + 53399.86794141051*$t);
   $Y2 += 0.00000000001*cos(2.25556473621 + 80597.14963507809*$t);
   $Y2 += 0.00000000001*cos(1.53347024755 + 49527.59527515889*$t);
   $Y2 += 0.00000000001*cos(2.15927670335 + 36300.94487030169*$t);
   $Y2 += 0.00000000001*cos(0.13876413209 + 116918.00808376308*$t);
   $Y2 += 0.00000000001*cos(0.13834609478 + 26080.54577708989*$t);
   $Y2 += 0.00000000001*cos(1.87343313975 + 60056.13925397089*$t);
   $Y2 += 0.00000000001*cos(3.24836507326 + 10022.0810975829*$t);
   $Y2 += 0.00000000001*cos(3.54409288852 + 51123.7817174435*$t);
   $Y2 += 0.00000000001*cos(2.04668299759 + 51645.87150057029*$t);
   $Y2 += 0.00000000001*cos(6.13056634943 + 76.50988875911*$t);
   $Y2 += 0.00000000001*cos(5.70833789847 + 106570.61348796709*$t);
   $Y2 += 0.00000000001*cos(1.85475666159 + 25021.16384690069*$t);
   $Y2 += 0.00000000001*cos(3.08169426022 + 15874.37377787969*$t);
   $Y2 += 0.00000000001*cos(5.01616234440 + 86144.04239554508*$t);
   $Y2 += 0.00000000001*cos(1.82601661336 + 52644.0150909863*$t);
   $Y2 += 0.00000000001*cos(4.40416127197 + 26094.77551795771*$t);
   $Y2 += 0.00000000001*cos(1.26427504022 + 26081.51840015769*$t);
   $Y2 += 0.00000000001*cos(1.17801372459 + 6.86972951729*$t);
   $Y2 += 0.00000000001*cos(5.00258165716 + 16703.3059509825*$t);
   $Y2 += 0.00000000001*cos(3.75732229340 + 35472.9879671329*$t);
   $Y2 += 0.00000000001*cos(5.03560870343 + 42154.2128205325*$t);
   $Y2 += 0.00000000001*cos(2.82069930264 + 77410.75686045409*$t);
   $Y2 += 0.00000000001*cos(1.94570699226 + 103925.25819290428*$t);
   $Y2 += 0.00000000001*cos(0.43527786277 + 52329.8289111167*$t);
   $Y2 += 0.00000000001*cos(1.53490250185 + 6681.46867088311*$t);
   $Y2 += 0.00000000001*cos(5.27958367842 + 6770.9544187291*$t);
   $Y2 += 0.00000000001*cos(3.09122313934 + 955.8435590921*$t);
   $Y2 += 0.00000000001*cos(5.47023024103 + 51109.55462344189*$t);
   $Y2 += 0.00000000001*cos(2.82652928734 + 29530.2342690561*$t);
   $Y2 += 0.00000000001*cos(3.34999395144 + 26067.9895631909*$t);
   $Y2 += 0.00000000001*cos(5.60842191407 + 25447.76203777669*$t);
   $Y2 += 0.00000000001*cos(1.24359184874 + 52705.25343075949*$t);
   $Y2 += 0.00000000001*cos(2.58826883392 + 33967.74847742969*$t);
   $Y2 += 0.00000000001*cos(1.58743932556 + 25035.39094090229*$t);
   $Y2 += 0.00000000001*cos(1.49321197683 + 26107.32908499049*$t);
   $Y2 += 0.00000000001*cos(5.64817713358 + 23969.3830986793*$t);
   $Y2 += 0.00000000001*cos(4.09696575895 + 19316.9487228451*$t);
   $Y2 += 0.00000000001*cos(0.38657846461 + 26091.5406594487*$t);
   $Y2 += 0.00000000001*cos(3.96518956917 + 53131.16220727349*$t);
   $Y2 += 0.00000000001*cos(1.17035557979 + 95247.94938966268*$t);
   $Y2 += 0.00000000001*cos(3.79316376096 + 103299.58800059189*$t);
   $Y2 += 0.00000000001*cos(6.16466999792 + 26094.77287109149*$t);
   $Y2 += 0.00000000001*cos(1.55660546282 + 102233.09252340188*$t);
   $Y2 += 0.00000000001*cos(3.72893918712 + 25668.17468021549*$t);
   $Y2 += 0.00000000000*cos(1.31187403896 + 12565.9078824993*$t);
   $Y2 += 0.00000000000*cos(0.36788997270 + 54394.31956985169*$t);
   $Y2 += 0.00000000000*cos(1.40783572565 + 111122.56698502589*$t);
   $Y2 += 0.00000000001*cos(1.72210842613 + 12432.2864678813*$t);
   $Y2 += 0.00000000001*cos(2.54599021645 + 115674.52048208468*$t);
   $Y2 += 0.00000000000*cos(3.09980373381 + 25234.46294233869*$t);
   $Y2 += 0.00000000001*cos(2.39000910681 + 25661.06113321469*$t);
   $Y2 += 0.00000000001*cos(3.51239889125 + 39609.41076568209*$t);
   $Y2 += 0.00000000000*cos(6.00302318824 + 45892.48661567349*$t);
   $Y2 += 0.00000000000*cos(3.95111212963 + 77830.24150432929*$t);
   $Y2 += 0.00000000000*cos(4.29974909652 + 116783.89903417809*$t);
   $Y2 += 0.00000000000*cos(2.30803098529 + 28420.8557169627*$t);
   $Y2 += 0.00000000000*cos(3.48757973551 + 103241.99019671988*$t);
   $Y2 += 0.00000000000*cos(5.47063609410 + 52026.4869034973*$t);
   $Y2 += 0.00000000000*cos(3.31063006560 + 19804.5834740993*$t);
   $Y2 += 0.00000000001*cos(4.42730882654 + 26083.7779887327*$t);
   $Y2 += 0.00000000000*cos(0.97906814023 + 53242.5455778219*$t);
   $Y2 += 0.00000000000*cos(5.69544477873 + 1588.82907780029*$t);
   $Y2 += 0.00000000000*cos(5.67543121769 + 51219.96272405629*$t);
   $Y2 += 0.00000000000*cos(3.92348521122 + 157637.04121902208*$t);
   $Y2 += 0.00000000000*cos(3.88298321255 + 72602.13355808688*$t);
   $Y2 += 0.00000000000*cos(6.24610291970 + 83924.79765639128*$t);
   $Y2 += 0.00000000000*cos(2.99531203217 + 130420.08976445469*$t);
   $Y2 += 0.00000000000*cos(3.68023238451 + 25668.6623151825*$t);
   $Y2 += 0.00000000000*cos(5.98068946715 + 3340.8562441833*$t);
   $Y2 += 0.00000000000*cos(2.59880667110 + 77204.08367704989*$t);
   $Y2 += 0.00000000000*cos(2.26912005110 + 207594.09039798369*$t);
   $Y2 += 0.00000000000*cos(0.62064943415 + 419.72846135871*$t);
   $Y2 += 0.00000000000*cos(0.91508299271 + 32370.73517408209*$t);
   $Y2 += 0.00000000000*cos(0.42512526285 + 78903.8505285201*$t);
   $Y2 += 0.00000000000*cos(1.63629020299 + 44936.88687406489*$t);
   $Y2 += 0.00000000000*cos(6.12146886950 + 286966.69073983253*$t);
   return $Y2*$t*$t;
}



   function Mercury_Y3($t) // 135 terms of order 3
{
   $Y3  = 0.00000180660*cos(6.24763771127 + 0.2438174835*$t);
   $Y3 += 0.00000056109*cos(6.21607723411 + 52176.0501006319*$t);
   $Y3 += 0.00000023909*cos(3.52205756332 + 78263.95324220609*$t);
   $Y3 += 0.00000011317*cos(0.65434522049 + 104351.85638378029*$t);
   $Y3 += 0.00000011202*cos(4.63472742214 + 26088.1469590577*$t);
   $Y3 += 0.00000007321*cos(3.06113980531 + 26087.65932409069*$t);
   $Y3 += 0.00000005011*cos(3.87936295964 + 130439.75952535449*$t);
   $Y3 += 0.00000002025*cos(0.73521591485 + 156527.66266692868*$t);
   $Y3 += 0.00000001152*cos(1.69823188077 + 52175.56246566489*$t);
   $Y3 += 0.00000000763*cos(3.83589288112 + 182615.56580850288*$t);
   $Y3 += 0.00000000387*cos(4.87852090068 + 78263.46560723908*$t);
   $Y3 += 0.00000000274*cos(0.63429416338 + 208703.46895007708*$t);
   $Y3 += 0.00000000112*cos(1.64880744224 + 104351.36874881329*$t);
   $Y3 += 0.00000000095*cos(3.70512425843 + 234791.37209165128*$t);
   $Y3 += 0.00000000037*cos(2.93820472611 + 24978.7684069643*$t);
   $Y3 += 0.00000000034*cos(2.70920545196 + 27197.5255111511*$t);
   $Y3 += 0.00000000032*cos(0.48595621544 + 260879.27523322542*$t);
   $Y3 += 0.00000000024*cos(0.88877136179 + 51066.6715485385*$t);
   $Y3 += 0.00000000032*cos(4.71795792013 + 130439.27189038748*$t);
   $Y3 += 0.00000000013*cos(4.14981392057 + 77154.57469011268*$t);
   $Y3 += 0.00000000010*cos(3.54374573861 + 286967.17837479962*$t);
   $Y3 += 0.00000000013*cos(4.83147497313 + 53285.4286527253*$t);
   $Y3 += 0.00000000010*cos(1.53331553777 + 156527.17503196168*$t);
   $Y3 += 0.00000000006*cos(0.11733989890 + 1109.1347346099*$t);
   $Y3 += 0.00000000007*cos(1.42096187453 + 1059.6257476727*$t);
   $Y3 += 0.00000000006*cos(0.36688147081 + 20426.8149099055*$t);
   $Y3 += 0.00000000006*cos(0.99179814069 + 103242.47783168689*$t);
   $Y3 += 0.00000000004*cos(5.28779415998 + 31749.4790082099*$t);
   $Y3 += 0.00000000004*cos(1.32226409901 + 79373.33179429949*$t);
   $Y3 += 0.00000000003*cos(0.32143664919 + 313055.08151637383*$t);
   $Y3 += 0.00000000003*cos(5.50927594249 + 4552.1973145423*$t);
   $Y3 += 0.00000000003*cos(4.66013795345 + 182615.07817353586*$t);
   $Y3 += 0.00000000003*cos(4.10710790486 + 46514.7180514797*$t);
   $Y3 += 0.00000000003*cos(0.25274361593 + 21536.1934619989*$t);
   $Y3 += 0.00000000002*cos(4.79995023814 + 27147.5288892469*$t);
   $Y3 += 0.00000000002*cos(2.39000042999 + 1589.3167127673*$t);
   $Y3 += 0.00000000002*cos(5.16055835758 + 25132.5472174491*$t);
   $Y3 += 0.00000000002*cos(4.23604489079 + 14765.48286075331*$t);
   $Y3 += 0.00000000002*cos(4.24590446693 + 51116.6681704427*$t);
   $Y3 += 0.00000000002*cos(4.07863052508 + 129330.38097326108*$t);
   $Y3 += 0.00000000002*cos(6.04806962418 + 27197.03787618409*$t);
   $Y3 += 0.00000000002*cos(1.19379164655 + 26617.8379241523*$t);
   $Y3 += 0.00000000002*cos(0.44240554373 + 27043.7467006663*$t);
   $Y3 += 0.00000000002*cos(4.67526593433 + 25558.4559939631*$t);
   $Y3 += 0.00000000002*cos(1.31409357670 + 72602.62119305388*$t);
   $Y3 += 0.00000000002*cos(0.23323058095 + 26068.47719815791*$t);
   $Y3 += 0.00000000002*cos(5.43511468464 + 26107.81671995749*$t);
   $Y3 += 0.00000000001*cos(3.58800786511 + 26092.0282944157*$t);
   $Y3 += 0.00000000001*cos(2.08042717023 + 26084.2656236997*$t);
   $Y3 += 0.00000000001*cos(5.04284012325 + 24978.28077199729*$t);
   $Y3 += 0.00000000001*cos(1.47519959792 + 37410.8110573621*$t);
   $Y3 += 0.00000000001*cos(5.02608519942 + 30640.1004561165*$t);
   $Y3 += 0.00000000001*cos(5.10218274945 + 52705.7410657265*$t);
   $Y3 += 0.00000000001*cos(3.44712662114 + 51220.4503590233*$t);
   $Y3 += 0.00000000001*cos(0.20321095099 + 47624.0966035731*$t);
   $Y3 += 0.00000000001*cos(1.50125338315 + 208702.98131511007*$t);
   $Y3 += 0.00000000001*cos(1.69903209468 + 9104.1508116011*$t);
   $Y3 += 0.00000000001*cos(4.53006298092 + 98690.52433462809*$t);
   $Y3 += 0.00000000001*cos(4.29961422700 + 955.3559241251*$t);
   $Y3 += 0.00000000001*cos(1.16934270913 + 1109.6223695769*$t);
   $Y3 += 0.00000000001*cos(0.43847536572 + 19317.43635781211*$t);
   $Y3 += 0.00000000001*cos(3.28791774568 + 50586.97720534809*$t);
   $Y3 += 0.00000000001*cos(3.80818275289 + 12566.3955174663*$t);
   $Y3 += 0.00000000001*cos(1.94170233989 + 40853.3860023275*$t);
   $Y3 += 0.00000000001*cos(2.16988291625 + 53235.43203082109*$t);
   $Y3 += 0.00000000001*cos(3.36850892547 + 339142.98465794802*$t);
   $Y3 += 0.00000000001*cos(2.07799856696 + 426.8420083595*$t);
   $Y3 += 0.00000000001*cos(1.13583428934 + 49957.2929964451*$t);
   $Y3 += 0.00000000001*cos(2.48860782643 + 26095.2605060585*$t);
   $Y3 += 0.00000000001*cos(3.22444138778 + 26081.03341205689*$t);
   $Y3 += 0.00000000001*cos(2.99647624764 + 23869.3898548709*$t);
   $Y3 += 0.00000000001*cos(0.34236528707 + 77308.35350059748*$t);
   $Y3 += 0.00000000001*cos(4.24503082495 + 105461.23493587368*$t);
   $Y3 += 0.00000000001*cos(0.10746803930 + 51646.3591355373*$t);
   $Y3 += 0.00000000001*cos(0.85601964696 + 155418.28411483529*$t);
   $Y3 += 0.00000000001*cos(3.18986978065 + 56728.0035976907*$t);
   $Y3 += 0.00000000001*cos(1.94353556293 + 53131.6498422405*$t);
   $Y3 += 0.00000000001*cos(5.49395066730 + 3442.81876244891*$t);
   $Y3 += 0.00000000001*cos(2.08016019343 + 78793.64420730069*$t);
   $Y3 += 0.00000000001*cos(4.54434565803 + 54394.80720481869*$t);
   $Y3 += 0.00000000001*cos(5.42235344143 + 79323.33517239528*$t);
   $Y3 += 0.00000000001*cos(2.66875497262 + 28306.90406324449*$t);
   $Y3 += 0.00000000001*cos(4.76053635454 + 45405.3394993863*$t);
   $Y3 += 0.00000000001*cos(5.22561098934 + 32858.8575603033*$t);
   $Y3 += 0.00000000000*cos(4.27647999319 + 529.44714761109*$t);
   $Y3 += 0.00000000001*cos(0.08839061971 + 25028.7650288685*$t);
   $Y3 += 0.00000000000*cos(1.03862778956 + 25874.8478636197*$t);
   $Y3 += 0.00000000001*cos(0.19202613793 + 22645.57201409229*$t);
   $Y3 += 0.00000000000*cos(0.74532357887 + 640.1411037975*$t);
   $Y3 += 0.00000000000*cos(5.60594877309 + 529.9347825781*$t);
   $Y3 += 0.00000000000*cos(4.64452132771 + 26301.4460544957*$t);
   $Y3 += 0.00000000000*cos(2.15255497625 + 83925.28529135829*$t);
   $Y3 += 0.00000000000*cos(0.24839982675 + 82815.90673926489*$t);
   $Y3 += 0.00000000001*cos(0.02689519967 + 27677.2198543415*$t);
   $Y3 += 0.00000000000*cos(5.17804795670 + 11322.42028082089*$t);
   $Y3 += 0.00000000000*cos(4.78725393347 + 36301.4325052687*$t);
   $Y3 += 0.00000000000*cos(2.95601248059 + 5661.5758666357*$t);
   $Y3 += 0.00000000000*cos(1.50383537850 + 18849.4713674577*$t);
   $Y3 += 0.00000000000*cos(2.67564423190 + 6770.46678376209*$t);
   $Y3 += 0.00000000000*cos(3.97584628325 + 43072.1431065143*$t);
   $Y3 += 0.00000000000*cos(3.28546823218 + 52156.38033973211*$t);
   $Y3 += 0.00000000000*cos(1.87043752965 + 39609.8984006491*$t);
   $Y3 += 0.00000000000*cos(4.81536972160 + 1052.51220067191*$t);
   $Y3 += 0.00000000000*cos(0.85194708260 + 15874.8614128467*$t);
   $Y3 += 0.00000000000*cos(4.33911068639 + 1066.7392946735*$t);
   $Y3 += 0.00000000000*cos(5.18479261632 + 24499.0740637739*$t);
   $Y3 += 0.00000000000*cos(3.01728960403 + 2119.00767786191*$t);
   $Y3 += 0.00000000000*cos(0.36191373868 + 52179.9314359899*$t);
   $Y3 += 0.00000000000*cos(4.28779427258 + 27043.2590656993*$t);
   $Y3 += 0.00000000000*cos(5.49560028256 + 29530.7219040231*$t);
   $Y3 += 0.00000000000*cos(2.44474876799 + 20426.32727493849*$t);
   $Y3 += 0.00000000000*cos(4.93651363037 + 26514.7451499337*$t);
   $Y3 += 0.00000000000*cos(0.52831113499 + 38654.2986590405*$t);
   $Y3 += 0.00000000000*cos(1.94862504810 + 51066.18391357149*$t);
   $Y3 += 0.00000000000*cos(4.51508383281 + 63498.71419893629*$t);
   $Y3 += 0.00000000000*cos(4.63260338147 + 234790.88445668427*$t);
   $Y3 += 0.00000000000*cos(5.16055252636 + 110013.18843293248*$t);
   $Y3 += 0.00000000000*cos(2.19677657341 + 52195.71986153169*$t);
   $Y3 += 0.00000000000*cos(4.97627187962 + 76674.88034692229*$t);
   $Y3 += 0.00000000000*cos(4.63491468562 + 4551.7096795753*$t);
   $Y3 += 0.00000000000*cos(6.26527903795 + 7.3573644843*$t);
   $Y3 += 0.00000000000*cos(4.12020457586 + 33326.8225506577*$t);
   $Y3 += 0.00000000000*cos(0.12999351127 + 2218.51328670329*$t);
   $Y3 += 0.00000000000*cos(3.64724829310 + 51749.45190975589*$t);
   $Y3 += 0.00000000000*cos(1.65008357433 + 7238.4317741165*$t);
   $Y3 += 0.00000000000*cos(5.13058700728 + 52172.1687652739*$t);
   $Y3 += 0.00000000000*cos(2.61914963849 + 53284.94101775829*$t);
   $Y3 += 0.00000000000*cos(5.02388999987 + 66941.2891439017*$t);
   $Y3 += 0.00000000000*cos(6.01418094422 + 52168.93655363109*$t);
   $Y3 += 0.00000000000*cos(2.01993495347 + 58946.76070187749*$t);
   $Y3 += 0.00000000000*cos(0.98221708355 + 30639.61282114949*$t);
   $Y3 += 0.00000000000*cos(1.52131721681 + 62389.33564684289*$t);
   $Y3 += 0.00000000000*cos(2.07662809015 + 105411.23831396949*$t);
   $Y3 += 0.00000000000*cos(0.99094878754 + 103292.47445359109*$t);
   $Y3 += 0.00000000000*cos(5.07197491918 + 23868.9022199039*$t);
   return $Y3*$t*$t*$t;
}



   function Mercury_Y4($t) // 42 terms of order 4
{
   $Y4  = 0.00000043303*cos(1.13774685024 + 26088.1469590577*$t);
   $Y4 += 0.00000016746*cos(1.28029969371 + 0.2438174835*$t);
   $Y4 += 0.00000005097*cos(4.24955975906 + 52176.0501006319*$t);
   $Y4 += 0.00000001110*cos(1.28336867360 + 78263.95324220609*$t);
   $Y4 += 0.00000000968*cos(4.75917629222 + 26087.65932409069*$t);
   $Y4 += 0.00000000362*cos(4.81899532084 + 104351.85638378029*$t);
   $Y4 += 0.00000000161*cos(1.99262264168 + 130439.75952535449*$t);
   $Y4 += 0.00000000075*cos(5.26842672327 + 156527.66266692868*$t);
   $Y4 += 0.00000000076*cos(1.36013166836 + 52175.56246566489*$t);
   $Y4 += 0.00000000033*cos(2.15747243303 + 182615.56580850288*$t);
   $Y4 += 0.00000000014*cos(3.97795789451 + 78263.46560723908*$t);
   $Y4 += 0.00000000014*cos(5.27638874760 + 208703.46895007708*$t);
   $Y4 += 0.00000000005*cos(2.08976600519 + 234791.37209165128*$t);
   $Y4 += 0.00000000004*cos(0.41935591421 + 104351.36874881329*$t);
   $Y4 += 0.00000000002*cos(1.30411243109 + 24978.7684069643*$t);
   $Y4 += 0.00000000002*cos(4.27909717270 + 27197.5255111511*$t);
   $Y4 += 0.00000000002*cos(5.16384952851 + 260879.27523322542*$t);
   $Y4 += 0.00000000001*cos(3.31456236049 + 130439.27189038748*$t);
   $Y4 += 0.00000000001*cos(1.94555546011 + 286967.17837479962*$t);
   $Y4 += 0.00000000001*cos(4.78035947332 + 51066.6715485385*$t);
   $Y4 += 0.00000000001*cos(6.04727171347 + 1059.6257476727*$t);
   $Y4 += 0.00000000001*cos(4.90982478671 + 20426.8149099055*$t);
   $Y4 += 0.00000000000*cos(0.02105735648 + 156527.17503196168*$t);
   $Y4 += 0.00000000000*cos(1.46282089336 + 53285.4286527253*$t);
   $Y4 += 0.00000000000*cos(1.85776832505 + 1109.1347346099*$t);
   $Y4 += 0.00000000000*cos(0.55166694814 + 31749.4790082099*$t);
   $Y4 += 0.00000000000*cos(2.05256123583 + 77154.57469011268*$t);
   $Y4 += 0.00000000000*cos(3.70596272569 + 4552.1973145423*$t);
   $Y4 += 0.00000000000*cos(5.69177628153 + 51116.6681704427*$t);
   $Y4 += 0.00000000000*cos(4.47226271461 + 5661.0882316687*$t);
   $Y4 += 0.00000000000*cos(1.80113707019 + 21536.1934619989*$t);
   $Y4 += 0.00000000000*cos(3.04937726063 + 182615.07817353586*$t);
   $Y4 += 0.00000000000*cos(1.73584606362 + 46514.7180514797*$t);
   $Y4 += 0.00000000000*cos(0.81363243390 + 1589.3167127673*$t);
   $Y4 += 0.00000000000*cos(2.86001163068 + 27147.5288892469*$t);
   $Y4 += 0.00000000000*cos(3.76188191169 + 25132.5472174491*$t);
   $Y4 += 0.00000000000*cos(5.61217194797 + 26617.8379241523*$t);
   $Y4 += 0.00000000000*cos(2.50285436895 + 14765.48286075331*$t);
   $Y4 += 0.00000000000*cos(0.06217996093 + 25558.4559939631*$t);
   $Y4 += 0.00000000000*cos(1.78288877958 + 27043.7467006663*$t);
   $Y4 += 0.00000000000*cos(3.14225315293 + 529.9347825781*$t);
   $Y4 += 0.00000000000*cos(3.58247145620 + 57837.3821497841*$t);
   return $Y4*$t*$t*$t*$t;
}



   function Mercury_Y5($t) // 16 terms of order 5
{
   $Y5  = 0.00000000414*cos(2.51938027426 + 0.2438174835*$t);
   $Y5 += 0.00000000327*cos(1.26814697300 + 26088.1469590577*$t);
   $Y5 += 0.00000000134*cos(2.94456567084 + 52176.0501006319*$t);
   $Y5 += 0.00000000046*cos(5.94506878756 + 78263.95324220609*$t);
   $Y5 += 0.00000000016*cos(2.88715140580 + 104351.85638378029*$t);
   $Y5 += 0.00000000007*cos(0.85960797708 + 26087.65932409069*$t);
   $Y5 += 0.00000000006*cos(6.26965738190 + 130439.75952535449*$t);
   $Y5 += 0.00000000003*cos(3.37112838196 + 156527.66266692868*$t);
   $Y5 += 0.00000000001*cos(0.36849890320 + 182615.56580850288*$t);
   $Y5 += 0.00000000001*cos(5.73351518627 + 52175.56246566489*$t);
   $Y5 += 0.00000000001*cos(3.53868886531 + 208703.46895007708*$t);
   $Y5 += 0.00000000000*cos(2.56994818453 + 78263.46560723908*$t);
   $Y5 += 0.00000000000*cos(0.33722437006 + 234791.37209165128*$t);
   $Y5 += 0.00000000000*cos(5.49215263350 + 104351.36874881329*$t);
   $Y5 += 0.00000000000*cos(6.01269492359 + 24978.7684069643*$t);
   $Y5 += 0.00000000000*cos(5.68183447366 + 27197.5255111511*$t);
   return $Y5*$t*$t*$t*$t*$t;
}



   function Mercury_Z0($t) // 598 terms of order 0
{
   $Z0  = 0.04607665326*cos(1.99295081967 + 26087.9031415742*$t);
   $Z0 -= 0.00708734365;
   $Z0 += 0.00469171617*cos(5.04215742764 + 52175.8062831484*$t);
   $Z0 += 0.00071626395*cos(1.80894256071 + 78263.70942472259*$t);
   $Z0 += 0.00012957446*cos(4.85922032010 + 104351.61256629678*$t);
   $Z0 += 0.00002575002*cos(1.62646731545 + 130439.51570787099*$t);
   $Z0 += 0.00000543259*cos(4.67698860167 + 156527.41884944518*$t);
   $Z0 += 0.00000119462*cos(1.44437994097 + 182615.32199101939*$t);
   $Z0 += 0.00000079477*cos(4.94442849343 + 27197.2816936676*$t);
   $Z0 += 0.00000082635*cos(2.03127961938 + 24978.5245894808*$t);
   $Z0 += 0.00000044712*cos(0.52659463739 + 1059.3819301892*$t);
   $Z0 += 0.00000043131*cos(5.66922412963 + 20426.571092422*$t);
   $Z0 += 0.00000034677*cos(2.36736170874 + 25028.521211385*$t);
   $Z0 += 0.00000032337*cos(1.29099631099 + 31749.2351907264*$t);
   $Z0 += 0.00000027465*cos(3.57680072404 + 5661.3320491522*$t);
   $Z0 += 0.00000027081*cos(4.49499356963 + 208703.22513259358*$t);
   $Z0 += 0.00000020978*cos(4.46131555233 + 4551.9534970588*$t);
   $Z0 += 0.00000020602*cos(1.12768557199 + 1109.3785520934*$t);
   $Z0 += 0.00000020252*cos(0.11722157720 + 51116.4243529592*$t);
   $Z0 += 0.00000023561*cos(1.82361021215 + 53285.1848352418*$t);
   $Z0 += 0.00000014150*cos(4.42485698284 + 57837.1383323006*$t);
   $Z0 += 0.00000016632*cos(5.09118864825 + 51066.427731055*$t);
   $Z0 += 0.00000011609*cos(3.48812792468 + 21535.9496445154*$t);
   $Z0 += 0.00000009897*cos(2.80940292918 + 27043.5028831828*$t);
   $Z0 += 0.00000008380*cos(2.46820093103 + 46514.4742339962*$t);
   $Z0 += 0.00000007950*cos(0.44767912341 + 10213.285546211*$t);
   $Z0 += 0.00000009234*cos(3.80445817621 + 529.6909650946*$t);
   $Z0 += 0.00000007834*cos(4.08006285427 + 25132.3033999656*$t);
   $Z0 += 0.00000007169*cos(2.43861625280 + 47623.8527860896*$t);
   $Z0 += 0.00000006283*cos(1.26244592145 + 234791.12827416777*$t);
   $Z0 += 0.00000006189*cos(3.24598311063 + 77204.32749453338*$t);
   $Z0 += 0.00000006312*cos(3.60156222051 + 27147.28507176339*$t);
   $Z0 += 0.00000005874*cos(3.18517530495 + 14765.2390432698*$t);
   $Z0 += 0.00000005464*cos(0.18067146403 + 13521.7514415914*$t);
   $Z0 += 0.00000005844*cos(4.88684915394 + 79373.08797681599*$t);
   $Z0 += 0.00000005207*cos(1.04360609206 + 426.598190876*$t);
   $Z0 += 0.00000005084*cos(2.88826076001 + 12566.1516999828*$t);
   $Z0 += 0.00000004862*cos(1.05211425705 + 1589.0728952838*$t);
   $Z0 += 0.00000005778*cos(4.74518603718 + 30639.856638633*$t);
   $Z0 += 0.00000004470*cos(5.94243776120 + 11322.6640983044*$t);
   $Z0 += 0.00000004084*cos(6.10730721947 + 41962.5207369374*$t);
   $Z0 += 0.00000003921*cos(1.19592374632 + 83925.04147387479*$t);
   $Z0 += 0.00000003933*cos(1.60795544806 + 25661.3049506982*$t);
   $Z0 += 0.00000004046*cos(3.08469176646 + 51646.11531805379*$t);
   $Z0 += 0.00000003739*cos(2.04428825509 + 15874.6175953632*$t);
   $Z0 += 0.00000003728*cos(1.74332154184 + 24498.8302462904*$t);
   $Z0 += 0.00000003797*cos(1.86233694539 + 77154.33087262919*$t);
   $Z0 += 0.00000003417*cos(4.10830949198 + 39609.6545831656*$t);
   $Z0 += 0.00000003691*cos(3.50893180252 + 37410.5672398786*$t);
   $Z0 += 0.00000002644*cos(5.60089521868 + 73711.75592766379*$t);
   $Z0 += 0.00000003230*cos(5.39015797984 + 955.5997416086*$t);
   $Z0 += 0.00000002653*cos(5.52156950287 + 26617.5941066688*$t);
   $Z0 += 0.00000002526*cos(5.49112478707 + 6283.0758499914*$t);
   $Z0 += 0.00000003017*cos(6.08863660917 + 53131.406024757*$t);
   $Z0 += 0.00000003020*cos(5.46768108443 + 25558.2121764796*$t);
   $Z0 += 0.00000002169*cos(5.85843688724 + 50586.73338786459*$t);
   $Z0 += 0.00000002217*cos(5.81930894103 + 51749.20809227239*$t);
   $Z0 += 0.00000002036*cos(2.79411907992 + 7238.67559160*$t);
   $Z0 += 0.00000001904*cos(0.02429449494 + 33326.5787331742*$t);
   $Z0 += 0.00000001841*cos(3.31987542265 + 36301.18868778519*$t);
   $Z0 += 0.00000001693*cos(0.04022611885 + 103292.23063610759*$t);
   $Z0 += 0.00000001814*cos(0.58414302882 + 63498.47038145279*$t);
   $Z0 += 0.00000001910*cos(5.54302892735 + 72602.37737557039*$t);
   $Z0 += 0.00000001560*cos(0.96215455240 + 65697.55772473979*$t);
   $Z0 += 0.00000001605*cos(0.84854708960 + 51220.20654153979*$t);
   $Z0 += 0.00000001595*cos(5.65216619091 + 26068.2333806744*$t);
   $Z0 += 0.00000001487*cos(1.66950376778 + 105460.99111839019*$t);
   $Z0 += 0.00000001595*cos(1.47238921630 + 26107.57290247399*$t);
   $Z0 += 0.00000001485*cos(4.31307667056 + 260879.03141574195*$t);
   $Z0 += 0.00000001344*cos(3.05188579262 + 68050.42387851159*$t);
   $Z0 += 0.00000001601*cos(0.69333064931 + 9103.9069941176*$t);
   $Z0 += 0.00000001224*cos(3.56543079781 + 19804.8272915828*$t);
   $Z0 += 0.00000001258*cos(2.62817126488 + 52705.49724824299*$t);
   $Z0 += 0.00000001386*cos(0.42909271065 + 53235.18821333759*$t);
   $Z0 += 0.00000001392*cos(0.41896396083 + 18849.2275499742*$t);
   $Z0 += 0.00000001104*cos(1.37365365216 + 45892.73043315699*$t);
   $Z0 += 0.00000001191*cos(5.76715631957 + 19317.1925403286*$t);
   $Z0 += 0.00000001218*cos(6.16642870453 + 77734.01845962799*$t);
   $Z0 += 0.00000000991*cos(0.77508695125 + 32858.61374281979*$t);
   $Z0 += 0.00000001039*cos(4.25003847002 + 110012.94461544899*$t);
   $Z0 += 0.00000001155*cos(1.83291524170 + 16983.9961474566*$t);
   $Z0 += 0.00000000913*cos(4.91386835375 + 103242.23401420339*$t);
   $Z0 += 0.00000001096*cos(2.77782845562 + 26091.7844769322*$t);
   $Z0 += 0.00000000843*cos(2.10283893013 + 23869.1460373874*$t);
   $Z0 += 0.00000001147*cos(1.40069226758 + 56727.7597802072*$t);
   $Z0 += 0.00000001096*cos(4.34969785012 + 26084.0218062162*$t);
   $Z0 += 0.00000000888*cos(5.96405172933 + 38654.05484155699*$t);
   $Z0 += 0.00000000798*cos(5.90979724117 + 43071.8992890308*$t);
   $Z0 += 0.00000000894*cos(6.20891949330 + 40853.142184844*$t);
   $Z0 += 0.00000000751*cos(2.38280961577 + 99799.65906923798*$t);
   $Z0 += 0.00000000858*cos(3.48194203031 + 59414.4818747484*$t);
   $Z0 += 0.00000000903*cos(4.51539427543 + 3442.5749449654*$t);
   $Z0 += 0.00000000816*cos(4.68549846425 + 28306.66024576099*$t);
   $Z0 += 0.00000000739*cos(4.03898100455 + 26514.5013324502*$t);
   $Z0 += 0.00000000874*cos(3.33462533468 + 22645.32819660879*$t);
   $Z0 += 0.00000000685*cos(2.68994541603 + 76674.63652943878*$t);
   $Z0 += 0.00000000933*cos(3.52008984237 + 6770.7106012456*$t);
   $Z0 += 0.00000000668*cos(2.67447570603 + 77837.11123384659*$t);
   $Z0 += 0.00000000688*cos(4.13735845510 + 27676.976036858*$t);
   $Z0 += 0.00000000643*cos(0.26212576857 + 639.897286314*$t);
   $Z0 += 0.00000000747*cos(2.88413728742 + 79219.30916633119*$t);
   $Z0 += 0.00000000594*cos(4.72204801083 + 29530.4780865396*$t);
   $Z0 += 0.00000000580*cos(0.49131159361 + 62389.09182935939*$t);
   $Z0 += 0.00000000560*cos(0.58980434738 + 26080.78959457339*$t);
   $Z0 += 0.00000000636*cos(5.01277329265 + 7.1135470008*$t);
   $Z0 += 0.00000000664*cos(5.40615713600 + 213.299095438*$t);
   $Z0 += 0.00000000460*cos(2.41008353609 + 25448.00585526019*$t);
   $Z0 += 0.00000000453*cos(4.02646064195 + 91785.46086631398*$t);
   $Z0 += 0.00000000441*cos(3.60120922804 + 89586.37352302698*$t);
   $Z0 += 0.00000000473*cos(3.19147656788 + 25874.6040461362*$t);
   $Z0 += 0.00000000452*cos(3.10205575226 + 129380.13377768178*$t);
   $Z0 += 0.00000000406*cos(1.57776301723 + 2118.7638603784*$t);
   $Z0 += 0.00000000495*cos(4.35094686160 + 58946.51688439399*$t);
   $Z0 += 0.00000000395*cos(0.29737033271 + 25934.1243310894*$t);
   $Z0 += 0.00000000392*cos(0.53546585935 + 26241.681952059*$t);
   $Z0 += 0.00000000382*cos(4.72731471129 + 131548.89425996438*$t);
   $Z0 += 0.00000000382*cos(4.49198737842 + 71980.63357473118*$t);
   $Z0 += 0.00000000368*cos(3.89892172772 + 77308.10968311399*$t);
   $Z0 += 0.00000000351*cos(1.04745682138 + 2218.7571041868*$t);
   $Z0 += 0.00000000356*cos(6.12309999532 + 94138.32702008578*$t);
   $Z0 += 0.00000000458*cos(2.31746139528 + 98690.28051714458*$t);
   $Z0 += 0.00000000327*cos(4.52406643775 + 52195.47604404819*$t);
   $Z0 += 0.00000000393*cos(3.09725258397 + 69159.80243060499*$t);
   $Z0 += 0.00000000310*cos(1.10653094623 + 23969.1392811958*$t);
   $Z0 += 0.00000000356*cos(1.08062840940 + 286966.93455731616*$t);
   $Z0 += 0.00000000345*cos(5.71957868514 + 78793.40038981718*$t);
   $Z0 += 0.00000000293*cos(1.74897333943 + 54394.56338733519*$t);
   $Z0 += 0.00000000275*cos(4.05372155800 + 149.5631971346*$t);
   $Z0 += 0.00000000325*cos(2.41844865673 + 52156.1365222486*$t);
   $Z0 += 0.00000000273*cos(0.34374545554 + 51535.90899683439*$t);
   $Z0 += 0.00000000256*cos(3.57544759221 + 1066.49547719*$t);
   $Z0 += 0.00000000252*cos(2.07789295059 + 48733.23133818299*$t);
   $Z0 += 0.00000000326*cos(3.49941746899 + 79323.09135491178*$t);
   $Z0 += 0.00000000327*cos(2.94651885500 + 103821.92160120218*$t);
   $Z0 += 0.00000000300*cos(3.53612598449 + 26301.2022370122*$t);
   $Z0 += 0.00000000253*cos(0.67254115542 + 1052.2683831884*$t);
   $Z0 += 0.00000000274*cos(1.02128544399 + 136100.84775702319*$t);
   $Z0 += 0.00000000227*cos(1.68291413296 + 129330.13715577759*$t);
   $Z0 += 0.00000000264*cos(4.42527687575 + 82815.66292178139*$t);
   $Z0 += 0.00000000280*cos(3.47711681992 + 44937.1306915484*$t);
   $Z0 += 0.00000000207*cos(5.26698515692 + 25938.3399444396*$t);
   $Z0 += 0.00000000233*cos(1.58956295605 + 26095.016688575*$t);
   $Z0 += 0.00000000211*cos(2.55179422569 + 45405.0956819028*$t);
   $Z0 += 0.00000000223*cos(1.11566460119 + 52171.9249477904*$t);
   $Z0 += 0.00000000195*cos(5.61180266071 + 25021.4076643842*$t);
   $Z0 += 0.00000000204*cos(5.44212122344 + 125887.56221081219*$t);
   $Z0 += 0.00000000193*cos(2.20507758963 + 25035.6347583858*$t);
   $Z0 += 0.00000000189*cos(5.78823648500 + 102762.53967101299*$t);
   $Z0 += 0.00000000213*cos(1.31985764486 + 26555.8681319286*$t);
   $Z0 += 0.00000000191*cos(0.24359410467 + 85502.38501632259*$t);
   $Z0 += 0.00000000195*cos(5.97596513107 + 105307.21230790539*$t);
   $Z0 += 0.00000000218*cos(5.85009667782 + 25619.9381512198*$t);
   $Z0 += 0.00000000181*cos(5.75215140812 + 103925.01437542078*$t);
   $Z0 += 0.00000000190*cos(3.25444605074 + 13655.8604911764*$t);
   $Z0 += 0.00000000177*cos(5.31586012519 + 50057.04242277*$t);
   $Z0 += 0.00000000207*cos(2.75433176036 + 64741.95798313119*$t);
   $Z0 += 0.00000000224*cos(5.82663546732 + 52179.6876185064*$t);
   $Z0 += 0.00000000164*cos(3.65486891804 + 76.2660712756*$t);
   $Z0 += 0.00000000166*cos(5.16708694657 + 49957.0491789616*$t);
   $Z0 += 0.00000000171*cos(1.99193197138 + 26094.53170047421*$t);
   $Z0 += 0.00000000159*cos(2.10978774324 + 32370.9789915656*$t);
   $Z0 += 0.00000000164*cos(5.82981713266 + 12432.0426503978*$t);
   $Z0 += 0.00000000155*cos(5.39879388941 + 6681.2248533996*$t);
   $Z0 += 0.00000000199*cos(3.05926705479 + 66941.04532641819*$t);
   $Z0 += 0.00000000171*cos(5.13520423660 + 26081.27458267419*$t);
   $Z0 += 0.00000000150*cos(0.90885651206 + 52389.1053785864*$t);
   $Z0 += 0.00000000184*cos(1.74487163243 + 51962.5071877104*$t);
   $Z0 += 0.00000000164*cos(0.91431121775 + 52602.4044740244*$t);
   $Z0 += 0.00000000138*cos(5.40226519416 + 26011.6370702986*$t);
   $Z0 += 0.00000000149*cos(2.17444239275 + 52168.69273614759*$t);
   $Z0 += 0.00000000184*cos(1.61828580097 + 20760.4270331914*$t);
   $Z0 += 0.00000000152*cos(0.96120452018 + 53764.8791784322*$t);
   $Z0 += 0.00000000146*cos(5.57207251195 + 58458.88213313979*$t);
   $Z0 += 0.00000000150*cos(3.58807734183 + 88476.99497093359*$t);
   $Z0 += 0.00000000124*cos(1.42633058836 + 55618.3812281138*$t);
   $Z0 += 0.00000000141*cos(1.44033578815 + 5327.4761083828*$t);
   $Z0 += 0.00000000126*cos(0.80278847447 + 117873.36400788819*$t);
   $Z0 += 0.00000000118*cos(3.91167705851 + 19406.6782881746*$t);
   $Z0 += 0.00000000117*cos(4.68118162480 + 52182.9198301492*$t);
   $Z0 += 0.00000000130*cos(3.12963980623 + 38519.945791972*$t);
   $Z0 += 0.00000000123*cos(2.90959272560 + 52026.2430860138*$t);
   $Z0 += 0.00000000114*cos(3.34729061863 + 51109.31080595839*$t);
   $Z0 += 0.00000000109*cos(0.37062017000 + 115674.27666460119*$t);
   $Z0 += 0.00000000108*cos(5.35523619654 + 46848.3301747656*$t);
   $Z0 += 0.00000000120*cos(6.15890498480 + 155468.03691925599*$t);
   $Z0 += 0.00000000105*cos(1.26949313866 + 98068.53671630539*$t);
   $Z0 += 0.00000000124*cos(5.59073645014 + 74821.13447975718*$t);
   $Z0 += 0.00000000113*cos(5.37283602123 + 124778.18365871879*$t);
   $Z0 += 0.00000000101*cos(1.50859657746 + 157636.79740153858*$t);
   $Z0 += 0.00000000112*cos(6.24044765899 + 51123.53789995999*$t);
   $Z0 += 0.00000000089*cos(0.66689996363 + 103396.01282468818*$t);
   $Z0 += 0.00000000088*cos(3.24234443321 + 26727.8004278882*$t);
   $Z0 += 0.00000000083*cos(3.46887356727 + 77623.81213840858*$t);
   $Z0 += 0.00000000082*cos(3.58489505049 + 52329.58509363319*$t);
   $Z0 += 0.00000000082*cos(2.76598729250 + 35191.8101356918*$t);
   $Z0 += 0.00000000092*cos(2.89919284588 + 120226.23016165999*$t);
   $Z0 += 0.00000000081*cos(3.34888362248 + 52022.0274726636*$t);
   $Z0 += 0.00000000080*cos(1.61800826930 + 45494.58142974879*$t);
   $Z0 += 0.00000000087*cos(5.99930110618 + 129909.82474277639*$t);
   $Z0 += 0.00000000075*cos(1.29165560989 + 78283.37918562238*$t);
   $Z0 += 0.00000000097*cos(0.94285631414 + 85034.42002596818*$t);
   $Z0 += 0.00000000075*cos(0.93132779667 + 433.7117378768*$t);
   $Z0 += 0.00000000087*cos(3.29295494560 + 3340.6124266998*$t);
   $Z0 += 0.00000000083*cos(4.56076128644 + 419.4846438752*$t);
   $Z0 += 0.00000000080*cos(0.27627260080 + 105410.99449648599*$t);
   $Z0 += 0.00000000079*cos(5.99645052852 + 95247.70557217918*$t);
   $Z0 += 0.00000000074*cos(0.63932580906 + 64607.84893354619*$t);
   $Z0 += 0.00000000066*cos(4.71152255697 + 80482.46652890938*$t);
   $Z0 += 0.00000000073*cos(3.21815072307 + 52099.5402118728*$t);
   $Z0 += 0.00000000066*cos(2.98683203711 + 23754.70674870219*$t);
   $Z0 += 0.00000000091*cos(2.50052149852 + 104881.30353139139*$t);
   $Z0 += 0.00000000062*cos(5.73156315784 + 853.196381752*$t);
   $Z0 += 0.00000000086*cos(4.13151425024 + 313054.83769889036*$t);
   $Z0 += 0.00000000064*cos(1.17129038283 + 108903.56606335558*$t);
   $Z0 += 0.00000000059*cos(0.79111914848 + 7994.5284420242*$t);
   $Z0 += 0.00000000065*cos(0.25068571900 + 71025.03383312259*$t);
   $Z0 += 0.00000000057*cos(6.23642054033 + 1692.1656695024*$t);
   $Z0 += 0.00000000057*cos(4.67121384993 + 28206.6670019526*$t);
   $Z0 += 0.00000000059*cos(4.92822158961 + 78050.41032928458*$t);
   $Z0 += 0.00000000056*cos(2.09215892689 + 76144.94556434419*$t);
   $Z0 += 0.00000000056*cos(1.73304008242 + 25654.19140369739*$t);
   $Z0 += 0.00000000057*cos(4.73385475413 + 155418.04029735178*$t);
   $Z0 += 0.00000000056*cos(5.88720785632 + 26037.90651966999*$t);
   $Z0 += 0.00000000074*cos(5.46823409002 + 78244.0396638228*$t);
   $Z0 += 0.00000000061*cos(4.50066150046 + 25668.418497699*$t);
   $Z0 += 0.00000000071*cos(4.06855895670 + 162188.75089859738*$t);
   $Z0 += 0.00000000052*cos(2.74630211595 + 131395.11544947959*$t);
   $Z0 += 0.00000000055*cos(1.23376269855 + 26137.8997634784*$t);
   $Z0 += 0.00000000051*cos(4.16406216247 + 78259.82808936459*$t);
   $Z0 += 0.00000000048*cos(5.64584502444 + 71492.99882347698*$t);
   $Z0 += 0.00000000056*cos(2.21556357721 + 151975.46535238638*$t);
   $Z0 += 0.00000000052*cos(2.58403290426 + 128850.44281258718*$t);
   $Z0 += 0.00000000048*cos(2.53427528158 + 130012.91751699499*$t);
   $Z0 += 0.00000000045*cos(3.30208209778 + 111590.28815789679*$t);
   $Z0 += 0.00000000051*cos(2.59172476683 + 78267.59076008058*$t);
   $Z0 += 0.00000000047*cos(1.65481269433 + 18093.37469954999*$t);
   $Z0 += 0.00000000045*cos(4.61275436981 + 2333.196392872*$t);
   $Z0 += 0.00000000043*cos(4.08074925073 + 1596.1864422846*$t);
   $Z0 += 0.00000000042*cos(2.94276331302 + 24395.7374720718*$t);
   $Z0 += 0.00000000042*cos(4.02117925826 + 78477.00852016058*$t);
   $Z0 += 0.00000000056*cos(4.77237421864 + 28421.0995344462*$t);
   $Z0 += 0.00000000047*cos(6.11377086686 + 93028.94846799239*$t);
   $Z0 += 0.00000000040*cos(3.23768263734 + 25234.70675982219*$t);
   $Z0 += 0.00000000045*cos(5.03235979949 + 78256.59587772179*$t);
   $Z0 += 0.00000000053*cos(6.07709485433 + 10021.8372800994*$t);
   $Z0 += 0.00000000050*cos(5.81013134251 + 90829.86112470538*$t);
   $Z0 += 0.00000000038*cos(1.96485604602 + 632.7837393132*$t);
   $Z0 += 0.00000000051*cos(3.90465378478 + 74.7815985673*$t);
   $Z0 += 0.00000000036*cos(6.02764042024 + 78114.14622758799*$t);
   $Z0 += 0.00000000035*cos(0.19483010119 + 77197.21394753258*$t);
   $Z0 += 0.00000000038*cos(0.85372328004 + 26237.46633870879*$t);
   $Z0 += 0.00000000036*cos(3.61160802929 + 48835.19385644859*$t);
   $Z0 += 0.00000000036*cos(0.36428632576 + 27154.3986187642*$t);
   $Z0 += 0.00000000037*cos(2.58618864774 + 51707.84129279399*$t);
   $Z0 += 0.00000000047*cos(3.11077813997 + 16066.0658614748*$t);
   $Z0 += 0.00000000034*cos(5.90024410478 + 103.0927742186*$t);
   $Z0 += 0.00000000035*cos(1.89948168639 + 52169.17772424839*$t);
   $Z0 += 0.00000000035*cos(3.09037355598 + 77211.44104153418*$t);
   $Z0 += 0.00000000035*cos(5.86095139046 + 18207.81398823521*$t);
   $Z0 += 0.00000000038*cos(1.94104879109 + 76044.9523205358*$t);
   $Z0 += 0.00000000032*cos(1.45277855585 + 78270.82297172339*$t);
   $Z0 += 0.00000000044*cos(4.36195482299 + 52643.7712735028*$t);
   $Z0 += 0.00000000043*cos(0.13191769323 + 1581.959348283*$t);
   $Z0 += 0.00000000032*cos(4.70833495807 + 71582.48457132299*$t);
   $Z0 += 0.00000000037*cos(3.75248447021 + 27140.17152476259*$t);
   $Z0 += 0.00000000042*cos(2.34341217670 + 84546.78527471398*$t);
   $Z0 += 0.00000000031*cos(2.09939558148 + 2648.454825473*$t);
   $Z0 += 0.00000000033*cos(4.22087782315 + 111122.32316754239*$t);
   $Z0 += 0.00000000032*cos(5.93849066304 + 51742.09454527159*$t);
   $Z0 += 0.00000000037*cos(2.22778724531 + 25973.46385288896*$t);
   $Z0 += 0.00000000031*cos(0.06282672228 + 536.8045120954*$t);
   $Z0 += 0.00000000030*cos(0.17325551658 + 224.3447957019*$t);
   $Z0 += 0.00000000030*cos(4.46570213013 + 81706.28436968799*$t);
   $Z0 += 0.00000000029*cos(4.99680928394 + 24491.71669928959*$t);
   $Z0 += 0.00000000035*cos(5.04036468235 + 52182.4348420484*$t);
   $Z0 += 0.00000000033*cos(4.30956969733 + 14477.35118320*$t);
   $Z0 += 0.00000000039*cos(3.99296635763 + 78690.30761559859*$t);
   $Z0 += 0.00000000028*cos(3.41723065188 + 7880.08915333899*$t);
   $Z0 += 0.00000000039*cos(0.38970280197 + 114564.89811250778*$t);
   $Z0 += 0.00000000033*cos(3.85407359005 + 143961.26714946238*$t);
   $Z0 += 0.00000000029*cos(2.14304937049 + 150866.08680029298*$t);
   $Z0 += 0.00000000028*cos(4.95187621332 + 956.2891559706*$t);
   $Z0 += 0.00000000036*cos(4.89757043872 + 26202.34243025941*$t);
   $Z0 += 0.00000000030*cos(0.55025093927 + 33967.99229491319*$t);
   $Z0 += 0.00000000028*cos(1.52043005918 + 40565.2543247742*$t);
   $Z0 += 0.00000000027*cos(5.29884427476 + 1162.4747044078*$t);
   $Z0 += 0.00000000027*cos(3.44440418351 + 141762.17980617538*$t);
   $Z0 += 0.00000000035*cos(2.50821332798 + 72936.23331633979*$t);
   $Z0 += 0.00000000028*cos(0.83354394743 + 42153.969003049*$t);
   $Z0 += 0.00000000036*cos(4.02142487381 + 79852.78232000639*$t);
   $Z0 += 0.00000000026*cos(1.15352991801 + 51322.60990139639*$t);
   $Z0 += 0.00000000026*cos(0.13577439886 + 22747.2907148744*$t);
   $Z0 += 0.00000000027*cos(4.82379477303 + 110.2063212194*$t);
   $Z0 += 0.00000000025*cos(0.71097564554 + 50483.640613646*$t);
   $Z0 += 0.00000000024*cos(2.92248010137 + 121335.60871375339*$t);
   $Z0 += 0.00000000026*cos(4.56151347419 + 183724.70054311278*$t);
   $Z0 += 0.00000000024*cos(2.16545180970 + 32769.1279949738*$t);
   $Z0 += 0.00000000031*cos(2.33336639968 + 51756.3216392732*$t);
   $Z0 += 0.00000000032*cos(2.93016714121 + 181555.94006083018*$t);
   $Z0 += 0.00000000027*cos(4.32935860229 + 124156.43985787958*$t);
   $Z0 += 0.00000000023*cos(6.16949223424 + 39743.7636327506*$t);
   $Z0 += 0.00000000023*cos(2.77333133719 + 155997.72788435058*$t);
   $Z0 += 0.00000000022*cos(2.90047805068 + 25863.55834587229*$t);
   $Z0 += 0.00000000023*cos(0.26296960044 + 103711.71527998279*$t);
   $Z0 += 0.00000000022*cos(0.49237419293 + 23439.44831610119*$t);
   $Z0 += 0.00000000027*cos(2.77269830479 + 24505.94379329119*$t);
   $Z0 += 0.00000000022*cos(0.39938514656 + 26164.1692128498*$t);
   $Z0 += 0.00000000022*cos(0.07820939757 + 78187.44335344699*$t);
   $Z0 += 0.00000000022*cos(3.71330627735 + 129483.91596626239*$t);
   $Z0 += 0.00000000023*cos(5.63887355161 + 522.5774180938*$t);
   $Z0 += 0.00000000021*cos(3.36821177399 + 1375.7737998458*$t);
   $Z0 += 0.00000000029*cos(5.62929042847 + 44181.27784112419*$t);
   $Z0 += 0.00000000022*cos(4.35422573947 + 26720.68688088739*$t);
   $Z0 += 0.00000000021*cos(5.12497732226 + 11610.5519583742*$t);
   $Z0 += 0.00000000025*cos(5.36835518856 + 26013.1215430069*$t);
   $Z0 += 0.00000000024*cos(5.55813303813 + 130969.20667296558*$t);
   $Z0 += 0.00000000020*cos(4.79424075731 + 25780.3455206046*$t);
   $Z0 += 0.00000000020*cos(3.82740705898 + 24925.4284371664*$t);
   $Z0 += 0.00000000020*cos(4.26878389727 + 25131.61398560359*$t);
   $Z0 += 0.00000000021*cos(5.10505150441 + 25977.69682035479*$t);
   $Z0 += 0.00000000020*cos(3.32855955978 + 131498.89763806018*$t);
   $Z0 += 0.00000000020*cos(1.29805086237 + 467.9649903544*$t);
   $Z0 += 0.00000000023*cos(2.06377601353 + 100909.03762133139*$t);
   $Z0 += 0.00000000027*cos(4.46165349883 + 316.3918696566*$t);
   $Z0 += 0.00000000019*cos(2.33132135274 + 26395.46076254379*$t);
   $Z0 += 0.00000000019*cos(0.35229627877 + 78417.48823520739*$t);
   $Z0 += 0.00000000019*cos(2.86176933014 + 50579.61984086379*$t);
   $Z0 += 0.00000000018*cos(4.33826105585 + 104371.28232719658*$t);
   $Z0 += 0.00000000022*cos(1.69381062404 + 106570.36967048359*$t);
   $Z0 += 0.00000000021*cos(0.89581658307 + 339142.74084046457*$t);
   $Z0 += 0.00000000025*cos(5.95532312645 + 146314.13330323418*$t);
   $Z0 += 0.00000000020*cos(0.11725114676 + 52815.7035694624*$t);
   $Z0 += 0.00000000018*cos(1.66756826028 + 23888.81579828719*$t);
   $Z0 += 0.00000000022*cos(4.37150668135 + 1478.8665740644*$t);
   $Z0 += 0.00000000017*cos(4.39063105551 + 60055.89543648739*$t);
   $Z0 += 0.00000000017*cos(3.17668084118 + 70269.18098269838*$t);
   $Z0 += 0.00000000017*cos(1.74029229152 + 104138.31347085879*$t);
   $Z0 += 0.00000000017*cos(5.24411288462 + 102232.84870591838*$t);
   $Z0 += 0.00000000019*cos(0.11691865573 + 78109.93061423779*$t);
   $Z0 += 0.00000000017*cos(3.28697052824 + 25565.3257234804*$t);
   $Z0 += 0.00000000019*cos(0.84289230962 + 188276.65404017158*$t);
   $Z0 += 0.00000000018*cos(0.47252163890 + 27999.1026247914*$t);
   $Z0 += 0.00000000016*cos(2.81320924670 + 52101.02468458109*$t);
   $Z0 += 0.00000000016*cos(4.20710393311 + 134991.46920492979*$t);
   $Z0 += 0.00000000019*cos(0.54267379272 + 50593.84693486539*$t);
   $Z0 += 0.00000000015*cos(5.78613478653 + 24712.1293417284*$t);
   $Z0 += 0.00000000018*cos(2.23136091809 + 104331.94280539699*$t);
   $Z0 += 0.00000000016*cos(3.30957717580 + 97112.93697469679*$t);
   $Z0 += 0.00000000016*cos(1.02655918243 + 26162.6847401415*$t);
   $Z0 += 0.00000000015*cos(0.51501957594 + 13362.4497067992*$t);
   $Z0 += 0.00000000015*cos(5.44457668379 + 28286.9904848612*$t);
   $Z0 += 0.00000000015*cos(3.33697839260 + 206.1855484372*$t);
   $Z0 += 0.00000000015*cos(3.09319808895 + 25984.8103673556*$t);
   $Z0 += 0.00000000019*cos(5.95214817336 + 29428.515568274*$t);
   $Z0 += 0.00000000017*cos(1.60411390680 + 26507.38778544939*$t);
   $Z0 += 0.00000000016*cos(3.62113927445 + 26610.48055966799*$t);
   $Z0 += 0.00000000018*cos(4.55509949626 + 51543.0225438352*$t);
   $Z0 += 0.00000000016*cos(4.95762784074 + 25771.5112719176*$t);
   $Z0 += 0.00000000014*cos(0.15853597470 + 647.0108333148*$t);
   $Z0 += 0.00000000016*cos(3.53658275362 + 25551.09862947879*$t);
   $Z0 += 0.00000000013*cos(2.28786159405 + 12725.453434775*$t);
   $Z0 += 0.00000000016*cos(3.05451392033 + 26190.99591579279*$t);
   $Z0 += 0.00000000013*cos(5.81922389967 + 157483.01859105378*$t);
   $Z0 += 0.00000000013*cos(4.03160498458 + 24356.7807886416*$t);
   $Z0 += 0.00000000013*cos(4.49878848868 + 29416.03879785439*$t);
   $Z0 += 0.00000000016*cos(1.14380482430 + 26404.2950112308*$t);
   $Z0 += 0.00000000016*cos(4.77704247519 + 24609.0365675098*$t);
   $Z0 += 0.00000000013*cos(1.33944288754 + 52808.59002246159*$t);
   $Z0 += 0.00000000015*cos(1.50513017421 + 181505.94343892598*$t);
   $Z0 += 0.00000000012*cos(0.92918175452 + 104347.73123093879*$t);
   $Z0 += 0.00000000014*cos(2.44451494167 + 26421.7590823436*$t);
   $Z0 += 0.00000000012*cos(1.66798607577 + 51013.33157874059*$t);
   $Z0 += 0.00000000012*cos(4.24892890325 + 52225.8029050526*$t);
   $Z0 += 0.00000000014*cos(4.68967753702 + 25754.0472008048*$t);
   $Z0 += 0.00000000013*cos(0.63063491269 + 51639.00177105299*$t);
   $Z0 += 0.00000000013*cos(0.51980975221 + 51951.46148744649*$t);
   $Z0 += 0.00000000013*cos(0.11999669275 + 26294.08869001139*$t);
   $Z0 += 0.00000000012*cos(5.63990253843 + 104355.49390165479*$t);
   $Z0 += 0.00000000013*cos(0.37271479520 + 74923.09699802278*$t);
   $Z0 += 0.00000000015*cos(2.19958896111 + 22759.76748529401*$t);
   $Z0 += 0.00000000013*cos(1.47842980598 + 54294.57014352679*$t);
   $Z0 += 0.00000000012*cos(3.10508217703 + 27819.0254945068*$t);
   $Z0 += 0.00000000011*cos(2.42209220629 + 97580.90196505119*$t);
   $Z0 += 0.00000000013*cos(2.58513127775 + 116917.76426627958*$t);
   $Z0 += 0.00000000011*cos(4.76885794954 + 49527.35145767539*$t);
   $Z0 += 0.00000000011*cos(3.92593978045 + 26521.614879451*$t);
   $Z0 += 0.00000000011*cos(2.89444277066 + 119116.85160956658*$t);
   $Z0 += 0.00000000012*cos(5.58737020094 + 25881.717593137*$t);
   $Z0 += 0.00000000013*cos(5.59537220560 + 156100.82065856918*$t);
   $Z0 += 0.00000000012*cos(2.01299993075 + 51219.51712717779*$t);
   $Z0 += 0.00000000011*cos(0.99253734697 + 25455.119402261*$t);
   $Z0 += 0.00000000014*cos(5.68122574937 + 154938.34595416138*$t);
   $Z0 += 0.00000000015*cos(5.27886514517 + 178063.36849396059*$t);
   $Z0 += 0.00000000014*cos(6.12654802167 + 52278.89905736699*$t);
   $Z0 += 0.00000000011*cos(0.06908636848 + 52072.71350892979*$t);
   $Z0 += 0.00000000011*cos(0.80982228740 + 104564.91166173479*$t);
   $Z0 += 0.00000000013*cos(3.96505351243 + 68241.87214462319*$t);
   $Z0 += 0.00000000011*cos(2.65559265085 + 52125.80966124419*$t);
   $Z0 += 0.00000000012*cos(6.26057108211 + 24176.703658357*$t);
   $Z0 += 0.00000000012*cos(1.73021765373 + 104344.49901929598*$t);
   $Z0 += 0.00000000010*cos(2.77364193598 + 77829.99768684579*$t);
   $Z0 += 0.00000000010*cos(2.83314383566 + 104202.04936916218*$t);
   $Z0 += 0.00000000010*cos(4.05963464737 + 26089.38761428249*$t);
   $Z0 += 0.00000000012*cos(5.40316858194 + 110634.68841628819*$t);
   $Z0 += 0.00000000010*cos(6.17247219966 + 103299.34418310839*$t);
   $Z0 += 0.00000000010*cos(0.02153234343 + 137678.19129947099*$t);
   $Z0 += 0.00000000012*cos(2.25188175428 + 52065.59996192899*$t);
   $Z0 += 0.00000000010*cos(3.11650831039 + 26086.4186688659*$t);
   $Z0 += 0.00000000010*cos(3.27287049435 + 103285.11708910679*$t);
   $Z0 += 0.00000000010*cos(1.38532261128 + 25446.4895798352*$t);
   $Z0 += 0.00000000012*cos(1.52019007531 + 54509.0026760204*$t);
   $Z0 += 0.00000000011*cos(6.21574870463 + 61279.713277266*$t);
   $Z0 += 0.00000000009*cos(0.49710408415 + 2221.856634597*$t);
   $Z0 += 0.00000000009*cos(2.88821691711 + 90695.75207512038*$t);
   $Z0 += 0.00000000011*cos(2.16522629249 + 26624.70765366959*$t);
   $Z0 += 0.00000000009*cos(4.80391342687 + 27311.72098235281*$t);
   $Z0 += 0.00000000009*cos(4.23373474411 + 52250.5878817157*$t);
   $Z0 += 0.00000000009*cos(5.74437259170 + 26729.31670331319*$t);
   $Z0 += 0.00000000010*cos(3.47517485960 + 140652.80125408198*$t);
   $Z0 += 0.00000000010*cos(2.80299508279 + 55516.4187098482*$t);
   $Z0 += 0.00000000013*cos(4.07250716348 + 34282.1784747828*$t);
   $Z0 += 0.00000000009*cos(0.66918270135 + 52698.38370124219*$t);
   $Z0 += 0.00000000010*cos(5.43273059228 + 126996.94076290558*$t);
   $Z0 += 0.00000000009*cos(1.51413989531 + 97670.38771289718*$t);
   $Z0 += 0.00000000010*cos(1.07944281160 + 742.9900605326*$t);
   $Z0 += 0.00000000009*cos(3.33909678408 + 1911.1994832172*$t);
   $Z0 += 0.00000000009*cos(6.28167845492 + 147423.51185532758*$t);
   $Z0 += 0.00000000010*cos(0.71259250113 + 137210.22630911658*$t);
   $Z0 += 0.00000000009*cos(3.57005441415 + 50800.03248330259*$t);
   $Z0 += 0.00000000009*cos(5.50160718751 + 2199.087343287*$t);
   $Z0 += 0.00000000009*cos(2.08845571909 + 51859.41441349179*$t);
   $Z0 += 0.00000000009*cos(4.25308230419 + 26222.0121911592*$t);
   $Z0 += 0.00000000009*cos(2.50157520492 + 24864.08530079559*$t);
   $Z0 += 0.00000000008*cos(2.47794653703 + 25440.89230825939*$t);
   $Z0 += 0.00000000009*cos(2.85482376043 + 25953.79409198919*$t);
   $Z0 += 0.00000000008*cos(2.29776576843 + 23866.04650697719*$t);
   $Z0 += 0.00000000009*cos(0.78685007135 + 104778.21075717278*$t);
   $Z0 += 0.00000000008*cos(4.62944551319 + 52492.19815280499*$t);
   $Z0 += 0.00000000009*cos(5.48506768547 + 77844.22478084739*$t);
   $Z0 += 0.00000000010*cos(1.12934836943 + 78731.674415077*$t);
   $Z0 += 0.00000000008*cos(2.59549267900 + 26941.0995233262*$t);
   $Z0 += 0.00000000008*cos(4.95780242181 + 78257.08086582259*$t);
   $Z0 += 0.00000000008*cos(2.55696609752 + 17893.6278083656*$t);
   $Z0 += 0.00000000008*cos(4.52002369897 + 104358.72611329758*$t);
   $Z0 += 0.00000000008*cos(3.82529998469 + 76571.54375522019*$t);
   $Z0 += 0.00000000010*cos(2.73498400513 + 323.5054166574*$t);
   $Z0 += 0.00000000009*cos(1.57363188943 + 24998.19435038059*$t);
   $Z0 += 0.00000000008*cos(0.65301230309 + 8194.2753332086*$t);
   $Z0 += 0.00000000008*cos(4.30691637723 + 77410.51304297059*$t);
   $Z0 += 0.00000000008*cos(5.61648727808 + 77795.74443436819*$t);
   $Z0 += 0.00000000009*cos(2.56370604921 + 50696.93970908399*$t);
   $Z0 += 0.00000000008*cos(5.53494196837 + 2008.557539159*$t);
   $Z0 += 0.00000000009*cos(0.81310827208 + 105940.68546158058*$t);
   $Z0 += 0.00000000008*cos(3.03053180726 + 27780.06881107659*$t);
   $Z0 += 0.00000000008*cos(1.09072579794 + 949.1756089698*$t);
   $Z0 += 0.00000000008*cos(5.21537078773 + 176953.98994186718*$t);
   $Z0 += 0.00000000008*cos(2.88040588282 + 36109.7404216736*$t);
   $Z0 += 0.00000000008*cos(3.90879186991 + 54087.0057663656*$t);
   $Z0 += 0.00000000007*cos(2.09304055777 + 25344.9130810416*$t);
   $Z0 += 0.00000000007*cos(3.77982679537 + 19.66976089979*$t);
   $Z0 += 0.00000000008*cos(2.01169539344 + 31415.379249957*$t);
   $Z0 += 0.00000000008*cos(6.01136820214 + 207643.84320240439*$t);
   $Z0 += 0.00000000008*cos(6.24227744862 + 38813.3565763492*$t);
   $Z0 += 0.00000000008*cos(1.33108336808 + 55503.94193942859*$t);
   $Z0 += 0.00000000007*cos(3.30804741677 + 23919.1426592916*$t);
   $Z0 += 0.00000000008*cos(3.48746520229 + 53242.3017603384*$t);
   $Z0 += 0.00000000008*cos(5.56477337187 + 27177.6119327678*$t);
   $Z0 += 0.00000000009*cos(5.00034866965 + 102132.85546210999*$t);
   $Z0 += 0.00000000007*cos(0.40751314845 + 2111.6503133776*$t);
   $Z0 += 0.00000000008*cos(1.66471235682 + 52290.24557183361*$t);
   $Z0 += 0.00000000007*cos(0.77215567463 + 26073.67604757259*$t);
   $Z0 += 0.00000000007*cos(5.32306426879 + 52595.29092702359*$t);
   $Z0 += 0.00000000007*cos(2.57023734016 + 24448.8336243862*$t);
   $Z0 += 0.00000000007*cos(1.40330361432 + 150244.34299945379*$t);
   $Z0 += 0.00000000008*cos(3.96541390761 + 52325.36948028299*$t);
   $Z0 += 0.00000000008*cos(0.64215744063 + 53228.07466633679*$t);
   $Z0 += 0.00000000007*cos(5.40994893955 + 25508.2155545754*$t);
   $Z0 += 0.00000000009*cos(0.63097343517 + 170049.17029103659*$t);
   $Z0 += 0.00000000006*cos(2.73735624509 + 153.7788104848*$t);
   $Z0 += 0.00000000006*cos(1.57808561874 + 49842.60989027639*$t);
   $Z0 += 0.00000000006*cos(5.85639973088 + 182085.63102592478*$t);
   $Z0 += 0.00000000007*cos(1.22602076365 + 107794.18751126219*$t);
   $Z0 += 0.00000000007*cos(2.99828544115 + 22625.658435709*$t);
   $Z0 += 0.00000000007*cos(3.38113527243 + 12546.481939083*$t);
   $Z0 += 0.00000000006*cos(5.98824736570 + 76667.52298243798*$t);
   $Z0 += 0.00000000007*cos(2.75252330601 + 172402.03644480839*$t);
   $Z0 += 0.00000000007*cos(0.93184314111 + 3328.13565628019*$t);
   $Z0 += 0.00000000006*cos(2.72361970528 + 26709.6469424134*$t);
   $Z0 += 0.00000000008*cos(1.81192469081 + 78270.33798362259*$t);
   $Z0 += 0.00000000006*cos(2.31595886989 + 157057.10981453978*$t);
   $Z0 += 0.00000000006*cos(1.73557063122 + 26667.590728573*$t);
   $Z0 += 0.00000000006*cos(4.37863242786 + 25466.159340735*$t);
   $Z0 += 0.00000000006*cos(3.19403160700 + 104275.34649502118*$t);
   $Z0 += 0.00000000006*cos(1.09823646265 + 25867.49049913539*$t);
   $Z0 += 0.00000000007*cos(0.11236053840 + 25764.39772491679*$t);
   $Z0 += 0.00000000007*cos(5.69537766284 + 66653.15746634839*$t);
   $Z0 += 0.00000000007*cos(1.98238737869 + 846.0828347512*$t);
   $Z0 += 0.00000000006*cos(0.65894058731 + 116783.65521669458*$t);
   $Z0 += 0.00000000005*cos(4.66761334131 + 2125.8774073792*$t);
   $Z0 += 0.00000000006*cos(4.52221761723 + 27726.9726587622*$t);
   $Z0 += 0.00000000006*cos(0.09840227420 + 167850.08294774959*$t);
   $Z0 += 0.00000000006*cos(3.36608415071 + 129799.61842155698*$t);
   $Z0 += 0.00000000007*cos(5.27549434329 + 52061.36699446317*$t);
   $Z0 += 0.00000000007*cos(1.32305784951 + 209812.60368468695*$t);
   $Z0 += 0.00000000006*cos(3.60884349448 + 78903.60671103658*$t);
   $Z0 += 0.00000000005*cos(3.76183094594 + 28256.66362385679*$t);
   $Z0 += 0.00000000006*cos(5.26397966812 + 75930.51303185058*$t);
   $Z0 += 0.00000000006*cos(3.23097479261 + 27669.86248985719*$t);
   $Z0 += 0.00000000005*cos(5.68060753615 + 1265.5674786264*$t);
   $Z0 += 0.00000000005*cos(3.87273283224 + 214364.55718174578*$t);
   $Z0 += 0.00000000005*cos(3.93047654027 + 365230.64398203876*$t);
   $Z0 += 0.00000000006*cos(2.60562458198 + 44295.7171298094*$t);
   $Z0 += 0.00000000005*cos(1.56438384826 + 60370.08161635699*$t);
   $Z0 += 0.00000000005*cos(0.91437456221 + 27684.0895838588*$t);
   $Z0 += 0.00000000005*cos(2.78304413268 + 25169.9728555924*$t);
   $Z0 += 0.00000000006*cos(5.76131236705 + 58857.03113654799*$t);
   $Z0 += 0.00000000005*cos(2.67085954875 + 38.1330356378*$t);
   $Z0 += 0.00000000006*cos(0.50453577398 + 155571.81910783658*$t);
   $Z0 += 0.00000000005*cos(3.86250449828 + 52381.99183158559*$t);
   $Z0 += 0.00000000005*cos(4.32361771821 + 27005.83342755599*$t);
   $Z0 += 0.00000000005*cos(1.63783306460 + 34082.4315835984*$t);
   $Z0 += 0.00000000006*cos(4.13235037436 + 29550.14784743939*$t);
   $Z0 += 0.00000000005*cos(0.09069856194 + 13541.42120249119*$t);
   $Z0 += 0.00000000005*cos(5.50663993071 + 26402.0893214438*$t);
   $Z0 += 0.00000000004*cos(5.22554265930 + 28736.3579670472*$t);
   $Z0 += 0.00000000005*cos(5.88997457155 + 78188.92782615528*$t);
   $Z0 += 0.00000000005*cos(1.29427545799 + 77630.92568540938*$t);
   $Z0 += 0.00000000006*cos(3.65818107099 + 76681.75007643958*$t);
   $Z0 += 0.00000000005*cos(1.59378640261 + 25773.71696170459*$t);
   $Z0 += 0.00000000005*cos(5.53985466815 + 24203.0019781568*$t);
   $Z0 += 0.00000000005*cos(1.10121885185 + 130459.18546877075*$t);
   $Z0 += 0.00000000005*cos(3.55866759420 + 52252.07235442399*$t);
   $Z0 += 0.00000000004*cos(5.49843651098 + 52712.61079524379*$t);
   $Z0 += 0.00000000005*cos(3.39959787962 + 104505.39137678158*$t);
   $Z0 += 0.00000000005*cos(1.94175393423 + 1272.6810256272*$t);
   $Z0 += 0.00000000004*cos(0.58115883263 + 26118.2300025786*$t);
   $Z0 += 0.00000000004*cos(0.54682947961 + 157586.80077963437*$t);
   $Z0 += 0.00000000004*cos(2.04212740892 + 148.0787244263*$t);
   $Z0 += 0.00000000005*cos(1.64853656178 + 26198.1094627936*$t);
   $Z0 += 0.00000000005*cos(0.18946812001 + 1.4844727083*$t);
   $Z0 += 0.00000000004*cos(5.04774492766 + 112231.70171963578*$t);
   $Z0 += 0.00000000004*cos(3.24889530044 + 26312.2479372761*$t);
   $Z0 += 0.00000000005*cos(2.14793916248 + 128320.75184749259*$t);
   $Z0 += 0.00000000004*cos(3.03681019540 + 151.0476698429*$t);
   $Z0 += 0.00000000004*cos(0.23608219351 + 26057.57628056979*$t);
   $Z0 += 0.00000000004*cos(2.78449581166 + 25138.7275326044*$t);
   $Z0 += 0.00000000004*cos(1.74750321635 + 27044.1922975448*$t);
   $Z0 += 0.00000000004*cos(5.26585590029 + 138319.60486120995*$t);
   $Z0 += 0.00000000004*cos(2.16361705057 + 78366.80219894118*$t);
   $Z0 += 0.00000000005*cos(1.86438708559 + 51653.22886505459*$t);
   $Z0 += 0.00000000004*cos(3.67745684549 + 148532.89040742096*$t);
   $Z0 += 0.00000000005*cos(3.36418574107 + 86457.98475793119*$t);
   $Z0 += 0.00000000004*cos(4.39817949768 + 78896.49316403578*$t);
   $Z0 += 0.00000000004*cos(3.16495363173 + 104197.83375581198*$t);
   $Z0 += 0.00000000004*cos(5.96720583626 + 1485.9801210652*$t);
   $Z0 += 0.00000000003*cos(3.62369510287 + 51852.30086649099*$t);
   $Z0 += 0.00000000004*cos(4.95850373202 + 80382.47328510099*$t);
   $Z0 += 0.00000000003*cos(0.24273955075 + 128106.31931499895*$t);
   $Z0 += 0.00000000004*cos(6.09423095974 + 51528.79544983359*$t);
   $Z0 += 0.00000000004*cos(2.26832726016 + 64901.25971792339*$t);
   $Z0 += 0.00000000004*cos(5.37231956323 + 99024.13645791399*$t);
   $Z0 += 0.00000000004*cos(5.83099981006 + 26411.4085582316*$t);
   $Z0 += 0.00000000003*cos(5.27877050710 + 14.2270940016*$t);
   $Z0 += 0.00000000004*cos(1.01079656557 + 16703.062133499*$t);
   $Z0 += 0.00000000003*cos(5.29624442233 + 51432.81622261579*$t);
   $Z0 += 0.00000000003*cos(1.99372517333 + 9384.8410080752*$t);
   $Z0 += 0.00000000003*cos(1.08549694329 + 92741.06060792258*$t);
   $Z0 += 0.00000000004*cos(2.61532511231 + 27250.37784598199*$t);
   $Z0 += 0.00000000004*cos(3.84384246230 + 65831.6667743248*$t);
   $Z0 += 0.00000000004*cos(5.07647835306 + 118828.96374949679*$t);
   $Z0 += 0.00000000004*cos(0.95691561960 + 1795.258443721*$t);
   $Z0 += 0.00000000003*cos(4.56071055194 + 130419.8459469712*$t);
   $Z0 += 0.00000000004*cos(3.21002699852 + 51969.62073471119*$t);
   $Z0 += 0.00000000003*cos(0.53542089836 + 25985.94062330859*$t);
   $Z0 += 0.00000000003*cos(0.86346795613 + 27972.80430499159*$t);
   $Z0 += 0.00000000004*cos(1.11850373850 + 52609.51802102519*$t);
   $Z0 += 0.00000000003*cos(5.13697350283 + 26189.8656598398*$t);
   $Z0 += 0.00000000003*cos(3.83162907268 + 207593.84658050018*$t);
   $Z0 += 0.00000000003*cos(3.70405700948 + 193937.98608932378*$t);
   $Z0 += 0.00000000003*cos(1.82907554275 + 153084.84390447979*$t);
   $Z0 += 0.00000000003*cos(1.75677414102 + 158746.17595363196*$t);
   $Z0 += 0.00000000003*cos(0.34579536632 + 161079.37234650398*$t);
   $Z0 += 0.00000000003*cos(3.85483830173 + 39629.32434406539*$t);
   $Z0 += 0.00000000003*cos(0.82694378727 + 51868.2486621788*$t);
   $Z0 += 0.00000000003*cos(0.16328069810 + 27463.67694142*$t);
   $Z0 += 0.00000000003*cos(4.08971647963 + 130226.21661243298*$t);
   $Z0 += 0.00000000004*cos(3.70176043302 + 163766.09444104518*$t);
   $Z0 += 0.00000000003*cos(1.31853873267 + 204151.27163553478*$t);
   $Z0 += 0.00000000004*cos(2.50849600287 + 220.4126424388*$t);
   $Z0 += 0.00000000003*cos(3.29076294593 + 125112.03959948818*$t);
   $Z0 += 0.00000000003*cos(5.82527117029 + 49953.94964855139*$t);
   $Z0 += 0.00000000003*cos(1.83258674312 + 37698.4550999484*$t);
   $Z0 += 0.00000000003*cos(1.84707936363 + 26076.8574413103*$t);
   $Z0 += 0.00000000003*cos(3.96840940578 + 130432.40216087017*$t);
   $Z0 += 0.00000000003*cos(0.01192870778 + 26098.9488418381*$t);
   $Z0 += 0.00000000003*cos(2.24128507678 + 78160.61665050399*$t);
   $Z0 += 0.00000000003*cos(1.67457691866 + 182188.72380014337*$t);
   $Z0 += 0.00000000003*cos(1.36021422148 + 136722.59155786238*$t);
   return $Z0;
}



   function Mercury_Z1($t) // 360 terms of order 1
{
   $Z1  = 0.00172388569*cos(3.47961470688 + 26087.9031415742*$t);
   $Z1 -= 0.00084745328;
   $Z1 += 0.00004790619*cos(6.21921644635 + 52175.8062831484*$t);
   $Z1 += 0.00001359100*cos(0.49329143004 + 78263.70942472259*$t);
   $Z1 += 0.00000602217*cos(3.41138035633 + 104351.61256629678*$t);
   $Z1 += 0.00000191001*cos(0.14519065991 + 130439.51570787099*$t);
   $Z1 += 0.00000055361*cos(3.18058121998 + 156527.41884944518*$t);
   $Z1 += 0.00000015488*cos(6.22255945229 + 182615.32199101939*$t);
   $Z1 += 0.00000010854*cos(0.29329316365 + 27197.2816936676*$t);
   $Z1 += 0.00000004262*cos(2.98444809115 + 208703.22513259358*$t);
   $Z1 += 0.00000004999*cos(0.56827577867 + 24978.5245894808*$t);
   $Z1 += 0.00000002282*cos(2.87281226699 + 31749.2351907264*$t);
   $Z1 += 0.00000002500*cos(3.37464102020 + 53285.1848352418*$t);
   $Z1 += 0.00000001480*cos(3.59930190722 + 51066.427731055*$t);
   $Z1 += 0.00000001163*cos(6.03119799366 + 234791.12827416777*$t);
   $Z1 += 0.00000001151*cos(1.94233400975 + 51116.4243529592*$t);
   $Z1 += 0.00000000808*cos(0.63296975442 + 25028.521211385*$t);
   $Z1 += 0.00000000781*cos(1.83801930165 + 5661.3320491522*$t);
   $Z1 += 0.00000000676*cos(4.44275341808 + 27043.5028831828*$t);
   $Z1 += 0.00000000873*cos(3.99274661843 + 21535.9496445154*$t);
   $Z1 += 0.00000000636*cos(0.60897167697 + 20426.571092422*$t);
   $Z1 += 0.00000000538*cos(5.92697698725 + 57837.1383323006*$t);
   $Z1 += 0.00000000528*cos(1.89223267777 + 1059.3819301892*$t);
   $Z1 += 0.00000000503*cos(4.12049535056 + 47623.8527860896*$t);
   $Z1 += 0.00000000450*cos(6.08529735298 + 529.6909650946*$t);
   $Z1 += 0.00000000553*cos(2.50090977683 + 1109.3785520934*$t);
   $Z1 += 0.00000000445*cos(0.35693140869 + 77154.33087262919*$t);
   $Z1 += 0.00000000459*cos(0.10915184787 + 79373.08797681599*$t);
   $Z1 += 0.00000000370*cos(2.24953657104 + 10213.285546211*$t);
   $Z1 += 0.00000000414*cos(3.01450201380 + 26107.57290247399*$t);
   $Z1 += 0.00000000417*cos(5.31402407370 + 37410.5672398786*$t);
   $Z1 += 0.00000000306*cos(3.10769212775 + 25558.2121764796*$t);
   $Z1 += 0.00000000316*cos(2.79570706285 + 260879.03141574195*$t);
   $Z1 += 0.00000000296*cos(4.12556468959 + 26068.2333806744*$t);
   $Z1 += 0.00000000332*cos(5.93758113684 + 26084.0218062162*$t);
   $Z1 += 0.00000000234*cos(3.03370038245 + 25132.3033999656*$t);
   $Z1 += 0.00000000250*cos(1.18440188253 + 26091.7844769322*$t);
   $Z1 += 0.00000000215*cos(1.41611963529 + 41962.5207369374*$t);
   $Z1 += 0.00000000206*cos(4.97267994525 + 77204.32749453338*$t);
   $Z1 += 0.00000000208*cos(5.60196522278 + 4551.9534970588*$t);
   $Z1 += 0.00000000251*cos(3.12319370712 + 30639.856638633*$t);
   $Z1 += 0.00000000230*cos(0.95862112790 + 50586.73338786459*$t);
   $Z1 += 0.00000000193*cos(4.90260211702 + 13521.7514415914*$t);
   $Z1 += 0.00000000243*cos(0.30112320330 + 1589.0728952838*$t);
   $Z1 += 0.00000000186*cos(5.87650538684 + 39609.6545831656*$t);
   $Z1 += 0.00000000211*cos(1.27593674125 + 53131.406024757*$t);
   $Z1 += 0.00000000188*cos(0.09339135236 + 28306.66024576099*$t);
   $Z1 += 0.00000000191*cos(4.77084065304 + 51646.11531805379*$t);
   $Z1 += 0.00000000167*cos(2.62739427985 + 32858.61374281979*$t);
   $Z1 += 0.00000000162*cos(5.02172677195 + 15874.6175953632*$t);
   $Z1 += 0.00000000140*cos(1.37984211372 + 51749.20809227239*$t);
   $Z1 += 0.00000000134*cos(0.57882342916 + 23869.1460373874*$t);
   $Z1 += 0.00000000132*cos(3.40338891449 + 103242.23401420339*$t);
   $Z1 += 0.00000000144*cos(4.41836879515 + 22645.32819660879*$t);
   $Z1 += 0.00000000143*cos(5.55896069473 + 25661.3049506982*$t);
   $Z1 += 0.00000000131*cos(2.17510821075 + 63498.47038145279*$t);
   $Z1 += 0.00000000143*cos(1.22805602725 + 46514.4742339962*$t);
   $Z1 += 0.00000000136*cos(2.45569156259 + 24498.8302462904*$t);
   $Z1 += 0.00000000120*cos(2.89969435814 + 7.1135470008*$t);
   $Z1 += 0.00000000104*cos(0.89744950821 + 73711.75592766379*$t);
   $Z1 += 0.00000000119*cos(1.44014554390 + 43071.8992890308*$t);
   $Z1 += 0.00000000126*cos(1.93369301602 + 14765.2390432698*$t);
   $Z1 += 0.00000000107*cos(4.29215813932 + 19317.1925403286*$t);
   $Z1 += 0.00000000089*cos(5.84175706917 + 51220.20654153979*$t);
   $Z1 += 0.00000000093*cos(2.27928877976 + 19804.8272915828*$t);
   $Z1 += 0.00000000083*cos(3.17188415740 + 29530.4780865396*$t);
   $Z1 += 0.00000000076*cos(5.10839374532 + 36301.18868778519*$t);
   $Z1 += 0.00000000079*cos(2.34370769368 + 1052.2683831884*$t);
   $Z1 += 0.00000000080*cos(2.11190196063 + 426.598190876*$t);
   $Z1 += 0.00000000076*cos(6.05836593763 + 52195.47604404819*$t);
   $Z1 += 0.00000000072*cos(1.89073508118 + 1066.49547719*$t);
   $Z1 += 0.00000000082*cos(3.03552784633 + 3442.5749449654*$t);
   $Z1 += 0.00000000075*cos(3.11962204135 + 105460.99111839019*$t);
   $Z1 += 0.00000000086*cos(5.84461147075 + 286966.93455731616*$t);
   $Z1 += 0.00000000069*cos(1.89458844998 + 27147.28507176339*$t);
   $Z1 += 0.00000000089*cos(4.10904716360 + 72602.37737557039*$t);
   $Z1 += 0.00000000077*cos(5.34990441360 + 213.299095438*$t);
   $Z1 += 0.00000000087*cos(5.53929952701 + 9103.9069941176*$t);
   $Z1 += 0.00000000070*cos(4.43072071341 + 639.897286314*$t);
   $Z1 += 0.00000000059*cos(3.32089922507 + 54394.56338733519*$t);
   $Z1 += 0.00000000061*cos(0.53721787795 + 25035.6347583858*$t);
   $Z1 += 0.00000000062*cos(1.36724381328 + 6283.0758499914*$t);
   $Z1 += 0.00000000073*cos(6.16650137658 + 56727.7597802072*$t);
   $Z1 += 0.00000000069*cos(0.88932506586 + 52156.1365222486*$t);
   $Z1 += 0.00000000055*cos(2.54693938612 + 65697.55772473979*$t);
   $Z1 += 0.00000000061*cos(2.70208782780 + 52171.9249477904*$t);
   $Z1 += 0.00000000058*cos(3.90075823666 + 76674.63652943878*$t);
   $Z1 += 0.00000000064*cos(6.09232375776 + 26080.78959457339*$t);
   $Z1 += 0.00000000052*cos(2.85118534529 + 16983.9961474566*$t);
   $Z1 += 0.00000000063*cos(5.90351345253 + 58946.51688439399*$t);
   $Z1 += 0.00000000052*cos(1.08065292487 + 25021.4076643842*$t);
   $Z1 += 0.00000000050*cos(5.98403755810 + 955.5997416086*$t);
   $Z1 += 0.00000000050*cos(2.34891444356 + 51535.90899683439*$t);
   $Z1 += 0.00000000048*cos(4.15049070651 + 12566.1516999828*$t);
   $Z1 += 0.00000000047*cos(3.35524401909 + 52168.69273614759*$t);
   $Z1 += 0.00000000060*cos(3.40462418234 + 25874.6040461362*$t);
   $Z1 += 0.00000000059*cos(4.98660394607 + 6770.7106012456*$t);
   $Z1 += 0.00000000065*cos(1.57121191195 + 52705.49724824299*$t);
   $Z1 += 0.00000000055*cos(5.05311644127 + 53235.18821333759*$t);
   $Z1 += 0.00000000056*cos(4.79703863542 + 25448.00585526019*$t);
   $Z1 += 0.00000000045*cos(3.87928453463 + 48733.23133818299*$t);
   $Z1 += 0.00000000054*cos(2.11712920969 + 33326.5787331742*$t);
   $Z1 += 0.00000000044*cos(2.58011573987 + 2218.7571041868*$t);
   $Z1 += 0.00000000057*cos(4.23895384399 + 52179.6876185064*$t);
   $Z1 += 0.00000000042*cos(2.38859796348 + 83925.04147387479*$t);
   $Z1 += 0.00000000040*cos(5.07476924931 + 51109.31080595839*$t);
   $Z1 += 0.00000000039*cos(0.16837719613 + 129330.13715577759*$t);
   $Z1 += 0.00000000041*cos(5.06879737373 + 59414.4818747484*$t);
   $Z1 += 0.00000000045*cos(0.74507723970 + 2118.7638603784*$t);
   $Z1 += 0.00000000045*cos(3.04665350718 + 27676.976036858*$t);
   $Z1 += 0.00000000039*cos(4.70158354540 + 69159.80243060499*$t);
   $Z1 += 0.00000000041*cos(5.93310326966 + 26095.016688575*$t);
   $Z1 += 0.00000000031*cos(3.33432475040 + 45892.73043315699*$t);
   $Z1 += 0.00000000030*cos(2.52574953749 + 77308.10968311399*$t);
   $Z1 += 0.00000000032*cos(3.64034199518 + 49957.0491789616*$t);
   $Z1 += 0.00000000030*cos(1.16170502578 + 26301.2022370122*$t);
   $Z1 += 0.00000000039*cos(4.76885068068 + 40853.142184844*$t);
   $Z1 += 0.00000000030*cos(3.03342131333 + 52182.9198301492*$t);
   $Z1 += 0.00000000033*cos(2.00778363573 + 7238.67559160*$t);
   $Z1 += 0.00000000034*cos(0.83933724452 + 98690.28051714458*$t);
   $Z1 += 0.00000000028*cos(2.06072938372 + 25934.1243310894*$t);
   $Z1 += 0.00000000033*cos(4.21710497504 + 79219.30916633119*$t);
   $Z1 += 0.00000000026*cos(4.99085554385 + 38519.945791972*$t);
   $Z1 += 0.00000000026*cos(6.08273991212 + 18849.2275499742*$t);
   $Z1 += 0.00000000025*cos(4.07659089438 + 26617.5941066688*$t);
   $Z1 += 0.00000000024*cos(4.39675973371 + 77837.11123384659*$t);
   $Z1 += 0.00000000030*cos(0.25587368521 + 50057.04242277*$t);
   $Z1 += 0.00000000024*cos(1.03737813626 + 45405.0956819028*$t);
   $Z1 += 0.00000000024*cos(1.77374954914 + 13655.8604911764*$t);
   $Z1 += 0.00000000026*cos(1.35857651349 + 77734.01845962799*$t);
   $Z1 += 0.00000000028*cos(4.51227694460 + 78793.40038981718*$t);
   $Z1 += 0.00000000024*cos(2.91891144270 + 82815.66292178139*$t);
   $Z1 += 0.00000000021*cos(5.13585062761 + 89586.37352302698*$t);
   $Z1 += 0.00000000024*cos(0.58794385685 + 11322.6640983044*$t);
   $Z1 += 0.00000000028*cos(1.99718125621 + 23969.1392811958*$t);
   $Z1 += 0.00000000022*cos(6.06219672387 + 419.4846438752*$t);
   $Z1 += 0.00000000020*cos(5.52785966186 + 433.7117378768*$t);
   $Z1 += 0.00000000028*cos(4.50140690236 + 51123.53789995999*$t);
   $Z1 += 0.00000000020*cos(4.37342000676 + 68050.42387851159*$t);
   $Z1 += 0.00000000019*cos(6.20105117352 + 55618.3812281138*$t);
   $Z1 += 0.00000000023*cos(2.61451042759 + 313054.83769889036*$t);
   $Z1 += 0.00000000018*cos(4.29485153949 + 23754.70674870219*$t);
   $Z1 += 0.00000000022*cos(1.87766446801 + 79323.09135491178*$t);
   $Z1 += 0.00000000020*cos(1.07656223897 + 25619.9381512198*$t);
   $Z1 += 0.00000000016*cos(1.03604660588 + 12432.0426503978*$t);
   $Z1 += 0.00000000016*cos(3.43674109707 + 25654.19140369739*$t);
   $Z1 += 0.00000000020*cos(2.97083013896 + 110012.94461544899*$t);
   $Z1 += 0.00000000017*cos(4.43931851455 + 38654.05484155699*$t);
   $Z1 += 0.00000000015*cos(2.81279895693 + 78283.37918562238*$t);
   $Z1 += 0.00000000018*cos(3.78270508656 + 25973.46385288896*$t);
   $Z1 += 0.00000000018*cos(5.12059646549 + 52389.1053785864*$t);
   $Z1 += 0.00000000017*cos(2.85670047821 + 25668.418497699*$t);
   $Z1 += 0.00000000018*cos(3.94184263610 + 78244.0396638228*$t);
   $Z1 += 0.00000000016*cos(1.58918523327 + 66941.04532641819*$t);
   $Z1 += 0.00000000013*cos(2.50556550656 + 1596.1864422846*$t);
   $Z1 += 0.00000000013*cos(6.23648492936 + 80482.46652890938*$t);
   $Z1 += 0.00000000015*cos(1.01343393682 + 78267.59076008058*$t);
   $Z1 += 0.00000000013*cos(5.74551369637 + 78259.82808936459*$t);
   $Z1 += 0.00000000013*cos(5.49092142595 + 77623.81213840858*$t);
   $Z1 += 0.00000000015*cos(3.65531240129 + 853.196381752*$t);
   $Z1 += 0.00000000016*cos(3.34679351675 + 26202.34243025941*$t);
   $Z1 += 0.00000000012*cos(4.99331123174 + 27154.3986187642*$t);
   $Z1 += 0.00000000012*cos(0.15067899984 + 78256.59587772179*$t);
   $Z1 += 0.00000000012*cos(1.23744980330 + 26727.8004278882*$t);
   $Z1 += 0.00000000012*cos(1.88171610012 + 77197.21394753258*$t);
   $Z1 += 0.00000000016*cos(1.02871649406 + 74821.13447975718*$t);
   $Z1 += 0.00000000014*cos(6.00648288256 + 53764.8791784322*$t);
   $Z1 += 0.00000000011*cos(1.03664866140 + 51962.5071877104*$t);
   $Z1 += 0.00000000012*cos(0.48930963657 + 102762.53967101299*$t);
   $Z1 += 0.00000000012*cos(3.21811087935 + 155418.04029735178*$t);
   $Z1 += 0.00000000012*cos(1.85815756639 + 103292.23063610759*$t);
   $Z1 += 0.00000000012*cos(3.87370634811 + 124778.18365871879*$t);
   $Z1 += 0.00000000011*cos(1.35784311134 + 51742.09454527159*$t);
   $Z1 += 0.00000000011*cos(2.25817150238 + 44937.1306915484*$t);
   $Z1 += 0.00000000012*cos(2.64551987584 + 26137.8997634784*$t);
   $Z1 += 0.00000000011*cos(2.75252916894 + 26514.5013324502*$t);
   $Z1 += 0.00000000010*cos(5.54274166657 + 103396.01282468818*$t);
   $Z1 += 0.00000000009*cos(6.12478323746 + 78270.82297172339*$t);
   $Z1 += 0.00000000013*cos(5.85664456058 + 136100.84775702319*$t);
   $Z1 += 0.00000000011*cos(2.43318609972 + 85034.42002596818*$t);
   $Z1 += 0.00000000011*cos(2.32000962427 + 64607.84893354619*$t);
   $Z1 += 0.00000000013*cos(3.22527913647 + 28421.0995344462*$t);
   $Z1 += 0.00000000009*cos(0.40604041041 + 24491.71669928959*$t);
   $Z1 += 0.00000000011*cos(1.88427152362 + 1581.959348283*$t);
   $Z1 += 0.00000000009*cos(1.56488206261 + 129380.13377768178*$t);
   $Z1 += 0.00000000009*cos(3.73371803074 + 99799.65906923798*$t);
   $Z1 += 0.00000000009*cos(3.14695429609 + 2333.196392872*$t);
   $Z1 += 0.00000000010*cos(1.21339698089 + 104881.30353139139*$t);
   $Z1 += 0.00000000009*cos(1.39539227982 + 77211.44104153418*$t);
   $Z1 += 0.00000000008*cos(4.87898553246 + 632.7837393132*$t);
   $Z1 += 0.00000000009*cos(5.97666796617 + 131548.89425996438*$t);
   $Z1 += 0.00000000008*cos(5.63267469085 + 7994.5284420242*$t);
   $Z1 += 0.00000000008*cos(0.82184344341 + 46848.3301747656*$t);
   $Z1 += 0.00000000008*cos(3.94765597932 + 25455.119402261*$t);
   $Z1 += 0.00000000010*cos(5.48154668085 + 27140.17152476259*$t);
   $Z1 += 0.00000000009*cos(3.19254939333 + 76144.94556434419*$t);
   $Z1 += 0.00000000007*cos(4.71909828780 + 536.8045120954*$t);
   $Z1 += 0.00000000008*cos(5.52314941870 + 52602.4044740244*$t);
   $Z1 += 0.00000000008*cos(3.24434973969 + 51322.60990139639*$t);
   $Z1 += 0.00000000010*cos(6.10170513742 + 32370.9789915656*$t);
   $Z1 += 0.00000000007*cos(5.95089721308 + 108903.56606335558*$t);
   $Z1 += 0.00000000008*cos(0.41129689957 + 76044.9523205358*$t);
   $Z1 += 0.00000000009*cos(1.20083825078 + 64741.95798313119*$t);
   $Z1 += 0.00000000007*cos(4.12190282008 + 71492.99882347698*$t);
   $Z1 += 0.00000000008*cos(4.96879374914 + 105410.99449648599*$t);
   $Z1 += 0.00000000008*cos(4.56629366973 + 26037.90651966999*$t);
   $Z1 += 0.00000000007*cos(0.70887703358 + 24505.94379329119*$t);
   $Z1 += 0.00000000007*cos(4.46371843319 + 50579.61984086379*$t);
   $Z1 += 0.00000000008*cos(5.56132218876 + 62389.09182935939*$t);
   $Z1 += 0.00000000007*cos(3.12746642701 + 18093.37469954999*$t);
   $Z1 += 0.00000000007*cos(4.68816053110 + 52026.2430860138*$t);
   $Z1 += 0.00000000007*cos(0.17502761083 + 35191.8101356918*$t);
   $Z1 += 0.00000000008*cos(5.48340707498 + 25234.70675982219*$t);
   $Z1 += 0.00000000007*cos(4.34617041823 + 18207.81398823521*$t);
   $Z1 += 0.00000000006*cos(2.06324851601 + 78477.00852016058*$t);
   $Z1 += 0.00000000006*cos(3.48502574572 + 26094.53170047421*$t);
   $Z1 += 0.00000000008*cos(2.34450938553 + 33967.99229491319*$t);
   $Z1 += 0.00000000006*cos(0.49326410196 + 5327.4761083828*$t);
   $Z1 += 0.00000000007*cos(1.20744112899 + 44181.27784112419*$t);
   $Z1 += 0.00000000006*cos(4.73779414587 + 647.0108333148*$t);
   $Z1 += 0.00000000007*cos(0.69979076833 + 51756.3216392732*$t);
   $Z1 += 0.00000000006*cos(5.63847965133 + 339142.74084046457*$t);
   $Z1 += 0.00000000008*cos(3.68367545759 + 28206.6670019526*$t);
   $Z1 += 0.00000000006*cos(0.34516328945 + 26081.27458267419*$t);
   $Z1 += 0.00000000006*cos(2.90716455101 + 81706.28436968799*$t);
   $Z1 += 0.00000000007*cos(2.12491220048 + 88476.99497093359*$t);
   $Z1 += 0.00000000005*cos(3.34028647657 + 45494.58142974879*$t);
   $Z1 += 0.00000000005*cos(4.65551599269 + 93028.94846799239*$t);
   $Z1 += 0.00000000005*cos(5.37501413358 + 26610.48055966799*$t);
   $Z1 += 0.00000000006*cos(0.81299872270 + 95247.70557217918*$t);
   $Z1 += 0.00000000005*cos(3.47000694038 + 26507.38778544939*$t);
   $Z1 += 0.00000000006*cos(3.28193656342 + 26241.681952059*$t);
   $Z1 += 0.00000000006*cos(4.64447087045 + 155468.03691925599*$t);
   $Z1 += 0.00000000006*cos(4.82010175617 + 94138.32702008578*$t);
   $Z1 += 0.00000000006*cos(1.32053124690 + 2648.454825473*$t);
   $Z1 += 0.00000000005*cos(3.22819989009 + 25938.3399444396*$t);
   $Z1 += 0.00000000005*cos(2.61718935389 + 162188.75089859738*$t);
   $Z1 += 0.00000000005*cos(0.67832900959 + 104331.94280539699*$t);
   $Z1 += 0.00000000004*cos(5.21344084187 + 71025.03383312259*$t);
   $Z1 += 0.00000000004*cos(4.01714451988 + 78050.41032928458*$t);
   $Z1 += 0.00000000004*cos(5.08291206251 + 52099.5402118728*$t);
   $Z1 += 0.00000000004*cos(4.09094572411 + 104355.49390165479*$t);
   $Z1 += 0.00000000005*cos(4.64872616599 + 26190.99591579279*$t);
   $Z1 += 0.00000000005*cos(5.32437142333 + 25551.09862947879*$t);
   $Z1 += 0.00000000004*cos(2.72976343615 + 79852.78232000639*$t);
   $Z1 += 0.00000000004*cos(4.86893081526 + 50593.84693486539*$t);
   $Z1 += 0.00000000004*cos(0.65479452219 + 150866.08680029298*$t);
   $Z1 += 0.00000000004*cos(5.05003950789 + 7880.08915333899*$t);
   $Z1 += 0.00000000005*cos(0.56998430384 + 522.5774180938*$t);
   $Z1 += 0.00000000004*cos(2.18434126488 + 51639.00177105299*$t);
   $Z1 += 0.00000000004*cos(5.16982498302 + 149.5631971346*$t);
   $Z1 += 0.00000000004*cos(4.45668113275 + 52815.7035694624*$t);
   $Z1 += 0.00000000004*cos(0.69135494074 + 22759.76748529401*$t);
   $Z1 += 0.00000000004*cos(1.45087522595 + 20760.4270331914*$t);
   $Z1 += 0.00000000003*cos(2.24345480860 + 26521.614879451*$t);
   $Z1 += 0.00000000004*cos(1.44955412166 + 120226.23016165999*$t);
   $Z1 += 0.00000000004*cos(0.16879575063 + 29416.03879785439*$t);
   $Z1 += 0.00000000003*cos(4.57090150345 + 103299.34418310839*$t);
   $Z1 += 0.00000000003*cos(4.59598906499 + 39743.7636327506*$t);
   $Z1 += 0.00000000004*cos(2.57869053238 + 42153.969003049*$t);
   $Z1 += 0.00000000004*cos(0.10237777214 + 52290.24557183361*$t);
   $Z1 += 0.00000000003*cos(5.55031090013 + 3340.6124266998*$t);
   $Z1 += 0.00000000004*cos(6.17863510125 + 26555.8681319286*$t);
   $Z1 += 0.00000000003*cos(4.17269750233 + 130969.20667296558*$t);
   $Z1 += 0.00000000003*cos(4.68682814951 + 129909.82474277639*$t);
   $Z1 += 0.00000000004*cos(2.02800775775 + 53242.3017603384*$t);
   $Z1 += 0.00000000003*cos(5.35296159846 + 91785.46086631398*$t);
   $Z1 += 0.00000000003*cos(2.90685522022 + 104358.72611329758*$t);
   $Z1 += 0.00000000003*cos(3.88120838753 + 24864.08530079559*$t);
   $Z1 += 0.00000000003*cos(3.64751061524 + 26011.6370702986*$t);
   $Z1 += 0.00000000003*cos(3.23195009997 + 27311.72098235281*$t);
   $Z1 += 0.00000000003*cos(5.88557504342 + 104371.28232719658*$t);
   $Z1 += 0.00000000004*cos(3.12210700112 + 22747.2907148744*$t);
   $Z1 += 0.00000000004*cos(0.64204557090 + 105307.21230790539*$t);
   $Z1 += 0.00000000003*cos(6.24152399429 + 181505.94343892598*$t);
   $Z1 += 0.00000000004*cos(4.13065969197 + 125887.56221081219*$t);
   $Z1 += 0.00000000004*cos(4.45779901612 + 10021.8372800994*$t);
   $Z1 += 0.00000000004*cos(4.30265545797 + 90829.86112470538*$t);
   $Z1 += 0.00000000003*cos(2.25871507982 + 129483.91596626239*$t);
   $Z1 += 0.00000000003*cos(4.48812754789 + 77829.99768684579*$t);
   $Z1 += 0.00000000003*cos(1.95423245378 + 103821.92160120218*$t);
   $Z1 += 0.00000000003*cos(2.31574317697 + 53228.07466633679*$t);
   $Z1 += 0.00000000003*cos(5.16799762022 + 52022.0274726636*$t);
   $Z1 += 0.00000000003*cos(0.00175159073 + 54509.0026760204*$t);
   $Z1 += 0.00000000004*cos(1.27199373061 + 85502.38501632259*$t);
   $Z1 += 0.00000000003*cos(2.62911266804 + 100909.03762133139*$t);
   $Z1 += 0.00000000003*cos(5.08550718888 + 13362.4497067992*$t);
   $Z1 += 0.00000000004*cos(0.35429190140 + 26624.70765366959*$t);
   $Z1 += 0.00000000003*cos(5.11624187785 + 206.1855484372*$t);
   $Z1 += 0.00000000003*cos(1.69621650635 + 131498.89763806018*$t);
   $Z1 += 0.00000000003*cos(3.11541432650 + 106570.36967048359*$t);
   $Z1 += 0.00000000003*cos(3.95256477489 + 77844.22478084739*$t);
   $Z1 += 0.00000000002*cos(0.69658812162 + 6681.2248533996*$t);
   $Z1 += 0.00000000003*cos(3.36003785702 + 128850.44281258718*$t);
   $Z1 += 0.00000000003*cos(0.51920338716 + 26941.0995233262*$t);
   $Z1 += 0.00000000003*cos(6.07817701344 + 1478.8665740644*$t);
   $Z1 += 0.00000000002*cos(5.59272300692 + 27684.0895838588*$t);
   $Z1 += 0.00000000003*cos(0.55085259267 + 52061.36699446317*$t);
   $Z1 += 0.00000000002*cos(5.99908279167 + 98068.53671630539*$t);
   $Z1 += 0.00000000002*cos(4.99656134147 + 103285.11708910679*$t);
   $Z1 += 0.00000000003*cos(3.71061275134 + 90695.75207512038*$t);
   $Z1 += 0.00000000002*cos(5.86031214770 + 111122.32316754239*$t);
   $Z1 += 0.00000000003*cos(5.17758096862 + 114564.89811250778*$t);
   $Z1 += 0.00000000003*cos(2.34733115087 + 78690.30761559859*$t);
   $Z1 += 0.00000000002*cos(5.09808138852 + 76.2660712756*$t);
   $Z1 += 0.00000000002*cos(5.31294016066 + 104564.91166173479*$t);
   $Z1 += 0.00000000002*cos(0.98459946560 + 97580.90196505119*$t);
   $Z1 += 0.00000000003*cos(5.89794789328 + 49527.35145767539*$t);
   $Z1 += 0.00000000003*cos(3.28062974222 + 104344.49901929598*$t);
   $Z1 += 0.00000000003*cos(2.42068250003 + 103711.71527998279*$t);
   $Z1 += 0.00000000002*cos(1.05901421415 + 51543.0225438352*$t);
   $Z1 += 0.00000000002*cos(5.70093401914 + 117873.36400788819*$t);
   $Z1 += 0.00000000002*cos(3.31505268739 + 58458.88213313979*$t);
   $Z1 += 0.00000000003*cos(0.82349684851 + 151975.46535238638*$t);
   $Z1 += 0.00000000002*cos(1.52195733318 + 181555.94006083018*$t);
   $Z1 += 0.00000000003*cos(2.24270782677 + 25565.3257234804*$t);
   $Z1 += 0.00000000003*cos(5.23472870141 + 60055.89543648739*$t);
   $Z1 += 0.00000000002*cos(4.19499586662 + 956.2891559706*$t);
   $Z1 += 0.00000000002*cos(0.43182759927 + 1162.4747044078*$t);
   $Z1 += 0.00000000002*cos(1.55169232531 + 52329.58509363319*$t);
   $Z1 += 0.00000000002*cos(5.54133200346 + 188276.65404017158*$t);
   $Z1 += 0.00000000002*cos(1.63007261166 + 26294.08869001139*$t);
   $Z1 += 0.00000000002*cos(4.84459398809 + 70269.18098269838*$t);
   $Z1 += 0.00000000002*cos(2.45157941038 + 52698.38370124219*$t);
   $Z1 += 0.00000000002*cos(3.50351080444 + 102132.85546210999*$t);
   $Z1 += 0.00000000002*cos(1.28285332683 + 115674.27666460119*$t);
   $Z1 += 0.00000000002*cos(1.03461044928 + 31415.379249957*$t);
   $Z1 += 0.00000000002*cos(1.31658394171 + 52125.80966124419*$t);
   $Z1 += 0.00000000002*cos(2.43230222606 + 103.0927742186*$t);
   $Z1 += 0.00000000002*cos(2.41277818747 + 3328.13565628019*$t);
   $Z1 += 0.00000000002*cos(3.38189136419 + 25881.717593137*$t);
   $Z1 += 0.00000000002*cos(0.33148559099 + 54294.57014352679*$t);
   $Z1 += 0.00000000002*cos(0.15981568000 + 77410.51304297059*$t);
   $Z1 += 0.00000000002*cos(2.90582069750 + 52643.7712735028*$t);
   $Z1 += 0.00000000002*cos(0.45204563942 + 71980.63357473118*$t);
   $Z1 += 0.00000000001*cos(4.12045734648 + 25440.89230825939*$t);
   $Z1 += 0.00000000001*cos(1.18539094152 + 12546.481939083*$t);
   $Z1 += 0.00000000002*cos(4.49448672471 + 146314.13330323418*$t);
   $Z1 += 0.00000000001*cos(3.99060758894 + 25867.49049913539*$t);
   $Z1 += 0.00000000001*cos(0.11291308321 + 105940.68546158058*$t);
   $Z1 += 0.00000000001*cos(6.21239484932 + 104138.31347085879*$t);
   $Z1 += 0.00000000002*cos(1.60021595888 + 51528.79544983359*$t);
   $Z1 += 0.00000000002*cos(2.03829619352 + 134991.46920492979*$t);
   $Z1 += 0.00000000002*cos(0.76189495741 + 84546.78527471398*$t);
   $Z1 += 0.00000000001*cos(1.19290119810 + 103925.01437542078*$t);
   $Z1 += 0.00000000001*cos(0.26092608641 + 155997.72788435058*$t);
   $Z1 += 0.00000000001*cos(0.91369257474 + 157636.79740153858*$t);
   $Z1 += 0.00000000002*cos(2.43141471440 + 143961.26714946238*$t);
   $Z1 += 0.00000000001*cos(2.58462364639 + 11610.5519583742*$t);
   $Z1 += 0.00000000001*cos(5.08144757931 + 32769.1279949738*$t);
   $Z1 += 0.00000000001*cos(2.90527234589 + 124156.43985787958*$t);
   $Z1 += 0.00000000001*cos(6.04124419781 + 16066.0658614748*$t);
   $Z1 += 0.00000000001*cos(0.82693925421 + 1692.1656695024*$t);
   $Z1 += 0.00000000001*cos(1.72510786190 + 26237.46633870879*$t);
   $Z1 += 0.00000000001*cos(0.74557141998 + 29428.515568274*$t);
   $Z1 += 0.00000000001*cos(1.18363932898 + 130012.91751699499*$t);
   $Z1 += 0.00000000001*cos(5.53709511500 + 26720.68688088739*$t);
   $Z1 += 0.00000000001*cos(2.22421236230 + 25984.8103673556*$t);
   $Z1 += 0.00000000001*cos(4.51546243970 + 48835.19385644859*$t);
   $Z1 += 0.00000000001*cos(3.61219119527 + 131395.11544947959*$t);
   return $Z1*$t;
}



   function Mercury_Z2($t) // 167 terms of order 2
{
   $Z2  = 0.00004475377*cos(4.81703460547 + 26087.9031415742*$t);
   $Z2 += 0.00001109532;
   $Z2 += 0.00000180327*cos(1.19420644188 + 52175.8062831484*$t);
   $Z2 += 0.00000032581*cos(4.62632031743 + 78263.70942472259*$t);
   $Z2 += 0.00000017342*cos(1.72679813654 + 104351.61256629678*$t);
   $Z2 += 0.00000007781*cos(4.84181443647 + 130439.51570787099*$t);
   $Z2 += 0.00000002976*cos(1.62432394721 + 156527.41884944518*$t);
   $Z2 += 0.00000001040*cos(4.67846690150 + 182615.32199101939*$t);
   $Z2 += 0.00000000680*cos(1.83005840786 + 27197.2816936676*$t);
   $Z2 += 0.00000000344*cos(1.44603022303 + 208703.22513259358*$t);
   $Z2 += 0.00000000110*cos(4.49542230374 + 234791.12827416777*$t);
   $Z2 += 0.00000000124*cos(4.86657119280 + 53285.1848352418*$t);
   $Z2 += 0.00000000112*cos(5.14312161072 + 24978.5245894808*$t);
   $Z2 += 0.00000000079*cos(4.25092448424 + 31749.2351907264*$t);
   $Z2 += 0.00000000058*cos(2.00193276015 + 51066.427731055*$t);
   $Z2 += 0.00000000046*cos(5.63734699624 + 27043.5028831828*$t);
   $Z2 += 0.00000000054*cos(5.70534232394 + 21535.9496445154*$t);
   $Z2 += 0.00000000048*cos(3.15710696395 + 51116.4243529592*$t);
   $Z2 += 0.00000000034*cos(1.26182479499 + 260879.03141574195*$t);
   $Z2 += 0.00000000027*cos(5.76991833962 + 529.6909650946*$t);
   $Z2 += 0.00000000024*cos(5.07488580193 + 77154.33087262919*$t);
   $Z2 += 0.00000000024*cos(0.95399123610 + 5661.3320491522*$t);
   $Z2 += 0.00000000019*cos(1.78858873244 + 28306.66024576099*$t);
   $Z2 += 0.00000000021*cos(3.57893007020 + 25558.2121764796*$t);
   $Z2 += 0.00000000022*cos(0.13783545204 + 25028.521211385*$t);
   $Z2 += 0.00000000018*cos(5.52049109830 + 47623.8527860896*$t);
   $Z2 += 0.00000000022*cos(1.64692693544 + 20426.571092422*$t);
   $Z2 += 0.00000000014*cos(5.59601085083 + 22645.32819660879*$t);
   $Z2 += 0.00000000014*cos(3.31820162449 + 1059.3819301892*$t);
   $Z2 += 0.00000000018*cos(0.54978030810 + 37410.5672398786*$t);
   $Z2 += 0.00000000016*cos(1.54285068472 + 79373.08797681599*$t);
   $Z2 += 0.00000000015*cos(4.49153081628 + 26107.57290247399*$t);
   $Z2 += 0.00000000014*cos(0.87791325175 + 57837.1383323006*$t);
   $Z2 += 0.00000000013*cos(5.18106920630 + 1109.3785520934*$t);
   $Z2 += 0.00000000011*cos(2.36351366554 + 53131.406024757*$t);
   $Z2 += 0.00000000012*cos(4.34261596283 + 32858.61374281979*$t);
   $Z2 += 0.00000000015*cos(2.28255722706 + 50586.73338786459*$t);
   $Z2 += 0.00000000012*cos(5.63353087898 + 26068.2333806744*$t);
   $Z2 += 0.00000000013*cos(2.08539778974 + 25132.3033999656*$t);
   $Z2 += 0.00000000013*cos(3.31771880069 + 10213.285546211*$t);
   $Z2 += 0.00000000013*cos(0.37928151736 + 4551.9534970588*$t);
   $Z2 += 0.00000000009*cos(5.32896907669 + 23869.1460373874*$t);
   $Z2 += 0.00000000009*cos(1.85195473467 + 103242.23401420339*$t);
   $Z2 += 0.00000000010*cos(2.69290690744 + 26091.7844769322*$t);
   $Z2 += 0.00000000012*cos(1.13570211007 + 26084.0218062162*$t);
   $Z2 += 0.00000000010*cos(4.31085447586 + 286966.93455731616*$t);
   $Z2 += 0.00000000008*cos(5.82534551009 + 25874.6040461362*$t);
   $Z2 += 0.00000000008*cos(6.17561594460 + 51646.11531805379*$t);
   $Z2 += 0.00000000008*cos(2.73528822352 + 41962.5207369374*$t);
   $Z2 += 0.00000000008*cos(0.86132391139 + 39609.6545831656*$t);
   $Z2 += 0.00000000007*cos(5.90586936446 + 77204.32749453338*$t);
   $Z2 += 0.00000000006*cos(2.23725426587 + 26080.78959457339*$t);
   $Z2 += 0.00000000007*cos(5.14284397401 + 1589.0728952838*$t);
   $Z2 += 0.00000000007*cos(3.13934336281 + 43071.8992890308*$t);
   $Z2 += 0.00000000005*cos(4.86318139233 + 54394.56338733519*$t);
   $Z2 += 0.00000000007*cos(2.45439783289 + 213.299095438*$t);
   $Z2 += 0.00000000006*cos(1.58442144371 + 26095.016688575*$t);
   $Z2 += 0.00000000006*cos(5.45060244038 + 25973.46385288896*$t);
   $Z2 += 0.00000000005*cos(4.79894457233 + 46514.4742339962*$t);
   $Z2 += 0.00000000005*cos(2.20020439243 + 6283.0758499914*$t);
   $Z2 += 0.00000000005*cos(5.38203064600 + 15874.6175953632*$t);
   $Z2 += 0.00000000005*cos(2.75432060530 + 19317.1925403286*$t);
   $Z2 += 0.00000000004*cos(2.96762251508 + 33326.5787331742*$t);
   $Z2 += 0.00000000005*cos(3.58879974911 + 63498.47038145279*$t);
   $Z2 += 0.00000000004*cos(6.00635657777 + 7.1135470008*$t);
   $Z2 += 0.00000000004*cos(3.43811673157 + 1066.49547719*$t);
   $Z2 += 0.00000000004*cos(3.46360896326 + 25661.3049506982*$t);
   $Z2 += 0.00000000004*cos(4.83154673482 + 51220.20654153979*$t);
   $Z2 += 0.00000000005*cos(1.65155678587 + 26202.34243025941*$t);
   $Z2 += 0.00000000004*cos(4.05043524436 + 51535.90899683439*$t);
   $Z2 += 0.00000000004*cos(3.87269622429 + 1052.2683831884*$t);
   $Z2 += 0.00000000004*cos(1.10462215070 + 58946.51688439399*$t);
   $Z2 += 0.00000000004*cos(5.41502198784 + 14765.2390432698*$t);
   $Z2 += 0.00000000004*cos(2.03873056084 + 26617.5941066688*$t);
   $Z2 += 0.00000000003*cos(2.19389680754 + 30639.856638633*$t);
   $Z2 += 0.00000000003*cos(4.90884264121 + 129330.13715577759*$t);
   $Z2 += 0.00000000004*cos(6.22828785556 + 36301.18868778519*$t);
   $Z2 += 0.00000000004*cos(3.68258570333 + 13521.7514415914*$t);
   $Z2 += 0.00000000003*cos(5.62688231105 + 48733.23133818299*$t);
   $Z2 += 0.00000000004*cos(2.45849991842 + 51749.20809227239*$t);
   $Z2 += 0.00000000003*cos(3.06502787272 + 16983.9961474566*$t);
   $Z2 += 0.00000000003*cos(4.57742754340 + 24498.8302462904*$t);
   $Z2 += 0.00000000003*cos(1.46286237530 + 3442.5749449654*$t);
   $Z2 += 0.00000000003*cos(1.55999070785 + 29530.4780865396*$t);
   $Z2 += 0.00000000003*cos(5.14417203650 + 76674.63652943878*$t);
   $Z2 += 0.00000000003*cos(1.09829861469 + 313054.83769889036*$t);
   $Z2 += 0.00000000003*cos(2.10570480254 + 49957.0491789616*$t);
   $Z2 += 0.00000000002*cos(0.49635100411 + 38519.945791972*$t);
   $Z2 += 0.00000000003*cos(3.19744325207 + 426.598190876*$t);
   $Z2 += 0.00000000003*cos(2.27361066514 + 72602.37737557039*$t);
   $Z2 += 0.00000000002*cos(5.42166315666 + 12566.1516999828*$t);
   $Z2 += 0.00000000002*cos(4.11021645418 + 2218.7571041868*$t);
   $Z2 += 0.00000000002*cos(6.01977464255 + 2118.7638603784*$t);
   $Z2 += 0.00000000003*cos(3.82490235305 + 9103.9069941176*$t);
   $Z2 += 0.00000000002*cos(1.55999258470 + 50057.04242277*$t);
   $Z2 += 0.00000000002*cos(2.83741388675 + 639.897286314*$t);
   $Z2 += 0.00000000003*cos(5.50522133834 + 23754.70674870219*$t);
   $Z2 += 0.00000000002*cos(4.64349469451 + 56727.7597802072*$t);
   $Z2 += 0.00000000002*cos(1.95986167578 + 73711.75592766379*$t);
   $Z2 += 0.00000000002*cos(0.55672894560 + 955.5997416086*$t);
   $Z2 += 0.00000000002*cos(1.93244538678 + 103292.23063610759*$t);
   $Z2 += 0.00000000002*cos(6.05929958767 + 25448.00585526019*$t);
   $Z2 += 0.00000000002*cos(3.03995453086 + 40853.142184844*$t);
   $Z2 += 0.00000000001*cos(2.47665559131 + 77734.01845962799*$t);
   $Z2 += 0.00000000002*cos(6.25822253418 + 52705.49724824299*$t);
   $Z2 += 0.00000000002*cos(1.38627663309 + 77308.10968311399*$t);
   $Z2 += 0.00000000001*cos(1.59643416291 + 27147.28507176339*$t);
   $Z2 += 0.00000000002*cos(3.54992576302 + 83925.04147387479*$t);
   $Z2 += 0.00000000001*cos(5.82473062390 + 45405.0956819028*$t);
   $Z2 += 0.00000000001*cos(0.32361625099 + 13655.8604911764*$t);
   $Z2 += 0.00000000001*cos(4.59226455833 + 55618.3812281138*$t);
   $Z2 += 0.00000000001*cos(1.77801233677 + 52156.1365222486*$t);
   $Z2 += 0.00000000001*cos(3.59899238770 + 53235.18821333759*$t);
   $Z2 += 0.00000000001*cos(0.75186170269 + 52195.47604404819*$t);
   $Z2 += 0.00000000002*cos(4.21419422689 + 105460.99111839019*$t);
   $Z2 += 0.00000000001*cos(3.02724177299 + 51962.5071877104*$t);
   $Z2 += 0.00000000001*cos(5.54016425013 + 98690.28051714458*$t);
   $Z2 += 0.00000000001*cos(4.96163083687 + 18849.2275499742*$t);
   $Z2 += 0.00000000001*cos(5.13453198510 + 52179.6876185064*$t);
   $Z2 += 0.00000000001*cos(4.54050452164 + 52168.69273614759*$t);
   $Z2 += 0.00000000001*cos(3.67122666149 + 52171.9249477904*$t);
   $Z2 += 0.00000000001*cos(4.16002499033 + 45892.73043315699*$t);
   $Z2 += 0.00000000001*cos(3.11767834677 + 78793.40038981718*$t);
   $Z2 += 0.00000000001*cos(4.70607503583 + 52290.24557183361*$t);
   $Z2 += 0.00000000001*cos(1.72196789636 + 155418.04029735178*$t);
   $Z2 += 0.00000000001*cos(1.43678225289 + 82815.66292178139*$t);
   $Z2 += 0.00000000001*cos(3.64195400420 + 65697.55772473979*$t);
   $Z2 += 0.00000000001*cos(2.34232573897 + 27676.976036858*$t);
   $Z2 += 0.00000000001*cos(2.22519315150 + 52061.36699446317*$t);
   $Z2 += 0.00000000001*cos(4.06167314974 + 339142.74084046457*$t);
   $Z2 += 0.00000000001*cos(1.84893906586 + 853.196381752*$t);
   $Z2 += 0.00000000001*cos(5.42775732068 + 26514.5013324502*$t);
   $Z2 += 0.00000000001*cos(6.03028910593 + 59414.4818747484*$t);
   $Z2 += 0.00000000001*cos(5.09856883029 + 76044.9523205358*$t);
   $Z2 += 0.00000000001*cos(1.68523311659 + 28421.0995344462*$t);
   $Z2 += 0.00000000001*cos(5.32883383904 + 79219.30916633119*$t);
   $Z2 += 0.00000000001*cos(0.84570925458 + 19804.8272915828*$t);
   $Z2 += 0.00000000001*cos(1.06906655760 + 124778.18365871879*$t);
   $Z2 += 0.00000000001*cos(5.63868454693 + 69159.80243060499*$t);
   $Z2 += 0.00000000001*cos(5.38153004139 + 26301.2022370122*$t);
   $Z2 += 0.00000000001*cos(4.77073547131 + 104881.30353139139*$t);
   $Z2 += 0.00000000001*cos(4.74588142049 + 129380.13377768178*$t);
   $Z2 += 0.00000000001*cos(2.25545108286 + 52602.4044740244*$t);
   $Z2 += 0.00000000001*cos(3.98601687793 + 11322.6640983044*$t);
   $Z2 += 0.00000000001*cos(0.07845866028 + 66941.04532641819*$t);
   $Z2 += 0.00000000001*cos(2.88984683172 + 38654.05484155699*$t);
   $Z2 += 0.00000000001*cos(4.00503893845 + 52182.9198301492*$t);
   $Z2 += 0.00000000001*cos(3.49574656538 + 52389.1053785864*$t);
   $Z2 += 0.00000000001*cos(0.33429045009 + 7238.67559160*$t);
   $Z2 += 0.00000000001*cos(0.31985247795 + 79323.09135491178*$t);
   $Z2 += 0.00000000001*cos(6.10860920195 + 51109.31080595839*$t);
   $Z2 += 0.00000000001*cos(2.84676327671 + 7994.5284420242*$t);
   $Z2 += 0.00000000001*cos(3.11496551907 + 25934.1243310894*$t);
   $Z2 += 0.00000000001*cos(5.40172380413 + 78477.00852016058*$t);
   $Z2 += 0.00000000001*cos(0.41949488018 + 6770.7106012456*$t);
   $Z2 += 0.00000000001*cos(4.62233743377 + 64741.95798313119*$t);
   $Z2 += 0.00000000001*cos(4.27662369360 + 12432.0426503978*$t);
   $Z2 += 0.00000000000*cos(0.88019140659 + 110012.94461544899*$t);
   $Z2 += 0.00000000000*cos(1.18882526931 + 28206.6670019526*$t);
   $Z2 += 0.00000000000*cos(5.60067399390 + 51123.53789995999*$t);
   $Z2 += 0.00000000000*cos(0.66520680194 + 419.4846438752*$t);
   $Z2 += 0.00000000000*cos(4.17948666644 + 78267.59076008058*$t);
   $Z2 += 0.00000000000*cos(5.93970677792 + 68050.42387851159*$t);
   $Z2 += 0.00000000000*cos(4.76480820754 + 78244.0396638228*$t);
   $Z2 += 0.00000000001*cos(0.17150693291 + 433.7117378768*$t);
   $Z2 += 0.00000000000*cos(2.10468974660 + 25619.9381512198*$t);
   $Z2 += 0.00000000001*cos(5.16942669114 + 77837.11123384659*$t);
   return $Z2*$t*$t;
}



   function Mercury_Z3($t) // 47 terms of order 3
{
   $Z3  = 0.00000099250*cos(0.29443229590 + 26087.9031415742*$t);
   $Z3 += 0.00000092976;
   $Z3 += 0.00000003230*cos(4.02309993038 + 52175.8062831484*$t);
   $Z3 += 0.00000000640*cos(2.16717843388 + 78263.70942472259*$t);
   $Z3 += 0.00000000369*cos(6.09477192866 + 104351.61256629678*$t);
   $Z3 += 0.00000000224*cos(3.14630379889 + 130439.51570787099*$t);
   $Z3 += 0.00000000111*cos(0.00722359195 + 156527.41884944518*$t);
   $Z3 += 0.00000000048*cos(3.09594836549 + 182615.32199101939*$t);
   $Z3 += 0.00000000019*cos(6.16655603798 + 208703.22513259358*$t);
   $Z3 += 0.00000000021*cos(3.31046636942 + 27197.2816936676*$t);
   $Z3 += 0.00000000007*cos(2.94356012175 + 234791.12827416777*$t);
   $Z3 += 0.00000000007*cos(0.65628834083 + 24978.5245894808*$t);
   $Z3 += 0.00000000002*cos(5.99334261589 + 260879.03141574195*$t);
   $Z3 += 0.00000000002*cos(6.22873921629 + 53285.1848352418*$t);
   $Z3 += 0.00000000002*cos(5.78810325925 + 31749.2351907264*$t);
   $Z3 += 0.00000000001*cos(0.84479650691 + 27043.5028831828*$t);
   $Z3 += 0.00000000001*cos(1.53587211633 + 1109.3785520934*$t);
   $Z3 += 0.00000000001*cos(0.78761433232 + 21535.9496445154*$t);
   $Z3 += 0.00000000001*cos(4.53925772521 + 51116.4243529592*$t);
   $Z3 += 0.00000000001*cos(2.76184993272 + 286966.93455731616*$t);
   $Z3 += 0.00000000001*cos(4.91700706367 + 25558.2121764796*$t);
   $Z3 += 0.00000000001*cos(5.91899219772 + 1059.3819301892*$t);
   $Z3 += 0.00000000001*cos(3.91598529129 + 25132.3033999656*$t);
   $Z3 += 0.00000000000*cos(0.95989869222 + 1589.0728952838*$t);
   $Z3 += 0.00000000001*cos(4.83279660629 + 20426.571092422*$t);
   $Z3 += 0.00000000000*cos(3.46721290636 + 77154.33087262919*$t);
   $Z3 += 0.00000000000*cos(1.74357991507 + 37410.5672398786*$t);
   $Z3 += 0.00000000000*cos(5.52724719384 + 5661.3320491522*$t);
   $Z3 += 0.00000000000*cos(2.42242414793 + 46514.4742339962*$t);
   $Z3 += 0.00000000000*cos(3.95720083996 + 4551.9534970588*$t);
   $Z3 += 0.00000000000*cos(0.55620001878 + 47623.8527860896*$t);
   $Z3 += 0.00000000000*cos(2.51891896481 + 57837.1383323006*$t);
   $Z3 += 0.00000000000*cos(4.06757949551 + 51066.427731055*$t);
   $Z3 += 0.00000000000*cos(2.83782582761 + 79373.08797681599*$t);
   $Z3 += 0.00000000000*cos(3.41743579353 + 50586.73338786459*$t);
   $Z3 += 0.00000000000*cos(0.30267502280 + 103242.23401420339*$t);
   $Z3 += 0.00000000000*cos(2.80685029082 + 28306.66024576099*$t);
   $Z3 += 0.00000000000*cos(5.70633333610 + 26107.57290247399*$t);
   $Z3 += 0.00000000000*cos(0.34541196749 + 22645.32819660879*$t);
   $Z3 += 0.00000000000*cos(0.46757777229 + 26068.2333806744*$t);
   $Z3 += 0.00000000000*cos(5.38981564610 + 32858.61374281979*$t);
   $Z3 += 0.00000000000*cos(2.34364280368 + 26084.0218062162*$t);
   $Z3 += 0.00000000000*cos(3.81913361795 + 26091.7844769322*$t);
   $Z3 += 0.00000000000*cos(5.09641110635 + 10213.285546211*$t);
   $Z3 += 0.00000000000*cos(1.30349643922 + 51646.11531805379*$t);
   $Z3 += 0.00000000000*cos(3.55639554083 + 53131.406024757*$t);
   $Z3 += 0.00000000000*cos(1.15022684382 + 25874.6040461362*$t);
   return $Z3*$t*$t*$t;
}



   function Mercury_Z4($t) // 12 terms of order 4
{
   $Z4  = 0.00000001500*cos(1.82376153122 + 26087.9031415742*$t);
   $Z4 -= 0.00000000592;
   $Z4 += 0.00000000056*cos(3.48599666355 + 52175.8062831484*$t);
   $Z4 += 0.00000000022*cos(0.17608711147 + 78263.70942472259*$t);
   $Z4 += 0.00000000009*cos(3.89825290813 + 104351.61256629678*$t);
   $Z4 += 0.00000000005*cos(1.27729295657 + 130439.51570787099*$t);
   $Z4 += 0.00000000003*cos(4.59766720323 + 156527.41884944518*$t);
   $Z4 += 0.00000000002*cos(1.46273885253 + 182615.32199101939*$t);
   $Z4 += 0.00000000001*cos(4.58832110028 + 208703.22513259358*$t);
   $Z4 += 0.00000000000*cos(4.64908450350 + 27197.2816936676*$t);
   $Z4 += 0.00000000000*cos(1.37450705839 + 234791.12827416777*$t);
   $Z4 += 0.00000000000*cos(1.42617733060 + 24978.5245894808*$t);
   return $Z4*$t*$t*$t*$t;
}



   function Mercury_Z5($t) // 7 terms of order 5
{
   $Z5  = 0.00000000045*cos(3.82268203713 + 26087.9031415742*$t);
   $Z5 -= 0.00000000043;
   $Z5 += 0.00000000003*cos(1.05444326177 + 52175.8062831484*$t);
   $Z5 += 0.00000000001*cos(4.50060145511 + 78263.70942472259*$t);
   $Z5 += 0.00000000000*cos(1.68820918658 + 104351.61256629678*$t);
   $Z5 += 0.00000000000*cos(5.41907824804 + 130439.51570787099*$t);
   $Z5 += 0.00000000000*cos(2.90287004404 + 156527.41884944518*$t);
   return $Z5*$t*$t*$t*$t*$t;
}







/*

   ******************************************************
   EARTH

   Heliocentric Coords Module - VSOP 87 Theory - Series C3

   Heliocentric rectangular X,Y,Z,R coordinates for the
   mean dynamical ecliptic and equinox of the date.

   Module Range: 2000 BC  to  6000 AD

   Written in PHP 4.3.11

   (c) Jay Tanner - NeoProgrammics - 2007 Jun 15 - Fri

   Ref:
   Planetary Theories in Rectangular and Spherical Variables
   VSOP87 Solutions

   Astronomy & Astrophysics
   vol. 202, p309-p315
   Pierre Bretagnon, Gerard Francou
   1988

   Source code provided under the provisions of
   GNU Affero General Public License, version 3.
   http://www.fsf.org/licensing/licenses/agpl-3.0.html

   Jay Tanner
   VSOP87@NeoProgrammics.com

   ******************************************************

*/


   function Earth_hXYZR($JDArg)
{

// Compute time in Julian millennia from J2000.0
   $t = ($JDArg - 2451545) / 365250;

// Compute the heliocentric coordinates for time ($t)
   $hX = Earth_X0($t) + Earth_X1($t) + Earth_X2($t)
       + Earth_X3($t) + Earth_X4($t) + Earth_X5($t);

   $hY = Earth_Y0($t) + Earth_Y1($t) + Earth_Y2($t)
       + Earth_Y3($t) + Earth_Y4($t) + Earth_Y5($t);

   $hZ = Earth_Z0($t) + Earth_Z1($t) + Earth_Z2($t)
       + Earth_Z3($t) + Earth_Z4($t);

// Compute heliocentric radius vector (distance).
   $hR = SqRt($hX*$hX + $hY*$hY + $hZ*$hZ);

// Return the heliocentric XYZR coordinates
// in a space-delimited data vector.
   return "$hX $hY $hZ $hR";

}




/*

   Earth - VSOP87 Series Version C3 - Rectangular (X,Y,Z) Coordinates

   X = X-Coordinate in AU (Astronomical Units)
   Y = Y-Coordinate in AU
   Z = Z-Coordinate in AU

   R*R = (X*X + Y*Y + Z*Z)

   HELIOCENTRIC DYNAMICAL ECLIPTIC AND EQUINOX OF THE DATE

   PHP Language
   VSOP87 Functions Source Code
   Generated By NeoProgrammics VSOP87 Code Generator
   (c) Jay Tanner 2008

   Ref:
   Planetary Theories in Rectangular and Spherical Variables
   VSOP87 Solutions

   Astronomy & Astrophysics
   vol. 202, p309-p315
   Pierre Bretagnon, Gerard Francou
   1988

   Source code provided under the provisions of
   GNU Affero General Public License, version 3.
   http://www.fsf.org/licensing/licenses/agpl-3.0.html

*/


   function Earth_X0($t) // 1007 terms of order 0
{
   $X0  = 0.99986069925*cos(1.75347045757 + 6283.3196674749*$t);
   $X0 += 0.02506324281*cos(4.93819429098 + 0.2438174835*$t);
   $X0 += 0.00835274807*cos(1.71033525539 + 12566.3955174663*$t);
   $X0 += 0.00010466796*cos(1.66721984219 + 18849.4713674577*$t);
   $X0 += 0.00003490608*cos(4.44373803231 + 6282.83203250789*$t);
   $X0 += 0.00003110838*cos(0.66875189331 + 83997.09113559539*$t);
   $X0 += 0.00002561408*cos(0.58588607490 + 529.9347825781*$t);
   $X0 += 0.00002142365*cos(1.09204474884 + 1577.0997249643*$t);
   $X0 += 0.00001709161*cos(0.49540863237 + 6279.7965491259*$t);
   $X0 += 0.00001707934*cos(6.15314019418 + 6286.84278582391*$t);
   $X0 += 0.00001442753*cos(3.47210398336 + 2353.1099712553*$t);
   $X0 += 0.00001113027*cos(3.69621650479 + 5223.4501023187*$t);
   $X0 += 0.00000934484*cos(6.07385378286 + 12036.7045523717*$t);
   $X0 += 0.00000899854*cos(3.17607463681 + 10213.5293636945*$t);
   $X0 += 0.00000566820*cos(2.15241946891 + 1059.6257476727*$t);
   $X0 += 0.00000661858*cos(1.31175222119 + 5753.6287023803*$t);
   $X0 += 0.00000739644*cos(4.36662524112 + 398.3928208917*$t);
   $X0 += 0.00000681381*cos(2.21815397940 + 4705.9761250271*$t);
   $X0 += 0.00000611293*cos(5.38470180335 + 6813.0106325695*$t);
   $X0 += 0.00000451129*cos(6.09315891204 + 5885.1706640667*$t);
   $X0 += 0.00000451985*cos(1.27931036318 + 6257.0213476751*$t);
   $X0 += 0.00000449743*cos(5.36941929064 + 6309.61798727471*$t);
   $X0 += 0.00000406291*cos(0.54369369385 + 6681.46867088311*$t);
   $X0 += 0.00000541115*cos(0.78670634299 + 775.7664288075*$t);
   $X0 += 0.00000546682*cos(1.46109463961 + 14143.7390599141*$t);
   $X0 += 0.00000513780*cos(4.43695037790 + 7860.6632099227*$t);
   $X0 += 0.00000263916*cos(5.39551752630 + 5507.7970561509*$t);
   $X0 += 0.00000217220*cos(4.51265697015 + 11507.0135872771*$t);
   $X0 += 0.00000292146*cos(2.51372357301 + 5507.3094211839*$t);
   $X0 += 0.00000227915*cos(1.23940916733 + 7058.8422787989*$t);
   $X0 += 0.00000226835*cos(3.27356345008 + 4693.75913722409*$t);
   $X0 += 0.00000255833*cos(2.26545070197 + 12168.2465140581*$t);
   $X0 += 0.00000256157*cos(1.45478864804 + 710.1768660418*$t);
   $X0 += 0.00000178679*cos(2.96645630109 + 796.5418242999*$t);
   $X0 += 0.00000178303*cos(0.40472561766 + 6283.25235717211*$t);
   $X0 += 0.00000178303*cos(6.24380795499 + 6283.3869777777*$t);
   $X0 += 0.00000155471*cos(1.62414733243 + 25132.5472174491*$t);
   $X0 += 0.00000155246*cos(1.41969109555 + 5487.0216606585*$t);
   $X0 += 0.00000207640*cos(5.85326148962 + 11790.8729061423*$t);
   $X0 += 0.00000199988*cos(4.07216830110 + 17790.08943726851*$t);
   $X0 += 0.00000128970*cos(5.21698329136 + 7079.61767429131*$t);
   $X0 += 0.00000128155*cos(4.80258701251 + 3739.0052475915*$t);
   $X0 += 0.00000152241*cos(0.86901699174 + 213.5429129215*$t);
   $X0 += 0.00000078109*cos(1.84890519772 + 8827.6340873583*$t);
   $X0 += 0.00000073503*cos(2.69517697923 + 1589.3167127673*$t);
   $X0 += 0.00000093001*cos(3.93777887053 + 9437.5191174035*$t);
   $X0 += 0.00000065526*cos(3.65568333160 + 11770.0975106499*$t);
   $X0 += 0.00000063240*cos(2.24124127597 + 6262.5442719825*$t);
   $X0 += 0.00000062961*cos(4.40729385308 + 6304.0950629673*$t);
   $X0 += 0.00000085870*cos(3.02430778985 + 167284.00540514899*$t);
   $X0 += 0.00000083233*cos(6.19780778434 + 3340.8562441833*$t);
   $X0 += 0.00000062850*cos(2.03823135444 + 7.3573644843*$t);
   $X0 += 0.00000079971*cos(2.49335782552 + 6070.0205720369*$t);
   $X0 += 0.00000060043*cos(3.39955163445 + 4137.1542509997*$t);
   $X0 += 0.00000075613*cos(4.15948190002 + 6496.6187629129*$t);
   $X0 += 0.00000053731*cos(1.56268892040 + 1194.6908277081*$t);
   $X0 += 0.00000057574*cos(3.96971590315 + 6284.2999885431*$t);
   $X0 += 0.00000057574*cos(2.67881766950 + 6282.3393464067*$t);
   $X0 += 0.00000072385*cos(4.85114866885 + 10976.83498721549*$t);
   $X0 += 0.00000051135*cos(1.31237333336 + 6290.4332144757*$t);
   $X0 += 0.00000050604*cos(5.34520074613 + 6276.2061204741*$t);
   $X0 += 0.00000049613*cos(2.64370047145 + 6127.8992680407*$t);
   $X0 += 0.00000048009*cos(4.00571885519 + 6438.7400669091*$t);
   $X0 += 0.00000059233*cos(3.78322753249 + 3154.4432674121*$t);
   $X0 += 0.00000048264*cos(4.77126143981 + 801.5771136403*$t);
   $X0 += 0.00000047368*cos(6.10696378485 + 3128.6325825793*$t);
   $X0 += 0.00000039225*cos(3.25281924495 + 8429.4850839501*$t);
   $X0 += 0.00000051307*cos(5.78448747277 + 12353.0964220283*$t);
   $X0 += 0.00000050639*cos(2.05215225623 + 11856.462468908*$t);
   $X0 += 0.00000039555*cos(4.93022654622 + 5481.4987363511*$t);
   $X0 += 0.00000042757*cos(3.64063452002 + 17260.3984721739*$t);
   $X0 += 0.00000048941*cos(0.45355151773 + 9225.7830907665*$t);
   $X0 += 0.00000048302*cos(0.60366334071 + 2544.07060239989*$t);
   $X0 += 0.00000037971*cos(3.04093006964 + 426.8420083595*$t);
   $X0 += 0.00000043643*cos(3.14688065147 + 5856.23384163189*$t);
   $X0 += 0.00000031830*cos(2.00885127406 + 2145.92159899169*$t);
   $X0 += 0.00000033959*cos(1.72230567102 + 7085.1405985987*$t);
   $X0 += 0.00000030184*cos(3.96882361878 + 10977.32262218251*$t);
   $X0 += 0.00000032562*cos(2.84647033523 + 4164.0681721295*$t);
   $X0 += 0.00000038887*cos(5.36022568355 + 13367.7288136231*$t);
   $X0 += 0.00000028515*cos(6.10933021457 + 12569.91863581531*$t);
   $X0 += 0.00000028873*cos(1.01228967614 + 3930.4535137031*$t);
   $X0 += 0.00000036633*cos(2.90798803424 + 18073.9487561337*$t);
   $X0 += 0.00000024984*cos(2.17919076736 + 26.0545023163*$t);
   $X0 += 0.00000026364*cos(0.54621601823 + 9438.0067523705*$t);
   $X0 += 0.00000030326*cos(1.99372292658 + 4535.3032544079*$t);
   $X0 += 0.00000028166*cos(0.48127325062 + 12562.8723991173*$t);
   $X0 += 0.00000028001*cos(2.82603231075 + 5088.8726572503*$t);
   $X0 += 0.00000021157*cos(5.07256589577 + 11371.9485072417*$t);
   $X0 += 0.00000023113*cos(0.85996930170 + 5223.93773728571*$t);
   $X0 += 0.00000021642*cos(4.63524157818 + 12559.2819704655*$t);
   $X0 += 0.00000019448*cos(4.40136974621 + 12565.9078824993*$t);
   $X0 += 0.00000026055*cos(0.30340363466 + 71430.45180064559*$t);
   $X0 += 0.00000018232*cos(3.41362095130 + 1747.7725955835*$t);
   $X0 += 0.00000023248*cos(6.19009992884 + 18319.7804023631*$t);
   $X0 += 0.00000022785*cos(1.16821848960 + 5753.14106741329*$t);
   $X0 += 0.00000017091*cos(0.58584172932 + 4933.4522578161*$t);
   $X0 += 0.00000019399*cos(4.43188457477 + 10447.14402212089*$t);
   $X0 += 0.00000021067*cos(3.80033599994 + 7477.7666776995*$t);
   $X0 += 0.00000021310*cos(4.65919931667 + 8031.3360805419*$t);
   $X0 += 0.00000014189*cos(1.30988546287 + 2942.7072407751*$t);
   $X0 += 0.00000013864*cos(4.59350621319 + 8636.18582124671*$t);
   $X0 += 0.00000018884*cos(2.42694418444 + 156137.71980228279*$t);
   $X0 += 0.00000017875*cos(0.15660631596 + 1592.8398311163*$t);
   $X0 += 0.00000018602*cos(0.50024326019 + 17297.9385098427*$t);
   $X0 += 0.00000013605*cos(5.19720916665 + 13096.0864825609*$t);
   $X0 += 0.00000015064*cos(3.12148038350 + 16496.6052136859*$t);
   $X0 += 0.00000012969*cos(6.06491705231 + 7633.1870771337*$t);
   $X0 += 0.00000014403*cos(1.43725922780 + 20426.8149099055*$t);
   $X0 += 0.00000013311*cos(3.60732173221 + 12139.7973265903*$t);
   $X0 += 0.00000013645*cos(5.14981984839 + 13368.2164485901*$t);
   $X0 += 0.00000011583*cos(5.46063496654 + 5331.6012612243*$t);
   $X0 += 0.00000011405*cos(4.40016025355 + 529.44714761109*$t);
   $X0 += 0.00000015057*cos(5.21434071429 + 7084.6529636317*$t);
   $X0 += 0.00000014330*cos(6.02775140319 + 7342.7015976641*$t);
   $X0 += 0.00000012375*cos(3.10941922715 + 6279.72923882311*$t);
   $X0 += 0.00000012366*cos(3.53911446993 + 6286.9100961267*$t);
   $X0 += 0.00000010016*cos(2.24542867964 + 397.9051859247*$t);
   $X0 += 0.00000012435*cos(1.06626484546 + 15110.7099373497*$t);
   $X0 += 0.00000009513*cos(1.19056065248 + 7235.0380737255*$t);
   $X0 += 0.00000010997*cos(2.19198404583 + 10989.0519750185*$t);
   $X0 += 0.00000011583*cos(4.05250824543 + 5729.7502646325*$t);
   $X0 += 0.00000009034*cos(5.42875830514 + 9623.9320941747*$t);
   $X0 += 0.00000011842*cos(4.16957446176 + 15721.0826023619*$t);
   $X0 += 0.00000008869*cos(5.04690919891 + 6148.2545874395*$t);
   $X0 += 0.00000008614*cos(1.60161199451 + 6418.38474751031*$t);
   $X0 += 0.00000010312*cos(2.59789642414 + 6836.8890703173*$t);
   $X0 += 0.00000008411*cos(2.44368872285 + 14712.5609339415*$t);
   $X0 += 0.00000007911*cos(3.15428722892 + 2119.00767786191*$t);
   $X0 += 0.00000010379*cos(4.81870974023 + 1349.62359217529*$t);
   $X0 += 0.00000010374*cos(2.40569215505 + 5486.53402569149*$t);
   $X0 += 0.00000010480*cos(2.95288921695 + 5999.4603486097*$t);
   $X0 += 0.00000007404*cos(5.41419049870 + 6040.5910635009*$t);
   $X0 += 0.00000007357*cos(3.83057351958 + 5088.3850222833*$t);
   $X0 += 0.00000009820*cos(3.69556241034 + 6567.1789863401*$t);
   $X0 += 0.00000007018*cos(1.23420241153 + 6526.04827144891*$t);
   $X0 += 0.00000009668*cos(1.92238869394 + 21228.1482060623*$t);
   $X0 += 0.00000007545*cos(1.23604547808 + 12540.0971976665*$t);
   $X0 += 0.00000007202*cos(5.88391537700 + 6245.2919948391*$t);
   $X0 += 0.00000007144*cos(0.76464656573 + 6321.3473401107*$t);
   $X0 += 0.00000008883*cos(3.89630774866 + 5642.93474619389*$t);
   $X0 += 0.00000006626*cos(0.70123567856 + 5327.7199258663*$t);
   $X0 += 0.00000007015*cos(4.13622360700 + 7860.17557495569*$t);
   $X0 += 0.00000006159*cos(0.94592300466 + 12964.5445208745*$t);
   $X0 += 0.00000007413*cos(5.03498725246 + 23543.4743221653*$t);
   $X0 += 0.00000006355*cos(5.03541099132 + 1990.9888345245*$t);
   $X0 += 0.00000007286*cos(3.58219614228 + 537.0483295789*$t);
   $X0 += 0.00000006314*cos(6.22485628824 + 951.4745887671*$t);
   $X0 += 0.00000006257*cos(4.23208117686 + 4690.7236538421*$t);
   $X0 += 0.00000006484*cos(3.20813291319 + 16730.70750707931*$t);
   $X0 += 0.00000005803*cos(6.09476150956 + 955.3559241251*$t);
   $X0 += 0.00000006391*cos(3.97601526750 + 24073.1652872599*$t);
   $X0 += 0.00000007530*cos(0.20298795021 + 10973.7995038335*$t);
   $X0 += 0.00000006921*cos(0.69969701816 + 522.8212355773*$t);
   $X0 += 0.00000007626*cos(4.29336813013 + 22004.1584523533*$t);
   $X0 += 0.00000007531*cos(3.51464444079 + 18422.8731765817*$t);
   $X0 += 0.00000006902*cos(2.75534627331 + 155.17658195069*$t);
   $X0 += 0.00000005434*cos(5.94625981048 + 7238.9194090835*$t);
   $X0 += 0.00000006223*cos(2.24396249102 + 18451.3223640495*$t);
   $X0 += 0.00000007296*cos(5.98982040270 + 16730.21987211229*$t);
   $X0 += 0.00000005833*cos(2.32930728499 + 640.1411037975*$t);
   $X0 += 0.00000007171*cos(6.14759594277 + 3929.96587873609*$t);
   $X0 += 0.00000006251*cos(2.19374266424 + 6277.7967431675*$t);
   $X0 += 0.00000006251*cos(4.45479090841 + 6288.8425917823*$t);
   $X0 += 0.00000005683*cos(3.49552871951 + 1551.2890401315*$t);
   $X0 += 0.00000006383*cos(0.64667497225 + 5216.33655531789*$t);
   $X0 += 0.00000006194*cos(3.56522608256 + 5230.5636493195*$t);
   $X0 += 0.00000005036*cos(3.83694551305 + 14314.4119305333*$t);
   $X0 += 0.00000006774*cos(1.99358309023 + 102.84895673509*$t);
   $X0 += 0.00000006224*cos(0.35231413534 + 11014.8626598513*$t);
   $X0 += 0.00000004621*cos(1.34668136864 + 553.32558535889*$t);
   $X0 += 0.00000006293*cos(2.32035231613 + 5650.5359281617*$t);
   $X0 += 0.00000006297*cos(4.40170593853 + 26088.1469590577*$t);
   $X0 += 0.00000005186*cos(0.71187314766 + 77714.01528560399*$t);
   $X0 += 0.00000006196*cos(2.58705857336 + 84672.23202702119*$t);
   $X0 += 0.00000006160*cos(4.78250009298 + 239424.63407183639*$t);
   $X0 += 0.00000005414*cos(2.99961957880 + 6180.2268932563*$t);
   $X0 += 0.00000005413*cos(3.65415218952 + 6386.4124416935*$t);
   $X0 += 0.00000004863*cos(3.76722359858 + 90280.16698558679*$t);
   $X0 += 0.00000005711*cos(4.32712250291 + 6916.1034067881*$t);
   $X0 += 0.00000005150*cos(4.10583483822 + 1577.5873599313*$t);
   $X0 += 0.00000004269*cos(2.38358764863 + 7875.91568110771*$t);
   $X0 += 0.00000004017*cos(3.78580903100 + 6254.8704800071*$t);
   $X0 += 0.00000004069*cos(2.42056660765 + 3634.37720703489*$t);
   $X0 += 0.00000003993*cos(2.86272128776 + 6311.7688549427*$t);
   $X0 += 0.00000004016*cos(5.24041079205 + 4690.23601887509*$t);
   $X0 += 0.00000005290*cos(3.34482936009 + 25158.3579022819*$t);
   $X0 += 0.00000004831*cos(2.83916520281 + 5760.7422493811*$t);
   $X0 += 0.00000003487*cos(3.51068129706 + 10447.6316570879*$t);
   $X0 += 0.00000004120*cos(3.29656037233 + 6709.91785835091*$t);
   $X0 += 0.00000004410*cos(3.80589939396 + 6805.89708556871*$t);
   $X0 += 0.00000004023*cos(6.22474978871 + 4731.7868098599*$t);
   $X0 += 0.00000003980*cos(3.31931976159 + 5856.7214765989*$t);
   $X0 += 0.00000003173*cos(5.19128151293 + 1066.7392946735*$t);
   $X0 += 0.00000003293*cos(4.01637480858 + 9917.45305702629*$t);
   $X0 += 0.00000002935*cos(3.67590980046 + 12592.6938372661*$t);
   $X0 += 0.00000002979*cos(6.20067573402 + 12566.4628277691*$t);
   $X0 += 0.00000003980*cos(1.19681754169 + 13341.9181287903*$t);
   $X0 += 0.00000003892*cos(5.52571426134 + 75.0254160508*$t);
   $X0 += 0.00000003042*cos(3.19425693799 + 18053.1733606413*$t);
   $X0 += 0.00000003608*cos(5.60952778146 + 6208.5380689076*$t);
   $X0 += 0.00000003721*cos(5.22104546686 + 5966.9277978183*$t);
   $X0 += 0.00000003581*cos(1.03995231220 + 6358.1012660422*$t);
   $X0 += 0.00000002880*cos(1.61225556126 + 10575.6505004253*$t);
   $X0 += 0.00000002806*cos(4.21551614220 + 5863.8350235997*$t);
   $X0 += 0.00000003626*cos(1.42632163253 + 6599.7115371315*$t);
   $X0 += 0.00000002794*cos(5.69269191706 + 8030.8484455749*$t);
   $X0 += 0.00000002729*cos(4.17319557124 + 3.76693583251*$t);
   $X0 += 0.00000003625*cos(3.00192313835 + 6020.2357441021*$t);
   $X0 += 0.00000003206*cos(2.15397091751 + 11506.52595231009*$t);
   $X0 += 0.00000003556*cos(4.28947576556 + 8428.9974489831*$t);
   $X0 += 0.00000002856*cos(3.33625143564 + 12721.8159169005*$t);
   $X0 += 0.00000002611*cos(2.43261867894 + 6702.8043113501*$t);
   $X0 += 0.00000002535*cos(1.58310702384 + 31415.6230674405*$t);
   $X0 += 0.00000003520*cos(5.37986371510 + 250570.91967470259*$t);
   $X0 += 0.00000003434*cos(3.64656923781 + 6546.4035908477*$t);
   $X0 += 0.00000002572*cos(4.79831420582 + 5884.68302909969*$t);
   $X0 += 0.00000002647*cos(3.81972272861 + 2352.6223362883*$t);
   $X0 += 0.00000003026*cos(5.24078074980 + 13916.2629271251*$t);
   $X0 += 0.00000002378*cos(3.62833713669 + 2389.1378379327*$t);
   $X0 += 0.00000002979*cos(0.36159339669 + 12566.32820716351*$t);
   $X0 += 0.00000003082*cos(1.77514222265 + 14945.0723560709*$t);
   $X0 += 0.00000002716*cos(3.06935389211 + 12029.5910053709*$t);
   $X0 += 0.00000002215*cos(5.26643739208 + 13362.6935242827*$t);
   $X0 += 0.00000003032*cos(4.76553829426 + 29088.5675985015*$t);
   $X0 += 0.00000002421*cos(3.15326621321 + 11015.3502948183*$t);
   $X0 += 0.00000002275*cos(5.95465292259 + 6077.1341190377*$t);
   $X0 += 0.00000002466*cos(5.59166943387 + 16200.52890701769*$t);
   $X0 += 0.00000002529*cos(5.94709131586 + 12043.8180993725*$t);
   $X0 += 0.00000002015*cos(2.48432239832 + 6262.96434807611*$t);
   $X0 += 0.00000002218*cos(0.69324011482 + 6489.5052159121*$t);
   $X0 += 0.00000002005*cos(4.16423892775 + 6303.6749868737*$t);
   $X0 += 0.00000002010*cos(4.11254992503 + 4700.87268422489*$t);
   $X0 += 0.00000002641*cos(4.28275048815 + 4590.6663630055*$t);
   $X0 += 0.00000002438*cos(6.12910205560 + 149.8070146181*$t);
   $X0 += 0.00000001893*cos(0.98057464900 + 6279.4383321169*$t);
   $X0 += 0.00000001893*cos(5.66795892365 + 6287.2010028329*$t);
   $X0 += 0.00000002156*cos(2.00902236290 + 18139.5383188994*$t);
   $X0 += 0.00000002071*cos(4.32179144410 + 11926.4982311523*$t);
   $X0 += 0.00000002395*cos(0.13148357341 + 1162.7185218913*$t);
   $X0 += 0.00000001954*cos(0.37275781655 + 13518.1139237169*$t);
   $X0 += 0.00000002006*cos(0.31048315304 + 3154.9309023791*$t);
   $X0 += 0.00000001776*cos(0.14855204887 + 13521.9952590749*$t);
   $X0 += 0.00000001964*cos(0.25103153440 + 4686.6455902233*$t);
   $X0 += 0.00000002332*cos(4.40649103371 + 10022.0810975829*$t);
   $X0 += 0.00000001863*cos(5.26030609599 + 242.97242145751*$t);
   $X0 += 0.00000001928*cos(4.59250450967 + 5746.5151553795*$t);
   $X0 += 0.00000002436*cos(4.40770823480 + 95143.37673846159*$t);
   $X0 += 0.00000002163*cos(1.41797907369 + 6037.4880212455*$t);
   $X0 += 0.00000002250*cos(2.39728922664 + 12669.4882916849*$t);
   $X0 += 0.00000002052*cos(5.23006608728 + 6529.1513137043*$t);
   $X0 += 0.00000001886*cos(3.61213584641 + 7238.4317741165*$t);
   $X0 += 0.00000002170*cos(5.49596575387 + 19651.2922985815*$t);
   $X0 += 0.00000002058*cos(4.69821533065 + 23013.7833570707*$t);
   $X0 += 0.00000001774*cos(2.08482289391 + 6820.1241795703*$t);
   $X0 += 0.00000001841*cos(0.34653293659 + 65147.37595065419*$t);
   $X0 += 0.00000001578*cos(5.64311385009 + 4292.5746504339*$t);
   $X0 += 0.00000001577*cos(0.80779559190 + 7632.6994421667*$t);
   $X0 += 0.00000001541*cos(1.80359816798 + 13119.9649203087*$t);
   $X0 += 0.00000002012*cos(0.36995312887 + 4292.0870154669*$t);
   $X0 += 0.00000001543*cos(0.68755069426 + 6293.95633282471*$t);
   $X0 += 0.00000001543*cos(5.96098287839 + 6272.6830021251*$t);
   $X0 += 0.00000001513*cos(0.65061890512 + 20.5315780089*$t);
   $X0 += 0.00000001489*cos(2.95205535091 + 110.45013870291*$t);
   $X0 += 0.00000001639*cos(4.44076102657 + 17655.0243572331*$t);
   $X0 += 0.00000001443*cos(2.26329309493 + 1052.51220067191*$t);
   $X0 += 0.00000001337*cos(3.06401553951 + 2544.5582373669*$t);
   $X0 += 0.00000001797*cos(6.18654785499 + 33018.7772947211*$t);
   $X0 += 0.00000001488*cos(5.67450527487 + 76.50988875911*$t);
   $X0 += 0.00000001453*cos(5.46078654600 + 633.0275567967*$t);
   $X0 += 0.00000001349*cos(5.75097749599 + 6016.7126257531*$t);
   $X0 += 0.00000001477*cos(4.14004774348 + 12779.6946129043*$t);
   $X0 += 0.00000001688*cos(3.20087734666 + 18875.2820522905*$t);
   $X0 += 0.00000001491*cos(6.22974184712 + 9411.7084325707*$t);
   $X0 += 0.00000001471*cos(5.75779240419 + 18636.1722720197*$t);
   $X0 += 0.00000001263*cos(0.91728971350 + 12573.5090644671*$t);
   $X0 += 0.00000001685*cos(5.70877589839 + 25934.3681485729*$t);
   $X0 += 0.00000001375*cos(1.08927160683 + 2378.9206560881*$t);
   $X0 += 0.00000001641*cos(6.27664360507 + 6.86972951729*$t);
   $X0 += 0.00000001200*cos(1.47556274227 + 11499.9000402763*$t);
   $X0 += 0.00000001272*cos(0.89738913067 + 6549.9267091967*$t);
   $X0 += 0.00000001214*cos(1.22660393299 + 10973.3118688665*$t);
   $X0 += 0.00000001406*cos(5.67596795657 + 11790.3852711753*$t);
   $X0 += 0.00000001524*cos(4.27066787572 + 18209.5740811437*$t);
   $X0 += 0.00000001149*cos(3.01961706159 + 10177.5014970171*$t);
   $X0 += 0.00000001547*cos(4.02915828753 + 11926.01059618529*$t);
   $X0 += 0.00000001460*cos(4.97235137781 + 246.0754637129*$t);
   $X0 += 0.00000001323*cos(1.38571450422 + 8661.9965060795*$t);
   $X0 += 0.00000001252*cos(1.41166880118 + 6993.2527160332*$t);
   $X0 += 0.00000001459*cos(5.29807880334 + 38.3768531213*$t);
   $X0 += 0.00000001235*cos(2.84096519816 + 24357.0246061251*$t);
   $X0 += 0.00000001028*cos(2.51029997292 + 6112.6467968557*$t);
   $X0 += 0.00000001330*cos(1.00065727327 + 2146.4092339587*$t);
   $X0 += 0.00000001312*cos(1.10091584033 + 12491.613918899*$t);
   $X0 += 0.00000001246*cos(4.66430542078 + 5429.63565075589*$t);
   $X0 += 0.00000000992*cos(4.13817744468 + 6453.9925380941*$t);
   $X0 += 0.00000000988*cos(0.97969587833 + 8274.0646845159*$t);
   $X0 += 0.00000001365*cos(6.11146862666 + 11371.4608722747*$t);
   $X0 += 0.00000001020*cos(2.82319817767 + 4732.2744448269*$t);
   $X0 += 0.00000001191*cos(3.43008039922 + 6290.36590417291*$t);
   $X0 += 0.00000001191*cos(3.21845317343 + 6276.2734307769*$t);
   $X0 += 0.00000000978*cos(4.73852758176 + 6247.2918007975*$t);
   $X0 += 0.00000000971*cos(1.91000724147 + 6319.34753415231*$t);
   $X0 += 0.00000000962*cos(2.63568544853 + 12565.4151963981*$t);
   $X0 += 0.00000001025*cos(2.21768401152 + 12545.6201219739*$t);
   $X0 += 0.00000000990*cos(0.39039931315 + 4694.2467721911*$t);
   $X0 += 0.00000000964*cos(1.79192739930 + 3893.93801205869*$t);
   $X0 += 0.00000000944*cos(5.86379546667 + 6259.44122972711*$t);
   $X0 += 0.00000000944*cos(0.78473810598 + 6307.1981052227*$t);
   $X0 += 0.00000001092*cos(3.92065202481 + 17797.2029842693*$t);
   $X0 += 0.00000001132*cos(1.00628522970 + 17782.9758902677*$t);
   $X0 += 0.00000000859*cos(1.48109589441 + 1692.40948698591*$t);
   $X0 += 0.00000001213*cos(4.04235990086 + 82576.73740351178*$t);
   $X0 += 0.00000000942*cos(5.22790737110 + 6298.57213865991*$t);
   $X0 += 0.00000000942*cos(1.42062620155 + 6268.0671962899*$t);
   $X0 += 0.00000001045*cos(0.46195584294 + 15508.8589407579*$t);
   $X0 += 0.00000000851*cos(5.19678367921 + 6173.1133462555*$t);
   $X0 += 0.00000000850*cos(1.45204280805 + 6393.5259886943*$t);
   $X0 += 0.00000000895*cos(2.40089211752 + 3904.1551939033*$t);
   $X0 += 0.00000001020*cos(0.71539416375 + 220.6564599223*$t);
   $X0 += 0.00000000845*cos(5.75499771232 + 17256.8753538249*$t);
   $X0 += 0.00000000863*cos(4.43158774725 + 949.4194264533*$t);
   $X0 += 0.00000000853*cos(2.75614337272 + 16201.0165419847*$t);
   $X0 += 0.00000000811*cos(6.13459778682 + 4803.9654584435*$t);
   $X0 += 0.00000000966*cos(4.14597367445 + 206.42936592071*$t);
   $X0 += 0.00000000917*cos(2.47007329129 + 149854.64395229139*$t);
   $X0 += 0.00000001083*cos(1.32514794865 + 36948.9869909407*$t);
   $X0 += 0.00000000779*cos(3.58339682996 + 2648.6986429565*$t);
   $X0 += 0.00000000964*cos(0.77431665261 + 796.0541893329*$t);
   $X0 += 0.00000000995*cos(0.25423339373 + 11403.9208130585*$t);
   $X0 += 0.00000000962*cos(3.92658368218 + 12567.37583853451*$t);
   $X0 += 0.00000000779*cos(0.45683397157 + 10213.0417287275*$t);
   $X0 += 0.00000000980*cos(4.62590378445 + 22805.4917485101*$t);
   $X0 += 0.00000000930*cos(2.22316684933 + 2787.2868413409*$t);
   $X0 += 0.00000000829*cos(5.09643797682 + 5120.35732810009*$t);
   $X0 += 0.00000000733*cos(2.60949856930 + 10575.1628654583*$t);
   $X0 += 0.00000000743*cos(3.79905998257 + 7834.3648901229*$t);
   $X0 += 0.00000000818*cos(1.64368328490 + 5572.8989839496*$t);
   $X0 += 0.00000000905*cos(3.81538816339 + 6284.8041401832*$t);
   $X0 += 0.00000000905*cos(2.83314540925 + 6281.8351947666*$t);
   $X0 += 0.00000000798*cos(2.62501051906 + 12410.9751180321*$t);
   $X0 += 0.00000000817*cos(0.52083502825 + 12416.8323203317*$t);
   $X0 += 0.00000000884*cos(0.84319899817 + 22483.60475700909*$t);
   $X0 += 0.00000000755*cos(3.82941056281 + 4060.97539791089*$t);
   $X0 += 0.00000000651*cos(3.93683821152 + 17259.91083720689*$t);
   $X0 += 0.00000000616*cos(5.70474451820 + 1596.43025976811*$t);
   $X0 += 0.00000000651*cos(2.41491885723 + 1748.2602305505*$t);
   $X0 += 0.00000000723*cos(3.06743374106 + 161000.92955515759*$t);
   $X0 += 0.00000000784*cos(0.87758060898 + 4907.05823266209*$t);
   $X0 += 0.00000000784*cos(2.20169255256 + 7234.5504387585*$t);
   $X0 += 0.00000000785*cos(3.43654258027 + 846.3266522347*$t);
   $X0 += 0.00000000611*cos(1.53152362816 + 853.4401992355*$t);
   $X0 += 0.00000000576*cos(1.22798260830 + 12323.6669134923*$t);
   $X0 += 0.00000000570*cos(4.54665961586 + 12587.1709129587*$t);
   $X0 += 0.00000000576*cos(3.06908128494 + 13915.77529215809*$t);
   $X0 += 0.00000000770*cos(5.60311452827 + 6069.53293706989*$t);
   $X0 += 0.00000000773*cos(0.21363273558 + 6133.2688353733*$t);
   $X0 += 0.00000000581*cos(1.34025743471 + 11933.6117781531*$t);
   $X0 += 0.00000000719*cos(0.90840576142 + 15720.5949673949*$t);
   $X0 += 0.00000000578*cos(4.26997200274 + 8662.48414104651*$t);
   $X0 += 0.00000000536*cos(6.06660590951 + 18852.9944858067*$t);
   $X0 += 0.00000000633*cos(1.17258360285 + 52176.0501006319*$t);
   $X0 += 0.00000000581*cos(2.92798889289 + 5334.1440585051*$t);
   $X0 += 0.00000000532*cos(4.58762178742 + 18842.3578204569*$t);
   $X0 += 0.00000000657*cos(3.25871870022 + 5849.1202946311*$t);
   $X0 += 0.00000000599*cos(2.11793230261 + 6151.7777057885*$t);
   $X0 += 0.00000000540*cos(3.24412649701 + 6286.6060248927*$t);
   $X0 += 0.00000000540*cos(3.40440707563 + 6280.0333100571*$t);
   $X0 += 0.00000000628*cos(0.31668062965 + 17298.4261448097*$t);
   $X0 += 0.00000000532*cos(4.41105068397 + 11514.1271342779*$t);
   $X0 += 0.00000000513*cos(3.70291605445 + 12456.1891962469*$t);
   $X0 += 0.00000000649*cos(4.90568628758 + 11764.5745863425*$t);
   $X0 += 0.00000000582*cos(4.53092429894 + 6414.8616291613*$t);
   $X0 += 0.00000000517*cos(4.06133972007 + 3340.36860921629*$t);
   $X0 += 0.00000000609*cos(3.94494306228 + 10420.23010099111*$t);
   $X0 += 0.00000000615*cos(4.71270964329 + 10983.94853421629*$t);
   $X0 += 0.00000000622*cos(5.53858241889 + 5326.5428765373*$t);
   $X0 += 0.00000000536*cos(3.70224967645 + 7232.49527644471*$t);
   $X0 += 0.00000000516*cos(2.92986975188 + 433.9555553603*$t);
   $X0 += 0.00000000613*cos(1.79095811234 + 10969.72144021469*$t);
   $X0 += 0.00000000583*cos(6.15311267171 + 5863.3473886327*$t);
   $X0 += 0.00000000636*cos(6.06042539245 + 26735.7014447297*$t);
   $X0 += 0.00000000665*cos(2.74682435671 + 40879.1966871603*$t);
   $X0 += 0.00000000583*cos(2.82325235087 + 12592.2062022991*$t);
   $X0 += 0.00000000479*cos(1.33548820034 + 5547.4431539431*$t);
   $X0 += 0.00000000571*cos(2.58673571504 + 6062.9070250361*$t);
   $X0 += 0.00000000565*cos(3.92092198777 + 4171.1817191303*$t);
   $X0 += 0.00000000462*cos(1.96630641591 + 3104.6862419403*$t);
   $X0 += 0.00000000551*cos(4.03874430699 + 6503.7323099137*$t);
   $X0 += 0.00000000504*cos(5.14547341413 + 15670.83794192309*$t);
   $X0 += 0.00000000535*cos(6.12277243019 + 173567.08125514039*$t);
   $X0 += 0.00000000460*cos(3.18401450277 + 3495.78900865049*$t);
   $X0 += 0.00000000449*cos(5.09833649130 + 4274.27449334889*$t);
   $X0 += 0.00000000453*cos(3.57566005736 + 9387.76209193169*$t);
   $X0 += 0.00000000527*cos(6.24827137997 + 24602.8562523545*$t);
   $X0 += 0.00000000502*cos(0.97407216668 + 12490.1294461907*$t);
   $X0 += 0.00000000603*cos(0.85487069842 + 322711.54834138996*$t);
   $X0 += 0.00000000532*cos(1.83659178149 + 5120.8449630671*$t);
   $X0 += 0.00000000530*cos(0.43491203890 + 18845.9482491087*$t);
   $X0 += 0.00000000431*cos(5.31145947559 + 7019.19618100671*$t);
   $X0 += 0.00000000465*cos(5.83298981980 + 8827.14645239129*$t);
   $X0 += 0.00000000559*cos(1.71560880010 + 1582.2031657665*$t);
   $X0 += 0.00000000485*cos(6.13610027053 + 29296.8592070621*$t);
   $X0 += 0.00000000584*cos(0.07138791653 + 72850.80553272919*$t);
   $X0 += 0.00000000550*cos(3.95560592819 + 213.0552779545*$t);
   $X0 += 0.00000000481*cos(1.86946119801 + 14.47091148511*$t);
   $X0 += 0.00000000569*cos(2.95240683895 + 97238.871361971*$t);
   $X0 += 0.00000000551*cos(1.64539130884 + 14313.9242955663*$t);
   $X0 += 0.00000000389*cos(3.43047350256 + 6245.1866318371*$t);
   $X0 += 0.00000000389*cos(3.21806007009 + 6321.4527031127*$t);
   $X0 += 0.00000000388*cos(1.13963224225 + 6297.5467614765*$t);
   $X0 += 0.00000000388*cos(5.50890133040 + 6269.0925734733*$t);
   $X0 += 0.00000000515*cos(1.67075264819 + 12320.56387123691*$t);
   $X0 += 0.00000000394*cos(6.10437801091 + 4156.95462512869*$t);
   $X0 += 0.00000000491*cos(5.96788884074 + 1479.11039154791*$t);
   $X0 += 0.00000000440*cos(0.97316688838 + 5650.0482931947*$t);
   $X0 += 0.00000000382*cos(3.07660544855 + 9917.9406919933*$t);
   $X0 += 0.00000000503*cos(4.65767245093 + 17157.3056979553*$t);
   $X0 += 0.00000000439*cos(4.82562869150 + 233141.55822184499*$t);
   $X0 += 0.00000000381*cos(2.02484381212 + 14143.2514249471*$t);
   $X0 += 0.00000000371*cos(4.59067170554 + 5643.4223811609*$t);
   $X0 += 0.00000000516*cos(5.61424407240 + 135.30889751891*$t);
   $X0 += 0.00000000417*cos(3.74461559548 + 13760.8425276909*$t);
   $X0 += 0.00000000483*cos(4.41750912984 + 9779.3524936089*$t);
   $X0 += 0.00000000381*cos(4.36951119319 + 14919.2616712381*$t);
   $X0 += 0.00000000378*cos(4.51952349691 + 17267.5120191747*$t);
   $X0 += 0.00000000393*cos(1.56678612595 + 7872.39256275871*$t);
   $X0 += 0.00000000376*cos(4.45581102621 + 13517.62628874989*$t);
   $X0 += 0.00000000365*cos(2.11020607854 + 6923.2169537889*$t);
   $X0 += 0.00000000371*cos(5.76233679446 + 13625.7774476555*$t);
   $X0 += 0.00000000355*cos(5.24317955258 + 10874.2298479639*$t);
   $X0 += 0.00000000499*cos(2.72562606986 + 161710.8626037159*$t);
   $X0 += 0.00000000379*cos(0.81621597880 + 3185.43584474911*$t);
   $X0 += 0.00000000471*cos(4.78294711940 + 11712.71150074729*$t);
   $X0 += 0.00000000417*cos(3.33073591690 + 22779.6810636773*$t);
   $X0 += 0.00000000485*cos(1.35444198321 + 12528.26248182851*$t);
   $X0 += 0.00000000358*cos(3.49115357527 + 6295.0490203109*$t);
   $X0 += 0.00000000358*cos(3.15737999738 + 6271.5903146389*$t);
   $X0 += 0.00000000365*cos(3.62050447597 + 6836.40143535029*$t);
   $X0 += 0.00000000351*cos(2.21887831196 + 11617.21990849651*$t);
   $X0 += 0.00000000351*cos(2.96356670458 + 205.9417309537*$t);
   $X0 += 0.00000000429*cos(0.79790217165 + 3894.4256470257*$t);
   $X0 += 0.00000000388*cos(0.49080052992 + 956.53297345411*$t);
   $X0 += 0.00000000464*cos(0.69464877419 + 23581.5019948011*$t);
   $X0 += 0.00000000396*cos(2.35908361907 + 5231.0512842865*$t);
   $X0 += 0.00000000421*cos(4.82739444320 + 7445.7943718827*$t);
   $X0 += 0.00000000366*cos(0.62612486089 + 17253.2849251731*$t);
   $X0 += 0.00000000360*cos(0.81896878015 + 21393.7857873411*$t);
   $X0 += 0.00000000336*cos(1.41330667750 + 6279.3875142118*$t);
   $X0 += 0.00000000336*cos(5.23522689515 + 6287.251820738*$t);
   $X0 += 0.00000000326*cos(4.93781139708 + 1059.1381127057*$t);
   $X0 += 0.00000000395*cos(0.53943839302 + 5642.4420600927*$t);
   $X0 += 0.00000000450*cos(0.23150267191 + 1385.3177574676*$t);
   $X0 += 0.00000000363*cos(4.29615345291 + 22484.0923919761*$t);
   $X0 += 0.00000000399*cos(0.80548840004 + 16858.7263504167*$t);
   $X0 += 0.00000000316*cos(4.03075171777 + 20995.6367839329*$t);
   $X0 += 0.00000000430*cos(2.39021484637 + 19650.8046636145*$t);
   $X0 += 0.00000000363*cos(4.29069778349 + 7335.5880506633*$t);
   $X0 += 0.00000000317*cos(1.52889301942 + 11769.6098756829*$t);
   $X0 += 0.00000000395*cos(5.39427984473 + 5905.94605955911*$t);
   $X0 += 0.00000000324*cos(3.70774573147 + 37.7838551523*$t);
   $X0 += 0.00000000299*cos(3.86306301151 + 641.12142486571*$t);
   $X0 += 0.00000000304*cos(0.15985948407 + 5750.1055840313*$t);
   $X0 += 0.00000000298*cos(3.82763680613 + 1350.1112271423*$t);
   $X0 += 0.00000000414*cos(4.17890604161 + 44809.4063833799*$t);
   $X0 += 0.00000000339*cos(4.29176984719 + 3.6883357796*$t);
   $X0 += 0.00000000351*cos(5.93395494462 + 12345.9828750275*$t);
   $X0 += 0.00000000362*cos(0.42214982034 + 21953.91379191449*$t);
   $X0 += 0.00000000304*cos(4.93815385347 + 29826.5501721567*$t);
   $X0 += 0.00000000395*cos(0.07721971830 + 4176.2851599325*$t);
   $X0 += 0.00000000320*cos(2.49901226571 + 10818.3791043993*$t);
   $X0 += 0.00000000384*cos(4.01872268681 + 10177.01386205009*$t);
   $X0 += 0.00000000279*cos(0.96409855547 + 10970.2090751817*$t);
   $X0 += 0.00000000365*cos(1.25425361388 + 6660.6932753907*$t);
   $X0 += 0.00000000377*cos(0.82514906124 + 29864.5778447925*$t);
   $X0 += 0.00000000275*cos(5.73507538514 + 20597.4877805247*$t);
   $X0 += 0.00000000316*cos(2.59688143484 + 316.6356871401*$t);
   $X0 += 0.00000000339*cos(6.10829032638 + 6924.1972748571*$t);
   $X0 += 0.00000000386*cos(2.29032835664 + 2636.9692901205*$t);
   $X0 += 0.00000000273*cos(1.09519308783 + 26709.8907598969*$t);
   $X0 += 0.00000000314*cos(1.47223989608 + 14945.5599910379*$t);
   $X0 += 0.00000000281*cos(4.86653427016 + 16858.23871544969*$t);
   $X0 += 0.00000000287*cos(3.30150277561 + 18073.46112116669*$t);
   $X0 += 0.00000000265*cos(5.11010310272 + 19379.1623325523*$t);
   $X0 += 0.00000000325*cos(2.49278036693 + 12360.2099690291*$t);
   $X0 += 0.00000000362*cos(1.18157001338 + 30665.9111409493*$t);
   $X0 += 0.00000000269*cos(0.20537228942 + 6816.53375091851*$t);
   $X0 += 0.00000000344*cos(3.51559724211 + 6147.69434246491*$t);
   $X0 += 0.00000000255*cos(4.94623609008 + 1376.0176173293*$t);
   $X0 += 0.00000000334*cos(3.13303124260 + 6418.9449924849*$t);
   $X0 += 0.00000000316*cos(5.54205330844 + 6055.8435346859*$t);
   $X0 += 0.00000000283*cos(4.23095234926 + 28287.2343023447*$t);
   $X0 += 0.00000000305*cos(4.26252488384 + 16522.4158985187*$t);
   $X0 += 0.00000000270*cos(0.05215867170 + 283.6155013817*$t);
   $X0 += 0.00000000243*cos(2.54680473739 + 6255.9181113781*$t);
   $X0 += 0.00000000243*cos(4.10172883526 + 6310.7212235717*$t);
   $X0 += 0.00000000296*cos(0.07899510796 + 6129.5408569901*$t);
   $X0 += 0.00000000301*cos(1.10648037942 + 6510.7958002639*$t);
   $X0 += 0.00000000329*cos(1.15683223901 + 377.6174253993*$t);
   $X0 += 0.00000000267*cos(3.42011505320 + 24705.9490265731*$t);
   $X0 += 0.00000000285*cos(2.50494570372 + 5469.7693835151*$t);
   $X0 += 0.00000000286*cos(0.28647364691 + 6437.0984779597*$t);
   $X0 += 0.00000000311*cos(3.21063889136 + 11720.3126827151*$t);
   $X0 += 0.00000000319*cos(4.71059907821 + 169379.5000286584*$t);
   $X0 += 0.00000000241*cos(2.99439330770 + 632.5399218297*$t);
   $X0 += 0.00000000229*cos(1.01522459538 + 1265.81129610991*$t);
   $X0 += 0.00000000235*cos(3.36978882786 + 4487.57358878689*$t);
   $X0 += 0.00000000253*cos(0.51356087882 + 4377.3672675675*$t);
   $X0 += 0.00000000277*cos(1.58719268637 + 419.2408263917*$t);
   $X0 += 0.00000000234*cos(5.12574585310 + 11713.1991357143*$t);
   $X0 += 0.00000000216*cos(5.01445941939 + 10454.25756912169*$t);
   $X0 += 0.00000000241*cos(5.44670796756 + 103.3365917021*$t);
   $X0 += 0.00000000214*cos(2.09481715597 + 2222.1004520805*$t);
   $X0 += 0.00000000213*cos(3.79438383373 + 30356.2411372513*$t);
   $X0 += 0.00000000240*cos(2.57405501441 + 6309.1303523077*$t);
   $X0 += 0.00000000212*cos(2.36637350989 + 262.84010588929*$t);
   $X0 += 0.00000000259*cos(3.21725974670 + 6283.56348495841*$t);
   $X0 += 0.00000000259*cos(3.43127382595 + 6283.0758499914*$t);
   $X0 += 0.00000000222*cos(1.42789121927 + 10440.03047512009*$t);
   $X0 += 0.00000000201*cos(5.11101601554 + 5746.0275204125*$t);
   $X0 += 0.00000000274*cos(3.85387261790 + 23581.0143598341*$t);
   $X0 += 0.00000000243*cos(4.14346519403 + 7096.8699514347*$t);
   $X0 += 0.00000000234*cos(5.23697266935 + 5573.3866189166*$t);
   $X0 += 0.00000000203*cos(0.00533428175 + 16460.08971204149*$t);
   $X0 += 0.00000000246*cos(5.83654379480 + 8672.21368792411*$t);
   $X0 += 0.00000000216*cos(4.93054195434 + 5437.23683272371*$t);
   $X0 += 0.00000000210*cos(5.82498971470 + 9381.2034902007*$t);
   $X0 += 0.00000000199*cos(0.93528415346 + 11216.5281078075*$t);
   $X0 += 0.00000000270*cos(0.40019527935 + 284.1031363487*$t);
   $X0 += 0.00000000207*cos(3.06628694399 + 12562.80508881451*$t);
   $X0 += 0.00000000212*cos(1.75471606720 + 7129.4025022261*$t);
   $X0 += 0.00000000269*cos(1.52668960070 + 70755.31090921978*$t);
   $X0 += 0.00000000221*cos(3.40184768346 + 77713.52765063698*$t);
   $X0 += 0.00000000225*cos(4.01628844723 + 5635.8211991931*$t);
   $X0 += 0.00000000197*cos(3.36122016267 + 14712.07329897449*$t);
   $X0 += 0.00000000192*cos(2.04581063866 + 17272.1278250099*$t);
   $X0 += 0.00000000241*cos(5.09422826000 + 15907.0079441661*$t);
   $X0 += 0.00000000261*cos(5.60213035381 + 48739.6160795995*$t);
   $X0 += 0.00000000255*cos(0.53150571182 + 6206.5659612323*$t);
   $X0 += 0.00000000256*cos(2.30148639986 + 224.5886131854*$t);
   $X0 += 0.00000000182*cos(4.41917983264 + 18848.98373249069*$t);
   $X0 += 0.00000000250*cos(4.24183231344 + 5934.39524702691*$t);
   $X0 += 0.00000000196*cos(2.21911266897 + 16460.5773470085*$t);
   $X0 += 0.00000000187*cos(4.75761261391 + 22003.6708173863*$t);
   $X0 += 0.00000000229*cos(0.16732487654 + 2942.21960580809*$t);
   $X0 += 0.00000000175*cos(5.41173900219 + 11614.6771112157*$t);
   $X0 += 0.00000000187*cos(5.47425531847 + 9778.8648586419*$t);
   $X0 += 0.00000000175*cos(4.58223305763 + 3744.5835282543*$t);
   $X0 += 0.00000000241*cos(0.28794939753 + 8390.3541750173*$t);
   $X0 += 0.00000000236*cos(2.02485303450 + 1.7282901918*$t);
   $X0 += 0.00000000217*cos(5.02460023035 + 17996.2749857057*$t);
   $X0 += 0.00000000173*cos(1.08399000143 + 6275.71848550709*$t);
   $X0 += 0.00000000191*cos(6.27756282518 + 394.86970254271*$t);
   $X0 += 0.00000000231*cos(2.60472823534 + 34596.1208371689*$t);
   $X0 += 0.00000000169*cos(5.11378923747 + 6438.2524319421*$t);
   $X0 += 0.00000000235*cos(3.47396101480 + 17256.38771885789*$t);
   $X0 += 0.00000000188*cos(5.59662915440 + 401.91593924071*$t);
   $X0 += 0.00000000217*cos(4.97442863783 + 10984.4361691833*$t);
   $X0 += 0.00000000230*cos(2.40692408305 + 6632.2440879229*$t);
   $X0 += 0.00000000208*cos(0.72059454942 + 11087.5289434019*$t);
   $X0 += 0.00000000165*cos(2.37182548424 + 743.23387801611*$t);
   $X0 += 0.00000000177*cos(4.73067975820 + 4796.85191144269*$t);
   $X0 += 0.00000000208*cos(4.59043597135 + 3097.64000524229*$t);
   $X0 += 0.00000000167*cos(3.97387947664 + 5714.4977934475*$t);
   $X0 += 0.00000000198*cos(3.26732644351 + 5539.8419719753*$t);
   $X0 += 0.00000000169*cos(3.73983013574 + 12132.6837795895*$t);
   $X0 += 0.00000000214*cos(4.34216809137 + 24492.6499311351*$t);
   $X0 += 0.00000000172*cos(1.25880044898 + 6233.5626420031*$t);
   $X0 += 0.00000000172*cos(5.38973312367 + 6333.0766929467*$t);
   $X0 += 0.00000000223*cos(4.99990847060 + 266.85085920531*$t);
   $X0 += 0.00000000159*cos(3.90246086071 + 13199.1792567795*$t);
   $X0 += 0.00000000219*cos(5.40805183208 + 10344.05124790229*$t);
   $X0 += 0.00000000207*cos(3.49598218672 + 12569.9859461181*$t);
   $X0 += 0.00000000176*cos(4.05199209274 + 12012.8261146239*$t);
   $X0 += 0.00000000170*cos(6.01039539440 + 6294.36536773881*$t);
   $X0 += 0.00000000170*cos(0.63813817825 + 6272.273967211*$t);
   $X0 += 0.00000000205*cos(5.87166506948 + 13119.47728534169*$t);
   $X0 += 0.00000000167*cos(5.54385526640 + 17686.9966630499*$t);
   $X0 += 0.00000000210*cos(0.88175440791 + 13521.50762410789*$t);
   $X0 += 0.00000000163*cos(1.46796231870 + 802.0647486073*$t);
   $X0 += 0.00000000159*cos(4.85018558111 + 5017.2645538815*$t);
   $X0 += 0.00000000148*cos(0.75075299347 + 419.72846135871*$t);
   $X0 += 0.00000000165*cos(0.87966880312 + 20199.3387771165*$t);
   $X0 += 0.00000000196*cos(3.57999903444 + 33326.33491569069*$t);
   $X0 += 0.00000000146*cos(0.39484750798 + 19800.7021387413*$t);
   $X0 += 0.00000000148*cos(2.67465416652 + 6852.1415415023*$t);
   $X0 += 0.00000000146*cos(0.23882077577 + 17370.6047933933*$t);
   $X0 += 0.00000000143*cos(0.85202349514 + 5618.5636223449*$t);
   $X0 += 0.00000000142*cos(5.37141800193 + 17654.5367222661*$t);
   $X0 += 0.00000000140*cos(1.22241558577 + 2008.8013566425*$t);
   $X0 += 0.00000000153*cos(1.79747194391 + 5436.7491977567*$t);
   $X0 += 0.00000000162*cos(3.70215026876 + 775.2787938405*$t);
   $X0 += 0.00000000162*cos(4.78754155685 + 12552.1684234647*$t);
   $X0 += 0.00000000143*cos(2.32218951292 + 5010.1510068807*$t);
   $X0 += 0.00000000188*cos(3.68702080121 + 28.6930049513*$t);
   $X0 += 0.00000000155*cos(6.15650399253 + 11610.1063614957*$t);
   $X0 += 0.00000000174*cos(5.71964679538 + 20452.6255947383*$t);
   $X0 += 0.00000000179*cos(5.30407615411 + 27511.2240560537*$t);
   $X0 += 0.00000000135*cos(0.07815962516 + 12431.3304374309*$t);
   $X0 += 0.00000000164*cos(5.60920106543 + 28767.1682419675*$t);
   $X0 += 0.00000000134*cos(5.44099179137 + 16840.9138282987*$t);
   $X0 += 0.00000000174*cos(4.27755511799 + 19805.0711090663*$t);
   $X0 += 0.00000000146*cos(1.55848605346 + 12701.4605975017*$t);
   $X0 += 0.00000000137*cos(2.60101256262 + 11.2895177474*$t);
   $X0 += 0.00000000167*cos(1.27059509188 + 17473.6975676119*$t);
   $X0 += 0.00000000152*cos(4.19104806587 + 16627.6147328607*$t);
   $X0 += 0.00000000124*cos(5.79651030598 + 6948.0757126049*$t);
   $X0 += 0.00000000135*cos(3.38729000615 + 3531.2844328163*$t);
   $X0 += 0.00000000172*cos(4.94261447200 + 167959.14629657479*$t);
   $X0 += 0.00000000123*cos(5.21069118841 + 23013.29572210369*$t);
   $X0 += 0.00000000122*cos(5.69263982892 + 3583.5848481573*$t);
   $X0 += 0.00000000170*cos(1.45223428169 + 333857.83394425618*$t);
   $X0 += 0.00000000155*cos(4.05745332726 + 6058.48723680599*$t);
   $X0 += 0.00000000120*cos(1.19140907580 + 12809.12412144031*$t);
   $X0 += 0.00000000141*cos(5.52535299558 + 162420.79565227419*$t);
   $X0 += 0.00000000120*cos(5.84076902832 + 12528.3678448305*$t);
   $X0 += 0.00000000123*cos(6.20892018031 + 25933.8805136059*$t);
   $X0 += 0.00000000144*cos(2.13726910032 + 95.7354097343*$t);
   $X0 += 0.00000000165*cos(0.74320909184 + 52669.8257758191*$t);
   $X0 += 0.00000000119*cos(1.19695838858 + 19247.6203708659*$t);
   $X0 += 0.00000000117*cos(0.73795876397 + 11610.7957758577*$t);
   $X0 += 0.00000000163*cos(3.11536370558 + 661.4767442645*$t);
   $X0 += 0.00000000163*cos(4.35849093916 + 9929.6700448293*$t);
   $X0 += 0.00000000119*cos(4.28708216617 + 12250.0036478097*$t);
   $X0 += 0.00000000145*cos(3.90603326867 + 6205.6458970469*$t);
   $X0 += 0.00000000145*cos(2.74250030398 + 6360.99343790291*$t);
   $X0 += 0.00000000159*cos(4.18513637906 + 228278.34846897019*$t);
   $X0 += 0.00000000114*cos(1.84089435963 + 19402.55313533309*$t);
   $X0 += 0.00000000147*cos(4.06094470975 + 38526.3305333885*$t);
   $X0 += 0.00000000156*cos(2.57788751226 + 4307.8271216189*$t);
   $X0 += 0.00000000131*cos(1.70667997590 + 21228.63584102931*$t);
   $X0 += 0.00000000118*cos(5.40927777429 + 6263.64990657511*$t);
   $X0 += 0.00000000118*cos(1.23925579836 + 6302.9894283747*$t);
   $X0 += 0.00000000137*cos(2.54415078163 + 6315.8522182663*$t);
   $X0 += 0.00000000137*cos(4.10438279102 + 6250.7871166835*$t);
   $X0 += 0.00000000109*cos(2.76724250809 + 11925.5179100841*$t);
   $X0 += 0.00000000111*cos(4.37410999057 + 6226.4212925393*$t);
   $X0 += 0.00000000111*cos(2.27442358208 + 6340.2180424105*$t);
   $X0 += 0.00000000123*cos(2.37617280556 + 24734.3982140409*$t);
   $X0 += 0.00000000107*cos(2.16622176546 + 12463.30274324771*$t);
   $X0 += 0.00000000133*cos(3.03066152119 + 18875.76968725751*$t);
   $X0 += 0.00000000123*cos(5.64258939044 + 6260.5444660241*$t);
   $X0 += 0.00000000123*cos(1.00594418221 + 6306.09486892571*$t);
   $X0 += 0.00000000113*cos(1.95402250286 + 2111.8941308611*$t);
   $X0 += 0.00000000131*cos(3.61193663544 + 18415.7596295809*$t);
   $X0 += 0.00000000110*cos(1.76186229885 + 6289.94822637491*$t);
   $X0 += 0.00000000110*cos(4.88667127380 + 6276.69110857489*$t);
   $X0 += 0.00000000103*cos(3.89043805400 + 6241.7688764901*$t);
   $X0 += 0.00000000103*cos(2.75809551865 + 6324.8704584597*$t);
   $X0 += 0.00000000113*cos(2.20489389956 + 3496.2766436175*$t);
   $X0 += 0.00000000103*cos(4.57936207476 + 10344.5388828693*$t);
   $X0 += 0.00000000115*cos(2.66876079900 + 24336.2492106327*$t);
   $X0 += 0.00000000107*cos(1.72027504380 + 83974.07917321339*$t);
   $X0 += 0.00000000107*cos(2.75882626470 + 84020.10309797739*$t);
   $X0 += 0.00000000106*cos(3.27680652019 + 12772.58106590351*$t);
   $X0 += 0.00000000143*cos(1.51224865614 + 2069.2506523901*$t);
   $X0 += 0.00000000143*cos(0.91984229557 + 18773.2052961821*$t);
   $X0 += 0.00000000115*cos(3.78326005147 + 3641.4907540357*$t);
   $X0 += 0.00000000112*cos(5.73842498855 + 11499.41240530929*$t);
   $X0 += 0.00000000099*cos(1.79019915134 + 11190.6217176205*$t);
   $X0 += 0.00000000126*cos(1.19294266238 + 18823.1730476579*$t);
   $X0 += 0.00000000115*cos(4.66544469064 + 12570.3276707294*$t);
   $X0 += 0.00000000100*cos(3.52903073097 + 16062.4283436003*$t);
   $X0 += 0.00000000107*cos(4.02566335272 + 5216.8241902849*$t);
   $X0 += 0.00000000120*cos(4.96595619835 + 9814.36028280769*$t);
   $X0 += 0.00000000138*cos(4.55918305844 + 6210.0225416159*$t);
   $X0 += 0.00000000138*cos(2.08935051421 + 6356.6167933339*$t);
   $X0 += 0.00000000105*cos(1.03351724810 + 12721.32828193349*$t);
   $X0 += 0.00000000135*cos(0.57056498904 + 18699.9081703231*$t);
   $X0 += 0.00000000104*cos(2.15061044327 + 12560.8725931589*$t);
   $X0 += 0.00000000098*cos(5.57514638218 + 5815.3546771205*$t);
   $X0 += 0.00000000111*cos(3.97355954616 + 10239.3400485273*$t);
   $X0 += 0.00000000100*cos(4.55824596261 + 263.3277408563*$t);
   $X0 += 0.00000000095*cos(1.74741902247 + 155.6642169177*$t);
   $X0 += 0.00000000130*cos(2.12273849218 + 27511.7116910207*$t);
   $X0 += 0.00000000120*cos(0.27869701115 + 31441.4337522733*$t);
   $X0 += 0.00000000114*cos(3.75372093481 + 6155.3008241375*$t);
   $X0 += 0.00000000114*cos(2.89481263784 + 6411.3385108123*$t);
   $X0 += 0.00000000119*cos(5.21922596573 + 951.9622237341*$t);
   $X0 += 0.00000000117*cos(1.90078630587 + 28236.98964190589*$t);
   $X0 += 0.00000000107*cos(1.90914871846 + 21.0192129759*$t);
   $X0 += 0.00000000095*cos(5.63307991850 + 11300.8280388399*$t);
   $X0 += 0.00000000110*cos(2.79120257993 + 6312.74917601091*$t);
   $X0 += 0.00000000110*cos(3.85733099272 + 6253.8901589389*$t);
   $X0 += 0.00000000097*cos(4.22068998833 + 78263.95324220609*$t);
   $X0 += 0.00000000095*cos(5.98375623165 + 23938.1002072245*$t);
   $X0 += 0.00000000091*cos(2.99457476326 + 12829.4794408391*$t);
   $X0 += 0.00000000112*cos(4.31165167302 + 16737.8210540801*$t);
   $X0 += 0.00000000126*cos(3.10862438843 + 3.2793008655*$t);
   $X0 += 0.00000000100*cos(3.03286435761 + 6133.7564703403*$t);
   $X0 += 0.00000000127*cos(5.41503454763 + 1293.24040609909*$t);
   $X0 += 0.00000000119*cos(5.99165259734 + 17893.1822114871*$t);
   $X0 += 0.00000000089*cos(1.33458093593 + 23539.9512038163*$t);
   $X0 += 0.00000000126*cos(0.25750692887 + 311565.26273852374*$t);
   $X0 += 0.00000000093*cos(5.67599033319 + 736.1203310153*$t);
   $X0 += 0.00000000120*cos(2.08797711521 + 14158.9915310991*$t);
   $X0 += 0.00000000118*cos(1.43212365415 + 16061.94070863329*$t);
   $X0 += 0.00000000099*cos(3.61958161583 + 6432.8828646095*$t);
   $X0 += 0.00000000095*cos(5.99715593195 + 2699.49100183409*$t);
   $X0 += 0.00000000089*cos(2.30094279014 + 15671.3255768901*$t);
   $X0 += 0.00000000124*cos(0.48008010598 + 178430.29100801519*$t);
   $X0 += 0.00000000089*cos(1.07338684504 + 6751.28465782931*$t);
   $X0 += 0.00000000096*cos(2.64494034094 + 7349.81514466491*$t);
   $X0 += 0.00000000096*cos(0.74221743876 + 24066.0517402591*$t);
   $X0 += 0.00000000103*cos(4.37747381017 + 18202.4605341429*$t);
   $X0 += 0.00000000099*cos(6.06683620849 + 6252.40554183991*$t);
   $X0 += 0.00000000099*cos(0.58169736416 + 6314.2337931099*$t);
   $X0 += 0.00000000090*cos(0.38965345976 + 58864.30010066279*$t);
   $X0 += 0.00000000086*cos(0.59831141560 + 9380.71585523369*$t);
   $X0 += 0.00000000086*cos(0.93909542206 + 10557.35034334029*$t);
   $X0 += 0.00000000098*cos(3.41472072058 + 6439.7203879773*$t);
   $X0 += 0.00000000098*cos(3.23381285207 + 6126.91894697251*$t);
   $X0 += 0.00000000086*cos(1.34148300572 + 23123.9896782901*$t);
   $X0 += 0.00000000118*cos(3.73844359380 + 13951.9570924174*$t);
   $X0 += 0.00000000117*cos(4.10902635534 + 89570.23393702849*$t);
   $X0 += 0.00000000083*cos(3.14133645510 + 8858.0711268371*$t);
   $X0 += 0.00000000092*cos(4.01226998426 + 12342.0507217644*$t);
   $X0 += 0.00000000085*cos(2.29415888429 + 5017.7521888485*$t);
   $X0 += 0.00000000101*cos(0.23020335948 + 18429.9867235825*$t);
   $X0 += 0.00000000088*cos(3.76744436736 + 17054.2129237367*$t);
   $X0 += 0.00000000113*cos(2.05049922518 + 12985.88016134151*$t);
   $X0 += 0.00000000083*cos(0.90577628609 + 20597.0001455577*$t);
   $X0 += 0.00000000082*cos(0.19375496576 + 5483.4985423095*$t);
   $X0 += 0.00000000087*cos(0.10696172464 + 21424.2228268199*$t);
   $X0 += 0.00000000107*cos(2.21757059467 + 522.3336006103*$t);
   $X0 += 0.00000000084*cos(2.41107951633 + 6187.3404402571*$t);
   $X0 += 0.00000000084*cos(4.23745405632 + 6379.2988946927*$t);
   $X0 += 0.00000000107*cos(0.88149755532 + 24382.8352909579*$t);
   $X0 += 0.00000000082*cos(0.94957627406 + 8983.0544867925*$t);
   $X0 += 0.00000000105*cos(3.87374571694 + 6131.4223863897*$t);
   $X0 += 0.00000000105*cos(2.77478785571 + 6435.2169485601*$t);
   $X0 += 0.00000000114*cos(4.07080782247 + 8258.81221333091*$t);
   $X0 += 0.00000000082*cos(3.82951941927 + 3957.8826236923*$t);
   $X0 += 0.00000000112*cos(2.70888594040 + 3738.51761262449*$t);
   $X0 += 0.00000000102*cos(2.89783593458 + 5767.8557963819*$t);
   $X0 += 0.00000000102*cos(3.75069763806 + 6798.7835385679*$t);
   $X0 += 0.00000000092*cos(1.35671084183 + 18216.6876281445*$t);
   $X0 += 0.00000000083*cos(1.35246903804 + 29864.0902098255*$t);
   $X0 += 0.00000000112*cos(5.48905138227 + 12189.0219095505*$t);
   $X0 += 0.00000000080*cos(3.59069348639 + 24080.2788342607*$t);
   $X0 += 0.00000000080*cos(4.80792276932 + 15141.14697682849*$t);
   $X0 += 0.00000000088*cos(3.00407783628 + 1573.57660661529*$t);
   $X0 += 0.00000000081*cos(3.72088724498 + 1550.8014051645*$t);
   $X0 += 0.00000000092*cos(4.36458397294 + 101426.452588453*$t);
   $X0 += 0.00000000110*cos(0.37007007331 + 78423.9483341623*$t);
   $X0 += 0.00000000087*cos(2.32197271603 + 1580.6228433133*$t);
   $X0 += 0.00000000085*cos(6.14240130116 + 27043.2590656993*$t);
   $X0 += 0.00000000078*cos(1.45118645187 + 6812.5229976025*$t);
   $X0 += 0.00000000080*cos(3.36946826019 + 6081.06627230081*$t);
   $X0 += 0.00000000080*cos(3.27906531246 + 6485.573062649*$t);
   $X0 += 0.00000000079*cos(4.53816488559 + 36109.6260221481*$t);
   $X0 += 0.00000000088*cos(0.87069012239 + 16944.0066025173*$t);
   $X0 += 0.00000000081*cos(1.41865192078 + 5.2791068239*$t);
   $X0 += 0.00000000093*cos(5.85619230131 + 16737.33341911309*$t);
   $X0 += 0.00000000088*cos(3.24901264480 + 12537.9463299985*$t);
   $X0 += 0.00000000096*cos(5.19394745813 + 20198.85114214949*$t);
   $X0 += 0.00000000106*cos(2.16665694222 + 56600.0354720387*$t);
   $X0 += 0.00000000097*cos(0.14089796919 + 6040.10342853389*$t);
   $X0 += 0.00000000081*cos(3.62140277149 + 3956.2641985359*$t);
   $X0 += 0.00000000077*cos(1.39348893935 + 40796.5827401577*$t);
   $X0 += 0.00000000076*cos(2.20964866296 + 22743.16556203289*$t);
   $X0 += 0.00000000098*cos(5.48885750823 + 42456.5402296081*$t);
   $X0 += 0.00000000083*cos(2.20640709608 + 19801.1897737083*$t);
   $X0 += 0.00000000079*cos(4.50777033878 + 5622.08674069391*$t);
   $X0 += 0.00000000096*cos(1.98497858569 + 5888.6937824157*$t);
   $X0 += 0.00000000096*cos(4.66355498696 + 6677.9455525341*$t);
   $X0 += 0.00000000075*cos(6.24025577336 + 41194.7317435659*$t);
   $X0 += 0.00000000074*cos(0.66729085356 + 6261.9840270079*$t);
   $X0 += 0.00000000074*cos(5.98124271908 + 6304.6553079419*$t);
   $X0 += 0.00000000073*cos(4.02724017625 + 5870.9485706005*$t);
   $X0 += 0.00000000073*cos(2.62129339640 + 6695.6907643493*$t);
   $X0 += 0.00000000085*cos(3.65241534932 + 12850.2548363315*$t);
   $X0 += 0.00000000072*cos(0.04178130909 + 6253.4982293261*$t);
   $X0 += 0.00000000072*cos(0.32356695638 + 6313.14110562371*$t);
   $X0 += 0.00000000100*cos(5.37794526764 + 5316.3487900393*$t);
   $X0 += 0.00000000085*cos(2.90983978106 + 12282.5361986011*$t);
   $X0 += 0.00000000075*cos(1.96590105616 + 24422.6141688908*$t);
   $X0 += 0.00000000076*cos(1.98227687206 + 63659.1215683211*$t);
   $X0 += 0.00000000072*cos(0.25938390676 + 16723.5939600785*$t);
   $X0 += 0.00000000097*cos(4.92418361725 + 17995.78735073869*$t);
   $X0 += 0.00000000087*cos(3.63417247018 + 18106.4813069251*$t);
   $X0 += 0.00000000080*cos(5.17587638062 + 17363.4912463925*$t);
   $X0 += 0.00000000070*cos(4.63132769081 + 6124.37614969171*$t);
   $X0 += 0.00000000070*cos(2.01720588184 + 6442.2631852581*$t);
   $X0 += 0.00000000070*cos(2.85493203288 + 4705.4884900601*$t);
   $X0 += 0.00000000078*cos(4.94849285823 + 23550.5878691661*$t);
   $X0 += 0.00000000070*cos(2.27501820444 + 12036.21691740469*$t);
   $X0 += 0.00000000077*cos(3.43499900992 + 5237.67719632029*$t);
   $X0 += 0.00000000069*cos(6.02127545231 + 16207.64245401849*$t);
   $X0 += 0.00000000082*cos(5.48703292615 + 6774.98295993371*$t);
   $X0 += 0.00000000067*cos(0.17045217443 + 7083.14079264031*$t);
   $X0 += 0.00000000067*cos(1.83540783927 + 6394.50630976251*$t);
   $X0 += 0.00000000067*cos(4.81312573338 + 6172.1330251873*$t);
   $X0 += 0.00000000077*cos(4.99636632687 + 9924.5666040271*$t);
   $X0 += 0.00000000080*cos(5.36758481138 + 22380.9996177575*$t);
   $X0 += 0.00000000070*cos(4.60174692010 + 6390.9831914135*$t);
   $X0 += 0.00000000070*cos(2.04678665255 + 6175.6561435363*$t);
   $X0 += 0.00000000074*cos(2.55353968467 + 16193.41536001689*$t);
   $X0 += 0.00000000076*cos(5.64799287134 + 32217.4439985643*$t);
   $X0 += 0.00000000091*cos(2.47526506655 + 6653.0194834153*$t);
   $X0 += 0.00000000091*cos(4.17326850610 + 5913.6198515345*$t);
   $X0 += 0.00000000067*cos(0.23293815799 + 6265.50714535691*$t);
   $X0 += 0.00000000067*cos(0.13241010748 + 6301.1321895929*$t);
   $X0 += 0.00000000067*cos(1.44106493302 + 5959.3266158505*$t);
   $X0 += 0.00000000083*cos(2.94569161083 + 16723.10632511149*$t);
   $X0 += 0.00000000080*cos(0.77941152413 + 23646.5670963839*$t);
   $X0 += 0.00000000068*cos(3.64312239532 + 4897.4243911387*$t);
   $X0 += 0.00000000069*cos(2.14076318935 + 6944.5525942559*$t);
   $X0 += 0.00000000089*cos(1.99980743200 + 10660.44311755889*$t);
   $X0 += 0.00000000079*cos(1.67507258686 + 35371.6434484929*$t);
   $X0 += 0.00000000073*cos(2.11438932219 + 6370.62787201471*$t);
   $X0 += 0.00000000073*cos(4.53414425046 + 6196.0114629351*$t);
   $X0 += 0.00000000065*cos(3.66592557921 + 22345.0165586247*$t);
   $X0 += 0.00000000064*cos(4.23436375372 + 15265.64270181689*$t);
   $X0 += 0.00000000066*cos(2.69579215047 + 6315.2919732917*$t);
   $X0 += 0.00000000066*cos(3.95274142218 + 6251.34736165811*$t);
   $X0 += 0.00000000065*cos(5.01328627787 + 323.74923414091*$t);
   $X0 += 0.00000000076*cos(6.17321822941 + 10873.74221299689*$t);
   $X0 += 0.00000000084*cos(5.21213167261 + 109.9625037359*$t);
   $X0 += 0.00000000063*cos(4.27995662426 + 11823.4054569337*$t);
   $X0 += 0.00000000064*cos(0.09093186873 + 28774.2817889683*$t);
   $X0 += 0.00000000072*cos(0.54185576116 + 18099.7594409665*$t);
   $X0 += 0.00000000066*cos(2.78134607936 + 17576.7903418305*$t);
   $X0 += 0.00000000072*cos(1.59783352501 + 245707.70992182779*$t);
   $X0 += 0.00000000061*cos(5.48721212671 + 10557.8379783073*$t);
   $X0 += 0.00000000064*cos(0.75473231286 + 71430.9394356126*$t);
   $X0 += 0.00000000068*cos(2.90814349886 + 28760.05469496669*$t);
   $X0 += 0.00000000062*cos(0.67966569221 + 3854.7898494737*$t);
   $X0 += 0.00000000073*cos(6.18332621846 + 23440.3815479467*$t);
   $X0 += 0.00000000060*cos(3.22950754418 + 13088.9729355601*$t);
   $X0 += 0.00000000078*cos(4.56094059497 + 12564.91104475801*$t);
   $X0 += 0.00000000060*cos(4.47214922427 + 7548.8871461013*$t);
   $X0 += 0.00000000059*cos(2.55184049976 + 6286.3551508569*$t);
   $X0 += 0.00000000075*cos(4.42124944201 + 18625.1265717558*$t);
   $X0 += 0.00000000058*cos(0.47683719349 + 35050.2440919589*$t);
   $X0 += 0.00000000060*cos(0.20507190989 + 553.8132203259*$t);
   $X0 += 0.00000000069*cos(5.82409350668 + 167993.93845370729*$t);
   $X0 += 0.00000000059*cos(0.42390666617 + 41991.0297503823*$t);
   $X0 += 0.00000000062*cos(2.87556762555 + 15663.79170522509*$t);
   $X0 += 0.00000000081*cos(1.27058846781 + 7250.29054491051*$t);
   $X0 += 0.00000000059*cos(2.68580345393 + 6277.2537518451*$t);
   $X0 += 0.00000000059*cos(3.96273011871 + 6289.3855831047*$t);
   $X0 += 0.00000000063*cos(2.13380171799 + 18003.38853270651*$t);
   $X0 += 0.00000000057*cos(4.87646676285 + 793.0187059509*$t);
   $X0 += 0.00000000057*cos(0.02714811761 + 10027.65937824569*$t);
   $X0 += 0.00000000060*cos(2.20528431296 + 647.25465079831*$t);
   $X0 += 0.00000000072*cos(4.16486482248 + 4597.77991000629*$t);
   $X0 += 0.00000000059*cos(3.82312915284 + 6279.30891415889*$t);
   $X0 += 0.00000000068*cos(2.71579349818 + 3166.66025521511*$t);
   $X0 += 0.00000000056*cos(4.67855976016 + 6226.5164053051*$t);
   $X0 += 0.00000000056*cos(1.96997381249 + 6340.12292964471*$t);
   $X0 += 0.00000000056*cos(2.52728721608 + 12303.3115940935*$t);
   $X0 += 0.00000000057*cos(3.09052199581 + 5864.3952685743*$t);
   $X0 += 0.00000000057*cos(3.55801157684 + 6702.2440663755*$t);
   $X0 += 0.00000000069*cos(1.90302432953 + 23536.3607751645*$t);
   $X0 += 0.00000000059*cos(0.79584524101 + 5760.25461441409*$t);
   $X0 += 0.00000000065*cos(0.39208551086 + 12139.30969162329*$t);
   $X0 += 0.00000000059*cos(3.38503180730 + 67589.3312645407*$t);
   $X0 += 0.00000000073*cos(0.51774809611 + 11079.92776143409*$t);
   $X0 += 0.00000000076*cos(2.34882980338 + 23227.0824525087*$t);
   $X0 += 0.00000000067*cos(2.11713130342 + 4804.4530934105*$t);
   $X0 += 0.00000000073*cos(6.00949211634 + 30349.1275902505*$t);
   $X0 += 0.00000000074*cos(4.74855693255 + 92747.9329843061*$t);
   $X0 += 0.00000000070*cos(1.56579326622 + 12299.7884757445*$t);
   $X0 += 0.00000000060*cos(0.37222072501 + 12171.7696324071*$t);
   $X0 += 0.00000000059*cos(2.60537764043 + 15.49628866851*$t);
   $X0 += 0.00000000074*cos(5.91207332800 + 3684.1342325067*$t);
   $X0 += 0.00000000058*cos(3.15618940951 + 6717.0314053517*$t);
   $X0 += 0.00000000053*cos(2.11594966811 + 26087.65932409069*$t);
   $X0 += 0.00000000059*cos(1.05069506955 + 12164.7233957091*$t);
   $X0 += 0.00000000063*cos(3.66132366701 + 5219.5179490556*$t);
   $X0 += 0.00000000056*cos(4.02457482788 + 3178.38960805111*$t);
   $X0 += 0.00000000063*cos(0.59069175384 + 5227.38225558179*$t);
   $X0 += 0.00000000070*cos(3.26289105386 + 19004.4041319249*$t);
   $X0 += 0.00000000072*cos(1.05516042130 + 4583.55281600469*$t);
   $X0 += 0.00000000052*cos(5.67763480740 + 3627.2636600341*$t);
   $X0 += 0.00000000052*cos(4.87358219235 + 6411.2712005095*$t);
   $X0 += 0.00000000052*cos(1.77495138030 + 6155.3681344403*$t);
   $X0 += 0.00000000054*cos(3.61747287023 + 5849.6079295981*$t);
   $X0 += 0.00000000066*cos(4.36702417592 + 5791.16874004909*$t);
   $X0 += 0.00000000066*cos(1.22543152233 + 5791.6563750161*$t);
   $X0 += 0.00000000051*cos(0.84172627776 + 5113.2437810993*$t);
   $X0 += 0.00000000058*cos(4.29307138428 + 30775.7257811265*$t);
   $X0 += 0.00000000061*cos(5.44071514919 + 14169.5497447469*$t);
   $X0 += 0.00000000054*cos(4.73181482869 + 10454.74520408871*$t);
   $X0 += 0.00000000056*cos(2.77114790322 + 23141.8022004081*$t);
   $X0 += 0.00000000067*cos(2.31997560907 + 28313.0449871775*$t);
   $X0 += 0.00000000056*cos(1.94933767144 + 5244.2930566845*$t);
   $X0 += 0.00000000062*cos(2.18007483018 + 5657.6494751625*$t);
   $X0 += 0.00000000062*cos(4.46845874247 + 6908.9898597873*$t);
   $X0 += 0.00000000050*cos(1.53914825533 + 2574.99527684569*$t);
   $X0 += 0.00000000050*cos(1.32272515661 + 536.5606946119*$t);
   $X0 += 0.00000000050*cos(1.95397557344 + 6684.9917892321*$t);
   $X0 += 0.00000000050*cos(4.69455799921 + 5881.6475457177*$t);
   $X0 += 0.00000000049*cos(5.07674655291 + 16310.73522823709*$t);
   $X0 += 0.00000000049*cos(5.07674655291 + 16311.2228632041*$t);
   $X0 += 0.00000000065*cos(0.63077087865 + 46386.7499258277*$t);
   $X0 += 0.00000000069*cos(3.59016732594 + 60530.2451682583*$t);
   $X0 += 0.00000000068*cos(6.19031950148 + 96678.14268052569*$t);
   $X0 += 0.00000000049*cos(2.44170258748 + 12323.17927852529*$t);
   $X0 += 0.00000000050*cos(3.95175048363 + 21954.4014268815*$t);
   $X0 += 0.00000000049*cos(0.26987394090 + 4164.5558070965*$t);
   $X0 += 0.00000000059*cos(2.11910349639 + 33794.7875410121*$t);
   $X0 += 0.00000000050*cos(3.21674377716 + 43739.0461634493*$t);
   $X0 += 0.00000000059*cos(4.28455887932 + 3981.73385156551*$t);
   $X0 += 0.00000000058*cos(1.59500717257 + 27707.29867681129*$t);
   $X0 += 0.00000000051*cos(3.00541030027 + 7669.21494381111*$t);
   $X0 += 0.00000000048*cos(4.80655958998 + 3646.59419483791*$t);
   $X0 += 0.00000000065*cos(3.20870046710 + 82534.6474380951*$t);
   $X0 += 0.00000000056*cos(4.19440679704 + 800.06494264891*$t);
   $X0 += 0.00000000057*cos(0.99121344005 + 11609.61872652869*$t);
   $X0 += 0.00000000048*cos(5.23363777755 + 26.5421372833*$t);
   $X0 += 0.00000000047*cos(1.10988199958 + 30640.1004561165*$t);
   $X0 += 0.00000000060*cos(1.97427905695 + 8982.5668518255*$t);
   $X0 += 0.00000000049*cos(3.72444308522 + 96563.24283557819*$t);
   $X0 += 0.00000000050*cos(5.17708029533 + 17583.9038888313*$t);
   $X0 += 0.00000000046*cos(2.75949485809 + 23539.46356884929*$t);
   $X0 += 0.00000000048*cos(2.83397317642 + 170.4290531357*$t);
   $X0 += 0.00000000055*cos(4.77764015710 + 735.6326960483*$t);
   $X0 += 0.00000000054*cos(4.33153690679 + 6680.98103591609*$t);
   $X0 += 0.00000000060*cos(1.72047044215 + 44033.8837720559*$t);
   $X0 += 0.00000000064*cos(4.40652103235 + 80181.29364935629*$t);
   $X0 += 0.00000000059*cos(5.18038765510 + 24279.3508356971*$t);
   $X0 += 0.00000000049*cos(5.16566622453 + 5490.5447790075*$t);
   $X0 += 0.00000000058*cos(1.24075349030 + 6490.0204047715*$t);
   $X0 += 0.00000000058*cos(5.40778008235 + 6076.6189301783*$t);
   $X0 += 0.00000000046*cos(6.06699951300 + 18208.1061251085*$t);
   $X0 += 0.00000000045*cos(1.62616739636 + 4480.46004178609*$t);
   $X0 += 0.00000000055*cos(3.56883579861 + 16097.9237677661*$t);
   $X0 += 0.00000000055*cos(0.42724314502 + 16097.43613279909*$t);
   $X0 += 0.00000000061*cos(5.10427594640 + 491.9071099423*$t);
   $X0 += 0.00000000054*cos(2.79730041476 + 6006.5738956105*$t);
   $X0 += 0.00000000054*cos(3.85123315789 + 6560.0654393393*$t);
   $X0 += 0.00000000044*cos(3.20288072834 + 5333.65642353809*$t);
   $X0 += 0.00000000052*cos(5.42298629738 + 244287.84382471119*$t);
   $X0 += 0.00000000044*cos(1.12093494405 + 2301.34199842589*$t);
   $X0 += 0.00000000045*cos(2.78871592108 + 33794.2999060451*$t);
   $X0 += 0.00000000046*cos(2.27794751316 + 4384.48081456829*$t);
   $X0 += 0.00000000048*cos(6.20709507263 + 35579.9350570535*$t);
   $X0 += 0.00000000051*cos(0.20647678216 + 18326.8939493639*$t);
   $X0 += 0.00000000045*cos(4.21002666289 + 533.45790092711*$t);
   $X0 += 0.00000000046*cos(4.72419290173 + 16703.3059509825*$t);
   $X0 += 0.00000000047*cos(0.82323755553 + 5209.2230083171*$t);
   $X0 += 0.00000000051*cos(2.88192111957 + 38650.4173236825*$t);
   $X0 += 0.00000000046*cos(4.04752097938 + 12555.3498172024*$t);
   $X0 += 0.00000000052*cos(6.21526623133 + 5992.3468016089*$t);
   $X0 += 0.00000000052*cos(0.43326734132 + 6574.2925333409*$t);
   $X0 += 0.00000000044*cos(5.37917268925 + 69942.1974183125*$t);
   $X0 += 0.00000000049*cos(5.19212576524 + 12146.9108735911*$t);
   $X0 += 0.00000000053*cos(3.10372910020 + 39301.8531447125*$t);
   $X0 += 0.00000000045*cos(6.22187415000 + 46360.9392409949*$t);
   $X0 += 0.00000000043*cos(3.11607958976 + 8402.0835278533*$t);
   $X0 += 0.00000000043*cos(1.76871622593 + 44137.1951668575*$t);
   $X0 += 0.00000000049*cos(1.97645700808 + 22029.9691371861*$t);
   $X0 += 0.00000000044*cos(3.04557845219 + 31441.92138724031*$t);
   $X0 += 0.00000000042*cos(1.28858480303 + 9225.29545579949*$t);
   $X0 += 0.00000000045*cos(4.79112566977 + 71519.54096076029*$t);
   $X0 += 0.00000000051*cos(4.52703512787 + 7762.1862415393*$t);
   $X0 += 0.00000000041*cos(0.43156198612 + 3735.48212924251*$t);
   $X0 += 0.00000000057*cos(2.81178684757 + 40398.4337367495*$t);
   $X0 += 0.00000000047*cos(4.46091937774 + 4707.9862312257*$t);
   $X0 += 0.00000000056*cos(0.65497165024 + 28664.0754677489*$t);
   $X0 += 0.00000000050*cos(4.15809563419 + 11919.3846841515*$t);
   $X0 += 0.00000000047*cos(0.89798744348 + 316428.47249139856*$t);
   $X0 += 0.00000000041*cos(4.27548568888 + 27278.7126339243*$t);
   $X0 += 0.00000000056*cos(2.85120735939 + 83659.5206898825*$t);
   $X0 += 0.00000000048*cos(2.50261741205 + 12662.3747446841*$t);
   $X0 += 0.00000000054*cos(0.83831240155 + 16627.12709789369*$t);
   $X0 += 0.00000000052*cos(4.41165168727 + 12571.9184417737*$t);
   $X0 += 0.00000000043*cos(4.69793524808 + 7322.34627826531*$t);
   $X0 += 0.00000000041*cos(3.56154893542 + 664.99986261351*$t);
   $X0 += 0.00000000042*cos(1.54738888492 + 5641.9544251257*$t);
   $X0 += 0.00000000044*cos(1.44765859641 + 6425.4982945111*$t);
   $X0 += 0.00000000044*cos(5.20087497624 + 6141.1410404387*$t);
   $X0 += 0.00000000039*cos(0.38205897719 + 22594.2987131955*$t);
   $X0 += 0.00000000040*cos(5.88641926833 + 20894.53186172529*$t);
   $X0 += 0.00000000044*cos(5.28307931868 + 5540.3296069423*$t);
   $X0 += 0.00000000050*cos(5.69294803866 + 17782.48825530069*$t);
   $X0 += 0.00000000054*cos(4.69164296803 + 6006.2846737335*$t);
   $X0 += 0.00000000054*cos(1.95689060462 + 6560.3546612163*$t);
   $X0 += 0.00000000045*cos(5.54499872381 + 142861.14741877468*$t);
   $X0 += 0.00000000047*cos(1.12158335918 + 24499.7634781359*$t);
   $X0 += 0.00000000043*cos(1.36615498859 + 7026.3097280075*$t);
   $X0 += 0.00000000047*cos(1.95434456607 + 955.8435590921*$t);
   $X0 += 0.00000000047*cos(4.50829221159 + 24485.5363841343*$t);
   $X0 += 0.00000000040*cos(2.60727518415 + 9388.2497268987*$t);
   $X0 += 0.00000000043*cos(5.81439105801 + 6359.5857387505*$t);
   $X0 += 0.00000000042*cos(3.96208965107 + 8635.69818627969*$t);
   $X0 += 0.00000000051*cos(2.46325624829 + 27831.79440179969*$t);
   $X0 += 0.00000000042*cos(1.94770311943 + 6334.60000533731*$t);
   $X0 += 0.00000000042*cos(4.70083045322 + 6232.03932961249*$t);
   $X0 += 0.00000000052*cos(3.53311914292 + 11905.1625906853*$t);
   $X0 += 0.00000000040*cos(1.50948151820 + 7076.0945559423*$t);
   $X0 += 0.00000000040*cos(0.29378456988 + 5444.3503797245*$t);
   $X0 += 0.00000000040*cos(0.07156369559 + 7122.2889552253*$t);
   $X0 += 0.00000000038*cos(2.24525725009 + 44933.4931736739*$t);
   $X0 += 0.00000000037*cos(0.27418012894 + 5715.6010297445*$t);
   $X0 += 0.00000000037*cos(0.09116813653 + 6851.0383052053*$t);
   $X0 += 0.00000000042*cos(0.79913949254 + 36147.6536947839*$t);
   $X0 += 0.00000000042*cos(2.85822305358 + 6238.1784478383*$t);
   $X0 += 0.00000000042*cos(3.79031051907 + 6328.4608871115*$t);
   $X0 += 0.00000000040*cos(3.64347872660 + 3214.8925631597*$t);
   $X0 += 0.00000000043*cos(0.65521409425 + 23141.3145654411*$t);
   $X0 += 0.00000000041*cos(3.95515629208 + 19804.5834740993*$t);
   $X0 += 0.00000000038*cos(2.90982798521 + 20.11150191529*$t);
   $X0 += 0.00000000050*cos(1.62788860301 + 84334.66158130829*$t);
   $X0 += 0.00000000050*cos(2.84763289309 + 60284.16619777939*$t);
   return $X0;
}



   function Earth_X1($t) // 600 terms of order 1
{
   $X1  = 0.00154550744*cos(0.64605836878 + 0.2438174835*$t);
   $X1 += 0.00051503383*cos(6.00263199393 + 12566.3955174663*$t);
   $X1 += 0.00001290763*cos(5.95941652859 + 18849.4713674577*$t);
   $X1 += 0.00000702576*cos(1.75347943445 + 6283.3196674749*$t);
   $X1 += 0.00000430422*cos(2.45216492471 + 6282.83203250789*$t);
   $X1 += 0.00000212689*cos(1.73380190045 + 6279.7965491259*$t);
   $X1 += 0.00000212524*cos(4.91484799365 + 6286.84278582391*$t);
   $X1 += 0.00000062308*cos(0.36193297710 + 4705.9761250271*$t);
   $X1 += 0.00000059808*cos(3.81187956678 + 6257.0213476751*$t);
   $X1 += 0.00000059474*cos(2.83659652893 + 6309.61798727471*$t);
   $X1 += 0.00000048914*cos(5.21332670904 + 775.7664288075*$t);
   $X1 += 0.00000042814*cos(0.43864033644 + 1059.6257476727*$t);
   $X1 += 0.00000046457*cos(0.01089905463 + 7860.6632099227*$t);
   $X1 += 0.00000036653*cos(2.19635713914 + 5753.6287023803*$t);
   $X1 += 0.00000035649*cos(1.44752011203 + 5885.1706640667*$t);
   $X1 += 0.00000035362*cos(4.47215447256 + 6813.0106325695*$t);
   $X1 += 0.00000032151*cos(5.19677261546 + 6681.46867088311*$t);
   $X1 += 0.00000028763*cos(5.91618988907 + 25132.5472174491*$t);
   $X1 += 0.00000028447*cos(1.14976229491 + 6127.8992680407*$t);
   $X1 += 0.00000027537*cos(5.49920443604 + 6438.7400669091*$t);
   $X1 += 0.00000024819*cos(2.92230738957 + 5487.0216606585*$t);
   $X1 += 0.00000020615*cos(3.71821460067 + 7079.61767429131*$t);
   $X1 += 0.00000019621*cos(2.88650825392 + 5507.7970561509*$t);
   $X1 += 0.00000018378*cos(1.46521342874 + 11790.8729061423*$t);
   $X1 += 0.00000016494*cos(6.21543454587 + 11507.0135872771*$t);
   $X1 += 0.00000016756*cos(3.81938434057 + 7058.8422787989*$t);
   $X1 += 0.00000014558*cos(5.97530104893 + 6290.4332144757*$t);
   $X1 += 0.00000014404*cos(0.68122522191 + 6276.2061204741*$t);
   $X1 += 0.00000014052*cos(1.45747121073 + 796.5418242999*$t);
   $X1 += 0.00000012261*cos(4.15789187748 + 4693.75913722409*$t);
   $X1 += 0.00000012758*cos(0.34831751321 + 7.3573644843*$t);
   $X1 += 0.00000010086*cos(3.29242550455 + 3739.0052475915*$t);
   $X1 += 0.00000009469*cos(4.63920457850 + 6070.0205720369*$t);
   $X1 += 0.00000010425*cos(2.38242547117 + 6284.2999885431*$t);
   $X1 += 0.00000010425*cos(4.26610810148 + 6282.3393464067*$t);
   $X1 += 0.00000009570*cos(1.89427100067 + 4137.1542509997*$t);
   $X1 += 0.00000009044*cos(2.00972967096 + 6496.6187629129*$t);
   $X1 += 0.00000008544*cos(0.05731253621 + 1194.6908277081*$t);
   $X1 += 0.00000008214*cos(1.25401494002 + 1589.3167127673*$t);
   $X1 += 0.00000006157*cos(3.35676469316 + 8827.6340873583*$t);
   $X1 += 0.00000006271*cos(4.75475820102 + 8429.4850839501*$t);
   $X1 += 0.00000005524*cos(5.36941014190 + 4933.4522578161*$t);
   $X1 += 0.00000007314*cos(0.49102526454 + 4535.3032544079*$t);
   $X1 += 0.00000005157*cos(5.14965163127 + 11770.0975106499*$t);
   $X1 += 0.00000006749*cos(4.32513020462 + 5088.8726572503*$t);
   $X1 += 0.00000004853*cos(0.62462823219 + 6040.5910635009*$t);
   $X1 += 0.00000005304*cos(5.64244320178 + 3154.4432674121*$t);
   $X1 += 0.00000004985*cos(4.62777616911 + 12569.91863581531*$t);
   $X1 += 0.00000004709*cos(3.96386608147 + 5331.6012612243*$t);
   $X1 += 0.00000004601*cos(6.02376124126 + 6526.04827144891*$t);
   $X1 += 0.00000004201*cos(1.28125193997 + 7633.1870771337*$t);
   $X1 += 0.00000005607*cos(2.55800509408 + 5729.7502646325*$t);
   $X1 += 0.00000004257*cos(4.12450123624 + 3930.4535137031*$t);
   $X1 += 0.00000004394*cos(0.04709285339 + 12559.2819704655*$t);
   $X1 += 0.00000003870*cos(2.68681123019 + 7235.0380737255*$t);
   $X1 += 0.00000005154*cos(6.15930214427 + 8031.3360805419*$t);
   $X1 += 0.00000005036*cos(4.09009014859 + 6836.8890703173*$t);
   $X1 += 0.00000005076*cos(2.30606090748 + 7477.7666776995*$t);
   $X1 += 0.00000003593*cos(2.40570544354 + 12565.9078824993*$t);
   $X1 += 0.00000003384*cos(5.40354177904 + 10977.32262218251*$t);
   $X1 += 0.00000003371*cos(0.28961103245 + 11371.9485072417*$t);
   $X1 += 0.00000003536*cos(3.73353119602 + 4164.0681721295*$t);
   $X1 += 0.00000004292*cos(4.93786086114 + 1592.8398311163*$t);
   $X1 += 0.00000003407*cos(2.40599273330 + 3128.6325825793*$t);
   $X1 += 0.00000003152*cos(1.73356525688 + 5223.93773728571*$t);
   $X1 += 0.00000002883*cos(4.92077247219 + 1747.7725955835*$t);
   $X1 += 0.00000002751*cos(5.24449444697 + 7342.7015976641*$t);
   $X1 += 0.00000003323*cos(1.91880015934 + 801.5771136403*$t);
   $X1 += 0.00000002999*cos(2.25355241223 + 8636.18582124671*$t);
   $X1 += 0.00000002496*cos(3.52078273708 + 2145.92159899169*$t);
   $X1 += 0.00000003358*cos(4.25271343539 + 155.17658195069*$t);
   $X1 += 0.00000002297*cos(4.53559364224 + 17260.3984721739*$t);
   $X1 += 0.00000002047*cos(3.53660263792 + 1990.9888345245*$t);
   $X1 += 0.00000002778*cos(1.46152428305 + 5481.4987363511*$t);
   $X1 += 0.00000002025*cos(1.44003078455 + 951.4745887671*$t);
   $X1 += 0.00000002663*cos(5.71398175957 + 26.0545023163*$t);
   $X1 += 0.00000002009*cos(5.73390835065 + 4690.7236538421*$t);
   $X1 += 0.00000002148*cos(1.89688144273 + 537.0483295789*$t);
   $X1 += 0.00000001853*cos(2.84762029989 + 553.32558535889*$t);
   $X1 += 0.00000002478*cos(0.03933370984 + 1349.62359217529*$t);
   $X1 += 0.00000002023*cos(2.39095052485 + 522.8212355773*$t);
   $X1 += 0.00000001735*cos(2.91944410412 + 529.44714761109*$t);
   $X1 += 0.00000002417*cos(5.16772880951 + 7085.1405985987*$t);
   $X1 += 0.00000001809*cos(0.78457177103 + 397.9051859247*$t);
   $X1 += 0.00000001893*cos(4.22746619412 + 9438.0067523705*$t);
   $X1 += 0.00000001708*cos(0.29122424525 + 10989.0519750185*$t);
   $X1 += 0.00000001898*cos(2.34007370780 + 5216.33655531789*$t);
   $X1 += 0.00000001820*cos(1.87728974480 + 5230.5636493195*$t);
   $X1 += 0.00000001541*cos(1.10077450497 + 426.8420083595*$t);
   $X1 += 0.00000001555*cos(3.67004127739 + 13096.0864825609*$t);
   $X1 += 0.00000001779*cos(1.80817581905 + 12562.8723991173*$t);
   $X1 += 0.00000001448*cos(0.18764842398 + 5753.14106741329*$t);
   $X1 += 0.00000001450*cos(0.99962470222 + 6262.96434807611*$t);
   $X1 += 0.00000001450*cos(5.64890887043 + 6303.6749868737*$t);
   $X1 += 0.00000001822*cos(1.70526332053 + 10973.7995038335*$t);
   $X1 += 0.00000001372*cos(0.89281758077 + 7875.91568110771*$t);
   $X1 += 0.00000001375*cos(0.76961577554 + 7084.6529636317*$t);
   $X1 += 0.00000001400*cos(4.85224030747 + 12721.8159169005*$t);
   $X1 += 0.00000001247*cos(1.86305212941 + 2119.00767786191*$t);
   $X1 += 0.00000001231*cos(4.10244270612 + 18319.7804023631*$t);
   $X1 += 0.00000001141*cos(3.49241515175 + 1066.7392946735*$t);
   $X1 += 0.00000001138*cos(2.80935582144 + 2942.7072407751*$t);
   $X1 += 0.00000001047*cos(6.02902034733 + 5642.93474619389*$t);
   $X1 += 0.00000001058*cos(3.76081315695 + 242.97242145751*$t);
   $X1 += 0.00000001025*cos(5.32138364615 + 10447.14402212089*$t);
   $X1 += 0.00000000973*cos(0.24159402695 + 640.1411037975*$t);
   $X1 += 0.00000001114*cos(5.94057908115 + 15721.0826023619*$t);
   $X1 += 0.00000000963*cos(2.13180404708 + 2389.1378379327*$t);
   $X1 += 0.00000000941*cos(5.78706319005 + 20426.8149099055*$t);
   $X1 += 0.00000001068*cos(0.37393702270 + 529.9347825781*$t);
   $X1 += 0.00000000929*cos(3.11135132378 + 10575.6505004253*$t);
   $X1 += 0.00000000899*cos(1.07875107899 + 16496.6052136859*$t);
   $X1 += 0.00000000889*cos(0.29953586851 + 6277.7967431675*$t);
   $X1 += 0.00000000889*cos(0.06581239696 + 6288.8425917823*$t);
   $X1 += 0.00000001018*cos(4.23379398746 + 12540.0971976665*$t);
   $X1 += 0.00000001060*cos(1.05626464973 + 5760.7422493811*$t);
   $X1 += 0.00000001017*cos(5.59172649862 + 6805.89708556871*$t);
   $X1 += 0.00000000797*cos(5.36416276600 + 14314.4119305333*$t);
   $X1 += 0.00000000795*cos(1.67821896162 + 6286.6060248927*$t);
   $X1 += 0.00000000795*cos(4.97031461103 + 6280.0333100571*$t);
   $X1 += 0.00000000800*cos(4.75445763497 + 12029.5910053709*$t);
   $X1 += 0.00000000719*cos(3.86746073982 + 9623.9320941747*$t);
   $X1 += 0.00000000698*cos(0.23124353834 + 6148.2545874395*$t);
   $X1 += 0.00000000686*cos(5.36448356218 + 5856.7214765989*$t);
   $X1 += 0.00000000673*cos(5.56184807840 + 12964.5445208745*$t);
   $X1 += 0.00000000902*cos(2.26778049127 + 13368.2164485901*$t);
   $X1 += 0.00000000678*cos(0.13408295783 + 6418.38474751031*$t);
   $X1 += 0.00000000671*cos(1.28940504615 + 6709.91785835091*$t);
   $X1 += 0.00000000744*cos(4.25820705300 + 12043.8180993725*$t);
   $X1 += 0.00000000706*cos(4.09030374624 + 16730.70750707931*$t);
   $X1 += 0.00000000661*cos(3.99296008106 + 14712.5609339415*$t);
   $X1 += 0.00000000633*cos(0.86914955896 + 4292.5746504339*$t);
   $X1 += 0.00000000621*cos(3.31200729066 + 13119.9649203087*$t);
   $X1 += 0.00000000638*cos(0.47109166303 + 4690.23601887509*$t);
   $X1 += 0.00000000625*cos(1.88693051693 + 13518.1139237169*$t);
   $X1 += 0.00000000637*cos(6.16733887432 + 5746.5151553795*$t);
   $X1 += 0.00000000628*cos(1.85369027207 + 4686.6455902233*$t);
   $X1 += 0.00000000653*cos(4.28657823528 + 3929.96587873609*$t);
   $X1 += 0.00000000578*cos(5.33616352365 + 5088.3850222833*$t);
   $X1 += 0.00000000563*cos(4.82044552939 + 10447.6316570879*$t);
   $X1 += 0.00000000610*cos(0.48716946879 + 6820.1241795703*$t);
   $X1 += 0.00000000666*cos(3.30009380787 + 3634.37720703489*$t);
   $X1 += 0.00000000720*cos(0.47572131456 + 13916.2629271251*$t);
   $X1 += 0.00000000620*cos(5.87390167695 + 31415.6230674405*$t);
   $X1 += 0.00000000553*cos(4.28779208332 + 6259.44122972711*$t);
   $X1 += 0.00000000553*cos(2.36074148933 + 6307.1981052227*$t);
   $X1 += 0.00000000523*cos(0.75768878632 + 14143.7390599141*$t);
   $X1 += 0.00000000524*cos(5.54806074141 + 12139.7973265903*$t);
   $X1 += 0.00000000569*cos(0.12658241682 + 12036.7045523717*$t);
   $X1 += 0.00000000514*cos(2.30528734098 + 4700.87268422489*$t);
   $X1 += 0.00000000563*cos(2.21509710432 + 11014.8626598513*$t);
   $X1 += 0.00000000495*cos(3.58067034949 + 13362.6935242827*$t);
   $X1 += 0.00000000481*cos(2.50655660751 + 6279.4383321169*$t);
   $X1 += 0.00000000481*cos(4.14197696514 + 6287.2010028329*$t);
   $X1 += 0.00000000466*cos(4.51385713822 + 10177.5014970171*$t);
   $X1 += 0.00000000440*cos(0.45716811540 + 1577.5873599313*$t);
   $X1 += 0.00000000437*cos(3.18132586326 + 11499.9000402763*$t);
   $X1 += 0.00000000452*cos(5.55409870988 + 12573.5090644671*$t);
   $X1 += 0.00000000499*cos(1.08746970886 + 12410.9751180321*$t);
   $X1 += 0.00000000440*cos(2.15051971988 + 24073.1652872599*$t);
   $X1 += 0.00000000587*cos(4.71917094180 + 6.86972951729*$t);
   $X1 += 0.00000000388*cos(2.04118602600 + 7860.17557495569*$t);
   $X1 += 0.00000000398*cos(5.77147994683 + 8274.0646845159*$t);
   $X1 += 0.00000000423*cos(1.16599451585 + 4694.2467721911*$t);
   $X1 += 0.00000000530*cos(3.00672799320 + 12592.6938372661*$t);
   $X1 += 0.00000000379*cos(2.02388854283 + 6180.2268932563*$t);
   $X1 += 0.00000000379*cos(4.62464502982 + 6386.4124416935*$t);
   $X1 += 0.00000000377*cos(5.90352173005 + 7872.39256275871*$t);
   $X1 += 0.00000000419*cos(5.62219874401 + 5327.7199258663*$t);
   $X1 += 0.00000000383*cos(6.24432820927 + 6247.2918007975*$t);
   $X1 += 0.00000000383*cos(0.40420536338 + 6319.34753415231*$t);
   $X1 += 0.00000000375*cos(5.62315844522 + 2352.6223362883*$t);
   $X1 += 0.00000000457*cos(4.19200440747 + 6077.1341190377*$t);
   $X1 += 0.00000000457*cos(2.45652916518 + 6489.5052159121*$t);
   $X1 += 0.00000000481*cos(1.89483272077 + 4292.0870154669*$t);
   $X1 += 0.00000000344*cos(0.28668049892 + 18451.3223640495*$t);
   $X1 += 0.00000000453*cos(0.72577236980 + 2787.2868413409*$t);
   $X1 += 0.00000000364*cos(4.39782645564 + 6245.2919948391*$t);
   $X1 += 0.00000000364*cos(2.25070711701 + 6321.3473401107*$t);
   $X1 += 0.00000000327*cos(2.71208539186 + 12323.6669134923*$t);
   $X1 += 0.00000000358*cos(4.87210297876 + 9917.45305702629*$t);
   $X1 += 0.00000000342*cos(0.54833716779 + 6262.5442719825*$t);
   $X1 += 0.00000000342*cos(6.10019640486 + 6304.0950629673*$t);
   $X1 += 0.00000000318*cos(0.13364568371 + 11926.4982311523*$t);
   $X1 += 0.00000000320*cos(2.70295484544 + 77714.01528560399*$t);
   $X1 += 0.00000000354*cos(1.02467818407 + 7238.9194090835*$t);
   $X1 += 0.00000000318*cos(5.32503015719 + 6254.8704800071*$t);
   $X1 += 0.00000000318*cos(1.32350341546 + 6311.7688549427*$t);
   $X1 += 0.00000000304*cos(2.00970548291 + 12779.6946129043*$t);
   $X1 += 0.00000000343*cos(3.71302827314 + 6298.57213865991*$t);
   $X1 += 0.00000000343*cos(2.93550529951 + 6268.0671962899*$t);
   $X1 += 0.00000000359*cos(4.09955266779 + 1052.51220067191*$t);
   $X1 += 0.00000000398*cos(0.12298117861 + 1551.2890401315*$t);
   $X1 += 0.00000000306*cos(2.58873546625 + 5863.8350235997*$t);
   $X1 += 0.00000000300*cos(1.77608786075 + 90280.16698558679*$t);
   $X1 += 0.00000000307*cos(3.29263568042 + 3893.93801205869*$t);
   $X1 += 0.00000000297*cos(0.51221148619 + 5429.63565075589*$t);
   $X1 += 0.00000000365*cos(2.74542548477 + 10022.0810975829*$t);
   $X1 += 0.00000000336*cos(2.72364909606 + 17782.9758902677*$t);
   $X1 += 0.00000000295*cos(4.05850212848 + 6702.8043113501*$t);
   $X1 += 0.00000000288*cos(2.18795828306 + 18073.9487561337*$t);
   $X1 += 0.00000000287*cos(3.33776996109 + 1577.0997249643*$t);
   $X1 += 0.00000000378*cos(0.39946174567 + 2353.1099712553*$t);
   $X1 += 0.00000000289*cos(3.04361347297 + 213.5429129215*$t);
   $X1 += 0.00000000347*cos(0.72838501040 + 5223.4501023187*$t);
   $X1 += 0.00000000321*cos(2.23787969105 + 17797.2029842693*$t);
   $X1 += 0.00000000289*cos(5.30650717041 + 220.6564599223*$t);
   $X1 += 0.00000000260*cos(1.04284839158 + 955.3559241251*$t);
   $X1 += 0.00000000268*cos(3.62847841862 + 14945.0723560709*$t);
   $X1 += 0.00000000242*cos(4.34386674771 + 2544.5582373669*$t);
   $X1 += 0.00000000245*cos(2.31719119888 + 7632.6994421667*$t);
   $X1 += 0.00000000234*cos(4.07393437960 + 1596.43025976811*$t);
   $X1 += 0.00000000267*cos(5.83055956746 + 206.42936592071*$t);
   $X1 += 0.00000000282*cos(4.50260656021 + 13341.9181287903*$t);
   $X1 += 0.00000000279*cos(3.13773429847 + 4731.7868098599*$t);
   $X1 += 0.00000000290*cos(2.03216763842 + 5642.4420600927*$t);
   $X1 += 0.00000000298*cos(0.93822937797 + 17790.08943726851*$t);
   $X1 += 0.00000000270*cos(2.16529088691 + 12168.2465140581*$t);
   $X1 += 0.00000000294*cos(2.52895912407 + 2146.4092339587*$t);
   $X1 += 0.00000000214*cos(0.93543623291 + 8030.8484455749*$t);
   $X1 += 0.00000000207*cos(1.60966002008 + 213.0552779545*$t);
   $X1 += 0.00000000216*cos(5.60316710178 + 3185.43584474911*$t);
   $X1 += 0.00000000195*cos(2.87143133187 + 5884.68302909969*$t);
   $X1 += 0.00000000199*cos(3.93236191602 + 1748.2602305505*$t);
   $X1 += 0.00000000250*cos(4.61596023287 + 6924.1972748571*$t);
   $X1 += 0.00000000194*cos(2.36749003244 + 641.12142486571*$t);
   $X1 += 0.00000000203*cos(2.25356528795 + 6503.7323099137*$t);
   $X1 += 0.00000000203*cos(4.39496828470 + 6062.9070250361*$t);
   $X1 += 0.00000000258*cos(5.63025007570 + 796.0541893329*$t);
   $X1 += 0.00000000189*cos(1.63377883836 + 11506.52595231009*$t);
   $X1 += 0.00000000186*cos(0.12949416246 + 18209.5740811437*$t);
   $X1 += 0.00000000184*cos(4.20955149479 + 12566.4628277691*$t);
   $X1 += 0.00000000180*cos(5.76808682495 + 853.4401992355*$t);
   $X1 += 0.00000000185*cos(4.68506649147 + 3495.78900865049*$t);
   $X1 += 0.00000000205*cos(4.93663483706 + 5849.1202946311*$t);
   $X1 += 0.00000000187*cos(5.91135912176 + 10213.5293636945*$t);
   $X1 += 0.00000000187*cos(2.05685668523 + 6290.36590417291*$t);
   $X1 += 0.00000000187*cos(4.59167688742 + 6276.2734307769*$t);
   $X1 += 0.00000000235*cos(5.91611965384 + 9779.3524936089*$t);
   $X1 += 0.00000000173*cos(1.38702654435 + 19651.2922985815*$t);
   $X1 += 0.00000000184*cos(4.65365446464 + 12566.32820716351*$t);
   $X1 += 0.00000000230*cos(2.23774195109 + 12567.37583853451*$t);
   $X1 += 0.00000000166*cos(1.32597581910 + 110.45013870291*$t);
   $X1 += 0.00000000189*cos(2.84387211432 + 3.76693583251*$t);
   $X1 += 0.00000000166*cos(1.19482070134 + 433.9555553603*$t);
   $X1 += 0.00000000186*cos(4.46969004938 + 5863.3473886327*$t);
   $X1 += 0.00000000184*cos(3.01631602481 + 10983.94853421629*$t);
   $X1 += 0.00000000184*cos(3.38049875173 + 6037.4880212455*$t);
   $X1 += 0.00000000156*cos(2.34433370292 + 2648.6986429565*$t);
   $X1 += 0.00000000182*cos(3.47271515850 + 10969.72144021469*$t);
   $X1 += 0.00000000175*cos(3.26771548230 + 6529.1513137043*$t);
   $X1 += 0.00000000154*cos(5.67725689556 + 6453.9925380941*$t);
   $X1 += 0.00000000154*cos(0.97127667708 + 6112.6467968557*$t);
   $X1 += 0.00000000164*cos(4.52273465985 + 12353.0964220283*$t);
   $X1 += 0.00000000146*cos(4.05729660897 + 149.8070146181*$t);
   $X1 += 0.00000000184*cos(0.20513580951 + 11015.3502948183*$t);
   $X1 += 0.00000000191*cos(2.29402324776 + 3894.4256470257*$t);
   $X1 += 0.00000000144*cos(3.76178854682 + 18636.1722720197*$t);
   $X1 += 0.00000000146*cos(1.99164382254 + 13760.8425276909*$t);
   $X1 += 0.00000000142*cos(1.34565609994 + 4156.95462512869*$t);
   $X1 += 0.00000000185*cos(3.72101619927 + 7234.5504387585*$t);
   $X1 += 0.00000000133*cos(0.01824676794 + 18139.5383188994*$t);
   $X1 += 0.00000000131*cos(4.75893185547 + 5507.3094211839*$t);
   $X1 += 0.00000000146*cos(5.03857686201 + 18875.2820522905*$t);
   $X1 += 0.00000000128*cos(3.41348197806 + 6069.53293706989*$t);
   $X1 += 0.00000000128*cos(0.18086618365 + 16200.52890701769*$t);
   $X1 += 0.00000000123*cos(5.45134858107 + 20.5315780089*$t);
   $X1 += 0.00000000121*cos(0.98195264587 + 17256.8753538249*$t);
   $X1 += 0.00000000123*cos(0.10275700246 + 6393.5259886943*$t);
   $X1 += 0.00000000123*cos(0.26259126301 + 6173.1133462555*$t);
   $X1 += 0.00000000117*cos(4.47711713050 + 18852.9944858067*$t);
   $X1 += 0.00000000120*cos(1.03913122140 + 9381.2034902007*$t);
   $X1 += 0.00000000113*cos(2.33706395068 + 65147.37595065419*$t);
   $X1 += 0.00000000137*cos(3.68199834086 + 16201.0165419847*$t);
   $X1 += 0.00000000113*cos(5.07439785395 + 6836.40143535029*$t);
   $X1 += 0.00000000123*cos(4.42839393502 + 12565.4151963981*$t);
   $X1 += 0.00000000132*cos(6.08414353296 + 17655.0243572331*$t);
   $X1 += 0.00000000113*cos(4.12294240934 + 10575.1628654583*$t);
   $X1 += 0.00000000113*cos(2.09696400140 + 10213.0417287275*$t);
   $X1 += 0.00000000116*cos(2.23828708218 + 17253.2849251731*$t);
   $X1 += 0.00000000120*cos(2.28034205827 + 10420.23010099111*$t);
   $X1 += 0.00000000109*cos(0.50597705280 + 633.0275567967*$t);
   $X1 += 0.00000000103*cos(0.11832394316 + 6295.0490203109*$t);
   $X1 += 0.00000000103*cos(0.24702432231 + 6271.5903146389*$t);
   $X1 += 0.00000000104*cos(5.58102765204 + 23013.7833570707*$t);
   $X1 += 0.00000000105*cos(3.75605193168 + 11790.3852711753*$t);
   $X1 += 0.00000000109*cos(2.65465737803 + 3340.36860921629*$t);
   $X1 += 0.00000000102*cos(2.59681923716 + 10970.2090751817*$t);
   $X1 += 0.00000000101*cos(4.42190708814 + 4803.9654584435*$t);
   $X1 += 0.00000000114*cos(2.21750546661 + 4171.1817191303*$t);
   $X1 += 0.00000000111*cos(3.66232090402 + 1582.2031657665*$t);
   $X1 += 0.00000000107*cos(2.47220154064 + 23543.4743221653*$t);
   $X1 += 0.00000000099*cos(2.67705199399 + 10973.3118688665*$t);
   $X1 += 0.00000000102*cos(5.67042027327 + 6549.9267091967*$t);
   $X1 += 0.00000000102*cos(0.97811329938 + 6016.7126257531*$t);
   $X1 += 0.00000000120*cos(2.49695982999 + 11514.1271342779*$t);
   $X1 += 0.00000000095*cos(2.69903868611 + 17267.5120191747*$t);
   $X1 += 0.00000000121*cos(0.05336928445 + 13625.7774476555*$t);
   $X1 += 0.00000000100*cos(2.04355401267 + 10976.83498721549*$t);
   $X1 += 0.00000000090*cos(3.68840859712 + 76.50988875911*$t);
   $X1 += 0.00000000100*cos(4.37665365923 + 8661.9965060795*$t);
   $X1 += 0.00000000089*cos(5.33255005639 + 1350.1112271423*$t);
   $X1 += 0.00000000087*cos(4.33390332130 + 9917.9406919933*$t);
   $X1 += 0.00000000086*cos(5.93949751968 + 12809.12412144031*$t);
   $X1 += 0.00000000095*cos(1.34358559692 + 12345.9828750275*$t);
   $X1 += 0.00000000084*cos(1.54530867800 + 775.2787938405*$t);
   $X1 += 0.00000000106*cos(5.14280637669 + 11216.5281078075*$t);
   $X1 += 0.00000000095*cos(2.50046187937 + 12012.8261146239*$t);
   $X1 += 0.00000000082*cos(0.01600918310 + 5643.4223811609*$t);
   $X1 += 0.00000000081*cos(3.84772162011 + 11614.6771112157*$t);
   $X1 += 0.00000000084*cos(1.14755262027 + 6255.9181113781*$t);
   $X1 += 0.00000000084*cos(5.50098095238 + 6310.7212235717*$t);
   $X1 += 0.00000000080*cos(0.34711727404 + 6923.2169537889*$t);
   $X1 += 0.00000000085*cos(5.79487336888 + 5635.8211991931*$t);
   $X1 += 0.00000000084*cos(3.04423309467 + 10440.03047512009*$t);
   $X1 += 0.00000000080*cos(4.19702378753 + 3583.5848481573*$t);
   $X1 += 0.00000000092*cos(0.87932993874 + 10818.3791043993*$t);
   $X1 += 0.00000000077*cos(5.70308186355 + 6993.2527160332*$t);
   $X1 += 0.00000000080*cos(1.36958890049 + 13521.9952590749*$t);
   $X1 += 0.00000000104*cos(5.42045444776 + 5650.0482931947*$t);
   $X1 += 0.00000000093*cos(0.29330855376 + 4732.2744448269*$t);
   $X1 += 0.00000000085*cos(0.17489631763 + 22805.4917485101*$t);
   $X1 += 0.00000000086*cos(0.81273058913 + 12360.2099690291*$t);
   $X1 += 0.00000000075*cos(4.00156639703 + 5120.35732810009*$t);
   $X1 += 0.00000000090*cos(0.62558959530 + 6370.62787201471*$t);
   $X1 += 0.00000000090*cos(6.02294397735 + 6196.0114629351*$t);
   $X1 += 0.00000000082*cos(0.19767948100 + 18842.3578204569*$t);
   $X1 += 0.00000000101*cos(6.08995730057 + 3097.64000524229*$t);
   $X1 += 0.00000000094*cos(5.45896547971 + 10177.01386205009*$t);
   $X1 += 0.00000000083*cos(5.31992956371 + 18415.7596295809*$t);
   $X1 += 0.00000000073*cos(6.09826272325 + 949.4194264533*$t);
   $X1 += 0.00000000101*cos(2.86248284119 + 3104.6862419403*$t);
   $X1 += 0.00000000074*cos(5.96635290969 + 13517.62628874989*$t);
   $X1 += 0.00000000092*cos(2.35039346805 + 16858.7263504167*$t);
   $X1 += 0.00000000075*cos(0.00198703030 + 1059.1381127057*$t);
   $X1 += 0.00000000085*cos(6.09242867296 + 398.3928208917*$t);
   $X1 += 0.00000000069*cos(0.89145782253 + 11713.1991357143*$t);
   $X1 += 0.00000000084*cos(4.35477901935 + 2378.9206560881*$t);
   $X1 += 0.00000000069*cos(0.61262618132 + 11712.71150074729*$t);
   $X1 += 0.00000000075*cos(1.62832949134 + 30356.2411372513*$t);
   $X1 += 0.00000000067*cos(4.61214364579 + 3154.9309023791*$t);
   $X1 += 0.00000000076*cos(4.83480996889 + 18429.9867235825*$t);
   $X1 += 0.00000000068*cos(4.26265140929 + 11925.5179100841*$t);
   $X1 += 0.00000000067*cos(3.24609638890 + 10454.25756912169*$t);
   $X1 += 0.00000000073*cos(6.07047724510 + 15670.83794192309*$t);
   $X1 += 0.00000000065*cos(0.20786131115 + 6438.2524319421*$t);
   $X1 += 0.00000000069*cos(3.29499829591 + 17298.4261448097*$t);
   $X1 += 0.00000000087*cos(5.09754892709 + 3904.1551939033*$t);
   $X1 += 0.00000000076*cos(0.73384165663 + 5231.0512842865*$t);
   $X1 += 0.00000000064*cos(3.69394501139 + 3496.2766436175*$t);
   $X1 += 0.00000000078*cos(4.31639568529 + 8672.21368792411*$t);
   $X1 += 0.00000000066*cos(3.41816290372 + 14143.2514249471*$t);
   $X1 += 0.00000000063*cos(0.99537559299 + 24357.0246061251*$t);
   $X1 += 0.00000000063*cos(5.29204726636 + 15720.5949673949*$t);
   $X1 += 0.00000000061*cos(3.72699038600 + 16460.5773470085*$t);
   $X1 += 0.00000000073*cos(4.50408758477 + 9387.76209193169*$t);
   $X1 += 0.00000000057*cos(2.80099560279 + 17259.91083720689*$t);
   $X1 += 0.00000000059*cos(0.69432195316 + 9778.8648586419*$t);
   $X1 += 0.00000000058*cos(3.24953926506 + 155.6642169177*$t);
   $X1 += 0.00000000060*cos(3.51548937348 + 34570.3101523361*$t);
   $X1 += 0.00000000057*cos(4.46074610322 + 149854.64395229139*$t);
   $X1 += 0.00000000056*cos(5.33393809517 + 12456.1891962469*$t);
   $X1 += 0.00000000056*cos(0.91240244730 + 17996.2749857057*$t);
   $X1 += 0.00000000056*cos(2.17966368743 + 11764.5745863425*$t);
   $X1 += 0.00000000054*cos(2.96145051332 + 4705.4884900601*$t);
   $X1 += 0.00000000055*cos(4.53949491324 + 13915.77529215809*$t);
   $X1 += 0.00000000061*cos(0.15191781763 + 28237.4772768729*$t);
   $X1 += 0.00000000065*cos(5.87045161229 + 7335.5880506633*$t);
   $X1 += 0.00000000072*cos(0.77556931591 + 6055.8435346859*$t);
   $X1 += 0.00000000072*cos(5.87296425674 + 6510.7958002639*$t);
   $X1 += 0.00000000071*cos(0.23083621209 + 12545.6201219739*$t);
   $X1 += 0.00000000061*cos(1.27148175728 + 6312.74917601091*$t);
   $X1 += 0.00000000061*cos(5.37705181537 + 6253.8901589389*$t);
   $X1 += 0.00000000052*cos(0.26640147565 + 5326.5428765373*$t);
   $X1 += 0.00000000054*cos(1.21229891621 + 2699.49100183409*$t);
   $X1 += 0.00000000052*cos(2.44818566509 + 8983.0544867925*$t);
   $X1 += 0.00000000066*cos(2.35337977576 + 14919.2616712381*$t);
   $X1 += 0.00000000050*cos(5.93502263343 + 5572.8989839496*$t);
   $X1 += 0.00000000051*cos(0.76420455409 + 16522.4158985187*$t);
   $X1 += 0.00000000049*cos(4.18738074622 + 35579.9350570535*$t);
   $X1 += 0.00000000053*cos(1.36303997543 + 6208.5380689076*$t);
   $X1 += 0.00000000053*cos(5.28549359722 + 6358.1012660422*$t);
   $X1 += 0.00000000049*cos(4.92672897893 + 21393.7857873411*$t);
   $X1 += 0.00000000049*cos(6.04327871377 + 5547.4431539431*$t);
   $X1 += 0.00000000049*cos(0.60525485888 + 7019.19618100671*$t);
   $X1 += 0.00000000049*cos(2.59123701567 + 12416.8323203317*$t);
   $X1 += 0.00000000050*cos(5.65157500725 + 12592.2062022991*$t);
   $X1 += 0.00000000050*cos(0.07520350695 + 26084.2656236997*$t);
   $X1 += 0.00000000051*cos(3.40680645218 + 23006.6698100699*$t);
   $X1 += 0.00000000058*cos(2.46543782438 + 15141.14697682849*$t);
   $X1 += 0.00000000047*cos(0.97665096734 + 24279.3508356971*$t);
   $X1 += 0.00000000049*cos(4.98006572526 + 6816.53375091851*$t);
   $X1 += 0.00000000049*cos(1.66846784739 + 5750.1055840313*$t);
   $X1 += 0.00000000046*cos(3.45134537804 + 19379.1623325523*$t);
   $X1 += 0.00000000052*cos(4.17729143118 + 16737.33341911309*$t);
   $X1 += 0.00000000047*cos(2.09298679166 + 24066.0517402591*$t);
   $X1 += 0.00000000046*cos(5.73796654718 + 104351.85638378029*$t);
   $X1 += 0.00000000063*cos(1.54245276158 + 8662.48414104651*$t);
   $X1 += 0.00000000048*cos(1.71424177408 + 26735.7014447297*$t);
   $X1 += 0.00000000046*cos(5.48854983581 + 12132.6837795895*$t);
   $X1 += 0.00000000050*cos(4.49443808542 + 24602.8562523545*$t);
   $X1 += 0.00000000054*cos(0.03975141461 + 7834.3648901229*$t);
   $X1 += 0.00000000045*cos(5.05887207146 + 161000.92955515759*$t);
   $X1 += 0.00000000049*cos(4.51484833854 + 29303.9727540629*$t);
   $X1 += 0.00000000049*cos(3.00236481482 + 10984.4361691833*$t);
   $X1 += 0.00000000045*cos(5.42122497288 + 45584.9289947039*$t);
   $X1 += 0.00000000046*cos(3.44074843825 + 5244.2930566845*$t);
   $X1 += 0.00000000048*cos(1.96431249949 + 23581.0143598341*$t);
   $X1 += 0.00000000049*cos(0.60097746518 + 19650.8046636145*$t);
   $X1 += 0.00000000049*cos(4.98977220505 + 29289.7456600613*$t);
   $X1 += 0.00000000044*cos(3.01088620106 + 6390.9831914135*$t);
   $X1 += 0.00000000044*cos(3.63764737159 + 6175.6561435363*$t);
   $X1 += 0.00000000051*cos(1.08500510566 + 17789.60180230149*$t);
   $X1 += 0.00000000042*cos(1.47304377234 + 24705.9490265731*$t);
   $X1 += 0.00000000042*cos(4.33726197214 + 15663.79170522509*$t);
   $X1 += 0.00000000043*cos(1.51170891404 + 16460.08971204149*$t);
   $X1 += 0.00000000043*cos(1.85670214007 + 553.8132203259*$t);
   $X1 += 0.00000000041*cos(4.78695133916 + 17370.6047933933*$t);
   $X1 += 0.00000000053*cos(1.02071442021 + 13119.47728534169*$t);
   $X1 += 0.00000000042*cos(0.40756986853 + 4164.5558070965*$t);
   $X1 += 0.00000000041*cos(3.62600931231 + 23020.8969040715*$t);
   $X1 += 0.00000000045*cos(2.13148338898 + 54247.1693182669*$t);
   $X1 += 0.00000000040*cos(3.00393862246 + 32243.2546833971*$t);
   $X1 += 0.00000000040*cos(0.45945430396 + 24336.2492106327*$t);
   $X1 += 0.00000000040*cos(0.14772670564 + 11769.6098756829*$t);
   $X1 += 0.00000000045*cos(4.68905632727 + 16723.10632511149*$t);
   $X1 += 0.00000000045*cos(1.54746367368 + 16723.5939600785*$t);
   $X1 += 0.00000000042*cos(4.32247442790 + 8827.14645239129*$t);
   $X1 += 0.00000000040*cos(0.00254937606 + 8402.0835278533*$t);
   $X1 += 0.00000000039*cos(5.06793905702 + 16062.4283436003*$t);
   $X1 += 0.00000000044*cos(2.87431244066 + 5856.23384163189*$t);
   $X1 += 0.00000000038*cos(2.71291798518 + 6297.5467614765*$t);
   $X1 += 0.00000000038*cos(3.93561558747 + 6269.0925734733*$t);
   $X1 += 0.00000000041*cos(4.95313128130 + 15508.8589407579*$t);
   $X1 += 0.00000000038*cos(0.25558032671 + 6394.50630976251*$t);
   $X1 += 0.00000000038*cos(0.10976793876 + 6172.1330251873*$t);
   $X1 += 0.00000000039*cos(3.31895424299 + 29826.5501721567*$t);
   $X1 += 0.00000000046*cos(5.02561530595 + 17297.9385098427*$t);
   $X1 += 0.00000000041*cos(2.47236149910 + 6309.1303523077*$t);
   $X1 += 0.00000000038*cos(1.11170997081 + 29089.0552334685*$t);
   $X1 += 0.00000000038*cos(6.16488744565 + 11933.6117781531*$t);
   $X1 += 0.00000000037*cos(4.91170884792 + 26482.4146271079*$t);
   $X1 += 0.00000000037*cos(2.07642652738 + 20452.6255947383*$t);
   $X1 += 0.00000000037*cos(5.02138933475 + 18073.46112116669*$t);
   $X1 += 0.00000000036*cos(1.41330554556 + 11499.41240530929*$t);
   $X1 += 0.00000000045*cos(0.04410300421 + 3340.8562441833*$t);
   $X1 += 0.00000000049*cos(5.78167784671 + 18216.6876281445*$t);
   $X1 += 0.00000000038*cos(6.16266694488 + 18202.4605341429*$t);
   $X1 += 0.00000000041*cos(5.48085477947 + 5216.8241902849*$t);
   $X1 += 0.00000000036*cos(4.64059514280 + 4060.97539791089*$t);
   $X1 += 0.00000000048*cos(0.43106257187 + 951.9622237341*$t);
   $X1 += 0.00000000041*cos(2.33852383906 + 18848.98373249069*$t);
   $X1 += 0.00000000038*cos(6.17960213131 + 26709.8907598969*$t);
   $X1 += 0.00000000036*cos(4.69601378640 + 28230.4310401749*$t);
   $X1 += 0.00000000036*cos(0.39760453863 + 24491.18197509989*$t);
   $X1 += 0.00000000036*cos(6.28050310342 + 9924.5666040271*$t);
   $X1 += 0.00000000048*cos(0.32621587842 + 8982.5668518255*$t);
   $X1 += 0.00000000037*cos(5.17748996647 + 22484.0923919761*$t);
   $X1 += 0.00000000047*cos(1.64189885412 + 6040.10342853389*$t);
   $X1 += 0.00000000034*cos(5.26052334707 + 6418.9449924849*$t);
   $X1 += 0.00000000034*cos(1.38801022558 + 6147.69434246491*$t);
   $X1 += 0.00000000034*cos(2.09927479760 + 9380.71585523369*$t);
   $X1 += 0.00000000036*cos(5.01642644412 + 7238.4317741165*$t);
   $X1 += 0.00000000034*cos(1.97967235505 + 17272.1278250099*$t);
   $X1 += 0.00000000037*cos(2.80234472663 + 58953.3892607775*$t);
   $X1 += 0.00000000034*cos(1.90506635542 + 29026.7290469913*$t);
   $X1 += 0.00000000033*cos(2.30422309015 + 12721.32828193349*$t);
   $X1 += 0.00000000035*cos(3.20661909223 + 7322.34627826531*$t);
   $X1 += 0.00000000044*cos(2.79016934175 + 3981.73385156551*$t);
   $X1 += 0.00000000033*cos(2.02741588975 + 19804.5834740993*$t);
   $X1 += 0.00000000033*cos(4.13127522861 + 173567.08125514039*$t);
   $X1 += 0.00000000033*cos(4.59578048276 + 205.9417309537*$t);
   $X1 += 0.00000000033*cos(3.87810188817 + 23013.29572210369*$t);
   $X1 += 0.00000000034*cos(0.56182222293 + 49515.1386909235*$t);
   $X1 += 0.00000000034*cos(4.95130100456 + 38500.5198485557*$t);
   $X1 += 0.00000000032*cos(4.38300866562 + 17686.9966630499*$t);
   $X1 += 0.00000000036*cos(0.10343586811 + 5641.9544251257*$t);
   $X1 += 0.00000000032*cos(4.48170445266 + 21228.63584102931*$t);
   $X1 += 0.00000000033*cos(1.70919210465 + 34520.5531268643*$t);
   $X1 += 0.00000000033*cos(4.08300924175 + 8635.69818627969*$t);
   $X1 += 0.00000000031*cos(0.30962555206 + 5746.0275204125*$t);
   $X1 += 0.00000000041*cos(4.09101330171 + 21202.3375212295*$t);
   $X1 += 0.00000000037*cos(1.21597687953 + 7349.81514466491*$t);
   $X1 += 0.00000000034*cos(0.61019544663 + 16496.11757871889*$t);
   $X1 += 0.00000000032*cos(2.47223060088 + 33019.2649296881*$t);
   $X1 += 0.00000000032*cos(6.25412877228 + 21150.56954840009*$t);
   $X1 += 0.00000000037*cos(3.41910013684 + 29296.8592070621*$t);
   $X1 += 0.00000000036*cos(3.45170661535 + 27511.2240560537*$t);
   $X1 += 0.00000000035*cos(2.20855027459 + 11933.12414318609*$t);
   $X1 += 0.00000000035*cos(2.69261850768 + 11918.89704918449*$t);
   $X1 += 0.00000000035*cos(0.83074174405 + 29062.7569136687*$t);
   $X1 += 0.00000000030*cos(2.96012497927 + 12490.1294461907*$t);
   $X1 += 0.00000000032*cos(0.38688555099 + 6334.60000533731*$t);
   $X1 += 0.00000000032*cos(6.26164802166 + 6232.03932961249*$t);
   $X1 += 0.00000000029*cos(2.48914084100 + 419.2408263917*$t);
   $X1 += 0.00000000029*cos(2.48914084100 + 419.72846135871*$t);
   $X1 += 0.00000000029*cos(0.09098498823 + 16858.23871544969*$t);
   $X1 += 0.00000000030*cos(4.30269873097 + 16208.13008898551*$t);
   $X1 += 0.00000000029*cos(2.26254947152 + 27278.7126339243*$t);
   $X1 += 0.00000000032*cos(3.16446260317 + 5017.2645538815*$t);
   $X1 += 0.00000000028*cos(1.87847577782 + 24080.2788342607*$t);
   $X1 += 0.00000000030*cos(2.57345584103 + 23539.9512038163*$t);
   $X1 += 0.00000000029*cos(6.17121903514 + 18606.25512851669*$t);
   $X1 += 0.00000000030*cos(3.16071544619 + 30665.9111409493*$t);
   $X1 += 0.00000000034*cos(5.86279015300 + 6660.6932753907*$t);
   $X1 += 0.00000000034*cos(0.78574341965 + 5905.94605955911*$t);
   $X1 += 0.00000000038*cos(5.12222686816 + 31571.0434668747*$t);
   $X1 += 0.00000000038*cos(2.95935499043 + 16061.94070863329*$t);
   $X1 += 0.00000000029*cos(0.07805875622 + 23539.46356884929*$t);
   $X1 += 0.00000000033*cos(1.31621330654 + 3738.51761262449*$t);
   $X1 += 0.00000000028*cos(4.45185289388 + 2942.21960580809*$t);
   $X1 += 0.00000000032*cos(4.32330584281 + 15110.7099373497*$t);
   $X1 += 0.00000000030*cos(1.39195112722 + 55023.1795645579*$t);
   $X1 += 0.00000000027*cos(0.53286078729 + 233141.55822184499*$t);
   $X1 += 0.00000000030*cos(2.32405719366 + 12587.1709129587*$t);
   $X1 += 0.00000000028*cos(5.54771281521 + 34911.6558935745*$t);
   $X1 += 0.00000000027*cos(6.17188911074 + 1692.40948698591*$t);
   $X1 += 0.00000000027*cos(4.56663555607 + 2544.07060239989*$t);
   $X1 += 0.00000000027*cos(3.42500443489 + 24382.8352909579*$t);
   $X1 += 0.00000000027*cos(5.99215939135 + 6205.6458970469*$t);
   $X1 += 0.00000000027*cos(0.65637418130 + 6360.99343790291*$t);
   $X1 += 0.00000000027*cos(0.24480889857 + 31172.8944634665*$t);
   $X1 += 0.00000000026*cos(3.76734683925 + 36949.4746259077*$t);
   $X1 += 0.00000000034*cos(6.02674277322 + 18053.1733606413*$t);
   $X1 += 0.00000000027*cos(2.32047706036 + 32367.3414736911*$t);
   $X1 += 0.00000000027*cos(1.77181133426 + 4487.57358878689*$t);
   $X1 += 0.00000000028*cos(3.80331914659 + 10239.3400485273*$t);
   $X1 += 0.00000000026*cos(6.01470923824 + 4796.85191144269*$t);
   $X1 += 0.00000000026*cos(6.27941236443 + 6226.4212925393*$t);
   $X1 += 0.00000000026*cos(0.36912120822 + 6340.2180424105*$t);
   $X1 += 0.00000000029*cos(0.18788001431 + 5974.0413448191*$t);
   $X1 += 0.00000000029*cos(0.17746825116 + 6592.5979901307*$t);
   $X1 += 0.00000000026*cos(3.39870851542 + 19402.55313533309*$t);
   $X1 += 0.00000000028*cos(2.42845493949 + 46386.7499258277*$t);
   $X1 += 0.00000000032*cos(0.33223863826 + 9225.7830907665*$t);
   $X1 += 0.00000000026*cos(2.54665382210 + 9910.33951002549*$t);
   $X1 += 0.00000000025*cos(5.74078884001 + 15265.64270181689*$t);
   $X1 += 0.00000000029*cos(5.90495655038 + 52176.0501006319*$t);
   $X1 += 0.00000000027*cos(4.22338082584 + 62883.5989569971*$t);
   $X1 += 0.00000000024*cos(0.55027097677 + 11617.21990849651*$t);
   $X1 += 0.00000000024*cos(2.62016281496 + 5120.8449630671*$t);
   $X1 += 0.00000000028*cos(3.25064295131 + 6680.98103591609*$t);
   $X1 += 0.00000000024*cos(4.40689701323 + 522.3336006103*$t);
   $X1 += 0.00000000025*cos(3.86362291910 + 6263.64990657511*$t);
   $X1 += 0.00000000025*cos(2.78491065355 + 6302.9894283747*$t);
   $X1 += 0.00000000029*cos(2.44405132476 + 23581.5019948011*$t);
   $X1 += 0.00000000024*cos(0.50709291410 + 29424.8780503995*$t);
   $X1 += 0.00000000028*cos(2.66240986547 + 16737.8210540801*$t);
   $X1 += 0.00000000024*cos(1.98880441412 + 22743.16556203289*$t);
   $X1 += 0.00000000024*cos(5.13039706771 + 22743.6531969999*$t);
   $X1 += 0.00000000024*cos(5.85302121512 + 25158.8455372489*$t);
   $X1 += 0.00000000025*cos(3.95115098429 + 21424.2228268199*$t);
   $X1 += 0.00000000025*cos(0.80955833070 + 21424.7104617869*$t);
   $X1 += 0.00000000024*cos(1.92863541994 + 14712.07329897449*$t);
   $X1 += 0.00000000028*cos(4.50601984314 + 5760.25461441409*$t);
   $X1 += 0.00000000024*cos(3.73455640738 + 31969.1924702829*$t);
   $X1 += 0.00000000023*cos(0.62345777089 + 17893.1822114871*$t);
   $X1 += 0.00000000030*cos(3.85563309519 + 8584.9054833843*$t);
   $X1 += 0.00000000024*cos(3.21746306158 + 23550.5878691661*$t);
   $X1 += 0.00000000023*cos(1.36063534536 + 6717.0314053517*$t);
   $X1 += 0.00000000022*cos(3.99741791962 + 7445.7943718827*$t);
   $X1 += 0.00000000023*cos(5.13650786055 + 5849.6079295981*$t);
   $X1 += 0.00000000025*cos(0.81555419576 + 14155.4684127501*$t);
   $X1 += 0.00000000023*cos(3.58569412080 + 6294.36536773881*$t);
   $X1 += 0.00000000023*cos(3.06283945185 + 6272.273967211*$t);
   $X1 += 0.00000000024*cos(4.24597953395 + 18208.1061251085*$t);
   $X1 += 0.00000000024*cos(1.10438688036 + 18208.5937600755*$t);
   $X1 += 0.00000000021*cos(5.78251945962 + 22779.6810636773*$t);
   $X1 += 0.00000000029*cos(3.31448732534 + 28628.5800435831*$t);
   $X1 += 0.00000000021*cos(5.13427576878 + 40879.6843221273*$t);
   $X1 += 0.00000000021*cos(4.08575884387 + 30221.1760572159*$t);
   $X1 += 0.00000000022*cos(1.38262024424 + 4379.8828549737*$t);
   $X1 += 0.00000000022*cos(5.26591980546 + 8186.7564799761*$t);
   $X1 += 0.00000000022*cos(4.52225402167 + 28760.05469496669*$t);
   $X1 += 0.00000000026*cos(3.41053115091 + 18422.8731765817*$t);
   $X1 += 0.00000000020*cos(0.80720267360 + 12036.21691740469*$t);
   $X1 += 0.00000000020*cos(4.78724181293 + 28313.0449871775*$t);
   $X1 += 0.00000000027*cos(0.67000770864 + 34513.5068901663*$t);
   $X1 += 0.00000000020*cos(2.07630932980 + 34115.3578867581*$t);
   $X1 += 0.00000000020*cos(1.72148703277 + 21548.7185518083*$t);
   $X1 += 0.00000000020*cos(3.68576430300 + 17654.5367222661*$t);
   $X1 += 0.00000000025*cos(3.53209868087 + 55798.7021758819*$t);
   $X1 += 0.00000000020*cos(0.81961551511 + 20597.4877805247*$t);
   $X1 += 0.00000000020*cos(2.60684350676 + 18875.76968725751*$t);
   $X1 += 0.00000000022*cos(0.44774356245 + 1588.82907780029*$t);
   $X1 += 0.00000000024*cos(1.20852424503 + 9411.7084325707*$t);
   $X1 += 0.00000000019*cos(5.85945140949 + 536.5606946119*$t);
   $X1 += 0.00000000020*cos(4.99560905506 + 6187.3404402571*$t);
   $X1 += 0.00000000020*cos(1.65292451759 + 6379.2988946927*$t);
   $X1 += 0.00000000022*cos(4.44471803531 + 18326.8939493639*$t);
   $X1 += 0.00000000022*cos(2.18348213630 + 6233.5626420031*$t);
   $X1 += 0.00000000022*cos(4.46505143635 + 6333.0766929467*$t);
   $X1 += 0.00000000018*cos(2.03036294822 + 5972.4788686065*$t);
   $X1 += 0.00000000018*cos(4.61817062443 + 6594.1604663433*$t);
   $X1 += 0.00000000019*cos(4.28397604583 + 6948.0757126049*$t);
   $X1 += 0.00000000019*cos(2.36455752682 + 5618.5636223449*$t);
   $X1 += 0.00000000019*cos(0.18461366541 + 83974.07917321339*$t);
   $X1 += 0.00000000019*cos(4.29574443999 + 84020.10309797739*$t);
   $X1 += 0.00000000020*cos(3.84957027230 + 50316.9596220473*$t);
   $X1 += 0.00000000020*cos(2.85391796921 + 9070.3626913323*$t);
   return $X1*$t;
}



   function Earth_X2($t) // 248 terms of order 2
{
   $X2  = 0.00052911498*cos(3.32403354915 + 6283.3196674749*$t);
   $X2 += 0.00006074441*cos(2.05485843872 + 0.2438174835*$t);
   $X2 += 0.00002406871*cos(4.23086027149 + 12566.3955174663*$t);
   $X2 += 0.00000096033*cos(4.14485769516 + 18849.4713674577*$t);
   $X2 += 0.00000029888*cos(0.73436192593 + 6282.83203250789*$t);
   $X2 += 0.00000014021*cos(3.21121684298 + 6279.7965491259*$t);
   $X2 += 0.00000013375*cos(3.31400005464 + 6286.84278582391*$t);
   $X2 += 0.00000008148*cos(2.19949162987 + 6309.61798727471*$t);
   $X2 += 0.00000008195*cos(4.38954108022 + 6257.0213476751*$t);
   $X2 += 0.00000008001*cos(5.91956929382 + 6127.8992680407*$t);
   $X2 += 0.00000007754*cos(0.72231700953 + 6438.7400669091*$t);
   $X2 += 0.00000007223*cos(5.38114271434 + 83997.09113559539*$t);
   $X2 += 0.00000004616*cos(6.16703227370 + 710.1768660418*$t);
   $X2 += 0.00000003060*cos(4.83964573312 + 4705.9761250271*$t);
   $X2 += 0.00000003717*cos(2.45040909403 + 5507.7970561509*$t);
   $X2 += 0.00000002991*cos(4.05993241608 + 25132.5472174491*$t);
   $X2 += 0.00000003235*cos(4.10584554352 + 7058.8422787989*$t);
   $X2 += 0.00000002393*cos(3.40776365917 + 775.7664288075*$t);
   $X2 += 0.00000002034*cos(4.34141229883 + 6290.4332144757*$t);
   $X2 += 0.00000002028*cos(2.28023581144 + 6276.2061204741*$t);
   $X2 += 0.00000002064*cos(1.58527926677 + 7860.6632099227*$t);
   $X2 += 0.00000001914*cos(4.42314112548 + 5487.0216606585*$t);
   $X2 += 0.00000001770*cos(5.05862783743 + 1059.6257476727*$t);
   $X2 += 0.00000001761*cos(3.37902852940 + 5753.6287023803*$t);
   $X2 += 0.00000001558*cos(2.13660228192 + 7079.61767429131*$t);
   $X2 += 0.00000001551*cos(2.12932375831 + 6040.5910635009*$t);
   $X2 += 0.00000001436*cos(2.85243683562 + 5885.1706640667*$t);
   $X2 += 0.00000001469*cos(4.51404554645 + 6526.04827144891*$t);
   $X2 += 0.00000001319*cos(3.46234944603 + 6681.46867088311*$t);
   $X2 += 0.00000001180*cos(2.99170595830 + 6813.0106325695*$t);
   $X2 += 0.00000001337*cos(1.03982518814 + 5729.7502646325*$t);
   $X2 += 0.00000001039*cos(5.82323058792 + 6282.3393464067*$t);
   $X2 += 0.00000001313*cos(2.16977183919 + 529.9347825781*$t);
   $X2 += 0.00000001041*cos(0.76562479642 + 6284.2999885431*$t);
   $X2 += 0.00000001202*cos(5.59909179750 + 6836.8890703173*$t);
   $X2 += 0.00000000904*cos(2.42848578586 + 5331.6012612243*$t);
   $X2 += 0.00000000870*cos(3.83740101245 + 4933.4522578161*$t);
   $X2 += 0.00000001149*cos(4.59530313333 + 167284.00540514899*$t);
   $X2 += 0.00000001069*cos(5.80442130983 + 1577.0997249643*$t);
   $X2 += 0.00000000772*cos(4.23369730112 + 7235.0380737255*$t);
   $X2 += 0.00000000846*cos(2.93878609085 + 11790.8729061423*$t);
   $X2 += 0.00000000742*cos(0.30768536261 + 4137.1542509997*$t);
   $X2 += 0.00000000966*cos(3.62307276164 + 11856.462468908*$t);
   $X2 += 0.00000000664*cos(2.78855764568 + 7633.1870771337*$t);
   $X2 += 0.00000000662*cos(4.76048207091 + 1194.6908277081*$t);
   $X2 += 0.00000000659*cos(1.23774551244 + 11507.0135872771*$t);
   $X2 += 0.00000000853*cos(5.22418483353 + 4535.3032544079*$t);
   $X2 += 0.00000000823*cos(5.77365627974 + 155.17658195069*$t);
   $X2 += 0.00000000783*cos(5.87375907691 + 5088.8726572503*$t);
   $X2 += 0.00000000785*cos(5.03738396313 + 2353.1099712553*$t);
   $X2 += 0.00000000694*cos(1.93706172082 + 801.5771136403*$t);
   $X2 += 0.00000000566*cos(6.00002295544 + 796.5418242999*$t);
   $X2 += 0.00000000684*cos(2.68347359202 + 3128.6325825793*$t);
   $X2 += 0.00000000499*cos(2.99674498554 + 12569.91863581531*$t);
   $X2 += 0.00000000596*cos(1.38539217884 + 8031.3360805419*$t);
   $X2 += 0.00000000486*cos(6.11316978391 + 1589.3167127673*$t);
   $X2 += 0.00000000483*cos(6.25283705744 + 8429.4850839501*$t);
   $X2 += 0.00000000441*cos(0.27873568624 + 6496.6187629129*$t);
   $X2 += 0.00000000412*cos(5.58840963851 + 4693.75913722409*$t);
   $X2 += 0.00000000561*cos(0.75091675093 + 7477.7666776995*$t);
   $X2 += 0.00000000390*cos(6.20968064366 + 6070.0205720369*$t);
   $X2 += 0.00000000405*cos(1.55299195368 + 3739.0052475915*$t);
   $X2 += 0.00000000501*cos(3.39229516665 + 1592.8398311163*$t);
   $X2 += 0.00000000392*cos(5.39455294636 + 26.0545023163*$t);
   $X2 += 0.00000000369*cos(4.36331143713 + 553.32558535889*$t);
   $X2 += 0.00000000483*cos(1.36538995021 + 12036.7045523717*$t);
   $X2 += 0.00000000489*cos(2.13588971106 + 5223.4501023187*$t);
   $X2 += 0.00000000481*cos(4.74275484789 + 10213.5293636945*$t);
   $X2 += 0.00000000448*cos(0.85608336591 + 156137.71980228279*$t);
   $X2 += 0.00000000321*cos(2.96911879180 + 951.4745887671*$t);
   $X2 += 0.00000000320*cos(2.00218171333 + 1990.9888345245*$t);
   $X2 += 0.00000000361*cos(0.59519501420 + 12565.9078824993*$t);
   $X2 += 0.00000000361*cos(0.08584353459 + 12721.8159169005*$t);
   $X2 += 0.00000000412*cos(5.91338851626 + 398.3928208917*$t);
   $X2 += 0.00000000304*cos(0.96839213314 + 4690.7236538421*$t);
   $X2 += 0.00000000356*cos(1.28855678128 + 5481.4987363511*$t);
   $X2 += 0.00000000283*cos(2.23992178340 + 242.97242145751*$t);
   $X2 += 0.00000000362*cos(3.88633843420 + 9438.0067523705*$t);
   $X2 += 0.00000000248*cos(4.75322854174 + 8827.6340873583*$t);
   $X2 += 0.00000000252*cos(1.16164987849 + 3154.4432674121*$t);
   $X2 += 0.00000000257*cos(1.79452152688 + 11371.9485072417*$t);
   $X2 += 0.00000000327*cos(3.02081233401 + 14143.7390599141*$t);
   $X2 += 0.00000000230*cos(0.23367695814 + 1747.7725955835*$t);
   $X2 += 0.00000000289*cos(5.24697142787 + 7085.1405985987*$t);
   $X2 += 0.00000000297*cos(1.59965866765 + 1349.62359217529*$t);
   $X2 += 0.00000000209*cos(0.27341674720 + 11770.0975106499*$t);
   $X2 += 0.00000000207*cos(4.92862210539 + 4164.0681721295*$t);
   $X2 += 0.00000000188*cos(5.64673454004 + 7875.91568110771*$t);
   $X2 += 0.00000000189*cos(0.60545229996 + 2389.1378379327*$t);
   $X2 += 0.00000000194*cos(0.41046959310 + 10977.32262218251*$t);
   $X2 += 0.00000000176*cos(4.08570919572 + 7342.7015976641*$t);
   $X2 += 0.00000000164*cos(2.79150408226 + 5223.93773728571*$t);
   $X2 += 0.00000000205*cos(3.21370431881 + 10973.7995038335*$t);
   $X2 += 0.00000000205*cos(2.25101785528 + 13368.2164485901*$t);
   $X2 += 0.00000000139*cos(4.61869477858 + 10575.6505004253*$t);
   $X2 += 0.00000000155*cos(5.81412821499 + 12410.9751180321*$t);
   $X2 += 0.00000000152*cos(0.10500467147 + 3930.4535137031*$t);
   $X2 += 0.00000000129*cos(5.52193535090 + 397.9051859247*$t);
   $X2 += 0.00000000127*cos(0.96654930095 + 12592.6938372661*$t);
   $X2 += 0.00000000132*cos(0.16052766322 + 8636.18582124671*$t);
   $X2 += 0.00000000122*cos(4.78358483655 + 13119.9649203087*$t);
   $X2 += 0.00000000163*cos(0.93280092706 + 5507.3094211839*$t);
   $X2 += 0.00000000114*cos(5.70431693679 + 17260.3984721739*$t);
   $X2 += 0.00000000118*cos(2.37854817453 + 4292.5746504339*$t);
   $X2 += 0.00000000148*cos(5.63695738547 + 17790.08943726851*$t);
   $X2 += 0.00000000123*cos(3.75785910050 + 12562.8723991173*$t);
   $X2 += 0.00000000103*cos(3.36994741884 + 13518.1139237169*$t);
   $X2 += 0.00000000138*cos(3.83865460083 + 12168.2465140581*$t);
   $X2 += 0.00000000107*cos(4.33977826153 + 10989.0519750185*$t);
   $X2 += 0.00000000104*cos(5.27997888785 + 2145.92159899169*$t);
   $X2 += 0.00000000096*cos(1.97792917854 + 6283.25235717211*$t);
   $X2 += 0.00000000099*cos(2.27831564394 + 13096.0864825609*$t);
   $X2 += 0.00000000096*cos(1.53382620869 + 6283.3869777777*$t);
   $X2 += 0.00000000084*cos(5.20596620689 + 16730.70750707931*$t);
   $X2 += 0.00000000111*cos(3.96322491245 + 12540.0971976665*$t);
   $X2 += 0.00000000087*cos(6.02515097465 + 10177.5014970171*$t);
   $X2 += 0.00000000078*cos(4.18422621711 + 12323.6669134923*$t);
   $X2 += 0.00000000076*cos(5.70862044549 + 6709.91785835091*$t);
   $X2 += 0.00000000105*cos(3.53845405180 + 5642.4420600927*$t);
   $X2 += 0.00000000073*cos(0.89693480363 + 5856.7214765989*$t);
   $X2 += 0.00000000102*cos(0.66758153957 + 250570.91967470259*$t);
   $X2 += 0.00000000070*cos(5.22505818809 + 5753.14106741329*$t);
   $X2 += 0.00000000071*cos(0.56166089035 + 14314.4119305333*$t);
   $X2 += 0.00000000092*cos(1.96485842296 + 13916.2629271251*$t);
   $X2 += 0.00000000070*cos(2.53450052921 + 7084.6529636317*$t);
   $X2 += 0.00000000091*cos(3.10960565577 + 6924.1972748571*$t);
   $X2 += 0.00000000083*cos(3.98989749432 + 31415.6230674405*$t);
   $X2 += 0.00000000088*cos(5.01539409795 + 71430.45180064559*$t);
   $X2 += 0.00000000074*cos(1.96538390591 + 529.44714761109*$t);
   $X2 += 0.00000000085*cos(5.97852929798 + 95143.37673846159*$t);
   $X2 += 0.00000000061*cos(4.23114381660 + 8274.0646845159*$t);
   $X2 += 0.00000000060*cos(5.52155170015 + 16496.6052136859*$t);
   $X2 += 0.00000000058*cos(1.47891111226 + 5642.93474619389*$t);
   $X2 += 0.00000000078*cos(2.41141378465 + 213.5429129215*$t);
   $X2 += 0.00000000059*cos(4.67526538731 + 640.1411037975*$t);
   $X2 += 0.00000000057*cos(2.06129454632 + 4690.23601887509*$t);
   $X2 += 0.00000000056*cos(3.80170948025 + 13341.9181287903*$t);
   $X2 += 0.00000000065*cos(3.38872777921 + 4731.7868098599*$t);
   $X2 += 0.00000000051*cos(4.81168622601 + 3893.93801205869*$t);
   $X2 += 0.00000000050*cos(4.17369676080 + 12573.5090644671*$t);
   $X2 += 0.00000000050*cos(4.41669813786 + 3634.37720703489*$t);
   $X2 += 0.00000000049*cos(2.27712508812 + 18319.7804023631*$t);
   $X2 += 0.00000000066*cos(5.49585584500 + 2787.2868413409*$t);
   $X2 += 0.00000000049*cos(1.99905022377 + 13362.6935242827*$t);
   $X2 += 0.00000000048*cos(3.76233131596 + 12964.5445208745*$t);
   $X2 += 0.00000000053*cos(4.12374270394 + 14919.2616712381*$t);
   $X2 += 0.00000000058*cos(1.23083144160 + 1577.5873599313*$t);
   $X2 += 0.00000000052*cos(1.27828874996 + 15721.0826023619*$t);
   $X2 += 0.00000000061*cos(3.42876383277 + 4292.0870154669*$t);
   $X2 += 0.00000000044*cos(0.31046230867 + 24073.1652872599*$t);
   $X2 += 0.00000000044*cos(0.38954461131 + 10447.14402212089*$t);
   $X2 += 0.00000000042*cos(4.09986255075 + 20426.8149099055*$t);
   $X2 += 0.00000000059*cos(1.01599200374 + 84672.23202702119*$t);
   $X2 += 0.00000000051*cos(5.38279738858 + 6370.62787201471*$t);
   $X2 += 0.00000000051*cos(1.26573618407 + 6196.0114629351*$t);
   $X2 += 0.00000000046*cos(0.52171400874 + 2119.00767786191*$t);
   $X2 += 0.00000000041*cos(4.09327874029 + 3185.43584474911*$t);
   $X2 += 0.00000000056*cos(3.27189314076 + 10976.83498721549*$t);
   $X2 += 0.00000000051*cos(2.37524851068 + 9437.5191174035*$t);
   $X2 += 0.00000000049*cos(3.21159567950 + 239424.63407183639*$t);
   $X2 += 0.00000000036*cos(6.20393463674 + 3495.78900865049*$t);
   $X2 += 0.00000000044*cos(3.69396549327 + 18139.5383188994*$t);
   $X2 += 0.00000000035*cos(3.61386877088 + 3929.96587873609*$t);
   $X2 += 0.00000000044*cos(1.66014811397 + 8661.9965060795*$t);
   $X2 += 0.00000000046*cos(3.78615165617 + 3894.4256470257*$t);
   $X2 += 0.00000000034*cos(3.81444510796 + 6262.5442719825*$t);
   $X2 += 0.00000000034*cos(5.98049768477 + 6304.0950629673*$t);
   $X2 += 0.00000000035*cos(3.58408872678 + 18875.2820522905*$t);
   $X2 += 0.00000000035*cos(6.09075949155 + 10420.23010099111*$t);
   $X2 += 0.00000000032*cos(0.62535692558 + 23013.7833570707*$t);
   $X2 += 0.00000000034*cos(3.61147349841 + 7.3573644843*$t);
   $X2 += 0.00000000045*cos(1.48783646485 + 3340.8562441833*$t);
   $X2 += 0.00000000035*cos(0.38590076970 + 10213.0417287275*$t);
   $X2 += 0.00000000031*cos(5.38472872675 + 14712.5609339415*$t);
   $X2 += 0.00000000030*cos(4.39715700541 + 12809.12412144031*$t);
   $X2 += 0.00000000039*cos(1.14782571257 + 9779.3524936089*$t);
   $X2 += 0.00000000041*cos(5.61324415849 + 82576.73740351178*$t);
   $X2 += 0.00000000031*cos(0.76672410297 + 22779.6810636773*$t);
   $X2 += 0.00000000028*cos(4.07975589699 + 2942.7072407751*$t);
   $X2 += 0.00000000028*cos(2.36060849192 + 5429.63565075589*$t);
   $X2 += 0.00000000027*cos(4.04275590321 + 11014.8626598513*$t);
   $X2 += 0.00000000027*cos(2.11023646997 + 9623.9320941747*$t);
   $X2 += 0.00000000025*cos(1.72579553961 + 18209.5740811437*$t);
   $X2 += 0.00000000026*cos(2.55064187749 + 9381.2034902007*$t);
   $X2 += 0.00000000025*cos(2.68812943588 + 3583.5848481573*$t);
   $X2 += 0.00000000030*cos(5.98905764002 + 10447.6316570879*$t);
   $X2 += 0.00000000032*cos(4.08880306095 + 796.0541893329*$t);
   $X2 += 0.00000000023*cos(6.09814489767 + 9917.45305702629*$t);
   $X2 += 0.00000000026*cos(0.90493478768 + 12012.8261146239*$t);
   $X2 += 0.00000000024*cos(1.88327707702 + 14143.2514249471*$t);
   $X2 += 0.00000000023*cos(2.26727226510 + 26709.8907598969*$t);
   $X2 += 0.00000000021*cos(3.86895197800 + 7632.6994421667*$t);
   $X2 += 0.00000000021*cos(2.37138453193 + 17256.8753538249*$t);
   $X2 += 0.00000000028*cos(1.07450535792 + 12353.0964220283*$t);
   $X2 += 0.00000000022*cos(4.83463880427 + 77714.01528560399*$t);
   $X2 += 0.00000000021*cos(5.98744320172 + 6993.2527160332*$t);
   $X2 += 0.00000000020*cos(0.31265663265 + 6836.40143535029*$t);
   $X2 += 0.00000000020*cos(5.44332565028 + 1748.2602305505*$t);
   $X2 += 0.00000000026*cos(2.02675372626 + 9225.7830907665*$t);
   $X2 += 0.00000000023*cos(0.80832702606 + 149854.64395229139*$t);
   $X2 += 0.00000000026*cos(5.31358634432 + 2544.07060239989*$t);
   $X2 += 0.00000000018*cos(2.28255799994 + 11614.6771112157*$t);
   $X2 += 0.00000000020*cos(4.61413285151 + 426.8420083595*$t);
   $X2 += 0.00000000019*cos(3.66100662326 + 30640.1004561165*$t);
   $X2 += 0.00000000018*cos(4.77794649120 + 155.6642169177*$t);
   $X2 += 0.00000000018*cos(1.89582517195 + 11926.4982311523*$t);
   $X2 += 0.00000000022*cos(3.74943338657 + 17298.4261448097*$t);
   $X2 += 0.00000000019*cos(3.25511318252 + 18073.46112116669*$t);
   $X2 += 0.00000000020*cos(2.60030261119 + 19651.2922985815*$t);
   $X2 += 0.00000000024*cos(1.57388276311 + 5856.23384163189*$t);
   $X2 += 0.00000000023*cos(4.78368329384 + 72850.80553272919*$t);
   $X2 += 0.00000000017*cos(3.95703430839 + 8983.0544867925*$t);
   $X2 += 0.00000000016*cos(5.17069195690 + 16460.5773470085*$t);
   $X2 += 0.00000000016*cos(1.20099644560 + 17259.91083720689*$t);
   $X2 += 0.00000000021*cos(3.77366558760 + 16858.7263504167*$t);
   $X2 += 0.00000000020*cos(0.39427566601 + 23543.4743221653*$t);
   $X2 += 0.00000000021*cos(3.79445696120 + 13367.7288136231*$t);
   $X2 += 0.00000000020*cos(4.04662298246 + 2146.4092339587*$t);
   $X2 += 0.00000000014*cos(0.14269366851 + 22484.0923919761*$t);
   $X2 += 0.00000000014*cos(4.66600220934 + 16200.52890701769*$t);
   $X2 += 0.00000000015*cos(4.21847977142 + 20199.3387771165*$t);
   $X2 += 0.00000000019*cos(2.81420484440 + 8672.21368792411*$t);
   $X2 += 0.00000000019*cos(4.46915157584 + 18073.9487561337*$t);
   $X2 += 0.00000000013*cos(4.71429939378 + 16201.0165419847*$t);
   $X2 += 0.00000000017*cos(1.38925168648 + 7860.17557495569*$t);
   $X2 += 0.00000000013*cos(4.69427401677 + 18451.3223640495*$t);
   $X2 += 0.00000000016*cos(3.33196861521 + 5572.8989839496*$t);
   $X2 += 0.00000000012*cos(2.70989366066 + 17996.2749857057*$t);
   $X2 += 0.00000000012*cos(0.72221452149 + 19403.0407703001*$t);
   $X2 += 0.00000000012*cos(5.53894834507 + 14945.0723560709*$t);
   $X2 += 0.00000000015*cos(2.04879024946 + 12559.2819704655*$t);
   $X2 += 0.00000000012*cos(4.37879205675 + 10818.3791043993*$t);
   $X2 += 0.00000000015*cos(0.57625721169 + 12567.37583853451*$t);
   $X2 += 0.00000000011*cos(1.78471307843 + 28767.1682419675*$t);
   $X2 += 0.00000000013*cos(1.09547571668 + 90280.16698558679*$t);
   $X2 += 0.00000000011*cos(5.02230424673 + 22805.4917485101*$t);
   $X2 += 0.00000000011*cos(2.24605985689 + 35372.1310834599*$t);
   $X2 += 0.00000000010*cos(6.26009139838 + 11506.52595231009*$t);
   $X2 += 0.00000000014*cos(1.93802583909 + 21228.63584102931*$t);
   $X2 += 0.00000000010*cos(0.13064402023 + 12779.6946129043*$t);
   $X2 += 0.00000000010*cos(6.05309072952 + 23141.8022004081*$t);
   $X2 += 0.00000000010*cos(1.18002886182 + 1350.1112271423*$t);
   $X2 += 0.00000000010*cos(2.50762435685 + 10575.1628654583*$t);
   $X2 += 0.00000000010*cos(2.55643759758 + 22345.5041935917*$t);
   $X2 += 0.00000000010*cos(5.33273079625 + 9778.8648586419*$t);
   $X2 += 0.00000000009*cos(1.49151911226 + 29826.5501721567*$t);
   $X2 += 0.00000000010*cos(4.24000912367 + 23581.5019948011*$t);
   $X2 += 0.00000000011*cos(0.39610887139 + 25158.8455372489*$t);
   return $X2*$t*$t;
}



   function Earth_X3($t) // 46 terms of order 3
{
   $X3  = 0.00000232790*cos(3.40634928966 + 0.2438174835*$t);
   $X3 += 0.00000076843*cos(2.52439403387 + 12566.3955174663*$t);
   $X3 += 0.00000035331*cos(3.34616699853 + 6283.3196674749*$t);
   $X3 += 0.00000005282*cos(2.40489818589 + 18849.4713674577*$t);
   $X3 += 0.00000001631*cos(5.43859760597 + 6282.83203250789*$t);
   $X3 += 0.00000001483*cos(2.22678849266 + 6438.7400669091*$t);
   $X3 += 0.00000001479*cos(4.40151760084 + 6127.8992680407*$t);
   $X3 += 0.00000000652*cos(4.70876083993 + 6279.7965491259*$t);
   $X3 += 0.00000000656*cos(1.58695055624 + 6286.84278582391*$t);
   $X3 += 0.00000000317*cos(3.00237328519 + 6526.04827144891*$t);
   $X3 += 0.00000000318*cos(3.63024148359 + 6040.5910635009*$t);
   $X3 += 0.00000000227*cos(2.27961613476 + 25132.5472174491*$t);
   $X3 += 0.00000000201*cos(0.81376672334 + 6836.8890703173*$t);
   $X3 += 0.00000000201*cos(5.80642900510 + 5729.7502646325*$t);
   $X3 += 0.00000000036*cos(1.33765000112 + 12569.91863581531*$t);
   $X3 += 0.00000000034*cos(1.93992455025 + 4705.9761250271*$t);
   $X3 += 0.00000000033*cos(4.22745351005 + 12410.9751180321*$t);
   $X3 += 0.00000000032*cos(5.38348719867 + 6257.0213476751*$t);
   $X3 += 0.00000000032*cos(4.41212819179 + 6309.61798727471*$t);
   $X3 += 0.00000000026*cos(0.50334838608 + 775.7664288075*$t);
   $X3 += 0.00000000023*cos(2.01776542272 + 1059.6257476727*$t);
   $X3 += 0.00000000025*cos(1.58405990040 + 7860.6632099227*$t);
   $X3 += 0.00000000027*cos(5.17969059866 + 12565.9078824993*$t);
   $X3 += 0.00000000020*cos(3.76292921295 + 5753.6287023803*$t);
   $X3 += 0.00000000019*cos(3.01515389134 + 5885.1706640667*$t);
   $X3 += 0.00000000019*cos(6.05056350877 + 6813.0106325695*$t);
   $X3 += 0.00000000018*cos(2.29290644779 + 12721.8159169005*$t);
   $X3 += 0.00000000017*cos(0.49235818139 + 6681.46867088311*$t);
   $X3 += 0.00000000013*cos(4.49557806784 + 5487.0216606585*$t);
   $X3 += 0.00000000011*cos(5.29141894629 + 7079.61767429131*$t);
   $X3 += 0.00000000010*cos(4.46246654504 + 5507.7970561509*$t);
   $X3 += 0.00000000010*cos(3.03842434664 + 11790.8729061423*$t);
   $X3 += 0.00000000009*cos(1.50545167256 + 11507.0135872771*$t);
   $X3 += 0.00000000011*cos(6.25880735981 + 12592.6938372661*$t);
   $X3 += 0.00000000009*cos(5.39258795934 + 7058.8422787989*$t);
   $X3 += 0.00000000008*cos(1.26531983919 + 6290.4332144757*$t);
   $X3 += 0.00000000008*cos(2.25393064493 + 6276.2061204741*$t);
   $X3 += 0.00000000008*cos(3.03070423945 + 796.5418242999*$t);
   $X3 += 0.00000000007*cos(2.58386169059 + 4693.75913722409*$t);
   $X3 += 0.00000000007*cos(1.92157668691 + 7.3573644843*$t);
   $X3 += 0.00000000005*cos(4.86563371701 + 3739.0052475915*$t);
   $X3 += 0.00000000005*cos(6.21240311845 + 6070.0205720369*$t);
   $X3 += 0.00000000006*cos(3.95562930301 + 6284.2999885431*$t);
   $X3 += 0.00000000006*cos(5.83931193332 + 6282.3393464067*$t);
   $X3 += 0.00000000005*cos(3.46747665337 + 4137.1542509997*$t);
   $X3 += 0.00000000005*cos(3.58293336989 + 6496.6187629129*$t);
   return $X3*$t*$t*$t;
}



   function Earth_X4($t) // 20 terms of order 4
{
   $X4  = 0.00000114918*cos(0.06053023506 + 6283.3196674749*$t);
   $X4 += 0.00000006817*cos(4.47624663983 + 0.2438174835*$t);
   $X4 += 0.00000003158*cos(0.47910545815 + 12566.3955174663*$t);
   $X4 += 0.00000000211*cos(3.72351646714 + 6438.7400669091*$t);
   $X4 += 0.00000000210*cos(2.88489921647 + 6127.8992680407*$t);
   $X4 += 0.00000000253*cos(0.64665875683 + 18849.4713674577*$t);
   $X4 += 0.00000000073*cos(3.92376796583 + 6282.83203250789*$t);
   $X4 += 0.00000000042*cos(4.60322795936 + 710.1768660418*$t);
   $X4 += 0.00000000023*cos(6.04513529243 + 6279.7965491259*$t);
   $X4 += 0.00000000022*cos(6.04140763882 + 6286.84278582391*$t);
   $X4 += 0.00000000012*cos(0.51640078868 + 25132.5472174491*$t);
   $X4 += 0.00000000009*cos(4.21008055560 + 83997.09113559539*$t);
   $X4 += 0.00000000009*cos(5.19392488383 + 11856.462468908*$t);
   $X4 += 0.00000000008*cos(6.16587249238 + 167284.00540514899*$t);
   $X4 += 0.00000000005*cos(5.97913996910 + 6257.0213476751*$t);
   $X4 += 0.00000000005*cos(3.78878463461 + 6309.61798727471*$t);
   $X4 += 0.00000000005*cos(2.68915538708 + 12410.9751180321*$t);
   $X4 += 0.00000000005*cos(5.56850457487 + 156137.71980228279*$t);
   $X4 += 0.00000000004*cos(3.40118346072 + 12565.9078824993*$t);
   $X4 += 0.00000000003*cos(5.17221494066 + 529.9347825781*$t);
   return $X4*$t*$t*$t*$t;
}



   function Earth_X5($t) // 7 terms of order 5
{
   $X5  = 0.00000000877*cos(0.16136296068 + 6283.3196674749*$t);
   $X5 += 0.00000000305*cos(5.62973274927 + 0.2438174835*$t);
   $X5 += 0.00000000101*cos(4.88076699149 + 12566.3955174663*$t);
   $X5 += 0.00000000025*cos(5.30593958386 + 6438.7400669091*$t);
   $X5 += 0.00000000025*cos(1.27879113841 + 6127.8992680407*$t);
   $X5 += 0.00000000010*cos(5.11399286577 + 18849.4713674577*$t);
   $X5 += 0.00000000003*cos(2.56205735968 + 6282.83203250789*$t);
   return $X5*$t*$t*$t*$t*$t;
}



   function Earth_Y0($t) // 1007 terms of order 0
{
   $Y0  = 0.99986069925*cos(0.18267413078 + 6283.3196674749*$t);
   $Y0 += 0.02506324281*cos(3.36739796418 + 0.2438174835*$t);
   $Y0 += 0.00835274807*cos(0.13953892859 + 12566.3955174663*$t);
   $Y0 += 0.00010466796*cos(0.09642351540 + 18849.4713674577*$t);
   $Y0 += 0.00003490608*cos(6.01453435910 + 6282.83203250789*$t);
   $Y0 += 0.00003110838*cos(5.38114087369 + 83997.09113559539*$t);
   $Y0 += 0.00002561408*cos(5.29827505528 + 529.9347825781*$t);
   $Y0 += 0.00002142365*cos(2.66284107563 + 1577.0997249643*$t);
   $Y0 += 0.00001709161*cos(5.20779761275 + 6279.7965491259*$t);
   $Y0 += 0.00001707934*cos(4.58234386738 + 6286.84278582391*$t);
   $Y0 += 0.00001442753*cos(1.90130765657 + 2353.1099712553*$t);
   $Y0 += 0.00001113027*cos(5.26701283158 + 5223.4501023187*$t);
   $Y0 += 0.00000934484*cos(4.50305745607 + 12036.7045523717*$t);
   $Y0 += 0.00000899854*cos(1.60527831001 + 10213.5293636945*$t);
   $Y0 += 0.00000566820*cos(0.58162314212 + 1059.6257476727*$t);
   $Y0 += 0.00000661858*cos(6.02414120157 + 5753.6287023803*$t);
   $Y0 += 0.00000739644*cos(2.79582891432 + 398.3928208917*$t);
   $Y0 += 0.00000681381*cos(0.64735765260 + 4705.9761250271*$t);
   $Y0 += 0.00000611293*cos(3.81390547656 + 6813.0106325695*$t);
   $Y0 += 0.00000451129*cos(4.52236258525 + 5885.1706640667*$t);
   $Y0 += 0.00000451985*cos(5.99169934357 + 6257.0213476751*$t);
   $Y0 += 0.00000449743*cos(3.79862296384 + 6309.61798727471*$t);
   $Y0 += 0.00000406291*cos(5.25608267423 + 6681.46867088311*$t);
   $Y0 += 0.00000541115*cos(5.49909532338 + 775.7664288075*$t);
   $Y0 += 0.00000546682*cos(6.17348361999 + 14143.7390599141*$t);
   $Y0 += 0.00000513780*cos(2.86615405111 + 7860.6632099227*$t);
   $Y0 += 0.00000263916*cos(3.82472119950 + 5507.7970561509*$t);
   $Y0 += 0.00000217220*cos(2.94186064336 + 11507.0135872771*$t);
   $Y0 += 0.00000292146*cos(4.08451989980 + 5507.3094211839*$t);
   $Y0 += 0.00000227915*cos(5.95179814771 + 7058.8422787989*$t);
   $Y0 += 0.00000226835*cos(4.84435977688 + 4693.75913722409*$t);
   $Y0 += 0.00000255833*cos(0.69465437518 + 12168.2465140581*$t);
   $Y0 += 0.00000256157*cos(6.16717762843 + 710.1768660418*$t);
   $Y0 += 0.00000178679*cos(1.39565997429 + 796.5418242999*$t);
   $Y0 += 0.00000178303*cos(5.11711459804 + 6283.25235717211*$t);
   $Y0 += 0.00000178303*cos(4.67301162820 + 6283.3869777777*$t);
   $Y0 += 0.00000155471*cos(0.05335100564 + 25132.5472174491*$t);
   $Y0 += 0.00000155246*cos(6.13208007594 + 5487.0216606585*$t);
   $Y0 += 0.00000207640*cos(4.28246516283 + 11790.8729061423*$t);
   $Y0 += 0.00000199988*cos(2.50137197430 + 17790.08943726851*$t);
   $Y0 += 0.00000128970*cos(3.64618696457 + 7079.61767429131*$t);
   $Y0 += 0.00000128155*cos(3.23179068571 + 3739.0052475915*$t);
   $Y0 += 0.00000152241*cos(5.58140597212 + 213.5429129215*$t);
   $Y0 += 0.00000078109*cos(0.27810887092 + 8827.6340873583*$t);
   $Y0 += 0.00000073503*cos(1.12438065243 + 1589.3167127673*$t);
   $Y0 += 0.00000093001*cos(5.50857519732 + 9437.5191174035*$t);
   $Y0 += 0.00000065526*cos(2.08488700481 + 11770.0975106499*$t);
   $Y0 += 0.00000063240*cos(0.67044494917 + 6262.5442719825*$t);
   $Y0 += 0.00000062961*cos(2.83649752628 + 6304.0950629673*$t);
   $Y0 += 0.00000085870*cos(1.45351146305 + 167284.00540514899*$t);
   $Y0 += 0.00000083233*cos(4.62701145755 + 3340.8562441833*$t);
   $Y0 += 0.00000062850*cos(0.46743502764 + 7.3573644843*$t);
   $Y0 += 0.00000079971*cos(0.92256149872 + 6070.0205720369*$t);
   $Y0 += 0.00000060043*cos(1.82875530765 + 4137.1542509997*$t);
   $Y0 += 0.00000075613*cos(2.58868557323 + 6496.6187629129*$t);
   $Y0 += 0.00000053731*cos(6.27507790078 + 1194.6908277081*$t);
   $Y0 += 0.00000057574*cos(2.39891957636 + 6284.2999885431*$t);
   $Y0 += 0.00000057574*cos(1.10802134270 + 6282.3393464067*$t);
   $Y0 += 0.00000072385*cos(0.13875968847 + 10976.83498721549*$t);
   $Y0 += 0.00000051135*cos(6.02476231374 + 6290.4332144757*$t);
   $Y0 += 0.00000050604*cos(3.77440441934 + 6276.2061204741*$t);
   $Y0 += 0.00000049613*cos(1.07290414466 + 6127.8992680407*$t);
   $Y0 += 0.00000048009*cos(2.43492252840 + 6438.7400669091*$t);
   $Y0 += 0.00000059233*cos(5.35402385929 + 3154.4432674121*$t);
   $Y0 += 0.00000048264*cos(0.05887245943 + 801.5771136403*$t);
   $Y0 += 0.00000047368*cos(4.53616745806 + 3128.6325825793*$t);
   $Y0 += 0.00000039225*cos(1.68202291816 + 8429.4850839501*$t);
   $Y0 += 0.00000051307*cos(4.21369114598 + 12353.0964220283*$t);
   $Y0 += 0.00000050639*cos(0.48135592944 + 11856.462468908*$t);
   $Y0 += 0.00000039555*cos(3.35943021943 + 5481.4987363511*$t);
   $Y0 += 0.00000042757*cos(2.06983819322 + 17260.3984721739*$t);
   $Y0 += 0.00000048941*cos(5.16594049812 + 9225.7830907665*$t);
   $Y0 += 0.00000048302*cos(2.17445966751 + 2544.07060239989*$t);
   $Y0 += 0.00000037971*cos(1.47013374285 + 426.8420083595*$t);
   $Y0 += 0.00000043643*cos(4.71767697827 + 5856.23384163189*$t);
   $Y0 += 0.00000031830*cos(3.57964760085 + 2145.92159899169*$t);
   $Y0 += 0.00000033959*cos(0.15150934422 + 7085.1405985987*$t);
   $Y0 += 0.00000030184*cos(2.39802729198 + 10977.32262218251*$t);
   $Y0 += 0.00000032562*cos(4.41726666203 + 4164.0681721295*$t);
   $Y0 += 0.00000038887*cos(0.64783670316 + 13367.7288136231*$t);
   $Y0 += 0.00000028515*cos(4.53853388778 + 12569.91863581531*$t);
   $Y0 += 0.00000028873*cos(5.72467865653 + 3930.4535137031*$t);
   $Y0 += 0.00000036633*cos(1.33719170745 + 18073.9487561337*$t);
   $Y0 += 0.00000024984*cos(3.74998709416 + 26.0545023163*$t);
   $Y0 += 0.00000026364*cos(5.25860499861 + 9438.0067523705*$t);
   $Y0 += 0.00000030326*cos(0.42292659979 + 4535.3032544079*$t);
   $Y0 += 0.00000028166*cos(5.19366223100 + 12562.8723991173*$t);
   $Y0 += 0.00000028001*cos(1.25523598395 + 5088.8726572503*$t);
   $Y0 += 0.00000021157*cos(3.50176956898 + 11371.9485072417*$t);
   $Y0 += 0.00000023113*cos(5.57235828208 + 5223.93773728571*$t);
   $Y0 += 0.00000021642*cos(3.06444525139 + 12559.2819704655*$t);
   $Y0 += 0.00000019448*cos(5.97216607301 + 12565.9078824993*$t);
   $Y0 += 0.00000026055*cos(1.87419996146 + 71430.45180064559*$t);
   $Y0 += 0.00000018232*cos(4.98441727809 + 1747.7725955835*$t);
   $Y0 += 0.00000023248*cos(4.61930360205 + 18319.7804023631*$t);
   $Y0 += 0.00000022785*cos(2.73901481639 + 5753.14106741329*$t);
   $Y0 += 0.00000017091*cos(5.29823070971 + 4933.4522578161*$t);
   $Y0 += 0.00000019399*cos(6.00268090156 + 10447.14402212089*$t);
   $Y0 += 0.00000021067*cos(2.22953967314 + 7477.7666776995*$t);
   $Y0 += 0.00000021310*cos(3.08840298987 + 8031.3360805419*$t);
   $Y0 += 0.00000014189*cos(6.02227444325 + 2942.7072407751*$t);
   $Y0 += 0.00000013864*cos(3.02270988640 + 8636.18582124671*$t);
   $Y0 += 0.00000018884*cos(0.85614785764 + 156137.71980228279*$t);
   $Y0 += 0.00000017875*cos(4.86899529635 + 1592.8398311163*$t);
   $Y0 += 0.00000018602*cos(2.07103958699 + 17297.9385098427*$t);
   $Y0 += 0.00000013605*cos(3.62641283985 + 13096.0864825609*$t);
   $Y0 += 0.00000015064*cos(1.55068405670 + 16496.6052136859*$t);
   $Y0 += 0.00000012969*cos(4.49412072551 + 7633.1870771337*$t);
   $Y0 += 0.00000014403*cos(6.14964820819 + 20426.8149099055*$t);
   $Y0 += 0.00000013311*cos(2.03652540541 + 12139.7973265903*$t);
   $Y0 += 0.00000013645*cos(3.57902352159 + 13368.2164485901*$t);
   $Y0 += 0.00000011583*cos(3.88983863974 + 5331.6012612243*$t);
   $Y0 += 0.00000011405*cos(5.97095658034 + 529.44714761109*$t);
   $Y0 += 0.00000015057*cos(0.50195173390 + 7084.6529636317*$t);
   $Y0 += 0.00000014330*cos(4.45695507640 + 7342.7015976641*$t);
   $Y0 += 0.00000012375*cos(1.53862290035 + 6279.72923882311*$t);
   $Y0 += 0.00000012366*cos(1.96831814313 + 6286.9100961267*$t);
   $Y0 += 0.00000010016*cos(3.81622500644 + 397.9051859247*$t);
   $Y0 += 0.00000012435*cos(5.77865382585 + 15110.7099373497*$t);
   $Y0 += 0.00000009513*cos(5.90294963286 + 7235.0380737255*$t);
   $Y0 += 0.00000010997*cos(0.62118771904 + 10989.0519750185*$t);
   $Y0 += 0.00000011583*cos(2.48171191864 + 5729.7502646325*$t);
   $Y0 += 0.00000009034*cos(3.85796197835 + 9623.9320941747*$t);
   $Y0 += 0.00000011842*cos(2.59877813496 + 15721.0826023619*$t);
   $Y0 += 0.00000008869*cos(3.47611287212 + 6148.2545874395*$t);
   $Y0 += 0.00000008614*cos(0.03081566771 + 6418.38474751031*$t);
   $Y0 += 0.00000010312*cos(1.02710009735 + 6836.8890703173*$t);
   $Y0 += 0.00000008411*cos(0.87289239606 + 14712.5609339415*$t);
   $Y0 += 0.00000007911*cos(1.58349090213 + 2119.00767786191*$t);
   $Y0 += 0.00000010379*cos(0.10632075985 + 1349.62359217529*$t);
   $Y0 += 0.00000010374*cos(3.97648848184 + 5486.53402569149*$t);
   $Y0 += 0.00000010480*cos(1.38209289015 + 5999.4603486097*$t);
   $Y0 += 0.00000007404*cos(3.84339417190 + 6040.5910635009*$t);
   $Y0 += 0.00000007357*cos(5.40136984638 + 5088.3850222833*$t);
   $Y0 += 0.00000009820*cos(2.12476608355 + 6567.1789863401*$t);
   $Y0 += 0.00000007018*cos(5.94659139192 + 6526.04827144891*$t);
   $Y0 += 0.00000009668*cos(3.49318502073 + 21228.1482060623*$t);
   $Y0 += 0.00000007545*cos(5.94843445846 + 12540.0971976665*$t);
   $Y0 += 0.00000007202*cos(4.31311905021 + 6245.2919948391*$t);
   $Y0 += 0.00000007144*cos(5.47703554612 + 6321.3473401107*$t);
   $Y0 += 0.00000008883*cos(5.46710407546 + 5642.93474619389*$t);
   $Y0 += 0.00000006626*cos(5.41362465894 + 5327.7199258663*$t);
   $Y0 += 0.00000007015*cos(5.70701993379 + 7860.17557495569*$t);
   $Y0 += 0.00000006159*cos(5.65831198505 + 12964.5445208745*$t);
   $Y0 += 0.00000007413*cos(3.46419092567 + 23543.4743221653*$t);
   $Y0 += 0.00000006355*cos(3.46461466453 + 1990.9888345245*$t);
   $Y0 += 0.00000007286*cos(2.01139981548 + 537.0483295789*$t);
   $Y0 += 0.00000006314*cos(1.51246730786 + 951.4745887671*$t);
   $Y0 += 0.00000006257*cos(2.66128485006 + 4690.7236538421*$t);
   $Y0 += 0.00000006484*cos(1.63733658639 + 16730.70750707931*$t);
   $Y0 += 0.00000005803*cos(1.38237252918 + 955.3559241251*$t);
   $Y0 += 0.00000006391*cos(2.40521894071 + 24073.1652872599*$t);
   $Y0 += 0.00000007530*cos(4.91537693059 + 10973.7995038335*$t);
   $Y0 += 0.00000006921*cos(5.41208599854 + 522.8212355773*$t);
   $Y0 += 0.00000007626*cos(2.72257180334 + 22004.1584523533*$t);
   $Y0 += 0.00000007531*cos(1.94384811400 + 18422.8731765817*$t);
   $Y0 += 0.00000006902*cos(4.32614260011 + 155.17658195069*$t);
   $Y0 += 0.00000005434*cos(4.37546348368 + 7238.9194090835*$t);
   $Y0 += 0.00000006223*cos(0.67316616422 + 18451.3223640495*$t);
   $Y0 += 0.00000007296*cos(1.27743142232 + 16730.21987211229*$t);
   $Y0 += 0.00000005833*cos(0.75851095819 + 640.1411037975*$t);
   $Y0 += 0.00000007171*cos(1.43520696239 + 3929.96587873609*$t);
   $Y0 += 0.00000006251*cos(0.62294633745 + 6277.7967431675*$t);
   $Y0 += 0.00000006251*cos(2.88399458161 + 6288.8425917823*$t);
   $Y0 += 0.00000005683*cos(1.92473239272 + 1551.2890401315*$t);
   $Y0 += 0.00000006383*cos(2.21747129905 + 5216.33655531789*$t);
   $Y0 += 0.00000006194*cos(5.13602240936 + 5230.5636493195*$t);
   $Y0 += 0.00000005036*cos(2.26614918626 + 14314.4119305333*$t);
   $Y0 += 0.00000006774*cos(3.56437941703 + 102.84895673509*$t);
   $Y0 += 0.00000006224*cos(1.92311046214 + 11014.8626598513*$t);
   $Y0 += 0.00000004621*cos(2.91747769543 + 553.32558535889*$t);
   $Y0 += 0.00000006293*cos(0.74955598933 + 5650.5359281617*$t);
   $Y0 += 0.00000006297*cos(2.83090961174 + 26088.1469590577*$t);
   $Y0 += 0.00000005186*cos(5.42426212805 + 77714.01528560399*$t);
   $Y0 += 0.00000006196*cos(4.15785490016 + 84672.23202702119*$t);
   $Y0 += 0.00000006160*cos(3.21170376619 + 239424.63407183639*$t);
   $Y0 += 0.00000005414*cos(1.42882325200 + 6180.2268932563*$t);
   $Y0 += 0.00000005413*cos(2.08335586272 + 6386.4124416935*$t);
   $Y0 += 0.00000004863*cos(2.19642727179 + 90280.16698558679*$t);
   $Y0 += 0.00000005711*cos(2.75632617612 + 6916.1034067881*$t);
   $Y0 += 0.00000005150*cos(2.53503851142 + 1577.5873599313*$t);
   $Y0 += 0.00000004269*cos(0.81279132183 + 7875.91568110771*$t);
   $Y0 += 0.00000004017*cos(2.21501270421 + 6254.8704800071*$t);
   $Y0 += 0.00000004069*cos(3.99136293444 + 3634.37720703489*$t);
   $Y0 += 0.00000003993*cos(1.29192496097 + 6311.7688549427*$t);
   $Y0 += 0.00000004016*cos(0.52802181166 + 4690.23601887509*$t);
   $Y0 += 0.00000005290*cos(4.91562568689 + 25158.3579022819*$t);
   $Y0 += 0.00000004831*cos(1.26836887601 + 5760.7422493811*$t);
   $Y0 += 0.00000003487*cos(1.93988497026 + 10447.6316570879*$t);
   $Y0 += 0.00000004120*cos(1.72576404553 + 6709.91785835091*$t);
   $Y0 += 0.00000004410*cos(2.23510306717 + 6805.89708556871*$t);
   $Y0 += 0.00000004023*cos(1.51236080833 + 4731.7868098599*$t);
   $Y0 += 0.00000003980*cos(1.74852343480 + 5856.7214765989*$t);
   $Y0 += 0.00000003173*cos(3.62048518613 + 1066.7392946735*$t);
   $Y0 += 0.00000003293*cos(5.58717113537 + 9917.45305702629*$t);
   $Y0 += 0.00000002935*cos(2.10511347366 + 12592.6938372661*$t);
   $Y0 += 0.00000002979*cos(4.62987940723 + 12566.4628277691*$t);
   $Y0 += 0.00000003980*cos(5.90920652207 + 13341.9181287903*$t);
   $Y0 += 0.00000003892*cos(3.95491793455 + 75.0254160508*$t);
   $Y0 += 0.00000003042*cos(1.62346061120 + 18053.1733606413*$t);
   $Y0 += 0.00000003608*cos(4.03873145467 + 6208.5380689076*$t);
   $Y0 += 0.00000003721*cos(3.65024914006 + 5966.9277978183*$t);
   $Y0 += 0.00000003581*cos(5.75234129258 + 6358.1012660422*$t);
   $Y0 += 0.00000002880*cos(0.04145923446 + 10575.6505004253*$t);
   $Y0 += 0.00000002806*cos(2.64471981541 + 5863.8350235997*$t);
   $Y0 += 0.00000003626*cos(6.13871061291 + 6599.7115371315*$t);
   $Y0 += 0.00000002794*cos(0.98030293667 + 8030.8484455749*$t);
   $Y0 += 0.00000002729*cos(2.60239924444 + 3.76693583251*$t);
   $Y0 += 0.00000003625*cos(1.43112681156 + 6020.2357441021*$t);
   $Y0 += 0.00000003206*cos(3.72476724430 + 11506.52595231009*$t);
   $Y0 += 0.00000003556*cos(5.86027209236 + 8428.9974489831*$t);
   $Y0 += 0.00000002856*cos(1.76545510884 + 12721.8159169005*$t);
   $Y0 += 0.00000002611*cos(0.86182235214 + 6702.8043113501*$t);
   $Y0 += 0.00000002535*cos(0.01231069705 + 31415.6230674405*$t);
   $Y0 += 0.00000003520*cos(3.80906738831 + 250570.91967470259*$t);
   $Y0 += 0.00000003434*cos(2.07577291101 + 6546.4035908477*$t);
   $Y0 += 0.00000002572*cos(0.08592522543 + 5884.68302909969*$t);
   $Y0 += 0.00000002647*cos(5.39051905541 + 2352.6223362883*$t);
   $Y0 += 0.00000003026*cos(3.66998442300 + 13916.2629271251*$t);
   $Y0 += 0.00000002378*cos(2.05754080989 + 2389.1378379327*$t);
   $Y0 += 0.00000002979*cos(5.07398237707 + 12566.32820716351*$t);
   $Y0 += 0.00000003082*cos(3.34593854945 + 14945.0723560709*$t);
   $Y0 += 0.00000002716*cos(1.49855756532 + 12029.5910053709*$t);
   $Y0 += 0.00000002215*cos(3.69564106529 + 13362.6935242827*$t);
   $Y0 += 0.00000003032*cos(0.05314931388 + 29088.5675985015*$t);
   $Y0 += 0.00000002421*cos(1.58246988642 + 11015.3502948183*$t);
   $Y0 += 0.00000002275*cos(4.38385659579 + 6077.1341190377*$t);
   $Y0 += 0.00000002466*cos(0.87928045349 + 16200.52890701769*$t);
   $Y0 += 0.00000002529*cos(4.37629498907 + 12043.8180993725*$t);
   $Y0 += 0.00000002015*cos(0.91352607152 + 6262.96434807611*$t);
   $Y0 += 0.00000002218*cos(5.40562909521 + 6489.5052159121*$t);
   $Y0 += 0.00000002005*cos(2.59344260095 + 6303.6749868737*$t);
   $Y0 += 0.00000002010*cos(5.68334625182 + 4700.87268422489*$t);
   $Y0 += 0.00000002641*cos(5.85354681494 + 4590.6663630055*$t);
   $Y0 += 0.00000002438*cos(4.55830572881 + 149.8070146181*$t);
   $Y0 += 0.00000001893*cos(5.69296362938 + 6279.4383321169*$t);
   $Y0 += 0.00000001893*cos(4.09716259686 + 6287.2010028329*$t);
   $Y0 += 0.00000002156*cos(0.43822603611 + 18139.5383188994*$t);
   $Y0 += 0.00000002071*cos(2.75099511730 + 11926.4982311523*$t);
   $Y0 += 0.00000002395*cos(4.84387255380 + 1162.7185218913*$t);
   $Y0 += 0.00000001954*cos(5.08514679694 + 13518.1139237169*$t);
   $Y0 += 0.00000002006*cos(5.02287213343 + 3154.9309023791*$t);
   $Y0 += 0.00000001776*cos(4.86094102925 + 13521.9952590749*$t);
   $Y0 += 0.00000001964*cos(1.82182786120 + 4686.6455902233*$t);
   $Y0 += 0.00000002332*cos(2.83569470692 + 10022.0810975829*$t);
   $Y0 += 0.00000001863*cos(3.68950976920 + 242.97242145751*$t);
   $Y0 += 0.00000001928*cos(3.02170818288 + 5746.5151553795*$t);
   $Y0 += 0.00000002436*cos(2.83691190801 + 95143.37673846159*$t);
   $Y0 += 0.00000002163*cos(6.13036805408 + 6037.4880212455*$t);
   $Y0 += 0.00000002250*cos(0.82649289984 + 12669.4882916849*$t);
   $Y0 += 0.00000002052*cos(3.65926976048 + 6529.1513137043*$t);
   $Y0 += 0.00000001886*cos(5.18293217320 + 7238.4317741165*$t);
   $Y0 += 0.00000002170*cos(3.92516942708 + 19651.2922985815*$t);
   $Y0 += 0.00000002058*cos(3.12741900385 + 23013.7833570707*$t);
   $Y0 += 0.00000001774*cos(0.51402656711 + 6820.1241795703*$t);
   $Y0 += 0.00000001841*cos(1.91732926338 + 65147.37595065419*$t);
   $Y0 += 0.00000001578*cos(4.07231752329 + 4292.5746504339*$t);
   $Y0 += 0.00000001577*cos(2.37859191870 + 7632.6994421667*$t);
   $Y0 += 0.00000001541*cos(0.23280184119 + 13119.9649203087*$t);
   $Y0 += 0.00000002012*cos(1.94074945567 + 4292.0870154669*$t);
   $Y0 += 0.00000001543*cos(5.39993967465 + 6293.95633282471*$t);
   $Y0 += 0.00000001543*cos(4.39018655159 + 6272.6830021251*$t);
   $Y0 += 0.00000001513*cos(2.22141523192 + 20.5315780089*$t);
   $Y0 += 0.00000001489*cos(1.38125902411 + 110.45013870291*$t);
   $Y0 += 0.00000001639*cos(2.86996469977 + 17655.0243572331*$t);
   $Y0 += 0.00000001443*cos(0.69249676813 + 1052.51220067191*$t);
   $Y0 += 0.00000001337*cos(1.49321921272 + 2544.5582373669*$t);
   $Y0 += 0.00000001797*cos(1.47415887460 + 33018.7772947211*$t);
   $Y0 += 0.00000001488*cos(4.10370894807 + 76.50988875911*$t);
   $Y0 += 0.00000001453*cos(3.88999021921 + 633.0275567967*$t);
   $Y0 += 0.00000001349*cos(4.18018116919 + 6016.7126257531*$t);
   $Y0 += 0.00000001477*cos(2.56925141669 + 12779.6946129043*$t);
   $Y0 += 0.00000001688*cos(4.77167367345 + 18875.2820522905*$t);
   $Y0 += 0.00000001491*cos(4.65894552032 + 9411.7084325707*$t);
   $Y0 += 0.00000001471*cos(4.18699607739 + 18636.1722720197*$t);
   $Y0 += 0.00000001263*cos(5.62967869388 + 12573.5090644671*$t);
   $Y0 += 0.00000001685*cos(4.13797957160 + 25934.3681485729*$t);
   $Y0 += 0.00000001375*cos(2.66006793362 + 2378.9206560881*$t);
   $Y0 += 0.00000001641*cos(1.56425462469 + 6.86972951729*$t);
   $Y0 += 0.00000001200*cos(6.18795172265 + 11499.9000402763*$t);
   $Y0 += 0.00000001272*cos(5.60977811105 + 6549.9267091967*$t);
   $Y0 += 0.00000001214*cos(2.79740025979 + 10973.3118688665*$t);
   $Y0 += 0.00000001406*cos(0.96357897619 + 11790.3852711753*$t);
   $Y0 += 0.00000001524*cos(2.69987154893 + 18209.5740811437*$t);
   $Y0 += 0.00000001149*cos(1.44882073480 + 10177.5014970171*$t);
   $Y0 += 0.00000001547*cos(5.59995461433 + 11926.01059618529*$t);
   $Y0 += 0.00000001460*cos(3.40155505101 + 246.0754637129*$t);
   $Y0 += 0.00000001323*cos(2.95651083102 + 8661.9965060795*$t);
   $Y0 += 0.00000001252*cos(6.12405778157 + 6993.2527160332*$t);
   $Y0 += 0.00000001459*cos(3.72728247655 + 38.3768531213*$t);
   $Y0 += 0.00000001235*cos(1.27016887136 + 24357.0246061251*$t);
   $Y0 += 0.00000001028*cos(0.93950364612 + 6112.6467968557*$t);
   $Y0 += 0.00000001330*cos(5.71304625366 + 2146.4092339587*$t);
   $Y0 += 0.00000001312*cos(5.81330482072 + 12491.613918899*$t);
   $Y0 += 0.00000001246*cos(6.23510174757 + 5429.63565075589*$t);
   $Y0 += 0.00000000992*cos(2.56738111789 + 6453.9925380941*$t);
   $Y0 += 0.00000000988*cos(5.69208485871 + 8274.0646845159*$t);
   $Y0 += 0.00000001365*cos(1.39907964627 + 11371.4608722747*$t);
   $Y0 += 0.00000001020*cos(1.25240185088 + 4732.2744448269*$t);
   $Y0 += 0.00000001191*cos(1.85928407242 + 6290.36590417291*$t);
   $Y0 += 0.00000001191*cos(1.64765684663 + 6276.2734307769*$t);
   $Y0 += 0.00000000978*cos(3.16773125497 + 6247.2918007975*$t);
   $Y0 += 0.00000000971*cos(0.33921091467 + 6319.34753415231*$t);
   $Y0 += 0.00000000962*cos(1.06488912173 + 12565.4151963981*$t);
   $Y0 += 0.00000001025*cos(0.64688768472 + 12545.6201219739*$t);
   $Y0 += 0.00000000990*cos(5.10278829353 + 4694.2467721911*$t);
   $Y0 += 0.00000000964*cos(3.36272372610 + 3893.93801205869*$t);
   $Y0 += 0.00000000944*cos(4.29299913988 + 6259.44122972711*$t);
   $Y0 += 0.00000000944*cos(5.49712708636 + 6307.1981052227*$t);
   $Y0 += 0.00000001092*cos(2.34985569802 + 17797.2029842693*$t);
   $Y0 += 0.00000001132*cos(5.71867421008 + 17782.9758902677*$t);
   $Y0 += 0.00000000859*cos(6.19348487479 + 1692.40948698591*$t);
   $Y0 += 0.00000001213*cos(5.61315622766 + 82576.73740351178*$t);
   $Y0 += 0.00000000942*cos(3.65711104430 + 6298.57213865991*$t);
   $Y0 += 0.00000000942*cos(6.13301518193 + 6268.0671962899*$t);
   $Y0 += 0.00000001045*cos(5.17434482333 + 15508.8589407579*$t);
   $Y0 += 0.00000000851*cos(3.62598735241 + 6173.1133462555*$t);
   $Y0 += 0.00000000850*cos(6.16443178844 + 6393.5259886943*$t);
   $Y0 += 0.00000000895*cos(0.83009579073 + 3904.1551939033*$t);
   $Y0 += 0.00000001020*cos(5.42778314413 + 220.6564599223*$t);
   $Y0 += 0.00000000845*cos(4.18420138552 + 17256.8753538249*$t);
   $Y0 += 0.00000000863*cos(2.86079142045 + 949.4194264533*$t);
   $Y0 += 0.00000000853*cos(1.18534704592 + 16201.0165419847*$t);
   $Y0 += 0.00000000811*cos(1.42220880644 + 4803.9654584435*$t);
   $Y0 += 0.00000000966*cos(2.57517734766 + 206.42936592071*$t);
   $Y0 += 0.00000000917*cos(0.89927696450 + 149854.64395229139*$t);
   $Y0 += 0.00000001083*cos(2.89594427545 + 36948.9869909407*$t);
   $Y0 += 0.00000000779*cos(2.01260050316 + 2648.6986429565*$t);
   $Y0 += 0.00000000964*cos(2.34511297941 + 796.0541893329*$t);
   $Y0 += 0.00000000995*cos(4.96662237412 + 11403.9208130585*$t);
   $Y0 += 0.00000000962*cos(2.35578735539 + 12567.37583853451*$t);
   $Y0 += 0.00000000779*cos(2.02763029837 + 10213.0417287275*$t);
   $Y0 += 0.00000000980*cos(6.19670011124 + 22805.4917485101*$t);
   $Y0 += 0.00000000930*cos(0.65237052253 + 2787.2868413409*$t);
   $Y0 += 0.00000000829*cos(0.38404899643 + 5120.35732810009*$t);
   $Y0 += 0.00000000733*cos(4.18029489609 + 10575.1628654583*$t);
   $Y0 += 0.00000000743*cos(2.22826365578 + 7834.3648901229*$t);
   $Y0 += 0.00000000818*cos(3.21447961169 + 5572.8989839496*$t);
   $Y0 += 0.00000000905*cos(2.24459183660 + 6284.8041401832*$t);
   $Y0 += 0.00000000905*cos(1.26234908246 + 6281.8351947666*$t);
   $Y0 += 0.00000000798*cos(1.05421419226 + 12410.9751180321*$t);
   $Y0 += 0.00000000817*cos(5.23322400864 + 12416.8323203317*$t);
   $Y0 += 0.00000000884*cos(2.41399532496 + 22483.60475700909*$t);
   $Y0 += 0.00000000755*cos(5.40020688961 + 4060.97539791089*$t);
   $Y0 += 0.00000000651*cos(5.50763453832 + 17259.91083720689*$t);
   $Y0 += 0.00000000616*cos(4.13394819140 + 1596.43025976811*$t);
   $Y0 += 0.00000000651*cos(0.84412253043 + 1748.2602305505*$t);
   $Y0 += 0.00000000723*cos(1.49663741427 + 161000.92955515759*$t);
   $Y0 += 0.00000000784*cos(2.44837693578 + 4907.05823266209*$t);
   $Y0 += 0.00000000784*cos(3.77248887935 + 7234.5504387585*$t);
   $Y0 += 0.00000000785*cos(1.86574625348 + 846.3266522347*$t);
   $Y0 += 0.00000000611*cos(6.24391260854 + 853.4401992355*$t);
   $Y0 += 0.00000000576*cos(5.94037158869 + 12323.6669134923*$t);
   $Y0 += 0.00000000570*cos(2.97586328906 + 12587.1709129587*$t);
   $Y0 += 0.00000000576*cos(4.63987761174 + 13915.77529215809*$t);
   $Y0 += 0.00000000770*cos(0.89072554788 + 6069.53293706989*$t);
   $Y0 += 0.00000000773*cos(1.78442906238 + 6133.2688353733*$t);
   $Y0 += 0.00000000581*cos(6.05264641509 + 11933.6117781531*$t);
   $Y0 += 0.00000000719*cos(2.47920208822 + 15720.5949673949*$t);
   $Y0 += 0.00000000578*cos(2.69917567595 + 8662.48414104651*$t);
   $Y0 += 0.00000000536*cos(4.49580958272 + 18852.9944858067*$t);
   $Y0 += 0.00000000633*cos(5.88497258324 + 52176.0501006319*$t);
   $Y0 += 0.00000000581*cos(1.35719256610 + 5334.1440585051*$t);
   $Y0 += 0.00000000532*cos(3.01682546062 + 18842.3578204569*$t);
   $Y0 += 0.00000000657*cos(4.82951502701 + 5849.1202946311*$t);
   $Y0 += 0.00000000599*cos(0.54713597581 + 6151.7777057885*$t);
   $Y0 += 0.00000000540*cos(1.67333017022 + 6286.6060248927*$t);
   $Y0 += 0.00000000540*cos(1.83361074884 + 6280.0333100571*$t);
   $Y0 += 0.00000000628*cos(5.02906961003 + 17298.4261448097*$t);
   $Y0 += 0.00000000532*cos(2.84025435717 + 11514.1271342779*$t);
   $Y0 += 0.00000000513*cos(2.13211972766 + 12456.1891962469*$t);
   $Y0 += 0.00000000649*cos(3.33488996078 + 11764.5745863425*$t);
   $Y0 += 0.00000000582*cos(2.96012797214 + 6414.8616291613*$t);
   $Y0 += 0.00000000517*cos(5.63213604686 + 3340.36860921629*$t);
   $Y0 += 0.00000000609*cos(2.37414673548 + 10420.23010099111*$t);
   $Y0 += 0.00000000615*cos(0.00032066291 + 10983.94853421629*$t);
   $Y0 += 0.00000000622*cos(0.82619343851 + 5326.5428765373*$t);
   $Y0 += 0.00000000536*cos(2.13145334966 + 7232.49527644471*$t);
   $Y0 += 0.00000000516*cos(1.35907342509 + 433.9555553603*$t);
   $Y0 += 0.00000000613*cos(3.36175443914 + 10969.72144021469*$t);
   $Y0 += 0.00000000583*cos(1.44072369133 + 5863.3473886327*$t);
   $Y0 += 0.00000000636*cos(1.34803641207 + 26735.7014447297*$t);
   $Y0 += 0.00000000665*cos(4.31762068350 + 40879.1966871603*$t);
   $Y0 += 0.00000000583*cos(4.39404867767 + 12592.2062022991*$t);
   $Y0 += 0.00000000479*cos(6.04787718072 + 5547.4431539431*$t);
   $Y0 += 0.00000000571*cos(1.01593938825 + 6062.9070250361*$t);
   $Y0 += 0.00000000565*cos(5.49171831457 + 4171.1817191303*$t);
   $Y0 += 0.00000000462*cos(3.53710274271 + 3104.6862419403*$t);
   $Y0 += 0.00000000551*cos(2.46794798020 + 6503.7323099137*$t);
   $Y0 += 0.00000000504*cos(0.43308443375 + 15670.83794192309*$t);
   $Y0 += 0.00000000535*cos(4.55197610339 + 173567.08125514039*$t);
   $Y0 += 0.00000000460*cos(4.75481082956 + 3495.78900865049*$t);
   $Y0 += 0.00000000449*cos(0.38594751092 + 4274.27449334889*$t);
   $Y0 += 0.00000000453*cos(5.14645638415 + 9387.76209193169*$t);
   $Y0 += 0.00000000527*cos(4.67747505318 + 24602.8562523545*$t);
   $Y0 += 0.00000000502*cos(5.68646114706 + 12490.1294461907*$t);
   $Y0 += 0.00000000603*cos(5.56725967880 + 322711.54834138996*$t);
   $Y0 += 0.00000000532*cos(0.26579545469 + 5120.8449630671*$t);
   $Y0 += 0.00000000530*cos(5.14730101928 + 18845.9482491087*$t);
   $Y0 += 0.00000000431*cos(3.74066314880 + 7019.19618100671*$t);
   $Y0 += 0.00000000465*cos(1.12060083942 + 8827.14645239129*$t);
   $Y0 += 0.00000000559*cos(0.14481247330 + 1582.2031657665*$t);
   $Y0 += 0.00000000485*cos(4.56530394374 + 29296.8592070621*$t);
   $Y0 += 0.00000000584*cos(4.78377689692 + 72850.80553272919*$t);
   $Y0 += 0.00000000550*cos(5.52640225498 + 213.0552779545*$t);
   $Y0 += 0.00000000481*cos(0.29866487122 + 14.47091148511*$t);
   $Y0 += 0.00000000569*cos(1.38161051216 + 97238.871361971*$t);
   $Y0 += 0.00000000551*cos(3.21618763564 + 14313.9242955663*$t);
   $Y0 += 0.00000000389*cos(1.85967717576 + 6245.1866318371*$t);
   $Y0 += 0.00000000389*cos(1.64726374330 + 6321.4527031127*$t);
   $Y0 += 0.00000000388*cos(5.85202122263 + 6297.5467614765*$t);
   $Y0 += 0.00000000388*cos(3.93810500361 + 6269.0925734733*$t);
   $Y0 += 0.00000000515*cos(0.09995632139 + 12320.56387123691*$t);
   $Y0 += 0.00000000394*cos(1.39198903052 + 4156.95462512869*$t);
   $Y0 += 0.00000000491*cos(4.39709251394 + 1479.11039154791*$t);
   $Y0 += 0.00000000440*cos(2.54396321518 + 5650.0482931947*$t);
   $Y0 += 0.00000000382*cos(1.50580912176 + 9917.9406919933*$t);
   $Y0 += 0.00000000503*cos(3.08687612414 + 17157.3056979553*$t);
   $Y0 += 0.00000000439*cos(3.25483236471 + 233141.55822184499*$t);
   $Y0 += 0.00000000381*cos(3.59564013891 + 14143.2514249471*$t);
   $Y0 += 0.00000000371*cos(3.01987537874 + 5643.4223811609*$t);
   $Y0 += 0.00000000516*cos(4.04344774561 + 135.30889751891*$t);
   $Y0 += 0.00000000417*cos(2.17381926869 + 13760.8425276909*$t);
   $Y0 += 0.00000000483*cos(2.84671280305 + 9779.3524936089*$t);
   $Y0 += 0.00000000381*cos(2.79871486639 + 14919.2616712381*$t);
   $Y0 += 0.00000000378*cos(2.94872717012 + 17267.5120191747*$t);
   $Y0 += 0.00000000393*cos(6.27917510634 + 7872.39256275871*$t);
   $Y0 += 0.00000000376*cos(6.02660735301 + 13517.62628874989*$t);
   $Y0 += 0.00000000365*cos(0.53940975175 + 6923.2169537889*$t);
   $Y0 += 0.00000000371*cos(4.19154046767 + 13625.7774476555*$t);
   $Y0 += 0.00000000355*cos(3.67238322579 + 10874.2298479639*$t);
   $Y0 += 0.00000000499*cos(1.15482974306 + 161710.8626037159*$t);
   $Y0 += 0.00000000379*cos(5.52860495919 + 3185.43584474911*$t);
   $Y0 += 0.00000000471*cos(0.07055813902 + 11712.71150074729*$t);
   $Y0 += 0.00000000417*cos(1.75993959010 + 22779.6810636773*$t);
   $Y0 += 0.00000000485*cos(6.06683096360 + 12528.26248182851*$t);
   $Y0 += 0.00000000358*cos(1.92035724848 + 6295.0490203109*$t);
   $Y0 += 0.00000000358*cos(1.58658367058 + 6271.5903146389*$t);
   $Y0 += 0.00000000365*cos(5.19130080276 + 6836.40143535029*$t);
   $Y0 += 0.00000000351*cos(0.64808198517 + 11617.21990849651*$t);
   $Y0 += 0.00000000351*cos(4.53436303138 + 205.9417309537*$t);
   $Y0 += 0.00000000429*cos(5.51029115203 + 3894.4256470257*$t);
   $Y0 += 0.00000000388*cos(5.20318951030 + 956.53297345411*$t);
   $Y0 += 0.00000000464*cos(5.40703775457 + 23581.5019948011*$t);
   $Y0 += 0.00000000396*cos(0.78828729228 + 5231.0512842865*$t);
   $Y0 += 0.00000000421*cos(3.25659811640 + 7445.7943718827*$t);
   $Y0 += 0.00000000366*cos(5.33851384127 + 17253.2849251731*$t);
   $Y0 += 0.00000000360*cos(5.53135776054 + 21393.7857873411*$t);
   $Y0 += 0.00000000336*cos(6.12569565789 + 6279.3875142118*$t);
   $Y0 += 0.00000000336*cos(3.66443056835 + 6287.251820738*$t);
   $Y0 += 0.00000000326*cos(0.22542241669 + 1059.1381127057*$t);
   $Y0 += 0.00000000395*cos(5.25182737341 + 5642.4420600927*$t);
   $Y0 += 0.00000000450*cos(1.80229899871 + 1385.3177574676*$t);
   $Y0 += 0.00000000363*cos(2.72535712612 + 22484.0923919761*$t);
   $Y0 += 0.00000000399*cos(5.51787738042 + 16858.7263504167*$t);
   $Y0 += 0.00000000316*cos(2.45995539098 + 20995.6367839329*$t);
   $Y0 += 0.00000000430*cos(3.96101117316 + 19650.8046636145*$t);
   $Y0 += 0.00000000363*cos(2.71990145670 + 7335.5880506633*$t);
   $Y0 += 0.00000000317*cos(3.09968934621 + 11769.6098756829*$t);
   $Y0 += 0.00000000395*cos(3.82348351794 + 5905.94605955911*$t);
   $Y0 += 0.00000000324*cos(5.27854205827 + 37.7838551523*$t);
   $Y0 += 0.00000000299*cos(2.29226668471 + 641.12142486571*$t);
   $Y0 += 0.00000000304*cos(4.87224846445 + 5750.1055840313*$t);
   $Y0 += 0.00000000298*cos(2.25684047934 + 1350.1112271423*$t);
   $Y0 += 0.00000000414*cos(5.74970236841 + 44809.4063833799*$t);
   $Y0 += 0.00000000339*cos(5.86256617398 + 3.6883357796*$t);
   $Y0 += 0.00000000351*cos(4.36315861783 + 12345.9828750275*$t);
   $Y0 += 0.00000000362*cos(1.99294614713 + 21953.91379191449*$t);
   $Y0 += 0.00000000304*cos(3.36735752668 + 29826.5501721567*$t);
   $Y0 += 0.00000000395*cos(4.78960869869 + 4176.2851599325*$t);
   $Y0 += 0.00000000320*cos(0.92821593891 + 10818.3791043993*$t);
   $Y0 += 0.00000000384*cos(5.58951901360 + 10177.01386205009*$t);
   $Y0 += 0.00000000279*cos(5.67648753585 + 10970.2090751817*$t);
   $Y0 += 0.00000000365*cos(5.96664259426 + 6660.6932753907*$t);
   $Y0 += 0.00000000377*cos(5.53753804163 + 29864.5778447925*$t);
   $Y0 += 0.00000000275*cos(4.16427905835 + 20597.4877805247*$t);
   $Y0 += 0.00000000316*cos(1.02608510804 + 316.6356871401*$t);
   $Y0 += 0.00000000339*cos(4.53749399958 + 6924.1972748571*$t);
   $Y0 += 0.00000000386*cos(0.71953202985 + 2636.9692901205*$t);
   $Y0 += 0.00000000273*cos(5.80758206821 + 26709.8907598969*$t);
   $Y0 += 0.00000000314*cos(6.18462887647 + 14945.5599910379*$t);
   $Y0 += 0.00000000281*cos(0.15414528977 + 16858.23871544969*$t);
   $Y0 += 0.00000000287*cos(4.87229910240 + 18073.46112116669*$t);
   $Y0 += 0.00000000265*cos(3.53930677592 + 19379.1623325523*$t);
   $Y0 += 0.00000000325*cos(0.92198404013 + 12360.2099690291*$t);
   $Y0 += 0.00000000362*cos(2.75236634018 + 30665.9111409493*$t);
   $Y0 += 0.00000000269*cos(4.91776126981 + 6816.53375091851*$t);
   $Y0 += 0.00000000344*cos(1.94480091531 + 6147.69434246491*$t);
   $Y0 += 0.00000000255*cos(3.37543976329 + 1376.0176173293*$t);
   $Y0 += 0.00000000334*cos(1.56223491581 + 6418.9449924849*$t);
   $Y0 += 0.00000000316*cos(3.97125698165 + 6055.8435346859*$t);
   $Y0 += 0.00000000283*cos(2.66015602247 + 28287.2343023447*$t);
   $Y0 += 0.00000000305*cos(5.83332121064 + 16522.4158985187*$t);
   $Y0 += 0.00000000270*cos(1.62295499850 + 283.6155013817*$t);
   $Y0 += 0.00000000243*cos(0.97600841060 + 6255.9181113781*$t);
   $Y0 += 0.00000000243*cos(2.53093250846 + 6310.7212235717*$t);
   $Y0 += 0.00000000296*cos(4.79138408835 + 6129.5408569901*$t);
   $Y0 += 0.00000000301*cos(5.81886935980 + 6510.7958002639*$t);
   $Y0 += 0.00000000329*cos(5.86922121940 + 377.6174253993*$t);
   $Y0 += 0.00000000267*cos(1.84931872641 + 24705.9490265731*$t);
   $Y0 += 0.00000000285*cos(0.93414937692 + 5469.7693835151*$t);
   $Y0 += 0.00000000286*cos(4.99886262730 + 6437.0984779597*$t);
   $Y0 += 0.00000000311*cos(1.63984256457 + 11720.3126827151*$t);
   $Y0 += 0.00000000319*cos(3.13980275142 + 169379.5000286584*$t);
   $Y0 += 0.00000000241*cos(4.56518963450 + 632.5399218297*$t);
   $Y0 += 0.00000000229*cos(5.72761357577 + 1265.81129610991*$t);
   $Y0 += 0.00000000235*cos(4.94058515466 + 4487.57358878689*$t);
   $Y0 += 0.00000000253*cos(2.08435720561 + 4377.3672675675*$t);
   $Y0 += 0.00000000277*cos(3.15798901317 + 419.2408263917*$t);
   $Y0 += 0.00000000234*cos(3.55494952630 + 11713.1991357143*$t);
   $Y0 += 0.00000000216*cos(0.30207043900 + 10454.25756912169*$t);
   $Y0 += 0.00000000241*cos(3.87591164077 + 103.3365917021*$t);
   $Y0 += 0.00000000214*cos(0.52402082918 + 2222.1004520805*$t);
   $Y0 += 0.00000000213*cos(2.22358750694 + 30356.2411372513*$t);
   $Y0 += 0.00000000240*cos(4.14485134121 + 6309.1303523077*$t);
   $Y0 += 0.00000000212*cos(3.93716983669 + 262.84010588929*$t);
   $Y0 += 0.00000000259*cos(1.64646341991 + 6283.56348495841*$t);
   $Y0 += 0.00000000259*cos(1.86047749915 + 6283.0758499914*$t);
   $Y0 += 0.00000000222*cos(2.99868754607 + 10440.03047512009*$t);
   $Y0 += 0.00000000201*cos(0.39862703515 + 5746.0275204125*$t);
   $Y0 += 0.00000000274*cos(5.42466894469 + 23581.0143598341*$t);
   $Y0 += 0.00000000243*cos(2.57266886723 + 7096.8699514347*$t);
   $Y0 += 0.00000000234*cos(3.66617634256 + 5573.3866189166*$t);
   $Y0 += 0.00000000203*cos(1.57613060854 + 16460.08971204149*$t);
   $Y0 += 0.00000000246*cos(4.26574746801 + 8672.21368792411*$t);
   $Y0 += 0.00000000216*cos(3.35974562754 + 5437.23683272371*$t);
   $Y0 += 0.00000000210*cos(4.25419338790 + 9381.2034902007*$t);
   $Y0 += 0.00000000199*cos(5.64767313384 + 11216.5281078075*$t);
   $Y0 += 0.00000000270*cos(5.11258425974 + 284.1031363487*$t);
   $Y0 += 0.00000000207*cos(1.49549061720 + 12562.80508881451*$t);
   $Y0 += 0.00000000212*cos(0.18391974040 + 7129.4025022261*$t);
   $Y0 += 0.00000000269*cos(6.23907858108 + 70755.31090921978*$t);
   $Y0 += 0.00000000221*cos(4.97264401025 + 77713.52765063698*$t);
   $Y0 += 0.00000000225*cos(5.58708477402 + 5635.8211991931*$t);
   $Y0 += 0.00000000197*cos(4.93201648946 + 14712.07329897449*$t);
   $Y0 += 0.00000000192*cos(0.47501431186 + 17272.1278250099*$t);
   $Y0 += 0.00000000241*cos(3.52343193321 + 15907.0079441661*$t);
   $Y0 += 0.00000000261*cos(0.88974137342 + 48739.6160795995*$t);
   $Y0 += 0.00000000255*cos(2.10230203862 + 6206.5659612323*$t);
   $Y0 += 0.00000000256*cos(0.73069007306 + 224.5886131854*$t);
   $Y0 += 0.00000000182*cos(5.98997615944 + 18848.98373249069*$t);
   $Y0 += 0.00000000250*cos(2.67103598665 + 5934.39524702691*$t);
   $Y0 += 0.00000000196*cos(0.64831634218 + 16460.5773470085*$t);
   $Y0 += 0.00000000187*cos(0.04522363352 + 22003.6708173863*$t);
   $Y0 += 0.00000000229*cos(1.73812120333 + 2942.21960580809*$t);
   $Y0 += 0.00000000175*cos(3.84094267540 + 11614.6771112157*$t);
   $Y0 += 0.00000000187*cos(0.76186633809 + 9778.8648586419*$t);
   $Y0 += 0.00000000175*cos(6.15302938442 + 3744.5835282543*$t);
   $Y0 += 0.00000000241*cos(5.00033837792 + 8390.3541750173*$t);
   $Y0 += 0.00000000236*cos(0.45405670770 + 1.7282901918*$t);
   $Y0 += 0.00000000217*cos(3.45380390356 + 17996.2749857057*$t);
   $Y0 += 0.00000000173*cos(2.65478632822 + 6275.71848550709*$t);
   $Y0 += 0.00000000191*cos(4.70676649838 + 394.86970254271*$t);
   $Y0 += 0.00000000231*cos(4.17552456214 + 34596.1208371689*$t);
   $Y0 += 0.00000000169*cos(0.40140025709 + 6438.2524319421*$t);
   $Y0 += 0.00000000235*cos(5.04475734159 + 17256.38771885789*$t);
   $Y0 += 0.00000000188*cos(4.02583282760 + 401.91593924071*$t);
   $Y0 += 0.00000000217*cos(3.40363231104 + 10984.4361691833*$t);
   $Y0 += 0.00000000230*cos(0.83612775625 + 6632.2440879229*$t);
   $Y0 += 0.00000000208*cos(5.43298352980 + 11087.5289434019*$t);
   $Y0 += 0.00000000165*cos(0.80102915744 + 743.23387801611*$t);
   $Y0 += 0.00000000177*cos(0.01829077781 + 4796.85191144269*$t);
   $Y0 += 0.00000000208*cos(6.16123229814 + 3097.64000524229*$t);
   $Y0 += 0.00000000167*cos(2.40308314985 + 5714.4977934475*$t);
   $Y0 += 0.00000000198*cos(4.83812277030 + 5539.8419719753*$t);
   $Y0 += 0.00000000169*cos(2.16903380895 + 12132.6837795895*$t);
   $Y0 += 0.00000000214*cos(2.77137176457 + 24492.6499311351*$t);
   $Y0 += 0.00000000172*cos(5.97118942936 + 6233.5626420031*$t);
   $Y0 += 0.00000000172*cos(3.81893679688 + 6333.0766929467*$t);
   $Y0 += 0.00000000223*cos(3.42911214381 + 266.85085920531*$t);
   $Y0 += 0.00000000159*cos(2.33166453392 + 13199.1792567795*$t);
   $Y0 += 0.00000000219*cos(0.69566285170 + 10344.05124790229*$t);
   $Y0 += 0.00000000207*cos(1.92518585993 + 12569.9859461181*$t);
   $Y0 += 0.00000000176*cos(2.48119576594 + 12012.8261146239*$t);
   $Y0 += 0.00000000170*cos(4.43959906760 + 6294.36536773881*$t);
   $Y0 += 0.00000000170*cos(5.35052715864 + 6272.273967211*$t);
   $Y0 += 0.00000000205*cos(1.15927608909 + 13119.47728534169*$t);
   $Y0 += 0.00000000167*cos(3.97305893961 + 17686.9966630499*$t);
   $Y0 += 0.00000000210*cos(2.45255073470 + 13521.50762410789*$t);
   $Y0 += 0.00000000163*cos(6.18035129909 + 802.0647486073*$t);
   $Y0 += 0.00000000159*cos(0.13779660073 + 5017.2645538815*$t);
   $Y0 += 0.00000000148*cos(5.46314197385 + 419.72846135871*$t);
   $Y0 += 0.00000000165*cos(5.59205778351 + 20199.3387771165*$t);
   $Y0 += 0.00000000196*cos(5.15079536124 + 33326.33491569069*$t);
   $Y0 += 0.00000000146*cos(1.96564383477 + 19800.7021387413*$t);
   $Y0 += 0.00000000148*cos(1.10385783973 + 6852.1415415023*$t);
   $Y0 += 0.00000000146*cos(4.95120975616 + 17370.6047933933*$t);
   $Y0 += 0.00000000143*cos(5.56441247552 + 5618.5636223449*$t);
   $Y0 += 0.00000000142*cos(0.65902902155 + 17654.5367222661*$t);
   $Y0 += 0.00000000140*cos(5.93480456615 + 2008.8013566425*$t);
   $Y0 += 0.00000000153*cos(3.36826827070 + 5436.7491977567*$t);
   $Y0 += 0.00000000162*cos(5.27294659555 + 775.2787938405*$t);
   $Y0 += 0.00000000162*cos(3.21674523006 + 12552.1684234647*$t);
   $Y0 += 0.00000000143*cos(3.89298583971 + 5010.1510068807*$t);
   $Y0 += 0.00000000188*cos(2.11622447442 + 28.6930049513*$t);
   $Y0 += 0.00000000155*cos(4.58570766573 + 11610.1063614957*$t);
   $Y0 += 0.00000000174*cos(1.00725781500 + 20452.6255947383*$t);
   $Y0 += 0.00000000179*cos(0.59168717373 + 27511.2240560537*$t);
   $Y0 += 0.00000000135*cos(4.79054860554 + 12431.3304374309*$t);
   $Y0 += 0.00000000164*cos(4.03840473864 + 28767.1682419675*$t);
   $Y0 += 0.00000000134*cos(3.87019546457 + 16840.9138282987*$t);
   $Y0 += 0.00000000174*cos(2.70675879120 + 19805.0711090663*$t);
   $Y0 += 0.00000000146*cos(6.27087503385 + 12701.4605975017*$t);
   $Y0 += 0.00000000137*cos(1.03021623583 + 11.2895177474*$t);
   $Y0 += 0.00000000167*cos(5.98298407227 + 17473.6975676119*$t);
   $Y0 += 0.00000000152*cos(2.62025173907 + 16627.6147328607*$t);
   $Y0 += 0.00000000124*cos(4.22571397919 + 6948.0757126049*$t);
   $Y0 += 0.00000000135*cos(4.95808633295 + 3531.2844328163*$t);
   $Y0 += 0.00000000172*cos(0.23022549161 + 167959.14629657479*$t);
   $Y0 += 0.00000000123*cos(0.49830220802 + 23013.29572210369*$t);
   $Y0 += 0.00000000122*cos(4.12184350213 + 3583.5848481573*$t);
   $Y0 += 0.00000000170*cos(6.16462326207 + 333857.83394425618*$t);
   $Y0 += 0.00000000155*cos(5.62824965405 + 6058.48723680599*$t);
   $Y0 += 0.00000000120*cos(5.90379805618 + 12809.12412144031*$t);
   $Y0 += 0.00000000141*cos(3.95455666878 + 162420.79565227419*$t);
   $Y0 += 0.00000000120*cos(4.26997270152 + 12528.3678448305*$t);
   $Y0 += 0.00000000123*cos(1.49653119992 + 25933.8805136059*$t);
   $Y0 += 0.00000000144*cos(3.70806542711 + 95.7354097343*$t);
   $Y0 += 0.00000000165*cos(2.31400541864 + 52669.8257758191*$t);
   $Y0 += 0.00000000119*cos(5.90934736896 + 19247.6203708659*$t);
   $Y0 += 0.00000000117*cos(5.45034774436 + 11610.7957758577*$t);
   $Y0 += 0.00000000163*cos(1.54456737879 + 661.4767442645*$t);
   $Y0 += 0.00000000163*cos(2.78769461236 + 9929.6700448293*$t);
   $Y0 += 0.00000000119*cos(2.71628583938 + 12250.0036478097*$t);
   $Y0 += 0.00000000145*cos(2.33523694188 + 6205.6458970469*$t);
   $Y0 += 0.00000000145*cos(1.17170397718 + 6360.99343790291*$t);
   $Y0 += 0.00000000159*cos(2.61434005227 + 228278.34846897019*$t);
   $Y0 += 0.00000000114*cos(3.41169068642 + 19402.55313533309*$t);
   $Y0 += 0.00000000147*cos(5.63174103655 + 38526.3305333885*$t);
   $Y0 += 0.00000000156*cos(1.00709118546 + 4307.8271216189*$t);
   $Y0 += 0.00000000131*cos(0.13588364911 + 21228.63584102931*$t);
   $Y0 += 0.00000000118*cos(3.83848144749 + 6263.64990657511*$t);
   $Y0 += 0.00000000118*cos(5.95164477875 + 6302.9894283747*$t);
   $Y0 += 0.00000000137*cos(0.97335445483 + 6315.8522182663*$t);
   $Y0 += 0.00000000137*cos(2.53358646423 + 6250.7871166835*$t);
   $Y0 += 0.00000000109*cos(1.19644618130 + 11925.5179100841*$t);
   $Y0 += 0.00000000111*cos(2.80331366377 + 6226.4212925393*$t);
   $Y0 += 0.00000000111*cos(0.70362725529 + 6340.2180424105*$t);
   $Y0 += 0.00000000123*cos(0.80537647876 + 24734.3982140409*$t);
   $Y0 += 0.00000000107*cos(0.59542543867 + 12463.30274324771*$t);
   $Y0 += 0.00000000133*cos(1.45986519440 + 18875.76968725751*$t);
   $Y0 += 0.00000000123*cos(4.07179306364 + 6260.5444660241*$t);
   $Y0 += 0.00000000123*cos(5.71833316259 + 6306.09486892571*$t);
   $Y0 += 0.00000000113*cos(0.38322617607 + 2111.8941308611*$t);
   $Y0 += 0.00000000131*cos(2.04114030865 + 18415.7596295809*$t);
   $Y0 += 0.00000000110*cos(0.19106597205 + 6289.94822637491*$t);
   $Y0 += 0.00000000110*cos(3.31587494701 + 6276.69110857489*$t);
   $Y0 += 0.00000000103*cos(2.31964172720 + 6241.7688764901*$t);
   $Y0 += 0.00000000103*cos(1.18729919186 + 6324.8704584597*$t);
   $Y0 += 0.00000000113*cos(0.63409757277 + 3496.2766436175*$t);
   $Y0 += 0.00000000103*cos(3.00856574797 + 10344.5388828693*$t);
   $Y0 += 0.00000000115*cos(1.09796447220 + 24336.2492106327*$t);
   $Y0 += 0.00000000107*cos(0.14947871700 + 83974.07917321339*$t);
   $Y0 += 0.00000000107*cos(1.18802993790 + 84020.10309797739*$t);
   $Y0 += 0.00000000106*cos(1.70601019340 + 12772.58106590351*$t);
   $Y0 += 0.00000000143*cos(6.22463763653 + 2069.2506523901*$t);
   $Y0 += 0.00000000143*cos(5.63223127595 + 18773.2052961821*$t);
   $Y0 += 0.00000000115*cos(5.35405637827 + 3641.4907540357*$t);
   $Y0 += 0.00000000112*cos(1.02603600816 + 11499.41240530929*$t);
   $Y0 += 0.00000000099*cos(0.21940282455 + 11190.6217176205*$t);
   $Y0 += 0.00000000126*cos(5.90533164276 + 18823.1730476579*$t);
   $Y0 += 0.00000000115*cos(3.09464836385 + 12570.3276707294*$t);
   $Y0 += 0.00000000100*cos(1.95823440417 + 16062.4283436003*$t);
   $Y0 += 0.00000000107*cos(2.45486702592 + 5216.8241902849*$t);
   $Y0 += 0.00000000120*cos(0.25356721797 + 9814.36028280769*$t);
   $Y0 += 0.00000000138*cos(2.98838673165 + 6210.0225416159*$t);
   $Y0 += 0.00000000138*cos(0.51855418741 + 6356.6167933339*$t);
   $Y0 += 0.00000000105*cos(2.60431357489 + 12721.32828193349*$t);
   $Y0 += 0.00000000135*cos(5.28295396943 + 18699.9081703231*$t);
   $Y0 += 0.00000000104*cos(0.57981411648 + 12560.8725931589*$t);
   $Y0 += 0.00000000098*cos(4.00435005538 + 5815.3546771205*$t);
   $Y0 += 0.00000000111*cos(5.54435587296 + 10239.3400485273*$t);
   $Y0 += 0.00000000100*cos(2.98744963581 + 263.3277408563*$t);
   $Y0 += 0.00000000095*cos(0.17662269567 + 155.6642169177*$t);
   $Y0 += 0.00000000130*cos(0.55194216539 + 27511.7116910207*$t);
   $Y0 += 0.00000000120*cos(1.84949333794 + 31441.4337522733*$t);
   $Y0 += 0.00000000114*cos(2.18292460802 + 6155.3008241375*$t);
   $Y0 += 0.00000000114*cos(1.32401631104 + 6411.3385108123*$t);
   $Y0 += 0.00000000119*cos(3.64842963894 + 951.9622237341*$t);
   $Y0 += 0.00000000117*cos(3.47158263266 + 28236.98964190589*$t);
   $Y0 += 0.00000000107*cos(0.33835239167 + 21.0192129759*$t);
   $Y0 += 0.00000000095*cos(4.06228359170 + 11300.8280388399*$t);
   $Y0 += 0.00000000110*cos(1.22040625314 + 6312.74917601091*$t);
   $Y0 += 0.00000000110*cos(2.28653466592 + 6253.8901589389*$t);
   $Y0 += 0.00000000097*cos(2.64989366154 + 78263.95324220609*$t);
   $Y0 += 0.00000000095*cos(4.41295990485 + 23938.1002072245*$t);
   $Y0 += 0.00000000091*cos(1.42377843647 + 12829.4794408391*$t);
   $Y0 += 0.00000000112*cos(2.74085534623 + 16737.8210540801*$t);
   $Y0 += 0.00000000126*cos(4.67942071523 + 3.2793008655*$t);
   $Y0 += 0.00000000100*cos(1.46206803081 + 6133.7564703403*$t);
   $Y0 += 0.00000000127*cos(0.70264556724 + 1293.24040609909*$t);
   $Y0 += 0.00000000119*cos(4.42085627054 + 17893.1822114871*$t);
   $Y0 += 0.00000000089*cos(6.04696991631 + 23539.9512038163*$t);
   $Y0 += 0.00000000126*cos(4.96989590925 + 311565.26273852374*$t);
   $Y0 += 0.00000000093*cos(4.10519400640 + 736.1203310153*$t);
   $Y0 += 0.00000000120*cos(0.51718078841 + 14158.9915310991*$t);
   $Y0 += 0.00000000118*cos(3.00291998094 + 16061.94070863329*$t);
   $Y0 += 0.00000000099*cos(2.04878528903 + 6432.8828646095*$t);
   $Y0 += 0.00000000095*cos(1.28476695157 + 2699.49100183409*$t);
   $Y0 += 0.00000000089*cos(0.73014646335 + 15671.3255768901*$t);
   $Y0 += 0.00000000124*cos(5.19246908636 + 178430.29100801519*$t);
   $Y0 += 0.00000000089*cos(5.78577582542 + 6751.28465782931*$t);
   $Y0 += 0.00000000096*cos(1.07414401415 + 7349.81514466491*$t);
   $Y0 += 0.00000000096*cos(5.45460641914 + 24066.0517402591*$t);
   $Y0 += 0.00000000103*cos(2.80667748337 + 18202.4605341429*$t);
   $Y0 += 0.00000000099*cos(4.49603988169 + 6252.40554183991*$t);
   $Y0 += 0.00000000099*cos(5.29408634455 + 6314.2337931099*$t);
   $Y0 += 0.00000000090*cos(1.96044978655 + 58864.30010066279*$t);
   $Y0 += 0.00000000086*cos(2.16910774240 + 9380.71585523369*$t);
   $Y0 += 0.00000000086*cos(2.50989174885 + 10557.35034334029*$t);
   $Y0 += 0.00000000098*cos(1.84392439378 + 6439.7203879773*$t);
   $Y0 += 0.00000000098*cos(1.66301652528 + 6126.91894697251*$t);
   $Y0 += 0.00000000086*cos(6.05387198611 + 23123.9896782901*$t);
   $Y0 += 0.00000000118*cos(2.16764726700 + 13951.9570924174*$t);
   $Y0 += 0.00000000117*cos(2.53823002855 + 89570.23393702849*$t);
   $Y0 += 0.00000000083*cos(4.71213278190 + 8858.0711268371*$t);
   $Y0 += 0.00000000092*cos(2.44147365747 + 12342.0507217644*$t);
   $Y0 += 0.00000000085*cos(0.72336255749 + 5017.7521888485*$t);
   $Y0 += 0.00000000101*cos(4.94259233986 + 18429.9867235825*$t);
   $Y0 += 0.00000000088*cos(2.19664804056 + 17054.2129237367*$t);
   $Y0 += 0.00000000113*cos(0.47970289838 + 12985.88016134151*$t);
   $Y0 += 0.00000000083*cos(2.47657261289 + 20597.0001455577*$t);
   $Y0 += 0.00000000082*cos(4.90614394615 + 5483.4985423095*$t);
   $Y0 += 0.00000000087*cos(1.67775805144 + 21424.2228268199*$t);
   $Y0 += 0.00000000107*cos(3.78836692146 + 522.3336006103*$t);
   $Y0 += 0.00000000084*cos(0.84028318953 + 6187.3404402571*$t);
   $Y0 += 0.00000000084*cos(2.66665772953 + 6379.2988946927*$t);
   $Y0 += 0.00000000107*cos(2.45229388212 + 24382.8352909579*$t);
   $Y0 += 0.00000000082*cos(5.66196525445 + 8983.0544867925*$t);
   $Y0 += 0.00000000105*cos(2.30294939014 + 6131.4223863897*$t);
   $Y0 += 0.00000000105*cos(1.20399152892 + 6435.2169485601*$t);
   $Y0 += 0.00000000114*cos(2.50001149568 + 8258.81221333091*$t);
   $Y0 += 0.00000000082*cos(5.40031574607 + 3957.8826236923*$t);
   $Y0 += 0.00000000112*cos(4.27968226719 + 3738.51761262449*$t);
   $Y0 += 0.00000000102*cos(1.32703960779 + 5767.8557963819*$t);
   $Y0 += 0.00000000102*cos(2.17990131127 + 6798.7835385679*$t);
   $Y0 += 0.00000000092*cos(6.06909982221 + 18216.6876281445*$t);
   $Y0 += 0.00000000083*cos(2.92326536483 + 29864.0902098255*$t);
   $Y0 += 0.00000000112*cos(3.91825505548 + 12189.0219095505*$t);
   $Y0 += 0.00000000080*cos(2.01989715959 + 24080.2788342607*$t);
   $Y0 += 0.00000000080*cos(0.09553378894 + 15141.14697682849*$t);
   $Y0 += 0.00000000088*cos(4.57487416308 + 1573.57660661529*$t);
   $Y0 += 0.00000000081*cos(5.29168357177 + 1550.8014051645*$t);
   $Y0 += 0.00000000092*cos(2.79378764615 + 101426.452588453*$t);
   $Y0 += 0.00000000110*cos(5.08245905369 + 78423.9483341623*$t);
   $Y0 += 0.00000000087*cos(3.89276904283 + 1580.6228433133*$t);
   $Y0 += 0.00000000085*cos(1.43001232077 + 27043.2590656993*$t);
   $Y0 += 0.00000000078*cos(3.02198277867 + 6812.5229976025*$t);
   $Y0 += 0.00000000080*cos(1.79867193339 + 6081.06627230081*$t);
   $Y0 += 0.00000000080*cos(1.70826898567 + 6485.573062649*$t);
   $Y0 += 0.00000000079*cos(2.96736855880 + 36109.6260221481*$t);
   $Y0 += 0.00000000088*cos(5.58307910278 + 16944.0066025173*$t);
   $Y0 += 0.00000000081*cos(2.98944824758 + 5.2791068239*$t);
   $Y0 += 0.00000000093*cos(1.14380332092 + 16737.33341911309*$t);
   $Y0 += 0.00000000088*cos(1.67821631800 + 12537.9463299985*$t);
   $Y0 += 0.00000000096*cos(0.48155847774 + 20198.85114214949*$t);
   $Y0 += 0.00000000106*cos(3.73745326901 + 56600.0354720387*$t);
   $Y0 += 0.00000000097*cos(1.71169429599 + 6040.10342853389*$t);
   $Y0 += 0.00000000081*cos(5.19219909829 + 3956.2641985359*$t);
   $Y0 += 0.00000000077*cos(6.10587791973 + 40796.5827401577*$t);
   $Y0 += 0.00000000076*cos(3.78044498975 + 22743.16556203289*$t);
   $Y0 += 0.00000000098*cos(0.77646852784 + 42456.5402296081*$t);
   $Y0 += 0.00000000083*cos(0.63561076928 + 19801.1897737083*$t);
   $Y0 += 0.00000000079*cos(2.93697401198 + 5622.08674069391*$t);
   $Y0 += 0.00000000096*cos(0.41418225890 + 5888.6937824157*$t);
   $Y0 += 0.00000000096*cos(3.09275866016 + 6677.9455525341*$t);
   $Y0 += 0.00000000075*cos(4.66945944657 + 41194.7317435659*$t);
   $Y0 += 0.00000000074*cos(5.37967983395 + 6261.9840270079*$t);
   $Y0 += 0.00000000074*cos(4.41044639229 + 6304.6553079419*$t);
   $Y0 += 0.00000000073*cos(2.45644384945 + 5870.9485706005*$t);
   $Y0 += 0.00000000073*cos(1.05049706961 + 6695.6907643493*$t);
   $Y0 += 0.00000000085*cos(2.08161902252 + 12850.2548363315*$t);
   $Y0 += 0.00000000072*cos(4.75417028947 + 6253.4982293261*$t);
   $Y0 += 0.00000000072*cos(5.03595593677 + 6313.14110562371*$t);
   $Y0 += 0.00000000100*cos(3.80714894085 + 5316.3487900393*$t);
   $Y0 += 0.00000000085*cos(1.33904345426 + 12282.5361986011*$t);
   $Y0 += 0.00000000075*cos(0.39510472937 + 24422.6141688908*$t);
   $Y0 += 0.00000000076*cos(0.41148054527 + 63659.1215683211*$t);
   $Y0 += 0.00000000072*cos(4.97177288715 + 16723.5939600785*$t);
   $Y0 += 0.00000000097*cos(0.21179463686 + 17995.78735073869*$t);
   $Y0 += 0.00000000087*cos(2.06337614338 + 18106.4813069251*$t);
   $Y0 += 0.00000000080*cos(3.60508005382 + 17363.4912463925*$t);
   $Y0 += 0.00000000070*cos(3.06053136401 + 6124.37614969171*$t);
   $Y0 += 0.00000000070*cos(0.44640955505 + 6442.2631852581*$t);
   $Y0 += 0.00000000070*cos(4.42572835968 + 4705.4884900601*$t);
   $Y0 += 0.00000000078*cos(3.37769653143 + 23550.5878691661*$t);
   $Y0 += 0.00000000070*cos(3.84581453123 + 12036.21691740469*$t);
   $Y0 += 0.00000000077*cos(5.00579533671 + 5237.67719632029*$t);
   $Y0 += 0.00000000069*cos(1.30888647192 + 16207.64245401849*$t);
   $Y0 += 0.00000000082*cos(3.91623659935 + 6774.98295993371*$t);
   $Y0 += 0.00000000067*cos(4.88284115481 + 7083.14079264031*$t);
   $Y0 += 0.00000000067*cos(0.26461151247 + 6394.50630976251*$t);
   $Y0 += 0.00000000067*cos(3.24232940659 + 6172.1330251873*$t);
   $Y0 += 0.00000000077*cos(0.28397734649 + 9924.5666040271*$t);
   $Y0 += 0.00000000080*cos(3.79678848458 + 22380.9996177575*$t);
   $Y0 += 0.00000000070*cos(3.03095059330 + 6390.9831914135*$t);
   $Y0 += 0.00000000070*cos(0.47599032576 + 6175.6561435363*$t);
   $Y0 += 0.00000000074*cos(4.12433601146 + 16193.41536001689*$t);
   $Y0 += 0.00000000076*cos(4.07719654455 + 32217.4439985643*$t);
   $Y0 += 0.00000000091*cos(0.90446873976 + 6653.0194834153*$t);
   $Y0 += 0.00000000091*cos(2.60247217930 + 5913.6198515345*$t);
   $Y0 += 0.00000000067*cos(4.94532713838 + 6265.50714535691*$t);
   $Y0 += 0.00000000067*cos(4.84479908786 + 6301.1321895929*$t);
   $Y0 += 0.00000000067*cos(3.01186125982 + 5959.3266158505*$t);
   $Y0 += 0.00000000083*cos(4.51648793763 + 16723.10632511149*$t);
   $Y0 += 0.00000000080*cos(5.49180050451 + 23646.5670963839*$t);
   $Y0 += 0.00000000068*cos(2.07232606852 + 4897.4243911387*$t);
   $Y0 += 0.00000000069*cos(0.56996686256 + 6944.5525942559*$t);
   $Y0 += 0.00000000089*cos(3.57060375879 + 10660.44311755889*$t);
   $Y0 += 0.00000000079*cos(3.24586891365 + 35371.6434484929*$t);
   $Y0 += 0.00000000073*cos(0.54359299539 + 6370.62787201471*$t);
   $Y0 += 0.00000000073*cos(2.96334792367 + 6196.0114629351*$t);
   $Y0 += 0.00000000065*cos(5.23672190600 + 22345.0165586247*$t);
   $Y0 += 0.00000000064*cos(5.80516008052 + 15265.64270181689*$t);
   $Y0 += 0.00000000066*cos(1.12499582368 + 6315.2919732917*$t);
   $Y0 += 0.00000000066*cos(2.38194509538 + 6251.34736165811*$t);
   $Y0 += 0.00000000065*cos(3.44248995108 + 323.74923414091*$t);
   $Y0 += 0.00000000076*cos(1.46082924902 + 10873.74221299689*$t);
   $Y0 += 0.00000000084*cos(0.49974269222 + 109.9625037359*$t);
   $Y0 += 0.00000000063*cos(2.70916029747 + 11823.4054569337*$t);
   $Y0 += 0.00000000064*cos(4.80332084911 + 28774.2817889683*$t);
   $Y0 += 0.00000000072*cos(2.11265208796 + 18099.7594409665*$t);
   $Y0 += 0.00000000066*cos(1.21054975257 + 17576.7903418305*$t);
   $Y0 += 0.00000000072*cos(0.02703719822 + 245707.70992182779*$t);
   $Y0 += 0.00000000061*cos(3.91641579991 + 10557.8379783073*$t);
   $Y0 += 0.00000000064*cos(5.46712129325 + 71430.9394356126*$t);
   $Y0 += 0.00000000068*cos(1.33734717206 + 28760.05469496669*$t);
   $Y0 += 0.00000000062*cos(2.25046201901 + 3854.7898494737*$t);
   $Y0 += 0.00000000073*cos(4.61252989167 + 23440.3815479467*$t);
   $Y0 += 0.00000000060*cos(1.65871121739 + 13088.9729355601*$t);
   $Y0 += 0.00000000078*cos(2.99014426818 + 12564.91104475801*$t);
   $Y0 += 0.00000000060*cos(2.90135289748 + 7548.8871461013*$t);
   $Y0 += 0.00000000059*cos(4.12263682656 + 6286.3551508569*$t);
   $Y0 += 0.00000000075*cos(2.85045311521 + 18625.1265717558*$t);
   $Y0 += 0.00000000058*cos(5.18922617388 + 35050.2440919589*$t);
   $Y0 += 0.00000000060*cos(4.91746089028 + 553.8132203259*$t);
   $Y0 += 0.00000000069*cos(4.25329717989 + 167993.93845370729*$t);
   $Y0 += 0.00000000059*cos(5.13629564655 + 41991.0297503823*$t);
   $Y0 += 0.00000000062*cos(4.44636395235 + 15663.79170522509*$t);
   $Y0 += 0.00000000081*cos(5.98297744820 + 7250.29054491051*$t);
   $Y0 += 0.00000000059*cos(1.11500712714 + 6277.2537518451*$t);
   $Y0 += 0.00000000059*cos(2.39193379192 + 6289.3855831047*$t);
   $Y0 += 0.00000000063*cos(0.56300539120 + 18003.38853270651*$t);
   $Y0 += 0.00000000057*cos(3.30567043605 + 793.0187059509*$t);
   $Y0 += 0.00000000057*cos(1.59794444440 + 10027.65937824569*$t);
   $Y0 += 0.00000000060*cos(0.63448798616 + 647.25465079831*$t);
   $Y0 += 0.00000000072*cos(5.73566114928 + 4597.77991000629*$t);
   $Y0 += 0.00000000059*cos(5.39392547964 + 6279.30891415889*$t);
   $Y0 += 0.00000000068*cos(1.14499717138 + 3166.66025521511*$t);
   $Y0 += 0.00000000056*cos(3.10776343336 + 6226.5164053051*$t);
   $Y0 += 0.00000000056*cos(0.39917748570 + 6340.12292964471*$t);
   $Y0 += 0.00000000056*cos(0.95649088929 + 12303.3115940935*$t);
   $Y0 += 0.00000000057*cos(1.51972566902 + 5864.3952685743*$t);
   $Y0 += 0.00000000057*cos(1.98721525004 + 6702.2440663755*$t);
   $Y0 += 0.00000000069*cos(0.33222800273 + 23536.3607751645*$t);
   $Y0 += 0.00000000059*cos(2.36664156781 + 5760.25461441409*$t);
   $Y0 += 0.00000000065*cos(1.96288183765 + 12139.30969162329*$t);
   $Y0 += 0.00000000059*cos(1.81423548051 + 67589.3312645407*$t);
   $Y0 += 0.00000000073*cos(2.08854442290 + 11079.92776143409*$t);
   $Y0 += 0.00000000076*cos(0.77803347659 + 23227.0824525087*$t);
   $Y0 += 0.00000000067*cos(0.54633497662 + 4804.4530934105*$t);
   $Y0 += 0.00000000073*cos(4.43869578954 + 30349.1275902505*$t);
   $Y0 += 0.00000000074*cos(3.17776060575 + 92747.9329843061*$t);
   $Y0 += 0.00000000070*cos(6.27818224660 + 12299.7884757445*$t);
   $Y0 += 0.00000000060*cos(5.08460970539 + 12171.7696324071*$t);
   $Y0 += 0.00000000059*cos(1.03458131364 + 15.49628866851*$t);
   $Y0 += 0.00000000074*cos(1.19968434762 + 3684.1342325067*$t);
   $Y0 += 0.00000000058*cos(1.58539308271 + 6717.0314053517*$t);
   $Y0 += 0.00000000053*cos(3.68674599490 + 26087.65932409069*$t);
   $Y0 += 0.00000000059*cos(5.76308404993 + 12164.7233957091*$t);
   $Y0 += 0.00000000063*cos(5.23211999381 + 5219.5179490556*$t);
   $Y0 += 0.00000000056*cos(2.45377850108 + 3178.38960805111*$t);
   $Y0 += 0.00000000063*cos(2.16148808063 + 5227.38225558179*$t);
   $Y0 += 0.00000000070*cos(4.83368738065 + 19004.4041319249*$t);
   $Y0 += 0.00000000072*cos(2.62595674809 + 4583.55281600469*$t);
   $Y0 += 0.00000000052*cos(0.96524582701 + 3627.2636600341*$t);
   $Y0 += 0.00000000052*cos(3.30278586555 + 6411.2712005095*$t);
   $Y0 += 0.00000000052*cos(0.20415505351 + 6155.3681344403*$t);
   $Y0 += 0.00000000054*cos(2.04667654344 + 5849.6079295981*$t);
   $Y0 += 0.00000000066*cos(5.93782050272 + 5791.16874004909*$t);
   $Y0 += 0.00000000066*cos(5.93782050272 + 5791.6563750161*$t);
   $Y0 += 0.00000000051*cos(2.41252260456 + 5113.2437810993*$t);
   $Y0 += 0.00000000058*cos(2.72227505748 + 30775.7257811265*$t);
   $Y0 += 0.00000000061*cos(0.72832616881 + 14169.5497447469*$t);
   $Y0 += 0.00000000054*cos(3.16101850189 + 10454.74520408871*$t);
   $Y0 += 0.00000000056*cos(1.20035157643 + 23141.8022004081*$t);
   $Y0 += 0.00000000067*cos(3.89077193587 + 28313.0449871775*$t);
   $Y0 += 0.00000000056*cos(0.37854134464 + 5244.2930566845*$t);
   $Y0 += 0.00000000062*cos(0.60927850338 + 5657.6494751625*$t);
   $Y0 += 0.00000000062*cos(2.89766241568 + 6908.9898597873*$t);
   $Y0 += 0.00000000050*cos(3.10994458213 + 2574.99527684569*$t);
   $Y0 += 0.00000000050*cos(2.89352148340 + 536.5606946119*$t);
   $Y0 += 0.00000000050*cos(0.38317924664 + 6684.9917892321*$t);
   $Y0 += 0.00000000050*cos(3.12376167242 + 5881.6475457177*$t);
   $Y0 += 0.00000000049*cos(0.36435757253 + 16310.73522823709*$t);
   $Y0 += 0.00000000049*cos(3.50595022612 + 16311.2228632041*$t);
   $Y0 += 0.00000000065*cos(2.20156720545 + 46386.7499258277*$t);
   $Y0 += 0.00000000069*cos(5.16096365273 + 60530.2451682583*$t);
   $Y0 += 0.00000000068*cos(4.61952317468 + 96678.14268052569*$t);
   $Y0 += 0.00000000049*cos(4.01249891427 + 12323.17927852529*$t);
   $Y0 += 0.00000000050*cos(2.38095415684 + 21954.4014268815*$t);
   $Y0 += 0.00000000049*cos(4.98226292129 + 4164.5558070965*$t);
   $Y0 += 0.00000000059*cos(0.54830716960 + 33794.7875410121*$t);
   $Y0 += 0.00000000050*cos(1.64594745036 + 43739.0461634493*$t);
   $Y0 += 0.00000000059*cos(2.71376255252 + 3981.73385156551*$t);
   $Y0 += 0.00000000058*cos(3.16580349936 + 27707.29867681129*$t);
   $Y0 += 0.00000000051*cos(1.43461397348 + 7669.21494381111*$t);
   $Y0 += 0.00000000048*cos(3.23576326319 + 3646.59419483791*$t);
   $Y0 += 0.00000000065*cos(1.63790414031 + 82534.6474380951*$t);
   $Y0 += 0.00000000056*cos(2.62361047024 + 800.06494264891*$t);
   $Y0 += 0.00000000057*cos(2.56200976684 + 11609.61872652869*$t);
   $Y0 += 0.00000000048*cos(3.66284145075 + 26.5421372833*$t);
   $Y0 += 0.00000000047*cos(5.82227097996 + 30640.1004561165*$t);
   $Y0 += 0.00000000060*cos(3.54507538375 + 8982.5668518255*$t);
   $Y0 += 0.00000000049*cos(2.15364675842 + 96563.24283557819*$t);
   $Y0 += 0.00000000050*cos(3.60628396854 + 17583.9038888313*$t);
   $Y0 += 0.00000000046*cos(4.33029118489 + 23539.46356884929*$t);
   $Y0 += 0.00000000048*cos(4.40476950322 + 170.4290531357*$t);
   $Y0 += 0.00000000055*cos(0.06525117672 + 735.6326960483*$t);
   $Y0 += 0.00000000054*cos(5.90233323358 + 6680.98103591609*$t);
   $Y0 += 0.00000000060*cos(3.29126676894 + 44033.8837720559*$t);
   $Y0 += 0.00000000064*cos(5.97731735914 + 80181.29364935629*$t);
   $Y0 += 0.00000000059*cos(3.60959132830 + 24279.3508356971*$t);
   $Y0 += 0.00000000049*cos(3.59486989773 + 5490.5447790075*$t);
   $Y0 += 0.00000000058*cos(5.95314247069 + 6490.0204047715*$t);
   $Y0 += 0.00000000058*cos(3.83698375555 + 6076.6189301783*$t);
   $Y0 += 0.00000000046*cos(1.35461053261 + 18208.1061251085*$t);
   $Y0 += 0.00000000045*cos(3.19696372316 + 4480.46004178609*$t);
   $Y0 += 0.00000000055*cos(1.99803947181 + 16097.9237677661*$t);
   $Y0 += 0.00000000055*cos(1.99803947181 + 16097.43613279909*$t);
   $Y0 += 0.00000000061*cos(3.53347961961 + 491.9071099423*$t);
   $Y0 += 0.00000000054*cos(1.22650408797 + 6006.5738956105*$t);
   $Y0 += 0.00000000054*cos(2.28043683109 + 6560.0654393393*$t);
   $Y0 += 0.00000000044*cos(4.77367705513 + 5333.65642353809*$t);
   $Y0 += 0.00000000052*cos(3.85218997059 + 244287.84382471119*$t);
   $Y0 += 0.00000000044*cos(2.69173127084 + 2301.34199842589*$t);
   $Y0 += 0.00000000045*cos(4.35951224787 + 33794.2999060451*$t);
   $Y0 += 0.00000000046*cos(3.84874383995 + 4384.48081456829*$t);
   $Y0 += 0.00000000048*cos(4.63629874583 + 35579.9350570535*$t);
   $Y0 += 0.00000000051*cos(4.91886576255 + 18326.8939493639*$t);
   $Y0 += 0.00000000045*cos(2.63923033609 + 533.45790092711*$t);
   $Y0 += 0.00000000046*cos(3.15339657493 + 16703.3059509825*$t);
   $Y0 += 0.00000000047*cos(2.39403388232 + 5209.2230083171*$t);
   $Y0 += 0.00000000051*cos(1.31112479278 + 38650.4173236825*$t);
   $Y0 += 0.00000000046*cos(2.47672465258 + 12555.3498172024*$t);
   $Y0 += 0.00000000052*cos(4.64446990453 + 5992.3468016089*$t);
   $Y0 += 0.00000000052*cos(5.14565632171 + 6574.2925333409*$t);
   $Y0 += 0.00000000044*cos(3.80837636245 + 69942.1974183125*$t);
   $Y0 += 0.00000000049*cos(3.62132943845 + 12146.9108735911*$t);
   $Y0 += 0.00000000053*cos(4.67452542700 + 39301.8531447125*$t);
   $Y0 += 0.00000000045*cos(4.65107782321 + 46360.9392409949*$t);
   $Y0 += 0.00000000043*cos(1.54528326296 + 8402.0835278533*$t);
   $Y0 += 0.00000000043*cos(0.19791989913 + 44137.1951668575*$t);
   $Y0 += 0.00000000049*cos(3.54725333487 + 22029.9691371861*$t);
   $Y0 += 0.00000000044*cos(1.47478212539 + 31441.92138724031*$t);
   $Y0 += 0.00000000042*cos(2.85938112982 + 9225.29545579949*$t);
   $Y0 += 0.00000000045*cos(3.22032934298 + 71519.54096076029*$t);
   $Y0 += 0.00000000051*cos(2.95623880108 + 7762.1862415393*$t);
   $Y0 += 0.00000000041*cos(5.14395096650 + 3735.48212924251*$t);
   $Y0 += 0.00000000057*cos(1.24099052077 + 40398.4337367495*$t);
   $Y0 += 0.00000000047*cos(6.03171570454 + 4707.9862312257*$t);
   $Y0 += 0.00000000056*cos(5.36736063063 + 28664.0754677489*$t);
   $Y0 += 0.00000000050*cos(2.58729930740 + 11919.3846841515*$t);
   $Y0 += 0.00000000047*cos(5.61037642387 + 316428.47249139856*$t);
   $Y0 += 0.00000000041*cos(2.70468936208 + 27278.7126339243*$t);
   $Y0 += 0.00000000056*cos(1.28041103259 + 83659.5206898825*$t);
   $Y0 += 0.00000000048*cos(0.93182108526 + 12662.3747446841*$t);
   $Y0 += 0.00000000054*cos(2.40910872834 + 16627.12709789369*$t);
   $Y0 += 0.00000000052*cos(2.84085536047 + 12571.9184417737*$t);
   $Y0 += 0.00000000043*cos(3.12713892129 + 7322.34627826531*$t);
   $Y0 += 0.00000000041*cos(1.99075260862 + 664.99986261351*$t);
   $Y0 += 0.00000000042*cos(3.11818521171 + 5641.9544251257*$t);
   $Y0 += 0.00000000044*cos(6.16004757680 + 6425.4982945111*$t);
   $Y0 += 0.00000000044*cos(3.63007864944 + 6141.1410404387*$t);
   $Y0 += 0.00000000039*cos(5.09444795757 + 22594.2987131955*$t);
   $Y0 += 0.00000000040*cos(1.17403028794 + 20894.53186172529*$t);
   $Y0 += 0.00000000044*cos(3.71228299188 + 5540.3296069423*$t);
   $Y0 += 0.00000000050*cos(0.98055905828 + 17782.48825530069*$t);
   $Y0 += 0.00000000054*cos(3.12084664124 + 6006.2846737335*$t);
   $Y0 += 0.00000000054*cos(0.38609427782 + 6560.3546612163*$t);
   $Y0 += 0.00000000045*cos(0.83260974343 + 142861.14741877468*$t);
   $Y0 += 0.00000000047*cos(5.83397233956 + 24499.7634781359*$t);
   $Y0 += 0.00000000043*cos(6.07854396898 + 7026.3097280075*$t);
   $Y0 += 0.00000000047*cos(0.38354823927 + 955.8435590921*$t);
   $Y0 += 0.00000000047*cos(2.93749588479 + 24485.5363841343*$t);
   $Y0 += 0.00000000040*cos(1.03647885736 + 9388.2497268987*$t);
   $Y0 += 0.00000000043*cos(4.24359473122 + 6359.5857387505*$t);
   $Y0 += 0.00000000042*cos(5.53288597787 + 8635.69818627969*$t);
   $Y0 += 0.00000000051*cos(4.03405257508 + 27831.79440179969*$t);
   $Y0 += 0.00000000042*cos(0.37690679264 + 6334.60000533731*$t);
   $Y0 += 0.00000000042*cos(3.13003412642 + 6232.03932961249*$t);
   $Y0 += 0.00000000052*cos(1.96232281613 + 11905.1625906853*$t);
   $Y0 += 0.00000000040*cos(6.22187049858 + 7076.0945559423*$t);
   $Y0 += 0.00000000040*cos(5.00617355027 + 5444.3503797245*$t);
   $Y0 += 0.00000000040*cos(4.78395267597 + 7122.2889552253*$t);
   $Y0 += 0.00000000038*cos(0.67446092329 + 44933.4931736739*$t);
   $Y0 += 0.00000000037*cos(4.98656910933 + 5715.6010297445*$t);
   $Y0 += 0.00000000037*cos(4.80355711691 + 6851.0383052053*$t);
   $Y0 += 0.00000000042*cos(5.51152847292 + 36147.6536947839*$t);
   $Y0 += 0.00000000042*cos(1.28742672679 + 6238.1784478383*$t);
   $Y0 += 0.00000000042*cos(2.21951419227 + 6328.4608871115*$t);
   $Y0 += 0.00000000040*cos(5.21427505339 + 3214.8925631597*$t);
   $Y0 += 0.00000000043*cos(2.22601042104 + 23141.3145654411*$t);
   $Y0 += 0.00000000041*cos(5.52595261887 + 19804.5834740993*$t);
   $Y0 += 0.00000000038*cos(4.48062431201 + 20.11150191529*$t);
   $Y0 += 0.00000000050*cos(0.05709227621 + 84334.66158130829*$t);
   $Y0 += 0.00000000050*cos(4.41842921988 + 60284.16619777939*$t);
   return $Y0;
}



   function Earth_Y1($t) // 600 terms of order 1
{
   $Y1  = 0.00154550744*cos(5.35844734917 + 0.2438174835*$t);
   $Y1 += 0.00051503383*cos(4.43183566713 + 12566.3955174663*$t);
   $Y1 += 0.00001290763*cos(4.38862020180 + 18849.4713674577*$t);
   $Y1 += 0.00000702576*cos(0.18268310766 + 6283.3196674749*$t);
   $Y1 += 0.00000430422*cos(4.02296125150 + 6282.83203250789*$t);
   $Y1 += 0.00000212689*cos(0.16300557365 + 6279.7965491259*$t);
   $Y1 += 0.00000212524*cos(3.34405166686 + 6286.84278582391*$t);
   $Y1 += 0.00000062308*cos(5.07432195748 + 4705.9761250271*$t);
   $Y1 += 0.00000059808*cos(2.24108323998 + 6257.0213476751*$t);
   $Y1 += 0.00000059474*cos(1.26580020213 + 6309.61798727471*$t);
   $Y1 += 0.00000048914*cos(3.64253038225 + 775.7664288075*$t);
   $Y1 += 0.00000042814*cos(5.15102931682 + 1059.6257476727*$t);
   $Y1 += 0.00000046457*cos(4.72328803502 + 7860.6632099227*$t);
   $Y1 += 0.00000036653*cos(0.62556081234 + 5753.6287023803*$t);
   $Y1 += 0.00000035649*cos(6.15990909241 + 5885.1706640667*$t);
   $Y1 += 0.00000035362*cos(2.90135814576 + 6813.0106325695*$t);
   $Y1 += 0.00000032151*cos(3.62597628866 + 6681.46867088311*$t);
   $Y1 += 0.00000028763*cos(4.34539356227 + 25132.5472174491*$t);
   $Y1 += 0.00000028447*cos(5.86215127530 + 6127.8992680407*$t);
   $Y1 += 0.00000027537*cos(3.92840810925 + 6438.7400669091*$t);
   $Y1 += 0.00000024819*cos(1.35151106278 + 5487.0216606585*$t);
   $Y1 += 0.00000020615*cos(2.14741827387 + 7079.61767429131*$t);
   $Y1 += 0.00000019621*cos(1.31571192713 + 5507.7970561509*$t);
   $Y1 += 0.00000018378*cos(6.17760240913 + 11790.8729061423*$t);
   $Y1 += 0.00000016494*cos(4.64463821908 + 11507.0135872771*$t);
   $Y1 += 0.00000016756*cos(2.24858801378 + 7058.8422787989*$t);
   $Y1 += 0.00000014558*cos(4.40450472214 + 6290.4332144757*$t);
   $Y1 += 0.00000014404*cos(5.39361420229 + 6276.2061204741*$t);
   $Y1 += 0.00000014052*cos(6.16986019111 + 796.5418242999*$t);
   $Y1 += 0.00000012261*cos(5.72868820428 + 4693.75913722409*$t);
   $Y1 += 0.00000012758*cos(5.06070649359 + 7.3573644843*$t);
   $Y1 += 0.00000010086*cos(1.72162917776 + 3739.0052475915*$t);
   $Y1 += 0.00000009469*cos(3.06840825171 + 6070.0205720369*$t);
   $Y1 += 0.00000010425*cos(0.81162914438 + 6284.2999885431*$t);
   $Y1 += 0.00000010425*cos(2.69531177468 + 6282.3393464067*$t);
   $Y1 += 0.00000009570*cos(0.32347467387 + 4137.1542509997*$t);
   $Y1 += 0.00000009044*cos(0.43893334417 + 6496.6187629129*$t);
   $Y1 += 0.00000008544*cos(4.76970151659 + 1194.6908277081*$t);
   $Y1 += 0.00000008214*cos(5.96640392040 + 1589.3167127673*$t);
   $Y1 += 0.00000006157*cos(1.78596836636 + 8827.6340873583*$t);
   $Y1 += 0.00000006271*cos(3.18396187423 + 8429.4850839501*$t);
   $Y1 += 0.00000005524*cos(3.79861381511 + 4933.4522578161*$t);
   $Y1 += 0.00000007314*cos(5.20341424492 + 4535.3032544079*$t);
   $Y1 += 0.00000005157*cos(3.57885530448 + 11770.0975106499*$t);
   $Y1 += 0.00000006749*cos(2.75433387783 + 5088.8726572503*$t);
   $Y1 += 0.00000004853*cos(5.33701721257 + 6040.5910635009*$t);
   $Y1 += 0.00000005304*cos(0.93005422139 + 3154.4432674121*$t);
   $Y1 += 0.00000004985*cos(3.05697984231 + 12569.91863581531*$t);
   $Y1 += 0.00000004709*cos(2.39306975468 + 5331.6012612243*$t);
   $Y1 += 0.00000004601*cos(4.45296491447 + 6526.04827144891*$t);
   $Y1 += 0.00000004201*cos(5.99364092036 + 7633.1870771337*$t);
   $Y1 += 0.00000005607*cos(0.98720876729 + 5729.7502646325*$t);
   $Y1 += 0.00000004257*cos(2.55370490945 + 3930.4535137031*$t);
   $Y1 += 0.00000004394*cos(4.75948183378 + 12559.2819704655*$t);
   $Y1 += 0.00000003870*cos(1.11601490340 + 7235.0380737255*$t);
   $Y1 += 0.00000005154*cos(4.58850581748 + 8031.3360805419*$t);
   $Y1 += 0.00000005036*cos(2.51929382179 + 6836.8890703173*$t);
   $Y1 += 0.00000005076*cos(0.73526458069 + 7477.7666776995*$t);
   $Y1 += 0.00000003593*cos(3.97650177033 + 12565.9078824993*$t);
   $Y1 += 0.00000003384*cos(3.83274545225 + 10977.32262218251*$t);
   $Y1 += 0.00000003371*cos(5.00200001284 + 11371.9485072417*$t);
   $Y1 += 0.00000003536*cos(5.30432752281 + 4164.0681721295*$t);
   $Y1 += 0.00000004292*cos(3.36706453434 + 1592.8398311163*$t);
   $Y1 += 0.00000003407*cos(0.83519640650 + 3128.6325825793*$t);
   $Y1 += 0.00000003152*cos(0.16276893009 + 5223.93773728571*$t);
   $Y1 += 0.00000002883*cos(0.20838349180 + 1747.7725955835*$t);
   $Y1 += 0.00000002751*cos(3.67369812017 + 7342.7015976641*$t);
   $Y1 += 0.00000003323*cos(3.48959648614 + 801.5771136403*$t);
   $Y1 += 0.00000002999*cos(0.68275608544 + 8636.18582124671*$t);
   $Y1 += 0.00000002496*cos(5.09157906387 + 2145.92159899169*$t);
   $Y1 += 0.00000003358*cos(5.82350976219 + 155.17658195069*$t);
   $Y1 += 0.00000002297*cos(2.96479731544 + 17260.3984721739*$t);
   $Y1 += 0.00000002047*cos(1.96580631113 + 1990.9888345245*$t);
   $Y1 += 0.00000002778*cos(6.17391326343 + 5481.4987363511*$t);
   $Y1 += 0.00000002025*cos(3.01082711135 + 951.4745887671*$t);
   $Y1 += 0.00000002663*cos(1.00159277919 + 26.0545023163*$t);
   $Y1 += 0.00000002009*cos(4.16311202386 + 4690.7236538421*$t);
   $Y1 += 0.00000002148*cos(0.32608511593 + 537.0483295789*$t);
   $Y1 += 0.00000001853*cos(4.41841662668 + 553.32558535889*$t);
   $Y1 += 0.00000002478*cos(1.61013003664 + 1349.62359217529*$t);
   $Y1 += 0.00000002023*cos(0.82015419806 + 522.8212355773*$t);
   $Y1 += 0.00000001735*cos(4.49024043092 + 529.44714761109*$t);
   $Y1 += 0.00000002417*cos(3.59693248271 + 7085.1405985987*$t);
   $Y1 += 0.00000001809*cos(2.35536809783 + 397.9051859247*$t);
   $Y1 += 0.00000001893*cos(2.65666986732 + 9438.0067523705*$t);
   $Y1 += 0.00000001708*cos(5.00361322564 + 10989.0519750185*$t);
   $Y1 += 0.00000001898*cos(3.91087003459 + 5216.33655531789*$t);
   $Y1 += 0.00000001820*cos(3.44808607160 + 5230.5636493195*$t);
   $Y1 += 0.00000001541*cos(5.81316348536 + 426.8420083595*$t);
   $Y1 += 0.00000001555*cos(2.09924495059 + 13096.0864825609*$t);
   $Y1 += 0.00000001779*cos(0.23737949225 + 12562.8723991173*$t);
   $Y1 += 0.00000001448*cos(1.75844475078 + 5753.14106741329*$t);
   $Y1 += 0.00000001450*cos(5.71201368260 + 6262.96434807611*$t);
   $Y1 += 0.00000001450*cos(4.07811254364 + 6303.6749868737*$t);
   $Y1 += 0.00000001822*cos(0.13446699374 + 10973.7995038335*$t);
   $Y1 += 0.00000001372*cos(5.60520656116 + 7875.91568110771*$t);
   $Y1 += 0.00000001375*cos(2.34041210234 + 7084.6529636317*$t);
   $Y1 += 0.00000001400*cos(3.28144398068 + 12721.8159169005*$t);
   $Y1 += 0.00000001247*cos(0.29225580262 + 2119.00767786191*$t);
   $Y1 += 0.00000001231*cos(2.53164637932 + 18319.7804023631*$t);
   $Y1 += 0.00000001141*cos(1.92161882496 + 1066.7392946735*$t);
   $Y1 += 0.00000001138*cos(1.23855949465 + 2942.7072407751*$t);
   $Y1 += 0.00000001047*cos(1.31663136695 + 5642.93474619389*$t);
   $Y1 += 0.00000001058*cos(2.19001683016 + 242.97242145751*$t);
   $Y1 += 0.00000001025*cos(0.60899466577 + 10447.14402212089*$t);
   $Y1 += 0.00000000973*cos(4.95398300734 + 640.1411037975*$t);
   $Y1 += 0.00000001114*cos(4.36978275435 + 15721.0826023619*$t);
   $Y1 += 0.00000000963*cos(0.56100772029 + 2389.1378379327*$t);
   $Y1 += 0.00000000941*cos(4.21626686326 + 20426.8149099055*$t);
   $Y1 += 0.00000001068*cos(5.08632600308 + 529.9347825781*$t);
   $Y1 += 0.00000000929*cos(1.54055499699 + 10575.6505004253*$t);
   $Y1 += 0.00000000899*cos(5.79114005938 + 16496.6052136859*$t);
   $Y1 += 0.00000000889*cos(5.01192484889 + 6277.7967431675*$t);
   $Y1 += 0.00000000889*cos(4.77820137734 + 6288.8425917823*$t);
   $Y1 += 0.00000001018*cos(2.66299766067 + 12540.0971976665*$t);
   $Y1 += 0.00000001060*cos(5.76865363011 + 5760.7422493811*$t);
   $Y1 += 0.00000001017*cos(4.02093017183 + 6805.89708556871*$t);
   $Y1 += 0.00000000797*cos(3.79336643920 + 14314.4119305333*$t);
   $Y1 += 0.00000000795*cos(0.10742263482 + 6286.6060248927*$t);
   $Y1 += 0.00000000795*cos(3.39951828424 + 6280.0333100571*$t);
   $Y1 += 0.00000000800*cos(3.18366130818 + 12029.5910053709*$t);
   $Y1 += 0.00000000719*cos(2.29666441302 + 9623.9320941747*$t);
   $Y1 += 0.00000000698*cos(4.94363251872 + 6148.2545874395*$t);
   $Y1 += 0.00000000686*cos(3.79368723538 + 5856.7214765989*$t);
   $Y1 += 0.00000000673*cos(3.99105175160 + 12964.5445208745*$t);
   $Y1 += 0.00000000902*cos(0.69698416448 + 13368.2164485901*$t);
   $Y1 += 0.00000000678*cos(4.84647193821 + 6418.38474751031*$t);
   $Y1 += 0.00000000671*cos(6.00179402654 + 6709.91785835091*$t);
   $Y1 += 0.00000000744*cos(2.68741072621 + 12043.8180993725*$t);
   $Y1 += 0.00000000706*cos(2.51950741945 + 16730.70750707931*$t);
   $Y1 += 0.00000000661*cos(2.42216375426 + 14712.5609339415*$t);
   $Y1 += 0.00000000633*cos(5.58153853935 + 4292.5746504339*$t);
   $Y1 += 0.00000000621*cos(1.74121096386 + 13119.9649203087*$t);
   $Y1 += 0.00000000638*cos(2.04188798982 + 4690.23601887509*$t);
   $Y1 += 0.00000000625*cos(0.31613419014 + 13518.1139237169*$t);
   $Y1 += 0.00000000637*cos(4.59654254753 + 5746.5151553795*$t);
   $Y1 += 0.00000000628*cos(3.42448659886 + 4686.6455902233*$t);
   $Y1 += 0.00000000653*cos(5.85737456207 + 3929.96587873609*$t);
   $Y1 += 0.00000000578*cos(0.62377454326 + 5088.3850222833*$t);
   $Y1 += 0.00000000563*cos(3.24964920260 + 10447.6316570879*$t);
   $Y1 += 0.00000000610*cos(5.19955844918 + 6820.1241795703*$t);
   $Y1 += 0.00000000666*cos(4.87089013467 + 3634.37720703489*$t);
   $Y1 += 0.00000000720*cos(5.18811029494 + 13916.2629271251*$t);
   $Y1 += 0.00000000620*cos(4.30310535016 + 31415.6230674405*$t);
   $Y1 += 0.00000000553*cos(2.71699575652 + 6259.44122972711*$t);
   $Y1 += 0.00000000553*cos(0.78994516254 + 6307.1981052227*$t);
   $Y1 += 0.00000000523*cos(5.47007776670 + 14143.7390599141*$t);
   $Y1 += 0.00000000524*cos(3.97726441461 + 12139.7973265903*$t);
   $Y1 += 0.00000000569*cos(4.83897139720 + 12036.7045523717*$t);
   $Y1 += 0.00000000514*cos(3.87608366777 + 4700.87268422489*$t);
   $Y1 += 0.00000000563*cos(3.78589343112 + 11014.8626598513*$t);
   $Y1 += 0.00000000495*cos(2.00987402269 + 13362.6935242827*$t);
   $Y1 += 0.00000000481*cos(0.93576028071 + 6279.4383321169*$t);
   $Y1 += 0.00000000481*cos(2.57118063835 + 6287.2010028329*$t);
   $Y1 += 0.00000000466*cos(2.94306081142 + 10177.5014970171*$t);
   $Y1 += 0.00000000440*cos(5.16955709579 + 1577.5873599313*$t);
   $Y1 += 0.00000000437*cos(1.61052953646 + 11499.9000402763*$t);
   $Y1 += 0.00000000452*cos(3.98330238308 + 12573.5090644671*$t);
   $Y1 += 0.00000000499*cos(5.79985868924 + 12410.9751180321*$t);
   $Y1 += 0.00000000440*cos(0.57972339309 + 24073.1652872599*$t);
   $Y1 += 0.00000000587*cos(0.00678196141 + 6.86972951729*$t);
   $Y1 += 0.00000000388*cos(3.61198235279 + 7860.17557495569*$t);
   $Y1 += 0.00000000398*cos(4.20068362004 + 8274.0646845159*$t);
   $Y1 += 0.00000000423*cos(5.87838349624 + 4694.2467721911*$t);
   $Y1 += 0.00000000530*cos(1.43593166640 + 12592.6938372661*$t);
   $Y1 += 0.00000000379*cos(0.45309221604 + 6180.2268932563*$t);
   $Y1 += 0.00000000379*cos(3.05384870302 + 6386.4124416935*$t);
   $Y1 += 0.00000000377*cos(4.33272540326 + 7872.39256275871*$t);
   $Y1 += 0.00000000419*cos(4.05140241721 + 5327.7199258663*$t);
   $Y1 += 0.00000000383*cos(4.67353188247 + 6247.2918007975*$t);
   $Y1 += 0.00000000383*cos(5.11659434377 + 6319.34753415231*$t);
   $Y1 += 0.00000000375*cos(0.91076946484 + 2352.6223362883*$t);
   $Y1 += 0.00000000457*cos(2.62120808067 + 6077.1341190377*$t);
   $Y1 += 0.00000000457*cos(0.88573283839 + 6489.5052159121*$t);
   $Y1 += 0.00000000481*cos(3.46562904756 + 4292.0870154669*$t);
   $Y1 += 0.00000000344*cos(4.99906947930 + 18451.3223640495*$t);
   $Y1 += 0.00000000453*cos(5.43816135018 + 2787.2868413409*$t);
   $Y1 += 0.00000000364*cos(2.82703012884 + 6245.2919948391*$t);
   $Y1 += 0.00000000364*cos(0.67991079022 + 6321.3473401107*$t);
   $Y1 += 0.00000000327*cos(1.14128906507 + 12323.6669134923*$t);
   $Y1 += 0.00000000358*cos(0.15971399837 + 9917.45305702629*$t);
   $Y1 += 0.00000000342*cos(5.26072614817 + 6262.5442719825*$t);
   $Y1 += 0.00000000342*cos(4.52940007806 + 6304.0950629673*$t);
   $Y1 += 0.00000000318*cos(4.84603466410 + 11926.4982311523*$t);
   $Y1 += 0.00000000320*cos(1.13215851865 + 77714.01528560399*$t);
   $Y1 += 0.00000000354*cos(5.73706716445 + 7238.9194090835*$t);
   $Y1 += 0.00000000318*cos(3.75423383039 + 6254.8704800071*$t);
   $Y1 += 0.00000000318*cos(6.03589239585 + 6311.7688549427*$t);
   $Y1 += 0.00000000304*cos(0.43890915611 + 12779.6946129043*$t);
   $Y1 += 0.00000000343*cos(2.14223194635 + 6298.57213865991*$t);
   $Y1 += 0.00000000343*cos(1.36470897271 + 6268.0671962899*$t);
   $Y1 += 0.00000000359*cos(2.52875634099 + 1052.51220067191*$t);
   $Y1 += 0.00000000398*cos(4.83537015899 + 1551.2890401315*$t);
   $Y1 += 0.00000000306*cos(1.01793913946 + 5863.8350235997*$t);
   $Y1 += 0.00000000300*cos(0.20529153395 + 90280.16698558679*$t);
   $Y1 += 0.00000000307*cos(4.86343200722 + 3893.93801205869*$t);
   $Y1 += 0.00000000297*cos(2.08300781298 + 5429.63565075589*$t);
   $Y1 += 0.00000000365*cos(1.17462915798 + 10022.0810975829*$t);
   $Y1 += 0.00000000336*cos(1.15285276926 + 17782.9758902677*$t);
   $Y1 += 0.00000000295*cos(2.48770580169 + 6702.8043113501*$t);
   $Y1 += 0.00000000288*cos(0.61716195626 + 18073.9487561337*$t);
   $Y1 += 0.00000000287*cos(4.90856628789 + 1577.0997249643*$t);
   $Y1 += 0.00000000378*cos(5.11185072605 + 2353.1099712553*$t);
   $Y1 += 0.00000000289*cos(1.47281714617 + 213.5429129215*$t);
   $Y1 += 0.00000000347*cos(2.29918133719 + 5223.4501023187*$t);
   $Y1 += 0.00000000321*cos(0.66708336426 + 17797.2029842693*$t);
   $Y1 += 0.00000000289*cos(3.73571084362 + 220.6564599223*$t);
   $Y1 += 0.00000000260*cos(2.61364471837 + 955.3559241251*$t);
   $Y1 += 0.00000000268*cos(5.19927474541 + 14945.0723560709*$t);
   $Y1 += 0.00000000242*cos(2.77307042091 + 2544.5582373669*$t);
   $Y1 += 0.00000000245*cos(3.88798752568 + 7632.6994421667*$t);
   $Y1 += 0.00000000234*cos(2.50313805280 + 1596.43025976811*$t);
   $Y1 += 0.00000000267*cos(4.25976324066 + 206.42936592071*$t);
   $Y1 += 0.00000000282*cos(2.93181023342 + 13341.9181287903*$t);
   $Y1 += 0.00000000279*cos(4.70853062526 + 4731.7868098599*$t);
   $Y1 += 0.00000000290*cos(0.46137131163 + 5642.4420600927*$t);
   $Y1 += 0.00000000298*cos(5.65061835836 + 17790.08943726851*$t);
   $Y1 += 0.00000000270*cos(0.59449456012 + 12168.2465140581*$t);
   $Y1 += 0.00000000294*cos(0.95816279728 + 2146.4092339587*$t);
   $Y1 += 0.00000000214*cos(2.50623255971 + 8030.8484455749*$t);
   $Y1 += 0.00000000207*cos(3.18045634687 + 213.0552779545*$t);
   $Y1 += 0.00000000216*cos(4.03237077499 + 3185.43584474911*$t);
   $Y1 += 0.00000000195*cos(4.44222765867 + 5884.68302909969*$t);
   $Y1 += 0.00000000199*cos(2.36156558922 + 1748.2602305505*$t);
   $Y1 += 0.00000000250*cos(3.04516390608 + 6924.1972748571*$t);
   $Y1 += 0.00000000194*cos(0.79669370564 + 641.12142486571*$t);
   $Y1 += 0.00000000203*cos(0.68276896116 + 6503.7323099137*$t);
   $Y1 += 0.00000000203*cos(2.82417195790 + 6062.9070250361*$t);
   $Y1 += 0.00000000258*cos(0.91786109532 + 796.0541893329*$t);
   $Y1 += 0.00000000189*cos(3.20457516516 + 11506.52595231009*$t);
   $Y1 += 0.00000000186*cos(4.84188314285 + 18209.5740811437*$t);
   $Y1 += 0.00000000184*cos(2.63875516800 + 12566.4628277691*$t);
   $Y1 += 0.00000000180*cos(4.19729049815 + 853.4401992355*$t);
   $Y1 += 0.00000000185*cos(6.25586281827 + 3495.78900865049*$t);
   $Y1 += 0.00000000205*cos(0.22424585668 + 5849.1202946311*$t);
   $Y1 += 0.00000000187*cos(4.34056279496 + 10213.5293636945*$t);
   $Y1 += 0.00000000187*cos(0.48606035843 + 6290.36590417291*$t);
   $Y1 += 0.00000000187*cos(3.02088056063 + 6276.2734307769*$t);
   $Y1 += 0.00000000235*cos(4.34532332705 + 9779.3524936089*$t);
   $Y1 += 0.00000000173*cos(6.09941552473 + 19651.2922985815*$t);
   $Y1 += 0.00000000184*cos(3.08285813784 + 12566.32820716351*$t);
   $Y1 += 0.00000000230*cos(0.66694562429 + 12567.37583853451*$t);
   $Y1 += 0.00000000166*cos(6.03836479949 + 110.45013870291*$t);
   $Y1 += 0.00000000189*cos(1.27307578752 + 3.76693583251*$t);
   $Y1 += 0.00000000166*cos(5.90720968173 + 433.9555553603*$t);
   $Y1 += 0.00000000186*cos(6.04048637618 + 5863.3473886327*$t);
   $Y1 += 0.00000000184*cos(4.58711235161 + 10983.94853421629*$t);
   $Y1 += 0.00000000184*cos(1.80970242494 + 6037.4880212455*$t);
   $Y1 += 0.00000000156*cos(0.77353737613 + 2648.6986429565*$t);
   $Y1 += 0.00000000182*cos(5.04351148530 + 10969.72144021469*$t);
   $Y1 += 0.00000000175*cos(1.69691915551 + 6529.1513137043*$t);
   $Y1 += 0.00000000154*cos(4.10646056877 + 6453.9925380941*$t);
   $Y1 += 0.00000000154*cos(5.68366565747 + 6112.6467968557*$t);
   $Y1 += 0.00000000164*cos(2.95193833305 + 12353.0964220283*$t);
   $Y1 += 0.00000000146*cos(2.48650028218 + 149.8070146181*$t);
   $Y1 += 0.00000000184*cos(4.91752478989 + 11015.3502948183*$t);
   $Y1 += 0.00000000191*cos(0.72322692096 + 3894.4256470257*$t);
   $Y1 += 0.00000000144*cos(2.19099222002 + 18636.1722720197*$t);
   $Y1 += 0.00000000146*cos(0.42084749575 + 13760.8425276909*$t);
   $Y1 += 0.00000000142*cos(2.91645242674 + 4156.95462512869*$t);
   $Y1 += 0.00000000185*cos(5.29181252606 + 7234.5504387585*$t);
   $Y1 += 0.00000000133*cos(4.73063574833 + 18139.5383188994*$t);
   $Y1 += 0.00000000131*cos(0.04654287508 + 5507.3094211839*$t);
   $Y1 += 0.00000000146*cos(0.32618788162 + 18875.2820522905*$t);
   $Y1 += 0.00000000128*cos(4.98427830485 + 6069.53293706989*$t);
   $Y1 += 0.00000000128*cos(1.75166251045 + 16200.52890701769*$t);
   $Y1 += 0.00000000123*cos(0.73895960068 + 20.5315780089*$t);
   $Y1 += 0.00000000121*cos(5.69434162626 + 17256.8753538249*$t);
   $Y1 += 0.00000000123*cos(4.81514598285 + 6393.5259886943*$t);
   $Y1 += 0.00000000123*cos(4.97498024339 + 6173.1133462555*$t);
   $Y1 += 0.00000000117*cos(2.90632080371 + 18852.9944858067*$t);
   $Y1 += 0.00000000120*cos(5.75152020178 + 9381.2034902007*$t);
   $Y1 += 0.00000000113*cos(3.90786027747 + 65147.37595065419*$t);
   $Y1 += 0.00000000137*cos(2.11120201406 + 16201.0165419847*$t);
   $Y1 += 0.00000000113*cos(0.36200887356 + 6836.40143535029*$t);
   $Y1 += 0.00000000123*cos(2.85759760823 + 12565.4151963981*$t);
   $Y1 += 0.00000000132*cos(4.51334720616 + 17655.0243572331*$t);
   $Y1 += 0.00000000113*cos(5.69373873614 + 10575.1628654583*$t);
   $Y1 += 0.00000000113*cos(3.66776032820 + 10213.0417287275*$t);
   $Y1 += 0.00000000116*cos(0.66749075538 + 17253.2849251731*$t);
   $Y1 += 0.00000000120*cos(0.70954573147 + 10420.23010099111*$t);
   $Y1 += 0.00000000109*cos(5.21836603318 + 633.0275567967*$t);
   $Y1 += 0.00000000103*cos(4.83071292354 + 6295.0490203109*$t);
   $Y1 += 0.00000000103*cos(4.95941330270 + 6271.5903146389*$t);
   $Y1 += 0.00000000104*cos(4.01023132524 + 23013.7833570707*$t);
   $Y1 += 0.00000000105*cos(5.32684825848 + 11790.3852711753*$t);
   $Y1 += 0.00000000109*cos(4.22545370482 + 3340.36860921629*$t);
   $Y1 += 0.00000000102*cos(1.02602291037 + 10970.2090751817*$t);
   $Y1 += 0.00000000101*cos(5.99270341494 + 4803.9654584435*$t);
   $Y1 += 0.00000000114*cos(3.78830179341 + 4171.1817191303*$t);
   $Y1 += 0.00000000111*cos(2.09152457722 + 1582.2031657665*$t);
   $Y1 += 0.00000000107*cos(0.90140521385 + 23543.4743221653*$t);
   $Y1 += 0.00000000099*cos(4.24784832079 + 10973.3118688665*$t);
   $Y1 += 0.00000000102*cos(4.09962394648 + 6549.9267091967*$t);
   $Y1 += 0.00000000102*cos(5.69050227976 + 6016.7126257531*$t);
   $Y1 += 0.00000000120*cos(0.92616350319 + 11514.1271342779*$t);
   $Y1 += 0.00000000095*cos(1.12824235931 + 17267.5120191747*$t);
   $Y1 += 0.00000000121*cos(4.76575826483 + 13625.7774476555*$t);
   $Y1 += 0.00000000100*cos(3.61435033946 + 10976.83498721549*$t);
   $Y1 += 0.00000000090*cos(2.11761227033 + 76.50988875911*$t);
   $Y1 += 0.00000000100*cos(5.94744998603 + 8661.9965060795*$t);
   $Y1 += 0.00000000089*cos(3.76175372959 + 1350.1112271423*$t);
   $Y1 += 0.00000000087*cos(2.76310699450 + 9917.9406919933*$t);
   $Y1 += 0.00000000086*cos(4.36870119289 + 12809.12412144031*$t);
   $Y1 += 0.00000000095*cos(6.05597457730 + 12345.9828750275*$t);
   $Y1 += 0.00000000084*cos(3.11610500480 + 775.2787938405*$t);
   $Y1 += 0.00000000106*cos(3.57201004990 + 11216.5281078075*$t);
   $Y1 += 0.00000000095*cos(0.92966555257 + 12012.8261146239*$t);
   $Y1 += 0.00000000082*cos(4.72839816349 + 5643.4223811609*$t);
   $Y1 += 0.00000000081*cos(2.27692529332 + 11614.6771112157*$t);
   $Y1 += 0.00000000084*cos(5.85994160066 + 6255.9181113781*$t);
   $Y1 += 0.00000000084*cos(3.93018462558 + 6310.7212235717*$t);
   $Y1 += 0.00000000080*cos(5.05950625443 + 6923.2169537889*$t);
   $Y1 += 0.00000000085*cos(1.08248438849 + 5635.8211991931*$t);
   $Y1 += 0.00000000084*cos(4.61502942146 + 10440.03047512009*$t);
   $Y1 += 0.00000000080*cos(2.62622746074 + 3583.5848481573*$t);
   $Y1 += 0.00000000092*cos(5.59171891913 + 10818.3791043993*$t);
   $Y1 += 0.00000000077*cos(4.13228553675 + 6993.2527160332*$t);
   $Y1 += 0.00000000080*cos(6.08197788088 + 13521.9952590749*$t);
   $Y1 += 0.00000000104*cos(0.70806546737 + 5650.0482931947*$t);
   $Y1 += 0.00000000093*cos(5.00569753414 + 4732.2744448269*$t);
   $Y1 += 0.00000000085*cos(1.74569264442 + 22805.4917485101*$t);
   $Y1 += 0.00000000086*cos(5.52511956951 + 12360.2099690291*$t);
   $Y1 += 0.00000000075*cos(5.57236272383 + 5120.35732810009*$t);
   $Y1 += 0.00000000090*cos(5.33797857569 + 6370.62787201471*$t);
   $Y1 += 0.00000000090*cos(4.45214765055 + 6196.0114629351*$t);
   $Y1 += 0.00000000082*cos(4.91006846138 + 18842.3578204569*$t);
   $Y1 += 0.00000000101*cos(1.37756832018 + 3097.64000524229*$t);
   $Y1 += 0.00000000094*cos(0.74657649932 + 10177.01386205009*$t);
   $Y1 += 0.00000000083*cos(3.74913323692 + 18415.7596295809*$t);
   $Y1 += 0.00000000073*cos(4.52746639645 + 949.4194264533*$t);
   $Y1 += 0.00000000101*cos(4.43327916798 + 3104.6862419403*$t);
   $Y1 += 0.00000000074*cos(1.25396392931 + 13517.62628874989*$t);
   $Y1 += 0.00000000092*cos(0.77959714125 + 16858.7263504167*$t);
   $Y1 += 0.00000000075*cos(1.57278335709 + 1059.1381127057*$t);
   $Y1 += 0.00000000085*cos(4.52163234616 + 398.3928208917*$t);
   $Y1 += 0.00000000069*cos(5.60384680292 + 11713.1991357143*$t);
   $Y1 += 0.00000000084*cos(5.92557534615 + 2378.9206560881*$t);
   $Y1 += 0.00000000069*cos(2.18342250812 + 11712.71150074729*$t);
   $Y1 += 0.00000000075*cos(0.05753316454 + 30356.2411372513*$t);
   $Y1 += 0.00000000067*cos(3.04134731899 + 3154.9309023791*$t);
   $Y1 += 0.00000000076*cos(3.26401364210 + 18429.9867235825*$t);
   $Y1 += 0.00000000068*cos(2.69185508250 + 11925.5179100841*$t);
   $Y1 += 0.00000000067*cos(4.81689271569 + 10454.25756912169*$t);
   $Y1 += 0.00000000073*cos(1.35808826471 + 15670.83794192309*$t);
   $Y1 += 0.00000000065*cos(1.77865763794 + 6438.2524319421*$t);
   $Y1 += 0.00000000069*cos(1.72420196911 + 17298.4261448097*$t);
   $Y1 += 0.00000000087*cos(3.52675260029 + 3904.1551939033*$t);
   $Y1 += 0.00000000076*cos(5.44623063701 + 5231.0512842865*$t);
   $Y1 += 0.00000000064*cos(2.12314868459 + 3496.2766436175*$t);
   $Y1 += 0.00000000078*cos(2.74559935849 + 8672.21368792411*$t);
   $Y1 += 0.00000000066*cos(4.98895923052 + 14143.2514249471*$t);
   $Y1 += 0.00000000063*cos(5.70776457338 + 24357.0246061251*$t);
   $Y1 += 0.00000000063*cos(0.57965828598 + 15720.5949673949*$t);
   $Y1 += 0.00000000061*cos(2.15619405920 + 16460.5773470085*$t);
   $Y1 += 0.00000000073*cos(6.07488391156 + 9387.76209193169*$t);
   $Y1 += 0.00000000057*cos(4.37179192959 + 17259.91083720689*$t);
   $Y1 += 0.00000000059*cos(2.26511827996 + 9778.8648586419*$t);
   $Y1 += 0.00000000058*cos(1.67874293827 + 155.6642169177*$t);
   $Y1 += 0.00000000060*cos(1.94469304669 + 34570.3101523361*$t);
   $Y1 += 0.00000000057*cos(2.88994977642 + 149854.64395229139*$t);
   $Y1 += 0.00000000056*cos(3.76314176838 + 12456.1891962469*$t);
   $Y1 += 0.00000000056*cos(5.62479142768 + 17996.2749857057*$t);
   $Y1 += 0.00000000056*cos(0.60886736064 + 11764.5745863425*$t);
   $Y1 += 0.00000000054*cos(4.53224684011 + 4705.4884900601*$t);
   $Y1 += 0.00000000055*cos(6.11029124003 + 13915.77529215809*$t);
   $Y1 += 0.00000000061*cos(4.86430679802 + 28237.4772768729*$t);
   $Y1 += 0.00000000065*cos(4.29965528550 + 7335.5880506633*$t);
   $Y1 += 0.00000000072*cos(5.48795829629 + 6055.8435346859*$t);
   $Y1 += 0.00000000072*cos(4.30216792995 + 6510.7958002639*$t);
   $Y1 += 0.00000000071*cos(4.94322519248 + 12545.6201219739*$t);
   $Y1 += 0.00000000061*cos(5.98387073766 + 6312.74917601091*$t);
   $Y1 += 0.00000000061*cos(3.80625548858 + 6253.8901589389*$t);
   $Y1 += 0.00000000052*cos(1.83719780245 + 5326.5428765373*$t);
   $Y1 += 0.00000000054*cos(2.78309524301 + 2699.49100183409*$t);
   $Y1 += 0.00000000052*cos(0.87738933830 + 8983.0544867925*$t);
   $Y1 += 0.00000000066*cos(0.78258344897 + 14919.2616712381*$t);
   $Y1 += 0.00000000050*cos(1.22263365304 + 5572.8989839496*$t);
   $Y1 += 0.00000000051*cos(2.33500088089 + 16522.4158985187*$t);
   $Y1 += 0.00000000049*cos(2.61658441943 + 35579.9350570535*$t);
   $Y1 += 0.00000000053*cos(6.07542895582 + 6208.5380689076*$t);
   $Y1 += 0.00000000053*cos(3.71469727042 + 6358.1012660422*$t);
   $Y1 += 0.00000000049*cos(3.35593265214 + 21393.7857873411*$t);
   $Y1 += 0.00000000049*cos(4.47248238697 + 5547.4431539431*$t);
   $Y1 += 0.00000000049*cos(5.31764383927 + 7019.19618100671*$t);
   $Y1 += 0.00000000049*cos(1.02044068888 + 12416.8323203317*$t);
   $Y1 += 0.00000000050*cos(0.93918602686 + 12592.2062022991*$t);
   $Y1 += 0.00000000050*cos(4.78759248733 + 26084.2656236997*$t);
   $Y1 += 0.00000000051*cos(1.83601012538 + 23006.6698100699*$t);
   $Y1 += 0.00000000058*cos(4.03623415117 + 15141.14697682849*$t);
   $Y1 += 0.00000000047*cos(5.68903994773 + 24279.3508356971*$t);
   $Y1 += 0.00000000049*cos(3.40926939847 + 6816.53375091851*$t);
   $Y1 += 0.00000000049*cos(0.09767152059 + 5750.1055840313*$t);
   $Y1 += 0.00000000046*cos(1.88054905125 + 19379.1623325523*$t);
   $Y1 += 0.00000000052*cos(5.74808775798 + 16737.33341911309*$t);
   $Y1 += 0.00000000047*cos(0.52219046486 + 24066.0517402591*$t);
   $Y1 += 0.00000000046*cos(4.16717022039 + 104351.85638378029*$t);
   $Y1 += 0.00000000063*cos(6.25484174196 + 8662.48414104651*$t);
   $Y1 += 0.00000000048*cos(3.28503810088 + 26735.7014447297*$t);
   $Y1 += 0.00000000046*cos(3.91775350902 + 12132.6837795895*$t);
   $Y1 += 0.00000000050*cos(2.92364175862 + 24602.8562523545*$t);
   $Y1 += 0.00000000054*cos(4.75214039499 + 7834.3648901229*$t);
   $Y1 += 0.00000000045*cos(3.48807574467 + 161000.92955515759*$t);
   $Y1 += 0.00000000049*cos(2.94405201174 + 29303.9727540629*$t);
   $Y1 += 0.00000000049*cos(1.43156848803 + 10984.4361691833*$t);
   $Y1 += 0.00000000045*cos(0.70883599249 + 45584.9289947039*$t);
   $Y1 += 0.00000000046*cos(1.86995211145 + 5244.2930566845*$t);
   $Y1 += 0.00000000048*cos(3.53510882628 + 23581.0143598341*$t);
   $Y1 += 0.00000000049*cos(2.17177379198 + 19650.8046636145*$t);
   $Y1 += 0.00000000049*cos(3.41897587826 + 29289.7456600613*$t);
   $Y1 += 0.00000000044*cos(1.44008987426 + 6390.9831914135*$t);
   $Y1 += 0.00000000044*cos(2.06685104480 + 6175.6561435363*$t);
   $Y1 += 0.00000000051*cos(2.65580143245 + 17789.60180230149*$t);
   $Y1 += 0.00000000042*cos(6.18543275273 + 24705.9490265731*$t);
   $Y1 += 0.00000000042*cos(5.90805829894 + 15663.79170522509*$t);
   $Y1 += 0.00000000043*cos(3.08250524084 + 16460.08971204149*$t);
   $Y1 += 0.00000000043*cos(0.28590581328 + 553.8132203259*$t);
   $Y1 += 0.00000000041*cos(3.21615501236 + 17370.6047933933*$t);
   $Y1 += 0.00000000053*cos(2.59151074700 + 13119.47728534169*$t);
   $Y1 += 0.00000000042*cos(5.11995884892 + 4164.5558070965*$t);
   $Y1 += 0.00000000041*cos(2.05521298551 + 23020.8969040715*$t);
   $Y1 += 0.00000000045*cos(3.70227971577 + 54247.1693182669*$t);
   $Y1 += 0.00000000040*cos(4.57473494926 + 32243.2546833971*$t);
   $Y1 += 0.00000000040*cos(5.17184328434 + 24336.2492106327*$t);
   $Y1 += 0.00000000040*cos(1.71852303243 + 11769.6098756829*$t);
   $Y1 += 0.00000000045*cos(6.25985265406 + 16723.10632511149*$t);
   $Y1 += 0.00000000045*cos(6.25985265406 + 16723.5939600785*$t);
   $Y1 += 0.00000000042*cos(5.89327075469 + 8827.14645239129*$t);
   $Y1 += 0.00000000040*cos(4.71493835645 + 8402.0835278533*$t);
   $Y1 += 0.00000000039*cos(3.49714273023 + 16062.4283436003*$t);
   $Y1 += 0.00000000044*cos(4.44510876745 + 5856.23384163189*$t);
   $Y1 += 0.00000000038*cos(1.14212165839 + 6297.5467614765*$t);
   $Y1 += 0.00000000038*cos(2.36481926067 + 6269.0925734733*$t);
   $Y1 += 0.00000000041*cos(3.38233495451 + 15508.8589407579*$t);
   $Y1 += 0.00000000038*cos(4.96796930709 + 6394.50630976251*$t);
   $Y1 += 0.00000000038*cos(4.82215691915 + 6172.1330251873*$t);
   $Y1 += 0.00000000039*cos(1.74815791619 + 29826.5501721567*$t);
   $Y1 += 0.00000000046*cos(0.31322632556 + 17297.9385098427*$t);
   $Y1 += 0.00000000041*cos(4.04315782590 + 6309.1303523077*$t);
   $Y1 += 0.00000000038*cos(5.82409895120 + 29089.0552334685*$t);
   $Y1 += 0.00000000038*cos(4.59409111885 + 11933.6117781531*$t);
   $Y1 += 0.00000000037*cos(3.34091252113 + 26482.4146271079*$t);
   $Y1 += 0.00000000037*cos(3.64722285417 + 20452.6255947383*$t);
   $Y1 += 0.00000000037*cos(0.30900035436 + 18073.46112116669*$t);
   $Y1 += 0.00000000036*cos(2.98410187235 + 11499.41240530929*$t);
   $Y1 += 0.00000000045*cos(4.75649198460 + 3340.8562441833*$t);
   $Y1 += 0.00000000049*cos(4.21088151991 + 18216.6876281445*$t);
   $Y1 += 0.00000000038*cos(4.59187061808 + 18202.4605341429*$t);
   $Y1 += 0.00000000041*cos(3.91005845267 + 5216.8241902849*$t);
   $Y1 += 0.00000000036*cos(6.21139146959 + 4060.97539791089*$t);
   $Y1 += 0.00000000048*cos(5.14345155225 + 951.9622237341*$t);
   $Y1 += 0.00000000041*cos(3.90932016585 + 18848.98373249069*$t);
   $Y1 += 0.00000000038*cos(4.60880580452 + 26709.8907598969*$t);
   $Y1 += 0.00000000036*cos(3.12521745961 + 28230.4310401749*$t);
   $Y1 += 0.00000000036*cos(1.96840086543 + 24491.18197509989*$t);
   $Y1 += 0.00000000036*cos(1.56811412304 + 9924.5666040271*$t);
   $Y1 += 0.00000000048*cos(1.89701220521 + 8982.5668518255*$t);
   $Y1 += 0.00000000037*cos(3.60669363968 + 22484.0923919761*$t);
   $Y1 += 0.00000000047*cos(3.21269518092 + 6040.10342853389*$t);
   $Y1 += 0.00000000034*cos(3.68972702028 + 6418.9449924849*$t);
   $Y1 += 0.00000000034*cos(6.10039920596 + 6147.69434246491*$t);
   $Y1 += 0.00000000034*cos(3.67007112439 + 9380.71585523369*$t);
   $Y1 += 0.00000000036*cos(0.30403746373 + 7238.4317741165*$t);
   $Y1 += 0.00000000034*cos(0.40887602826 + 17272.1278250099*$t);
   $Y1 += 0.00000000037*cos(1.23154839984 + 58953.3892607775*$t);
   $Y1 += 0.00000000034*cos(0.33427002863 + 29026.7290469913*$t);
   $Y1 += 0.00000000033*cos(3.87501941695 + 12721.32828193349*$t);
   $Y1 += 0.00000000035*cos(1.63582276543 + 7322.34627826531*$t);
   $Y1 += 0.00000000044*cos(1.21937301495 + 3981.73385156551*$t);
   $Y1 += 0.00000000033*cos(3.59821221655 + 19804.5834740993*$t);
   $Y1 += 0.00000000033*cos(2.56047890182 + 173567.08125514039*$t);
   $Y1 += 0.00000000033*cos(6.16657680956 + 205.9417309537*$t);
   $Y1 += 0.00000000033*cos(5.44889821497 + 23013.29572210369*$t);
   $Y1 += 0.00000000034*cos(2.13261854972 + 49515.1386909235*$t);
   $Y1 += 0.00000000034*cos(3.38050467777 + 38500.5198485557*$t);
   $Y1 += 0.00000000032*cos(2.81221233882 + 17686.9966630499*$t);
   $Y1 += 0.00000000036*cos(1.67423219491 + 5641.9544251257*$t);
   $Y1 += 0.00000000032*cos(2.91090812586 + 21228.63584102931*$t);
   $Y1 += 0.00000000033*cos(0.13839577785 + 34520.5531268643*$t);
   $Y1 += 0.00000000033*cos(5.65380556855 + 8635.69818627969*$t);
   $Y1 += 0.00000000031*cos(1.88042187886 + 5746.0275204125*$t);
   $Y1 += 0.00000000041*cos(2.52021697492 + 21202.3375212295*$t);
   $Y1 += 0.00000000037*cos(5.92836585991 + 7349.81514466491*$t);
   $Y1 += 0.00000000034*cos(2.18099177342 + 16496.11757871889*$t);
   $Y1 += 0.00000000032*cos(0.90143427409 + 33019.2649296881*$t);
   $Y1 += 0.00000000032*cos(1.54173979189 + 21150.56954840009*$t);
   $Y1 += 0.00000000037*cos(1.84830381005 + 29296.8592070621*$t);
   $Y1 += 0.00000000036*cos(5.02250294215 + 27511.2240560537*$t);
   $Y1 += 0.00000000035*cos(3.77934660139 + 11933.12414318609*$t);
   $Y1 += 0.00000000035*cos(4.26341483447 + 11918.89704918449*$t);
   $Y1 += 0.00000000035*cos(5.54313072444 + 29062.7569136687*$t);
   $Y1 += 0.00000000030*cos(1.38932865248 + 12490.1294461907*$t);
   $Y1 += 0.00000000032*cos(5.09927453137 + 6334.60000533731*$t);
   $Y1 += 0.00000000032*cos(4.69085169487 + 6232.03932961249*$t);
   $Y1 += 0.00000000029*cos(4.05993716780 + 419.2408263917*$t);
   $Y1 += 0.00000000029*cos(0.91834451421 + 419.72846135871*$t);
   $Y1 += 0.00000000029*cos(1.66178131503 + 16858.23871544969*$t);
   $Y1 += 0.00000000030*cos(2.73190240418 + 16208.13008898551*$t);
   $Y1 += 0.00000000029*cos(0.69175314473 + 27278.7126339243*$t);
   $Y1 += 0.00000000032*cos(4.73525892997 + 5017.2645538815*$t);
   $Y1 += 0.00000000028*cos(0.30767945102 + 24080.2788342607*$t);
   $Y1 += 0.00000000030*cos(1.00265951424 + 23539.9512038163*$t);
   $Y1 += 0.00000000029*cos(1.45883005475 + 18606.25512851669*$t);
   $Y1 += 0.00000000030*cos(4.73151177298 + 30665.9111409493*$t);
   $Y1 += 0.00000000034*cos(4.29199382621 + 6660.6932753907*$t);
   $Y1 += 0.00000000034*cos(5.49813240003 + 5905.94605955911*$t);
   $Y1 += 0.00000000038*cos(3.55143054136 + 31571.0434668747*$t);
   $Y1 += 0.00000000038*cos(4.53015131722 + 16061.94070863329*$t);
   $Y1 += 0.00000000029*cos(1.64885508302 + 23539.46356884929*$t);
   $Y1 += 0.00000000033*cos(2.88700963334 + 3738.51761262449*$t);
   $Y1 += 0.00000000028*cos(6.02264922068 + 2942.21960580809*$t);
   $Y1 += 0.00000000032*cos(2.75250951602 + 15110.7099373497*$t);
   $Y1 += 0.00000000030*cos(6.10434010761 + 55023.1795645579*$t);
   $Y1 += 0.00000000027*cos(5.24524976767 + 233141.55822184499*$t);
   $Y1 += 0.00000000030*cos(0.75326086687 + 12587.1709129587*$t);
   $Y1 += 0.00000000028*cos(3.97691648841 + 34911.6558935745*$t);
   $Y1 += 0.00000000027*cos(4.60109278395 + 1692.40948698591*$t);
   $Y1 += 0.00000000027*cos(6.13743188286 + 2544.07060239989*$t);
   $Y1 += 0.00000000027*cos(4.99580076168 + 24382.8352909579*$t);
   $Y1 += 0.00000000027*cos(4.42136306455 + 6205.6458970469*$t);
   $Y1 += 0.00000000027*cos(5.36876316169 + 6360.99343790291*$t);
   $Y1 += 0.00000000027*cos(4.95719787895 + 31172.8944634665*$t);
   $Y1 += 0.00000000026*cos(2.19655051246 + 36949.4746259077*$t);
   $Y1 += 0.00000000034*cos(4.45594644643 + 18053.1733606413*$t);
   $Y1 += 0.00000000027*cos(0.74968073356 + 32367.3414736911*$t);
   $Y1 += 0.00000000027*cos(3.34260766105 + 4487.57358878689*$t);
   $Y1 += 0.00000000028*cos(5.37411547339 + 10239.3400485273*$t);
   $Y1 += 0.00000000026*cos(1.30232025786 + 4796.85191144269*$t);
   $Y1 += 0.00000000026*cos(4.70861603764 + 6226.4212925393*$t);
   $Y1 += 0.00000000026*cos(5.08151018860 + 6340.2180424105*$t);
   $Y1 += 0.00000000029*cos(4.90026899469 + 5974.0413448191*$t);
   $Y1 += 0.00000000029*cos(4.88985723155 + 6592.5979901307*$t);
   $Y1 += 0.00000000026*cos(4.96950484222 + 19402.55313533309*$t);
   $Y1 += 0.00000000028*cos(3.99925126629 + 46386.7499258277*$t);
   $Y1 += 0.00000000032*cos(5.04462761864 + 9225.7830907665*$t);
   $Y1 += 0.00000000026*cos(4.11745014890 + 9910.33951002549*$t);
   $Y1 += 0.00000000025*cos(1.02839985963 + 15265.64270181689*$t);
   $Y1 += 0.00000000029*cos(4.33416022359 + 52176.0501006319*$t);
   $Y1 += 0.00000000027*cos(2.65258449905 + 62883.5989569971*$t);
   $Y1 += 0.00000000024*cos(5.26265995716 + 11617.21990849651*$t);
   $Y1 += 0.00000000024*cos(1.04936648816 + 5120.8449630671*$t);
   $Y1 += 0.00000000028*cos(4.82143927810 + 6680.98103591609*$t);
   $Y1 += 0.00000000024*cos(5.97769334002 + 522.3336006103*$t);
   $Y1 += 0.00000000025*cos(2.29282659231 + 6263.64990657511*$t);
   $Y1 += 0.00000000025*cos(1.21411432675 + 6302.9894283747*$t);
   $Y1 += 0.00000000029*cos(0.87325499796 + 23581.5019948011*$t);
   $Y1 += 0.00000000024*cos(5.21948189448 + 29424.8780503995*$t);
   $Y1 += 0.00000000028*cos(1.09161353868 + 16737.8210540801*$t);
   $Y1 += 0.00000000024*cos(3.55960074091 + 22743.16556203289*$t);
   $Y1 += 0.00000000024*cos(3.55960074091 + 22743.6531969999*$t);
   $Y1 += 0.00000000024*cos(4.28222488832 + 25158.8455372489*$t);
   $Y1 += 0.00000000025*cos(5.52194731109 + 21424.2228268199*$t);
   $Y1 += 0.00000000025*cos(5.52194731109 + 21424.7104617869*$t);
   $Y1 += 0.00000000024*cos(3.49943174674 + 14712.07329897449*$t);
   $Y1 += 0.00000000028*cos(6.07681616993 + 5760.25461441409*$t);
   $Y1 += 0.00000000024*cos(2.16376008059 + 31969.1924702829*$t);
   $Y1 += 0.00000000023*cos(5.33584675127 + 17893.1822114871*$t);
   $Y1 += 0.00000000030*cos(2.28483676839 + 8584.9054833843*$t);
   $Y1 += 0.00000000024*cos(1.64666673478 + 23550.5878691661*$t);
   $Y1 += 0.00000000023*cos(6.07302432574 + 6717.0314053517*$t);
   $Y1 += 0.00000000022*cos(2.42662159283 + 7445.7943718827*$t);
   $Y1 += 0.00000000023*cos(3.56571153376 + 5849.6079295981*$t);
   $Y1 += 0.00000000025*cos(5.52794317615 + 14155.4684127501*$t);
   $Y1 += 0.00000000023*cos(2.01489779401 + 6294.36536773881*$t);
   $Y1 += 0.00000000023*cos(1.49204312505 + 6272.273967211*$t);
   $Y1 += 0.00000000024*cos(5.81677586075 + 18208.1061251085*$t);
   $Y1 += 0.00000000024*cos(5.81677586075 + 18208.5937600755*$t);
   $Y1 += 0.00000000021*cos(4.21172313283 + 22779.6810636773*$t);
   $Y1 += 0.00000000029*cos(1.74369099855 + 28628.5800435831*$t);
   $Y1 += 0.00000000021*cos(3.56347944199 + 40879.6843221273*$t);
   $Y1 += 0.00000000021*cos(2.51496251708 + 30221.1760572159*$t);
   $Y1 += 0.00000000022*cos(6.09500922463 + 4379.8828549737*$t);
   $Y1 += 0.00000000022*cos(3.69512347867 + 8186.7564799761*$t);
   $Y1 += 0.00000000022*cos(2.95145769488 + 28760.05469496669*$t);
   $Y1 += 0.00000000026*cos(1.83973482412 + 18422.8731765817*$t);
   $Y1 += 0.00000000020*cos(2.37799900039 + 12036.21691740469*$t);
   $Y1 += 0.00000000020*cos(0.07485283254 + 28313.0449871775*$t);
   $Y1 += 0.00000000027*cos(5.38239668902 + 34513.5068901663*$t);
   $Y1 += 0.00000000020*cos(0.50551300300 + 34115.3578867581*$t);
   $Y1 += 0.00000000020*cos(3.29228335956 + 21548.7185518083*$t);
   $Y1 += 0.00000000020*cos(5.25656062979 + 17654.5367222661*$t);
   $Y1 += 0.00000000025*cos(1.96130235408 + 55798.7021758819*$t);
   $Y1 += 0.00000000020*cos(5.53200449549 + 20597.4877805247*$t);
   $Y1 += 0.00000000020*cos(1.03604717996 + 18875.76968725751*$t);
   $Y1 += 0.00000000022*cos(2.01853988925 + 1588.82907780029*$t);
   $Y1 += 0.00000000024*cos(5.92091322542 + 9411.7084325707*$t);
   $Y1 += 0.00000000019*cos(1.14706242910 + 536.5606946119*$t);
   $Y1 += 0.00000000020*cos(3.42481272827 + 6187.3404402571*$t);
   $Y1 += 0.00000000020*cos(0.08212819079 + 6379.2988946927*$t);
   $Y1 += 0.00000000022*cos(2.87392170851 + 18326.8939493639*$t);
   $Y1 += 0.00000000022*cos(0.61268580951 + 6233.5626420031*$t);
   $Y1 += 0.00000000022*cos(2.89425510955 + 6333.0766929467*$t);
   $Y1 += 0.00000000018*cos(0.45956662142 + 5972.4788686065*$t);
   $Y1 += 0.00000000018*cos(3.04737429764 + 6594.1604663433*$t);
   $Y1 += 0.00000000019*cos(2.71317971903 + 6948.0757126049*$t);
   $Y1 += 0.00000000019*cos(0.79376120003 + 5618.5636223449*$t);
   $Y1 += 0.00000000019*cos(4.89700264579 + 83974.07917321339*$t);
   $Y1 += 0.00000000019*cos(2.72494811319 + 84020.10309797739*$t);
   $Y1 += 0.00000000020*cos(5.42036659910 + 50316.9596220473*$t);
   $Y1 += 0.00000000020*cos(1.28312164242 + 9070.3626913323*$t);
   return $Y1*$t;
}



   function Earth_Y2($t) // 248 terms of order 2
{
   $Y2  = 0.00052911498*cos(1.75323722235 + 6283.3196674749*$t);
   $Y2 += 0.00006074441*cos(0.48406211192 + 0.2438174835*$t);
   $Y2 += 0.00002406871*cos(2.66006394470 + 12566.3955174663*$t);
   $Y2 += 0.00000096033*cos(2.57406136836 + 18849.4713674577*$t);
   $Y2 += 0.00000029888*cos(2.30515825272 + 6282.83203250789*$t);
   $Y2 += 0.00000014021*cos(1.64042051619 + 6279.7965491259*$t);
   $Y2 += 0.00000013375*cos(1.74320372785 + 6286.84278582391*$t);
   $Y2 += 0.00000008148*cos(0.62869530307 + 6309.61798727471*$t);
   $Y2 += 0.00000008195*cos(2.81874475342 + 6257.0213476751*$t);
   $Y2 += 0.00000008001*cos(4.34877296703 + 6127.8992680407*$t);
   $Y2 += 0.00000007754*cos(5.43470598991 + 6438.7400669091*$t);
   $Y2 += 0.00000007223*cos(3.81034638754 + 83997.09113559539*$t);
   $Y2 += 0.00000004616*cos(4.59623594691 + 710.1768660418*$t);
   $Y2 += 0.00000003060*cos(3.26884940633 + 4705.9761250271*$t);
   $Y2 += 0.00000003717*cos(0.87961276723 + 5507.7970561509*$t);
   $Y2 += 0.00000002991*cos(2.48913608928 + 25132.5472174491*$t);
   $Y2 += 0.00000003235*cos(2.53504921673 + 7058.8422787989*$t);
   $Y2 += 0.00000002393*cos(1.83696733237 + 775.7664288075*$t);
   $Y2 += 0.00000002034*cos(2.77061597204 + 6290.4332144757*$t);
   $Y2 += 0.00000002028*cos(0.70943948464 + 6276.2061204741*$t);
   $Y2 += 0.00000002064*cos(0.01448293997 + 7860.6632099227*$t);
   $Y2 += 0.00000001914*cos(2.85234479868 + 5487.0216606585*$t);
   $Y2 += 0.00000001770*cos(3.48783151064 + 1059.6257476727*$t);
   $Y2 += 0.00000001761*cos(1.80823220260 + 5753.6287023803*$t);
   $Y2 += 0.00000001558*cos(0.56580595512 + 7079.61767429131*$t);
   $Y2 += 0.00000001551*cos(0.55852743151 + 6040.5910635009*$t);
   $Y2 += 0.00000001436*cos(1.28164050883 + 5885.1706640667*$t);
   $Y2 += 0.00000001469*cos(2.94324921966 + 6526.04827144891*$t);
   $Y2 += 0.00000001319*cos(1.89155311923 + 6681.46867088311*$t);
   $Y2 += 0.00000001180*cos(1.42090963151 + 6813.0106325695*$t);
   $Y2 += 0.00000001337*cos(5.75221416852 + 5729.7502646325*$t);
   $Y2 += 0.00000001039*cos(4.25243426113 + 6282.3393464067*$t);
   $Y2 += 0.00000001313*cos(0.59897551240 + 529.9347825781*$t);
   $Y2 += 0.00000001041*cos(5.47801377680 + 6284.2999885431*$t);
   $Y2 += 0.00000001202*cos(4.02829547071 + 6836.8890703173*$t);
   $Y2 += 0.00000000904*cos(0.85768945907 + 5331.6012612243*$t);
   $Y2 += 0.00000000870*cos(2.26660468565 + 4933.4522578161*$t);
   $Y2 += 0.00000001149*cos(3.02450680654 + 167284.00540514899*$t);
   $Y2 += 0.00000001069*cos(1.09203232944 + 1577.0997249643*$t);
   $Y2 += 0.00000000772*cos(2.66290097432 + 7235.0380737255*$t);
   $Y2 += 0.00000000846*cos(1.36798976406 + 11790.8729061423*$t);
   $Y2 += 0.00000000742*cos(5.02007434299 + 4137.1542509997*$t);
   $Y2 += 0.00000000966*cos(2.05227643484 + 11856.462468908*$t);
   $Y2 += 0.00000000664*cos(1.21776131888 + 7633.1870771337*$t);
   $Y2 += 0.00000000662*cos(3.18968574411 + 1194.6908277081*$t);
   $Y2 += 0.00000000659*cos(5.95013449282 + 11507.0135872771*$t);
   $Y2 += 0.00000000853*cos(3.65338850674 + 4535.3032544079*$t);
   $Y2 += 0.00000000823*cos(1.06126729936 + 155.17658195069*$t);
   $Y2 += 0.00000000783*cos(4.30296275012 + 5088.8726572503*$t);
   $Y2 += 0.00000000785*cos(3.46658763633 + 2353.1099712553*$t);
   $Y2 += 0.00000000694*cos(3.50785804761 + 801.5771136403*$t);
   $Y2 += 0.00000000566*cos(4.42922662864 + 796.5418242999*$t);
   $Y2 += 0.00000000684*cos(1.11267726523 + 3128.6325825793*$t);
   $Y2 += 0.00000000499*cos(1.42594865875 + 12569.91863581531*$t);
   $Y2 += 0.00000000596*cos(6.09778115923 + 8031.3360805419*$t);
   $Y2 += 0.00000000486*cos(4.54237345712 + 1589.3167127673*$t);
   $Y2 += 0.00000000483*cos(4.68204073064 + 8429.4850839501*$t);
   $Y2 += 0.00000000441*cos(4.99112466662 + 6496.6187629129*$t);
   $Y2 += 0.00000000412*cos(0.87602065813 + 4693.75913722409*$t);
   $Y2 += 0.00000000561*cos(5.46330573132 + 7477.7666776995*$t);
   $Y2 += 0.00000000390*cos(4.63888431687 + 6070.0205720369*$t);
   $Y2 += 0.00000000405*cos(6.26538093406 + 3739.0052475915*$t);
   $Y2 += 0.00000000501*cos(1.82149883986 + 1592.8398311163*$t);
   $Y2 += 0.00000000392*cos(0.68216396598 + 26.0545023163*$t);
   $Y2 += 0.00000000369*cos(5.93410776393 + 553.32558535889*$t);
   $Y2 += 0.00000000483*cos(6.07777893059 + 12036.7045523717*$t);
   $Y2 += 0.00000000489*cos(3.70668603785 + 5223.4501023187*$t);
   $Y2 += 0.00000000481*cos(3.17195852109 + 10213.5293636945*$t);
   $Y2 += 0.00000000448*cos(5.56847234629 + 156137.71980228279*$t);
   $Y2 += 0.00000000321*cos(4.53991511859 + 951.4745887671*$t);
   $Y2 += 0.00000000320*cos(0.43138538654 + 1990.9888345245*$t);
   $Y2 += 0.00000000361*cos(2.16599134100 + 12565.9078824993*$t);
   $Y2 += 0.00000000361*cos(4.79823251497 + 12721.8159169005*$t);
   $Y2 += 0.00000000412*cos(4.34259218946 + 398.3928208917*$t);
   $Y2 += 0.00000000304*cos(5.68078111352 + 4690.7236538421*$t);
   $Y2 += 0.00000000356*cos(6.00094576166 + 5481.4987363511*$t);
   $Y2 += 0.00000000283*cos(0.66912545661 + 242.97242145751*$t);
   $Y2 += 0.00000000362*cos(2.31554210740 + 9438.0067523705*$t);
   $Y2 += 0.00000000248*cos(3.18243221495 + 8827.6340873583*$t);
   $Y2 += 0.00000000252*cos(2.73244620528 + 3154.4432674121*$t);
   $Y2 += 0.00000000257*cos(0.22372520009 + 11371.9485072417*$t);
   $Y2 += 0.00000000327*cos(1.45001600721 + 14143.7390599141*$t);
   $Y2 += 0.00000000230*cos(1.80447328493 + 1747.7725955835*$t);
   $Y2 += 0.00000000289*cos(3.67617510107 + 7085.1405985987*$t);
   $Y2 += 0.00000000297*cos(3.17045499445 + 1349.62359217529*$t);
   $Y2 += 0.00000000209*cos(4.98580572759 + 11770.0975106499*$t);
   $Y2 += 0.00000000207*cos(0.21623312501 + 4164.0681721295*$t);
   $Y2 += 0.00000000188*cos(4.07593821324 + 7875.91568110771*$t);
   $Y2 += 0.00000000189*cos(5.31784128034 + 2389.1378379327*$t);
   $Y2 += 0.00000000194*cos(5.12285857348 + 10977.32262218251*$t);
   $Y2 += 0.00000000176*cos(2.51491286893 + 7342.7015976641*$t);
   $Y2 += 0.00000000164*cos(1.22070775546 + 5223.93773728571*$t);
   $Y2 += 0.00000000205*cos(1.64290799201 + 10973.7995038335*$t);
   $Y2 += 0.00000000205*cos(0.68022152849 + 13368.2164485901*$t);
   $Y2 += 0.00000000139*cos(3.04789845179 + 10575.6505004253*$t);
   $Y2 += 0.00000000155*cos(4.24333188820 + 12410.9751180321*$t);
   $Y2 += 0.00000000152*cos(4.81739365185 + 3930.4535137031*$t);
   $Y2 += 0.00000000129*cos(0.80954637051 + 397.9051859247*$t);
   $Y2 += 0.00000000127*cos(5.67893828134 + 12592.6938372661*$t);
   $Y2 += 0.00000000132*cos(4.87291664360 + 8636.18582124671*$t);
   $Y2 += 0.00000000122*cos(3.21278850975 + 13119.9649203087*$t);
   $Y2 += 0.00000000163*cos(2.50359725386 + 5507.3094211839*$t);
   $Y2 += 0.00000000114*cos(4.13352060999 + 17260.3984721739*$t);
   $Y2 += 0.00000000118*cos(0.80775184774 + 4292.5746504339*$t);
   $Y2 += 0.00000000148*cos(4.06616105867 + 17790.08943726851*$t);
   $Y2 += 0.00000000123*cos(2.18706277371 + 12562.8723991173*$t);
   $Y2 += 0.00000000103*cos(1.79915109205 + 13518.1139237169*$t);
   $Y2 += 0.00000000138*cos(2.26785827403 + 12168.2465140581*$t);
   $Y2 += 0.00000000107*cos(2.76898193474 + 10989.0519750185*$t);
   $Y2 += 0.00000000104*cos(0.56758990747 + 2145.92159899169*$t);
   $Y2 += 0.00000000096*cos(0.40713285174 + 6283.25235717211*$t);
   $Y2 += 0.00000000099*cos(0.70751931715 + 13096.0864825609*$t);
   $Y2 += 0.00000000096*cos(6.24621518908 + 6283.3869777777*$t);
   $Y2 += 0.00000000084*cos(3.63516988010 + 16730.70750707931*$t);
   $Y2 += 0.00000000111*cos(2.39242858566 + 12540.0971976665*$t);
   $Y2 += 0.00000000087*cos(4.45435464785 + 10177.5014970171*$t);
   $Y2 += 0.00000000078*cos(2.61342989032 + 12323.6669134923*$t);
   $Y2 += 0.00000000076*cos(4.13782411870 + 6709.91785835091*$t);
   $Y2 += 0.00000000105*cos(1.96765772501 + 5642.4420600927*$t);
   $Y2 += 0.00000000073*cos(5.60932378402 + 5856.7214765989*$t);
   $Y2 += 0.00000000102*cos(5.37997051996 + 250570.91967470259*$t);
   $Y2 += 0.00000000070*cos(0.51266920771 + 5753.14106741329*$t);
   $Y2 += 0.00000000071*cos(5.27404987073 + 14314.4119305333*$t);
   $Y2 += 0.00000000092*cos(0.39406209616 + 13916.2629271251*$t);
   $Y2 += 0.00000000070*cos(4.10529685600 + 7084.6529636317*$t);
   $Y2 += 0.00000000091*cos(1.53880932897 + 6924.1972748571*$t);
   $Y2 += 0.00000000083*cos(2.41910116753 + 31415.6230674405*$t);
   $Y2 += 0.00000000088*cos(0.30300511757 + 71430.45180064559*$t);
   $Y2 += 0.00000000074*cos(3.53618023271 + 529.44714761109*$t);
   $Y2 += 0.00000000085*cos(4.40773297118 + 95143.37673846159*$t);
   $Y2 += 0.00000000061*cos(2.66034748981 + 8274.0646845159*$t);
   $Y2 += 0.00000000060*cos(3.95075537336 + 16496.6052136859*$t);
   $Y2 += 0.00000000058*cos(3.04970743906 + 5642.93474619389*$t);
   $Y2 += 0.00000000078*cos(0.84061745785 + 213.5429129215*$t);
   $Y2 += 0.00000000059*cos(3.10446906052 + 640.1411037975*$t);
   $Y2 += 0.00000000057*cos(3.63209087311 + 4690.23601887509*$t);
   $Y2 += 0.00000000056*cos(2.23091315346 + 13341.9181287903*$t);
   $Y2 += 0.00000000065*cos(4.95952410600 + 4731.7868098599*$t);
   $Y2 += 0.00000000051*cos(0.09929724563 + 3893.93801205869*$t);
   $Y2 += 0.00000000050*cos(2.60290043400 + 12573.5090644671*$t);
   $Y2 += 0.00000000050*cos(5.98749446465 + 3634.37720703489*$t);
   $Y2 += 0.00000000049*cos(0.70632876133 + 18319.7804023631*$t);
   $Y2 += 0.00000000066*cos(3.92505951821 + 2787.2868413409*$t);
   $Y2 += 0.00000000049*cos(0.42825389697 + 13362.6935242827*$t);
   $Y2 += 0.00000000048*cos(2.19153498916 + 12964.5445208745*$t);
   $Y2 += 0.00000000053*cos(2.55294637714 + 14919.2616712381*$t);
   $Y2 += 0.00000000058*cos(5.94322042199 + 1577.5873599313*$t);
   $Y2 += 0.00000000052*cos(5.99067773035 + 15721.0826023619*$t);
   $Y2 += 0.00000000061*cos(4.99956015956 + 4292.0870154669*$t);
   $Y2 += 0.00000000044*cos(5.02285128905 + 24073.1652872599*$t);
   $Y2 += 0.00000000044*cos(1.96034093811 + 10447.14402212089*$t);
   $Y2 += 0.00000000042*cos(2.52906622396 + 20426.8149099055*$t);
   $Y2 += 0.00000000059*cos(2.58678833053 + 84672.23202702119*$t);
   $Y2 += 0.00000000051*cos(3.81200106179 + 6370.62787201471*$t);
   $Y2 += 0.00000000051*cos(5.97812516445 + 6196.0114629351*$t);
   $Y2 += 0.00000000046*cos(5.23410298912 + 2119.00767786191*$t);
   $Y2 += 0.00000000041*cos(2.52248241350 + 3185.43584474911*$t);
   $Y2 += 0.00000000056*cos(4.84268946755 + 10976.83498721549*$t);
   $Y2 += 0.00000000051*cos(3.94604483748 + 9437.5191174035*$t);
   $Y2 += 0.00000000049*cos(1.64079935270 + 239424.63407183639*$t);
   $Y2 += 0.00000000036*cos(1.49154565635 + 3495.78900865049*$t);
   $Y2 += 0.00000000044*cos(2.12316916648 + 18139.5383188994*$t);
   $Y2 += 0.00000000035*cos(5.18466509768 + 3929.96587873609*$t);
   $Y2 += 0.00000000044*cos(3.23094444076 + 8661.9965060795*$t);
   $Y2 += 0.00000000046*cos(2.21535532938 + 3894.4256470257*$t);
   $Y2 += 0.00000000034*cos(2.24364878116 + 6262.5442719825*$t);
   $Y2 += 0.00000000034*cos(4.40970135798 + 6304.0950629673*$t);
   $Y2 += 0.00000000035*cos(5.15488505358 + 18875.2820522905*$t);
   $Y2 += 0.00000000035*cos(4.51996316476 + 10420.23010099111*$t);
   $Y2 += 0.00000000032*cos(5.33774590597 + 23013.7833570707*$t);
   $Y2 += 0.00000000034*cos(2.04067717162 + 7.3573644843*$t);
   $Y2 += 0.00000000045*cos(6.20022544523 + 3340.8562441833*$t);
   $Y2 += 0.00000000035*cos(1.95669709650 + 10213.0417287275*$t);
   $Y2 += 0.00000000031*cos(3.81393239996 + 14712.5609339415*$t);
   $Y2 += 0.00000000030*cos(2.82636067861 + 12809.12412144031*$t);
   $Y2 += 0.00000000039*cos(5.86021469295 + 9779.3524936089*$t);
   $Y2 += 0.00000000041*cos(0.90085517811 + 82576.73740351178*$t);
   $Y2 += 0.00000000031*cos(5.47911308336 + 22779.6810636773*$t);
   $Y2 += 0.00000000028*cos(2.50895957020 + 2942.7072407751*$t);
   $Y2 += 0.00000000028*cos(3.93140481871 + 5429.63565075589*$t);
   $Y2 += 0.00000000027*cos(5.61355223000 + 11014.8626598513*$t);
   $Y2 += 0.00000000027*cos(0.53944014317 + 9623.9320941747*$t);
   $Y2 += 0.00000000025*cos(0.15499921281 + 18209.5740811437*$t);
   $Y2 += 0.00000000026*cos(0.97984555070 + 9381.2034902007*$t);
   $Y2 += 0.00000000025*cos(1.11733310909 + 3583.5848481573*$t);
   $Y2 += 0.00000000030*cos(4.41826131323 + 10447.6316570879*$t);
   $Y2 += 0.00000000032*cos(5.65959938774 + 796.0541893329*$t);
   $Y2 += 0.00000000023*cos(1.38575591729 + 9917.45305702629*$t);
   $Y2 += 0.00000000026*cos(5.61732376806 + 12012.8261146239*$t);
   $Y2 += 0.00000000024*cos(3.45407340381 + 14143.2514249471*$t);
   $Y2 += 0.00000000023*cos(0.69647593831 + 26709.8907598969*$t);
   $Y2 += 0.00000000021*cos(5.43974830479 + 7632.6994421667*$t);
   $Y2 += 0.00000000021*cos(0.80058820514 + 17256.8753538249*$t);
   $Y2 += 0.00000000028*cos(5.78689433830 + 12353.0964220283*$t);
   $Y2 += 0.00000000022*cos(3.26384247748 + 77714.01528560399*$t);
   $Y2 += 0.00000000021*cos(4.41664687493 + 6993.2527160332*$t);
   $Y2 += 0.00000000020*cos(1.88345295944 + 6836.40143535029*$t);
   $Y2 += 0.00000000020*cos(3.87252932348 + 1748.2602305505*$t);
   $Y2 += 0.00000000026*cos(0.45595739946 + 9225.7830907665*$t);
   $Y2 += 0.00000000023*cos(5.52071600644 + 149854.64395229139*$t);
   $Y2 += 0.00000000026*cos(0.60119736394 + 2544.07060239989*$t);
   $Y2 += 0.00000000018*cos(0.71176167314 + 11614.6771112157*$t);
   $Y2 += 0.00000000020*cos(3.04333652471 + 426.8420083595*$t);
   $Y2 += 0.00000000019*cos(2.09021029647 + 30640.1004561165*$t);
   $Y2 += 0.00000000018*cos(3.20715016441 + 155.6642169177*$t);
   $Y2 += 0.00000000018*cos(0.32502884515 + 11926.4982311523*$t);
   $Y2 += 0.00000000022*cos(2.17863705978 + 17298.4261448097*$t);
   $Y2 += 0.00000000019*cos(4.82590950931 + 18073.46112116669*$t);
   $Y2 += 0.00000000020*cos(1.02950628439 + 19651.2922985815*$t);
   $Y2 += 0.00000000024*cos(3.14467908991 + 5856.23384163189*$t);
   $Y2 += 0.00000000023*cos(3.21288696705 + 72850.80553272919*$t);
   $Y2 += 0.00000000017*cos(2.38623798159 + 8983.0544867925*$t);
   $Y2 += 0.00000000016*cos(3.59989563011 + 16460.5773470085*$t);
   $Y2 += 0.00000000016*cos(2.77179277240 + 17259.91083720689*$t);
   $Y2 += 0.00000000021*cos(2.20286926080 + 16858.7263504167*$t);
   $Y2 += 0.00000000020*cos(5.10666464640 + 23543.4743221653*$t);
   $Y2 += 0.00000000021*cos(5.36525328799 + 13367.7288136231*$t);
   $Y2 += 0.00000000020*cos(2.47582665567 + 2146.4092339587*$t);
   $Y2 += 0.00000000014*cos(4.85508264889 + 22484.0923919761*$t);
   $Y2 += 0.00000000014*cos(6.23679853613 + 16200.52890701769*$t);
   $Y2 += 0.00000000015*cos(2.64768344463 + 20199.3387771165*$t);
   $Y2 += 0.00000000019*cos(1.24340851761 + 8672.21368792411*$t);
   $Y2 += 0.00000000019*cos(2.89835524904 + 18073.9487561337*$t);
   $Y2 += 0.00000000013*cos(3.14350306699 + 16201.0165419847*$t);
   $Y2 += 0.00000000017*cos(2.96004801328 + 7860.17557495569*$t);
   $Y2 += 0.00000000013*cos(3.12347768998 + 18451.3223640495*$t);
   $Y2 += 0.00000000016*cos(4.90276494200 + 5572.8989839496*$t);
   $Y2 += 0.00000000012*cos(1.13909733386 + 17996.2749857057*$t);
   $Y2 += 0.00000000012*cos(5.43460350187 + 19403.0407703001*$t);
   $Y2 += 0.00000000012*cos(0.82655936469 + 14945.0723560709*$t);
   $Y2 += 0.00000000015*cos(0.47799392266 + 12559.2819704655*$t);
   $Y2 += 0.00000000012*cos(2.80799572996 + 10818.3791043993*$t);
   $Y2 += 0.00000000015*cos(5.28864619208 + 12567.37583853451*$t);
   $Y2 += 0.00000000011*cos(0.21391675163 + 28767.1682419675*$t);
   $Y2 += 0.00000000013*cos(5.80786469706 + 90280.16698558679*$t);
   $Y2 += 0.00000000011*cos(0.30991526634 + 22805.4917485101*$t);
   $Y2 += 0.00000000011*cos(0.67526353010 + 35372.1310834599*$t);
   $Y2 += 0.00000000010*cos(1.54770241799 + 11506.52595231009*$t);
   $Y2 += 0.00000000014*cos(0.36722951229 + 21228.63584102931*$t);
   $Y2 += 0.00000000010*cos(4.84303300061 + 12779.6946129043*$t);
   $Y2 += 0.00000000010*cos(4.48229440272 + 23141.8022004081*$t);
   $Y2 += 0.00000000010*cos(5.89241784221 + 1350.1112271423*$t);
   $Y2 += 0.00000000010*cos(4.07842068364 + 10575.1628654583*$t);
   $Y2 += 0.00000000010*cos(0.98564127078 + 22345.5041935917*$t);
   $Y2 += 0.00000000010*cos(0.62034181586 + 9778.8648586419*$t);
   $Y2 += 0.00000000009*cos(6.20390809264 + 29826.5501721567*$t);
   $Y2 += 0.00000000010*cos(2.66921279687 + 23581.5019948011*$t);
   $Y2 += 0.00000000011*cos(5.10849785178 + 25158.8455372489*$t);
   return $Y2*$t*$t;
}



   function Earth_Y3($t) // 46 terms of order 3
{
   $Y3  = 0.00000232790*cos(1.83555296287 + 0.2438174835*$t);
   $Y3 += 0.00000076843*cos(0.95359770708 + 12566.3955174663*$t);
   $Y3 += 0.00000035331*cos(1.77537067174 + 6283.3196674749*$t);
   $Y3 += 0.00000005282*cos(0.83410185910 + 18849.4713674577*$t);
   $Y3 += 0.00000001631*cos(0.72620862558 + 6282.83203250789*$t);
   $Y3 += 0.00000001483*cos(0.65599216587 + 6438.7400669091*$t);
   $Y3 += 0.00000001479*cos(2.83072127404 + 6127.8992680407*$t);
   $Y3 += 0.00000000652*cos(3.13796451313 + 6279.7965491259*$t);
   $Y3 += 0.00000000656*cos(0.01615422945 + 6286.84278582391*$t);
   $Y3 += 0.00000000317*cos(1.43157695840 + 6526.04827144891*$t);
   $Y3 += 0.00000000318*cos(2.05944515679 + 6040.5910635009*$t);
   $Y3 += 0.00000000227*cos(0.70881980797 + 25132.5472174491*$t);
   $Y3 += 0.00000000201*cos(5.52615570373 + 6836.8890703173*$t);
   $Y3 += 0.00000000201*cos(4.23563267830 + 5729.7502646325*$t);
   $Y3 += 0.00000000036*cos(6.05003898151 + 12569.91863581531*$t);
   $Y3 += 0.00000000034*cos(0.36912822345 + 4705.9761250271*$t);
   $Y3 += 0.00000000033*cos(2.65665718326 + 12410.9751180321*$t);
   $Y3 += 0.00000000032*cos(3.81269087187 + 6257.0213476751*$t);
   $Y3 += 0.00000000032*cos(2.84133186500 + 6309.61798727471*$t);
   $Y3 += 0.00000000026*cos(5.21573736647 + 775.7664288075*$t);
   $Y3 += 0.00000000023*cos(0.44696909593 + 1059.6257476727*$t);
   $Y3 += 0.00000000025*cos(0.01326357360 + 7860.6632099227*$t);
   $Y3 += 0.00000000027*cos(0.46730161827 + 12565.9078824993*$t);
   $Y3 += 0.00000000020*cos(2.19213288615 + 5753.6287023803*$t);
   $Y3 += 0.00000000019*cos(1.44435756454 + 5885.1706640667*$t);
   $Y3 += 0.00000000019*cos(4.47976718197 + 6813.0106325695*$t);
   $Y3 += 0.00000000018*cos(0.72211012099 + 12721.8159169005*$t);
   $Y3 += 0.00000000017*cos(5.20474716177 + 6681.46867088311*$t);
   $Y3 += 0.00000000013*cos(2.92478174105 + 5487.0216606585*$t);
   $Y3 += 0.00000000011*cos(3.72062261949 + 7079.61767429131*$t);
   $Y3 += 0.00000000010*cos(2.89167021825 + 5507.7970561509*$t);
   $Y3 += 0.00000000010*cos(1.46762801985 + 11790.8729061423*$t);
   $Y3 += 0.00000000009*cos(6.21784065295 + 11507.0135872771*$t);
   $Y3 += 0.00000000011*cos(4.68801103302 + 12592.6938372661*$t);
   $Y3 += 0.00000000009*cos(3.82179163254 + 7058.8422787989*$t);
   $Y3 += 0.00000000008*cos(5.97770881958 + 6290.4332144757*$t);
   $Y3 += 0.00000000008*cos(0.68313431813 + 6276.2061204741*$t);
   $Y3 += 0.00000000008*cos(1.45990791265 + 796.5418242999*$t);
   $Y3 += 0.00000000007*cos(4.15465801739 + 4693.75913722409*$t);
   $Y3 += 0.00000000007*cos(0.35078036011 + 7.3573644843*$t);
   $Y3 += 0.00000000005*cos(3.29483739022 + 3739.0052475915*$t);
   $Y3 += 0.00000000005*cos(4.64160679165 + 6070.0205720369*$t);
   $Y3 += 0.00000000006*cos(2.38483297622 + 6284.2999885431*$t);
   $Y3 += 0.00000000006*cos(4.26851560652 + 6282.3393464067*$t);
   $Y3 += 0.00000000005*cos(1.89668032657 + 4137.1542509997*$t);
   $Y3 += 0.00000000005*cos(2.01213704309 + 6496.6187629129*$t);
   return $Y3*$t*$t*$t;
}



   function Earth_Y4($t) // 20 terms of order 4
{
   $Y4  = 0.00000114918*cos(4.77291921544 + 6283.3196674749*$t);
   $Y4 += 0.00000006817*cos(2.90545031303 + 0.2438174835*$t);
   $Y4 += 0.00000003158*cos(5.19149443854 + 12566.3955174663*$t);
   $Y4 += 0.00000000211*cos(2.15272014034 + 6438.7400669091*$t);
   $Y4 += 0.00000000210*cos(1.31410288968 + 6127.8992680407*$t);
   $Y4 += 0.00000000253*cos(5.35904773722 + 18849.4713674577*$t);
   $Y4 += 0.00000000073*cos(5.49456429263 + 6282.83203250789*$t);
   $Y4 += 0.00000000042*cos(3.03243163257 + 710.1768660418*$t);
   $Y4 += 0.00000000023*cos(4.47433896564 + 6279.7965491259*$t);
   $Y4 += 0.00000000022*cos(4.47061131203 + 6286.84278582391*$t);
   $Y4 += 0.00000000012*cos(5.22878976906 + 25132.5472174491*$t);
   $Y4 += 0.00000000009*cos(2.63928422881 + 83997.09113559539*$t);
   $Y4 += 0.00000000009*cos(3.62312855703 + 11856.462468908*$t);
   $Y4 += 0.00000000008*cos(4.59507616558 + 167284.00540514899*$t);
   $Y4 += 0.00000000005*cos(4.40834364231 + 6257.0213476751*$t);
   $Y4 += 0.00000000005*cos(2.21798830782 + 6309.61798727471*$t);
   $Y4 += 0.00000000005*cos(1.11835906029 + 12410.9751180321*$t);
   $Y4 += 0.00000000005*cos(3.99770824807 + 156137.71980228279*$t);
   $Y4 += 0.00000000004*cos(4.97197978751 + 12565.9078824993*$t);
   $Y4 += 0.00000000003*cos(3.60141861387 + 529.9347825781*$t);
   return $Y4*$t*$t*$t*$t;
}



   function Earth_Y5($t) // 7 terms of order 5
{
   $Y5  = 0.00000000877*cos(4.87375194107 + 6283.3196674749*$t);
   $Y5 += 0.00000000305*cos(4.05893642248 + 0.2438174835*$t);
   $Y5 += 0.00000000101*cos(3.30997066470 + 12566.3955174663*$t);
   $Y5 += 0.00000000025*cos(3.73514325706 + 6438.7400669091*$t);
   $Y5 += 0.00000000025*cos(5.99118011879 + 6127.8992680407*$t);
   $Y5 += 0.00000000010*cos(3.54319653898 + 18849.4713674577*$t);
   $Y5 += 0.00000000003*cos(4.13285368647 + 6282.83203250789*$t);
   return $Y5*$t*$t*$t*$t*$t;
}



   function Earth_Z0($t) // 178 terms of order 0
{
   $Z0  = 0.00000279620*cos(3.19870156017 + 84334.66158130829*$t);
   $Z0 += 0.00000101625*cos(5.42248110597 + 5507.5532386674*$t);
   $Z0 += 0.00000080461*cos(3.88027157914 + 5223.6939198022*$t);
   $Z0 += 0.00000043855*cos(3.70430347723 + 2352.8661537718*$t);
   $Z0 += 0.00000031853*cos(3.99997840986 + 1577.3435424478*$t);
   $Z0 += 0.00000022724*cos(3.98473831560 + 1047.7473117547*$t);
   $Z0 += 0.00000016395*cos(3.56456162523 + 5856.4776591154*$t);
   $Z0 += 0.00000018150*cos(4.98479613321 + 6283.0758499914*$t);
   $Z0 += 0.00000014477*cos(3.70258423465 + 9437.762934887*$t);
   $Z0 += 0.00000014325*cos(3.41020246931 + 10213.285546211*$t);
   $Z0 += 0.00000011204*cos(4.83021499098 + 14143.4952424306*$t);
   $Z0 += 0.00000010896*cos(2.08023708321 + 6812.766815086*$t);
   $Z0 += 0.00000009716*cos(3.47560205064 + 4694.0029547076*$t);
   $Z0 += 0.00000010363*cos(4.05663928093 + 71092.88135493269*$t);
   $Z0 += 0.00000008792*cos(4.44890729280 + 5753.3848848968*$t);
   $Z0 += 0.00000008337*cos(4.99167706048 + 7084.8967811152*$t);
   $Z0 += 0.00000006921*cos(4.32559054073 + 6275.9623029906*$t);
   $Z0 += 0.00000009145*cos(1.14182646613 + 6620.8901131878*$t);
   $Z0 += 0.00000007205*cos(3.62441599378 + 529.6909650946*$t);
   $Z0 += 0.00000007698*cos(5.55425745881 + 167621.57585086189*$t);
   $Z0 += 0.00000005186*cos(6.25384533126 + 18073.7049386502*$t);
   $Z0 += 0.00000005027*cos(2.49727910749 + 4705.7323075436*$t);
   $Z0 += 0.00000004529*cos(2.33827747356 + 6309.3741697912*$t);
   $Z0 += 0.00000004754*cos(0.71100829534 + 5884.9268465832*$t);
   $Z0 += 0.00000004292*cos(1.10034019036 + 6681.2248533996*$t);
   $Z0 += 0.00000003855*cos(1.82331477760 + 5486.777843175*$t);
   $Z0 += 0.00000005256*cos(4.42445744523 + 7860.4193924392*$t);
   $Z0 += 0.00000004025*cos(5.11990285618 + 13367.9726311066*$t);
   $Z0 += 0.00000004061*cos(6.02923989849 + 3930.2096962196*$t);
   $Z0 += 0.00000003798*cos(0.44370219832 + 3154.6870848956*$t);
   $Z0 += 0.00000002933*cos(5.12428135363 + 1059.3819301892*$t);
   $Z0 += 0.00000003424*cos(5.42065191700 + 6069.7767545534*$t);
   $Z0 += 0.00000003670*cos(4.58210192227 + 12194.0329146209*$t);
   $Z0 += 0.00000002912*cos(1.92688087702 + 10977.078804699*$t);
   $Z0 += 0.00000002506*cos(0.60484952637 + 6496.3749454294*$t);
   $Z0 += 0.00000002680*cos(1.39497359287 + 22003.9146348698*$t);
   $Z0 += 0.00000002120*cos(4.30691000285 + 5643.1785636774*$t);
   $Z0 += 0.00000002235*cos(0.81363184041 + 8635.9420037632*$t);
   $Z0 += 0.00000001819*cos(3.75748003107 + 3340.6124266998*$t);
   $Z0 += 0.00000002230*cos(2.76407822446 + 12036.4607348882*$t);
   $Z0 += 0.00000001945*cos(5.69949789177 + 11790.6290886588*$t);
   $Z0 += 0.00000001529*cos(1.98639348122 + 398.1490034082*$t);
   $Z0 += 0.00000001581*cos(3.19976230948 + 5088.6288397668*$t);
   $Z0 += 0.00000001424*cos(6.26116472313 + 2544.3144198834*$t);
   $Z0 += 0.00000001405*cos(4.69394873481 + 7058.5984613154*$t);
   $Z0 += 0.00000001588*cos(0.25704624784 + 17298.1823273262*$t);
   $Z0 += 0.00000001478*cos(2.81808207569 + 25934.1243310894*$t);
   $Z0 += 0.00000001693*cos(4.95689385293 + 156475.2902479957*$t);
   $Z0 += 0.00000001123*cos(2.38605285936 + 3738.761430108*$t);
   $Z0 += 0.00000000990*cos(4.29600699156 + 9225.539273283*$t);
   $Z0 += 0.00000000923*cos(3.07195736431 + 4164.311989613*$t);
   $Z0 += 0.00000000863*cos(0.55119493097 + 8429.2412664666*$t);
   $Z0 += 0.00000000984*cos(5.96784225392 + 7079.3738568078*$t);
   $Z0 -= 0.00000001086;
   $Z0 += 0.00000000824*cos(1.50050055147 + 10447.3878396044*$t);
   $Z0 += 0.00000000829*cos(3.51336978167 + 11506.7697697936*$t);
   $Z0 += 0.00000000908*cos(0.12102885474 + 11015.1064773348*$t);
   $Z0 += 0.00000000742*cos(1.99159139281 + 26087.9031415742*$t);
   $Z0 += 0.00000000850*cos(4.24120016095 + 29864.334027309*$t);
   $Z0 += 0.00000000762*cos(2.90028034892 + 4732.0306273434*$t);
   $Z0 += 0.00000000711*cos(1.38596151724 + 2146.1654164752*$t);
   $Z0 += 0.00000000766*cos(0.61397570576 + 796.2980068164*$t);
   $Z0 += 0.00000000708*cos(1.91406542362 + 8031.0922630584*$t);
   $Z0 += 0.00000000810*cos(5.13961498599 + 2942.4634232916*$t);
   $Z0 += 0.00000000751*cos(1.67479850166 + 21228.3920235458*$t);
   $Z0 += 0.00000000589*cos(2.02193316047 + 775.522611324*$t);
   $Z0 += 0.00000000513*cos(2.60734651128 + 12566.1516999828*$t);
   $Z0 += 0.00000000550*cos(1.61119298963 + 801.8209311238*$t);
   $Z0 += 0.00000000592*cos(4.59481504319 + 4690.4798363586*$t);
   $Z0 += 0.00000000519*cos(5.72925039114 + 8827.3902698748*$t);
   $Z0 += 0.00000000515*cos(4.09976487552 + 64809.80550494129*$t);
   $Z0 += 0.00000000503*cos(5.66433137112 + 33794.5437235286*$t);
   $Z0 += 0.00000000476*cos(3.69640472448 + 213.299095438*$t);
   $Z0 += 0.00000000463*cos(1.24987240483 + 15720.8387848784*$t);
   $Z0 += 0.00000000411*cos(0.10769444525 + 3128.3887650958*$t);
   $Z0 += 0.00000000504*cos(3.26207669160 + 7632.9432596502*$t);
   $Z0 += 0.00000000552*cos(1.02926440457 + 239762.20451754928*$t);
   $Z0 += 0.00000000394*cos(5.81496021156 + 426.598190876*$t);
   $Z0 += 0.00000000476*cos(3.52810472421 + 16496.3613962024*$t);
   $Z0 += 0.00000000503*cos(4.85802444134 + 6290.1893969922*$t);
   $Z0 += 0.00000000417*cos(0.81920713533 + 5216.5803728014*$t);
   $Z0 += 0.00000000363*cos(5.70015720452 + 1589.0728952838*$t);
   $Z0 += 0.00000000363*cos(1.28376436579 + 6206.8097787158*$t);
   $Z0 += 0.00000000353*cos(4.70059133110 + 7234.794256242*$t);
   $Z0 += 0.00000000381*cos(2.61890749829 + 7342.4577801806*$t);
   $Z0 += 0.00000000420*cos(0.97821135132 + 4136.9104335162*$t);
   $Z0 += 0.00000000335*cos(6.22057967320 + 12168.0026965746*$t);
   $Z0 += 0.00000000387*cos(3.09145061418 + 25158.6017197654*$t);
   $Z0 += 0.00000000349*cos(2.94840720073 + 9623.6882766912*$t);
   $Z0 += 0.00000000418*cos(3.75759994446 + 5230.807466803*$t);
   $Z0 += 0.00000000396*cos(1.22507712354 + 6438.4962494256*$t);
   $Z0 += 0.00000000322*cos(1.21162178805 + 8662.240323563*$t);
   $Z0 += 0.00000000323*cos(5.50254808471 + 1194.4470102246*$t);
   $Z0 += 0.00000000372*cos(1.72256107938 + 14945.3161735544*$t);
   $Z0 += 0.00000000305*cos(0.80429352049 + 37724.7534197482*$t);
   $Z0 += 0.00000000265*cos(6.10358507671 + 6836.6452528338*$t);
   $Z0 += 0.00000000250*cos(4.56452895547 + 7477.522860216*$t);
   $Z0 += 0.00000000266*cos(2.62926282354 + 7238.67559160*$t);
   $Z0 += 0.00000000262*cos(1.34297269105 + 11769.8536931664*$t);
   $Z0 += 0.00000000263*cos(6.22089501237 + 6133.5126528568*$t);
   $Z0 += 0.00000000302*cos(2.81139045072 + 1748.016413067*$t);
   $Z0 += 0.00000000316*cos(1.62662805006 + 250908.49012041549*$t);
   $Z0 += 0.00000000226*cos(2.42346415873 + 11371.7046897582*$t);
   $Z0 += 0.00000000216*cos(3.68721275185 + 5849.3641121146*$t);
   $Z0 += 0.00000000276*cos(2.33399767923 + 19651.048481098*$t);
   $Z0 += 0.00000000230*cos(0.36165162947 + 5863.5912061162*$t);
   $Z0 += 0.00000000200*cos(5.86073159059 + 4535.0594369244*$t);
   $Z0 += 0.00000000277*cos(4.65400292395 + 82239.16695779889*$t);
   $Z0 += 0.00000000199*cos(5.05186622555 + 5429.8794682394*$t);
   $Z0 += 0.00000000206*cos(3.70556027982 + 10973.55568635*$t);
   $Z0 += 0.00000000210*cos(4.50691909144 + 29088.811415985*$t);
   $Z0 += 0.00000000181*cos(6.00294783127 + 4292.3308329504*$t);
   $Z0 += 0.00000000249*cos(0.12900984422 + 154379.79562448629*$t);
   $Z0 += 0.00000000167*cos(6.27348370813 + 10988.808157535*$t);
   $Z0 += 0.00000000183*cos(3.19836584401 + 16730.4636895958*$t);
   $Z0 += 0.00000000225*cos(3.18339652605 + 18875.525869774*$t);
   $Z0 += 0.00000000192*cos(0.87192032514 + 77375.95720492408*$t);
   $Z0 += 0.00000000188*cos(2.22746128596 + 41654.9631159678*$t);
   $Z0 += 0.00000000160*cos(5.20207795189 + 5481.2549188676*$t);
   $Z0 += 0.00000000192*cos(3.89678943865 + 17789.845619785*$t);
   $Z0 += 0.00000000159*cos(5.32590846475 + 7.1135470008*$t);
   $Z0 += 0.00000000214*cos(4.08171403539 + 337.8142631964*$t);
   $Z0 += 0.00000000184*cos(3.75465072369 + 23581.2581773176*$t);
   $Z0 += 0.00000000160*cos(4.53897529970 + 18477.1087646123*$t);
   $Z0 += 0.00000000144*cos(5.19486374726 + 20426.571092422*$t);
   $Z0 += 0.00000000126*cos(1.69446958184 + 15110.4661198662*$t);
   $Z0 += 0.00000000149*cos(1.55306832217 + 10021.8372800994*$t);
   $Z0 += 0.00000000127*cos(5.31068233515 + 639.897286314*$t);
   $Z0 += 0.00000000118*cos(2.73268114062 + 18849.2275499742*$t);
   $Z0 += 0.00000000123*cos(2.55361087146 + 12592.4500197826*$t);
   $Z0 += 0.00000000113*cos(6.07178776726 + 6709.6740408674*$t);
   $Z0 += 0.00000000130*cos(3.94161517411 + 7330.8231617461*$t);
   $Z0 += 0.00000000110*cos(3.51203379263 + 18052.9295431578*$t);
   $Z0 += 0.00000000147*cos(4.63371971408 + 22805.7355659936*$t);
   $Z0 += 0.00000000113*cos(1.20711498905 + 14919.0178537546*$t);
   $Z0 += 0.00000000108*cos(3.76359880753 + 14314.1681130498*$t);
   $Z0 += 0.00000000148*cos(0.65447253687 + 95480.9471841745*$t);
   $Z0 += 0.00000000121*cos(3.13748782783 + 14712.317116458*$t);
   $Z0 += 0.00000000119*cos(5.92110458985 + 33019.0211122046*$t);
   $Z0 += 0.00000000124*cos(2.93638421840 + 32217.2001810808*$t);
   $Z0 += 0.00000000117*cos(3.65065271640 + 45585.1728121874*$t);
   $Z0 += 0.00000000127*cos(4.74596574209 + 49515.382508407*$t);
   $Z0 += 0.00000000120*cos(1.04211499785 + 6915.8595893046*$t);
   $Z0 += 0.00000000120*cos(5.60638811846 + 5650.2921106782*$t);
   $Z0 += 0.00000000127*cos(2.70766997307 + 12352.8526045448*$t);
   $Z0 += 0.00000000113*cos(5.39691616452 + 5235.3285382367*$t);
   $Z0 += 0.00000000097*cos(1.07908724794 + 9917.6968745098*$t);
   $Z0 += 0.00000000123*cos(5.15992792934 + 27511.4678735372*$t);
   $Z0 += 0.00000000099*cos(4.45774681732 + 4933.2084403326*$t);
   $Z0 += 0.00000000087*cos(3.93637812950 + 17654.7805397496*$t);
   $Z0 += 0.00000000122*cos(2.23956068680 + 83997.09113559539*$t);
   $Z0 += 0.00000000102*cos(3.97386522171 + 10818.1352869158*$t);
   $Z0 += 0.00000000084*cos(4.21241958885 + 22779.4372461938*$t);
   $Z0 += 0.00000000103*cos(4.63519841733 + 1349.8674096588*$t);
   $Z0 += 0.00000000081*cos(4.01655240430 + 1592.5960136328*$t);
   $Z0 += 0.00000000092*cos(3.24182657569 + 78051.5857313169*$t);
   $Z0 += 0.00000000094*cos(4.33873972800 + 36147.4098773004*$t);
   $Z0 += 0.00000000076*cos(2.83897554420 + 17260.1546546904*$t);
   $Z0 += 0.00000000100*cos(6.07733097102 + 26735.9452622132*$t);
   $Z0 += 0.00000000078*cos(0.72462427283 + 12779.4507954208*$t);
   $Z0 += 0.00000000086*cos(5.26029638250 + 28313.288804661*$t);
   $Z0 += 0.00000000093*cos(4.31900620254 + 44809.6502008634*$t);
   $Z0 += 0.00000000076*cos(6.22743405935 + 13521.7514415914*$t);
   $Z0 += 0.00000000081*cos(2.16420552322 + 13095.8426650774*$t);
   $Z0 += 0.00000000100*cos(1.38002787119 + 28286.9904848612*$t);
   $Z0 += 0.00000000072*cos(1.55820597747 + 6256.7775301916*$t);
   $Z0 += 0.00000000082*cos(4.95202664555 + 10575.4066829418*$t);
   $Z0 += 0.00000000082*cos(1.69647647075 + 1990.745017041*$t);
   $Z0 += 0.00000000096*cos(6.16038106485 + 24356.7807886416*$t);
   $Z0 += 0.00000000075*cos(2.29836095644 + 3634.6210245184*$t);
   $Z0 += 0.00000000075*cos(2.66367876557 + 16200.7727245012*$t);
   $Z0 += 0.00000000087*cos(0.26630214764 + 31441.6775697568*$t);
   $Z0 += 0.00000000080*cos(5.00001604436 + 150192.21439800429*$t);
   $Z0 += 0.00000000079*cos(0.01398391548 + 90617.7374312997*$t);
   $Z0 += 0.00000000054*cos(5.59738773448 + 161338.5000008705*$t);
   $Z0 += 0.00000000056*cos(2.60133794851 + 73188.3759784421*$t);
   $Z0 += 0.00000000055*cos(5.81483150022 + 143233.51002162008*$t);
   $Z0 += 0.00000000054*cos(3.38482031504 + 323049.11878710288*$t);
   return $Z0;
}



   function Earth_Z1($t) // 97 terms of order 1
{
   $Z1  = 0.00000009031*cos(3.89751156799 + 5507.5532386674*$t);
   $Z1 += 0.00000006179*cos(1.73051337995 + 5223.6939198022*$t);
   $Z1 += 0.00000003793*cos(5.24575814515 + 2352.8661537718*$t);
   $Z1 += 0.00000002840*cos(2.47694599818 + 1577.3435424478*$t);
   $Z1 += 0.00000001817*cos(0.41874743765 + 6283.0758499914*$t);
   $Z1 += 0.00000001499*cos(1.83320979291 + 5856.4776591154*$t);
   $Z1 += 0.00000001463*cos(5.68891324948 + 5753.3848848968*$t);
   $Z1 += 0.00000001302*cos(2.18800611215 + 9437.762934887*$t);
   $Z1 += 0.00000001239*cos(4.95327097854 + 10213.285546211*$t);
   $Z1 += 0.00000001029*cos(0.09367831611 + 7860.4193924392*$t);
   $Z1 += 0.00000000975*cos(0.08833256880 + 14143.4952424306*$t);
   $Z1 += 0.00000000851*cos(1.79547916132 + 3930.2096962196*$t);
   $Z1 += 0.00000000581*cos(2.26949174067 + 5884.9268465832*$t);
   $Z1 += 0.00000000515*cos(5.64196681593 + 529.6909650946*$t);
   $Z1 += 0.00000000473*cos(6.22750969242 + 6309.3741697912*$t);
   $Z1 += 0.00000000449*cos(1.52767374606 + 18073.7049386502*$t);
   $Z1 += 0.00000000360*cos(3.62344325932 + 13367.9726311066*$t);
   $Z1 += 0.00000000372*cos(3.22470721320 + 6275.9623029906*$t);
   $Z1 += 0.00000000260*cos(1.87645933242 + 11790.6290886588*$t);
   $Z1 += 0.00000000322*cos(0.94084045832 + 6069.7767545534*$t);
   $Z1 += 0.00000000232*cos(0.27531852596 + 7058.5984613154*$t);
   $Z1 += 0.00000000217*cos(6.03652873142 + 10977.078804699*$t);
   $Z1 += 0.00000000232*cos(2.93325646109 + 22003.9146348698*$t);
   $Z1 += 0.00000000204*cos(3.86264841382 + 6496.3749454294*$t);
   $Z1 += 0.00000000189*cos(2.88937704419 + 15720.8387848784*$t);
   $Z1 += 0.00000000179*cos(4.90537280911 + 12036.4607348882*$t);
   $Z1 += 0.00000000222*cos(3.98495366315 + 6812.766815086*$t);
   $Z1 += 0.00000000213*cos(1.57516933652 + 4694.0029547076*$t);
   $Z1 += 0.00000000157*cos(1.08259734788 + 5643.1785636774*$t);
   $Z1 += 0.00000000154*cos(5.99434678412 + 5486.777843175*$t);
   $Z1 += 0.00000000137*cos(2.67760436027 + 6290.1893969922*$t);
   $Z1 += 0.00000000179*cos(2.05905949693 + 7084.8967811152*$t);
   $Z1 += 0.00000000121*cos(5.90212574947 + 9225.539273283*$t);
   $Z1 += 0.00000000125*cos(2.24111392416 + 1059.3819301892*$t);
   $Z1 += 0.00000000150*cos(2.00175038718 + 5230.807466803*$t);
   $Z1 += 0.00000000149*cos(5.06157254516 + 17298.1823273262*$t);
   $Z1 += 0.00000000118*cos(5.39979058038 + 3340.6124266998*$t);
   $Z1 += 0.00000000161*cos(3.32421999691 + 6283.3196674749*$t);
   $Z1 += 0.00000000115*cos(5.92406672373 + 4705.7323075436*$t);
   $Z1 += 0.00000000118*cos(4.40207874911 + 19651.048481098*$t);
   $Z1 += 0.00000000128*cos(4.35489873365 + 25934.1243310894*$t);
   $Z1 += 0.00000000109*cos(2.52157834166 + 6438.4962494256*$t);
   $Z1 += 0.00000000099*cos(2.70727488041 + 5216.5803728014*$t);
   $Z1 += 0.00000000122;
   $Z1 += 0.00000000103*cos(0.93782340879 + 8827.3902698748*$t);
   $Z1 += 0.00000000083*cos(4.12473540351 + 8635.9420037632*$t);
   $Z1 += 0.00000000097*cos(5.50959692365 + 29864.334027309*$t);
   $Z1 += 0.00000000073*cos(1.73905345744 + 11506.7697697936*$t);
   $Z1 += 0.00000000083*cos(5.69169692653 + 775.522611324*$t);
   $Z1 += 0.00000000072*cos(0.21891639822 + 21228.3920235458*$t);
   $Z1 += 0.00000000071*cos(2.86755026812 + 6681.2248533996*$t);
   $Z1 += 0.00000000074*cos(2.20184828895 + 37724.7534197482*$t);
   $Z1 += 0.00000000063*cos(4.45586625948 + 7079.3738568078*$t);
   $Z1 += 0.00000000047*cos(2.04946724464 + 3128.3887650958*$t);
   $Z1 += 0.00000000061*cos(0.63918772258 + 33794.5437235286*$t);
   $Z1 += 0.00000000047*cos(3.32543843300 + 26087.9031415742*$t);
   $Z1 += 0.00000000049*cos(1.60680905005 + 6702.5604938666*$t);
   $Z1 += 0.00000000057*cos(0.11215813438 + 29088.811415985*$t);
   $Z1 += 0.00000000049*cos(3.02832204050 + 20426.571092422*$t);
   $Z1 += 0.00000000041*cos(5.55329394890 + 11015.1064773348*$t);
   $Z1 += 0.00000000041*cos(5.91861144924 + 23581.2581773176*$t);
   $Z1 += 0.00000000045*cos(2.00068583743 + 426.598190876*$t);
   $Z1 += 0.00000000045*cos(4.95273290181 + 5863.5912061162*$t);
   $Z1 += 0.00000000050*cos(3.62740835096 + 41654.9631159678*$t);
   $Z1 += 0.00000000046*cos(1.65798680284 + 25158.6017197654*$t);
   $Z1 += 0.00000000036*cos(5.61836577943 + 12566.1516999828*$t);
   $Z1 += 0.00000000036*cos(6.24373396652 + 6283.14316029419*$t);
   $Z1 += 0.00000000036*cos(0.40465162918 + 6283.0085396886*$t);
   $Z1 += 0.00000000032*cos(6.09025731476 + 64809.80550494129*$t);
   $Z1 += 0.00000000032*cos(6.03707103538 + 2942.4634232916*$t);
   $Z1 += 0.00000000041*cos(4.86809570283 + 1592.5960136328*$t);
   $Z1 += 0.00000000037*cos(1.04055368426 + 213.299095438*$t);
   $Z1 += 0.00000000031*cos(3.62641145030 + 13095.8426650774*$t);
   $Z1 += 0.00000000028*cos(4.38359423735 + 7632.9432596502*$t);
   $Z1 += 0.00000000030*cos(2.03616887071 + 12139.5535091068*$t);
   $Z1 += 0.00000000028*cos(6.03334294232 + 17789.845619785*$t);
   $Z1 += 0.00000000026*cos(3.88971333608 + 5331.3574437408*$t);
   $Z1 += 0.00000000031*cos(1.44666331503 + 16730.4636895958*$t);
   $Z1 += 0.00000000026*cos(6.26376705837 + 23543.23050468179*$t);
   $Z1 += 0.00000000023*cos(4.44388985550 + 18849.2275499742*$t);
   $Z1 += 0.00000000025*cos(4.13395006026 + 3154.6870848956*$t);
   $Z1 += 0.00000000028*cos(1.53862289477 + 6279.4854213396*$t);
   $Z1 += 0.00000000028*cos(1.96831814872 + 6286.6662786432*$t);
   $Z1 += 0.00000000028*cos(5.78094918529 + 15110.4661198662*$t);
   $Z1 += 0.00000000025*cos(0.62040343662 + 10988.808157535*$t);
   $Z1 += 0.00000000022*cos(6.02390113954 + 16496.3613962024*$t);
   $Z1 += 0.00000000026*cos(2.48165809843 + 5729.506447149*$t);
   $Z1 += 0.00000000020*cos(3.85655029499 + 9623.6882766912*$t);
   $Z1 += 0.00000000021*cos(5.83006047147 + 7234.794256242*$t);
   $Z1 += 0.00000000021*cos(0.69628570421 + 398.1490034082*$t);
   $Z1 += 0.00000000022*cos(5.02222806555 + 6127.6554505572*$t);
   $Z1 += 0.00000000020*cos(3.47611265290 + 6148.010769956*$t);
   $Z1 += 0.00000000020*cos(0.90769829044 + 5481.2549188676*$t);
   $Z1 += 0.00000000020*cos(0.03081589303 + 6418.1409300268*$t);
   $Z1 += 0.00000000020*cos(3.74220084927 + 1589.0728952838*$t);
   $Z1 += 0.00000000018*cos(1.58348238359 + 2118.7638603784*$t);
   $Z1 += 0.00000000019*cos(0.85407021371 + 14712.317116458*$t);
   return $Z1*$t;
}



   function Earth_Z2($t) // 47 terms of order 2
{
   $Z2  = 0.00000001662*cos(1.62703209173 + 84334.66158130829*$t);
   $Z2 += 0.00000000492*cos(2.41382223971 + 1047.7473117547*$t);
   $Z2 += 0.00000000344*cos(2.24353004539 + 5507.5532386674*$t);
   $Z2 += 0.00000000258*cos(6.00906896311 + 5223.6939198022*$t);
   $Z2 += 0.00000000131*cos(0.95447345240 + 6283.0758499914*$t);
   $Z2 += 0.00000000086*cos(1.69139956945 + 7860.4193924392*$t);
   $Z2 += 0.00000000090*cos(0.97606804452 + 1577.3435424478*$t);
   $Z2 += 0.00000000090*cos(0.37899871725 + 2352.8661537718*$t);
   $Z2 += 0.00000000089*cos(6.25807507963 + 10213.285546211*$t);
   $Z2 += 0.00000000075*cos(0.84213523741 + 167621.57585086189*$t);
   $Z2 += 0.00000000052*cos(1.70501566089 + 14143.4952424306*$t);
   $Z2 += 0.00000000057*cos(6.15295833679 + 12194.0329146209*$t);
   $Z2 += 0.00000000051*cos(1.27616016740 + 5753.3848848968*$t);
   $Z2 += 0.00000000051*cos(5.37229738682 + 6812.766815086*$t);
   $Z2 += 0.00000000034*cos(1.73672994279 + 7058.5984613154*$t);
   $Z2 += 0.00000000038*cos(2.77761031485 + 10988.808157535*$t);
   $Z2 += 0.00000000046*cos(3.38617099014 + 156475.2902479957*$t);
   $Z2 += 0.00000000021*cos(1.95248349228 + 8827.3902698748*$t);
   $Z2 += 0.00000000018*cos(3.33419222028 + 8429.2412664666*$t);
   $Z2 += 0.00000000019*cos(4.32945160287 + 17789.845619785*$t);
   $Z2 += 0.00000000017*cos(0.66191210656 + 6283.0085396886*$t);
   $Z2 += 0.00000000018*cos(3.74885333072 + 11769.8536931664*$t);
   $Z2 += 0.00000000017*cos(4.23058370776 + 10977.078804699*$t);
   $Z2 += 0.00000000017*cos(1.78116162721 + 5486.777843175*$t);
   $Z2 += 0.00000000018*cos(6.14348240716 + 11790.6290886588*$t);
   $Z2 += 0.00000000022*cos(1.34376831204 + 12036.4607348882*$t);
   $Z2 += 0.00000000017*cos(2.79601092529 + 796.2980068164*$t);
   $Z2 += 0.00000000015*cos(1.17032155085 + 213.299095438*$t);
   $Z2 += 0.00000000018*cos(2.85221514199 + 5088.6288397668*$t);
   $Z2 += 0.00000000017*cos(0.21780913672 + 6283.14316029419*$t);
   $Z2 += 0.00000000013*cos(1.21201504386 + 25132.3033999656*$t);
   $Z2 += 0.00000000012*cos(5.13714452592 + 7079.3738568078*$t);
   $Z2 += 0.00000000013*cos(3.79842135217 + 4933.2084403326*$t);
   $Z2 += 0.00000000012*cos(4.89407978213 + 3738.761430108*$t);
   $Z2 += 0.00000000015*cos(6.05682328852 + 398.1490034082*$t);
   $Z2 += 0.00000000014*cos(4.81029291856 + 4694.0029547076*$t);
   $Z2 += 0.00000000011*cos(0.61684523405 + 3128.3887650958*$t);
   $Z2 += 0.00000000011*cos(5.32876538500 + 6040.3472460174*$t);
   $Z2 += 0.00000000014*cos(5.27227350286 + 4535.0594369244*$t);
   $Z2 += 0.00000000011*cos(2.39292099451 + 5331.3574437408*$t);
   $Z2 += 0.00000000010*cos(4.45296532710 + 6525.8044539654*$t);
   $Z2 += 0.00000000014*cos(4.66400985037 + 8031.0922630584*$t);
   $Z2 += 0.00000000010*cos(3.22472385926 + 9437.762934887*$t);
   $Z2 += 0.00000000011*cos(3.80913404437 + 801.8209311238*$t);
   $Z2 += 0.00000000010*cos(5.15032130575 + 11371.7046897582*$t);
   $Z2 += 0.00000000013*cos(0.98720797401 + 5729.506447149*$t);
   $Z2 += 0.00000000009*cos(5.94191743597 + 7632.9432596502*$t);
   return $Z2*$t*$t;
}



   function Earth_Z3($t) // 11 terms of order 3
{
   $Z3  = 0.00000000011*cos(0.23877262399 + 7860.4193924392*$t);
   $Z3 += 0.00000000009*cos(1.16069982609 + 5507.5532386674*$t);
   $Z3 += 0.00000000008*cos(1.65357552925 + 5884.9268465832*$t);
   $Z3 += 0.00000000008*cos(2.86720038197 + 7058.5984613154*$t);
   $Z3 += 0.00000000007*cos(3.04818741666 + 5486.777843175*$t);
   $Z3 += 0.00000000007*cos(2.59437103785 + 529.6909650946*$t);
   $Z3 += 0.00000000008*cos(4.02863090524 + 6256.7775301916*$t);
   $Z3 += 0.00000000008*cos(2.42003508927 + 5753.3848848968*$t);
   $Z3 += 0.00000000006*cos(0.84181087594 + 6275.9623029906*$t);
   $Z3 += 0.00000000006*cos(5.40160929468 + 1577.3435424478*$t);
   $Z3 += 0.00000000007*cos(2.73399865247 + 6309.3741697912*$t);
   return $Z3*$t*$t*$t;
}



   function Earth_Z4($t) // 5 terms of order 4
{
   $Z4  = 0.00000000004*cos(0.79662198849 + 6438.4962494256*$t);
   $Z4 += 0.00000000005*cos(0.84308705203 + 1047.7473117547*$t);
   $Z4 += 0.00000000005*cos(0.05711572303 + 84334.66158130829*$t);
   $Z4 += 0.00000000003*cos(3.46779895686 + 6279.5527316424*$t);
   $Z4 += 0.00000000003*cos(2.89822201212 + 6127.6554505572*$t);
   return $Z4*$t*$t*$t*$t;
}





?>

